#define NDEBUG
#include <sdsl/int_vector.hpp>
#include <sdsl/util.hpp>
#include <sdsl/rank_support.hpp>
#include <sdsl/rank_support_v.hpp>
#include <sdsl/select_support.hpp>
#include <sdsl/suffixarrays.hpp>
#include <sdsl/wt.hpp>
#include <sdsl/nn_dict_dynamic.hpp>
#include "nn_dict_dynamic_naiv.cpp"
#include "outputbuffer.cpp"
#include "semiexternal.cpp"

using namespace sdsl;
using namespace std;

typedef bit_vector::size_type size_type;

string outputfilenameschema = "lcp_array_tmp_";				// Pattern for the temporary LCP arrays
rank_support_v<> ds_rank_support;					// Rank support for lcp_small
size_type lcp_value_offset = 0;					// Largest LCP value in LCP array, that was written on harddisk
uint8_t max_bb = 64;						// Maximal int_width to store all LCP values


void inline newLCPvalue(int_vector<> &lcp2, bit_vector &finished, size_type &N, size_type &space_in_bit_for_lcp, size_type &phase, size_type &lcp_value, size_type &lcp_value_max, size_type &buffer_size)
{
	// Write old values to disk
	if(lcp_value>=lcp_value_max)
	{
		if(phase)
		{
			write_R_output("lcp","merge to file    ","begin", 0, 0);
			mergeToHD(lcp2, outputfilenameschema + util::to_string(phase-1), finished, outputfilenameschema + util::to_string(phase), buffer_size, N, lcp_value, lcp_value_offset);
			write_R_output("lcp","merge to file    ","end", 0, 0);
		}
		else
		{
			write_R_output("lcp","write to file    ","begin", 0, 0);
			copyToHD(lcp2, outputfilenameschema + util::to_string(phase), N);
			write_R_output("lcp","write to file    ","end", 0, 0);
		}
		write_R_output("lcp","resize variables ","begin", 0, 0);
		// Create rank support
		ds_rank_support.init(&finished);

		// Recalculate lcp_value_max and resize lcp2
		lcp_value_offset = lcp_value_max-1;
		size_type remaining_lcp_values = finished.size()-ds_rank_support.rank(finished.size());
		size_type int_width_new = 1;
		size_type int_width_max = space_in_bit_for_lcp / remaining_lcp_values;
/*
		uint64_t limit = 1;
		for(int_width_new=1; int_width_new < int_width_max; ++int_width_new)
		{
			limit <<= 1;
			if(limit > N)
			{
				break;
			}
		}
		lcp_value_max = lcp_value_offset+limit;
cout << "# l=" << remaining_lcp_values << " b=" << (int)int_width_new << " lcp_value_max=" << lcp_value_max << endl;
*/
		int_width_new = max_bb;
		if(int_width_new > int_width_max)
		{
			int_width_new = int_width_max;
		}
		uint64_t limit = (1ULL<<int_width_new);
		lcp_value_max = lcp_value_offset+limit;
cout << "# l=" << remaining_lcp_values << " b=" << (int)int_width_new << " lcp_value_max=" << lcp_value_max << endl;


		lcp2.set_int_width(int_width_new);
		lcp2.resize(remaining_lcp_values);
		++phase;
		write_R_output("lcp","resize variables ","end", 0, 0);
	}
}

int main(int argc, char **argv)
{
	size_type buffer_size = 1000000;
	tMSS file_map;

	if(argc!=3)
	{
		cout << "Program takes path of the BWT and a FILENAME. It creates the LCP array." << endl;
		cout << "\tPath to BWT" << endl;
		cout << "\tFILENAME" << endl;
		cout << endl;
		cout << "\tOutputfile: lcp_FILENAME" << endl;
		return 1;
	}
	file_map["bwt"] = argv[1];
	string final_output_filename = "lcp_" + util::to_string(argv[2]);

	// create WaveletTree
	write_R_output("bwt","load huffman WT  ","begin", 0, 0);
	int_vector_file_buffer<8> bwt_buf(file_map["bwt"].c_str(), buffer_size);
	size_type N = bwt_buf.int_vector_size;


	wt_huff<rank_support_v<>, select_support_bs< rank_support_v5<> >, select_support_bs< rank_support_v5<> > > wt_bwt(bwt_buf, N);

	write_R_output("bwt","load huffman WT  ","end", 0, 0);

	// init
	write_R_output("lcp","init             ","begin", 0, 0);
	size_type a=0, b=N, a2, b2, a_new, b_new;				// begin and end of current interval, current interval interleaved and new interval
	size_type lcp_value = 0;						// current LCP value
	size_type sigma = 256;						// Maximum number of diffrent characters in the input
	size_type realsigma = wt_bwt.sigma;					// Number of different characters in the input


	unsigned char c = 0;						// Current character
	vector<size_type> C(sigma, 0);					// C-Array: C[i] = number of occurrences of characters < i in the input

	size_type quantity;							// quantity of characters in interval
	vector<unsigned char> pos2char(realsigma);				// list of characters in the interval
	vector<size_type> resi(realsigma);					// number of occurrence of character in [0 .. i-1]
	vector<size_type> resj(realsigma);					// number of occurrence of character in [0 .. j-1]

	// Storage of LCP-values
	bool new_lcp_value = 0;
	string filename_lcp_array_tmp="lcp_array_tmp";
	std::ofstream lcp_out_buf((filename_lcp_array_tmp).c_str(), std::ios::binary | std::ios::trunc | std::ios::out);
	uint8_t int_width = bit_magic::l1BP(2*N+1)+1;
	outputbuffer lcpstorage(lcp_out_buf, N+1, int_width, buffer_size);
	bit_vector finished(N+1, 0);

	// Storage of intervals
	size_type intervals = 0;						// Number of intervals which are currently stored
	size_type intervals_new = 0;						// Number of new intervals
	size_type store_in_queue = N/2048; 					// if intervals < use_queue_and_wt, then store intervals in queue, else in nn_dict_dynamic
	queue<size_type> q;							// Queue for storing the intervals
	nn_dict_dynamic_naiv dict[2];						// Dictionary for storing the intervals
	size_type bitarray_length = N+1+64-(N+1)%64;				// Length of the bitarray
	size_type source = 0, target = 1;					// Defines which dict is source and which is target
	char last_used = 'q';						// if intervals are stored in queue oder dict

	// create C-array
	wt_bwt.interval_symbols(0, N, quantity, pos2char, resi, resj);
	for(size_type i=0; i<quantity; ++i)
	{
		c = pos2char[i];
		C[c+1] = resj[i];
	}
	for(size_type i=1; i<sigma; ++i)
	{
		C[i+1] += C[i];
	}
	write_R_output("lcp","init             ","end", 0, 0);

	write_R_output("lcp","calc lcp values  ","begin", 0, 0);
	// store first interval
	q.push(a); 
	q.push(b); 
	intervals = 1;

	// calulate LCP positions
	lcpstorage.next_value(0);
	finished[0] = true;
//	finished[N+1] = true;
	while(intervals)
	{	
		if(intervals < store_in_queue && last_used == 'b')
		{
			write_R_output("lcp","BitVector -> Queue","begin", 0, lcp_value);
			dict[target].resize(1);

			// copy from bitvector to queue
			a2 = dict[source].next(0);
			b2 = dict[source].next(a2+1);
			while( b2 < dict[source].size() )
			{
				q.push((a2-1)>>1);
				q.push(b2>>1);
				// get next interval
				a2 = dict[source].next(b2+1);
				b2 = dict[source].next(a2+1);
			}
			dict[source].resize(1);
			write_R_output("lcp","BitVector -> Queue","end  ", 0, lcp_value);
		}		
		if(intervals >= store_in_queue && last_used == 'q')
		{
			write_R_output("lcp","Queue -> BitVector","begin", 0, lcp_value);
			dict[source].resize((bitarray_length<<1)+10);
			dict[source].clean();
			// copy from queue to bitvector
			while(!q.empty())
			{
				dict[source][ (q.front()<<1)+1 ] = 1;
				q.pop();
				dict[source][ (q.front()<<1)   ] = 1;
				q.pop();
			}
			dict[target].resize((bitarray_length<<1)+10);
			dict[target].clean();
			write_R_output("lcp","Queue -> BitVector","end  ", 0, lcp_value);
		}
		
		if(intervals < store_in_queue)
		{
			last_used = 'q';
			intervals_new = 0;
			while(intervals)
			{
				// get next interval
				a = q.front();
				q.pop();
				b =q.front();
				q.pop();
				--intervals;

				wt_bwt.interval_symbols(a, b, quantity, pos2char, resi, resj);
				for(size_type i=0; i<quantity; ++i)
				{
					c = pos2char[i];
					a_new = C[c] + resi[i];
					b_new = C[c] + resj[i];
					if(!finished[b_new])
					{
						if(new_lcp_value)
						{
							lcpstorage.next_value(b_new+N+42);
							new_lcp_value = false;
						}
						else
						{
							lcpstorage.next_value(b_new);
						}
						finished[b_new] = true;
//						saveLCPvalue(b_new, lcp_value);
						q.push(a_new);
						q.push(b_new);
						++intervals_new;
					}
				}
			}
			intervals = intervals_new;
		}	
		else
		{
			last_used = 'b';
			intervals = 0;

			// get next interval
			a2 = dict[source].next(0);
			b2 = dict[source].next(a2+1);

			while( b2 < dict[source].size() )
			{
				wt_bwt.interval_symbols(((a2-1)>>1), (b2>>1), quantity, pos2char, resi, resj);
				for(size_type i=0; i<quantity; ++i)
				{
					c = pos2char[i];
					a_new = C[c] + resi[i];
					b_new = C[c] + resj[i];
					if(!finished[b_new])
					{
						if(new_lcp_value)
						{
							lcpstorage.next_value(b_new+N+42);
							new_lcp_value = false;
						}
						else
						{
							lcpstorage.next_value(b_new);
						}
						finished[b_new] = true;
//						saveLCPvalue(b_new, lcp_value);
						dict[target][ (a_new<<1)+1] = 1;
						dict[target][ (b_new<<1)  ] = 1;
						++intervals;
					}
				}
				// get next interval
				a2 = dict[source].next(b2+1);
				b2 = dict[source].next(a2+1);
			}
			// switch source and target
			source = 1-source;
			target = 1-target;
//			util::set_zero_bits(dict[target]);
			dict[target].clean();
		
		}
		++lcp_value;
		new_lcp_value = true;
	}
	write_R_output("lcp","calc lcp values  ","end  ", 0, 0);

	write_R_output("lcp","prepare reorder  ","begin", 0, 0);
	lcpstorage.finish();
	dict[target].resize(1);
	dict[source].resize(1);
	size_type bb = (util::get_size_in_bytes(wt_bwt)*8+4*N)/N;
	max_bb = bit_magic::l1BP(lcp_value+1)+1;
	if(bb<4)
	{
		bb = 4;
	}
	if(bb>max_bb)
	{
		bb = max_bb;
	}
	wt_bwt = wt_huff<rank_support_v<>, select_support_bs< rank_support_v5<> >, select_support_bs< rank_support_v5<> > >();

	size_type phase = 0;
	size_type lcp_value_max = (1ULL<<bb)-1;
	size_type space_in_bit_for_lcp = (N+1)*bb;
	int_vector<> lcp2(N+1, 0, bb);							// LCP array
cout << "# l=" << N << " b=" << (int)bb << " lcp_value_max=" << lcp_value_max << endl;
	int_vector_file_buffer<> lcp_buffer((filename_lcp_array_tmp).c_str(), buffer_size);
	util::set_zero_bits(finished);
	write_R_output("lcp","prepare reordering","end  ", 0, 0);

	write_R_output("lcp","reordering var 2 ","begin", 0, 0);
	for(size_type i=0, r_sum=0, lcp_value=0, r=0; r_sum < N; )
	{
		for( ; i < r_sum+r; ++i)
		{
			size_type position = lcp_buffer[i-r_sum];
			if(position>N+40)
			{
				position = position - N - 42;
				++lcp_value;
				newLCPvalue(lcp2, finished, N, space_in_bit_for_lcp, phase, lcp_value, lcp_value_max, buffer_size);
			}

			// Save lcp_value
			if(phase)
			{
				lcp2[position-ds_rank_support.rank(position)] = lcp_value-lcp_value_offset;
			}
			else
			{
				lcp2[position] = lcp_value;
				finished[position] = true;
			}
		}
		r_sum += r;
		r = lcp_buffer.load_next_block();
	}
	remove(filename_lcp_array_tmp.c_str() );
	write_R_output("lcp","reordering var 2 ","end  ", 0, 0);
	if(phase)
	{
		write_R_output("lcp","merge to final   ","begin", 0, 0);
		mergeToHD(lcp2, outputfilenameschema + util::to_string(phase-1), finished, final_output_filename, buffer_size, N, lcp_value, lcp_value_offset);
		write_R_output("lcp","merge to final   ","end  ", 0, 0);
	}
	else
	{
		write_R_output("lcp","copy to final    ","begin", 0, 0);
		copyToHD(lcp2, final_output_filename, N);
		write_R_output("lcp","copy to final    ","end  ", 0, 0);
	}
	// Delete temporary created files
	write_R_output("lcp","delete tmp files ","begin", 0, 0);
	while(phase--)
	{
		remove( (outputfilenameschema + util::to_string(phase)).c_str() );
	}
	write_R_output("lcp","delete tmp files ","end  ", 0, 0);

	return 0;
}

