#define NDEBUG

#include <cstdio>
#include <cstdlib>
#include <cstring>

#include <sdsl/suffixarrays.hpp>
#include <sdsl/testutils.hpp>
#include "readfile.h"

using namespace sdsl;

char filename[1024];

int main(int argc, char **argv) {
	if (argc!=2) {
		fprintf(stderr, "usage: %s file\n", argv[0]);
		exit(1);
	}
	stop_watch sw;

	fprintf(stderr, "Reading files...\n");
	sw.start();
	uint32_t n;
	const char* text = readfile(argv[1], n);
	uint32_t *sa, *isa;
	{
		strcpy(filename, argv[1]);
		strcat(filename, ".sa");
		FILE *f = fopen(filename, "rb");
		fseek(f, 0L, SEEK_END);
//		n = ftell(f) / 4;
		rewind(f);
	   	sa = new uint32_t[n];
		isa = new uint32_t[n];
		uint32_t dummy = fread(sa, sizeof(uint32_t), n, f);
		fclose(f);
	}
	uint32_t *lcp=sa;//, lcp[n];
	sw.stop();
	fprintf(stderr, "%.0lf milliseconds\n\n", sw.get_user_time());

	fprintf(stderr, "Construct ISA...\n");
	sw.start();
	for (uint32_t i=0; i<n; i++) isa[sa[i]] = i;
	sw.stop();
	fprintf(stderr, "%.0lf milliseconds\n\n", sw.get_user_time());

	fprintf(stderr, "Kasai algorithm, overwrite SA...\n");
	sw.start();
	// kasai algorihtm, overwrite sa
	uint32_t h = 0L;
	uint64_t sum = 0L;
	for (uint32_t i=0; i<n; i++) {
		uint32_t isa_i = isa[i];
		if (isa_i>0) {
			uint32_t j = sa[isa_i-1];
			if (h) --h;
			while (text[i+h]==text[j+h]) ++h;
//			lcp[isa_i] = h;
			sa[isa_i-1] = h;
		} else {
			h = 0;
//			lcp[0] = 0;
		}
		sum += h;
	}
	fprintf(stderr, "mean lcp = %lf\n", (double)sum/n);
	sw.stop();
	fprintf(stderr, "%.0lf milliseconds\n\n", sw.get_user_time());

	// write lcp array as 4-byte-file
	{
		strcpy(filename, argv[1]);
		strcat(filename, ".lcp-kasai");
		FILE *f = fopen(filename, "wb");
		uint32_t dummy = n-1;
		fwrite(&dummy, sizeof(uint32_t), 1, f);
		fwrite(sa, sizeof(uint32_t), n-1, f);
		fclose(f);
	//	fprintf(stderr, ">>>>> DEBUG n=%d\n", n);
	}

	fprintf(stderr, "absolute user time: %llu\n", sw.get_abs_user_time());
	fprintf(stdout, "%llu\n", sw.get_abs_user_time() + sw.get_abs_sys_time());
	return 0;
}
