#define NDEBUG

#include <cstdio>
#include <cstdlib>
#include <cstring>

#include <sdsl/suffixarrays.hpp>
#include <sdsl/testutils.hpp>
#include "readfile.h"

using namespace sdsl;

char filename[1024];

int main(int argc, char **argv) {
	uint32_t n;
	if (argc!=2) {
		fprintf(stderr, "usage: %s file\n", argv[0]);
		exit(1);
	}
	stop_watch sw;

	fprintf(stderr, "Reading file...\n");
	sw.start();
	char* text = readfile(argv[1], n);
	sw.stop();
	fprintf(stderr, "%.0lf milliseconds\n\n", sw.get_user_time());

	fprintf(stderr, "Construct SA...\n");
	sw.start();
	//csa_uncompressed csa((const unsigned char*)text);
	//fprintf(stderr, "csa erstellt...\n");
	if (n<(1<<31)) {
		fprintf(stderr, "32bit Version\n", n);
		int_vector<32> data;
		fprintf(stderr, "vorher\n");
		algorithm::calculate_sa((const unsigned char*)text, n, data);
		fprintf(stderr, "nachher\n");
		
		free(text);
	
		// write suffix array as 4-byte-file
		{
			strcpy(filename, argv[1]);
			strcat(filename, ".sa");
			FILE *f = fopen(filename, "wb");
			fwrite(data.data(), sizeof(uint32_t), data.size(), f);
			fclose(f);
		}
	} else {
		fprintf(stderr, "64bit Version\n", n);
		int_vector<64> data;
		fprintf(stderr, "vorher\n");
		algorithm::calculate_sa((const unsigned char*)text, n, data);
		fprintf(stderr, "nachher\n");
		
		free(text);
	
		// write suffix array as 4-byte-file
		{
			strcpy(filename, argv[1]);
			strcat(filename, ".sa");
			FILE *f = fopen(filename, "wb");
			//fwrite(data.data(), sizeof(uint64_t), data.size(), f);
			for (size_t i=0; i<data.size(); i++) {
				uint32_t t= 0xffffffffllu & data[i];
				fwrite(&t, sizeof(uint32_t), 1, f);
			}
			fclose(f);
		}
	}

	sw.stop();
	fprintf(stderr, "%.0lf milliseconds\n\n", sw.get_user_time());

	fprintf(stderr, "absolute user time: %llu\n", sw.get_abs_user_time());
	return 0;
}
