#ifndef INCLUDED_HELPER
#define INCLUDED_HELPER

#include <sdsl/int_vector.hpp>
#include <sdsl/util.hpp>
#include <assert.h>

using namespace std;

namespace sdsl{

//! A class
class outputbuffer
{
	public:
		typedef int_vector<64>::size_type 				size_type;
	private:
		size_type wb;		// Number of bits that were already written
		int_vector<> out_buf;	// Outputbuffer
		std::ofstream& lcp_out_buf;
		size_type idx_out_buf;
		size_type idx_out_buf_sum;
		size_type m_N;

	public:
		outputbuffer(std::ofstream& output, size_type N, const uint8_t int_width, const size_type buffer_size):lcp_out_buf(output)
		{
			if((buffer_size & 0x7ULL))
			{
cerr << "ERROR: buffer_size=" << buffer_size << " is not a multiple of 8. Result would be wrong!" << endl;
abort();
			}
			size_type bit_size = N*int_width;					// Size of output file
			wb = 0;
			m_N = N;

			out_buf = int_vector<>(buffer_size, 0, int_width); 			// Outputbuffer
			idx_out_buf = 0;
			idx_out_buf_sum = 0;
			
			lcp_out_buf.write((char *) &(bit_size), sizeof(bit_size));			// Write length of vector
			lcp_out_buf.write((char *) &(int_width), sizeof(int_width));		// Write int-width of vector
		}

		void next_value(size_type value)
		{
//cerr << idx_out_buf_sum + idx_out_buf << ": " << value << endl;
			out_buf[idx_out_buf++] = value;
			if(idx_out_buf>=out_buf.size())
			{
				flush();
			}
		}
	
		void flush()
		{
			// Write all values from buffer to harddisk
			size_type cur_wb = (idx_out_buf*out_buf.get_int_width()+7)/8;
			lcp_out_buf.write((const char*)out_buf.data(), cur_wb );
			wb += cur_wb;

			idx_out_buf_sum += idx_out_buf;
			idx_out_buf = 0;	
		}
	
		void finish()
		{
			// Write values remaining values from buffer to harddisk
			flush();

			// Close file
//			cassert(idx_out_buf_sum==m_N);
			if(idx_out_buf_sum!=m_N)
			{
cerr << "FEHLER " << idx_out_buf_sum << "!=" << m_N << endl;
			}
			if(wb%8)
			{
				lcp_out_buf.write("\0\0\0\0\0\0\0\0", 8-wb%8);
			}
			lcp_out_buf.close();		
		}
};



} // end of namespace

#endif // end file 
