#!/bin/bash


# Path to the dBWT-algorithm and PLCP-algorithm
PATH2DBWT=""
PATH2PHI=""

# Number of runs
number_of_runs=5

# Init
DATUM=`date +"%Y%m%d_%H%M%S"`
INPUTDIR="./input"
LOGDIR="log/${DATUM}"
LOGDIR_TMP="log/tmp"
mkdir ${LOGDIR}
arr_file_sizes=(`stat -c"%s" ${INPUTDIR}/*`)

# Run for all input files
for INPUTFILE in ${INPUTDIR}/*
do
	FILENAME=`basename ${INPUTFILE}`
	FILEPATH=`dirname ${INPUTFILE}`

	echo "Create SA-array from input";			# Yuata Mori
	./monitor.sh ./tool_create_sa ${INPUTFILE} ${FILENAME} 2> /dev/null >> ${LOGDIR}/${FILENAME}.pids.log
	INPUTFILE_TEXT="text_${FILENAME}"
	INPUTFILE_SA="sa_${FILENAME}"

	echo "Create LCP-array and BWT with goPHI";		# Simon Gog and Enno Ohlebusch
	./monitor.sh ./tool_create_lcp ${INPUTFILE_TEXT} ${INPUTFILE_SA} "goPHI_${FILENAME}" 2> /dev/null >> ${LOGDIR}/${FILENAME}.pids.log
	INPUTFILE_BWT_goPHI="bwt_goPHI_${FILENAME}"
	INPUTFILE_LCP_goPHI="lcp_goPHI_${FILENAME}"
	rm ${INPUTFILE_TEXT}
	rm ${INPUTFILE_SA}
	rm ${INPUTFILE_LCP_goPHI}
	rm lcp_sml_*
	rm lcp_big_*

##
#	echo "Create BWT direct from input";			# Daisuke Okanohara and Kunihiko Sadakane
#	./monitor.sh ${PATH2DBWT}/dbwt ${INPUTFILE} 2> /dev/null >> ${LOGDIR}/${FILENAME}.pids.log
#	./convert_bwt output "bwt_${FILENAME}" > /dev/null
#	INPUTFILE_BWT="bwt_${FILENAME}"
#	rm ${INPUTFILE_BWT}
#	rm output.bw
#	rm output.lst

	echo "Create LCP-array from BWT";			# Timo Beller, Simon Gog, Enno Ohlebusch and Thomas Schnattinger
	./monitor.sh ./bwt_based_laca ${INPUTFILE_BWT_goPHI} ${FILENAME} 2> /dev/null >> ${LOGDIR}/${FILENAME}.pids.log
	INPUTFILE_LCP_bbLACA="lcp_${FILENAME}"
	rm ${INPUTFILE_LCP_bbLACA}

	echo "Create LCP-array from BWT 2";			# Timo Beller, Simon Gog, Enno Ohlebusch and Thomas Schnattinger
	./monitor.sh ./bwt_based_laca2 ${INPUTFILE_BWT_goPHI} ${FILENAME} 2> /dev/null >> ${LOGDIR}/${FILENAME}.pids.log
	INPUTFILE_LCP_bbLACA2="lcp_${FILENAME}"
	rm ${INPUTFILE_LCP_bbLACA2}

	rm ${INPUTFILE_BWT_goPHI}

	echo "Create SA-array with 4 byte integer";		# Yuata Mori
	./create_suffixarray ${INPUTFILE}  2> /dev/null >> ${LOGDIR}/${FILENAME}.pids.log

	echo "Create LCP-array with KLAAP";			# Toru Kasai, Gunho Lee, Hiroki Arimura, Setsuo Arikawa and Kunsoo Park
	./monitor.sh ./create_lcparray ${INPUTFILE} 2> /dev/null >> ${LOGDIR}/${FILENAME}.pids.log
	rm ${FILEPATH}/${FILENAME}.lcp-kasai

	echo "Create LCP-array with PHI- 1";			# Juha Kärkkäinen, Giovanni Manzini and Simon J. Puglisi
	./monitor.sh ./create_lcparray_phi ${INPUTFILE} 2> /dev/null >> ${LOGDIR}/${FILENAME}.pids.log
	rm ${FILEPATH}/${FILENAME}.lcp-phi

##
#	echo "Create LCP-array with PHI-64";			# Juha Kärkkäinen, Giovanni Manzini and Simon J. Puglisi
#	./monitor.sh ${PATH2PHI}/splcp ${INPUTFILE} 64 2> /dev/null >> ${LOGDIR}/${FILENAME}.pids.log
#	rm ${FILEPATH}/${FILENAME}.lcp

##
#	echo "Create LCP-array with PHI-64-semi"		# Juha Kärkkäinen, Giovanni Manzini and Simon J. Puglisi
#	cp ${INPUTFILE}.sa ${FILEPATH}/${FILENAME}.lcp
#	./monitor.sh ${PATH2PHI}/secsplcp ${INPUTFILE} 64 2> /dev/null >> ${LOGDIR}/${FILENAME}.pids.log
#	rm ${FILEPATH}/${FILENAME}.lcp
	
	# Evaluate space usage
	mv ./mem_of_* ${LOGDIR_TMP}
	for MEMFILE in ${LOGDIR_TMP}/mem_of_*
	do
		echo `awk -F"\t" '{if(max<$2) max=$2 } END {printf("%d", max)}' ${MEMFILE}` >> ${LOGDIR}/${FILENAME}.maxmem.all
	done
	
	# Save logfiles
	mv ${LOGDIR_TMP}/mem_of_* ${LOGDIR}
	mv ./phases_of_* ${LOGDIR}

	# Runtime tests
	for i in `seq 1 ${number_of_runs}`
	do
		echo "File ${FILENAME} - run ${i}"

		echo "Calculate SA from input";
		/usr/bin/time -o ${LOGDIR}/${FILENAME}.runtime.01.create_sa             -a -f "%e.%U.%S" ./tool_create_sa ${INPUTFILE} ${FILENAME} 2>> ${LOGDIR}/${FILENAME}.output_stderr_during_runtime 1>> ${LOGDIR}/${FILENAME}.output_stdout_during_runtime
		INPUTFILE_TEXT="text_${FILENAME}"
		INPUTFILE_SA="sa_${FILENAME}"

		echo "Calculate LCP and BWT with goPHI";
		/usr/bin/time -o ${LOGDIR}/${FILENAME}.runtime.02.create_lcp_goPHI      -a -f "%e.%U.%S" ./tool_create_lcp ${INPUTFILE_TEXT} ${INPUTFILE_SA} "goPHI_${FILENAME}" 2>> ${LOGDIR}/${FILENAME}.output_stderr_during_runtime 1>> ${LOGDIR}/${FILENAME}.output_stdout_during_runtime
		INPUTFILE_BWT_goPHI="bwt_goPHI_${FILENAME}"
		INPUTFILE_LCP_goPHI="lcp_goPHI_${FILENAME}"
		rm ${INPUTFILE_TEXT}
		rm ${INPUTFILE_SA}
		rm ${INPUTFILE_LCP_goPHI}
		rm lcp_sml_*
		rm lcp_big_*

##
#		echo "Calculate BWT direct from input";
#		/usr/bin/time -o ${LOGDIR}/${FILENAME}.runtime.03.create_dbwt           -a -f "%e.%U.%S" ${PATH2DBWT}/dbwt ${INPUTFILE} 2>> ${LOGDIR}/${FILENAME}.output_stderr_during_runtime 1>> ${LOGDIR}/${FILENAME}.output_stdout_during_runtime
#		./convert_bwt output "bwt_${FILENAME}" > /dev/null
#		INPUTFILE_BWT="bwt_${FILENAME}"
#		rm ${INPUTFILE_BWT}
#		rm output.bw
#		rm output.lst

		echo "Calculate LCP from BWT";
		/usr/bin/time -o ${LOGDIR}/${FILENAME}.runtime.04.create_lcp_bblaca     -a -f "%e.%U.%S" ./bwt_based_laca ${INPUTFILE_BWT_goPHI} ${FILENAME} 2>> ${LOGDIR}/${FILENAME}.output_stderr_during_runtime 1>> ${LOGDIR}/${FILENAME}.output_stdout_during_runtime
		INPUTFILE_LCP_bbLACA="lcp_${FILENAME}"
		rm ${INPUTFILE_LCP_bbLACA}

		echo "Calculate LCP from BWT 2";
		/usr/bin/time -o ${LOGDIR}/${FILENAME}.runtime.05.create_lcp_bblaca2    -a -f "%e.%U.%S" ./bwt_based_laca2 ${INPUTFILE_BWT_goPHI} ${FILENAME} 2>> ${LOGDIR}/${FILENAME}.output_stderr_during_runtime 1>> ${LOGDIR}/${FILENAME}.output_stdout_during_runtime
		INPUTFILE_LCP_bbLACA2="lcp_${FILENAME}"
		rm ${INPUTFILE_LCP_bbLACA2}

		rm ${INPUTFILE_BWT_goPHI}

		echo "Calculate LCP with KLAAP";
		/usr/bin/time -o ${LOGDIR}/${FILENAME}.runtime.06.create_lcp_klaap      -a -f "%e.%U.%S" ./create_lcparray ${INPUTFILE} 2> /dev/null >> ${LOGDIR}/${FILENAME}.pids.log
		rm ${FILEPATH}/${FILENAME}.lcp-kasai

		echo "Calculate LCP with PHI- 1";
		/usr/bin/time -o ${LOGDIR}/${FILENAME}.runtime.07.create_lcp_phi1       -a -f "%e.%U.%S" ./create_lcparray_phi ${INPUTFILE} 2> /dev/null >> ${LOGDIR}/${FILENAME}.pids.log
		rm ${FILEPATH}/${FILENAME}.lcp-phi

##
#		echo "Calculate LCP with PHI-64";
#		/usr/bin/time -o ${LOGDIR}/${FILENAME}.runtime.08.create_lcp_phi64      -a -f "%e.%U.%S" ${PATH2PHI}/splcp ${INPUTFILE} 64 2> /dev/null >> ${LOGDIR}/${FILENAME}.pids.log
#		rm ${FILEPATH}/${FILENAME}.lcp

##
#		echo "Calculate LCP with PHI-64-semi"
#		cp ${INPUTFILE}.sa ${FILEPATH}/${FILENAME}.lcp
#		/usr/bin/time -o ${LOGDIR}/${FILENAME}.runtime.09.create_lcp_phi64_semi -a -f "%e.%U.%S" ${PATH2PHI}/secsplcp ${INPUTFILE} 64 2> /dev/null >> ${LOGDIR}/${FILENAME}.pids.log
#		rm ${FILEPATH}/${FILENAME}.lcp
	done

	rm ${FILEPATH}/${FILENAME}.sa

	# Evaluate run time
#	echo "Evaluate run time"
	for f in ${LOGDIR}/${FILENAME}.runtime.*
	do
		ALGO=`echo ${f} | awk -F. '{print $5}'`

		awk -F. 'BEGIN {
			min_real=100000000;
			min_user=min_real;
			min_sys=min_real;
			max_real=0;
			max_user=0;
			max_sys=0;
			sum_real = 0;
			sum_user = 0;
			sum_syst = 0;
			t=0;
		} 
		{
			real = $1*1000+$2*10;
			user = $3*1000+$4*10;
			syst = $5*1000+$6*10;
			sum_real += real;
			sum_user += user;
			sum_syst += syst;
			if(min_real>real)
				min_real=real;
			if(min_user>user)
				min_user=user;
			if(min_sys>syst)
				min_sys=syst;
			if(max_real<real)
				max_real=real;
			if(max_user<user)
				max_user=user;
			if(max_sys<syst)
				max_sys=syst;
			t+=1;
		}
		END {
			printf("%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t", t, sum_real/t, sum_user/t, sum_syst/t, min_real, min_user, min_sys, max_real, max_user, max_sys)
		}' ${f} >> ${LOGDIR}/${FILENAME}.results.all
		echo "algo=${ALGO}" >> ${LOGDIR}/${FILENAME}.results.all
	done

	echo "Done."
	echo ""
	echo ""
done



# Present the results
declare -a array_maxmemo
declare -a array_runtime

##
#arr_algo=("SA construction " "BWT construction" "KLAAP           " "Phi             " "Phi64           " "Phi64-semi      " "go-Phi          " "new algorithm   " "new algorithm 2 ");
#zeilen_maxmemo=(0 2 5 6 7 8 1 3 4)
#zeilen_runtime=(0 2 5 6 7 8 1 3 4)


arr_algo=("SA construction " "KLAAP           " "Phi             " "go-Phi          " "new algorithm   " "new algorithm 2 ");
zeilen_maxmemo=(0 4 5 1 2 3)
zeilen_runtime=(0 4 5 1 2 3)

idx_array_maxmemo=0;
idx_array_runtime=0;
for INPUTFILE in ${LOGDIR}/*.maxmem.all
do
	declare -a array1
	array1=($(cat "${INPUTFILE}"))
	anz_memo=${#array1[*]}
	let anz_memo--

	anz=${#zeilen_maxmemo[*]}
	let anz--
	for i in `seq 0 ${anz}`
	do
		idx=${zeilen_maxmemo[${i}]}
		array_maxmemo[idx_array_maxmemo]=${array1[${idx}]}
		let idx_array_maxmemo++
	done

	let anz_memo++
done

anz_files=0
for INPUTFILE in ${LOGDIR}/*.results.all
do
	let anz_files++
	declare -a array1
	array1=($(cat "${INPUTFILE}" | awk '{printf "%s\n", $2}'))

	anz=${#zeilen_runtime[*]}
	let anz--
	for i in `seq 0 ${anz}`
	do
		idx=${zeilen_runtime[${i}]}
		array_runtime[idx_array_runtime]=${array1[${idx}]}
		let idx_array_runtime++
	done
done

anz_algo=${#arr_algo[*]}
let anz_algo_minus_eins=anz_algo-1
let anz_files--
for i in `seq 0 ${anz_algo_minus_eins}`
do
	line="${arr_algo[${i}]}"
	for j in `seq 0 ${anz_files}`
	do
		let pos=i+j*anz_algo
		time=$(echo "scale=1; ${array_runtime[${pos}]}/1000" | bc -l)
		memo=$(echo "scale=1; ${array_maxmemo[${pos}]}/${arr_file_sizes[${j}]}" | bc -l)
		line="${line} | ${time} | ${memo}"
	done
	echo -e "${line}"
done
