#include <sdsl/suffixarrays.hpp>

using namespace sdsl;
using namespace std;

typedef bit_vector::size_type size_type;
typedef csa_uncompressed Csa;

static bool construct_sa_uncompressed(std::string file_name, Csa &csa, tMSS &file_map, std::string dir="./", std::string id="")
{
	uint64_t fs = 0;
	char *ccc = NULL;
	if( (fs=file::read_text((const char*)file_name.c_str(), ccc))>0 )
	{
		size_type n = strlen((const char*)ccc);
		if(id == "")
		{
			id =  util::to_string(util::get_pid())+"_"+util::to_string(util::get_id()).c_str();
		}

		if( !util::store_to_file(char_array_serialize_wrapper<>((unsigned char*)ccc,n+1), (dir+"text_"+id).c_str() ) )
		{
			throw std::ios_base::failure( "#usa_construct: Cannot store text to file system!" );
			delete [] ccc;
			return false;
		}
		else
		{
			file_map["text"] = (dir+"text_"+id).c_str();
		}
		delete [] ccc;

		write_R_output("usa", "construct SA", "begin", 1, 0);
		int_vector_file_buffer<8> text_buf( file_map["text"].c_str() );
		typedef int_vector<>::size_type size_type;

		text_buf.reset();
		n = text_buf.int_vector_size;

		unsigned char *text = NULL;
		util::load_from_int_vector_buffer(text, text_buf);		

		int_vector<> sa = int_vector<>(n, 0, bit_magic::l1BP(n+1)+1);
		algorithm::calculate_sa(text,n, sa);	 // calculate the suffix array sa of str
		delete [] text;
		assert(sa.size() == n);

		write_R_output("usa", "construct SA", "end", 1, 0);
		write_R_output("usa", "store SA", "begin", 1, 0);

		if( !util::store_to_file(sa, (dir+"sa_"+id).c_str() ) )
		{
			throw std::ios_base::failure( "#usa_construct: Cannot store SA to file system!" );
			return false;
		}
		else
		{
			file_map["sa"] = dir+"sa_"+id;
		}

		{
			sa.resize(0);
			int_vector<> temp;
			temp.swap(sa);
		}

		write_R_output("usa", "store SA", "end", 1, 0);
	}
	else
	{
		return false;
	}
	return true;
}


int main(int argc, char **argv)
{
	stop_watch sw;
	sw.start();
	/* Suffix-Array */
	Csa sa;

	/* Check Parameters */
	if(argc!=3)
	{
		cout << "Program takes path of input and a FILENAME. It creates text file and the suffix array." << endl;
		cout << "\tPath to input" << endl;
		cout << "\tFILENAME" << endl;
		cout << endl;
		cout << "\tOutputfile: text_FILENAME, sa_FILENAME" << endl;
		return 1;
	}
	
	/* Get parameters */
	string file_name = argv[1];
	string id = argv[2];

	/* Init to create */
	tMSS file_map;
	string dir = "";

	/* Create suffix array */
	construct_sa_uncompressed(file_name, sa, file_map, dir, id);

	sw.stop();
	fprintf(stderr, "%llu\n", sw.get_abs_user_time() + sw.get_abs_sys_time());

	return 0;
}
