#include "sdsl/elias_delta_coder.hpp"

namespace sdsl{

namespace coder{

uint64_t elias_delta::decode_prefix_sum(const uint64_t *data, const size_type start_idx, const size_type end_idx, size_type n){
	if(n==0)
		return 0;
	const uint64_t *lastdata = data + ((end_idx+63) >> 6);
	data += (start_idx >> 6);
	uint64_t w = 0, value = 0;
	int16_t buffered = 0, read = start_idx & 0x3F;
	size_type i = 0;
	if(n + read <= 64 ){
		if( ((*data >> read)&bit_magic::Li1Mask[n]) == bit_magic::Li1Mask[n] )
			return n;
	}else{// n+read > 64
		if( (*data >> read) == bit_magic::Li1Mask[64-read] ){// all bits are set to 1
			value = 64-read;
			++data;
			n -= (64-read);
			read = 0;
			while(n >= 64){
				if(*data == 0xFFFFFFFFFFFFFFFFULL){
					value += 64;
					++data;
					n -= 64;
				}else
					goto start_decoding;
			}
			// 0 <= n <= 64
			if( (*data&bit_magic::Li1Mask[n]) == bit_magic::Li1Mask[n] )
				return value + n;
		}
	}
	
start_decoding:
	while(i<n){
		while(buffered < 64 and data < lastdata){
fill_buffer:			
			w |= (((*data)>>read)<<buffered);
			if( read >= buffered ){
				++data;
				buffered += 64-read;
				read = 0;
			}else{// read buffered
				read += 64-buffered;
				buffered = 64;
			}
		}
/*		if(w==0xFFFFFFFFFFFFFFFFULL){
			i += 64;
			value += 64;
			if(i >= n)
				return value - (i-n);
			buffered = 0;
			w = 0;
		//	continue;
			goto fill_buffer;
		}
*/		
//		uint32_t rbp = (w == 0xFFFFFFFFFFFFFFFFULL)?64:bit_magic::r1BP(~w);
		uint32_t rbp = bit_magic::r1BP(~w);
		if(rbp > 0){
			i += rbp;
			value += rbp;
			if(i >= n){// decoded too much
				return value - (i-n); // return corrected value
			}
			assert(buffered >= rbp);
			buffered -= rbp;
			w >>= rbp;
			if(buffered<16) 
//				continue;
				goto fill_buffer;
		}
//		assert(w!=0xFFFFFFFFFFFFFFFFULL);
//		else
		{// i < n
begin_decode:
			uint32_t psum = EliasDeltaPrefixSum[w&0x0000FFFF];	
			if( !psum or i+((psum>>16)&0x00FF) > n ){
				if( w==0 ){// buffer is not full
					w |= (((*data)>>read)<<buffered);
					if( read >= buffered ){ ++data; buffered += 64-read; read = 0; }else{ read += 64-buffered; buffered = 64;};
					if(!w){ w |= (((*data)>>read)<<buffered);
						if( read >= buffered ){ ++data; buffered += 64-read; read = 0; }else{ read += 64-buffered; buffered = 64;};
					}
				}
//				assert(w>0);
				uint16_t len_1_len = bit_magic::r1BP(w); // read length of length
				buffered -= (len_1_len+1);
				w >>= (len_1_len+1);
				if( len_1_len > buffered ){// buffer is not full
					w |= (((*data)>>read)<<buffered);
					if( read >= buffered ){ ++data; buffered += 64-read; read = 0; }else{ read += 64-buffered; buffered = 64;};
					if(len_1_len > buffered ){ w |= (((*data)>>read)<<buffered);
						if( read >= buffered ){ ++data; buffered += 64-read; read = 0; }else{ read += 64-buffered; buffered = 64;};
					}				
				}
//				assert(len_1_len <= buffered);
				uint16_t len_1 = (w&bit_magic::Li1Mask[len_1_len]) + (1ULL << len_1_len) - 1;
				buffered -= len_1_len;
				w >>= len_1_len;
				if( len_1 > buffered ){// buffer is not full
					w |= (((*data)>>read)<<buffered);
					if( read >= buffered ){ ++data; buffered += 64-read; read = 0; }else{ read += 64-buffered; buffered = 64;};
					if(len_1 > buffered ){ w |= (((*data)>>read)<<buffered);
						if( read >= buffered ){ ++data; buffered += 64-read; read = 0; }else{ read += 64-buffered; buffered = 64;};
					}				
				}
//				if( len_1 > buffered ){
//					std::cerr<<"len_1="<<len_1<<" buffered = "<<buffered<<std::endl;
//				}
//				assert(len_1 <= buffered);
				value	+= (w&bit_magic::Li1Mask[len_1]) + (1ULL << (len_1));
				buffered -= len_1;
				w >>= len_1;
				++i;	
				if(i==n)
					return value;
				if(buffered>=16)
					goto begin_decode;
			}else{
				value += (psum&0x0000FFFF);
				i += ((psum>>16)&0x00FF);
				if(i==n)
					return value;
				buffered -= (psum>>24);
				w >>= (psum>>24);
				if(buffered>=16)
					goto begin_decode;
			}
		}
//		std::cerr<<i<<" / "<<n<<std::endl;
	};
//	std::cerr<<value<<std::endl;
	return value;
}


uint64_t elias_delta::decode_prefix_sum(const uint64_t *data, const size_type start_idx, size_type n){
	if(n==0)
		return 0;
	data += (start_idx >> 6);
	uint64_t value = 0;
	size_type i = 0;
	uint8_t offset = start_idx & 0x3F;

	if( n < 24 ){
	if(n + offset <= 64 ){
		if( ((*data >> offset)&bit_magic::Li1Mask[n]) == bit_magic::Li1Mask[n] )
			return n;
	}else{// n+offset > 64
		if( (*data >> offset) == bit_magic::Li1Mask[64-offset] ){// all bits are set to 1
			value = 64-offset;
			++data;
			n -= (64-offset);
			offset = 0;
			while(n >= 64){
				if(*data == 0xFFFFFFFFFFFFFFFFULL){
					value += 64;
					++data;
					n -= 64;
				}else{
					uint8_t temp = bit_magic::r1BP(~(*data));
					value += temp;
					n -= temp;
					offset = temp;
					goto start_decoding;
				}
			}
			// 0 <= n <= 64
			if( (*data&bit_magic::Li1Mask[n]) == bit_magic::Li1Mask[n] )
				return value + n;
		}//else{
			
//		}
	}
	}
	
start_decoding:
	
	while( i < n ){// while not all values are decoded
		// n-i values to decode
		if( ((*data>>offset)&0xF)==0xF  ){
			uint8_t maxdecode = n-i > 63 ? 63 : n-i;
			uint8_t rbp = bit_magic::r1BP(~bit_magic::readInt(data, offset,maxdecode));
			i += rbp;
			value += rbp;
			if(rbp+offset>=64){ ++data; offset = (rbp+offset)&0x3F; }else{ offset += rbp; }
			if(rbp == maxdecode)
				continue;
		}
		while( i < n ){
			uint32_t psum = EliasDeltaPrefixSum[bit_magic::readInt(data, offset, 16)];
//			if( psum == 0 or i+((psum>>16)&0x00FF) > n ){ // value does not fit in 16 bits
			if( psum == 0 ){ // value does not fit in 16 bits
				goto decode_single;
			}
			else if( i+((psum>>16)&0x00FF) > n ){ // decoded too much 
				if(n-i <= 8){
					psum = EliasDeltaPrefixSum8bit[bit_magic::readInt(data, offset, 8) | ((n-i-1)<<8)];
					if( psum > 0 ){
						value += (psum&0xF);
						i += ((psum>>4)&0xF);
						offset += (psum>>8);
						if(offset>=64){offset&=0x3F; ++data; }
					}
				}
				break;
			}
			else{
				value += (psum&0x0000FFFF);
				i += ((psum>>16)&0x00FF);
				offset += (psum>>24);
				if(offset>=64){ offset&=0x3F; ++data; }
			}
		}
		if(i<n){
decode_single:			
			i++;	
			uint16_t len_1_len = bit_magic::readUnaryIntAndMove(data, offset); // read length of length of x
			uint16_t len_1 	=  bit_magic::readIntAndMove(data, offset, len_1_len) + (1ULL << len_1_len) - 1;
			value	+= bit_magic::readIntAndMove(data, offset, len_1) + (1ULL << (len_1));
		}
	}
	return value;
}

/*
 // naive decoding method
uint64_t elias_delta::decode_prefix_sum(const uint64_t *data, const size_type start_idx, size_type n){
	data += (start_idx >> 6);
	uint64_t value = 0;
	size_type i = 0;
	uint8_t offset = start_idx & 0x3F;//, maxdecode;
	while( i++ < n ){// while not all values are decoded
			uint16_t len_1_len = bit_magic::readUnaryIntAndMove(data, offset); // read length of length of x
			uint16_t len_1 	=  bit_magic::readIntAndMove(data, offset, len_1_len) + (1ULL << len_1_len) - 1;
			value	+= bit_magic::readIntAndMove(data, offset, len_1) + (1ULL << (len_1));
	}
	return value;
}
*/

const uint32_t elias_delta::EliasDeltaPrefixSum[]={
0x00000000,0x01010001,0x04010002,0x02020002,0x08010008,0x05020003,0x05010004,0x03030003,
0x0e010080,0x09020009,0x04010003,0x06030004,0x09010010,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f020081,0x05020003,0x0a03000a,0x0a010020,0x05020004,0x05010006,0x07040005,
0x0f010100,0x0a020011,0x05020004,0x07030006,0x0b010040,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x08020004,0x10030082,0x08010009,0x06030004,0x06020005,0x0b04000b,
0x10010200,0x0b020021,0x08020005,0x06030005,0x09010011,0x06020007,0x06020006,0x08050006,
0x00000000,0x10020101,0x06030004,0x0b030012,0x0a010021,0x06030005,0x06020007,0x08040007,
0x00000000,0x0c020041,0x06030005,0x07030007,0x0b010041,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0c02000a,0x02020002,0x0801000a,0x09030005,0x09020006,0x03030003,
0x00000000,0x0902000a,0x0c02000b,0x07040005,0x09010012,0x07030006,0x09020007,0x0c05000c,
0x00000000,0x01010001,0x09030005,0x0c030022,0x0a010022,0x09030006,0x09020008,0x07040006,
0x00000000,0x0a020012,0x09030006,0x07030008,0x0b010042,0x07030007,0x09020009,0x09060007,
0x00000000,0x01010001,0x09020006,0x02020002,0x0801000b,0x07040005,0x07030006,0x0c040013,
0x00000000,0x0b020022,0x09020007,0x07040006,0x09010013,0x07030008,0x07030007,0x09050008,
0x00000000,0x01010001,0x07040005,0x0d030042,0x0a010023,0x07040006,0x07030008,0x08040008,
0x00000000,0x0c020042,0x07040006,0x07030009,0x0b010043,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0801000c,0x0d03000b,0x0d02000c,0x03030003,
0x0e010081,0x0902000b,0x04010003,0x0a040006,0x09010014,0x0a030007,0x0d02000d,0x04040004,
0x00000000,0x01010001,0x0d03000b,0x0a03000b,0x0a010024,0x0d03000c,0x0d02000e,0x08050006,
0x0f010101,0x0a020013,0x0d03000c,0x08040007,0x0b010044,0x0a030008,0x0d02000f,0x0d06000d,
0x00000000,0x01010001,0x08020005,0x02020002,0x0801000d,0x0a040006,0x0a030007,0x0d040023,
0x10010201,0x0b020023,0x08020006,0x0a040007,0x09010015,0x0a030009,0x0a030008,0x08050007,
0x00000000,0x01010001,0x0a040006,0x0b030013,0x0a010025,0x0a040007,0x0a030009,0x08040009,
0x00000000,0x0c020043,0x0a040007,0x08040008,0x0b010045,0x0a03000a,0x0a03000a,0x0a070008,
0x00000000,0x01010001,0x0d020012,0x02020002,0x0801000e,0x0a030007,0x0a020008,0x03030003,
0x00000000,0x0902000c,0x0d020013,0x08050006,0x09010016,0x08040007,0x0a020009,0x0d050014,
0x00000000,0x01010001,0x0a030007,0x0c030023,0x0a010026,0x0a030008,0x0a02000a,0x08050007,
0x00000000,0x0a020014,0x0a030008,0x08040009,0x0b010046,0x08040008,0x0a02000b,0x0a060009,
0x00000000,0x01010001,0x09020007,0x02020002,0x0801000f,0x08050006,0x08040007,0x0e040043,
0x00000000,0x0b020024,0x09020008,0x08050007,0x09010017,0x08040009,0x08040008,0x09050009,
0x00000000,0x01010001,0x08050006,0x0d030043,0x0a010027,0x08050007,0x08040009,0x0804000a,
0x00000000,0x0c020044,0x08050007,0x0804000a,0x0b010047,0x0804000a,0x0804000a,0x08080008,
0x00000000,0x01010001,0x04010002,0x02020002,0x09020009,0x05020003,0x05010004,0x03030003,
0x0e010082,0x0902000d,0x04010003,0x0e04000c,0x09010018,0x0e03000d,0x05010005,0x04040004,
0x00000000,0x0f020082,0x05020003,0x0a03000c,0x0a010028,0x05020004,0x05010006,0x0b050007,
0x0f010102,0x0a020015,0x05020004,0x0b040008,0x0b010048,0x0e03000e,0x05010007,0x05050005,
0x00000000,0x01010001,0x09030005,0x02020002,0x0902000a,0x0e04000c,0x0e03000d,0x0b04000c,
0x10010202,0x0b020025,0x09030006,0x0e04000d,0x09010019,0x0e03000f,0x0e03000e,0x09060007,
0x00000000,0x10020102,0x0e04000c,0x0b030014,0x0a010029,0x0e04000d,0x0e03000f,0x09050008,
0x00000000,0x0c020045,0x0e04000d,0x0b040009,0x0b010049,0x0e030010,0x0e030010,0x0e07000e,
0x00000000,0x01010001,0x0e020022,0x02020002,0x0902000b,0x09030006,0x09020007,0x03030003,
0x00000000,0x0902000e,0x0e020023,0x0b050007,0x0901001a,0x0b040008,0x09020008,0x0e050024,
0x00000000,0x01010001,0x09030006,0x0c030024,0x0a01002a,0x09030007,0x09020009,0x0b050008,
0x00000000,0x0a020016,0x09030007,0x0b04000a,0x0b01004a,0x0b040009,0x0902000a,0x09060008,
0x00000000,0x01010001,0x09020008,0x02020002,0x0902000c,0x0b050007,0x0b040008,0x0c040014,
0x00000000,0x0b020026,0x09020009,0x0b050008,0x0901001b,0x0b04000a,0x0b040009,0x0905000a,
0x00000000,0x01010001,0x0b050007,0x0d030044,0x0a01002b,0x0b050008,0x0b04000a,0x09050009,
0x00000000,0x0c020046,0x0b050008,0x0b04000b,0x0b01004b,0x0b04000b,0x0b04000b,0x0b080009,
0x00000000,0x01010001,0x04010002,0x02020002,0x0902000d,0x0e030013,0x0e020014,0x03030003,
0x0e010083,0x0902000f,0x04010003,0x0b040008,0x0901001c,0x0b030009,0x0e020015,0x04040004,
0x00000000,0x01010001,0x0e030013,0x0a03000d,0x0a01002c,0x0e030014,0x0e020016,0x09060007,
0x0f010103,0x0a020017,0x0e030014,0x09050008,0x0b01004c,0x0b03000a,0x0e020017,0x0e060015,
0x00000000,0x01010001,0x09030006,0x02020002,0x0902000e,0x0b040008,0x0b030009,0x0d040024,
0x10010203,0x0b020027,0x09030007,0x0b040009,0x0901001d,0x0b03000b,0x0b03000a,0x09060008,
0x00000000,0x01010001,0x0b040008,0x0b030015,0x0a01002d,0x0b040009,0x0b03000b,0x0905000a,
0x00000000,0x0c020047,0x0b040009,0x09050009,0x0b01004d,0x0b03000c,0x0b03000c,0x0b07000a,
0x00000000,0x01010001,0x0f020042,0x02020002,0x0902000f,0x0a030008,0x0a020009,0x03030003,
0x00000000,0x09020010,0x0f020043,0x09060007,0x0901001e,0x09050008,0x0a02000a,0x0f050044,
0x00000000,0x01010001,0x0a030008,0x0c030025,0x0a01002e,0x0a030009,0x0a02000b,0x09060008,
0x00000000,0x0a020018,0x0a030009,0x0905000a,0x0b01004e,0x09050009,0x0a02000c,0x0a06000a,
0x00000000,0x01010001,0x09020009,0x02020002,0x09020010,0x09060007,0x09050008,0x0e040044,
0x00000000,0x0b020028,0x0902000a,0x09060008,0x0901001f,0x0905000a,0x09050009,0x0905000b,
0x00000000,0x01010001,0x09060007,0x0d030045,0x0a01002f,0x09060008,0x0905000a,0x0905000b,
0x00000000,0x0c020048,0x09060008,0x0905000b,0x0b01004f,0x0905000b,0x0905000b,0x09090009,
0x00000000,0x01010001,0x04010002,0x02020002,0x0c02000a,0x05020003,0x05010004,0x03030003,
0x0e010084,0x0a03000a,0x04010003,0x06030004,0x0a020011,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f020083,0x05020003,0x0a03000e,0x0a010030,0x05020004,0x05010006,0x0f05000d,
0x0f010104,0x0a020019,0x05020004,0x0f04000e,0x0b010050,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0c030006,0x10030083,0x0c02000b,0x06030004,0x06020005,0x0b04000d,
0x10010204,0x0b020029,0x0c030007,0x06030005,0x0a020012,0x06020007,0x06020006,0x0c060008,
0x00000000,0x10020103,0x06030004,0x0b030016,0x0a010031,0x06030005,0x06020007,0x0c050009,
0x00000000,0x0c020049,0x06030005,0x0f04000f,0x0b010051,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0c02000b,0x02020002,0x0c02000c,0x0a040006,0x0a030007,0x03030003,
0x00000000,0x0a03000b,0x0c02000c,0x0f05000d,0x0a020013,0x0f04000e,0x0a030008,0x0c05000d,
0x00000000,0x01010001,0x0a040006,0x0c030026,0x0a010032,0x0a040007,0x0a030009,0x0f05000e,
0x00000000,0x0a02001a,0x0a040007,0x0f040010,0x0b010052,0x0f04000f,0x0a03000a,0x0a070008,
0x00000000,0x01010001,0x0a030007,0x02020002,0x0c02000d,0x0f05000d,0x0f04000e,0x0c040015,
0x00000000,0x0b02002a,0x0a030008,0x0f05000e,0x0a020014,0x0f040010,0x0f04000f,0x0a060009,
0x00000000,0x01010001,0x0f05000d,0x0d030046,0x0a010033,0x0f05000e,0x0f040010,0x0c05000a,
0x00000000,0x0c02004a,0x0f05000e,0x0f040011,0x0b010053,0x0f040011,0x0f040011,0x0f08000f,
0x00000000,0x01010001,0x04010002,0x02020002,0x0c02000e,0x0f030023,0x0f020024,0x03030003,
0x0e010085,0x0a03000c,0x04010003,0x0a040007,0x0a020015,0x0a030008,0x0f020025,0x04040004,
0x00000000,0x01010001,0x0f030023,0x0a03000f,0x0a010034,0x0f030024,0x0f020026,0x0c060008,
0x0f010105,0x0a02001b,0x0f030024,0x0c050009,0x0b010054,0x0a030009,0x0f020027,0x0f060025,
0x00000000,0x01010001,0x0c030007,0x02020002,0x0c02000f,0x0a040007,0x0a030008,0x0d040025,
0x10010205,0x0b02002b,0x0c030008,0x0a040008,0x0a020016,0x0a03000a,0x0a030009,0x0c060009,
0x00000000,0x01010001,0x0a040007,0x0b030017,0x0a010035,0x0a040008,0x0a03000a,0x0c05000b,
0x00000000,0x0c02004b,0x0a040008,0x0c05000a,0x0b010055,0x0a03000b,0x0a03000b,0x0a070009,
0x00000000,0x01010001,0x0d020013,0x02020002,0x0c020010,0x0a030009,0x0a02000a,0x03030003,
0x00000000,0x0a03000d,0x0d020014,0x0c060008,0x0a020017,0x0c050009,0x0a02000b,0x0d050015,
0x00000000,0x01010001,0x0a030009,0x0c030027,0x0a010036,0x0a03000a,0x0a02000c,0x0c060009,
0x00000000,0x0a02001c,0x0a03000a,0x0c05000b,0x0b010056,0x0c05000a,0x0a02000d,0x0a06000b,
0x00000000,0x01010001,0x0a030008,0x02020002,0x0c020011,0x0c060008,0x0c050009,0x0e040045,
0x00000000,0x0b02002c,0x0a030009,0x0c060009,0x0a020018,0x0c05000b,0x0c05000a,0x0a06000a,
0x00000000,0x01010001,0x0c060008,0x0d030047,0x0a010037,0x0c060009,0x0c05000b,0x0c05000c,
0x00000000,0x0c02004c,0x0c060009,0x0c05000c,0x0b010057,0x0c05000c,0x0c05000c,0x0c09000a,
0x00000000,0x01010001,0x04010002,0x02020002,0x0a03000a,0x05020003,0x05010004,0x03030003,
0x0e010086,0x0a03000e,0x04010003,0x0f040014,0x0a020019,0x0f030015,0x05010005,0x04040004,
0x00000000,0x0f020084,0x05020003,0x0a030010,0x0a010038,0x05020004,0x05010006,0x0c050009,
0x0f010106,0x0a02001d,0x05020004,0x0c04000a,0x0b010058,0x0f030016,0x05010007,0x05050005,
0x00000000,0x01010001,0x0a040006,0x02020002,0x0a03000b,0x0f040014,0x0f030015,0x0b04000e,
0x10010206,0x0b02002d,0x0a040007,0x0f040015,0x0a02001a,0x0f030017,0x0f030016,0x0a070008,
0x00000000,0x10020104,0x0f040014,0x0b030018,0x0a010039,0x0f040015,0x0f030017,0x0a060009,
0x00000000,0x0c02004d,0x0f040015,0x0c04000b,0x0b010059,0x0f030018,0x0f030018,0x0f070016,
0x00000000,0x01010001,0x0e020023,0x02020002,0x0a03000c,0x0a040007,0x0a030008,0x03030003,
0x00000000,0x0a03000f,0x0e020024,0x0c050009,0x0a02001b,0x0c04000a,0x0a030009,0x0e050025,
0x00000000,0x01010001,0x0a040007,0x0c030028,0x0a01003a,0x0a040008,0x0a03000a,0x0c05000a,
0x00000000,0x0a02001e,0x0a040008,0x0c04000c,0x0b01005a,0x0c04000b,0x0a03000b,0x0a070009,
0x00000000,0x01010001,0x0a030009,0x02020002,0x0a03000d,0x0c050009,0x0c04000a,0x0c040016,
0x00000000,0x0b02002e,0x0a03000a,0x0c05000a,0x0a02001c,0x0c04000c,0x0c04000b,0x0a06000b,
0x00000000,0x01010001,0x0c050009,0x0d030048,0x0a01003b,0x0c05000a,0x0c04000c,0x0a06000a,
0x00000000,0x0c02004e,0x0c05000a,0x0c04000d,0x0b01005b,0x0c04000d,0x0c04000d,0x0c08000b,
0x00000000,0x01010001,0x04010002,0x02020002,0x0a03000e,0x10030043,0x10020044,0x03030003,
0x0e010087,0x0a030010,0x04010003,0x0b040009,0x0a02001d,0x0b03000a,0x10020045,0x04040004,
0x00000000,0x01010001,0x10030043,0x0a030011,0x0a01003c,0x10030044,0x10020046,0x0a070008,
0x0f010107,0x0a02001f,0x10030044,0x0a060009,0x0b01005c,0x0b03000b,0x10020047,0x10060045,
0x00000000,0x01010001,0x0a040007,0x02020002,0x0a03000f,0x0b040009,0x0b03000a,0x0d040026,
0x10010207,0x0b02002f,0x0a040008,0x0b04000a,0x0a02001e,0x0b03000c,0x0b03000b,0x0a070009,
0x00000000,0x01010001,0x0b040009,0x0b030019,0x0a01003d,0x0b04000a,0x0b03000c,0x0a06000b,
0x00000000,0x0c02004f,0x0b04000a,0x0a06000a,0x0b01005d,0x0b03000d,0x0b03000d,0x0b07000b,
0x00000000,0x01010001,0x0f020043,0x02020002,0x0a030010,0x0a03000a,0x0a02000b,0x03030003,
0x00000000,0x0a030011,0x0f020044,0x0a070008,0x0a02001f,0x0a060009,0x0a02000c,0x0f050045,
0x00000000,0x01010001,0x0a03000a,0x0c030029,0x0a01003e,0x0a03000b,0x0a02000d,0x0a070009,
0x00000000,0x0a020020,0x0a03000b,0x0a06000b,0x0b01005e,0x0a06000a,0x0a02000e,0x0a06000c,
0x00000000,0x01010001,0x0a03000a,0x02020002,0x0a030011,0x0a070008,0x0a060009,0x0e040046,
0x00000000,0x0b020030,0x0a03000b,0x0a070009,0x0a020020,0x0a06000b,0x0a06000a,0x0a06000c,
0x00000000,0x01010001,0x0a070008,0x0d030049,0x0a01003f,0x0a070009,0x0a06000b,0x0a06000c,
0x00000000,0x0c020050,0x0a070009,0x0a06000c,0x0b01005f,0x0a06000c,0x0a06000c,0x0a0a000a,
0x00000000,0x01010001,0x04010002,0x02020002,0x10020010,0x05020003,0x05010004,0x03030003,
0x0e010088,0x0d03000b,0x04010003,0x06030004,0x0d020012,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f020085,0x05020003,0x0b04000b,0x0b020021,0x05020004,0x05010006,0x07040005,
0x0f010108,0x0b030012,0x05020004,0x07030006,0x0b010060,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x1003000c,0x10030084,0x10020011,0x06030004,0x06020005,0x0b04000f,
0x10010208,0x0b020031,0x1003000d,0x06030005,0x0d020013,0x06020007,0x06020006,0x1006000e,
0x00000000,0x10020105,0x06030004,0x0b03001a,0x0b020022,0x06030005,0x06020007,0x1005000f,
0x00000000,0x0c020051,0x06030005,0x07030007,0x0b010061,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0c02000c,0x02020002,0x10020012,0x0d040007,0x0d030008,0x03030003,
0x00000000,0x0d03000c,0x0c02000d,0x07040005,0x0d020014,0x07030006,0x0d030009,0x0c05000e,
0x00000000,0x01010001,0x0d040007,0x0c03002a,0x0b020023,0x0d040008,0x0d03000a,0x07040006,
0x00000000,0x0b030013,0x0d040008,0x07030008,0x0b010062,0x07030007,0x0d03000b,0x0d070009,
0x00000000,0x01010001,0x0d030008,0x02020002,0x10020013,0x07040005,0x07030006,0x0c040017,
0x00000000,0x0b020032,0x0d030009,0x07040006,0x0d020015,0x07030008,0x07030007,0x0d06000a,
0x00000000,0x01010001,0x07040005,0x0d03004a,0x0b020024,0x07040006,0x07030008,0x10050010,
0x00000000,0x0c020052,0x07040006,0x07030009,0x0b010063,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x10020014,0x0d03000c,0x0d02000d,0x03030003,
0x0e010089,0x0d03000d,0x04010003,0x0b050007,0x0d020016,0x0b040008,0x0d02000e,0x04040004,
0x00000000,0x01010001,0x0d03000c,0x0b04000c,0x0b020025,0x0d03000d,0x0d02000f,0x1006000e,
0x0f010109,0x0b030014,0x0d03000d,0x1005000f,0x0b010064,0x0b040009,0x0d020010,0x0d06000e,
0x00000000,0x01010001,0x1003000d,0x02020002,0x10020015,0x0b050007,0x0b040008,0x0d040027,
0x10010209,0x0b020033,0x1003000e,0x0b050008,0x0d020017,0x0b04000a,0x0b040009,0x1006000f,
0x00000000,0x01010001,0x0b050007,0x0b03001b,0x0b020026,0x0b050008,0x0b04000a,0x10050011,
0x00000000,0x0c020053,0x0b050008,0x10050010,0x0b010065,0x0b04000b,0x0b04000b,0x0b080009,
0x00000000,0x01010001,0x0d020014,0x02020002,0x10020016,0x0b040008,0x0b030009,0x03030003,
0x00000000,0x0d03000e,0x0d020015,0x1006000e,0x0d020018,0x1005000f,0x0b03000a,0x0d050016,
0x00000000,0x01010001,0x0b040008,0x0c03002b,0x0b020027,0x0b040009,0x0b03000b,0x1006000f,
0x00000000,0x0b030015,0x0b040009,0x10050011,0x0b010066,0x10050010,0x0b03000c,0x0b07000a,
0x00000000,0x01010001,0x0d030009,0x02020002,0x10020017,0x1006000e,0x1005000f,0x0e040047,
0x00000000,0x0b020034,0x0d03000a,0x1006000f,0x0d020019,0x10050011,0x10050010,0x0d06000b,
0x00000000,0x01010001,0x1006000e,0x0d03004b,0x0b020028,0x1006000f,0x10050011,0x10050012,
0x00000000,0x0c020054,0x1006000f,0x10050012,0x0b010067,0x10050012,0x10050012,0x10090010,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d03000b,0x05020003,0x05010004,0x03030003,
0x0e01008a,0x0d03000f,0x04010003,0x10040024,0x0d02001a,0x10030025,0x05010005,0x04040004,
0x00000000,0x0f020086,0x05020003,0x0b04000d,0x0b020029,0x05020004,0x05010006,0x0b050008,
0x0f01010a,0x0b030016,0x05020004,0x0b040009,0x0b010068,0x10030026,0x05010007,0x05050005,
0x00000000,0x01010001,0x0d040007,0x02020002,0x0d03000c,0x10040024,0x10030025,0x0b040010,
0x1001020a,0x0b020035,0x0d040008,0x10040025,0x0d02001b,0x10030027,0x10030026,0x0d070009,
0x00000000,0x10020106,0x10040024,0x0b03001c,0x0b02002a,0x10040025,0x10030027,0x0d06000a,
0x00000000,0x0c020055,0x10040025,0x0b04000a,0x0b010069,0x10030028,0x10030028,0x10070026,
0x00000000,0x01010001,0x0e020024,0x02020002,0x0d03000d,0x0d040008,0x0d030009,0x03030003,
0x00000000,0x0d030010,0x0e020025,0x0b050008,0x0d02001c,0x0b040009,0x0d03000a,0x0e050026,
0x00000000,0x01010001,0x0d040008,0x0c03002c,0x0b02002b,0x0d040009,0x0d03000b,0x0b050009,
0x00000000,0x0b030017,0x0d040009,0x0b04000b,0x0b01006a,0x0b04000a,0x0d03000c,0x0d07000a,
0x00000000,0x01010001,0x0d03000a,0x02020002,0x0d03000e,0x0b050008,0x0b040009,0x0c040018,
0x00000000,0x0b020036,0x0d03000b,0x0b050009,0x0d02001d,0x0b04000b,0x0b04000a,0x0d06000c,
0x00000000,0x01010001,0x0b050008,0x0d03004c,0x0b02002c,0x0b050009,0x0b04000b,0x0d06000b,
0x00000000,0x0c020056,0x0b050009,0x0b04000c,0x0b01006b,0x0b04000c,0x0b04000c,0x0b08000a,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d03000f,0x0e030014,0x0e020015,0x03030003,
0x0e01008b,0x0d030011,0x04010003,0x0b04000a,0x0d02001e,0x0b03000b,0x0e020016,0x04040004,
0x00000000,0x01010001,0x0e030014,0x0b04000e,0x0b02002d,0x0e030015,0x0e020017,0x0d070009,
0x0f01010b,0x0b030018,0x0e030015,0x0d06000a,0x0b01006c,0x0b03000c,0x0e020018,0x0e060016,
0x00000000,0x01010001,0x0d040008,0x02020002,0x0d030010,0x0b04000a,0x0b03000b,0x0d040028,
0x1001020b,0x0b020037,0x0d040009,0x0b04000b,0x0d02001f,0x0b03000d,0x0b03000c,0x0d07000a,
0x00000000,0x01010001,0x0b04000a,0x0b03001d,0x0b02002e,0x0b04000b,0x0b03000d,0x0d06000c,
0x00000000,0x0c020057,0x0b04000b,0x0d06000b,0x0b01006d,0x0b03000e,0x0b03000e,0x0b07000c,
0x00000000,0x01010001,0x0f020044,0x02020002,0x0d030011,0x0b040009,0x0b03000a,0x03030003,
0x00000000,0x0d030012,0x0f020045,0x0d070009,0x0d020020,0x0d06000a,0x0b03000b,0x0f050046,
0x00000000,0x01010001,0x0b040009,0x0c03002d,0x0b02002f,0x0b04000a,0x0b03000c,0x0d07000a,
0x00000000,0x0b030019,0x0b04000a,0x0d06000c,0x0b01006e,0x0d06000b,0x0b03000d,0x0b07000b,
0x00000000,0x01010001,0x0d03000b,0x02020002,0x0d030012,0x0d070009,0x0d06000a,0x0e040048,
0x00000000,0x0b020038,0x0d03000c,0x0d07000a,0x0d020021,0x0d06000c,0x0d06000b,0x0d06000d,
0x00000000,0x01010001,0x0d070009,0x0d03004d,0x0b020030,0x0d07000a,0x0d06000c,0x0d06000d,
0x00000000,0x0c020058,0x0d07000a,0x0d06000d,0x0b01006f,0x0d06000d,0x0d06000d,0x0d0a000b,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d02000c,0x05020003,0x05010004,0x03030003,
0x0e01008c,0x0b04000b,0x04010003,0x06030004,0x0b030012,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f020087,0x05020003,0x0b04000f,0x0b020031,0x05020004,0x05010006,0x10050015,
0x0f01010c,0x0b03001a,0x05020004,0x10040016,0x0b010070,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0d030008,0x10030085,0x0d02000d,0x06030004,0x06020005,0x0b040011,
0x1001020c,0x0b020039,0x0d030009,0x06030005,0x0b030013,0x06020007,0x06020006,0x0d06000a,
0x00000000,0x10020107,0x06030004,0x0b03001e,0x0b020032,0x06030005,0x06020007,0x0d05000b,
0x00000000,0x0c020059,0x06030005,0x10040017,0x0b010071,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0c02000d,0x02020002,0x0d02000e,0x0b050007,0x0b040008,0x03030003,
0x00000000,0x0b04000c,0x0c02000e,0x10050015,0x0b030014,0x10040016,0x0b040009,0x0c05000f,
0x00000000,0x01010001,0x0b050007,0x0c03002e,0x0b020033,0x0b050008,0x0b04000a,0x10050016,
0x00000000,0x0b03001b,0x0b050008,0x10040018,0x0b010072,0x10040017,0x0b04000b,0x0b080009,
0x00000000,0x01010001,0x0b040008,0x02020002,0x0d02000f,0x10050015,0x10040016,0x0c040019,
0x00000000,0x0b02003a,0x0b040009,0x10050016,0x0b030015,0x10040018,0x10040017,0x0b07000a,
0x00000000,0x01010001,0x10050015,0x0d03004e,0x0b020034,0x10050016,0x10040018,0x0d05000c,
0x00000000,0x0c02005a,0x10050016,0x10040019,0x0b010073,0x10040019,0x10040019,0x10080017,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d020010,0x0f030024,0x0f020025,0x03030003,
0x0e01008d,0x0b04000d,0x04010003,0x0b050008,0x0b030016,0x0b040009,0x0f020026,0x04040004,
0x00000000,0x01010001,0x0f030024,0x0b040010,0x0b020035,0x0f030025,0x0f020027,0x0d06000a,
0x0f01010d,0x0b03001c,0x0f030025,0x0d05000b,0x0b010074,0x0b04000a,0x0f020028,0x0f060026,
0x00000000,0x01010001,0x0d030009,0x02020002,0x0d020011,0x0b050008,0x0b040009,0x0d040029,
0x1001020d,0x0b02003b,0x0d03000a,0x0b050009,0x0b030017,0x0b04000b,0x0b04000a,0x0d06000b,
0x00000000,0x01010001,0x0b050008,0x0b03001f,0x0b020036,0x0b050009,0x0b04000b,0x0d05000d,
0x00000000,0x0c02005b,0x0b050009,0x0d05000c,0x0b010075,0x0b04000c,0x0b04000c,0x0b08000a,
0x00000000,0x01010001,0x0d020015,0x02020002,0x0d020012,0x0b04000a,0x0b03000b,0x03030003,
0x00000000,0x0b04000e,0x0d020016,0x0d06000a,0x0b030018,0x0d05000b,0x0b03000c,0x0d050017,
0x00000000,0x01010001,0x0b04000a,0x0c03002f,0x0b020037,0x0b04000b,0x0b03000d,0x0d06000b,
0x00000000,0x0b03001d,0x0b04000b,0x0d05000d,0x0b010076,0x0d05000c,0x0b03000e,0x0b07000c,
0x00000000,0x01010001,0x0b040009,0x02020002,0x0d020013,0x0d06000a,0x0d05000b,0x0e040049,
0x00000000,0x0b02003c,0x0b04000a,0x0d06000b,0x0b030019,0x0d05000d,0x0d05000c,0x0b07000b,
0x00000000,0x01010001,0x0d06000a,0x0d03004f,0x0b020038,0x0d06000b,0x0d05000d,0x0d05000e,
0x00000000,0x0c02005c,0x0d06000b,0x0d05000e,0x0b010077,0x0d05000e,0x0d05000e,0x0d09000c,
0x00000000,0x01010001,0x04010002,0x02020002,0x0b04000b,0x05020003,0x05010004,0x03030003,
0x0e01008e,0x0b04000f,0x04010003,0x06030004,0x0b03001a,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f020088,0x05020003,0x0b040011,0x0b020039,0x05020004,0x05010006,0x0c05000a,
0x0f01010e,0x0b03001e,0x05020004,0x0c04000b,0x0b010078,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0b050007,0x02020002,0x0b04000c,0x06030004,0x06020005,0x0b040012,
0x1001020e,0x0b02003d,0x0b050008,0x06030005,0x0b03001b,0x06020007,0x06020006,0x0b080009,
0x00000000,0x10020108,0x06030004,0x0b030020,0x0b02003a,0x06030005,0x06020007,0x0b07000a,
0x00000000,0x0c02005d,0x06030005,0x0c04000c,0x0b010079,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0e020025,0x02020002,0x0b04000d,0x0b050008,0x0b040009,0x03030003,
0x00000000,0x0b040010,0x0e020026,0x0c05000a,0x0b03001c,0x0c04000b,0x0b04000a,0x0e050027,
0x00000000,0x01010001,0x0b050008,0x0c030030,0x0b02003b,0x0b050009,0x0b04000b,0x0c05000b,
0x00000000,0x0b03001f,0x0b050009,0x0c04000d,0x0b01007a,0x0c04000c,0x0b04000c,0x0b08000a,
0x00000000,0x01010001,0x0b04000a,0x02020002,0x0b04000e,0x0c05000a,0x0c04000b,0x0c04001a,
0x00000000,0x0b02003e,0x0b04000b,0x0c05000b,0x0b03001d,0x0c04000d,0x0c04000c,0x0b07000c,
0x00000000,0x01010001,0x0c05000a,0x0d030050,0x0b02003c,0x0c05000b,0x0c04000d,0x0b07000b,
0x00000000,0x0c02005e,0x0c05000b,0x0c04000e,0x0b01007b,0x0c04000e,0x0c04000e,0x0c08000c,
0x00000000,0x01010001,0x04010002,0x02020002,0x0b04000f,0x10030044,0x10020045,0x03030003,
0x0e01008f,0x0b040011,0x04010003,0x0b04000b,0x0b03001e,0x0b03000c,0x10020046,0x04040004,
0x00000000,0x01010001,0x10030044,0x0b040012,0x0b02003d,0x10030045,0x10020047,0x0b080009,
0x0f01010f,0x0b030020,0x10030045,0x0b07000a,0x0b01007c,0x0b03000d,0x10020048,0x10060046,
0x00000000,0x01010001,0x0b050008,0x02020002,0x0b040010,0x0b04000b,0x0b03000c,0x0d04002a,
0x1001020f,0x0b02003f,0x0b050009,0x0b04000c,0x0b03001f,0x0b03000e,0x0b03000d,0x0b08000a,
0x00000000,0x01010001,0x0b04000b,0x0b030021,0x0b02003e,0x0b04000c,0x0b03000e,0x0b07000c,
0x00000000,0x0c02005f,0x0b04000c,0x0b07000b,0x0b01007d,0x0b03000f,0x0b03000f,0x0b07000d,
0x00000000,0x01010001,0x0f020045,0x02020002,0x0b040011,0x0b04000b,0x0b03000c,0x03030003,
0x00000000,0x0b040012,0x0f020046,0x0b080009,0x0b030020,0x0b07000a,0x0b03000d,0x0f050047,
0x00000000,0x01010001,0x0b04000b,0x0c030031,0x0b02003f,0x0b04000c,0x0b03000e,0x0b08000a,
0x00000000,0x0b030021,0x0b04000c,0x0b07000c,0x0b01007e,0x0b07000b,0x0b03000f,0x0b07000d,
0x00000000,0x01010001,0x0b04000b,0x02020002,0x0b040012,0x0b080009,0x0b07000a,0x0e04004a,
0x00000000,0x0b020040,0x0b04000c,0x0b08000a,0x0b030021,0x0b07000c,0x0b07000b,0x0b07000d,
0x00000000,0x01010001,0x0b080009,0x0d030051,0x0b020040,0x0b08000a,0x0b07000c,0x0b07000d,
0x00000000,0x0c020060,0x0b08000a,0x0b07000d,0x0b01007f,0x0b07000d,0x0b07000d,0x0b0b000b,
0x00000000,0x01010001,0x04010002,0x02020002,0x08010008,0x05020003,0x05010004,0x03030003,
0x0e010090,0x09020009,0x04010003,0x06030004,0x09010010,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f020089,0x05020003,0x0e04000c,0x0e020022,0x05020004,0x05010006,0x07040005,
0x0f010110,0x0e030013,0x05020004,0x07030006,0x0c020041,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x08020004,0x10030086,0x08010009,0x06030004,0x06020005,0x0c05000c,
0x10010210,0x0c030022,0x08020005,0x06030005,0x09010011,0x06020007,0x06020006,0x08050006,
0x00000000,0x10020109,0x06030004,0x0c040013,0x0e020023,0x06030005,0x06020007,0x08040007,
0x00000000,0x0c020061,0x06030005,0x07030007,0x0c020042,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0c02000e,0x02020002,0x0801000a,0x09030005,0x09020006,0x03030003,
0x00000000,0x0902000a,0x0c02000f,0x07040005,0x09010012,0x07030006,0x09020007,0x0c050010,
0x00000000,0x01010001,0x09030005,0x0c030032,0x0e020024,0x09030006,0x09020008,0x07040006,
0x00000000,0x0e030014,0x09030006,0x07030008,0x0c020043,0x07030007,0x09020009,0x09060007,
0x00000000,0x01010001,0x09020006,0x02020002,0x0801000b,0x07040005,0x07030006,0x0c04001b,
0x00000000,0x0c030023,0x09020007,0x07040006,0x09010013,0x07030008,0x07030007,0x09050008,
0x00000000,0x01010001,0x07040005,0x0d030052,0x0e020025,0x07040006,0x07030008,0x08040008,
0x00000000,0x0c020062,0x07040006,0x07030009,0x0c020044,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0801000c,0x0d03000d,0x0d02000e,0x03030003,
0x0e010091,0x0902000b,0x04010003,0x0e050008,0x09010014,0x0e040009,0x0d02000f,0x04040004,
0x00000000,0x01010001,0x0d03000d,0x0e04000d,0x0e020026,0x0d03000e,0x0d020010,0x08050006,
0x0f010111,0x0e030015,0x0d03000e,0x08040007,0x0c020045,0x0e04000a,0x0d020011,0x0d06000f,
0x00000000,0x01010001,0x08020005,0x02020002,0x0801000d,0x0e050008,0x0e040009,0x0d04002b,
0x10010211,0x0c030024,0x08020006,0x0e050009,0x09010015,0x0e04000b,0x0e04000a,0x08050007,
0x00000000,0x01010001,0x0e050008,0x0c040014,0x0e020027,0x0e050009,0x0e04000b,0x08040009,
0x00000000,0x0c020063,0x0e050009,0x08040008,0x0c020046,0x0e04000c,0x0e04000c,0x0e08000a,
0x00000000,0x01010001,0x0d020016,0x02020002,0x0801000e,0x0e040009,0x0e03000a,0x03030003,
0x00000000,0x0902000c,0x0d020017,0x08050006,0x09010016,0x08040007,0x0e03000b,0x0d050018,
0x00000000,0x01010001,0x0e040009,0x0c030033,0x0e020028,0x0e04000a,0x0e03000c,0x08050007,
0x00000000,0x0e030016,0x0e04000a,0x08040009,0x0c020047,0x08040008,0x0e03000d,0x0e07000b,
0x00000000,0x01010001,0x09020007,0x02020002,0x0801000f,0x08050006,0x08040007,0x0e04004b,
0x00000000,0x0c030025,0x09020008,0x08050007,0x09010017,0x08040009,0x08040008,0x09050009,
0x00000000,0x01010001,0x08050006,0x0d030053,0x0e020029,0x08050007,0x08040009,0x0804000a,
0x00000000,0x0c020064,0x08050007,0x0804000a,0x0c020048,0x0804000a,0x0804000a,0x08080008,
0x00000000,0x01010001,0x04010002,0x02020002,0x09020009,0x05020003,0x05010004,0x03030003,
0x0e010092,0x0902000d,0x04010003,0x0e04000d,0x09010018,0x0e03000e,0x05010005,0x04040004,
0x00000000,0x0f02008a,0x05020003,0x0e04000e,0x0e02002a,0x05020004,0x05010006,0x0c060008,
0x0f010112,0x0e030017,0x05020004,0x0c050009,0x0c020049,0x0e03000f,0x05010007,0x05050005,
0x00000000,0x01010001,0x09030005,0x02020002,0x0902000a,0x0e04000d,0x0e03000e,0x0c05000d,
0x10010212,0x0c030026,0x09030006,0x0e04000e,0x09010019,0x0e030010,0x0e03000f,0x09060007,
0x00000000,0x1002010a,0x0e04000d,0x0c040015,0x0e02002b,0x0e04000e,0x0e030010,0x09050008,
0x00000000,0x0c020065,0x0e04000e,0x0c05000a,0x0c02004a,0x0e030011,0x0e030011,0x0e07000f,
0x00000000,0x01010001,0x0e020026,0x02020002,0x0902000b,0x09030006,0x09020007,0x03030003,
0x00000000,0x0902000e,0x0e020027,0x0c060008,0x0901001a,0x0c050009,0x09020008,0x0e050028,
0x00000000,0x01010001,0x09030006,0x0c030034,0x0e02002c,0x09030007,0x09020009,0x0c060009,
0x00000000,0x0e030018,0x09030007,0x0c05000b,0x0c02004b,0x0c05000a,0x0902000a,0x09060008,
0x00000000,0x01010001,0x09020008,0x02020002,0x0902000c,0x0c060008,0x0c050009,0x0c04001c,
0x00000000,0x0c030027,0x09020009,0x0c060009,0x0901001b,0x0c05000b,0x0c05000a,0x0905000a,
0x00000000,0x01010001,0x0c060008,0x0d030054,0x0e02002d,0x0c060009,0x0c05000b,0x09050009,
0x00000000,0x0c020066,0x0c060009,0x0c05000c,0x0c02004c,0x0c05000c,0x0c05000c,0x0c09000a,
0x00000000,0x01010001,0x04010002,0x02020002,0x0902000d,0x0e030015,0x0e020016,0x03030003,
0x0e010093,0x0902000f,0x04010003,0x0c050009,0x0901001c,0x0c04000a,0x0e020017,0x04040004,
0x00000000,0x01010001,0x0e030015,0x0e04000f,0x0e02002e,0x0e030016,0x0e020018,0x09060007,
0x0f010113,0x0e030019,0x0e030016,0x09050008,0x0c02004d,0x0c04000b,0x0e020019,0x0e060017,
0x00000000,0x01010001,0x09030006,0x02020002,0x0902000e,0x0c050009,0x0c04000a,0x0d04002c,
0x10010213,0x0c030028,0x09030007,0x0c05000a,0x0901001d,0x0c04000c,0x0c04000b,0x09060008,
0x00000000,0x01010001,0x0c050009,0x0c040016,0x0e02002f,0x0c05000a,0x0c04000c,0x0905000a,
0x00000000,0x0c020067,0x0c05000a,0x09050009,0x0c02004e,0x0c04000d,0x0c04000d,0x0c08000b,
0x00000000,0x01010001,0x0f020046,0x02020002,0x0902000f,0x0e04000a,0x0e03000b,0x03030003,
0x00000000,0x09020010,0x0f020047,0x09060007,0x0901001e,0x09050008,0x0e03000c,0x0f050048,
0x00000000,0x01010001,0x0e04000a,0x0c030035,0x0e020030,0x0e04000b,0x0e03000d,0x09060008,
0x00000000,0x0e03001a,0x0e04000b,0x0905000a,0x0c02004f,0x09050009,0x0e03000e,0x0e07000c,
0x00000000,0x01010001,0x09020009,0x02020002,0x09020010,0x09060007,0x09050008,0x0e04004c,
0x00000000,0x0c030029,0x0902000a,0x09060008,0x0901001f,0x0905000a,0x09050009,0x0905000b,
0x00000000,0x01010001,0x09060007,0x0d030055,0x0e020031,0x09060008,0x0905000a,0x0905000b,
0x00000000,0x0c020068,0x09060008,0x0905000b,0x0c020050,0x0905000b,0x0905000b,0x09090009,
0x00000000,0x01010001,0x04010002,0x02020002,0x0c02000b,0x05020003,0x05010004,0x03030003,
0x0e010094,0x0e04000c,0x04010003,0x06030004,0x0e030013,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f02008b,0x05020003,0x0e040010,0x0e020032,0x05020004,0x05010006,0x07040005,
0x0f010114,0x0e03001b,0x05020004,0x07030006,0x0c020051,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0c030007,0x10030087,0x0c02000c,0x06030004,0x06020005,0x0c05000e,
0x10010214,0x0c03002a,0x0c030008,0x06030005,0x0e030014,0x06020007,0x06020006,0x0c060009,
0x00000000,0x1002010b,0x06030004,0x0c040017,0x0e020033,0x06030005,0x06020007,0x0c05000a,
0x00000000,0x0c020069,0x06030005,0x07030007,0x0c020052,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0c02000f,0x02020002,0x0c02000d,0x0e050008,0x0e040009,0x03030003,
0x00000000,0x0e04000d,0x0c020010,0x07040005,0x0e030015,0x07030006,0x0e04000a,0x0c050011,
0x00000000,0x01010001,0x0e050008,0x0c030036,0x0e020034,0x0e050009,0x0e04000b,0x07040006,
0x00000000,0x0e03001c,0x0e050009,0x07030008,0x0c020053,0x07030007,0x0e04000c,0x0e08000a,
0x00000000,0x01010001,0x0e040009,0x02020002,0x0c02000e,0x07040005,0x07030006,0x0c04001d,
0x00000000,0x0c03002b,0x0e04000a,0x07040006,0x0e030016,0x07030008,0x07030007,0x0e07000b,
0x00000000,0x01010001,0x07040005,0x0d030056,0x0e020035,0x07040006,0x07030008,0x0c05000b,
0x00000000,0x0c02006a,0x07040006,0x07030009,0x0c020054,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0c02000f,0x0f030025,0x0f020026,0x03030003,
0x0e010095,0x0e04000e,0x04010003,0x0e050009,0x0e030017,0x0e04000a,0x0f020027,0x04040004,
0x00000000,0x01010001,0x0f030025,0x0e040011,0x0e020036,0x0f030026,0x0f020028,0x0c060009,
0x0f010115,0x0e03001d,0x0f030026,0x0c05000a,0x0c020055,0x0e04000b,0x0f020029,0x0f060027,
0x00000000,0x01010001,0x0c030008,0x02020002,0x0c020010,0x0e050009,0x0e04000a,0x0d04002d,
0x10010215,0x0c03002c,0x0c030009,0x0e05000a,0x0e030018,0x0e04000c,0x0e04000b,0x0c06000a,
0x00000000,0x01010001,0x0e050009,0x0c040018,0x0e020037,0x0e05000a,0x0e04000c,0x0c05000c,
0x00000000,0x0c02006b,0x0e05000a,0x0c05000b,0x0c020056,0x0e04000d,0x0e04000d,0x0e08000b,
0x00000000,0x01010001,0x0d020017,0x02020002,0x0c020011,0x0e04000b,0x0e03000c,0x03030003,
0x00000000,0x0e04000f,0x0d020018,0x0c060009,0x0e030019,0x0c05000a,0x0e03000d,0x0d050019,
0x00000000,0x01010001,0x0e04000b,0x0c030037,0x0e020038,0x0e04000c,0x0e03000e,0x0c06000a,
0x00000000,0x0e03001e,0x0e04000c,0x0c05000c,0x0c020057,0x0c05000b,0x0e03000f,0x0e07000d,
0x00000000,0x01010001,0x0e04000a,0x02020002,0x0c020012,0x0c060009,0x0c05000a,0x0e04004d,
0x00000000,0x0c03002d,0x0e04000b,0x0c06000a,0x0e03001a,0x0c05000c,0x0c05000b,0x0e07000c,
0x00000000,0x01010001,0x0c060009,0x0d030057,0x0e020039,0x0c06000a,0x0c05000c,0x0c05000d,
0x00000000,0x0c02006c,0x0c06000a,0x0c05000d,0x0c020058,0x0c05000d,0x0c05000d,0x0c09000b,
0x00000000,0x01010001,0x04010002,0x02020002,0x0e04000c,0x05020003,0x05010004,0x03030003,
0x0e010096,0x0e040010,0x04010003,0x0f040015,0x0e03001b,0x0f030016,0x05010005,0x04040004,
0x00000000,0x0f02008c,0x05020003,0x0e040012,0x0e02003a,0x05020004,0x05010006,0x0c05000b,
0x0f010116,0x0e03001f,0x05020004,0x0c04000c,0x0c020059,0x0f030017,0x05010007,0x05050005,
0x00000000,0x01010001,0x0e050008,0x02020002,0x0e04000d,0x0f040015,0x0f030016,0x0c05000f,
0x10010216,0x0c03002e,0x0e050009,0x0f040016,0x0e03001c,0x0f030018,0x0f030017,0x0e08000a,
0x00000000,0x1002010c,0x0f040015,0x0c040019,0x0e02003b,0x0f040016,0x0f030018,0x0e07000b,
0x00000000,0x0c02006d,0x0f040016,0x0c04000d,0x0c02005a,0x0f030019,0x0f030019,0x0f070017,
0x00000000,0x01010001,0x0e020027,0x02020002,0x0e04000e,0x0e050009,0x0e04000a,0x03030003,
0x00000000,0x0e040011,0x0e020028,0x0c05000b,0x0e03001d,0x0c04000c,0x0e04000b,0x0e050029,
0x00000000,0x01010001,0x0e050009,0x0c030038,0x0e02003c,0x0e05000a,0x0e04000c,0x0c05000c,
0x00000000,0x0e030020,0x0e05000a,0x0c04000e,0x0c02005b,0x0c04000d,0x0e04000d,0x0e08000b,
0x00000000,0x01010001,0x0e04000b,0x02020002,0x0e04000f,0x0c05000b,0x0c04000c,0x0c04001e,
0x00000000,0x0c03002f,0x0e04000c,0x0c05000c,0x0e03001e,0x0c04000e,0x0c04000d,0x0e07000d,
0x00000000,0x01010001,0x0c05000b,0x0d030058,0x0e02003d,0x0c05000c,0x0c04000e,0x0e07000c,
0x00000000,0x0c02006e,0x0c05000c,0x0c04000f,0x0c02005c,0x0c04000f,0x0c04000f,0x0c08000d,
0x00000000,0x01010001,0x04010002,0x02020002,0x0e040010,0x10030045,0x10020046,0x03030003,
0x0e010097,0x0e040012,0x04010003,0x0c05000a,0x0e03001f,0x0c04000b,0x10020047,0x04040004,
0x00000000,0x01010001,0x10030045,0x0e040013,0x0e02003e,0x10030046,0x10020048,0x0e08000a,
0x0f010117,0x0e030021,0x10030046,0x0e07000b,0x0c02005d,0x0c04000c,0x10020049,0x10060047,
0x00000000,0x01010001,0x0e050009,0x02020002,0x0e040011,0x0c05000a,0x0c04000b,0x0d04002e,
0x10010217,0x0c030030,0x0e05000a,0x0c05000b,0x0e030020,0x0c04000d,0x0c04000c,0x0e08000b,
0x00000000,0x01010001,0x0c05000a,0x0c04001a,0x0e02003f,0x0c05000b,0x0c04000d,0x0e07000d,
0x00000000,0x0c02006f,0x0c05000b,0x0e07000c,0x0c02005e,0x0c04000e,0x0c04000e,0x0c08000c,
0x00000000,0x01010001,0x0f020047,0x02020002,0x0e040012,0x0e04000c,0x0e03000d,0x03030003,
0x00000000,0x0e040013,0x0f020048,0x0e08000a,0x0e030021,0x0e07000b,0x0e03000e,0x0f050049,
0x00000000,0x01010001,0x0e04000c,0x0c030039,0x0e020040,0x0e04000d,0x0e03000f,0x0e08000b,
0x00000000,0x0e030022,0x0e04000d,0x0e07000d,0x0c02005f,0x0e07000c,0x0e030010,0x0e07000e,
0x00000000,0x01010001,0x0e04000c,0x02020002,0x0e040013,0x0e08000a,0x0e07000b,0x0e04004e,
0x00000000,0x0c030031,0x0e04000d,0x0e08000b,0x0e030022,0x0e07000d,0x0e07000c,0x0e07000e,
0x00000000,0x01010001,0x0e08000a,0x0d030059,0x0e020041,0x0e08000b,0x0e07000d,0x0e07000e,
0x00000000,0x0c020070,0x0e08000b,0x0e07000e,0x0c020060,0x0e07000e,0x0e07000e,0x0e0b000c,
0x00000000,0x01010001,0x04010002,0x02020002,0x08010008,0x05020003,0x05010004,0x03030003,
0x0e010098,0x0e03000d,0x04010003,0x06030004,0x0e020014,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f02008d,0x05020003,0x0c05000c,0x0c030022,0x05020004,0x05010006,0x07040005,
0x0f010118,0x0c040013,0x05020004,0x07030006,0x0c020061,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x08020004,0x10030088,0x08010009,0x06030004,0x06020005,0x0c050010,
0x10010218,0x0c030032,0x08020005,0x06030005,0x0e020015,0x06020007,0x06020006,0x08050006,
0x00000000,0x1002010d,0x06030004,0x0c04001b,0x0c030023,0x06030005,0x06020007,0x08040007,
0x00000000,0x0c020071,0x06030005,0x07030007,0x0c020062,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0c020010,0x02020002,0x0801000a,0x0e040009,0x0e03000a,0x03030003,
0x00000000,0x0e03000e,0x0c020011,0x07040005,0x0e020016,0x07030006,0x0e03000b,0x0c050012,
0x00000000,0x01010001,0x0e040009,0x0c03003a,0x0c030024,0x0e04000a,0x0e03000c,0x07040006,
0x00000000,0x0c040014,0x0e04000a,0x07030008,0x0c020063,0x07030007,0x0e03000d,0x0e07000b,
0x00000000,0x01010001,0x0e03000a,0x02020002,0x0801000b,0x07040005,0x07030006,0x0c04001f,
0x00000000,0x0c030033,0x0e03000b,0x07040006,0x0e020017,0x07030008,0x07030007,0x0e06000c,
0x00000000,0x01010001,0x07040005,0x0d03005a,0x0c030025,0x07040006,0x07030008,0x08040008,
0x00000000,0x0c020072,0x07040006,0x07030009,0x0c020064,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0801000c,0x0d03000e,0x0d02000f,0x03030003,
0x0e010099,0x0e03000f,0x04010003,0x0c060008,0x0e020018,0x0c050009,0x0d020010,0x04040004,
0x00000000,0x01010001,0x0d03000e,0x0c05000d,0x0c030026,0x0d03000f,0x0d020011,0x08050006,
0x0f010119,0x0c040015,0x0d03000f,0x08040007,0x0c020065,0x0c05000a,0x0d020012,0x0d060010,
0x00000000,0x01010001,0x08020005,0x02020002,0x0801000d,0x0c060008,0x0c050009,0x0d04002f,
0x10010219,0x0c030034,0x08020006,0x0c060009,0x0e020019,0x0c05000b,0x0c05000a,0x08050007,
0x00000000,0x01010001,0x0c060008,0x0c04001c,0x0c030027,0x0c060009,0x0c05000b,0x08040009,
0x00000000,0x0c020073,0x0c060009,0x08040008,0x0c020066,0x0c05000c,0x0c05000c,0x0c09000a,
0x00000000,0x01010001,0x0d020018,0x02020002,0x0801000e,0x0c050009,0x0c04000a,0x03030003,
0x00000000,0x0e030010,0x0d020019,0x08050006,0x0e02001a,0x08040007,0x0c04000b,0x0d05001a,
0x00000000,0x01010001,0x0c050009,0x0c03003b,0x0c030028,0x0c05000a,0x0c04000c,0x08050007,
0x00000000,0x0c040016,0x0c05000a,0x08040009,0x0c020067,0x08040008,0x0c04000d,0x0c08000b,
0x00000000,0x01010001,0x0e03000b,0x02020002,0x0801000f,0x08050006,0x08040007,0x0e04004f,
0x00000000,0x0c030035,0x0e03000c,0x08050007,0x0e02001b,0x08040009,0x08040008,0x0e06000d,
0x00000000,0x01010001,0x08050006,0x0d03005b,0x0c030029,0x08050007,0x08040009,0x0804000a,
0x00000000,0x0c020074,0x08050007,0x0804000a,0x0c020068,0x0804000a,0x0804000a,0x08080008,
0x00000000,0x01010001,0x04010002,0x02020002,0x0e03000d,0x05020003,0x05010004,0x03030003,
0x0e01009a,0x0e030011,0x04010003,0x10040025,0x0e02001c,0x10030026,0x05010005,0x04040004,
0x00000000,0x0f02008e,0x05020003,0x0c05000e,0x0c03002a,0x05020004,0x05010006,0x0c060009,
0x0f01011a,0x0c040017,0x05020004,0x0c05000a,0x0c020069,0x10030027,0x05010007,0x05050005,
0x00000000,0x01010001,0x0e040009,0x02020002,0x0e03000e,0x10040025,0x10030026,0x0c050011,
0x1001021a,0x0c030036,0x0e04000a,0x10040026,0x0e02001d,0x10030028,0x10030027,0x0e07000b,
0x00000000,0x1002010e,0x10040025,0x0c04001d,0x0c03002b,0x10040026,0x10030028,0x0e06000c,
0x00000000,0x0c020075,0x10040026,0x0c05000b,0x0c02006a,0x10030029,0x10030029,0x10070027,
0x00000000,0x01010001,0x0e020028,0x02020002,0x0e03000f,0x0e04000a,0x0e03000b,0x03030003,
0x00000000,0x0e030012,0x0e020029,0x0c060009,0x0e02001e,0x0c05000a,0x0e03000c,0x0e05002a,
0x00000000,0x01010001,0x0e04000a,0x0c03003c,0x0c03002c,0x0e04000b,0x0e03000d,0x0c06000a,
0x00000000,0x0c040018,0x0e04000b,0x0c05000c,0x0c02006b,0x0c05000b,0x0e03000e,0x0e07000c,
0x00000000,0x01010001,0x0e03000c,0x02020002,0x0e030010,0x0c060009,0x0c05000a,0x0c040020,
0x00000000,0x0c030037,0x0e03000d,0x0c06000a,0x0e02001f,0x0c05000c,0x0c05000b,0x0e06000e,
0x00000000,0x01010001,0x0c060009,0x0d03005c,0x0c03002d,0x0c06000a,0x0c05000c,0x0e06000d,
0x00000000,0x0c020076,0x0c06000a,0x0c05000d,0x0c02006c,0x0c05000d,0x0c05000d,0x0c09000b,
0x00000000,0x01010001,0x04010002,0x02020002,0x0e030011,0x0e030016,0x0e020017,0x03030003,
0x0e01009b,0x0e030013,0x04010003,0x0c05000b,0x0e020020,0x0c04000c,0x0e020018,0x04040004,
0x00000000,0x01010001,0x0e030016,0x0c05000f,0x0c03002e,0x0e030017,0x0e020019,0x0e07000b,
0x0f01011b,0x0c040019,0x0e030017,0x0e06000c,0x0c02006d,0x0c04000d,0x0e02001a,0x0e060018,
0x00000000,0x01010001,0x0e04000a,0x02020002,0x0e030012,0x0c05000b,0x0c04000c,0x0d040030,
0x1001021b,0x0c030038,0x0e04000b,0x0c05000c,0x0e020021,0x0c04000e,0x0c04000d,0x0e07000c,
0x00000000,0x01010001,0x0c05000b,0x0c04001e,0x0c03002f,0x0c05000c,0x0c04000e,0x0e06000e,
0x00000000,0x0c020077,0x0c05000c,0x0e06000d,0x0c02006e,0x0c04000f,0x0c04000f,0x0c08000d,
0x00000000,0x01010001,0x0f020048,0x02020002,0x0e030013,0x0c05000a,0x0c04000b,0x03030003,
0x00000000,0x0e030014,0x0f020049,0x0e07000b,0x0e020022,0x0e06000c,0x0c04000c,0x0f05004a,
0x00000000,0x01010001,0x0c05000a,0x0c03003d,0x0c030030,0x0c05000b,0x0c04000d,0x0e07000c,
0x00000000,0x0c04001a,0x0c05000b,0x0e06000e,0x0c02006f,0x0e06000d,0x0c04000e,0x0c08000c,
0x00000000,0x01010001,0x0e03000d,0x02020002,0x0e030014,0x0e07000b,0x0e06000c,0x0e040050,
0x00000000,0x0c030039,0x0e03000e,0x0e07000c,0x0e020023,0x0e06000e,0x0e06000d,0x0e06000f,
0x00000000,0x01010001,0x0e07000b,0x0d03005d,0x0c030031,0x0e07000c,0x0e06000e,0x0e06000f,
0x00000000,0x0c020078,0x0e07000c,0x0e06000f,0x0c020070,0x0e06000f,0x0e06000f,0x0e0a000d,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d02000d,0x05020003,0x05010004,0x03030003,
0x0e01009c,0x0c05000c,0x04010003,0x06030004,0x0c040013,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f02008f,0x05020003,0x0c050010,0x0c030032,0x05020004,0x05010006,0x07040005,
0x0f01011c,0x0c04001b,0x05020004,0x07030006,0x0c020071,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0d030009,0x10030089,0x0d02000e,0x06030004,0x06020005,0x0c050012,
0x1001021c,0x0c03003a,0x0d03000a,0x06030005,0x0c040014,0x06020007,0x06020006,0x0d06000b,
0x00000000,0x1002010f,0x06030004,0x0c04001f,0x0c030033,0x06030005,0x06020007,0x0d05000c,
0x00000000,0x0c020079,0x06030005,0x07030007,0x0c020072,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0c020011,0x02020002,0x0d02000f,0x0c060008,0x0c050009,0x03030003,
0x00000000,0x0c05000d,0x0c020012,0x07040005,0x0c040015,0x07030006,0x0c05000a,0x0c050013,
0x00000000,0x01010001,0x0c060008,0x0c03003e,0x0c030034,0x0c060009,0x0c05000b,0x07040006,
0x00000000,0x0c04001c,0x0c060009,0x07030008,0x0c020073,0x07030007,0x0c05000c,0x0c09000a,
0x00000000,0x01010001,0x0c050009,0x02020002,0x0d020010,0x07040005,0x07030006,0x0c040021,
0x00000000,0x0c03003b,0x0c05000a,0x07040006,0x0c040016,0x07030008,0x07030007,0x0c08000b,
0x00000000,0x01010001,0x07040005,0x0d03005e,0x0c030035,0x07040006,0x07030008,0x0d05000d,
0x00000000,0x0c02007a,0x07040006,0x07030009,0x0c020074,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d020011,0x0f030026,0x0f020027,0x03030003,
0x0e01009d,0x0c05000e,0x04010003,0x0c060009,0x0c040017,0x0c05000a,0x0f020028,0x04040004,
0x00000000,0x01010001,0x0f030026,0x0c050011,0x0c030036,0x0f030027,0x0f020029,0x0d06000b,
0x0f01011d,0x0c04001d,0x0f030027,0x0d05000c,0x0c020075,0x0c05000b,0x0f02002a,0x0f060028,
0x00000000,0x01010001,0x0d03000a,0x02020002,0x0d020012,0x0c060009,0x0c05000a,0x0d040031,
0x1001021d,0x0c03003c,0x0d03000b,0x0c06000a,0x0c040018,0x0c05000c,0x0c05000b,0x0d06000c,
0x00000000,0x01010001,0x0c060009,0x0c040020,0x0c030037,0x0c06000a,0x0c05000c,0x0d05000e,
0x00000000,0x0c02007b,0x0c06000a,0x0d05000d,0x0c020076,0x0c05000d,0x0c05000d,0x0c09000b,
0x00000000,0x01010001,0x0d020019,0x02020002,0x0d020013,0x0c05000b,0x0c04000c,0x03030003,
0x00000000,0x0c05000f,0x0d02001a,0x0d06000b,0x0c040019,0x0d05000c,0x0c04000d,0x0d05001b,
0x00000000,0x01010001,0x0c05000b,0x0c03003f,0x0c030038,0x0c05000c,0x0c04000e,0x0d06000c,
0x00000000,0x0c04001e,0x0c05000c,0x0d05000e,0x0c020077,0x0d05000d,0x0c04000f,0x0c08000d,
0x00000000,0x01010001,0x0c05000a,0x02020002,0x0d020014,0x0d06000b,0x0d05000c,0x0e040051,
0x00000000,0x0c03003d,0x0c05000b,0x0d06000c,0x0c04001a,0x0d05000e,0x0d05000d,0x0c08000c,
0x00000000,0x01010001,0x0d06000b,0x0d03005f,0x0c030039,0x0d06000c,0x0d05000e,0x0d05000f,
0x00000000,0x0c02007c,0x0d06000c,0x0d05000f,0x0c020078,0x0d05000f,0x0d05000f,0x0d09000d,
0x00000000,0x01010001,0x04010002,0x02020002,0x0c05000c,0x05020003,0x05010004,0x03030003,
0x0e01009e,0x0c050010,0x04010003,0x06030004,0x0c04001b,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f020090,0x05020003,0x0c050012,0x0c03003a,0x05020004,0x05010006,0x0c05000c,
0x0f01011e,0x0c04001f,0x05020004,0x0c04000d,0x0c020079,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0c060008,0x02020002,0x0c05000d,0x06030004,0x06020005,0x0c050013,
0x1001021e,0x0c03003e,0x0c060009,0x06030005,0x0c04001c,0x06020007,0x06020006,0x0c09000a,
0x00000000,0x10020110,0x06030004,0x0c040021,0x0c03003b,0x06030005,0x06020007,0x0c08000b,
0x00000000,0x0c02007d,0x06030005,0x0c04000e,0x0c02007a,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0e020029,0x02020002,0x0c05000e,0x0c060009,0x0c05000a,0x03030003,
0x00000000,0x0c050011,0x0e02002a,0x0c05000c,0x0c04001d,0x0c04000d,0x0c05000b,0x0e05002b,
0x00000000,0x01010001,0x0c060009,0x0c030040,0x0c03003c,0x0c06000a,0x0c05000c,0x0c05000d,
0x00000000,0x0c040020,0x0c06000a,0x0c04000f,0x0c02007b,0x0c04000e,0x0c05000d,0x0c09000b,
0x00000000,0x01010001,0x0c05000b,0x02020002,0x0c05000f,0x0c05000c,0x0c04000d,0x0c040022,
0x00000000,0x0c03003f,0x0c05000c,0x0c05000d,0x0c04001e,0x0c04000f,0x0c04000e,0x0c08000d,
0x00000000,0x01010001,0x0c05000c,0x0d030060,0x0c03003d,0x0c05000d,0x0c04000f,0x0c08000c,
0x00000000,0x0c02007e,0x0c05000d,0x0c040010,0x0c02007c,0x0c040010,0x0c040010,0x0c08000e,
0x00000000,0x01010001,0x04010002,0x02020002,0x0c050010,0x10030046,0x10020047,0x03030003,
0x0e01009f,0x0c050012,0x04010003,0x0c05000c,0x0c04001f,0x0c04000d,0x10020048,0x04040004,
0x00000000,0x01010001,0x10030046,0x0c050013,0x0c03003e,0x10030047,0x10020049,0x0c09000a,
0x0f01011f,0x0c040021,0x10030047,0x0c08000b,0x0c02007d,0x0c04000e,0x1002004a,0x10060048,
0x00000000,0x01010001,0x0c060009,0x02020002,0x0c050011,0x0c05000c,0x0c04000d,0x0d040032,
0x1001021f,0x0c030040,0x0c06000a,0x0c05000d,0x0c040020,0x0c04000f,0x0c04000e,0x0c09000b,
0x00000000,0x01010001,0x0c05000c,0x0c040022,0x0c03003f,0x0c05000d,0x0c04000f,0x0c08000d,
0x00000000,0x0c02007f,0x0c05000d,0x0c08000c,0x0c02007e,0x0c040010,0x0c040010,0x0c08000e,
0x00000000,0x01010001,0x0f020049,0x02020002,0x0c050012,0x0c05000c,0x0c04000d,0x03030003,
0x00000000,0x0c050013,0x0f02004a,0x0c09000a,0x0c040021,0x0c08000b,0x0c04000e,0x0f05004b,
0x00000000,0x01010001,0x0c05000c,0x0c030041,0x0c030040,0x0c05000d,0x0c04000f,0x0c09000b,
0x00000000,0x0c040022,0x0c05000d,0x0c08000d,0x0c02007f,0x0c08000c,0x0c040010,0x0c08000e,
0x00000000,0x01010001,0x0c05000c,0x02020002,0x0c050013,0x0c09000a,0x0c08000b,0x0e040052,
0x00000000,0x0c030041,0x0c05000d,0x0c09000b,0x0c040022,0x0c08000d,0x0c08000c,0x0c08000e,
0x00000000,0x01010001,0x0c09000a,0x0d030061,0x0c030041,0x0c09000b,0x0c08000d,0x0c08000e,
0x00000000,0x0c020080,0x0c09000b,0x0c08000e,0x0c020080,0x0c08000e,0x0c08000e,0x0c0c000c,
0x00000000,0x01010001,0x04010002,0x02020002,0x08010008,0x05020003,0x05010004,0x03030003,
0x0e0100a0,0x09020009,0x04010003,0x06030004,0x09010010,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f020091,0x05020003,0x0a03000a,0x0a010020,0x05020004,0x05010006,0x07040005,
0x0f010120,0x0a020011,0x05020004,0x07030006,0x0f020042,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x08020004,0x1003008a,0x08010009,0x06030004,0x06020005,0x0f05000d,
0x10010220,0x0f030023,0x08020005,0x06030005,0x09010011,0x06020007,0x06020006,0x08050006,
0x00000000,0x10020111,0x06030004,0x0f040014,0x0a010021,0x06030005,0x06020007,0x08040007,
0x00000000,0x0d030042,0x06030005,0x07030007,0x0f020043,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0d03000b,0x02020002,0x0801000a,0x09030005,0x09020006,0x03030003,
0x00000000,0x0902000a,0x0d03000c,0x07040005,0x09010012,0x07030006,0x09020007,0x0d06000d,
0x00000000,0x01010001,0x09030005,0x0d040023,0x0a010022,0x09030006,0x09020008,0x07040006,
0x00000000,0x0a020012,0x09030006,0x07030008,0x0f020044,0x07030007,0x09020009,0x09060007,
0x00000000,0x01010001,0x09020006,0x02020002,0x0801000b,0x07040005,0x07030006,0x0d050014,
0x00000000,0x0f030024,0x09020007,0x07040006,0x09010013,0x07030008,0x07030007,0x09050008,
0x00000000,0x01010001,0x07040005,0x0d030062,0x0a010023,0x07040006,0x07030008,0x08040008,
0x00000000,0x0d030043,0x07040006,0x07030009,0x0f020045,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0801000c,0x0d03000f,0x0d020010,0x03030003,
0x0e0100a1,0x0902000b,0x04010003,0x0a040006,0x09010014,0x0a030007,0x0d020011,0x04040004,
0x00000000,0x01010001,0x0d03000f,0x0a03000b,0x0a010024,0x0d030010,0x0d020012,0x08050006,
0x0f010121,0x0a020013,0x0d030010,0x08040007,0x0f020046,0x0a030008,0x0d020013,0x0d060011,
0x00000000,0x01010001,0x08020005,0x02020002,0x0801000d,0x0a040006,0x0a030007,0x0d040033,
0x10010221,0x0f030025,0x08020006,0x0a040007,0x09010015,0x0a030009,0x0a030008,0x08050007,
0x00000000,0x01010001,0x0a040006,0x0f040015,0x0a010025,0x0a040007,0x0a030009,0x08040009,
0x00000000,0x0d030044,0x0a040007,0x08040008,0x0f020047,0x0a03000a,0x0a03000a,0x0a070008,
0x00000000,0x01010001,0x0d02001a,0x02020002,0x0801000e,0x0a030007,0x0a020008,0x03030003,
0x00000000,0x0902000c,0x0d02001b,0x08050006,0x09010016,0x08040007,0x0a020009,0x0d05001c,
0x00000000,0x01010001,0x0a030007,0x0d040024,0x0a010026,0x0a030008,0x0a02000a,0x08050007,
0x00000000,0x0a020014,0x0a030008,0x08040009,0x0f020048,0x08040008,0x0a02000b,0x0a060009,
0x00000000,0x01010001,0x09020007,0x02020002,0x0801000f,0x08050006,0x08040007,0x0e040053,
0x00000000,0x0f030026,0x09020008,0x08050007,0x09010017,0x08040009,0x08040008,0x09050009,
0x00000000,0x01010001,0x08050006,0x0d030063,0x0a010027,0x08050007,0x08040009,0x0804000a,
0x00000000,0x0d030045,0x08050007,0x0804000a,0x0f020049,0x0804000a,0x0804000a,0x08080008,
0x00000000,0x01010001,0x04010002,0x02020002,0x09020009,0x05020003,0x05010004,0x03030003,
0x0e0100a2,0x0902000d,0x04010003,0x0e04000e,0x09010018,0x0e03000f,0x05010005,0x04040004,
0x00000000,0x0f020092,0x05020003,0x0a03000c,0x0a010028,0x05020004,0x05010006,0x0f060009,
0x0f010122,0x0a020015,0x05020004,0x0f05000a,0x0f02004a,0x0e030010,0x05010007,0x05050005,
0x00000000,0x01010001,0x09030005,0x02020002,0x0902000a,0x0e04000e,0x0e03000f,0x0f05000e,
0x10010222,0x0f030027,0x09030006,0x0e04000f,0x09010019,0x0e030011,0x0e030010,0x09060007,
0x00000000,0x10020112,0x0e04000e,0x0f040016,0x0a010029,0x0e04000f,0x0e030011,0x09050008,
0x00000000,0x0d030046,0x0e04000f,0x0f05000b,0x0f02004b,0x0e030012,0x0e030012,0x0e070010,
0x00000000,0x01010001,0x0e02002a,0x02020002,0x0902000b,0x09030006,0x09020007,0x03030003,
0x00000000,0x0902000e,0x0e02002b,0x0f060009,0x0901001a,0x0f05000a,0x09020008,0x0e05002c,
0x00000000,0x01010001,0x09030006,0x0d040025,0x0a01002a,0x09030007,0x09020009,0x0f06000a,
0x00000000,0x0a020016,0x09030007,0x0f05000c,0x0f02004c,0x0f05000b,0x0902000a,0x09060008,
0x00000000,0x01010001,0x09020008,0x02020002,0x0902000c,0x0f060009,0x0f05000a,0x0d050015,
0x00000000,0x0f030028,0x09020009,0x0f06000a,0x0901001b,0x0f05000c,0x0f05000b,0x0905000a,
0x00000000,0x01010001,0x0f060009,0x0d030064,0x0a01002b,0x0f06000a,0x0f05000c,0x09050009,
0x00000000,0x0d030047,0x0f06000a,0x0f05000d,0x0f02004d,0x0f05000d,0x0f05000d,0x0f09000b,
0x00000000,0x01010001,0x04010002,0x02020002,0x0902000d,0x0e030017,0x0e020018,0x03030003,
0x0e0100a3,0x0902000f,0x04010003,0x0f05000a,0x0901001c,0x0f04000b,0x0e020019,0x04040004,
0x00000000,0x01010001,0x0e030017,0x0a03000d,0x0a01002c,0x0e030018,0x0e02001a,0x09060007,
0x0f010123,0x0a020017,0x0e030018,0x09050008,0x0f02004e,0x0f04000c,0x0e02001b,0x0e060019,
0x00000000,0x01010001,0x09030006,0x02020002,0x0902000e,0x0f05000a,0x0f04000b,0x0d040034,
0x10010223,0x0f030029,0x09030007,0x0f05000b,0x0901001d,0x0f04000d,0x0f04000c,0x09060008,
0x00000000,0x01010001,0x0f05000a,0x0f040017,0x0a01002d,0x0f05000b,0x0f04000d,0x0905000a,
0x00000000,0x0d030048,0x0f05000b,0x09050009,0x0f02004f,0x0f04000e,0x0f04000e,0x0f08000c,
0x00000000,0x01010001,0x0f02004a,0x02020002,0x0902000f,0x0a030008,0x0a020009,0x03030003,
0x00000000,0x09020010,0x0f02004b,0x09060007,0x0901001e,0x09050008,0x0a02000a,0x0f05004c,
0x00000000,0x01010001,0x0a030008,0x0d040026,0x0a01002e,0x0a030009,0x0a02000b,0x09060008,
0x00000000,0x0a020018,0x0a030009,0x0905000a,0x0f020050,0x09050009,0x0a02000c,0x0a06000a,
0x00000000,0x01010001,0x09020009,0x02020002,0x09020010,0x09060007,0x09050008,0x0e040054,
0x00000000,0x0f03002a,0x0902000a,0x09060008,0x0901001f,0x0905000a,0x09050009,0x0905000b,
0x00000000,0x01010001,0x09060007,0x0d030065,0x0a01002f,0x09060008,0x0905000a,0x0905000b,
0x00000000,0x0d030049,0x09060008,0x0905000b,0x0f020051,0x0905000b,0x0905000b,0x09090009,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d03000b,0x05020003,0x05010004,0x03030003,
0x0e0100a4,0x0a03000a,0x04010003,0x06030004,0x0a020011,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f020093,0x05020003,0x0a03000e,0x0a010030,0x05020004,0x05010006,0x0f05000e,
0x0f010124,0x0a020019,0x05020004,0x0f04000f,0x0f020052,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0d040007,0x1003008b,0x0d03000c,0x06030004,0x06020005,0x0f05000f,
0x10010224,0x0f03002b,0x0d040008,0x06030005,0x0a020012,0x06020007,0x06020006,0x0d070009,
0x00000000,0x10020113,0x06030004,0x0f040018,0x0a010031,0x06030005,0x06020007,0x0d06000a,
0x00000000,0x0d03004a,0x06030005,0x0f040010,0x0f020053,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0d03000c,0x02020002,0x0d03000d,0x0a040006,0x0a030007,0x03030003,
0x00000000,0x0a03000b,0x0d03000d,0x0f05000e,0x0a020013,0x0f04000f,0x0a030008,0x0d06000e,
0x00000000,0x01010001,0x0a040006,0x0d040027,0x0a010032,0x0a040007,0x0a030009,0x0f05000f,
0x00000000,0x0a02001a,0x0a040007,0x0f040011,0x0f020054,0x0f040010,0x0a03000a,0x0a070008,
0x00000000,0x01010001,0x0a030007,0x02020002,0x0d03000e,0x0f05000e,0x0f04000f,0x0d050016,
0x00000000,0x0f03002c,0x0a030008,0x0f05000f,0x0a020014,0x0f040011,0x0f040010,0x0a060009,
0x00000000,0x01010001,0x0f05000e,0x0d030066,0x0a010033,0x0f05000f,0x0f040011,0x0d06000b,
0x00000000,0x0d03004b,0x0f05000f,0x0f040012,0x0f020055,0x0f040012,0x0f040012,0x0f080010,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d03000f,0x0f030027,0x0f020028,0x03030003,
0x0e0100a5,0x0a03000c,0x04010003,0x0a040007,0x0a020015,0x0a030008,0x0f020029,0x04040004,
0x00000000,0x01010001,0x0f030027,0x0a03000f,0x0a010034,0x0f030028,0x0f02002a,0x0d070009,
0x0f010125,0x0a02001b,0x0f030028,0x0d06000a,0x0f020056,0x0a030009,0x0f02002b,0x0f060029,
0x00000000,0x01010001,0x0d040008,0x02020002,0x0d030010,0x0a040007,0x0a030008,0x0d040035,
0x10010225,0x0f03002d,0x0d040009,0x0a040008,0x0a020016,0x0a03000a,0x0a030009,0x0d07000a,
0x00000000,0x01010001,0x0a040007,0x0f040019,0x0a010035,0x0a040008,0x0a03000a,0x0d06000c,
0x00000000,0x0d03004c,0x0a040008,0x0d06000b,0x0f020057,0x0a03000b,0x0a03000b,0x0a070009,
0x00000000,0x01010001,0x0d02001b,0x02020002,0x0d030011,0x0a030009,0x0a02000a,0x03030003,
0x00000000,0x0a03000d,0x0d02001c,0x0d070009,0x0a020017,0x0d06000a,0x0a02000b,0x0d05001d,
0x00000000,0x01010001,0x0a030009,0x0d040028,0x0a010036,0x0a03000a,0x0a02000c,0x0d07000a,
0x00000000,0x0a02001c,0x0a03000a,0x0d06000c,0x0f020058,0x0d06000b,0x0a02000d,0x0a06000b,
0x00000000,0x01010001,0x0a030008,0x02020002,0x0d030012,0x0d070009,0x0d06000a,0x0e040055,
0x00000000,0x0f03002e,0x0a030009,0x0d07000a,0x0a020018,0x0d06000c,0x0d06000b,0x0a06000a,
0x00000000,0x01010001,0x0d070009,0x0d030067,0x0a010037,0x0d07000a,0x0d06000c,0x0d06000d,
0x00000000,0x0d03004d,0x0d07000a,0x0d06000d,0x0f020059,0x0d06000d,0x0d06000d,0x0d0a000b,
0x00000000,0x01010001,0x04010002,0x02020002,0x0a03000a,0x05020003,0x05010004,0x03030003,
0x0e0100a6,0x0a03000e,0x04010003,0x0f040016,0x0a020019,0x0f030017,0x05010005,0x04040004,
0x00000000,0x0f020094,0x05020003,0x0a030010,0x0a010038,0x05020004,0x05010006,0x0d06000a,
0x0f010126,0x0a02001d,0x05020004,0x0d05000b,0x0f02005a,0x0f030018,0x05010007,0x05050005,
0x00000000,0x01010001,0x0a040006,0x02020002,0x0a03000b,0x0f040016,0x0f030017,0x0f050010,
0x10010226,0x0f03002f,0x0a040007,0x0f040017,0x0a02001a,0x0f030019,0x0f030018,0x0a070008,
0x00000000,0x10020114,0x0f040016,0x0f04001a,0x0a010039,0x0f040017,0x0f030019,0x0a060009,
0x00000000,0x0d03004e,0x0f040017,0x0d05000c,0x0f02005b,0x0f03001a,0x0f03001a,0x0f070018,
0x00000000,0x01010001,0x0e02002b,0x02020002,0x0a03000c,0x0a040007,0x0a030008,0x03030003,
0x00000000,0x0a03000f,0x0e02002c,0x0d06000a,0x0a02001b,0x0d05000b,0x0a030009,0x0e05002d,
0x00000000,0x01010001,0x0a040007,0x0d040029,0x0a01003a,0x0a040008,0x0a03000a,0x0d06000b,
0x00000000,0x0a02001e,0x0a040008,0x0d05000d,0x0f02005c,0x0d05000c,0x0a03000b,0x0a070009,
0x00000000,0x01010001,0x0a030009,0x02020002,0x0a03000d,0x0d06000a,0x0d05000b,0x0d050017,
0x00000000,0x0f030030,0x0a03000a,0x0d06000b,0x0a02001c,0x0d05000d,0x0d05000c,0x0a06000b,
0x00000000,0x01010001,0x0d06000a,0x0d030068,0x0a01003b,0x0d06000b,0x0d05000d,0x0a06000a,
0x00000000,0x0d03004f,0x0d06000b,0x0d05000e,0x0f02005d,0x0d05000e,0x0d05000e,0x0d09000c,
0x00000000,0x01010001,0x04010002,0x02020002,0x0a03000e,0x10030047,0x10020048,0x03030003,
0x0e0100a7,0x0a030010,0x04010003,0x0f05000b,0x0a02001d,0x0f04000c,0x10020049,0x04040004,
0x00000000,0x01010001,0x10030047,0x0a030011,0x0a01003c,0x10030048,0x1002004a,0x0a070008,
0x0f010127,0x0a02001f,0x10030048,0x0a060009,0x0f02005e,0x0f04000d,0x1002004b,0x10060049,
0x00000000,0x01010001,0x0a040007,0x02020002,0x0a03000f,0x0f05000b,0x0f04000c,0x0d040036,
0x10010227,0x0f030031,0x0a040008,0x0f05000c,0x0a02001e,0x0f04000e,0x0f04000d,0x0a070009,
0x00000000,0x01010001,0x0f05000b,0x0f04001b,0x0a01003d,0x0f05000c,0x0f04000e,0x0a06000b,
0x00000000,0x0d030050,0x0f05000c,0x0a06000a,0x0f02005f,0x0f04000f,0x0f04000f,0x0f08000d,
0x00000000,0x01010001,0x0f02004b,0x02020002,0x0a030010,0x0a03000a,0x0a02000b,0x03030003,
0x00000000,0x0a030011,0x0f02004c,0x0a070008,0x0a02001f,0x0a060009,0x0a02000c,0x0f05004d,
0x00000000,0x01010001,0x0a03000a,0x0d04002a,0x0a01003e,0x0a03000b,0x0a02000d,0x0a070009,
0x00000000,0x0a020020,0x0a03000b,0x0a06000b,0x0f020060,0x0a06000a,0x0a02000e,0x0a06000c,
0x00000000,0x01010001,0x0a03000a,0x02020002,0x0a030011,0x0a070008,0x0a060009,0x0e040056,
0x00000000,0x0f030032,0x0a03000b,0x0a070009,0x0a020020,0x0a06000b,0x0a06000a,0x0a06000c,
0x00000000,0x01010001,0x0a070008,0x0d030069,0x0a01003f,0x0a070009,0x0a06000b,0x0a06000c,
0x00000000,0x0d030051,0x0a070009,0x0a06000c,0x0f020061,0x0a06000c,0x0a06000c,0x0a0a000a,
0x00000000,0x01010001,0x04010002,0x02020002,0x08010008,0x05020003,0x05010004,0x03030003,
0x0e0100a8,0x0d03000c,0x04010003,0x06030004,0x0d020013,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f020095,0x05020003,0x0f05000d,0x0f030023,0x05020004,0x05010006,0x07040005,
0x0f010128,0x0f040014,0x05020004,0x07030006,0x0f020062,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x08020004,0x1003008c,0x08010009,0x06030004,0x06020005,0x0f050011,
0x10010228,0x0f030033,0x08020005,0x06030005,0x0d020014,0x06020007,0x06020006,0x08050006,
0x00000000,0x10020115,0x06030004,0x0f04001c,0x0f030024,0x06030005,0x06020007,0x08040007,
0x00000000,0x0d030052,0x06030005,0x07030007,0x0f020063,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0d03000d,0x02020002,0x0801000a,0x0d040008,0x0d030009,0x03030003,
0x00000000,0x0d03000d,0x0d03000e,0x07040005,0x0d020015,0x07030006,0x0d03000a,0x0d06000f,
0x00000000,0x01010001,0x0d040008,0x0d04002b,0x0f030025,0x0d040009,0x0d03000b,0x07040006,
0x00000000,0x0f040015,0x0d040009,0x07030008,0x0f020064,0x07030007,0x0d03000c,0x0d07000a,
0x00000000,0x01010001,0x0d030009,0x02020002,0x0801000b,0x07040005,0x07030006,0x0d050018,
0x00000000,0x0f030034,0x0d03000a,0x07040006,0x0d020016,0x07030008,0x07030007,0x0d06000b,
0x00000000,0x01010001,0x07040005,0x0d03006a,0x0f030026,0x07040006,0x07030008,0x08040008,
0x00000000,0x0d030053,0x07040006,0x07030009,0x0f020065,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0801000c,0x0d030010,0x0d020011,0x03030003,
0x0e0100a9,0x0d03000e,0x04010003,0x0f060009,0x0d020017,0x0f05000a,0x0d020012,0x04040004,
0x00000000,0x01010001,0x0d030010,0x0f05000e,0x0f030027,0x0d030011,0x0d020013,0x08050006,
0x0f010129,0x0f040016,0x0d030011,0x08040007,0x0f020066,0x0f05000b,0x0d020014,0x0d060012,
0x00000000,0x01010001,0x08020005,0x02020002,0x0801000d,0x0f060009,0x0f05000a,0x0d040037,
0x10010229,0x0f030035,0x08020006,0x0f06000a,0x0d020018,0x0f05000c,0x0f05000b,0x08050007,
0x00000000,0x01010001,0x0f060009,0x0f04001d,0x0f030028,0x0f06000a,0x0f05000c,0x08040009,
0x00000000,0x0d030054,0x0f06000a,0x08040008,0x0f020067,0x0f05000d,0x0f05000d,0x0f09000b,
0x00000000,0x01010001,0x0d02001c,0x02020002,0x0801000e,0x0f05000a,0x0f04000b,0x03030003,
0x00000000,0x0d03000f,0x0d02001d,0x08050006,0x0d020019,0x08040007,0x0f04000c,0x0d05001e,
0x00000000,0x01010001,0x0f05000a,0x0d04002c,0x0f030029,0x0f05000b,0x0f04000d,0x08050007,
0x00000000,0x0f040017,0x0f05000b,0x08040009,0x0f020068,0x08040008,0x0f04000e,0x0f08000c,
0x00000000,0x01010001,0x0d03000a,0x02020002,0x0801000f,0x08050006,0x08040007,0x0e040057,
0x00000000,0x0f030036,0x0d03000b,0x08050007,0x0d02001a,0x08040009,0x08040008,0x0d06000c,
0x00000000,0x01010001,0x08050006,0x0d03006b,0x0f03002a,0x08050007,0x08040009,0x0804000a,
0x00000000,0x0d030055,0x08050007,0x0804000a,0x0f020069,0x0804000a,0x0804000a,0x08080008,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d03000c,0x05020003,0x05010004,0x03030003,
0x0e0100aa,0x0d030010,0x04010003,0x10040026,0x0d02001b,0x10030027,0x05010005,0x04040004,
0x00000000,0x0f020096,0x05020003,0x0f05000f,0x0f03002b,0x05020004,0x05010006,0x0f06000a,
0x0f01012a,0x0f040018,0x05020004,0x0f05000b,0x0f02006a,0x10030028,0x05010007,0x05050005,
0x00000000,0x01010001,0x0d040008,0x02020002,0x0d03000d,0x10040026,0x10030027,0x0f050012,
0x1001022a,0x0f030037,0x0d040009,0x10040027,0x0d02001c,0x10030029,0x10030028,0x0d07000a,
0x00000000,0x10020116,0x10040026,0x0f04001e,0x0f03002c,0x10040027,0x10030029,0x0d06000b,
0x00000000,0x0d030056,0x10040027,0x0f05000c,0x0f02006b,0x1003002a,0x1003002a,0x10070028,
0x00000000,0x01010001,0x0e02002c,0x02020002,0x0d03000e,0x0d040009,0x0d03000a,0x03030003,
0x00000000,0x0d030011,0x0e02002d,0x0f06000a,0x0d02001d,0x0f05000b,0x0d03000b,0x0e05002e,
0x00000000,0x01010001,0x0d040009,0x0d04002d,0x0f03002d,0x0d04000a,0x0d03000c,0x0f06000b,
0x00000000,0x0f040019,0x0d04000a,0x0f05000d,0x0f02006c,0x0f05000c,0x0d03000d,0x0d07000b,
0x00000000,0x01010001,0x0d03000b,0x02020002,0x0d03000f,0x0f06000a,0x0f05000b,0x0d050019,
0x00000000,0x0f030038,0x0d03000c,0x0f06000b,0x0d02001e,0x0f05000d,0x0f05000c,0x0d06000d,
0x00000000,0x01010001,0x0f06000a,0x0d03006c,0x0f03002e,0x0f06000b,0x0f05000d,0x0d06000c,
0x00000000,0x0d030057,0x0f06000b,0x0f05000e,0x0f02006d,0x0f05000e,0x0f05000e,0x0f09000c,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d030010,0x0e030018,0x0e020019,0x03030003,
0x0e0100ab,0x0d030012,0x04010003,0x0f05000c,0x0d02001f,0x0f04000d,0x0e02001a,0x04040004,
0x00000000,0x01010001,0x0e030018,0x0f050010,0x0f03002f,0x0e030019,0x0e02001b,0x0d07000a,
0x0f01012b,0x0f04001a,0x0e030019,0x0d06000b,0x0f02006e,0x0f04000e,0x0e02001c,0x0e06001a,
0x00000000,0x01010001,0x0d040009,0x02020002,0x0d030011,0x0f05000c,0x0f04000d,0x0d040038,
0x1001022b,0x0f030039,0x0d04000a,0x0f05000d,0x0d020020,0x0f04000f,0x0f04000e,0x0d07000b,
0x00000000,0x01010001,0x0f05000c,0x0f04001f,0x0f030030,0x0f05000d,0x0f04000f,0x0d06000d,
0x00000000,0x0d030058,0x0f05000d,0x0d06000c,0x0f02006f,0x0f040010,0x0f040010,0x0f08000e,
0x00000000,0x01010001,0x0f02004c,0x02020002,0x0d030012,0x0f05000b,0x0f04000c,0x03030003,
0x00000000,0x0d030013,0x0f02004d,0x0d07000a,0x0d020021,0x0d06000b,0x0f04000d,0x0f05004e,
0x00000000,0x01010001,0x0f05000b,0x0d04002e,0x0f030031,0x0f05000c,0x0f04000e,0x0d07000b,
0x00000000,0x0f04001b,0x0f05000c,0x0d06000d,0x0f020070,0x0d06000c,0x0f04000f,0x0f08000d,
0x00000000,0x01010001,0x0d03000c,0x02020002,0x0d030013,0x0d07000a,0x0d06000b,0x0e040058,
0x00000000,0x0f03003a,0x0d03000d,0x0d07000b,0x0d020022,0x0d06000d,0x0d06000c,0x0d06000e,
0x00000000,0x01010001,0x0d07000a,0x0d03006d,0x0f030032,0x0d07000b,0x0d06000d,0x0d06000e,
0x00000000,0x0d030059,0x0d07000b,0x0d06000e,0x0f020071,0x0d06000e,0x0d06000e,0x0d0a000c,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d02000e,0x05020003,0x05010004,0x03030003,
0x0e0100ac,0x0f05000d,0x04010003,0x06030004,0x0f040014,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f020097,0x05020003,0x0f050011,0x0f030033,0x05020004,0x05010006,0x10050016,
0x0f01012c,0x0f04001c,0x05020004,0x10040017,0x0f020072,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0d03000a,0x1003008d,0x0d02000f,0x06030004,0x06020005,0x0f050013,
0x1001022c,0x0f03003b,0x0d03000b,0x06030005,0x0f040015,0x06020007,0x06020006,0x0d06000c,
0x00000000,0x10020117,0x06030004,0x0f040020,0x0f030034,0x06030005,0x06020007,0x0d05000d,
0x00000000,0x0d03005a,0x06030005,0x10040018,0x0f020073,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0d03000e,0x02020002,0x0d020010,0x0f060009,0x0f05000a,0x03030003,
0x00000000,0x0f05000e,0x0d03000f,0x10050016,0x0f040016,0x10040017,0x0f05000b,0x0d060010,
0x00000000,0x01010001,0x0f060009,0x0d04002f,0x0f030035,0x0f06000a,0x0f05000c,0x10050017,
0x00000000,0x0f04001d,0x0f06000a,0x10040019,0x0f020074,0x10040018,0x0f05000d,0x0f09000b,
0x00000000,0x01010001,0x0f05000a,0x02020002,0x0d020011,0x10050016,0x10040017,0x0d05001a,
0x00000000,0x0f03003c,0x0f05000b,0x10050017,0x0f040017,0x10040019,0x10040018,0x0f08000c,
0x00000000,0x01010001,0x10050016,0x0d03006e,0x0f030036,0x10050017,0x10040019,0x0d05000e,
0x00000000,0x0d03005b,0x10050017,0x1004001a,0x0f020075,0x1004001a,0x1004001a,0x10080018,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d020012,0x0f030028,0x0f020029,0x03030003,
0x0e0100ad,0x0f05000f,0x04010003,0x0f06000a,0x0f040018,0x0f05000b,0x0f02002a,0x04040004,
0x00000000,0x01010001,0x0f030028,0x0f050012,0x0f030037,0x0f030029,0x0f02002b,0x0d06000c,
0x0f01012d,0x0f04001e,0x0f030029,0x0d05000d,0x0f020076,0x0f05000c,0x0f02002c,0x0f06002a,
0x00000000,0x01010001,0x0d03000b,0x02020002,0x0d020013,0x0f06000a,0x0f05000b,0x0d040039,
0x1001022d,0x0f03003d,0x0d03000c,0x0f06000b,0x0f040019,0x0f05000d,0x0f05000c,0x0d06000d,
0x00000000,0x01010001,0x0f06000a,0x0f040021,0x0f030038,0x0f06000b,0x0f05000d,0x0d05000f,
0x00000000,0x0d03005c,0x0f06000b,0x0d05000e,0x0f020077,0x0f05000e,0x0f05000e,0x0f09000c,
0x00000000,0x01010001,0x0d02001d,0x02020002,0x0d020014,0x0f05000c,0x0f04000d,0x03030003,
0x00000000,0x0f050010,0x0d02001e,0x0d06000c,0x0f04001a,0x0d05000d,0x0f04000e,0x0d05001f,
0x00000000,0x01010001,0x0f05000c,0x0d040030,0x0f030039,0x0f05000d,0x0f04000f,0x0d06000d,
0x00000000,0x0f04001f,0x0f05000d,0x0d05000f,0x0f020078,0x0d05000e,0x0f040010,0x0f08000e,
0x00000000,0x01010001,0x0f05000b,0x02020002,0x0d020015,0x0d06000c,0x0d05000d,0x0e040059,
0x00000000,0x0f03003e,0x0f05000c,0x0d06000d,0x0f04001b,0x0d05000f,0x0d05000e,0x0f08000d,
0x00000000,0x01010001,0x0d06000c,0x0d03006f,0x0f03003a,0x0d06000d,0x0d05000f,0x0d050010,
0x00000000,0x0d03005d,0x0d06000d,0x0d050010,0x0f020079,0x0d050010,0x0d050010,0x0d09000e,
0x00000000,0x01010001,0x04010002,0x02020002,0x0f05000d,0x05020003,0x05010004,0x03030003,
0x0e0100ae,0x0f050011,0x04010003,0x06030004,0x0f04001c,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f020098,0x05020003,0x0f050013,0x0f03003b,0x05020004,0x05010006,0x0d06000b,
0x0f01012e,0x0f040020,0x05020004,0x0d05000c,0x0f02007a,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0f060009,0x02020002,0x0f05000e,0x06030004,0x06020005,0x0f050014,
0x1001022e,0x0f03003f,0x0f06000a,0x06030005,0x0f04001d,0x06020007,0x06020006,0x0f09000b,
0x00000000,0x10020118,0x06030004,0x0f040022,0x0f03003c,0x06030005,0x06020007,0x0f08000c,
0x00000000,0x0d03005e,0x06030005,0x0d05000d,0x0f02007b,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0e02002d,0x02020002,0x0f05000f,0x0f06000a,0x0f05000b,0x03030003,
0x00000000,0x0f050012,0x0e02002e,0x0d06000b,0x0f04001e,0x0d05000c,0x0f05000c,0x0e05002f,
0x00000000,0x01010001,0x0f06000a,0x0d040031,0x0f03003d,0x0f06000b,0x0f05000d,0x0d06000c,
0x00000000,0x0f040021,0x0f06000b,0x0d05000e,0x0f02007c,0x0d05000d,0x0f05000e,0x0f09000c,
0x00000000,0x01010001,0x0f05000c,0x02020002,0x0f050010,0x0d06000b,0x0d05000c,0x0d05001b,
0x00000000,0x0f030040,0x0f05000d,0x0d06000c,0x0f04001f,0x0d05000e,0x0d05000d,0x0f08000e,
0x00000000,0x01010001,0x0d06000b,0x0d030070,0x0f03003e,0x0d06000c,0x0d05000e,0x0f08000d,
0x00000000,0x0d03005f,0x0d06000c,0x0d05000f,0x0f02007d,0x0d05000f,0x0d05000f,0x0d09000d,
0x00000000,0x01010001,0x04010002,0x02020002,0x0f050011,0x10030048,0x10020049,0x03030003,
0x0e0100af,0x0f050013,0x04010003,0x0f05000d,0x0f040020,0x0f04000e,0x1002004a,0x04040004,
0x00000000,0x01010001,0x10030048,0x0f050014,0x0f03003f,0x10030049,0x1002004b,0x0f09000b,
0x0f01012f,0x0f040022,0x10030049,0x0f08000c,0x0f02007e,0x0f04000f,0x1002004c,0x1006004a,
0x00000000,0x01010001,0x0f06000a,0x02020002,0x0f050012,0x0f05000d,0x0f04000e,0x0d04003a,
0x1001022f,0x0f030041,0x0f06000b,0x0f05000e,0x0f040021,0x0f040010,0x0f04000f,0x0f09000c,
0x00000000,0x01010001,0x0f05000d,0x0f040023,0x0f030040,0x0f05000e,0x0f040010,0x0f08000e,
0x00000000,0x0d030060,0x0f05000e,0x0f08000d,0x0f02007f,0x0f040011,0x0f040011,0x0f08000f,
0x00000000,0x01010001,0x0f02004d,0x02020002,0x0f050013,0x0f05000d,0x0f04000e,0x03030003,
0x00000000,0x0f050014,0x0f02004e,0x0f09000b,0x0f040022,0x0f08000c,0x0f04000f,0x0f05004f,
0x00000000,0x01010001,0x0f05000d,0x0d040032,0x0f030041,0x0f05000e,0x0f040010,0x0f09000c,
0x00000000,0x0f040023,0x0f05000e,0x0f08000e,0x0f020080,0x0f08000d,0x0f040011,0x0f08000f,
0x00000000,0x01010001,0x0f05000d,0x02020002,0x0f050014,0x0f09000b,0x0f08000c,0x0e04005a,
0x00000000,0x0f030042,0x0f05000e,0x0f09000c,0x0f040023,0x0f08000e,0x0f08000d,0x0f08000f,
0x00000000,0x01010001,0x0f09000b,0x0d030071,0x0f030042,0x0f09000c,0x0f08000e,0x0f08000f,
0x00000000,0x0d030061,0x0f09000c,0x0f08000f,0x0f020081,0x0f08000f,0x0f08000f,0x0f0c000d,
0x00000000,0x01010001,0x04010002,0x02020002,0x08010008,0x05020003,0x05010004,0x03030003,
0x0e0100b0,0x09020009,0x04010003,0x06030004,0x09010010,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f020099,0x05020003,0x0f04000e,0x0f020024,0x05020004,0x05010006,0x07040005,
0x0f010130,0x0f030015,0x05020004,0x07030006,0x0d030042,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x08020004,0x1003008e,0x08010009,0x06030004,0x06020005,0x0d06000d,
0x10010230,0x0d040023,0x08020005,0x06030005,0x09010011,0x06020007,0x06020006,0x08050006,
0x00000000,0x10020119,0x06030004,0x0d050014,0x0f020025,0x06030005,0x06020007,0x08040007,
0x00000000,0x0d030062,0x06030005,0x07030007,0x0d030043,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0d03000f,0x02020002,0x0801000a,0x09030005,0x09020006,0x03030003,
0x00000000,0x0902000a,0x0d030010,0x07040005,0x09010012,0x07030006,0x09020007,0x0d060011,
0x00000000,0x01010001,0x09030005,0x0d040033,0x0f020026,0x09030006,0x09020008,0x07040006,
0x00000000,0x0f030016,0x09030006,0x07030008,0x0d030044,0x07030007,0x09020009,0x09060007,
0x00000000,0x01010001,0x09020006,0x02020002,0x0801000b,0x07040005,0x07030006,0x0d05001c,
0x00000000,0x0d040024,0x09020007,0x07040006,0x09010013,0x07030008,0x07030007,0x09050008,
0x00000000,0x01010001,0x07040005,0x0d030072,0x0f020027,0x07040006,0x07030008,0x08040008,
0x00000000,0x0d030063,0x07040006,0x07030009,0x0d030045,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0801000c,0x0d030011,0x0d020012,0x03030003,
0x0e0100b1,0x0902000b,0x04010003,0x0f05000a,0x09010014,0x0f04000b,0x0d020013,0x04040004,
0x00000000,0x01010001,0x0d030011,0x0f04000f,0x0f020028,0x0d030012,0x0d020014,0x08050006,
0x0f010131,0x0f030017,0x0d030012,0x08040007,0x0d030046,0x0f04000c,0x0d020015,0x0d060013,
0x00000000,0x01010001,0x08020005,0x02020002,0x0801000d,0x0f05000a,0x0f04000b,0x0d04003b,
0x10010231,0x0d040025,0x08020006,0x0f05000b,0x09010015,0x0f04000d,0x0f04000c,0x08050007,
0x00000000,0x01010001,0x0f05000a,0x0d050015,0x0f020029,0x0f05000b,0x0f04000d,0x08040009,
0x00000000,0x0d030064,0x0f05000b,0x08040008,0x0d030047,0x0f04000e,0x0f04000e,0x0f08000c,
0x00000000,0x01010001,0x0d02001e,0x02020002,0x0801000e,0x0f04000b,0x0f03000c,0x03030003,
0x00000000,0x0902000c,0x0d02001f,0x08050006,0x09010016,0x08040007,0x0f03000d,0x0d050020,
0x00000000,0x01010001,0x0f04000b,0x0d040034,0x0f02002a,0x0f04000c,0x0f03000e,0x08050007,
0x00000000,0x0f030018,0x0f04000c,0x08040009,0x0d030048,0x08040008,0x0f03000f,0x0f07000d,
0x00000000,0x01010001,0x09020007,0x02020002,0x0801000f,0x08050006,0x08040007,0x0e04005b,
0x00000000,0x0d040026,0x09020008,0x08050007,0x09010017,0x08040009,0x08040008,0x09050009,
0x00000000,0x01010001,0x08050006,0x0d030073,0x0f02002b,0x08050007,0x08040009,0x0804000a,
0x00000000,0x0d030065,0x08050007,0x0804000a,0x0d030049,0x0804000a,0x0804000a,0x08080008,
0x00000000,0x01010001,0x04010002,0x02020002,0x09020009,0x05020003,0x05010004,0x03030003,
0x0e0100b2,0x0902000d,0x04010003,0x0e04000f,0x09010018,0x0e030010,0x05010005,0x04040004,
0x00000000,0x0f02009a,0x05020003,0x0f040010,0x0f02002c,0x05020004,0x05010006,0x0d070009,
0x0f010132,0x0f030019,0x05020004,0x0d06000a,0x0d03004a,0x0e030011,0x05010007,0x05050005,
0x00000000,0x01010001,0x09030005,0x02020002,0x0902000a,0x0e04000f,0x0e030010,0x0d06000e,
0x10010232,0x0d040027,0x09030006,0x0e040010,0x09010019,0x0e030012,0x0e030011,0x09060007,
0x00000000,0x1002011a,0x0e04000f,0x0d050016,0x0f02002d,0x0e040010,0x0e030012,0x09050008,
0x00000000,0x0d030066,0x0e040010,0x0d06000b,0x0d03004b,0x0e030013,0x0e030013,0x0e070011,
0x00000000,0x01010001,0x0e02002e,0x02020002,0x0902000b,0x09030006,0x09020007,0x03030003,
0x00000000,0x0902000e,0x0e02002f,0x0d070009,0x0901001a,0x0d06000a,0x09020008,0x0e050030,
0x00000000,0x01010001,0x09030006,0x0d040035,0x0f02002e,0x09030007,0x09020009,0x0d07000a,
0x00000000,0x0f03001a,0x09030007,0x0d06000c,0x0d03004c,0x0d06000b,0x0902000a,0x09060008,
0x00000000,0x01010001,0x09020008,0x02020002,0x0902000c,0x0d070009,0x0d06000a,0x0d05001d,
0x00000000,0x0d040028,0x09020009,0x0d07000a,0x0901001b,0x0d06000c,0x0d06000b,0x0905000a,
0x00000000,0x01010001,0x0d070009,0x0d030074,0x0f02002f,0x0d07000a,0x0d06000c,0x09050009,
0x00000000,0x0d030067,0x0d07000a,0x0d06000d,0x0d03004d,0x0d06000d,0x0d06000d,0x0d0a000b,
0x00000000,0x01010001,0x04010002,0x02020002,0x0902000d,0x0e030019,0x0e02001a,0x03030003,
0x0e0100b3,0x0902000f,0x04010003,0x0d06000a,0x0901001c,0x0d05000b,0x0e02001b,0x04040004,
0x00000000,0x01010001,0x0e030019,0x0f040011,0x0f020030,0x0e03001a,0x0e02001c,0x09060007,
0x0f010133,0x0f03001b,0x0e03001a,0x09050008,0x0d03004e,0x0d05000c,0x0e02001d,0x0e06001b,
0x00000000,0x01010001,0x09030006,0x02020002,0x0902000e,0x0d06000a,0x0d05000b,0x0d04003c,
0x10010233,0x0d040029,0x09030007,0x0d06000b,0x0901001d,0x0d05000d,0x0d05000c,0x09060008,
0x00000000,0x01010001,0x0d06000a,0x0d050017,0x0f020031,0x0d06000b,0x0d05000d,0x0905000a,
0x00000000,0x0d030068,0x0d06000b,0x09050009,0x0d03004f,0x0d05000e,0x0d05000e,0x0d09000c,
0x00000000,0x01010001,0x0f02004e,0x02020002,0x0902000f,0x0f04000c,0x0f03000d,0x03030003,
0x00000000,0x09020010,0x0f02004f,0x09060007,0x0901001e,0x09050008,0x0f03000e,0x0f050050,
0x00000000,0x01010001,0x0f04000c,0x0d040036,0x0f020032,0x0f04000d,0x0f03000f,0x09060008,
0x00000000,0x0f03001c,0x0f04000d,0x0905000a,0x0d030050,0x09050009,0x0f030010,0x0f07000e,
0x00000000,0x01010001,0x09020009,0x02020002,0x09020010,0x09060007,0x09050008,0x0e04005c,
0x00000000,0x0d04002a,0x0902000a,0x09060008,0x0901001f,0x0905000a,0x09050009,0x0905000b,
0x00000000,0x01010001,0x09060007,0x0d030075,0x0f020033,0x09060008,0x0905000a,0x0905000b,
0x00000000,0x0d030069,0x09060008,0x0905000b,0x0d030051,0x0905000b,0x0905000b,0x09090009,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d03000c,0x05020003,0x05010004,0x03030003,
0x0e0100b4,0x0f04000e,0x04010003,0x06030004,0x0f030015,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f02009b,0x05020003,0x0f040012,0x0f020034,0x05020004,0x05010006,0x07040005,
0x0f010134,0x0f03001d,0x05020004,0x07030006,0x0d030052,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0d040008,0x1003008f,0x0d03000d,0x06030004,0x06020005,0x0d06000f,
0x10010234,0x0d04002b,0x0d040009,0x06030005,0x0f030016,0x06020007,0x06020006,0x0d07000a,
0x00000000,0x1002011b,0x06030004,0x0d050018,0x0f020035,0x06030005,0x06020007,0x0d06000b,
0x00000000,0x0d03006a,0x06030005,0x07030007,0x0d030053,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0d030010,0x02020002,0x0d03000e,0x0f05000a,0x0f04000b,0x03030003,
0x00000000,0x0f04000f,0x0d030011,0x07040005,0x0f030017,0x07030006,0x0f04000c,0x0d060012,
0x00000000,0x01010001,0x0f05000a,0x0d040037,0x0f020036,0x0f05000b,0x0f04000d,0x07040006,
0x00000000,0x0f03001e,0x0f05000b,0x07030008,0x0d030054,0x07030007,0x0f04000e,0x0f08000c,
0x00000000,0x01010001,0x0f04000b,0x02020002,0x0d03000f,0x07040005,0x07030006,0x0d05001e,
0x00000000,0x0d04002c,0x0f04000c,0x07040006,0x0f030018,0x07030008,0x07030007,0x0f07000d,
0x00000000,0x01010001,0x07040005,0x0d030076,0x0f020037,0x07040006,0x07030008,0x0d06000c,
0x00000000,0x0d03006b,0x07040006,0x07030009,0x0d030055,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d030010,0x0f030029,0x0f02002a,0x03030003,
0x0e0100b5,0x0f040010,0x04010003,0x0f05000b,0x0f030019,0x0f04000c,0x0f02002b,0x04040004,
0x00000000,0x01010001,0x0f030029,0x0f040013,0x0f020038,0x0f03002a,0x0f02002c,0x0d07000a,
0x0f010135,0x0f03001f,0x0f03002a,0x0d06000b,0x0d030056,0x0f04000d,0x0f02002d,0x0f06002b,
0x00000000,0x01010001,0x0d040009,0x02020002,0x0d030011,0x0f05000b,0x0f04000c,0x0d04003d,
0x10010235,0x0d04002d,0x0d04000a,0x0f05000c,0x0f03001a,0x0f04000e,0x0f04000d,0x0d07000b,
0x00000000,0x01010001,0x0f05000b,0x0d050019,0x0f020039,0x0f05000c,0x0f04000e,0x0d06000d,
0x00000000,0x0d03006c,0x0f05000c,0x0d06000c,0x0d030057,0x0f04000f,0x0f04000f,0x0f08000d,
0x00000000,0x01010001,0x0d02001f,0x02020002,0x0d030012,0x0f04000d,0x0f03000e,0x03030003,
0x00000000,0x0f040011,0x0d020020,0x0d07000a,0x0f03001b,0x0d06000b,0x0f03000f,0x0d050021,
0x00000000,0x01010001,0x0f04000d,0x0d040038,0x0f02003a,0x0f04000e,0x0f030010,0x0d07000b,
0x00000000,0x0f030020,0x0f04000e,0x0d06000d,0x0d030058,0x0d06000c,0x0f030011,0x0f07000f,
0x00000000,0x01010001,0x0f04000c,0x02020002,0x0d030013,0x0d07000a,0x0d06000b,0x0e04005d,
0x00000000,0x0d04002e,0x0f04000d,0x0d07000b,0x0f03001c,0x0d06000d,0x0d06000c,0x0f07000e,
0x00000000,0x01010001,0x0d07000a,0x0d030077,0x0f02003b,0x0d07000b,0x0d06000d,0x0d06000e,
0x00000000,0x0d03006d,0x0d07000b,0x0d06000e,0x0d030059,0x0d06000e,0x0d06000e,0x0d0a000c,
0x00000000,0x01010001,0x04010002,0x02020002,0x0f04000e,0x05020003,0x05010004,0x03030003,
0x0e0100b6,0x0f040012,0x04010003,0x0f040017,0x0f03001d,0x0f030018,0x05010005,0x04040004,
0x00000000,0x0f02009c,0x05020003,0x0f040014,0x0f02003c,0x05020004,0x05010006,0x0d06000c,
0x0f010136,0x0f030021,0x05020004,0x0d05000d,0x0d03005a,0x0f030019,0x05010007,0x05050005,
0x00000000,0x01010001,0x0f05000a,0x02020002,0x0f04000f,0x0f040017,0x0f030018,0x0d060010,
0x10010236,0x0d04002f,0x0f05000b,0x0f040018,0x0f03001e,0x0f03001a,0x0f030019,0x0f08000c,
0x00000000,0x1002011c,0x0f040017,0x0d05001a,0x0f02003d,0x0f040018,0x0f03001a,0x0f07000d,
0x00000000,0x0d03006e,0x0f040018,0x0d05000e,0x0d03005b,0x0f03001b,0x0f03001b,0x0f070019,
0x00000000,0x01010001,0x0e02002f,0x02020002,0x0f040010,0x0f05000b,0x0f04000c,0x03030003,
0x00000000,0x0f040013,0x0e020030,0x0d06000c,0x0f03001f,0x0d05000d,0x0f04000d,0x0e050031,
0x00000000,0x01010001,0x0f05000b,0x0d040039,0x0f02003e,0x0f05000c,0x0f04000e,0x0d06000d,
0x00000000,0x0f030022,0x0f05000c,0x0d05000f,0x0d03005c,0x0d05000e,0x0f04000f,0x0f08000d,
0x00000000,0x01010001,0x0f04000d,0x02020002,0x0f040011,0x0d06000c,0x0d05000d,0x0d05001f,
0x00000000,0x0d040030,0x0f04000e,0x0d06000d,0x0f030020,0x0d05000f,0x0d05000e,0x0f07000f,
0x00000000,0x01010001,0x0d06000c,0x0d030078,0x0f02003f,0x0d06000d,0x0d05000f,0x0f07000e,
0x00000000,0x0d03006f,0x0d06000d,0x0d050010,0x0d03005d,0x0d050010,0x0d050010,0x0d09000e,
0x00000000,0x01010001,0x04010002,0x02020002,0x0f040012,0x10030049,0x1002004a,0x03030003,
0x0e0100b7,0x0f040014,0x04010003,0x0d06000b,0x0f030021,0x0d05000c,0x1002004b,0x04040004,
0x00000000,0x01010001,0x10030049,0x0f040015,0x0f020040,0x1003004a,0x1002004c,0x0f08000c,
0x0f010137,0x0f030023,0x1003004a,0x0f07000d,0x0d03005e,0x0d05000d,0x1002004d,0x1006004b,
0x00000000,0x01010001,0x0f05000b,0x02020002,0x0f040013,0x0d06000b,0x0d05000c,0x0d04003e,
0x10010237,0x0d040031,0x0f05000c,0x0d06000c,0x0f030022,0x0d05000e,0x0d05000d,0x0f08000d,
0x00000000,0x01010001,0x0d06000b,0x0d05001b,0x0f020041,0x0d06000c,0x0d05000e,0x0f07000f,
0x00000000,0x0d030070,0x0d06000c,0x0f07000e,0x0d03005f,0x0d05000f,0x0d05000f,0x0d09000d,
0x00000000,0x01010001,0x0f02004f,0x02020002,0x0f040014,0x0f04000e,0x0f03000f,0x03030003,
0x00000000,0x0f040015,0x0f020050,0x0f08000c,0x0f030023,0x0f07000d,0x0f030010,0x0f050051,
0x00000000,0x01010001,0x0f04000e,0x0d04003a,0x0f020042,0x0f04000f,0x0f030011,0x0f08000d,
0x00000000,0x0f030024,0x0f04000f,0x0f07000f,0x0d030060,0x0f07000e,0x0f030012,0x0f070010,
0x00000000,0x01010001,0x0f04000e,0x02020002,0x0f040015,0x0f08000c,0x0f07000d,0x0e04005e,
0x00000000,0x0d040032,0x0f04000f,0x0f08000d,0x0f030024,0x0f07000f,0x0f07000e,0x0f070010,
0x00000000,0x01010001,0x0f08000c,0x0d030079,0x0f020043,0x0f08000d,0x0f07000f,0x0f070010,
0x00000000,0x0d030071,0x0f08000d,0x0f070010,0x0d030061,0x0f070010,0x0f070010,0x0f0b000e,
0x00000000,0x01010001,0x04010002,0x02020002,0x08010008,0x05020003,0x05010004,0x03030003,
0x0e0100b8,0x0e03000e,0x04010003,0x06030004,0x0e020015,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f02009d,0x05020003,0x0d06000d,0x0d040023,0x05020004,0x05010006,0x07040005,
0x0f010138,0x0d050014,0x05020004,0x07030006,0x0d030062,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x08020004,0x10030090,0x08010009,0x06030004,0x06020005,0x0d060011,
0x10010238,0x0d040033,0x08020005,0x06030005,0x0e020016,0x06020007,0x06020006,0x08050006,
0x00000000,0x1002011d,0x06030004,0x0d05001c,0x0d040024,0x06030005,0x06020007,0x08040007,
0x00000000,0x0d030072,0x06030005,0x07030007,0x0d030063,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0d030011,0x02020002,0x0801000a,0x0e04000a,0x0e03000b,0x03030003,
0x00000000,0x0e03000f,0x0d030012,0x07040005,0x0e020017,0x07030006,0x0e03000c,0x0d060013,
0x00000000,0x01010001,0x0e04000a,0x0d04003b,0x0d040025,0x0e04000b,0x0e03000d,0x07040006,
0x00000000,0x0d050015,0x0e04000b,0x07030008,0x0d030064,0x07030007,0x0e03000e,0x0e07000c,
0x00000000,0x01010001,0x0e03000b,0x02020002,0x0801000b,0x07040005,0x07030006,0x0d050020,
0x00000000,0x0d040034,0x0e03000c,0x07040006,0x0e020018,0x07030008,0x07030007,0x0e06000d,
0x00000000,0x01010001,0x07040005,0x0d03007a,0x0d040026,0x07040006,0x07030008,0x08040008,
0x00000000,0x0d030073,0x07040006,0x07030009,0x0d030065,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0801000c,0x0d030012,0x0d020013,0x03030003,
0x0e0100b9,0x0e030010,0x04010003,0x0d070009,0x0e020019,0x0d06000a,0x0d020014,0x04040004,
0x00000000,0x01010001,0x0d030012,0x0d06000e,0x0d040027,0x0d030013,0x0d020015,0x08050006,
0x0f010139,0x0d050016,0x0d030013,0x08040007,0x0d030066,0x0d06000b,0x0d020016,0x0d060014,
0x00000000,0x01010001,0x08020005,0x02020002,0x0801000d,0x0d070009,0x0d06000a,0x0d04003f,
0x10010239,0x0d040035,0x08020006,0x0d07000a,0x0e02001a,0x0d06000c,0x0d06000b,0x08050007,
0x00000000,0x01010001,0x0d070009,0x0d05001d,0x0d040028,0x0d07000a,0x0d06000c,0x08040009,
0x00000000,0x0d030074,0x0d07000a,0x08040008,0x0d030067,0x0d06000d,0x0d06000d,0x0d0a000b,
0x00000000,0x01010001,0x0d020020,0x02020002,0x0801000e,0x0d06000a,0x0d05000b,0x03030003,
0x00000000,0x0e030011,0x0d020021,0x08050006,0x0e02001b,0x08040007,0x0d05000c,0x0d050022,
0x00000000,0x01010001,0x0d06000a,0x0d04003c,0x0d040029,0x0d06000b,0x0d05000d,0x08050007,
0x00000000,0x0d050017,0x0d06000b,0x08040009,0x0d030068,0x08040008,0x0d05000e,0x0d09000c,
0x00000000,0x01010001,0x0e03000c,0x02020002,0x0801000f,0x08050006,0x08040007,0x0e04005f,
0x00000000,0x0d040036,0x0e03000d,0x08050007,0x0e02001c,0x08040009,0x08040008,0x0e06000e,
0x00000000,0x01010001,0x08050006,0x0d03007b,0x0d04002a,0x08050007,0x08040009,0x0804000a,
0x00000000,0x0d030075,0x08050007,0x0804000a,0x0d030069,0x0804000a,0x0804000a,0x08080008,
0x00000000,0x01010001,0x04010002,0x02020002,0x0e03000e,0x05020003,0x05010004,0x03030003,
0x0e0100ba,0x0e030012,0x04010003,0x10040027,0x0e02001d,0x10030028,0x05010005,0x04040004,
0x00000000,0x0f02009e,0x05020003,0x0d06000f,0x0d04002b,0x05020004,0x05010006,0x0d07000a,
0x0f01013a,0x0d050018,0x05020004,0x0d06000b,0x0d03006a,0x10030029,0x05010007,0x05050005,
0x00000000,0x01010001,0x0e04000a,0x02020002,0x0e03000f,0x10040027,0x10030028,0x0d060012,
0x1001023a,0x0d040037,0x0e04000b,0x10040028,0x0e02001e,0x1003002a,0x10030029,0x0e07000c,
0x00000000,0x1002011e,0x10040027,0x0d05001e,0x0d04002c,0x10040028,0x1003002a,0x0e06000d,
0x00000000,0x0d030076,0x10040028,0x0d06000c,0x0d03006b,0x1003002b,0x1003002b,0x10070029,
0x00000000,0x01010001,0x0e020030,0x02020002,0x0e030010,0x0e04000b,0x0e03000c,0x03030003,
0x00000000,0x0e030013,0x0e020031,0x0d07000a,0x0e02001f,0x0d06000b,0x0e03000d,0x0e050032,
0x00000000,0x01010001,0x0e04000b,0x0d04003d,0x0d04002d,0x0e04000c,0x0e03000e,0x0d07000b,
0x00000000,0x0d050019,0x0e04000c,0x0d06000d,0x0d03006c,0x0d06000c,0x0e03000f,0x0e07000d,
0x00000000,0x01010001,0x0e03000d,0x02020002,0x0e030011,0x0d07000a,0x0d06000b,0x0d050021,
0x00000000,0x0d040038,0x0e03000e,0x0d07000b,0x0e020020,0x0d06000d,0x0d06000c,0x0e06000f,
0x00000000,0x01010001,0x0d07000a,0x0d03007c,0x0d04002e,0x0d07000b,0x0d06000d,0x0e06000e,
0x00000000,0x0d030077,0x0d07000b,0x0d06000e,0x0d03006d,0x0d06000e,0x0d06000e,0x0d0a000c,
0x00000000,0x01010001,0x04010002,0x02020002,0x0e030012,0x0e03001a,0x0e02001b,0x03030003,
0x0e0100bb,0x0e030014,0x04010003,0x0d06000c,0x0e020021,0x0d05000d,0x0e02001c,0x04040004,
0x00000000,0x01010001,0x0e03001a,0x0d060010,0x0d04002f,0x0e03001b,0x0e02001d,0x0e07000c,
0x0f01013b,0x0d05001a,0x0e03001b,0x0e06000d,0x0d03006e,0x0d05000e,0x0e02001e,0x0e06001c,
0x00000000,0x01010001,0x0e04000b,0x02020002,0x0e030013,0x0d06000c,0x0d05000d,0x0d040040,
0x1001023b,0x0d040039,0x0e04000c,0x0d06000d,0x0e020022,0x0d05000f,0x0d05000e,0x0e07000d,
0x00000000,0x01010001,0x0d06000c,0x0d05001f,0x0d040030,0x0d06000d,0x0d05000f,0x0e06000f,
0x00000000,0x0d030078,0x0d06000d,0x0e06000e,0x0d03006f,0x0d050010,0x0d050010,0x0d09000e,
0x00000000,0x01010001,0x0f020050,0x02020002,0x0e030014,0x0d06000b,0x0d05000c,0x03030003,
0x00000000,0x0e030015,0x0f020051,0x0e07000c,0x0e020023,0x0e06000d,0x0d05000d,0x0f050052,
0x00000000,0x01010001,0x0d06000b,0x0d04003e,0x0d040031,0x0d06000c,0x0d05000e,0x0e07000d,
0x00000000,0x0d05001b,0x0d06000c,0x0e06000f,0x0d030070,0x0e06000e,0x0d05000f,0x0d09000d,
0x00000000,0x01010001,0x0e03000e,0x02020002,0x0e030015,0x0e07000c,0x0e06000d,0x0e040060,
0x00000000,0x0d04003a,0x0e03000f,0x0e07000d,0x0e020024,0x0e06000f,0x0e06000e,0x0e060010,
0x00000000,0x01010001,0x0e07000c,0x0d03007d,0x0d040032,0x0e07000d,0x0e06000f,0x0e060010,
0x00000000,0x0d030079,0x0e07000d,0x0e060010,0x0d030071,0x0e060010,0x0e060010,0x0e0a000e,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d02000f,0x05020003,0x05010004,0x03030003,
0x0e0100bc,0x0d06000d,0x04010003,0x06030004,0x0d050014,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f02009f,0x05020003,0x0d060011,0x0d040033,0x05020004,0x05010006,0x07040005,
0x0f01013c,0x0d05001c,0x05020004,0x07030006,0x0d030072,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0d03000b,0x10030091,0x0d020010,0x06030004,0x06020005,0x0d060013,
0x1001023c,0x0d04003b,0x0d03000c,0x06030005,0x0d050015,0x06020007,0x06020006,0x0d06000d,
0x00000000,0x1002011f,0x06030004,0x0d050020,0x0d040034,0x06030005,0x06020007,0x0d05000e,
0x00000000,0x0d03007a,0x06030005,0x07030007,0x0d030073,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0d030012,0x02020002,0x0d020011,0x0d070009,0x0d06000a,0x03030003,
0x00000000,0x0d06000e,0x0d030013,0x07040005,0x0d050016,0x07030006,0x0d06000b,0x0d060014,
0x00000000,0x01010001,0x0d070009,0x0d04003f,0x0d040035,0x0d07000a,0x0d06000c,0x07040006,
0x00000000,0x0d05001d,0x0d07000a,0x07030008,0x0d030074,0x07030007,0x0d06000d,0x0d0a000b,
0x00000000,0x01010001,0x0d06000a,0x02020002,0x0d020012,0x07040005,0x07030006,0x0d050022,
0x00000000,0x0d04003c,0x0d06000b,0x07040006,0x0d050017,0x07030008,0x07030007,0x0d09000c,
0x00000000,0x01010001,0x07040005,0x0d03007e,0x0d040036,0x07040006,0x07030008,0x0d05000f,
0x00000000,0x0d03007b,0x07040006,0x07030009,0x0d030075,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d020013,0x0f03002a,0x0f02002b,0x03030003,
0x0e0100bd,0x0d06000f,0x04010003,0x0d07000a,0x0d050018,0x0d06000b,0x0f02002c,0x04040004,
0x00000000,0x01010001,0x0f03002a,0x0d060012,0x0d040037,0x0f03002b,0x0f02002d,0x0d06000d,
0x0f01013d,0x0d05001e,0x0f03002b,0x0d05000e,0x0d030076,0x0d06000c,0x0f02002e,0x0f06002c,
0x00000000,0x01010001,0x0d03000c,0x02020002,0x0d020014,0x0d07000a,0x0d06000b,0x0d040041,
0x1001023d,0x0d04003d,0x0d03000d,0x0d07000b,0x0d050019,0x0d06000d,0x0d06000c,0x0d06000e,
0x00000000,0x01010001,0x0d07000a,0x0d050021,0x0d040038,0x0d07000b,0x0d06000d,0x0d050010,
0x00000000,0x0d03007c,0x0d07000b,0x0d05000f,0x0d030077,0x0d06000e,0x0d06000e,0x0d0a000c,
0x00000000,0x01010001,0x0d020021,0x02020002,0x0d020015,0x0d06000c,0x0d05000d,0x03030003,
0x00000000,0x0d060010,0x0d020022,0x0d06000d,0x0d05001a,0x0d05000e,0x0d05000e,0x0d050023,
0x00000000,0x01010001,0x0d06000c,0x0d040040,0x0d040039,0x0d06000d,0x0d05000f,0x0d06000e,
0x00000000,0x0d05001f,0x0d06000d,0x0d050010,0x0d030078,0x0d05000f,0x0d050010,0x0d09000e,
0x00000000,0x01010001,0x0d06000b,0x02020002,0x0d020016,0x0d06000d,0x0d05000e,0x0e040061,
0x00000000,0x0d04003e,0x0d06000c,0x0d06000e,0x0d05001b,0x0d050010,0x0d05000f,0x0d09000d,
0x00000000,0x01010001,0x0d06000d,0x0d03007f,0x0d04003a,0x0d06000e,0x0d050010,0x0d050011,
0x00000000,0x0d03007d,0x0d06000e,0x0d050011,0x0d030079,0x0d050011,0x0d050011,0x0d09000f,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d06000d,0x05020003,0x05010004,0x03030003,
0x0e0100be,0x0d060011,0x04010003,0x06030004,0x0d05001c,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f0200a0,0x05020003,0x0d060013,0x0d04003b,0x05020004,0x05010006,0x0d06000d,
0x0f01013e,0x0d050020,0x05020004,0x0d05000e,0x0d03007a,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0d070009,0x02020002,0x0d06000e,0x06030004,0x06020005,0x0d060014,
0x1001023e,0x0d04003f,0x0d07000a,0x06030005,0x0d05001d,0x06020007,0x06020006,0x0d0a000b,
0x00000000,0x10020120,0x06030004,0x0d050022,0x0d04003c,0x06030005,0x06020007,0x0d09000c,
0x00000000,0x0d03007e,0x06030005,0x0d05000f,0x0d03007b,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0e020031,0x02020002,0x0d06000f,0x0d07000a,0x0d06000b,0x03030003,
0x00000000,0x0d060012,0x0e020032,0x0d06000d,0x0d05001e,0x0d05000e,0x0d06000c,0x0e050033,
0x00000000,0x01010001,0x0d07000a,0x0d040041,0x0d04003d,0x0d07000b,0x0d06000d,0x0d06000e,
0x00000000,0x0d050021,0x0d07000b,0x0d050010,0x0d03007c,0x0d05000f,0x0d06000e,0x0d0a000c,
0x00000000,0x01010001,0x0d06000c,0x02020002,0x0d060010,0x0d06000d,0x0d05000e,0x0d050023,
0x00000000,0x0d040040,0x0d06000d,0x0d06000e,0x0d05001f,0x0d050010,0x0d05000f,0x0d09000e,
0x00000000,0x01010001,0x0d06000d,0x0d030080,0x0d04003e,0x0d06000e,0x0d050010,0x0d09000d,
0x00000000,0x0d03007f,0x0d06000e,0x0d050011,0x0d03007d,0x0d050011,0x0d050011,0x0d09000f,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d060011,0x1003004a,0x1002004b,0x03030003,
0x0e0100bf,0x0d060013,0x04010003,0x0d06000d,0x0d050020,0x0d05000e,0x1002004c,0x04040004,
0x00000000,0x01010001,0x1003004a,0x0d060014,0x0d04003f,0x1003004b,0x1002004d,0x0d0a000b,
0x0f01013f,0x0d050022,0x1003004b,0x0d09000c,0x0d03007e,0x0d05000f,0x1002004e,0x1006004c,
0x00000000,0x01010001,0x0d07000a,0x02020002,0x0d060012,0x0d06000d,0x0d05000e,0x0d040042,
0x1001023f,0x0d040041,0x0d07000b,0x0d06000e,0x0d050021,0x0d050010,0x0d05000f,0x0d0a000c,
0x00000000,0x01010001,0x0d06000d,0x0d050023,0x0d040040,0x0d06000e,0x0d050010,0x0d09000e,
0x00000000,0x0d030080,0x0d06000e,0x0d09000d,0x0d03007f,0x0d050011,0x0d050011,0x0d09000f,
0x00000000,0x01010001,0x0f020051,0x02020002,0x0d060013,0x0d06000d,0x0d05000e,0x03030003,
0x00000000,0x0d060014,0x0f020052,0x0d0a000b,0x0d050022,0x0d09000c,0x0d05000f,0x0f050053,
0x00000000,0x01010001,0x0d06000d,0x0d040042,0x0d040041,0x0d06000e,0x0d050010,0x0d0a000c,
0x00000000,0x0d050023,0x0d06000e,0x0d09000e,0x0d030080,0x0d09000d,0x0d050011,0x0d09000f,
0x00000000,0x01010001,0x0d06000d,0x02020002,0x0d060014,0x0d0a000b,0x0d09000c,0x0e040062,
0x00000000,0x0d040042,0x0d06000e,0x0d0a000c,0x0d050023,0x0d09000e,0x0d09000d,0x0d09000f,
0x00000000,0x01010001,0x0d0a000b,0x0d030081,0x0d040042,0x0d0a000c,0x0d09000e,0x0d09000f,
0x00000000,0x0d030081,0x0d0a000c,0x0d09000f,0x0d030081,0x0d09000f,0x0d09000f,0x0d0d000d,
0x00000000,0x01010001,0x04010002,0x02020002,0x08010008,0x05020003,0x05010004,0x03030003,
0x0e0100c0,0x09020009,0x04010003,0x06030004,0x09010010,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f0200a1,0x05020003,0x0a03000a,0x0a010020,0x05020004,0x05010006,0x07040005,
0x0f010140,0x0a020011,0x05020004,0x07030006,0x0b010040,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x08020004,0x10030092,0x08010009,0x06030004,0x06020005,0x0b04000b,
0x10010240,0x0b020021,0x08020005,0x06030005,0x09010011,0x06020007,0x06020006,0x08050006,
0x00000000,0x10020121,0x06030004,0x0b030012,0x0a010021,0x06030005,0x06020007,0x08040007,
0x00000000,0x10030043,0x06030005,0x07030007,0x0b010041,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x1003000c,0x02020002,0x0801000a,0x09030005,0x09020006,0x03030003,
0x00000000,0x0902000a,0x1003000d,0x07040005,0x09010012,0x07030006,0x09020007,0x1006000e,
0x00000000,0x01010001,0x09030005,0x10040024,0x0a010022,0x09030006,0x09020008,0x07040006,
0x00000000,0x0a020012,0x09030006,0x07030008,0x0b010042,0x07030007,0x09020009,0x09060007,
0x00000000,0x01010001,0x09020006,0x02020002,0x0801000b,0x07040005,0x07030006,0x10050015,
0x00000000,0x0b020022,0x09020007,0x07040006,0x09010013,0x07030008,0x07030007,0x09050008,
0x00000000,0x01010001,0x07040005,0x0e040043,0x0a010023,0x07040006,0x07030008,0x08040008,
0x00000000,0x10030044,0x07040006,0x07030009,0x0b010043,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0801000c,0x0e04000c,0x0e03000d,0x03030003,
0x0e0100c1,0x0902000b,0x04010003,0x0a040006,0x09010014,0x0a030007,0x0e03000e,0x04040004,
0x00000000,0x01010001,0x0e04000c,0x0a03000b,0x0a010024,0x0e04000d,0x0e03000f,0x08050006,
0x0f010141,0x0a020013,0x0e04000d,0x08040007,0x0b010044,0x0a030008,0x0e030010,0x0e07000e,
0x00000000,0x01010001,0x08020005,0x02020002,0x0801000d,0x0a040006,0x0a030007,0x0e050024,
0x10010241,0x0b020023,0x08020006,0x0a040007,0x09010015,0x0a030009,0x0a030008,0x08050007,
0x00000000,0x01010001,0x0a040006,0x0b030013,0x0a010025,0x0a040007,0x0a030009,0x08040009,
0x00000000,0x10030045,0x0a040007,0x08040008,0x0b010045,0x0a03000a,0x0a03000a,0x0a070008,
0x00000000,0x01010001,0x0e030013,0x02020002,0x0801000e,0x0a030007,0x0a020008,0x03030003,
0x00000000,0x0902000c,0x0e030014,0x08050006,0x09010016,0x08040007,0x0a020009,0x0e060015,
0x00000000,0x01010001,0x0a030007,0x10040025,0x0a010026,0x0a030008,0x0a02000a,0x08050007,
0x00000000,0x0a020014,0x0a030008,0x08040009,0x0b010046,0x08040008,0x0a02000b,0x0a060009,
0x00000000,0x01010001,0x09020007,0x02020002,0x0801000f,0x08050006,0x08040007,0x0e040063,
0x00000000,0x0b020024,0x09020008,0x08050007,0x09010017,0x08040009,0x08040008,0x09050009,
0x00000000,0x01010001,0x08050006,0x0e040044,0x0a010027,0x08050007,0x08040009,0x0804000a,
0x00000000,0x10030046,0x08050007,0x0804000a,0x0b010047,0x0804000a,0x0804000a,0x08080008,
0x00000000,0x01010001,0x04010002,0x02020002,0x09020009,0x05020003,0x05010004,0x03030003,
0x0e0100c2,0x0902000d,0x04010003,0x0e040010,0x09010018,0x0e030011,0x05010005,0x04040004,
0x00000000,0x0f0200a2,0x05020003,0x0a03000c,0x0a010028,0x05020004,0x05010006,0x0b050007,
0x0f010142,0x0a020015,0x05020004,0x0b040008,0x0b010048,0x0e030012,0x05010007,0x05050005,
0x00000000,0x01010001,0x09030005,0x02020002,0x0902000a,0x0e040010,0x0e030011,0x0b04000c,
0x10010242,0x0b020025,0x09030006,0x0e040011,0x09010019,0x0e030013,0x0e030012,0x09060007,
0x00000000,0x10020122,0x0e040010,0x0b030014,0x0a010029,0x0e040011,0x0e030013,0x09050008,
0x00000000,0x10030047,0x0e040011,0x0b040009,0x0b010049,0x0e030014,0x0e030014,0x0e070012,
0x00000000,0x01010001,0x0e020032,0x02020002,0x0902000b,0x09030006,0x09020007,0x03030003,
0x00000000,0x0902000e,0x0e020033,0x0b050007,0x0901001a,0x0b040008,0x09020008,0x0e050034,
0x00000000,0x01010001,0x09030006,0x10040026,0x0a01002a,0x09030007,0x09020009,0x0b050008,
0x00000000,0x0a020016,0x09030007,0x0b04000a,0x0b01004a,0x0b040009,0x0902000a,0x09060008,
0x00000000,0x01010001,0x09020008,0x02020002,0x0902000c,0x0b050007,0x0b040008,0x10050016,
0x00000000,0x0b020026,0x09020009,0x0b050008,0x0901001b,0x0b04000a,0x0b040009,0x0905000a,
0x00000000,0x01010001,0x0b050007,0x0e040045,0x0a01002b,0x0b050008,0x0b04000a,0x09050009,
0x00000000,0x10030048,0x0b050008,0x0b04000b,0x0b01004b,0x0b04000b,0x0b04000b,0x0b080009,
0x00000000,0x01010001,0x04010002,0x02020002,0x0902000d,0x0e03001b,0x0e02001c,0x03030003,
0x0e0100c3,0x0902000f,0x04010003,0x0b040008,0x0901001c,0x0b030009,0x0e02001d,0x04040004,
0x00000000,0x01010001,0x0e03001b,0x0a03000d,0x0a01002c,0x0e03001c,0x0e02001e,0x09060007,
0x0f010143,0x0a020017,0x0e03001c,0x09050008,0x0b01004c,0x0b03000a,0x0e02001f,0x0e06001d,
0x00000000,0x01010001,0x09030006,0x02020002,0x0902000e,0x0b040008,0x0b030009,0x0e050025,
0x10010243,0x0b020027,0x09030007,0x0b040009,0x0901001d,0x0b03000b,0x0b03000a,0x09060008,
0x00000000,0x01010001,0x0b040008,0x0b030015,0x0a01002d,0x0b040009,0x0b03000b,0x0905000a,
0x00000000,0x10030049,0x0b040009,0x09050009,0x0b01004d,0x0b03000c,0x0b03000c,0x0b07000a,
0x00000000,0x01010001,0x0f020052,0x02020002,0x0902000f,0x0a030008,0x0a020009,0x03030003,
0x00000000,0x09020010,0x0f020053,0x09060007,0x0901001e,0x09050008,0x0a02000a,0x0f050054,
0x00000000,0x01010001,0x0a030008,0x10040027,0x0a01002e,0x0a030009,0x0a02000b,0x09060008,
0x00000000,0x0a020018,0x0a030009,0x0905000a,0x0b01004e,0x09050009,0x0a02000c,0x0a06000a,
0x00000000,0x01010001,0x09020009,0x02020002,0x09020010,0x09060007,0x09050008,0x0e040064,
0x00000000,0x0b020028,0x0902000a,0x09060008,0x0901001f,0x0905000a,0x09050009,0x0905000b,
0x00000000,0x01010001,0x09060007,0x0e040046,0x0a01002f,0x09060008,0x0905000a,0x0905000b,
0x00000000,0x1003004a,0x09060008,0x0905000b,0x0b01004f,0x0905000b,0x0905000b,0x09090009,
0x00000000,0x01010001,0x04010002,0x02020002,0x1003000c,0x05020003,0x05010004,0x03030003,
0x0e0100c4,0x0a03000a,0x04010003,0x06030004,0x0a020011,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f0200a3,0x05020003,0x0a03000e,0x0a010030,0x05020004,0x05010006,0x0f05000f,
0x0f010144,0x0a020019,0x05020004,0x0f040010,0x0b010050,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x10040008,0x10030093,0x1003000d,0x06030004,0x06020005,0x0b04000d,
0x10010244,0x0b020029,0x10040009,0x06030005,0x0a020012,0x06020007,0x06020006,0x1007000a,
0x00000000,0x10020123,0x06030004,0x0b030016,0x0a010031,0x06030005,0x06020007,0x1006000b,
0x00000000,0x1003004b,0x06030005,0x0f040011,0x0b010051,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x1003000d,0x02020002,0x1003000e,0x0a040006,0x0a030007,0x03030003,
0x00000000,0x0a03000b,0x1003000e,0x0f05000f,0x0a020013,0x0f040010,0x0a030008,0x1006000f,
0x00000000,0x01010001,0x0a040006,0x10040028,0x0a010032,0x0a040007,0x0a030009,0x0f050010,
0x00000000,0x0a02001a,0x0a040007,0x0f040012,0x0b010052,0x0f040011,0x0a03000a,0x0a070008,
0x00000000,0x01010001,0x0a030007,0x02020002,0x1003000f,0x0f05000f,0x0f040010,0x10050017,
0x00000000,0x0b02002a,0x0a030008,0x0f050010,0x0a020014,0x0f040012,0x0f040011,0x0a060009,
0x00000000,0x01010001,0x0f05000f,0x0e040047,0x0a010033,0x0f050010,0x0f040012,0x1006000c,
0x00000000,0x1003004c,0x0f050010,0x0f040013,0x0b010053,0x0f040013,0x0f040013,0x0f080011,
0x00000000,0x01010001,0x04010002,0x02020002,0x10030010,0x0f03002b,0x0f02002c,0x03030003,
0x0e0100c5,0x0a03000c,0x04010003,0x0a040007,0x0a020015,0x0a030008,0x0f02002d,0x04040004,
0x00000000,0x01010001,0x0f03002b,0x0a03000f,0x0a010034,0x0f03002c,0x0f02002e,0x1007000a,
0x0f010145,0x0a02001b,0x0f03002c,0x1006000b,0x0b010054,0x0a030009,0x0f02002f,0x0f06002d,
0x00000000,0x01010001,0x10040009,0x02020002,0x10030011,0x0a040007,0x0a030008,0x0e050026,
0x10010245,0x0b02002b,0x1004000a,0x0a040008,0x0a020016,0x0a03000a,0x0a030009,0x1007000b,
0x00000000,0x01010001,0x0a040007,0x0b030017,0x0a010035,0x0a040008,0x0a03000a,0x1006000d,
0x00000000,0x1003004d,0x0a040008,0x1006000c,0x0b010055,0x0a03000b,0x0a03000b,0x0a070009,
0x00000000,0x01010001,0x0e030014,0x02020002,0x10030012,0x0a030009,0x0a02000a,0x03030003,
0x00000000,0x0a03000d,0x0e030015,0x1007000a,0x0a020017,0x1006000b,0x0a02000b,0x0e060016,
0x00000000,0x01010001,0x0a030009,0x10040029,0x0a010036,0x0a03000a,0x0a02000c,0x1007000b,
0x00000000,0x0a02001c,0x0a03000a,0x1006000d,0x0b010056,0x1006000c,0x0a02000d,0x0a06000b,
0x00000000,0x01010001,0x0a030008,0x02020002,0x10030013,0x1007000a,0x1006000b,0x0e040065,
0x00000000,0x0b02002c,0x0a030009,0x1007000b,0x0a020018,0x1006000d,0x1006000c,0x0a06000a,
0x00000000,0x01010001,0x1007000a,0x0e040048,0x0a010037,0x1007000b,0x1006000d,0x1006000e,
0x00000000,0x1003004e,0x1007000b,0x1006000e,0x0b010057,0x1006000e,0x1006000e,0x100a000c,
0x00000000,0x01010001,0x04010002,0x02020002,0x0a03000a,0x05020003,0x05010004,0x03030003,
0x0e0100c6,0x0a03000e,0x04010003,0x0f040018,0x0a020019,0x0f030019,0x05010005,0x04040004,
0x00000000,0x0f0200a4,0x05020003,0x0a030010,0x0a010038,0x05020004,0x05010006,0x1006000b,
0x0f010146,0x0a02001d,0x05020004,0x1005000c,0x0b010058,0x0f03001a,0x05010007,0x05050005,
0x00000000,0x01010001,0x0a040006,0x02020002,0x0a03000b,0x0f040018,0x0f030019,0x0b04000e,
0x10010246,0x0b02002d,0x0a040007,0x0f040019,0x0a02001a,0x0f03001b,0x0f03001a,0x0a070008,
0x00000000,0x10020124,0x0f040018,0x0b030018,0x0a010039,0x0f040019,0x0f03001b,0x0a060009,
0x00000000,0x1003004f,0x0f040019,0x1005000d,0x0b010059,0x0f03001c,0x0f03001c,0x0f07001a,
0x00000000,0x01010001,0x0e020033,0x02020002,0x0a03000c,0x0a040007,0x0a030008,0x03030003,
0x00000000,0x0a03000f,0x0e020034,0x1006000b,0x0a02001b,0x1005000c,0x0a030009,0x0e050035,
0x00000000,0x01010001,0x0a040007,0x1004002a,0x0a01003a,0x0a040008,0x0a03000a,0x1006000c,
0x00000000,0x0a02001e,0x0a040008,0x1005000e,0x0b01005a,0x1005000d,0x0a03000b,0x0a070009,
0x00000000,0x01010001,0x0a030009,0x02020002,0x0a03000d,0x1006000b,0x1005000c,0x10050018,
0x00000000,0x0b02002e,0x0a03000a,0x1006000c,0x0a02001c,0x1005000e,0x1005000d,0x0a06000b,
0x00000000,0x01010001,0x1006000b,0x0e040049,0x0a01003b,0x1006000c,0x1005000e,0x0a06000a,
0x00000000,0x10030050,0x1006000c,0x1005000f,0x0b01005b,0x1005000f,0x1005000f,0x1009000d,
0x00000000,0x01010001,0x04010002,0x02020002,0x0a03000e,0x1003004b,0x1002004c,0x03030003,
0x0e0100c7,0x0a030010,0x04010003,0x0b040009,0x0a02001d,0x0b03000a,0x1002004d,0x04040004,
0x00000000,0x01010001,0x1003004b,0x0a030011,0x0a01003c,0x1003004c,0x1002004e,0x0a070008,
0x0f010147,0x0a02001f,0x1003004c,0x0a060009,0x0b01005c,0x0b03000b,0x1002004f,0x1006004d,
0x00000000,0x01010001,0x0a040007,0x02020002,0x0a03000f,0x0b040009,0x0b03000a,0x0e050027,
0x10010247,0x0b02002f,0x0a040008,0x0b04000a,0x0a02001e,0x0b03000c,0x0b03000b,0x0a070009,
0x00000000,0x01010001,0x0b040009,0x0b030019,0x0a01003d,0x0b04000a,0x0b03000c,0x0a06000b,
0x00000000,0x10030051,0x0b04000a,0x0a06000a,0x0b01005d,0x0b03000d,0x0b03000d,0x0b07000b,
0x00000000,0x01010001,0x0f020053,0x02020002,0x0a030010,0x0a03000a,0x0a02000b,0x03030003,
0x00000000,0x0a030011,0x0f020054,0x0a070008,0x0a02001f,0x0a060009,0x0a02000c,0x0f050055,
0x00000000,0x01010001,0x0a03000a,0x1004002b,0x0a01003e,0x0a03000b,0x0a02000d,0x0a070009,
0x00000000,0x0a020020,0x0a03000b,0x0a06000b,0x0b01005e,0x0a06000a,0x0a02000e,0x0a06000c,
0x00000000,0x01010001,0x0a03000a,0x02020002,0x0a030011,0x0a070008,0x0a060009,0x0e040066,
0x00000000,0x0b020030,0x0a03000b,0x0a070009,0x0a020020,0x0a06000b,0x0a06000a,0x0a06000c,
0x00000000,0x01010001,0x0a070008,0x0e04004a,0x0a01003f,0x0a070009,0x0a06000b,0x0a06000c,
0x00000000,0x10030052,0x0a070009,0x0a06000c,0x0b01005f,0x0a06000c,0x0a06000c,0x0a0a000a,
0x00000000,0x01010001,0x04010002,0x02020002,0x10020011,0x05020003,0x05010004,0x03030003,
0x0e0100c8,0x0e04000c,0x04010003,0x06030004,0x0e030013,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f0200a5,0x05020003,0x0b04000b,0x0b020021,0x05020004,0x05010006,0x07040005,
0x0f010148,0x0b030012,0x05020004,0x07030006,0x0b010060,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x1003000d,0x10030094,0x10020012,0x06030004,0x06020005,0x0b04000f,
0x10010248,0x0b020031,0x1003000e,0x06030005,0x0e030014,0x06020007,0x06020006,0x1006000f,
0x00000000,0x10020125,0x06030004,0x0b03001a,0x0b020022,0x06030005,0x06020007,0x10050010,
0x00000000,0x10030053,0x06030005,0x07030007,0x0b010061,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x1003000e,0x02020002,0x10020013,0x0e050008,0x0e040009,0x03030003,
0x00000000,0x0e04000d,0x1003000f,0x07040005,0x0e030015,0x07030006,0x0e04000a,0x10060010,
0x00000000,0x01010001,0x0e050008,0x1004002c,0x0b020023,0x0e050009,0x0e04000b,0x07040006,
0x00000000,0x0b030013,0x0e050009,0x07030008,0x0b010062,0x07030007,0x0e04000c,0x0e08000a,
0x00000000,0x01010001,0x0e040009,0x02020002,0x10020014,0x07040005,0x07030006,0x10050019,
0x00000000,0x0b020032,0x0e04000a,0x07040006,0x0e030016,0x07030008,0x07030007,0x0e07000b,
0x00000000,0x01010001,0x07040005,0x0e04004b,0x0b020024,0x07040006,0x07030008,0x10050011,
0x00000000,0x10030054,0x07040006,0x07030009,0x0b010063,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x10020015,0x0e04000d,0x0e03000e,0x03030003,
0x0e0100c9,0x0e04000e,0x04010003,0x0b050007,0x0e030017,0x0b040008,0x0e03000f,0x04040004,
0x00000000,0x01010001,0x0e04000d,0x0b04000c,0x0b020025,0x0e04000e,0x0e030010,0x1006000f,
0x0f010149,0x0b030014,0x0e04000e,0x10050010,0x0b010064,0x0b040009,0x0e030011,0x0e07000f,
0x00000000,0x01010001,0x1003000e,0x02020002,0x10020016,0x0b050007,0x0b040008,0x0e050028,
0x10010249,0x0b020033,0x1003000f,0x0b050008,0x0e030018,0x0b04000a,0x0b040009,0x10060010,
0x00000000,0x01010001,0x0b050007,0x0b03001b,0x0b020026,0x0b050008,0x0b04000a,0x10050012,
0x00000000,0x10030055,0x0b050008,0x10050011,0x0b010065,0x0b04000b,0x0b04000b,0x0b080009,
0x00000000,0x01010001,0x0e030015,0x02020002,0x10020017,0x0b040008,0x0b030009,0x03030003,
0x00000000,0x0e04000f,0x0e030016,0x1006000f,0x0e030019,0x10050010,0x0b03000a,0x0e060017,
0x00000000,0x01010001,0x0b040008,0x1004002d,0x0b020027,0x0b040009,0x0b03000b,0x10060010,
0x00000000,0x0b030015,0x0b040009,0x10050012,0x0b010066,0x10050011,0x0b03000c,0x0b07000a,
0x00000000,0x01010001,0x0e04000a,0x02020002,0x10020018,0x1006000f,0x10050010,0x0e040067,
0x00000000,0x0b020034,0x0e04000b,0x10060010,0x0e03001a,0x10050012,0x10050011,0x0e07000c,
0x00000000,0x01010001,0x1006000f,0x0e04004c,0x0b020028,0x10060010,0x10050012,0x10050013,
0x00000000,0x10030056,0x10060010,0x10050013,0x0b010067,0x10050013,0x10050013,0x10090011,
0x00000000,0x01010001,0x04010002,0x02020002,0x0e04000c,0x05020003,0x05010004,0x03030003,
0x0e0100ca,0x0e040010,0x04010003,0x10040028,0x0e03001b,0x10030029,0x05010005,0x04040004,
0x00000000,0x0f0200a6,0x05020003,0x0b04000d,0x0b020029,0x05020004,0x05010006,0x0b050008,
0x0f01014a,0x0b030016,0x05020004,0x0b040009,0x0b010068,0x1003002a,0x05010007,0x05050005,
0x00000000,0x01010001,0x0e050008,0x02020002,0x0e04000d,0x10040028,0x10030029,0x0b040010,
0x1001024a,0x0b020035,0x0e050009,0x10040029,0x0e03001c,0x1003002b,0x1003002a,0x0e08000a,
0x00000000,0x10020126,0x10040028,0x0b03001c,0x0b02002a,0x10040029,0x1003002b,0x0e07000b,
0x00000000,0x10030057,0x10040029,0x0b04000a,0x0b010069,0x1003002c,0x1003002c,0x1007002a,
0x00000000,0x01010001,0x0e020034,0x02020002,0x0e04000e,0x0e050009,0x0e04000a,0x03030003,
0x00000000,0x0e040011,0x0e020035,0x0b050008,0x0e03001d,0x0b040009,0x0e04000b,0x0e050036,
0x00000000,0x01010001,0x0e050009,0x1004002e,0x0b02002b,0x0e05000a,0x0e04000c,0x0b050009,
0x00000000,0x0b030017,0x0e05000a,0x0b04000b,0x0b01006a,0x0b04000a,0x0e04000d,0x0e08000b,
0x00000000,0x01010001,0x0e04000b,0x02020002,0x0e04000f,0x0b050008,0x0b040009,0x1005001a,
0x00000000,0x0b020036,0x0e04000c,0x0b050009,0x0e03001e,0x0b04000b,0x0b04000a,0x0e07000d,
0x00000000,0x01010001,0x0b050008,0x0e04004d,0x0b02002c,0x0b050009,0x0b04000b,0x0e07000c,
0x00000000,0x10030058,0x0b050009,0x0b04000c,0x0b01006b,0x0b04000c,0x0b04000c,0x0b08000a,
0x00000000,0x01010001,0x04010002,0x02020002,0x0e040010,0x0e03001c,0x0e02001d,0x03030003,
0x0e0100cb,0x0e040012,0x04010003,0x0b04000a,0x0e03001f,0x0b03000b,0x0e02001e,0x04040004,
0x00000000,0x01010001,0x0e03001c,0x0b04000e,0x0b02002d,0x0e03001d,0x0e02001f,0x0e08000a,
0x0f01014b,0x0b030018,0x0e03001d,0x0e07000b,0x0b01006c,0x0b03000c,0x0e020020,0x0e06001e,
0x00000000,0x01010001,0x0e050009,0x02020002,0x0e040011,0x0b04000a,0x0b03000b,0x0e050029,
0x1001024b,0x0b020037,0x0e05000a,0x0b04000b,0x0e030020,0x0b03000d,0x0b03000c,0x0e08000b,
0x00000000,0x01010001,0x0b04000a,0x0b03001d,0x0b02002e,0x0b04000b,0x0b03000d,0x0e07000d,
0x00000000,0x10030059,0x0b04000b,0x0e07000c,0x0b01006d,0x0b03000e,0x0b03000e,0x0b07000c,
0x00000000,0x01010001,0x0f020054,0x02020002,0x0e040012,0x0b040009,0x0b03000a,0x03030003,
0x00000000,0x0e040013,0x0f020055,0x0e08000a,0x0e030021,0x0e07000b,0x0b03000b,0x0f050056,
0x00000000,0x01010001,0x0b040009,0x1004002f,0x0b02002f,0x0b04000a,0x0b03000c,0x0e08000b,
0x00000000,0x0b030019,0x0b04000a,0x0e07000d,0x0b01006e,0x0e07000c,0x0b03000d,0x0b07000b,
0x00000000,0x01010001,0x0e04000c,0x02020002,0x0e040013,0x0e08000a,0x0e07000b,0x0e040068,
0x00000000,0x0b020038,0x0e04000d,0x0e08000b,0x0e030022,0x0e07000d,0x0e07000c,0x0e07000e,
0x00000000,0x01010001,0x0e08000a,0x0e04004e,0x0b020030,0x0e08000b,0x0e07000d,0x0e07000e,
0x00000000,0x1003005a,0x0e08000b,0x0e07000e,0x0b01006f,0x0e07000e,0x0e07000e,0x0e0b000c,
0x00000000,0x01010001,0x04010002,0x02020002,0x0e03000d,0x05020003,0x05010004,0x03030003,
0x0e0100cc,0x0b04000b,0x04010003,0x06030004,0x0b030012,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f0200a7,0x05020003,0x0b04000f,0x0b020031,0x05020004,0x05010006,0x10050017,
0x0f01014c,0x0b03001a,0x05020004,0x10040018,0x0b010070,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0e040009,0x10030095,0x0e03000e,0x06030004,0x06020005,0x0b040011,
0x1001024c,0x0b020039,0x0e04000a,0x06030005,0x0b030013,0x06020007,0x06020006,0x0e07000b,
0x00000000,0x10020127,0x06030004,0x0b03001e,0x0b020032,0x06030005,0x06020007,0x0e06000c,
0x00000000,0x1003005b,0x06030005,0x10040019,0x0b010071,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x1003000f,0x02020002,0x0e03000f,0x0b050007,0x0b040008,0x03030003,
0x00000000,0x0b04000c,0x10030010,0x10050017,0x0b030014,0x10040018,0x0b040009,0x10060011,
0x00000000,0x01010001,0x0b050007,0x10040030,0x0b020033,0x0b050008,0x0b04000a,0x10050018,
0x00000000,0x0b03001b,0x0b050008,0x1004001a,0x0b010072,0x10040019,0x0b04000b,0x0b080009,
0x00000000,0x01010001,0x0b040008,0x02020002,0x0e030010,0x10050017,0x10040018,0x1005001b,
0x00000000,0x0b02003a,0x0b040009,0x10050018,0x0b030015,0x1004001a,0x10040019,0x0b07000a,
0x00000000,0x01010001,0x10050017,0x0e04004f,0x0b020034,0x10050018,0x1004001a,0x0e06000d,
0x00000000,0x1003005c,0x10050018,0x1004001b,0x0b010073,0x1004001b,0x1004001b,0x10080019,
0x00000000,0x01010001,0x04010002,0x02020002,0x0e030011,0x0f03002c,0x0f02002d,0x03030003,
0x0e0100cd,0x0b04000d,0x04010003,0x0b050008,0x0b030016,0x0b040009,0x0f02002e,0x04040004,
0x00000000,0x01010001,0x0f03002c,0x0b040010,0x0b020035,0x0f03002d,0x0f02002f,0x0e07000b,
0x0f01014d,0x0b03001c,0x0f03002d,0x0e06000c,0x0b010074,0x0b04000a,0x0f020030,0x0f06002e,
0x00000000,0x01010001,0x0e04000a,0x02020002,0x0e030012,0x0b050008,0x0b040009,0x0e05002a,
0x1001024d,0x0b02003b,0x0e04000b,0x0b050009,0x0b030017,0x0b04000b,0x0b04000a,0x0e07000c,
0x00000000,0x01010001,0x0b050008,0x0b03001f,0x0b020036,0x0b050009,0x0b04000b,0x0e06000e,
0x00000000,0x1003005d,0x0b050009,0x0e06000d,0x0b010075,0x0b04000c,0x0b04000c,0x0b08000a,
0x00000000,0x01010001,0x0e030016,0x02020002,0x0e030013,0x0b04000a,0x0b03000b,0x03030003,
0x00000000,0x0b04000e,0x0e030017,0x0e07000b,0x0b030018,0x0e06000c,0x0b03000c,0x0e060018,
0x00000000,0x01010001,0x0b04000a,0x10040031,0x0b020037,0x0b04000b,0x0b03000d,0x0e07000c,
0x00000000,0x0b03001d,0x0b04000b,0x0e06000e,0x0b010076,0x0e06000d,0x0b03000e,0x0b07000c,
0x00000000,0x01010001,0x0b040009,0x02020002,0x0e030014,0x0e07000b,0x0e06000c,0x0e040069,
0x00000000,0x0b02003c,0x0b04000a,0x0e07000c,0x0b030019,0x0e06000e,0x0e06000d,0x0b07000b,
0x00000000,0x01010001,0x0e07000b,0x0e040050,0x0b020038,0x0e07000c,0x0e06000e,0x0e06000f,
0x00000000,0x1003005e,0x0e07000c,0x0e06000f,0x0b010077,0x0e06000f,0x0e06000f,0x0e0a000d,
0x00000000,0x01010001,0x04010002,0x02020002,0x0b04000b,0x05020003,0x05010004,0x03030003,
0x0e0100ce,0x0b04000f,0x04010003,0x06030004,0x0b03001a,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f0200a8,0x05020003,0x0b040011,0x0b020039,0x05020004,0x05010006,0x1006000c,
0x0f01014e,0x0b03001e,0x05020004,0x1005000d,0x0b010078,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0b050007,0x02020002,0x0b04000c,0x06030004,0x06020005,0x0b040012,
0x1001024e,0x0b02003d,0x0b050008,0x06030005,0x0b03001b,0x06020007,0x06020006,0x0b080009,
0x00000000,0x10020128,0x06030004,0x0b030020,0x0b02003a,0x06030005,0x06020007,0x0b07000a,
0x00000000,0x1003005f,0x06030005,0x1005000e,0x0b010079,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0e020035,0x02020002,0x0b04000d,0x0b050008,0x0b040009,0x03030003,
0x00000000,0x0b040010,0x0e020036,0x1006000c,0x0b03001c,0x1005000d,0x0b04000a,0x0e050037,
0x00000000,0x01010001,0x0b050008,0x10040032,0x0b02003b,0x0b050009,0x0b04000b,0x1006000d,
0x00000000,0x0b03001f,0x0b050009,0x1005000f,0x0b01007a,0x1005000e,0x0b04000c,0x0b08000a,
0x00000000,0x01010001,0x0b04000a,0x02020002,0x0b04000e,0x1006000c,0x1005000d,0x1005001c,
0x00000000,0x0b02003e,0x0b04000b,0x1006000d,0x0b03001d,0x1005000f,0x1005000e,0x0b07000c,
0x00000000,0x01010001,0x1006000c,0x0e040051,0x0b02003c,0x1006000d,0x1005000f,0x0b07000b,
0x00000000,0x10030060,0x1006000d,0x10050010,0x0b01007b,0x10050010,0x10050010,0x1009000e,
0x00000000,0x01010001,0x04010002,0x02020002,0x0b04000f,0x1003004c,0x1002004d,0x03030003,
0x0e0100cf,0x0b040011,0x04010003,0x0b04000b,0x0b03001e,0x0b03000c,0x1002004e,0x04040004,
0x00000000,0x01010001,0x1003004c,0x0b040012,0x0b02003d,0x1003004d,0x1002004f,0x0b080009,
0x0f01014f,0x0b030020,0x1003004d,0x0b07000a,0x0b01007c,0x0b03000d,0x10020050,0x1006004e,
0x00000000,0x01010001,0x0b050008,0x02020002,0x0b040010,0x0b04000b,0x0b03000c,0x0e05002b,
0x1001024f,0x0b02003f,0x0b050009,0x0b04000c,0x0b03001f,0x0b03000e,0x0b03000d,0x0b08000a,
0x00000000,0x01010001,0x0b04000b,0x0b030021,0x0b02003e,0x0b04000c,0x0b03000e,0x0b07000c,
0x00000000,0x10030061,0x0b04000c,0x0b07000b,0x0b01007d,0x0b03000f,0x0b03000f,0x0b07000d,
0x00000000,0x01010001,0x0f020055,0x02020002,0x0b040011,0x0b04000b,0x0b03000c,0x03030003,
0x00000000,0x0b040012,0x0f020056,0x0b080009,0x0b030020,0x0b07000a,0x0b03000d,0x0f050057,
0x00000000,0x01010001,0x0b04000b,0x10040033,0x0b02003f,0x0b04000c,0x0b03000e,0x0b08000a,
0x00000000,0x0b030021,0x0b04000c,0x0b07000c,0x0b01007e,0x0b07000b,0x0b03000f,0x0b07000d,
0x00000000,0x01010001,0x0b04000b,0x02020002,0x0b040012,0x0b080009,0x0b07000a,0x0e04006a,
0x00000000,0x0b020040,0x0b04000c,0x0b08000a,0x0b030021,0x0b07000c,0x0b07000b,0x0b07000d,
0x00000000,0x01010001,0x0b080009,0x0e040052,0x0b020040,0x0b08000a,0x0b07000c,0x0b07000d,
0x00000000,0x10030062,0x0b08000a,0x0b07000d,0x0b01007f,0x0b07000d,0x0b07000d,0x0b0b000b,
0x00000000,0x01010001,0x04010002,0x02020002,0x08010008,0x05020003,0x05010004,0x03030003,
0x0e0100d0,0x09020009,0x04010003,0x06030004,0x09010010,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f0200a9,0x05020003,0x0e04000d,0x0e020023,0x05020004,0x05010006,0x07040005,
0x0f010150,0x0e030014,0x05020004,0x07030006,0x10030043,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x08020004,0x10030096,0x08010009,0x06030004,0x06020005,0x1006000e,
0x10010250,0x10040024,0x08020005,0x06030005,0x09010011,0x06020007,0x06020006,0x08050006,
0x00000000,0x10020129,0x06030004,0x10050015,0x0e020024,0x06030005,0x06020007,0x08040007,
0x00000000,0x10030063,0x06030005,0x07030007,0x10030044,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x10030010,0x02020002,0x0801000a,0x09030005,0x09020006,0x03030003,
0x00000000,0x0902000a,0x10030011,0x07040005,0x09010012,0x07030006,0x09020007,0x10060012,
0x00000000,0x01010001,0x09030005,0x10040034,0x0e020025,0x09030006,0x09020008,0x07040006,
0x00000000,0x0e030015,0x09030006,0x07030008,0x10030045,0x07030007,0x09020009,0x09060007,
0x00000000,0x01010001,0x09020006,0x02020002,0x0801000b,0x07040005,0x07030006,0x1005001d,
0x00000000,0x10040025,0x09020007,0x07040006,0x09010013,0x07030008,0x07030007,0x09050008,
0x00000000,0x01010001,0x07040005,0x0e040053,0x0e020026,0x07040006,0x07030008,0x08040008,
0x00000000,0x10030064,0x07040006,0x07030009,0x10030046,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0801000c,0x0e04000e,0x0e03000f,0x03030003,
0x0e0100d1,0x0902000b,0x04010003,0x0e050009,0x09010014,0x0e04000a,0x0e030010,0x04040004,
0x00000000,0x01010001,0x0e04000e,0x0e04000e,0x0e020027,0x0e04000f,0x0e030011,0x08050006,
0x0f010151,0x0e030016,0x0e04000f,0x08040007,0x10030047,0x0e04000b,0x0e030012,0x0e070010,
0x00000000,0x01010001,0x08020005,0x02020002,0x0801000d,0x0e050009,0x0e04000a,0x0e05002c,
0x10010251,0x10040026,0x08020006,0x0e05000a,0x09010015,0x0e04000c,0x0e04000b,0x08050007,
0x00000000,0x01010001,0x0e050009,0x10050016,0x0e020028,0x0e05000a,0x0e04000c,0x08040009,
0x00000000,0x10030065,0x0e05000a,0x08040008,0x10030048,0x0e04000d,0x0e04000d,0x0e08000b,
0x00000000,0x01010001,0x0e030017,0x02020002,0x0801000e,0x0e04000a,0x0e03000b,0x03030003,
0x00000000,0x0902000c,0x0e030018,0x08050006,0x09010016,0x08040007,0x0e03000c,0x0e060019,
0x00000000,0x01010001,0x0e04000a,0x10040035,0x0e020029,0x0e04000b,0x0e03000d,0x08050007,
0x00000000,0x0e030017,0x0e04000b,0x08040009,0x10030049,0x08040008,0x0e03000e,0x0e07000c,
0x00000000,0x01010001,0x09020007,0x02020002,0x0801000f,0x08050006,0x08040007,0x0e04006b,
0x00000000,0x10040027,0x09020008,0x08050007,0x09010017,0x08040009,0x08040008,0x09050009,
0x00000000,0x01010001,0x08050006,0x0e040054,0x0e02002a,0x08050007,0x08040009,0x0804000a,
0x00000000,0x10030066,0x08050007,0x0804000a,0x1003004a,0x0804000a,0x0804000a,0x08080008,
0x00000000,0x01010001,0x04010002,0x02020002,0x09020009,0x05020003,0x05010004,0x03030003,
0x0e0100d2,0x0902000d,0x04010003,0x0e040011,0x09010018,0x0e030012,0x05010005,0x04040004,
0x00000000,0x0f0200aa,0x05020003,0x0e04000f,0x0e02002b,0x05020004,0x05010006,0x1007000a,
0x0f010152,0x0e030018,0x05020004,0x1006000b,0x1003004b,0x0e030013,0x05010007,0x05050005,
0x00000000,0x01010001,0x09030005,0x02020002,0x0902000a,0x0e040011,0x0e030012,0x1006000f,
0x10010252,0x10040028,0x09030006,0x0e040012,0x09010019,0x0e030014,0x0e030013,0x09060007,
0x00000000,0x1002012a,0x0e040011,0x10050017,0x0e02002c,0x0e040012,0x0e030014,0x09050008,
0x00000000,0x10030067,0x0e040012,0x1006000c,0x1003004c,0x0e030015,0x0e030015,0x0e070013,
0x00000000,0x01010001,0x0e020036,0x02020002,0x0902000b,0x09030006,0x09020007,0x03030003,
0x00000000,0x0902000e,0x0e020037,0x1007000a,0x0901001a,0x1006000b,0x09020008,0x0e050038,
0x00000000,0x01010001,0x09030006,0x10040036,0x0e02002d,0x09030007,0x09020009,0x1007000b,
0x00000000,0x0e030019,0x09030007,0x1006000d,0x1003004d,0x1006000c,0x0902000a,0x09060008,
0x00000000,0x01010001,0x09020008,0x02020002,0x0902000c,0x1007000a,0x1006000b,0x1005001e,
0x00000000,0x10040029,0x09020009,0x1007000b,0x0901001b,0x1006000d,0x1006000c,0x0905000a,
0x00000000,0x01010001,0x1007000a,0x0e040055,0x0e02002e,0x1007000b,0x1006000d,0x09050009,
0x00000000,0x10030068,0x1007000b,0x1006000e,0x1003004e,0x1006000e,0x1006000e,0x100a000c,
0x00000000,0x01010001,0x04010002,0x02020002,0x0902000d,0x0e03001d,0x0e02001e,0x03030003,
0x0e0100d3,0x0902000f,0x04010003,0x1006000b,0x0901001c,0x1005000c,0x0e02001f,0x04040004,
0x00000000,0x01010001,0x0e03001d,0x0e040010,0x0e02002f,0x0e03001e,0x0e020020,0x09060007,
0x0f010153,0x0e03001a,0x0e03001e,0x09050008,0x1003004f,0x1005000d,0x0e020021,0x0e06001f,
0x00000000,0x01010001,0x09030006,0x02020002,0x0902000e,0x1006000b,0x1005000c,0x0e05002d,
0x10010253,0x1004002a,0x09030007,0x1006000c,0x0901001d,0x1005000e,0x1005000d,0x09060008,
0x00000000,0x01010001,0x1006000b,0x10050018,0x0e020030,0x1006000c,0x1005000e,0x0905000a,
0x00000000,0x10030069,0x1006000c,0x09050009,0x10030050,0x1005000f,0x1005000f,0x1009000d,
0x00000000,0x01010001,0x0f020056,0x02020002,0x0902000f,0x0e04000b,0x0e03000c,0x03030003,
0x00000000,0x09020010,0x0f020057,0x09060007,0x0901001e,0x09050008,0x0e03000d,0x0f050058,
0x00000000,0x01010001,0x0e04000b,0x10040037,0x0e020031,0x0e04000c,0x0e03000e,0x09060008,
0x00000000,0x0e03001b,0x0e04000c,0x0905000a,0x10030051,0x09050009,0x0e03000f,0x0e07000d,
0x00000000,0x01010001,0x09020009,0x02020002,0x09020010,0x09060007,0x09050008,0x0e04006c,
0x00000000,0x1004002b,0x0902000a,0x09060008,0x0901001f,0x0905000a,0x09050009,0x0905000b,
0x00000000,0x01010001,0x09060007,0x0e040056,0x0e020032,0x09060008,0x0905000a,0x0905000b,
0x00000000,0x1003006a,0x09060008,0x0905000b,0x10030052,0x0905000b,0x0905000b,0x09090009,
0x00000000,0x01010001,0x04010002,0x02020002,0x1003000d,0x05020003,0x05010004,0x03030003,
0x0e0100d4,0x0e04000d,0x04010003,0x06030004,0x0e030014,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f0200ab,0x05020003,0x0e040011,0x0e020033,0x05020004,0x05010006,0x07040005,
0x0f010154,0x0e03001c,0x05020004,0x07030006,0x10030053,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x10040009,0x10030097,0x1003000e,0x06030004,0x06020005,0x10060010,
0x10010254,0x1004002c,0x1004000a,0x06030005,0x0e030015,0x06020007,0x06020006,0x1007000b,
0x00000000,0x1002012b,0x06030004,0x10050019,0x0e020034,0x06030005,0x06020007,0x1006000c,
0x00000000,0x1003006b,0x06030005,0x07030007,0x10030054,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x10030011,0x02020002,0x1003000f,0x0e050009,0x0e04000a,0x03030003,
0x00000000,0x0e04000e,0x10030012,0x07040005,0x0e030016,0x07030006,0x0e04000b,0x10060013,
0x00000000,0x01010001,0x0e050009,0x10040038,0x0e020035,0x0e05000a,0x0e04000c,0x07040006,
0x00000000,0x0e03001d,0x0e05000a,0x07030008,0x10030055,0x07030007,0x0e04000d,0x0e08000b,
0x00000000,0x01010001,0x0e04000a,0x02020002,0x10030010,0x07040005,0x07030006,0x1005001f,
0x00000000,0x1004002d,0x0e04000b,0x07040006,0x0e030017,0x07030008,0x07030007,0x0e07000c,
0x00000000,0x01010001,0x07040005,0x0e040057,0x0e020036,0x07040006,0x07030008,0x1006000d,
0x00000000,0x1003006c,0x07040006,0x07030009,0x10030056,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x10030011,0x0f03002d,0x0f02002e,0x03030003,
0x0e0100d5,0x0e04000f,0x04010003,0x0e05000a,0x0e030018,0x0e04000b,0x0f02002f,0x04040004,
0x00000000,0x01010001,0x0f03002d,0x0e040012,0x0e020037,0x0f03002e,0x0f020030,0x1007000b,
0x0f010155,0x0e03001e,0x0f03002e,0x1006000c,0x10030057,0x0e04000c,0x0f020031,0x0f06002f,
0x00000000,0x01010001,0x1004000a,0x02020002,0x10030012,0x0e05000a,0x0e04000b,0x0e05002e,
0x10010255,0x1004002e,0x1004000b,0x0e05000b,0x0e030019,0x0e04000d,0x0e04000c,0x1007000c,
0x00000000,0x01010001,0x0e05000a,0x1005001a,0x0e020038,0x0e05000b,0x0e04000d,0x1006000e,
0x00000000,0x1003006d,0x0e05000b,0x1006000d,0x10030058,0x0e04000e,0x0e04000e,0x0e08000c,
0x00000000,0x01010001,0x0e030018,0x02020002,0x10030013,0x0e04000c,0x0e03000d,0x03030003,
0x00000000,0x0e040010,0x0e030019,0x1007000b,0x0e03001a,0x1006000c,0x0e03000e,0x0e06001a,
0x00000000,0x01010001,0x0e04000c,0x10040039,0x0e020039,0x0e04000d,0x0e03000f,0x1007000c,
0x00000000,0x0e03001f,0x0e04000d,0x1006000e,0x10030059,0x1006000d,0x0e030010,0x0e07000e,
0x00000000,0x01010001,0x0e04000b,0x02020002,0x10030014,0x1007000b,0x1006000c,0x0e04006d,
0x00000000,0x1004002f,0x0e04000c,0x1007000c,0x0e03001b,0x1006000e,0x1006000d,0x0e07000d,
0x00000000,0x01010001,0x1007000b,0x0e040058,0x0e02003a,0x1007000c,0x1006000e,0x1006000f,
0x00000000,0x1003006e,0x1007000c,0x1006000f,0x1003005a,0x1006000f,0x1006000f,0x100a000d,
0x00000000,0x01010001,0x04010002,0x02020002,0x0e04000d,0x05020003,0x05010004,0x03030003,
0x0e0100d6,0x0e040011,0x04010003,0x0f040019,0x0e03001c,0x0f03001a,0x05010005,0x04040004,
0x00000000,0x0f0200ac,0x05020003,0x0e040013,0x0e02003b,0x05020004,0x05010006,0x1006000d,
0x0f010156,0x0e030020,0x05020004,0x1005000e,0x1003005b,0x0f03001b,0x05010007,0x05050005,
0x00000000,0x01010001,0x0e050009,0x02020002,0x0e04000e,0x0f040019,0x0f03001a,0x10060011,
0x10010256,0x10040030,0x0e05000a,0x0f04001a,0x0e03001d,0x0f03001c,0x0f03001b,0x0e08000b,
0x00000000,0x1002012c,0x0f040019,0x1005001b,0x0e02003c,0x0f04001a,0x0f03001c,0x0e07000c,
0x00000000,0x1003006f,0x0f04001a,0x1005000f,0x1003005c,0x0f03001d,0x0f03001d,0x0f07001b,
0x00000000,0x01010001,0x0e020037,0x02020002,0x0e04000f,0x0e05000a,0x0e04000b,0x03030003,
0x00000000,0x0e040012,0x0e020038,0x1006000d,0x0e03001e,0x1005000e,0x0e04000c,0x0e050039,
0x00000000,0x01010001,0x0e05000a,0x1004003a,0x0e02003d,0x0e05000b,0x0e04000d,0x1006000e,
0x00000000,0x0e030021,0x0e05000b,0x10050010,0x1003005d,0x1005000f,0x0e04000e,0x0e08000c,
0x00000000,0x01010001,0x0e04000c,0x02020002,0x0e040010,0x1006000d,0x1005000e,0x10050020,
0x00000000,0x10040031,0x0e04000d,0x1006000e,0x0e03001f,0x10050010,0x1005000f,0x0e07000e,
0x00000000,0x01010001,0x1006000d,0x0e040059,0x0e02003e,0x1006000e,0x10050010,0x0e07000d,
0x00000000,0x10030070,0x1006000e,0x10050011,0x1003005e,0x10050011,0x10050011,0x1009000f,
0x00000000,0x01010001,0x04010002,0x02020002,0x0e040011,0x1003004d,0x1002004e,0x03030003,
0x0e0100d7,0x0e040013,0x04010003,0x1006000c,0x0e030020,0x1005000d,0x1002004f,0x04040004,
0x00000000,0x01010001,0x1003004d,0x0e040014,0x0e02003f,0x1003004e,0x10020050,0x0e08000b,
0x0f010157,0x0e030022,0x1003004e,0x0e07000c,0x1003005f,0x1005000e,0x10020051,0x1006004f,
0x00000000,0x01010001,0x0e05000a,0x02020002,0x0e040012,0x1006000c,0x1005000d,0x0e05002f,
0x10010257,0x10040032,0x0e05000b,0x1006000d,0x0e030021,0x1005000f,0x1005000e,0x0e08000c,
0x00000000,0x01010001,0x1006000c,0x1005001c,0x0e020040,0x1006000d,0x1005000f,0x0e07000e,
0x00000000,0x10030071,0x1006000d,0x0e07000d,0x10030060,0x10050010,0x10050010,0x1009000e,
0x00000000,0x01010001,0x0f020057,0x02020002,0x0e040013,0x0e04000d,0x0e03000e,0x03030003,
0x00000000,0x0e040014,0x0f020058,0x0e08000b,0x0e030022,0x0e07000c,0x0e03000f,0x0f050059,
0x00000000,0x01010001,0x0e04000d,0x1004003b,0x0e020041,0x0e04000e,0x0e030010,0x0e08000c,
0x00000000,0x0e030023,0x0e04000e,0x0e07000e,0x10030061,0x0e07000d,0x0e030011,0x0e07000f,
0x00000000,0x01010001,0x0e04000d,0x02020002,0x0e040014,0x0e08000b,0x0e07000c,0x0e04006e,
0x00000000,0x10040033,0x0e04000e,0x0e08000c,0x0e030023,0x0e07000e,0x0e07000d,0x0e07000f,
0x00000000,0x01010001,0x0e08000b,0x0e04005a,0x0e020042,0x0e08000c,0x0e07000e,0x0e07000f,
0x00000000,0x10030072,0x0e08000c,0x0e07000f,0x10030062,0x0e07000f,0x0e07000f,0x0e0b000d,
0x00000000,0x01010001,0x04010002,0x02020002,0x08010008,0x05020003,0x05010004,0x03030003,
0x0e0100d8,0x0e03000f,0x04010003,0x06030004,0x0e020016,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f0200ad,0x05020003,0x1006000e,0x10040024,0x05020004,0x05010006,0x07040005,
0x0f010158,0x10050015,0x05020004,0x07030006,0x10030063,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x08020004,0x10030098,0x08010009,0x06030004,0x06020005,0x10060012,
0x10010258,0x10040034,0x08020005,0x06030005,0x0e020017,0x06020007,0x06020006,0x08050006,
0x00000000,0x1002012d,0x06030004,0x1005001d,0x10040025,0x06030005,0x06020007,0x08040007,
0x00000000,0x10030073,0x06030005,0x07030007,0x10030064,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x10030012,0x02020002,0x0801000a,0x0e04000b,0x0e03000c,0x03030003,
0x00000000,0x0e030010,0x10030013,0x07040005,0x0e020018,0x07030006,0x0e03000d,0x10060014,
0x00000000,0x01010001,0x0e04000b,0x1004003c,0x10040026,0x0e04000c,0x0e03000e,0x07040006,
0x00000000,0x10050016,0x0e04000c,0x07030008,0x10030065,0x07030007,0x0e03000f,0x0e07000d,
0x00000000,0x01010001,0x0e03000c,0x02020002,0x0801000b,0x07040005,0x07030006,0x10050021,
0x00000000,0x10040035,0x0e03000d,0x07040006,0x0e020019,0x07030008,0x07030007,0x0e06000e,
0x00000000,0x01010001,0x07040005,0x0e04005b,0x10040027,0x07040006,0x07030008,0x08040008,
0x00000000,0x10030074,0x07040006,0x07030009,0x10030066,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0801000c,0x0e04000f,0x0e030010,0x03030003,
0x0e0100d9,0x0e030011,0x04010003,0x1007000a,0x0e02001a,0x1006000b,0x0e030011,0x04040004,
0x00000000,0x01010001,0x0e04000f,0x1006000f,0x10040028,0x0e040010,0x0e030012,0x08050006,
0x0f010159,0x10050017,0x0e040010,0x08040007,0x10030067,0x1006000c,0x0e030013,0x0e070011,
0x00000000,0x01010001,0x08020005,0x02020002,0x0801000d,0x1007000a,0x1006000b,0x0e050030,
0x10010259,0x10040036,0x08020006,0x1007000b,0x0e02001b,0x1006000d,0x1006000c,0x08050007,
0x00000000,0x01010001,0x1007000a,0x1005001e,0x10040029,0x1007000b,0x1006000d,0x08040009,
0x00000000,0x10030075,0x1007000b,0x08040008,0x10030068,0x1006000e,0x1006000e,0x100a000c,
0x00000000,0x01010001,0x0e030019,0x02020002,0x0801000e,0x1006000b,0x1005000c,0x03030003,
0x00000000,0x0e030012,0x0e03001a,0x08050006,0x0e02001c,0x08040007,0x1005000d,0x0e06001b,
0x00000000,0x01010001,0x1006000b,0x1004003d,0x1004002a,0x1006000c,0x1005000e,0x08050007,
0x00000000,0x10050018,0x1006000c,0x08040009,0x10030069,0x08040008,0x1005000f,0x1009000d,
0x00000000,0x01010001,0x0e03000d,0x02020002,0x0801000f,0x08050006,0x08040007,0x0e04006f,
0x00000000,0x10040037,0x0e03000e,0x08050007,0x0e02001d,0x08040009,0x08040008,0x0e06000f,
0x00000000,0x01010001,0x08050006,0x0e04005c,0x1004002b,0x08050007,0x08040009,0x0804000a,
0x00000000,0x10030076,0x08050007,0x0804000a,0x1003006a,0x0804000a,0x0804000a,0x08080008,
0x00000000,0x01010001,0x04010002,0x02020002,0x0e03000f,0x05020003,0x05010004,0x03030003,
0x0e0100da,0x0e030013,0x04010003,0x10040029,0x0e02001e,0x1003002a,0x05010005,0x04040004,
0x00000000,0x0f0200ae,0x05020003,0x10060010,0x1004002c,0x05020004,0x05010006,0x1007000b,
0x0f01015a,0x10050019,0x05020004,0x1006000c,0x1003006b,0x1003002b,0x05010007,0x05050005,
0x00000000,0x01010001,0x0e04000b,0x02020002,0x0e030010,0x10040029,0x1003002a,0x10060013,
0x1001025a,0x10040038,0x0e04000c,0x1004002a,0x0e02001f,0x1003002c,0x1003002b,0x0e07000d,
0x00000000,0x1002012e,0x10040029,0x1005001f,0x1004002d,0x1004002a,0x1003002c,0x0e06000e,
0x00000000,0x10030077,0x1004002a,0x1006000d,0x1003006c,0x1003002d,0x1003002d,0x1007002b,
0x00000000,0x01010001,0x0e020038,0x02020002,0x0e030011,0x0e04000c,0x0e03000d,0x03030003,
0x00000000,0x0e030014,0x0e020039,0x1007000b,0x0e020020,0x1006000c,0x0e03000e,0x0e05003a,
0x00000000,0x01010001,0x0e04000c,0x1004003e,0x1004002e,0x0e04000d,0x0e03000f,0x1007000c,
0x00000000,0x1005001a,0x0e04000d,0x1006000e,0x1003006d,0x1006000d,0x0e030010,0x0e07000e,
0x00000000,0x01010001,0x0e03000e,0x02020002,0x0e030012,0x1007000b,0x1006000c,0x10050022,
0x00000000,0x10040039,0x0e03000f,0x1007000c,0x0e020021,0x1006000e,0x1006000d,0x0e060010,
0x00000000,0x01010001,0x1007000b,0x0e04005d,0x1004002f,0x1007000c,0x1006000e,0x0e06000f,
0x00000000,0x10030078,0x1007000c,0x1006000f,0x1003006e,0x1006000f,0x1006000f,0x100a000d,
0x00000000,0x01010001,0x04010002,0x02020002,0x0e030013,0x0e03001e,0x0e02001f,0x03030003,
0x0e0100db,0x0e030015,0x04010003,0x1006000d,0x0e020022,0x1005000e,0x0e020020,0x04040004,
0x00000000,0x01010001,0x0e03001e,0x10060011,0x10040030,0x0e03001f,0x0e020021,0x0e07000d,
0x0f01015b,0x1005001b,0x0e03001f,0x0e06000e,0x1003006f,0x1005000f,0x0e020022,0x0e060020,
0x00000000,0x01010001,0x0e04000c,0x02020002,0x0e030014,0x1006000d,0x1005000e,0x0e050031,
0x1001025b,0x1004003a,0x0e04000d,0x1006000e,0x0e020023,0x10050010,0x1005000f,0x0e07000e,
0x00000000,0x01010001,0x1006000d,0x10050020,0x10040031,0x1006000e,0x10050010,0x0e060010,
0x00000000,0x10030079,0x1006000e,0x0e06000f,0x10030070,0x10050011,0x10050011,0x1009000f,
0x00000000,0x01010001,0x0f020058,0x02020002,0x0e030015,0x1006000c,0x1005000d,0x03030003,
0x00000000,0x0e030016,0x0f020059,0x0e07000d,0x0e020024,0x0e06000e,0x1005000e,0x0f05005a,
0x00000000,0x01010001,0x1006000c,0x1004003f,0x10040032,0x1006000d,0x1005000f,0x0e07000e,
0x00000000,0x1005001c,0x1006000d,0x0e060010,0x10030071,0x0e06000f,0x10050010,0x1009000e,
0x00000000,0x01010001,0x0e03000f,0x02020002,0x0e030016,0x0e07000d,0x0e06000e,0x0e040070,
0x00000000,0x1004003b,0x0e030010,0x0e07000e,0x0e020025,0x0e060010,0x0e06000f,0x0e060011,
0x00000000,0x01010001,0x0e07000d,0x0e04005e,0x10040033,0x0e07000e,0x0e060010,0x0e060011,
0x00000000,0x1003007a,0x0e07000e,0x0e060011,0x10030072,0x0e060011,0x0e060011,0x0e0a000f,
0x00000000,0x01010001,0x04010002,0x02020002,0x0e03000e,0x05020003,0x05010004,0x03030003,
0x0e0100dc,0x1006000e,0x04010003,0x06030004,0x10050015,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f0200af,0x05020003,0x10060012,0x10040034,0x05020004,0x05010006,0x07040005,
0x0f01015c,0x1005001d,0x05020004,0x07030006,0x10030073,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0e04000a,0x10030099,0x0e03000f,0x06030004,0x06020005,0x10060014,
0x1001025c,0x1004003c,0x0e04000b,0x06030005,0x10050016,0x06020007,0x06020006,0x0e07000c,
0x00000000,0x1002012f,0x06030004,0x10050021,0x10040035,0x06030005,0x06020007,0x0e06000d,
0x00000000,0x1003007b,0x06030005,0x07030007,0x10030074,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x10030013,0x02020002,0x0e030010,0x1007000a,0x1006000b,0x03030003,
0x00000000,0x1006000f,0x10030014,0x07040005,0x10050017,0x07030006,0x1006000c,0x10060015,
0x00000000,0x01010001,0x1007000a,0x10040040,0x10040036,0x1007000b,0x1006000d,0x07040006,
0x00000000,0x1005001e,0x1007000b,0x07030008,0x10030075,0x07030007,0x1006000e,0x100a000c,
0x00000000,0x01010001,0x1006000b,0x02020002,0x0e030011,0x07040005,0x07030006,0x10050023,
0x00000000,0x1004003d,0x1006000c,0x07040006,0x10050018,0x07030008,0x07030007,0x1009000d,
0x00000000,0x01010001,0x07040005,0x0e04005f,0x10040037,0x07040006,0x07030008,0x0e06000e,
0x00000000,0x1003007c,0x07040006,0x07030009,0x10030076,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0e030012,0x0f03002e,0x0f02002f,0x03030003,
0x0e0100dd,0x10060010,0x04010003,0x1007000b,0x10050019,0x1006000c,0x0f020030,0x04040004,
0x00000000,0x01010001,0x0f03002e,0x10060013,0x10040038,0x0f03002f,0x0f020031,0x0e07000c,
0x0f01015d,0x1005001f,0x0f03002f,0x0e06000d,0x10030077,0x1006000d,0x0f020032,0x0f060030,
0x00000000,0x01010001,0x0e04000b,0x02020002,0x0e030013,0x1007000b,0x1006000c,0x0e050032,
0x1001025d,0x1004003e,0x0e04000c,0x1007000c,0x1005001a,0x1006000e,0x1006000d,0x0e07000d,
0x00000000,0x01010001,0x1007000b,0x10050022,0x10040039,0x1007000c,0x1006000e,0x0e06000f,
0x00000000,0x1003007d,0x1007000c,0x0e06000e,0x10030078,0x1006000f,0x1006000f,0x100a000d,
0x00000000,0x01010001,0x0e03001a,0x02020002,0x0e030014,0x1006000d,0x1005000e,0x03030003,
0x00000000,0x10060011,0x0e03001b,0x0e07000c,0x1005001b,0x0e06000d,0x1005000f,0x0e06001c,
0x00000000,0x01010001,0x1006000d,0x10040041,0x1004003a,0x1006000e,0x10050010,0x0e07000d,
0x00000000,0x10050020,0x1006000e,0x0e06000f,0x10030079,0x0e06000e,0x10050011,0x1009000f,
0x00000000,0x01010001,0x1006000c,0x02020002,0x0e030015,0x0e07000c,0x0e06000d,0x0e040071,
0x00000000,0x1004003f,0x1006000d,0x0e07000d,0x1005001c,0x0e06000f,0x0e06000e,0x1009000e,
0x00000000,0x01010001,0x0e07000c,0x0e040060,0x1004003b,0x0e07000d,0x0e06000f,0x0e060010,
0x00000000,0x1003007e,0x0e07000d,0x0e060010,0x1003007a,0x0e060010,0x0e060010,0x0e0a000e,
0x00000000,0x01010001,0x04010002,0x02020002,0x1006000e,0x05020003,0x05010004,0x03030003,
0x0e0100de,0x10060012,0x04010003,0x06030004,0x1005001d,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f0200b0,0x05020003,0x10060014,0x1004003c,0x05020004,0x05010006,0x1006000e,
0x0f01015e,0x10050021,0x05020004,0x1005000f,0x1003007b,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x1007000a,0x02020002,0x1006000f,0x06030004,0x06020005,0x10060015,
0x1001025e,0x10040040,0x1007000b,0x06030005,0x1005001e,0x06020007,0x06020006,0x100a000c,
0x00000000,0x10020130,0x06030004,0x10050023,0x1004003d,0x06030005,0x06020007,0x1009000d,
0x00000000,0x1003007f,0x06030005,0x10050010,0x1003007c,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0e020039,0x02020002,0x10060010,0x1007000b,0x1006000c,0x03030003,
0x00000000,0x10060013,0x0e02003a,0x1006000e,0x1005001f,0x1005000f,0x1006000d,0x0e05003b,
0x00000000,0x01010001,0x1007000b,0x10040042,0x1004003e,0x1007000c,0x1006000e,0x1006000f,
0x00000000,0x10050022,0x1007000c,0x10050011,0x1003007d,0x10050010,0x1006000f,0x100a000d,
0x00000000,0x01010001,0x1006000d,0x02020002,0x10060011,0x1006000e,0x1005000f,0x10050024,
0x00000000,0x10040041,0x1006000e,0x1006000f,0x10050020,0x10050011,0x10050010,0x1009000f,
0x00000000,0x01010001,0x1006000e,0x0e040061,0x1004003f,0x1006000f,0x10050011,0x1009000e,
0x00000000,0x10030080,0x1006000f,0x10050012,0x1003007e,0x10050012,0x10050012,0x10090010,
0x00000000,0x01010001,0x04010002,0x02020002,0x10060012,0x1003004e,0x1002004f,0x03030003,
0x0e0100df,0x10060014,0x04010003,0x1006000e,0x10050021,0x1005000f,0x10020050,0x04040004,
0x00000000,0x01010001,0x1003004e,0x10060015,0x10040040,0x1003004f,0x10020051,0x100a000c,
0x0f01015f,0x10050023,0x1003004f,0x1009000d,0x1003007f,0x10050010,0x10020052,0x10060050,
0x00000000,0x01010001,0x1007000b,0x02020002,0x10060013,0x1006000e,0x1005000f,0x0e050033,
0x1001025f,0x10040042,0x1007000c,0x1006000f,0x10050022,0x10050011,0x10050010,0x100a000d,
0x00000000,0x01010001,0x1006000e,0x10050024,0x10040041,0x1006000f,0x10050011,0x1009000f,
0x00000000,0x10030081,0x1006000f,0x1009000e,0x10030080,0x10050012,0x10050012,0x10090010,
0x00000000,0x01010001,0x0f020059,0x02020002,0x10060014,0x1006000e,0x1005000f,0x03030003,
0x00000000,0x10060015,0x0f02005a,0x100a000c,0x10050023,0x1009000d,0x10050010,0x0f05005b,
0x00000000,0x01010001,0x1006000e,0x10040043,0x10040042,0x1006000f,0x10050011,0x100a000d,
0x00000000,0x10050024,0x1006000f,0x1009000f,0x10030081,0x1009000e,0x10050012,0x10090010,
0x00000000,0x01010001,0x1006000e,0x02020002,0x10060015,0x100a000c,0x1009000d,0x0e040072,
0x00000000,0x10040043,0x1006000f,0x100a000d,0x10050024,0x1009000f,0x1009000e,0x10090010,
0x00000000,0x01010001,0x100a000c,0x0e040062,0x10040043,0x100a000d,0x1009000f,0x10090010,
0x00000000,0x10030082,0x100a000d,0x10090010,0x10030082,0x10090010,0x10090010,0x100d000e,
0x00000000,0x01010001,0x04010002,0x02020002,0x08010008,0x05020003,0x05010004,0x03030003,
0x0e0100e0,0x09020009,0x04010003,0x06030004,0x09010010,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f0200b1,0x05020003,0x0a03000a,0x0a010020,0x05020004,0x05010006,0x07040005,
0x0f010160,0x0a020011,0x05020004,0x07030006,0x10020044,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x08020004,0x1003009a,0x08010009,0x06030004,0x06020005,0x1005000f,
0x10010260,0x10030025,0x08020005,0x06030005,0x09010011,0x06020007,0x06020006,0x08050006,
0x00000000,0x10020131,0x06030004,0x10040016,0x0a010021,0x06030005,0x06020007,0x08040007,
0x00000000,0x0e040043,0x06030005,0x07030007,0x10020045,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0e04000c,0x02020002,0x0801000a,0x09030005,0x09020006,0x03030003,
0x00000000,0x0902000a,0x0e04000d,0x07040005,0x09010012,0x07030006,0x09020007,0x0e07000e,
0x00000000,0x01010001,0x09030005,0x0e050024,0x0a010022,0x09030006,0x09020008,0x07040006,
0x00000000,0x0a020012,0x09030006,0x07030008,0x10020046,0x07030007,0x09020009,0x09060007,
0x00000000,0x01010001,0x09020006,0x02020002,0x0801000b,0x07040005,0x07030006,0x0e060015,
0x00000000,0x10030026,0x09020007,0x07040006,0x09010013,0x07030008,0x07030007,0x09050008,
0x00000000,0x01010001,0x07040005,0x0e040063,0x0a010023,0x07040006,0x07030008,0x08040008,
0x00000000,0x0e040044,0x07040006,0x07030009,0x10020047,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0801000c,0x0e040010,0x0e030011,0x03030003,
0x0e0100e1,0x0902000b,0x04010003,0x0a040006,0x09010014,0x0a030007,0x0e030012,0x04040004,
0x00000000,0x01010001,0x0e040010,0x0a03000b,0x0a010024,0x0e040011,0x0e030013,0x08050006,
0x0f010161,0x0a020013,0x0e040011,0x08040007,0x10020048,0x0a030008,0x0e030014,0x0e070012,
0x00000000,0x01010001,0x08020005,0x02020002,0x0801000d,0x0a040006,0x0a030007,0x0e050034,
0x10010261,0x10030027,0x08020006,0x0a040007,0x09010015,0x0a030009,0x0a030008,0x08050007,
0x00000000,0x01010001,0x0a040006,0x10040017,0x0a010025,0x0a040007,0x0a030009,0x08040009,
0x00000000,0x0e040045,0x0a040007,0x08040008,0x10020049,0x0a03000a,0x0a03000a,0x0a070008,
0x00000000,0x01010001,0x0e03001b,0x02020002,0x0801000e,0x0a030007,0x0a020008,0x03030003,
0x00000000,0x0902000c,0x0e03001c,0x08050006,0x09010016,0x08040007,0x0a020009,0x0e06001d,
0x00000000,0x01010001,0x0a030007,0x0e050025,0x0a010026,0x0a030008,0x0a02000a,0x08050007,
0x00000000,0x0a020014,0x0a030008,0x08040009,0x1002004a,0x08040008,0x0a02000b,0x0a060009,
0x00000000,0x01010001,0x09020007,0x02020002,0x0801000f,0x08050006,0x08040007,0x0e040073,
0x00000000,0x10030028,0x09020008,0x08050007,0x09010017,0x08040009,0x08040008,0x09050009,
0x00000000,0x01010001,0x08050006,0x0e040064,0x0a010027,0x08050007,0x08040009,0x0804000a,
0x00000000,0x0e040046,0x08050007,0x0804000a,0x1002004b,0x0804000a,0x0804000a,0x08080008,
0x00000000,0x01010001,0x04010002,0x02020002,0x09020009,0x05020003,0x05010004,0x03030003,
0x0e0100e2,0x0902000d,0x04010003,0x0e040012,0x09010018,0x0e030013,0x05010005,0x04040004,
0x00000000,0x0f0200b2,0x05020003,0x0a03000c,0x0a010028,0x05020004,0x05010006,0x1006000b,
0x0f010162,0x0a020015,0x05020004,0x1005000c,0x1002004c,0x0e030014,0x05010007,0x05050005,
0x00000000,0x01010001,0x09030005,0x02020002,0x0902000a,0x0e040012,0x0e030013,0x10050010,
0x10010262,0x10030029,0x09030006,0x0e040013,0x09010019,0x0e030015,0x0e030014,0x09060007,
0x00000000,0x10020132,0x0e040012,0x10040018,0x0a010029,0x0e040013,0x0e030015,0x09050008,
0x00000000,0x0e040047,0x0e040013,0x1005000d,0x1002004d,0x0e030016,0x0e030016,0x0e070014,
0x00000000,0x01010001,0x0e02003a,0x02020002,0x0902000b,0x09030006,0x09020007,0x03030003,
0x00000000,0x0902000e,0x0e02003b,0x1006000b,0x0901001a,0x1005000c,0x09020008,0x0e05003c,
0x00000000,0x01010001,0x09030006,0x0e050026,0x0a01002a,0x09030007,0x09020009,0x1006000c,
0x00000000,0x0a020016,0x09030007,0x1005000e,0x1002004e,0x1005000d,0x0902000a,0x09060008,
0x00000000,0x01010001,0x09020008,0x02020002,0x0902000c,0x1006000b,0x1005000c,0x0e060016,
0x00000000,0x1003002a,0x09020009,0x1006000c,0x0901001b,0x1005000e,0x1005000d,0x0905000a,
0x00000000,0x01010001,0x1006000b,0x0e040065,0x0a01002b,0x1006000c,0x1005000e,0x09050009,
0x00000000,0x0e040048,0x1006000c,0x1005000f,0x1002004f,0x1005000f,0x1005000f,0x1009000d,
0x00000000,0x01010001,0x04010002,0x02020002,0x0902000d,0x0e03001f,0x0e020020,0x03030003,
0x0e0100e3,0x0902000f,0x04010003,0x1005000c,0x0901001c,0x1004000d,0x0e020021,0x04040004,
0x00000000,0x01010001,0x0e03001f,0x0a03000d,0x0a01002c,0x0e030020,0x0e020022,0x09060007,
0x0f010163,0x0a020017,0x0e030020,0x09050008,0x10020050,0x1004000e,0x0e020023,0x0e060021,
0x00000000,0x01010001,0x09030006,0x02020002,0x0902000e,0x1005000c,0x1004000d,0x0e050035,
0x10010263,0x1003002b,0x09030007,0x1005000d,0x0901001d,0x1004000f,0x1004000e,0x09060008,
0x00000000,0x01010001,0x1005000c,0x10040019,0x0a01002d,0x1005000d,0x1004000f,0x0905000a,
0x00000000,0x0e040049,0x1005000d,0x09050009,0x10020051,0x10040010,0x10040010,0x1008000e,
0x00000000,0x01010001,0x0f02005a,0x02020002,0x0902000f,0x0a030008,0x0a020009,0x03030003,
0x00000000,0x09020010,0x0f02005b,0x09060007,0x0901001e,0x09050008,0x0a02000a,0x0f05005c,
0x00000000,0x01010001,0x0a030008,0x0e050027,0x0a01002e,0x0a030009,0x0a02000b,0x09060008,
0x00000000,0x0a020018,0x0a030009,0x0905000a,0x10020052,0x09050009,0x0a02000c,0x0a06000a,
0x00000000,0x01010001,0x09020009,0x02020002,0x09020010,0x09060007,0x09050008,0x0e040074,
0x00000000,0x1003002c,0x0902000a,0x09060008,0x0901001f,0x0905000a,0x09050009,0x0905000b,
0x00000000,0x01010001,0x09060007,0x0e040066,0x0a01002f,0x09060008,0x0905000a,0x0905000b,
0x00000000,0x0e04004a,0x09060008,0x0905000b,0x10020053,0x0905000b,0x0905000b,0x09090009,
0x00000000,0x01010001,0x04010002,0x02020002,0x0e04000c,0x05020003,0x05010004,0x03030003,
0x0e0100e4,0x0a03000a,0x04010003,0x06030004,0x0a020011,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f0200b3,0x05020003,0x0a03000e,0x0a010030,0x05020004,0x05010006,0x0f050010,
0x0f010164,0x0a020019,0x05020004,0x0f040011,0x10020054,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0e050008,0x1003009b,0x0e04000d,0x06030004,0x06020005,0x10050011,
0x10010264,0x1003002d,0x0e050009,0x06030005,0x0a020012,0x06020007,0x06020006,0x0e08000a,
0x00000000,0x10020133,0x06030004,0x1004001a,0x0a010031,0x06030005,0x06020007,0x0e07000b,
0x00000000,0x0e04004b,0x06030005,0x0f040012,0x10020055,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0e04000d,0x02020002,0x0e04000e,0x0a040006,0x0a030007,0x03030003,
0x00000000,0x0a03000b,0x0e04000e,0x0f050010,0x0a020013,0x0f040011,0x0a030008,0x0e07000f,
0x00000000,0x01010001,0x0a040006,0x0e050028,0x0a010032,0x0a040007,0x0a030009,0x0f050011,
0x00000000,0x0a02001a,0x0a040007,0x0f040013,0x10020056,0x0f040012,0x0a03000a,0x0a070008,
0x00000000,0x01010001,0x0a030007,0x02020002,0x0e04000f,0x0f050010,0x0f040011,0x0e060017,
0x00000000,0x1003002e,0x0a030008,0x0f050011,0x0a020014,0x0f040013,0x0f040012,0x0a060009,
0x00000000,0x01010001,0x0f050010,0x0e040067,0x0a010033,0x0f050011,0x0f040013,0x0e07000c,
0x00000000,0x0e04004c,0x0f050011,0x0f040014,0x10020057,0x0f040014,0x0f040014,0x0f080012,
0x00000000,0x01010001,0x04010002,0x02020002,0x0e040010,0x0f03002f,0x0f020030,0x03030003,
0x0e0100e5,0x0a03000c,0x04010003,0x0a040007,0x0a020015,0x0a030008,0x0f020031,0x04040004,
0x00000000,0x01010001,0x0f03002f,0x0a03000f,0x0a010034,0x0f030030,0x0f020032,0x0e08000a,
0x0f010165,0x0a02001b,0x0f030030,0x0e07000b,0x10020058,0x0a030009,0x0f020033,0x0f060031,
0x00000000,0x01010001,0x0e050009,0x02020002,0x0e040011,0x0a040007,0x0a030008,0x0e050036,
0x10010265,0x1003002f,0x0e05000a,0x0a040008,0x0a020016,0x0a03000a,0x0a030009,0x0e08000b,
0x00000000,0x01010001,0x0a040007,0x1004001b,0x0a010035,0x0a040008,0x0a03000a,0x0e07000d,
0x00000000,0x0e04004d,0x0a040008,0x0e07000c,0x10020059,0x0a03000b,0x0a03000b,0x0a070009,
0x00000000,0x01010001,0x0e03001c,0x02020002,0x0e040012,0x0a030009,0x0a02000a,0x03030003,
0x00000000,0x0a03000d,0x0e03001d,0x0e08000a,0x0a020017,0x0e07000b,0x0a02000b,0x0e06001e,
0x00000000,0x01010001,0x0a030009,0x0e050029,0x0a010036,0x0a03000a,0x0a02000c,0x0e08000b,
0x00000000,0x0a02001c,0x0a03000a,0x0e07000d,0x1002005a,0x0e07000c,0x0a02000d,0x0a06000b,
0x00000000,0x01010001,0x0a030008,0x02020002,0x0e040013,0x0e08000a,0x0e07000b,0x0e040075,
0x00000000,0x10030030,0x0a030009,0x0e08000b,0x0a020018,0x0e07000d,0x0e07000c,0x0a06000a,
0x00000000,0x01010001,0x0e08000a,0x0e040068,0x0a010037,0x0e08000b,0x0e07000d,0x0e07000e,
0x00000000,0x0e04004e,0x0e08000b,0x0e07000e,0x1002005b,0x0e07000e,0x0e07000e,0x0e0b000c,
0x00000000,0x01010001,0x04010002,0x02020002,0x0a03000a,0x05020003,0x05010004,0x03030003,
0x0e0100e6,0x0a03000e,0x04010003,0x0f04001a,0x0a020019,0x0f03001b,0x05010005,0x04040004,
0x00000000,0x0f0200b4,0x05020003,0x0a030010,0x0a010038,0x05020004,0x05010006,0x0e07000b,
0x0f010166,0x0a02001d,0x05020004,0x0e06000c,0x1002005c,0x0f03001c,0x05010007,0x05050005,
0x00000000,0x01010001,0x0a040006,0x02020002,0x0a03000b,0x0f04001a,0x0f03001b,0x10050012,
0x10010266,0x10030031,0x0a040007,0x0f04001b,0x0a02001a,0x0f03001d,0x0f03001c,0x0a070008,
0x00000000,0x10020134,0x0f04001a,0x1004001c,0x0a010039,0x0f04001b,0x0f03001d,0x0a060009,
0x00000000,0x0e04004f,0x0f04001b,0x0e06000d,0x1002005d,0x0f03001e,0x0f03001e,0x0f07001c,
0x00000000,0x01010001,0x0e02003b,0x02020002,0x0a03000c,0x0a040007,0x0a030008,0x03030003,
0x00000000,0x0a03000f,0x0e02003c,0x0e07000b,0x0a02001b,0x0e06000c,0x0a030009,0x0e05003d,
0x00000000,0x01010001,0x0a040007,0x0e05002a,0x0a01003a,0x0a040008,0x0a03000a,0x0e07000c,
0x00000000,0x0a02001e,0x0a040008,0x0e06000e,0x1002005e,0x0e06000d,0x0a03000b,0x0a070009,
0x00000000,0x01010001,0x0a030009,0x02020002,0x0a03000d,0x0e07000b,0x0e06000c,0x0e060018,
0x00000000,0x10030032,0x0a03000a,0x0e07000c,0x0a02001c,0x0e06000e,0x0e06000d,0x0a06000b,
0x00000000,0x01010001,0x0e07000b,0x0e040069,0x0a01003b,0x0e07000c,0x0e06000e,0x0a06000a,
0x00000000,0x0e040050,0x0e07000c,0x0e06000f,0x1002005f,0x0e06000f,0x0e06000f,0x0e0a000d,
0x00000000,0x01010001,0x04010002,0x02020002,0x0a03000e,0x1003004f,0x10020050,0x03030003,
0x0e0100e7,0x0a030010,0x04010003,0x1005000d,0x0a02001d,0x1004000e,0x10020051,0x04040004,
0x00000000,0x01010001,0x1003004f,0x0a030011,0x0a01003c,0x10030050,0x10020052,0x0a070008,
0x0f010167,0x0a02001f,0x10030050,0x0a060009,0x10020060,0x1004000f,0x10020053,0x10060051,
0x00000000,0x01010001,0x0a040007,0x02020002,0x0a03000f,0x1005000d,0x1004000e,0x0e050037,
0x10010267,0x10030033,0x0a040008,0x1005000e,0x0a02001e,0x10040010,0x1004000f,0x0a070009,
0x00000000,0x01010001,0x1005000d,0x1004001d,0x0a01003d,0x1005000e,0x10040010,0x0a06000b,
0x00000000,0x0e040051,0x1005000e,0x0a06000a,0x10020061,0x10040011,0x10040011,0x1008000f,
0x00000000,0x01010001,0x0f02005b,0x02020002,0x0a030010,0x0a03000a,0x0a02000b,0x03030003,
0x00000000,0x0a030011,0x0f02005c,0x0a070008,0x0a02001f,0x0a060009,0x0a02000c,0x0f05005d,
0x00000000,0x01010001,0x0a03000a,0x0e05002b,0x0a01003e,0x0a03000b,0x0a02000d,0x0a070009,
0x00000000,0x0a020020,0x0a03000b,0x0a06000b,0x10020062,0x0a06000a,0x0a02000e,0x0a06000c,
0x00000000,0x01010001,0x0a03000a,0x02020002,0x0a030011,0x0a070008,0x0a060009,0x0e040076,
0x00000000,0x10030034,0x0a03000b,0x0a070009,0x0a020020,0x0a06000b,0x0a06000a,0x0a06000c,
0x00000000,0x01010001,0x0a070008,0x0e04006a,0x0a01003f,0x0a070009,0x0a06000b,0x0a06000c,
0x00000000,0x0e040052,0x0a070009,0x0a06000c,0x10020063,0x0a06000c,0x0a06000c,0x0a0a000a,
0x00000000,0x01010001,0x04010002,0x02020002,0x08010008,0x05020003,0x05010004,0x03030003,
0x0e0100e8,0x0e04000d,0x04010003,0x06030004,0x0e030014,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f0200b5,0x05020003,0x1005000f,0x10030025,0x05020004,0x05010006,0x07040005,
0x0f010168,0x10040016,0x05020004,0x07030006,0x10020064,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x08020004,0x1003009c,0x08010009,0x06030004,0x06020005,0x10050013,
0x10010268,0x10030035,0x08020005,0x06030005,0x0e030015,0x06020007,0x06020006,0x08050006,
0x00000000,0x10020135,0x06030004,0x1004001e,0x10030026,0x06030005,0x06020007,0x08040007,
0x00000000,0x0e040053,0x06030005,0x07030007,0x10020065,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0e04000e,0x02020002,0x0801000a,0x0e050009,0x0e04000a,0x03030003,
0x00000000,0x0e04000e,0x0e04000f,0x07040005,0x0e030016,0x07030006,0x0e04000b,0x0e070010,
0x00000000,0x01010001,0x0e050009,0x0e05002c,0x10030027,0x0e05000a,0x0e04000c,0x07040006,
0x00000000,0x10040017,0x0e05000a,0x07030008,0x10020066,0x07030007,0x0e04000d,0x0e08000b,
0x00000000,0x01010001,0x0e04000a,0x02020002,0x0801000b,0x07040005,0x07030006,0x0e060019,
0x00000000,0x10030036,0x0e04000b,0x07040006,0x0e030017,0x07030008,0x07030007,0x0e07000c,
0x00000000,0x01010001,0x07040005,0x0e04006b,0x10030028,0x07040006,0x07030008,0x08040008,
0x00000000,0x0e040054,0x07040006,0x07030009,0x10020067,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0801000c,0x0e040011,0x0e030012,0x03030003,
0x0e0100e9,0x0e04000f,0x04010003,0x1006000b,0x0e030018,0x1005000c,0x0e030013,0x04040004,
0x00000000,0x01010001,0x0e040011,0x10050010,0x10030029,0x0e040012,0x0e030014,0x08050006,
0x0f010169,0x10040018,0x0e040012,0x08040007,0x10020068,0x1005000d,0x0e030015,0x0e070013,
0x00000000,0x01010001,0x08020005,0x02020002,0x0801000d,0x1006000b,0x1005000c,0x0e050038,
0x10010269,0x10030037,0x08020006,0x1006000c,0x0e030019,0x1005000e,0x1005000d,0x08050007,
0x00000000,0x01010001,0x1006000b,0x1004001f,0x1003002a,0x1006000c,0x1005000e,0x08040009,
0x00000000,0x0e040055,0x1006000c,0x08040008,0x10020069,0x1005000f,0x1005000f,0x1009000d,
0x00000000,0x01010001,0x0e03001d,0x02020002,0x0801000e,0x1005000c,0x1004000d,0x03030003,
0x00000000,0x0e040010,0x0e03001e,0x08050006,0x0e03001a,0x08040007,0x1004000e,0x0e06001f,
0x00000000,0x01010001,0x1005000c,0x0e05002d,0x1003002b,0x1005000d,0x1004000f,0x08050007,
0x00000000,0x10040019,0x1005000d,0x08040009,0x1002006a,0x08040008,0x10040010,0x1008000e,
0x00000000,0x01010001,0x0e04000b,0x02020002,0x0801000f,0x08050006,0x08040007,0x0e040077,
0x00000000,0x10030038,0x0e04000c,0x08050007,0x0e03001b,0x08040009,0x08040008,0x0e07000d,
0x00000000,0x01010001,0x08050006,0x0e04006c,0x1003002c,0x08050007,0x08040009,0x0804000a,
0x00000000,0x0e040056,0x08050007,0x0804000a,0x1002006b,0x0804000a,0x0804000a,0x08080008,
0x00000000,0x01010001,0x04010002,0x02020002,0x0e04000d,0x05020003,0x05010004,0x03030003,
0x0e0100ea,0x0e040011,0x04010003,0x1004002a,0x0e03001c,0x1003002b,0x05010005,0x04040004,
0x00000000,0x0f0200b6,0x05020003,0x10050011,0x1003002d,0x05020004,0x05010006,0x1006000c,
0x0f01016a,0x1004001a,0x05020004,0x1005000d,0x1002006c,0x1003002c,0x05010007,0x05050005,
0x00000000,0x01010001,0x0e050009,0x02020002,0x0e04000e,0x1004002a,0x1003002b,0x10050014,
0x1001026a,0x10030039,0x0e05000a,0x1004002b,0x0e03001d,0x1003002d,0x1003002c,0x0e08000b,
0x00000000,0x10020136,0x1004002a,0x10040020,0x1003002e,0x1004002b,0x1003002d,0x0e07000c,
0x00000000,0x0e040057,0x1004002b,0x1005000e,0x1002006d,0x1003002e,0x1003002e,0x1007002c,
0x00000000,0x01010001,0x0e02003c,0x02020002,0x0e04000f,0x0e05000a,0x0e04000b,0x03030003,
0x00000000,0x0e040012,0x0e02003d,0x1006000c,0x0e03001e,0x1005000d,0x0e04000c,0x0e05003e,
0x00000000,0x01010001,0x0e05000a,0x0e05002e,0x1003002f,0x0e05000b,0x0e04000d,0x1006000d,
0x00000000,0x1004001b,0x0e05000b,0x1005000f,0x1002006e,0x1005000e,0x0e04000e,0x0e08000c,
0x00000000,0x01010001,0x0e04000c,0x02020002,0x0e040010,0x1006000c,0x1005000d,0x0e06001a,
0x00000000,0x1003003a,0x0e04000d,0x1006000d,0x0e03001f,0x1005000f,0x1005000e,0x0e07000e,
0x00000000,0x01010001,0x1006000c,0x0e04006d,0x10030030,0x1006000d,0x1005000f,0x0e07000d,
0x00000000,0x0e040058,0x1006000d,0x10050010,0x1002006f,0x10050010,0x10050010,0x1009000e,
0x00000000,0x01010001,0x04010002,0x02020002,0x0e040011,0x0e030020,0x0e020021,0x03030003,
0x0e0100eb,0x0e040013,0x04010003,0x1005000e,0x0e030020,0x1004000f,0x0e020022,0x04040004,
0x00000000,0x01010001,0x0e030020,0x10050012,0x10030031,0x0e030021,0x0e020023,0x0e08000b,
0x0f01016b,0x1004001c,0x0e030021,0x0e07000c,0x10020070,0x10040010,0x0e020024,0x0e060022,
0x00000000,0x01010001,0x0e05000a,0x02020002,0x0e040012,0x1005000e,0x1004000f,0x0e050039,
0x1001026b,0x1003003b,0x0e05000b,0x1005000f,0x0e030021,0x10040011,0x10040010,0x0e08000c,
0x00000000,0x01010001,0x1005000e,0x10040021,0x10030032,0x1005000f,0x10040011,0x0e07000e,
0x00000000,0x0e040059,0x1005000f,0x0e07000d,0x10020071,0x10040012,0x10040012,0x10080010,
0x00000000,0x01010001,0x0f02005c,0x02020002,0x0e040013,0x1005000d,0x1004000e,0x03030003,
0x00000000,0x0e040014,0x0f02005d,0x0e08000b,0x0e030022,0x0e07000c,0x1004000f,0x0f05005e,
0x00000000,0x01010001,0x1005000d,0x0e05002f,0x10030033,0x1005000e,0x10040010,0x0e08000c,
0x00000000,0x1004001d,0x1005000e,0x0e07000e,0x10020072,0x0e07000d,0x10040011,0x1008000f,
0x00000000,0x01010001,0x0e04000d,0x02020002,0x0e040014,0x0e08000b,0x0e07000c,0x0e040078,
0x00000000,0x1003003c,0x0e04000e,0x0e08000c,0x0e030023,0x0e07000e,0x0e07000d,0x0e07000f,
0x00000000,0x01010001,0x0e08000b,0x0e04006e,0x10030034,0x0e08000c,0x0e07000e,0x0e07000f,
0x00000000,0x0e04005a,0x0e08000c,0x0e07000f,0x10020073,0x0e07000f,0x0e07000f,0x0e0b000d,
0x00000000,0x01010001,0x04010002,0x02020002,0x0e03000f,0x05020003,0x05010004,0x03030003,
0x0e0100ec,0x1005000f,0x04010003,0x06030004,0x10040016,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f0200b7,0x05020003,0x10050013,0x10030035,0x05020004,0x05010006,0x10050018,
0x0f01016c,0x1004001e,0x05020004,0x10040019,0x10020074,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0e04000b,0x1003009d,0x0e030010,0x06030004,0x06020005,0x10050015,
0x1001026c,0x1003003d,0x0e04000c,0x06030005,0x10040017,0x06020007,0x06020006,0x0e07000d,
0x00000000,0x10020137,0x06030004,0x10040022,0x10030036,0x06030005,0x06020007,0x0e06000e,
0x00000000,0x0e04005b,0x06030005,0x1004001a,0x10020075,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0e04000f,0x02020002,0x0e030011,0x1006000b,0x1005000c,0x03030003,
0x00000000,0x10050010,0x0e040010,0x10050018,0x10040018,0x10040019,0x1005000d,0x0e070011,
0x00000000,0x01010001,0x1006000b,0x0e050030,0x10030037,0x1006000c,0x1005000e,0x10050019,
0x00000000,0x1004001f,0x1006000c,0x1004001b,0x10020076,0x1004001a,0x1005000f,0x1009000d,
0x00000000,0x01010001,0x1005000c,0x02020002,0x0e030012,0x10050018,0x10040019,0x0e06001b,
0x00000000,0x1003003e,0x1005000d,0x10050019,0x10040019,0x1004001b,0x1004001a,0x1008000e,
0x00000000,0x01010001,0x10050018,0x0e04006f,0x10030038,0x10050019,0x1004001b,0x0e06000f,
0x00000000,0x0e04005c,0x10050019,0x1004001c,0x10020077,0x1004001c,0x1004001c,0x1008001a,
0x00000000,0x01010001,0x04010002,0x02020002,0x0e030013,0x0f030030,0x0f020031,0x03030003,
0x0e0100ed,0x10050011,0x04010003,0x1006000c,0x1004001a,0x1005000d,0x0f020032,0x04040004,
0x00000000,0x01010001,0x0f030030,0x10050014,0x10030039,0x0f030031,0x0f020033,0x0e07000d,
0x0f01016d,0x10040020,0x0f030031,0x0e06000e,0x10020078,0x1005000e,0x0f020034,0x0f060032,
0x00000000,0x01010001,0x0e04000c,0x02020002,0x0e030014,0x1006000c,0x1005000d,0x0e05003a,
0x1001026d,0x1003003f,0x0e04000d,0x1006000d,0x1004001b,0x1005000f,0x1005000e,0x0e07000e,
0x00000000,0x01010001,0x1006000c,0x10040023,0x1003003a,0x1006000d,0x1005000f,0x0e060010,
0x00000000,0x0e04005d,0x1006000d,0x0e06000f,0x10020079,0x10050010,0x10050010,0x1009000e,
0x00000000,0x01010001,0x0e03001e,0x02020002,0x0e030015,0x1005000e,0x1004000f,0x03030003,
0x00000000,0x10050012,0x0e03001f,0x0e07000d,0x1004001c,0x0e06000e,0x10040010,0x0e060020,
0x00000000,0x01010001,0x1005000e,0x0e050031,0x1003003b,0x1005000f,0x10040011,0x0e07000e,
0x00000000,0x10040021,0x1005000f,0x0e060010,0x1002007a,0x0e06000f,0x10040012,0x10080010,
0x00000000,0x01010001,0x1005000d,0x02020002,0x0e030016,0x0e07000d,0x0e06000e,0x0e040079,
0x00000000,0x10030040,0x1005000e,0x0e07000e,0x1004001d,0x0e060010,0x0e06000f,0x1008000f,
0x00000000,0x01010001,0x0e07000d,0x0e040070,0x1003003c,0x0e07000e,0x0e060010,0x0e060011,
0x00000000,0x0e04005e,0x0e07000e,0x0e060011,0x1002007b,0x0e060011,0x0e060011,0x0e0a000f,
0x00000000,0x01010001,0x04010002,0x02020002,0x1005000f,0x05020003,0x05010004,0x03030003,
0x0e0100ee,0x10050013,0x04010003,0x06030004,0x1004001e,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f0200b8,0x05020003,0x10050015,0x1003003d,0x05020004,0x05010006,0x0e07000c,
0x0f01016e,0x10040022,0x05020004,0x0e06000d,0x1002007c,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x1006000b,0x02020002,0x10050010,0x06030004,0x06020005,0x10050016,
0x1001026e,0x10030041,0x1006000c,0x06030005,0x1004001f,0x06020007,0x06020006,0x1009000d,
0x00000000,0x10020138,0x06030004,0x10040024,0x1003003e,0x06030005,0x06020007,0x1008000e,
0x00000000,0x0e04005f,0x06030005,0x0e06000e,0x1002007d,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0e02003d,0x02020002,0x10050011,0x1006000c,0x1005000d,0x03030003,
0x00000000,0x10050014,0x0e02003e,0x0e07000c,0x10040020,0x0e06000d,0x1005000e,0x0e05003f,
0x00000000,0x01010001,0x1006000c,0x0e050032,0x1003003f,0x1006000d,0x1005000f,0x0e07000d,
0x00000000,0x10040023,0x1006000d,0x0e06000f,0x1002007e,0x0e06000e,0x10050010,0x1009000e,
0x00000000,0x01010001,0x1005000e,0x02020002,0x10050012,0x0e07000c,0x0e06000d,0x0e06001c,
0x00000000,0x10030042,0x1005000f,0x0e07000d,0x10040021,0x0e06000f,0x0e06000e,0x10080010,
0x00000000,0x01010001,0x0e07000c,0x0e040071,0x10030040,0x0e07000d,0x0e06000f,0x1008000f,
0x00000000,0x0e040060,0x0e07000d,0x0e060010,0x1002007f,0x0e060010,0x0e060010,0x0e0a000e,
0x00000000,0x01010001,0x04010002,0x02020002,0x10050013,0x10030050,0x10020051,0x03030003,
0x0e0100ef,0x10050015,0x04010003,0x1005000f,0x10040022,0x10040010,0x10020052,0x04040004,
0x00000000,0x01010001,0x10030050,0x10050016,0x10030041,0x10030051,0x10020053,0x1009000d,
0x0f01016f,0x10040024,0x10030051,0x1008000e,0x10020080,0x10040011,0x10020054,0x10060052,
0x00000000,0x01010001,0x1006000c,0x02020002,0x10050014,0x1005000f,0x10040010,0x0e05003b,
0x1001026f,0x10030043,0x1006000d,0x10050010,0x10040023,0x10040012,0x10040011,0x1009000e,
0x00000000,0x01010001,0x1005000f,0x10040025,0x10030042,0x10050010,0x10040012,0x10080010,
0x00000000,0x0e040061,0x10050010,0x1008000f,0x10020081,0x10040013,0x10040013,0x10080011,
0x00000000,0x01010001,0x0f02005d,0x02020002,0x10050015,0x1005000f,0x10040010,0x03030003,
0x00000000,0x10050016,0x0f02005e,0x1009000d,0x10040024,0x1008000e,0x10040011,0x0f05005f,
0x00000000,0x01010001,0x1005000f,0x0e050033,0x10030043,0x10050010,0x10040012,0x1009000e,
0x00000000,0x10040025,0x10050010,0x10080010,0x10020082,0x1008000f,0x10040013,0x10080011,
0x00000000,0x01010001,0x1005000f,0x02020002,0x10050016,0x1009000d,0x1008000e,0x0e04007a,
0x00000000,0x10030044,0x10050010,0x1009000e,0x10040025,0x10080010,0x1008000f,0x10080011,
0x00000000,0x01010001,0x1009000d,0x0e040072,0x10030044,0x1009000e,0x10080010,0x10080011,
0x00000000,0x0e040062,0x1009000e,0x10080011,0x10020083,0x10080011,0x10080011,0x100c000f,
0x00000000,0x01010001,0x04010002,0x02020002,0x08010008,0x05020003,0x05010004,0x03030003,
0x0e0100f0,0x09020009,0x04010003,0x06030004,0x09010010,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f0200b9,0x05020003,0x0f04000f,0x0f020025,0x05020004,0x05010006,0x07040005,
0x0f010170,0x0f030016,0x05020004,0x07030006,0x0e040043,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x08020004,0x1003009e,0x08010009,0x06030004,0x06020005,0x0e07000e,
0x10010270,0x0e050024,0x08020005,0x06030005,0x09010011,0x06020007,0x06020006,0x08050006,
0x00000000,0x10020139,0x06030004,0x0e060015,0x0f020026,0x06030005,0x06020007,0x08040007,
0x00000000,0x0e040063,0x06030005,0x07030007,0x0e040044,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0e040010,0x02020002,0x0801000a,0x09030005,0x09020006,0x03030003,
0x00000000,0x0902000a,0x0e040011,0x07040005,0x09010012,0x07030006,0x09020007,0x0e070012,
0x00000000,0x01010001,0x09030005,0x0e050034,0x0f020027,0x09030006,0x09020008,0x07040006,
0x00000000,0x0f030017,0x09030006,0x07030008,0x0e040045,0x07030007,0x09020009,0x09060007,
0x00000000,0x01010001,0x09020006,0x02020002,0x0801000b,0x07040005,0x07030006,0x0e06001d,
0x00000000,0x0e050025,0x09020007,0x07040006,0x09010013,0x07030008,0x07030007,0x09050008,
0x00000000,0x01010001,0x07040005,0x0e040073,0x0f020028,0x07040006,0x07030008,0x08040008,
0x00000000,0x0e040064,0x07040006,0x07030009,0x0e040046,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0801000c,0x0e040012,0x0e030013,0x03030003,
0x0e0100f1,0x0902000b,0x04010003,0x0f05000b,0x09010014,0x0f04000c,0x0e030014,0x04040004,
0x00000000,0x01010001,0x0e040012,0x0f040010,0x0f020029,0x0e040013,0x0e030015,0x08050006,
0x0f010171,0x0f030018,0x0e040013,0x08040007,0x0e040047,0x0f04000d,0x0e030016,0x0e070014,
0x00000000,0x01010001,0x08020005,0x02020002,0x0801000d,0x0f05000b,0x0f04000c,0x0e05003c,
0x10010271,0x0e050026,0x08020006,0x0f05000c,0x09010015,0x0f04000e,0x0f04000d,0x08050007,
0x00000000,0x01010001,0x0f05000b,0x0e060016,0x0f02002a,0x0f05000c,0x0f04000e,0x08040009,
0x00000000,0x0e040065,0x0f05000c,0x08040008,0x0e040048,0x0f04000f,0x0f04000f,0x0f08000d,
0x00000000,0x01010001,0x0e03001f,0x02020002,0x0801000e,0x0f04000c,0x0f03000d,0x03030003,
0x00000000,0x0902000c,0x0e030020,0x08050006,0x09010016,0x08040007,0x0f03000e,0x0e060021,
0x00000000,0x01010001,0x0f04000c,0x0e050035,0x0f02002b,0x0f04000d,0x0f03000f,0x08050007,
0x00000000,0x0f030019,0x0f04000d,0x08040009,0x0e040049,0x08040008,0x0f030010,0x0f07000e,
0x00000000,0x01010001,0x09020007,0x02020002,0x0801000f,0x08050006,0x08040007,0x0e04007b,
0x00000000,0x0e050027,0x09020008,0x08050007,0x09010017,0x08040009,0x08040008,0x09050009,
0x00000000,0x01010001,0x08050006,0x0e040074,0x0f02002c,0x08050007,0x08040009,0x0804000a,
0x00000000,0x0e040066,0x08050007,0x0804000a,0x0e04004a,0x0804000a,0x0804000a,0x08080008,
0x00000000,0x01010001,0x04010002,0x02020002,0x09020009,0x05020003,0x05010004,0x03030003,
0x0e0100f2,0x0902000d,0x04010003,0x0e040013,0x09010018,0x0e030014,0x05010005,0x04040004,
0x00000000,0x0f0200ba,0x05020003,0x0f040011,0x0f02002d,0x05020004,0x05010006,0x0e08000a,
0x0f010172,0x0f03001a,0x05020004,0x0e07000b,0x0e04004b,0x0e030015,0x05010007,0x05050005,
0x00000000,0x01010001,0x09030005,0x02020002,0x0902000a,0x0e040013,0x0e030014,0x0e07000f,
0x10010272,0x0e050028,0x09030006,0x0e040014,0x09010019,0x0e030016,0x0e030015,0x09060007,
0x00000000,0x1002013a,0x0e040013,0x0e060017,0x0f02002e,0x0e040014,0x0e030016,0x09050008,
0x00000000,0x0e040067,0x0e040014,0x0e07000c,0x0e04004c,0x0e030017,0x0e030017,0x0e070015,
0x00000000,0x01010001,0x0e02003e,0x02020002,0x0902000b,0x09030006,0x09020007,0x03030003,
0x00000000,0x0902000e,0x0e02003f,0x0e08000a,0x0901001a,0x0e07000b,0x09020008,0x0e050040,
0x00000000,0x01010001,0x09030006,0x0e050036,0x0f02002f,0x09030007,0x09020009,0x0e08000b,
0x00000000,0x0f03001b,0x09030007,0x0e07000d,0x0e04004d,0x0e07000c,0x0902000a,0x09060008,
0x00000000,0x01010001,0x09020008,0x02020002,0x0902000c,0x0e08000a,0x0e07000b,0x0e06001e,
0x00000000,0x0e050029,0x09020009,0x0e08000b,0x0901001b,0x0e07000d,0x0e07000c,0x0905000a,
0x00000000,0x01010001,0x0e08000a,0x0e040075,0x0f020030,0x0e08000b,0x0e07000d,0x09050009,
0x00000000,0x0e040068,0x0e08000b,0x0e07000e,0x0e04004e,0x0e07000e,0x0e07000e,0x0e0b000c,
0x00000000,0x01010001,0x04010002,0x02020002,0x0902000d,0x0e030021,0x0e020022,0x03030003,
0x0e0100f3,0x0902000f,0x04010003,0x0e07000b,0x0901001c,0x0e06000c,0x0e020023,0x04040004,
0x00000000,0x01010001,0x0e030021,0x0f040012,0x0f020031,0x0e030022,0x0e020024,0x09060007,
0x0f010173,0x0f03001c,0x0e030022,0x09050008,0x0e04004f,0x0e06000d,0x0e020025,0x0e060023,
0x00000000,0x01010001,0x09030006,0x02020002,0x0902000e,0x0e07000b,0x0e06000c,0x0e05003d,
0x10010273,0x0e05002a,0x09030007,0x0e07000c,0x0901001d,0x0e06000e,0x0e06000d,0x09060008,
0x00000000,0x01010001,0x0e07000b,0x0e060018,0x0f020032,0x0e07000c,0x0e06000e,0x0905000a,
0x00000000,0x0e040069,0x0e07000c,0x09050009,0x0e040050,0x0e06000f,0x0e06000f,0x0e0a000d,
0x00000000,0x01010001,0x0f02005e,0x02020002,0x0902000f,0x0f04000d,0x0f03000e,0x03030003,
0x00000000,0x09020010,0x0f02005f,0x09060007,0x0901001e,0x09050008,0x0f03000f,0x0f050060,
0x00000000,0x01010001,0x0f04000d,0x0e050037,0x0f020033,0x0f04000e,0x0f030010,0x09060008,
0x00000000,0x0f03001d,0x0f04000e,0x0905000a,0x0e040051,0x09050009,0x0f030011,0x0f07000f,
0x00000000,0x01010001,0x09020009,0x02020002,0x09020010,0x09060007,0x09050008,0x0e04007c,
0x00000000,0x0e05002b,0x0902000a,0x09060008,0x0901001f,0x0905000a,0x09050009,0x0905000b,
0x00000000,0x01010001,0x09060007,0x0e040076,0x0f020034,0x09060008,0x0905000a,0x0905000b,
0x00000000,0x0e04006a,0x09060008,0x0905000b,0x0e040052,0x0905000b,0x0905000b,0x09090009,
0x00000000,0x01010001,0x04010002,0x02020002,0x0e04000d,0x05020003,0x05010004,0x03030003,
0x0e0100f4,0x0f04000f,0x04010003,0x06030004,0x0f030016,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f0200bb,0x05020003,0x0f040013,0x0f020035,0x05020004,0x05010006,0x07040005,
0x0f010174,0x0f03001e,0x05020004,0x07030006,0x0e040053,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0e050009,0x1003009f,0x0e04000e,0x06030004,0x06020005,0x0e070010,
0x10010274,0x0e05002c,0x0e05000a,0x06030005,0x0f030017,0x06020007,0x06020006,0x0e08000b,
0x00000000,0x1002013b,0x06030004,0x0e060019,0x0f020036,0x06030005,0x06020007,0x0e07000c,
0x00000000,0x0e04006b,0x06030005,0x07030007,0x0e040054,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0e040011,0x02020002,0x0e04000f,0x0f05000b,0x0f04000c,0x03030003,
0x00000000,0x0f040010,0x0e040012,0x07040005,0x0f030018,0x07030006,0x0f04000d,0x0e070013,
0x00000000,0x01010001,0x0f05000b,0x0e050038,0x0f020037,0x0f05000c,0x0f04000e,0x07040006,
0x00000000,0x0f03001f,0x0f05000c,0x07030008,0x0e040055,0x07030007,0x0f04000f,0x0f08000d,
0x00000000,0x01010001,0x0f04000c,0x02020002,0x0e040010,0x07040005,0x07030006,0x0e06001f,
0x00000000,0x0e05002d,0x0f04000d,0x07040006,0x0f030019,0x07030008,0x07030007,0x0f07000e,
0x00000000,0x01010001,0x07040005,0x0e040077,0x0f020038,0x07040006,0x07030008,0x0e07000d,
0x00000000,0x0e04006c,0x07040006,0x07030009,0x0e040056,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0e040011,0x0f030031,0x0f020032,0x03030003,
0x0e0100f5,0x0f040011,0x04010003,0x0f05000c,0x0f03001a,0x0f04000d,0x0f020033,0x04040004,
0x00000000,0x01010001,0x0f030031,0x0f040014,0x0f020039,0x0f030032,0x0f020034,0x0e08000b,
0x0f010175,0x0f030020,0x0f030032,0x0e07000c,0x0e040057,0x0f04000e,0x0f020035,0x0f060033,
0x00000000,0x01010001,0x0e05000a,0x02020002,0x0e040012,0x0f05000c,0x0f04000d,0x0e05003e,
0x10010275,0x0e05002e,0x0e05000b,0x0f05000d,0x0f03001b,0x0f04000f,0x0f04000e,0x0e08000c,
0x00000000,0x01010001,0x0f05000c,0x0e06001a,0x0f02003a,0x0f05000d,0x0f04000f,0x0e07000e,
0x00000000,0x0e04006d,0x0f05000d,0x0e07000d,0x0e040058,0x0f040010,0x0f040010,0x0f08000e,
0x00000000,0x01010001,0x0e030020,0x02020002,0x0e040013,0x0f04000e,0x0f03000f,0x03030003,
0x00000000,0x0f040012,0x0e030021,0x0e08000b,0x0f03001c,0x0e07000c,0x0f030010,0x0e060022,
0x00000000,0x01010001,0x0f04000e,0x0e050039,0x0f02003b,0x0f04000f,0x0f030011,0x0e08000c,
0x00000000,0x0f030021,0x0f04000f,0x0e07000e,0x0e040059,0x0e07000d,0x0f030012,0x0f070010,
0x00000000,0x01010001,0x0f04000d,0x02020002,0x0e040014,0x0e08000b,0x0e07000c,0x0e04007d,
0x00000000,0x0e05002f,0x0f04000e,0x0e08000c,0x0f03001d,0x0e07000e,0x0e07000d,0x0f07000f,
0x00000000,0x01010001,0x0e08000b,0x0e040078,0x0f02003c,0x0e08000c,0x0e07000e,0x0e07000f,
0x00000000,0x0e04006e,0x0e08000c,0x0e07000f,0x0e04005a,0x0e07000f,0x0e07000f,0x0e0b000d,
0x00000000,0x01010001,0x04010002,0x02020002,0x0f04000f,0x05020003,0x05010004,0x03030003,
0x0e0100f6,0x0f040013,0x04010003,0x0f04001b,0x0f03001e,0x0f03001c,0x05010005,0x04040004,
0x00000000,0x0f0200bc,0x05020003,0x0f040015,0x0f02003d,0x05020004,0x05010006,0x0e07000d,
0x0f010176,0x0f030022,0x05020004,0x0e06000e,0x0e04005b,0x0f03001d,0x05010007,0x05050005,
0x00000000,0x01010001,0x0f05000b,0x02020002,0x0f040010,0x0f04001b,0x0f03001c,0x0e070011,
0x10010276,0x0e050030,0x0f05000c,0x0f04001c,0x0f03001f,0x0f03001e,0x0f03001d,0x0f08000d,
0x00000000,0x1002013c,0x0f04001b,0x0e06001b,0x0f02003e,0x0f04001c,0x0f03001e,0x0f07000e,
0x00000000,0x0e04006f,0x0f04001c,0x0e06000f,0x0e04005c,0x0f03001f,0x0f03001f,0x0f07001d,
0x00000000,0x01010001,0x0e02003f,0x02020002,0x0f040011,0x0f05000c,0x0f04000d,0x03030003,
0x00000000,0x0f040014,0x0e020040,0x0e07000d,0x0f030020,0x0e06000e,0x0f04000e,0x0e050041,
0x00000000,0x01010001,0x0f05000c,0x0e05003a,0x0f02003f,0x0f05000d,0x0f04000f,0x0e07000e,
0x00000000,0x0f030023,0x0f05000d,0x0e060010,0x0e04005d,0x0e06000f,0x0f040010,0x0f08000e,
0x00000000,0x01010001,0x0f04000e,0x02020002,0x0f040012,0x0e07000d,0x0e06000e,0x0e060020,
0x00000000,0x0e050031,0x0f04000f,0x0e07000e,0x0f030021,0x0e060010,0x0e06000f,0x0f070010,
0x00000000,0x01010001,0x0e07000d,0x0e040079,0x0f020040,0x0e07000e,0x0e060010,0x0f07000f,
0x00000000,0x0e040070,0x0e07000e,0x0e060011,0x0e04005e,0x0e060011,0x0e060011,0x0e0a000f,
0x00000000,0x01010001,0x04010002,0x02020002,0x0f040013,0x10030051,0x10020052,0x03030003,
0x0e0100f7,0x0f040015,0x04010003,0x0e07000c,0x0f030022,0x0e06000d,0x10020053,0x04040004,
0x00000000,0x01010001,0x10030051,0x0f040016,0x0f020041,0x10030052,0x10020054,0x0f08000d,
0x0f010177,0x0f030024,0x10030052,0x0f07000e,0x0e04005f,0x0e06000e,0x10020055,0x10060053,
0x00000000,0x01010001,0x0f05000c,0x02020002,0x0f040014,0x0e07000c,0x0e06000d,0x0e05003f,
0x10010277,0x0e050032,0x0f05000d,0x0e07000d,0x0f030023,0x0e06000f,0x0e06000e,0x0f08000e,
0x00000000,0x01010001,0x0e07000c,0x0e06001c,0x0f020042,0x0e07000d,0x0e06000f,0x0f070010,
0x00000000,0x0e040071,0x0e07000d,0x0f07000f,0x0e040060,0x0e060010,0x0e060010,0x0e0a000e,
0x00000000,0x01010001,0x0f02005f,0x02020002,0x0f040015,0x0f04000f,0x0f030010,0x03030003,
0x00000000,0x0f040016,0x0f020060,0x0f08000d,0x0f030024,0x0f07000e,0x0f030011,0x0f050061,
0x00000000,0x01010001,0x0f04000f,0x0e05003b,0x0f020043,0x0f040010,0x0f030012,0x0f08000e,
0x00000000,0x0f030025,0x0f040010,0x0f070010,0x0e040061,0x0f07000f,0x0f030013,0x0f070011,
0x00000000,0x01010001,0x0f04000f,0x02020002,0x0f040016,0x0f08000d,0x0f07000e,0x0e04007e,
0x00000000,0x0e050033,0x0f040010,0x0f08000e,0x0f030025,0x0f070010,0x0f07000f,0x0f070011,
0x00000000,0x01010001,0x0f08000d,0x0e04007a,0x0f020044,0x0f08000e,0x0f070010,0x0f070011,
0x00000000,0x0e040072,0x0f08000e,0x0f070011,0x0e040062,0x0f070011,0x0f070011,0x0f0b000f,
0x00000000,0x01010001,0x04010002,0x02020002,0x08010008,0x05020003,0x05010004,0x03030003,
0x0e0100f8,0x0e030010,0x04010003,0x06030004,0x0e020017,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f0200bd,0x05020003,0x0e07000e,0x0e050024,0x05020004,0x05010006,0x07040005,
0x0f010178,0x0e060015,0x05020004,0x07030006,0x0e040063,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x08020004,0x100300a0,0x08010009,0x06030004,0x06020005,0x0e070012,
0x10010278,0x0e050034,0x08020005,0x06030005,0x0e020018,0x06020007,0x06020006,0x08050006,
0x00000000,0x1002013d,0x06030004,0x0e06001d,0x0e050025,0x06030005,0x06020007,0x08040007,
0x00000000,0x0e040073,0x06030005,0x07030007,0x0e040064,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0e040012,0x02020002,0x0801000a,0x0e04000c,0x0e03000d,0x03030003,
0x00000000,0x0e030011,0x0e040013,0x07040005,0x0e020019,0x07030006,0x0e03000e,0x0e070014,
0x00000000,0x01010001,0x0e04000c,0x0e05003c,0x0e050026,0x0e04000d,0x0e03000f,0x07040006,
0x00000000,0x0e060016,0x0e04000d,0x07030008,0x0e040065,0x07030007,0x0e030010,0x0e07000e,
0x00000000,0x01010001,0x0e03000d,0x02020002,0x0801000b,0x07040005,0x07030006,0x0e060021,
0x00000000,0x0e050035,0x0e03000e,0x07040006,0x0e02001a,0x07030008,0x07030007,0x0e06000f,
0x00000000,0x01010001,0x07040005,0x0e04007b,0x0e050027,0x07040006,0x07030008,0x08040008,
0x00000000,0x0e040074,0x07040006,0x07030009,0x0e040066,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0801000c,0x0e040013,0x0e030014,0x03030003,
0x0e0100f9,0x0e030012,0x04010003,0x0e08000a,0x0e02001b,0x0e07000b,0x0e030015,0x04040004,
0x00000000,0x01010001,0x0e040013,0x0e07000f,0x0e050028,0x0e040014,0x0e030016,0x08050006,
0x0f010179,0x0e060017,0x0e040014,0x08040007,0x0e040067,0x0e07000c,0x0e030017,0x0e070015,
0x00000000,0x01010001,0x08020005,0x02020002,0x0801000d,0x0e08000a,0x0e07000b,0x0e050040,
0x10010279,0x0e050036,0x08020006,0x0e08000b,0x0e02001c,0x0e07000d,0x0e07000c,0x08050007,
0x00000000,0x01010001,0x0e08000a,0x0e06001e,0x0e050029,0x0e08000b,0x0e07000d,0x08040009,
0x00000000,0x0e040075,0x0e08000b,0x08040008,0x0e040068,0x0e07000e,0x0e07000e,0x0e0b000c,
0x00000000,0x01010001,0x0e030021,0x02020002,0x0801000e,0x0e07000b,0x0e06000c,0x03030003,
0x00000000,0x0e030013,0x0e030022,0x08050006,0x0e02001d,0x08040007,0x0e06000d,0x0e060023,
0x00000000,0x01010001,0x0e07000b,0x0e05003d,0x0e05002a,0x0e07000c,0x0e06000e,0x08050007,
0x00000000,0x0e060018,0x0e07000c,0x08040009,0x0e040069,0x08040008,0x0e06000f,0x0e0a000d,
0x00000000,0x01010001,0x0e03000e,0x02020002,0x0801000f,0x08050006,0x08040007,0x0e04007f,
0x00000000,0x0e050037,0x0e03000f,0x08050007,0x0e02001e,0x08040009,0x08040008,0x0e060010,
0x00000000,0x01010001,0x08050006,0x0e04007c,0x0e05002b,0x08050007,0x08040009,0x0804000a,
0x00000000,0x0e040076,0x08050007,0x0804000a,0x0e04006a,0x0804000a,0x0804000a,0x08080008,
0x00000000,0x01010001,0x04010002,0x02020002,0x0e030010,0x05020003,0x05010004,0x03030003,
0x0e0100fa,0x0e030014,0x04010003,0x1004002b,0x0e02001f,0x1003002c,0x05010005,0x04040004,
0x00000000,0x0f0200be,0x05020003,0x0e070010,0x0e05002c,0x05020004,0x05010006,0x0e08000b,
0x0f01017a,0x0e060019,0x05020004,0x0e07000c,0x0e04006b,0x1003002d,0x05010007,0x05050005,
0x00000000,0x01010001,0x0e04000c,0x02020002,0x0e030011,0x1004002b,0x1003002c,0x0e070013,
0x1001027a,0x0e050038,0x0e04000d,0x1004002c,0x0e020020,0x1003002e,0x1003002d,0x0e07000e,
0x00000000,0x1002013e,0x1004002b,0x0e06001f,0x0e05002d,0x1004002c,0x1003002e,0x0e06000f,
0x00000000,0x0e040077,0x1004002c,0x0e07000d,0x0e04006c,0x1003002f,0x1003002f,0x1007002d,
0x00000000,0x01010001,0x0e020040,0x02020002,0x0e030012,0x0e04000d,0x0e03000e,0x03030003,
0x00000000,0x0e030015,0x0e020041,0x0e08000b,0x0e020021,0x0e07000c,0x0e03000f,0x0e050042,
0x00000000,0x01010001,0x0e04000d,0x0e05003e,0x0e05002e,0x0e04000e,0x0e030010,0x0e08000c,
0x00000000,0x0e06001a,0x0e04000e,0x0e07000e,0x0e04006d,0x0e07000d,0x0e030011,0x0e07000f,
0x00000000,0x01010001,0x0e03000f,0x02020002,0x0e030013,0x0e08000b,0x0e07000c,0x0e060022,
0x00000000,0x0e050039,0x0e030010,0x0e08000c,0x0e020022,0x0e07000e,0x0e07000d,0x0e060011,
0x00000000,0x01010001,0x0e08000b,0x0e04007d,0x0e05002f,0x0e08000c,0x0e07000e,0x0e060010,
0x00000000,0x0e040078,0x0e08000c,0x0e07000f,0x0e04006e,0x0e07000f,0x0e07000f,0x0e0b000d,
0x00000000,0x01010001,0x04010002,0x02020002,0x0e030014,0x0e030022,0x0e020023,0x03030003,
0x0e0100fb,0x0e030016,0x04010003,0x0e07000d,0x0e020023,0x0e06000e,0x0e020024,0x04040004,
0x00000000,0x01010001,0x0e030022,0x0e070011,0x0e050030,0x0e030023,0x0e020025,0x0e07000e,
0x0f01017b,0x0e06001b,0x0e030023,0x0e06000f,0x0e04006f,0x0e06000f,0x0e020026,0x0e060024,
0x00000000,0x01010001,0x0e04000d,0x02020002,0x0e030015,0x0e07000d,0x0e06000e,0x0e050041,
0x1001027b,0x0e05003a,0x0e04000e,0x0e07000e,0x0e020024,0x0e060010,0x0e06000f,0x0e07000f,
0x00000000,0x01010001,0x0e07000d,0x0e060020,0x0e050031,0x0e07000e,0x0e060010,0x0e060011,
0x00000000,0x0e040079,0x0e07000e,0x0e060010,0x0e040070,0x0e060011,0x0e060011,0x0e0a000f,
0x00000000,0x01010001,0x0f020060,0x02020002,0x0e030016,0x0e07000c,0x0e06000d,0x03030003,
0x00000000,0x0e030017,0x0f020061,0x0e07000e,0x0e020025,0x0e06000f,0x0e06000e,0x0f050062,
0x00000000,0x01010001,0x0e07000c,0x0e05003f,0x0e050032,0x0e07000d,0x0e06000f,0x0e07000f,
0x00000000,0x0e06001c,0x0e07000d,0x0e060011,0x0e040071,0x0e060010,0x0e060010,0x0e0a000e,
0x00000000,0x01010001,0x0e030010,0x02020002,0x0e030017,0x0e07000e,0x0e06000f,0x0e040080,
0x00000000,0x0e05003b,0x0e030011,0x0e07000f,0x0e020026,0x0e060011,0x0e060010,0x0e060012,
0x00000000,0x01010001,0x0e07000e,0x0e04007e,0x0e050033,0x0e07000f,0x0e060011,0x0e060012,
0x00000000,0x0e04007a,0x0e07000f,0x0e060012,0x0e040072,0x0e060012,0x0e060012,0x0e0a0010,
0x00000000,0x01010001,0x04010002,0x02020002,0x0e030010,0x05020003,0x05010004,0x03030003,
0x0e0100fc,0x0e07000e,0x04010003,0x06030004,0x0e060015,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f0200bf,0x05020003,0x0e070012,0x0e050034,0x05020004,0x05010006,0x07040005,
0x0f01017c,0x0e06001d,0x05020004,0x07030006,0x0e040073,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0e04000c,0x100300a1,0x0e030011,0x06030004,0x06020005,0x0e070014,
0x1001027c,0x0e05003c,0x0e04000d,0x06030005,0x0e060016,0x06020007,0x06020006,0x0e07000e,
0x00000000,0x1002013f,0x06030004,0x0e060021,0x0e050035,0x06030005,0x06020007,0x0e06000f,
0x00000000,0x0e04007b,0x06030005,0x07030007,0x0e040074,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0e040013,0x02020002,0x0e030012,0x0e08000a,0x0e07000b,0x03030003,
0x00000000,0x0e07000f,0x0e040014,0x07040005,0x0e060017,0x07030006,0x0e07000c,0x0e070015,
0x00000000,0x01010001,0x0e08000a,0x0e050040,0x0e050036,0x0e08000b,0x0e07000d,0x07040006,
0x00000000,0x0e06001e,0x0e08000b,0x07030008,0x0e040075,0x07030007,0x0e07000e,0x0e0b000c,
0x00000000,0x01010001,0x0e07000b,0x02020002,0x0e030013,0x07040005,0x07030006,0x0e060023,
0x00000000,0x0e05003d,0x0e07000c,0x07040006,0x0e060018,0x07030008,0x07030007,0x0e0a000d,
0x00000000,0x01010001,0x07040005,0x0e04007f,0x0e050037,0x07040006,0x07030008,0x0e060010,
0x00000000,0x0e04007c,0x07040006,0x07030009,0x0e040076,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0e030014,0x0f030032,0x0f020033,0x03030003,
0x0e0100fd,0x0e070010,0x04010003,0x0e08000b,0x0e060019,0x0e07000c,0x0f020034,0x04040004,
0x00000000,0x01010001,0x0f030032,0x0e070013,0x0e050038,0x0f030033,0x0f020035,0x0e07000e,
0x0f01017d,0x0e06001f,0x0f030033,0x0e06000f,0x0e040077,0x0e07000d,0x0f020036,0x0f060034,
0x00000000,0x01010001,0x0e04000d,0x02020002,0x0e030015,0x0e08000b,0x0e07000c,0x0e050042,
0x1001027d,0x0e05003e,0x0e04000e,0x0e08000c,0x0e06001a,0x0e07000e,0x0e07000d,0x0e07000f,
0x00000000,0x01010001,0x0e08000b,0x0e060022,0x0e050039,0x0e08000c,0x0e07000e,0x0e060011,
0x00000000,0x0e04007d,0x0e08000c,0x0e060010,0x0e040078,0x0e07000f,0x0e07000f,0x0e0b000d,
0x00000000,0x01010001,0x0e030022,0x02020002,0x0e030016,0x0e07000d,0x0e06000e,0x03030003,
0x00000000,0x0e070011,0x0e030023,0x0e07000e,0x0e06001b,0x0e06000f,0x0e06000f,0x0e060024,
0x00000000,0x01010001,0x0e07000d,0x0e050041,0x0e05003a,0x0e07000e,0x0e060010,0x0e07000f,
0x00000000,0x0e060020,0x0e07000e,0x0e060011,0x0e040079,0x0e060010,0x0e060011,0x0e0a000f,
0x00000000,0x01010001,0x0e07000c,0x02020002,0x0e030017,0x0e07000e,0x0e06000f,0x0e040081,
0x00000000,0x0e05003f,0x0e07000d,0x0e07000f,0x0e06001c,0x0e060011,0x0e060010,0x0e0a000e,
0x00000000,0x01010001,0x0e07000e,0x0e040080,0x0e05003b,0x0e07000f,0x0e060011,0x0e060012,
0x00000000,0x0e04007e,0x0e07000f,0x0e060012,0x0e04007a,0x0e060012,0x0e060012,0x0e0a0010,
0x00000000,0x01010001,0x04010002,0x02020002,0x0e07000e,0x05020003,0x05010004,0x03030003,
0x0e0100fe,0x0e070012,0x04010003,0x06030004,0x0e06001d,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f0200c0,0x05020003,0x0e070014,0x0e05003c,0x05020004,0x05010006,0x0e07000e,
0x0f01017e,0x0e060021,0x05020004,0x0e06000f,0x0e04007b,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0e08000a,0x02020002,0x0e07000f,0x06030004,0x06020005,0x0e070015,
0x1001027e,0x0e050040,0x0e08000b,0x06030005,0x0e06001e,0x06020007,0x06020006,0x0e0b000c,
0x00000000,0x10020140,0x06030004,0x0e060023,0x0e05003d,0x06030005,0x06020007,0x0e0a000d,
0x00000000,0x0e04007f,0x06030005,0x0e060010,0x0e04007c,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0e020041,0x02020002,0x0e070010,0x0e08000b,0x0e07000c,0x03030003,
0x00000000,0x0e070013,0x0e020042,0x0e07000e,0x0e06001f,0x0e06000f,0x0e07000d,0x0e050043,
0x00000000,0x01010001,0x0e08000b,0x0e050042,0x0e05003e,0x0e08000c,0x0e07000e,0x0e07000f,
0x00000000,0x0e060022,0x0e08000c,0x0e060011,0x0e04007d,0x0e060010,0x0e07000f,0x0e0b000d,
0x00000000,0x01010001,0x0e07000d,0x02020002,0x0e070011,0x0e07000e,0x0e06000f,0x0e060024,
0x00000000,0x0e050041,0x0e07000e,0x0e07000f,0x0e060020,0x0e060011,0x0e060010,0x0e0a000f,
0x00000000,0x01010001,0x0e07000e,0x0e040081,0x0e05003f,0x0e07000f,0x0e060011,0x0e0a000e,
0x00000000,0x0e040080,0x0e07000f,0x0e060012,0x0e04007e,0x0e060012,0x0e060012,0x0e0a0010,
0x00000000,0x01010001,0x04010002,0x02020002,0x0e070012,0x10030052,0x10020053,0x03030003,
0x0e0100ff,0x0e070014,0x04010003,0x0e07000e,0x0e060021,0x0e06000f,0x10020054,0x04040004,
0x00000000,0x01010001,0x10030052,0x0e070015,0x0e050040,0x10030053,0x10020055,0x0e0b000c,
0x0f01017f,0x0e060023,0x10030053,0x0e0a000d,0x0e04007f,0x0e060010,0x10020056,0x10060054,
0x00000000,0x01010001,0x0e08000b,0x02020002,0x0e070013,0x0e07000e,0x0e06000f,0x0e050043,
0x1001027f,0x0e050042,0x0e08000c,0x0e07000f,0x0e060022,0x0e060011,0x0e060010,0x0e0b000d,
0x00000000,0x01010001,0x0e07000e,0x0e060024,0x0e050041,0x0e07000f,0x0e060011,0x0e0a000f,
0x00000000,0x0e040081,0x0e07000f,0x0e0a000e,0x0e040080,0x0e060012,0x0e060012,0x0e0a0010,
0x00000000,0x01010001,0x0f020061,0x02020002,0x0e070014,0x0e07000e,0x0e06000f,0x03030003,
0x00000000,0x0e070015,0x0f020062,0x0e0b000c,0x0e060023,0x0e0a000d,0x0e060010,0x0f050063,
0x00000000,0x01010001,0x0e07000e,0x0e050043,0x0e050042,0x0e07000f,0x0e060011,0x0e0b000d,
0x00000000,0x0e060024,0x0e07000f,0x0e0a000f,0x0e040081,0x0e0a000e,0x0e060012,0x0e0a0010,
0x00000000,0x01010001,0x0e07000e,0x02020002,0x0e070015,0x0e0b000c,0x0e0a000d,0x0e040082,
0x00000000,0x0e050043,0x0e07000f,0x0e0b000d,0x0e060024,0x0e0a000f,0x0e0a000e,0x0e0a0010,
0x00000000,0x01010001,0x0e0b000c,0x0e040082,0x0e050043,0x0e0b000d,0x0e0a000f,0x0e0a0010,
0x00000000,0x0e040082,0x0e0b000d,0x0e0a0010,0x0e040082,0x0e0a0010,0x0e0a0010,0x0e0e000e,
0x00000000,0x01010001,0x04010002,0x02020002,0x08010008,0x05020003,0x05010004,0x03030003,
0x0f020081,0x09020009,0x04010003,0x06030004,0x09010010,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f0200c1,0x05020003,0x0a03000a,0x0a010020,0x05020004,0x05010006,0x07040005,
0x0f010180,0x0a020011,0x05020004,0x07030006,0x0b010040,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x08020004,0x100300a2,0x08010009,0x06030004,0x06020005,0x0b04000b,
0x10010280,0x0b020021,0x08020005,0x06030005,0x09010011,0x06020007,0x06020006,0x08050006,
0x00000000,0x10020141,0x06030004,0x0b030012,0x0a010021,0x06030005,0x06020007,0x08040007,
0x00000000,0x0c020041,0x06030005,0x07030007,0x0b010041,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0c02000a,0x02020002,0x0801000a,0x09030005,0x09020006,0x03030003,
0x00000000,0x0902000a,0x0c02000b,0x07040005,0x09010012,0x07030006,0x09020007,0x0c05000c,
0x00000000,0x01010001,0x09030005,0x0c030022,0x0a010022,0x09030006,0x09020008,0x07040006,
0x00000000,0x0a020012,0x09030006,0x07030008,0x0b010042,0x07030007,0x09020009,0x09060007,
0x00000000,0x01010001,0x09020006,0x02020002,0x0801000b,0x07040005,0x07030006,0x0c040013,
0x00000000,0x0b020022,0x09020007,0x07040006,0x09010013,0x07030008,0x07030007,0x09050008,
0x00000000,0x01010001,0x07040005,0x0d030042,0x0a010023,0x07040006,0x07030008,0x08040008,
0x00000000,0x0c020042,0x07040006,0x07030009,0x0b010043,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0801000c,0x0d03000b,0x0d02000c,0x03030003,
0x0f020082,0x0902000b,0x04010003,0x0a040006,0x09010014,0x0a030007,0x0d02000d,0x04040004,
0x00000000,0x01010001,0x0d03000b,0x0a03000b,0x0a010024,0x0d03000c,0x0d02000e,0x08050006,
0x0f010181,0x0a020013,0x0d03000c,0x08040007,0x0b010044,0x0a030008,0x0d02000f,0x0d06000d,
0x00000000,0x01010001,0x08020005,0x02020002,0x0801000d,0x0a040006,0x0a030007,0x0d040023,
0x10010281,0x0b020023,0x08020006,0x0a040007,0x09010015,0x0a030009,0x0a030008,0x08050007,
0x00000000,0x01010001,0x0a040006,0x0b030013,0x0a010025,0x0a040007,0x0a030009,0x08040009,
0x00000000,0x0c020043,0x0a040007,0x08040008,0x0b010045,0x0a03000a,0x0a03000a,0x0a070008,
0x00000000,0x01010001,0x0d020012,0x02020002,0x0801000e,0x0a030007,0x0a020008,0x03030003,
0x00000000,0x0902000c,0x0d020013,0x08050006,0x09010016,0x08040007,0x0a020009,0x0d050014,
0x00000000,0x01010001,0x0a030007,0x0c030023,0x0a010026,0x0a030008,0x0a02000a,0x08050007,
0x00000000,0x0a020014,0x0a030008,0x08040009,0x0b010046,0x08040008,0x0a02000b,0x0a060009,
0x00000000,0x01010001,0x09020007,0x02020002,0x0801000f,0x08050006,0x08040007,0x0f050044,
0x00000000,0x0b020024,0x09020008,0x08050007,0x09010017,0x08040009,0x08040008,0x09050009,
0x00000000,0x01010001,0x08050006,0x0d030043,0x0a010027,0x08050007,0x08040009,0x0804000a,
0x00000000,0x0c020044,0x08050007,0x0804000a,0x0b010047,0x0804000a,0x0804000a,0x08080008,
0x00000000,0x01010001,0x04010002,0x02020002,0x09020009,0x05020003,0x05010004,0x03030003,
0x0f020083,0x0902000d,0x04010003,0x0f05000d,0x09010018,0x0f04000e,0x05010005,0x04040004,
0x00000000,0x0f0200c2,0x05020003,0x0a03000c,0x0a010028,0x05020004,0x05010006,0x0b050007,
0x0f010182,0x0a020015,0x05020004,0x0b040008,0x0b010048,0x0f04000f,0x05010007,0x05050005,
0x00000000,0x01010001,0x09030005,0x02020002,0x0902000a,0x0f05000d,0x0f04000e,0x0b04000c,
0x10010282,0x0b020025,0x09030006,0x0f05000e,0x09010019,0x0f040010,0x0f04000f,0x09060007,
0x00000000,0x10020142,0x0f05000d,0x0b030014,0x0a010029,0x0f05000e,0x0f040010,0x09050008,
0x00000000,0x0c020045,0x0f05000e,0x0b040009,0x0b010049,0x0f040011,0x0f040011,0x0f08000f,
0x00000000,0x01010001,0x0f030023,0x02020002,0x0902000b,0x09030006,0x09020007,0x03030003,
0x00000000,0x0902000e,0x0f030024,0x0b050007,0x0901001a,0x0b040008,0x09020008,0x0f060025,
0x00000000,0x01010001,0x09030006,0x0c030024,0x0a01002a,0x09030007,0x09020009,0x0b050008,
0x00000000,0x0a020016,0x09030007,0x0b04000a,0x0b01004a,0x0b040009,0x0902000a,0x09060008,
0x00000000,0x01010001,0x09020008,0x02020002,0x0902000c,0x0b050007,0x0b040008,0x0c040014,
0x00000000,0x0b020026,0x09020009,0x0b050008,0x0901001b,0x0b04000a,0x0b040009,0x0905000a,
0x00000000,0x01010001,0x0b050007,0x0d030044,0x0a01002b,0x0b050008,0x0b04000a,0x09050009,
0x00000000,0x0c020046,0x0b050008,0x0b04000b,0x0b01004b,0x0b04000b,0x0b04000b,0x0b080009,
0x00000000,0x01010001,0x04010002,0x02020002,0x0902000d,0x0f040014,0x0f030015,0x03030003,
0x0f020084,0x0902000f,0x04010003,0x0b040008,0x0901001c,0x0b030009,0x0f030016,0x04040004,
0x00000000,0x01010001,0x0f040014,0x0a03000d,0x0a01002c,0x0f040015,0x0f030017,0x09060007,
0x0f010183,0x0a020017,0x0f040015,0x09050008,0x0b01004c,0x0b03000a,0x0f030018,0x0f070016,
0x00000000,0x01010001,0x09030006,0x02020002,0x0902000e,0x0b040008,0x0b030009,0x0d040024,
0x10010283,0x0b020027,0x09030007,0x0b040009,0x0901001d,0x0b03000b,0x0b03000a,0x09060008,
0x00000000,0x01010001,0x0b040008,0x0b030015,0x0a01002d,0x0b040009,0x0b03000b,0x0905000a,
0x00000000,0x0c020047,0x0b040009,0x09050009,0x0b01004d,0x0b03000c,0x0b03000c,0x0b07000a,
0x00000000,0x01010001,0x0f020062,0x02020002,0x0902000f,0x0a030008,0x0a020009,0x03030003,
0x00000000,0x09020010,0x0f020063,0x09060007,0x0901001e,0x09050008,0x0a02000a,0x0f050064,
0x00000000,0x01010001,0x0a030008,0x0c030025,0x0a01002e,0x0a030009,0x0a02000b,0x09060008,
0x00000000,0x0a020018,0x0a030009,0x0905000a,0x0b01004e,0x09050009,0x0a02000c,0x0a06000a,
0x00000000,0x01010001,0x09020009,0x02020002,0x09020010,0x09060007,0x09050008,0x0f050045,
0x00000000,0x0b020028,0x0902000a,0x09060008,0x0901001f,0x0905000a,0x09050009,0x0905000b,
0x00000000,0x01010001,0x09060007,0x0d030045,0x0a01002f,0x09060008,0x0905000a,0x0905000b,
0x00000000,0x0c020048,0x09060008,0x0905000b,0x0b01004f,0x0905000b,0x0905000b,0x09090009,
0x00000000,0x01010001,0x04010002,0x02020002,0x0c02000a,0x05020003,0x05010004,0x03030003,
0x0f020085,0x0a03000a,0x04010003,0x06030004,0x0a020011,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f0200c3,0x05020003,0x0a03000e,0x0a010030,0x05020004,0x05010006,0x0f050011,
0x0f010184,0x0a020019,0x05020004,0x0f040012,0x0b010050,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0c030006,0x100300a3,0x0c02000b,0x06030004,0x06020005,0x0b04000d,
0x10010284,0x0b020029,0x0c030007,0x06030005,0x0a020012,0x06020007,0x06020006,0x0c060008,
0x00000000,0x10020143,0x06030004,0x0b030016,0x0a010031,0x06030005,0x06020007,0x0c050009,
0x00000000,0x0c020049,0x06030005,0x0f040013,0x0b010051,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0c02000b,0x02020002,0x0c02000c,0x0a040006,0x0a030007,0x03030003,
0x00000000,0x0a03000b,0x0c02000c,0x0f050011,0x0a020013,0x0f040012,0x0a030008,0x0c05000d,
0x00000000,0x01010001,0x0a040006,0x0c030026,0x0a010032,0x0a040007,0x0a030009,0x0f050012,
0x00000000,0x0a02001a,0x0a040007,0x0f040014,0x0b010052,0x0f040013,0x0a03000a,0x0a070008,
0x00000000,0x01010001,0x0a030007,0x02020002,0x0c02000d,0x0f050011,0x0f040012,0x0c040015,
0x00000000,0x0b02002a,0x0a030008,0x0f050012,0x0a020014,0x0f040014,0x0f040013,0x0a060009,
0x00000000,0x01010001,0x0f050011,0x0d030046,0x0a010033,0x0f050012,0x0f040014,0x0c05000a,
0x00000000,0x0c02004a,0x0f050012,0x0f040015,0x0b010053,0x0f040015,0x0f040015,0x0f080013,
0x00000000,0x01010001,0x04010002,0x02020002,0x0c02000e,0x0f030033,0x0f020034,0x03030003,
0x0f020086,0x0a03000c,0x04010003,0x0a040007,0x0a020015,0x0a030008,0x0f020035,0x04040004,
0x00000000,0x01010001,0x0f030033,0x0a03000f,0x0a010034,0x0f030034,0x0f020036,0x0c060008,
0x0f010185,0x0a02001b,0x0f030034,0x0c050009,0x0b010054,0x0a030009,0x0f020037,0x0f060035,
0x00000000,0x01010001,0x0c030007,0x02020002,0x0c02000f,0x0a040007,0x0a030008,0x0d040025,
0x10010285,0x0b02002b,0x0c030008,0x0a040008,0x0a020016,0x0a03000a,0x0a030009,0x0c060009,
0x00000000,0x01010001,0x0a040007,0x0b030017,0x0a010035,0x0a040008,0x0a03000a,0x0c05000b,
0x00000000,0x0c02004b,0x0a040008,0x0c05000a,0x0b010055,0x0a03000b,0x0a03000b,0x0a070009,
0x00000000,0x01010001,0x0d020013,0x02020002,0x0c020010,0x0a030009,0x0a02000a,0x03030003,
0x00000000,0x0a03000d,0x0d020014,0x0c060008,0x0a020017,0x0c050009,0x0a02000b,0x0d050015,
0x00000000,0x01010001,0x0a030009,0x0c030027,0x0a010036,0x0a03000a,0x0a02000c,0x0c060009,
0x00000000,0x0a02001c,0x0a03000a,0x0c05000b,0x0b010056,0x0c05000a,0x0a02000d,0x0a06000b,
0x00000000,0x01010001,0x0a030008,0x02020002,0x0c020011,0x0c060008,0x0c050009,0x0f050046,
0x00000000,0x0b02002c,0x0a030009,0x0c060009,0x0a020018,0x0c05000b,0x0c05000a,0x0a06000a,
0x00000000,0x01010001,0x0c060008,0x0d030047,0x0a010037,0x0c060009,0x0c05000b,0x0c05000c,
0x00000000,0x0c02004c,0x0c060009,0x0c05000c,0x0b010057,0x0c05000c,0x0c05000c,0x0c09000a,
0x00000000,0x01010001,0x04010002,0x02020002,0x0a03000a,0x05020003,0x05010004,0x03030003,
0x0f020087,0x0a03000e,0x04010003,0x0f04001c,0x0a020019,0x0f03001d,0x05010005,0x04040004,
0x00000000,0x0f0200c4,0x05020003,0x0a030010,0x0a010038,0x05020004,0x05010006,0x0c050009,
0x0f010186,0x0a02001d,0x05020004,0x0c04000a,0x0b010058,0x0f03001e,0x05010007,0x05050005,
0x00000000,0x01010001,0x0a040006,0x02020002,0x0a03000b,0x0f04001c,0x0f03001d,0x0b04000e,
0x10010286,0x0b02002d,0x0a040007,0x0f04001d,0x0a02001a,0x0f03001f,0x0f03001e,0x0a070008,
0x00000000,0x10020144,0x0f04001c,0x0b030018,0x0a010039,0x0f04001d,0x0f03001f,0x0a060009,
0x00000000,0x0c02004d,0x0f04001d,0x0c04000b,0x0b010059,0x0f030020,0x0f030020,0x0f07001e,
0x00000000,0x01010001,0x0f030024,0x02020002,0x0a03000c,0x0a040007,0x0a030008,0x03030003,
0x00000000,0x0a03000f,0x0f030025,0x0c050009,0x0a02001b,0x0c04000a,0x0a030009,0x0f060026,
0x00000000,0x01010001,0x0a040007,0x0c030028,0x0a01003a,0x0a040008,0x0a03000a,0x0c05000a,
0x00000000,0x0a02001e,0x0a040008,0x0c04000c,0x0b01005a,0x0c04000b,0x0a03000b,0x0a070009,
0x00000000,0x01010001,0x0a030009,0x02020002,0x0a03000d,0x0c050009,0x0c04000a,0x0c040016,
0x00000000,0x0b02002e,0x0a03000a,0x0c05000a,0x0a02001c,0x0c04000c,0x0c04000b,0x0a06000b,
0x00000000,0x01010001,0x0c050009,0x0d030048,0x0a01003b,0x0c05000a,0x0c04000c,0x0a06000a,
0x00000000,0x0c02004e,0x0c05000a,0x0c04000d,0x0b01005b,0x0c04000d,0x0c04000d,0x0c08000b,
0x00000000,0x01010001,0x04010002,0x02020002,0x0a03000e,0x10030053,0x10020054,0x03030003,
0x0f020088,0x0a030010,0x04010003,0x0b040009,0x0a02001d,0x0b03000a,0x10020055,0x04040004,
0x00000000,0x01010001,0x10030053,0x0a030011,0x0a01003c,0x10030054,0x10020056,0x0a070008,
0x0f010187,0x0a02001f,0x10030054,0x0a060009,0x0b01005c,0x0b03000b,0x10020057,0x10060055,
0x00000000,0x01010001,0x0a040007,0x02020002,0x0a03000f,0x0b040009,0x0b03000a,0x0d040026,
0x10010287,0x0b02002f,0x0a040008,0x0b04000a,0x0a02001e,0x0b03000c,0x0b03000b,0x0a070009,
0x00000000,0x01010001,0x0b040009,0x0b030019,0x0a01003d,0x0b04000a,0x0b03000c,0x0a06000b,
0x00000000,0x0c02004f,0x0b04000a,0x0a06000a,0x0b01005d,0x0b03000d,0x0b03000d,0x0b07000b,
0x00000000,0x01010001,0x0f020063,0x02020002,0x0a030010,0x0a03000a,0x0a02000b,0x03030003,
0x00000000,0x0a030011,0x0f020064,0x0a070008,0x0a02001f,0x0a060009,0x0a02000c,0x0f050065,
0x00000000,0x01010001,0x0a03000a,0x0c030029,0x0a01003e,0x0a03000b,0x0a02000d,0x0a070009,
0x00000000,0x0a020020,0x0a03000b,0x0a06000b,0x0b01005e,0x0a06000a,0x0a02000e,0x0a06000c,
0x00000000,0x01010001,0x0a03000a,0x02020002,0x0a030011,0x0a070008,0x0a060009,0x0f050047,
0x00000000,0x0b020030,0x0a03000b,0x0a070009,0x0a020020,0x0a06000b,0x0a06000a,0x0a06000c,
0x00000000,0x01010001,0x0a070008,0x0d030049,0x0a01003f,0x0a070009,0x0a06000b,0x0a06000c,
0x00000000,0x0c020050,0x0a070009,0x0a06000c,0x0b01005f,0x0a06000c,0x0a06000c,0x0a0a000a,
0x00000000,0x01010001,0x04010002,0x02020002,0x10020012,0x05020003,0x05010004,0x03030003,
0x0f020089,0x0d03000b,0x04010003,0x06030004,0x0d020012,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f0200c5,0x05020003,0x0b04000b,0x0b020021,0x05020004,0x05010006,0x07040005,
0x0f010188,0x0b030012,0x05020004,0x07030006,0x0b010060,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x1003000e,0x100300a4,0x10020013,0x06030004,0x06020005,0x0b04000f,
0x10010288,0x0b020031,0x1003000f,0x06030005,0x0d020013,0x06020007,0x06020006,0x10060010,
0x00000000,0x10020145,0x06030004,0x0b03001a,0x0b020022,0x06030005,0x06020007,0x10050011,
0x00000000,0x0c020051,0x06030005,0x07030007,0x0b010061,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0c02000c,0x02020002,0x10020014,0x0d040007,0x0d030008,0x03030003,
0x00000000,0x0d03000c,0x0c02000d,0x07040005,0x0d020014,0x07030006,0x0d030009,0x0c05000e,
0x00000000,0x01010001,0x0d040007,0x0c03002a,0x0b020023,0x0d040008,0x0d03000a,0x07040006,
0x00000000,0x0b030013,0x0d040008,0x07030008,0x0b010062,0x07030007,0x0d03000b,0x0d070009,
0x00000000,0x01010001,0x0d030008,0x02020002,0x10020015,0x07040005,0x07030006,0x0c040017,
0x00000000,0x0b020032,0x0d030009,0x07040006,0x0d020015,0x07030008,0x07030007,0x0d06000a,
0x00000000,0x01010001,0x07040005,0x0d03004a,0x0b020024,0x07040006,0x07030008,0x10050012,
0x00000000,0x0c020052,0x07040006,0x07030009,0x0b010063,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x10020016,0x0d03000c,0x0d02000d,0x03030003,
0x0f02008a,0x0d03000d,0x04010003,0x0b050007,0x0d020016,0x0b040008,0x0d02000e,0x04040004,
0x00000000,0x01010001,0x0d03000c,0x0b04000c,0x0b020025,0x0d03000d,0x0d02000f,0x10060010,
0x0f010189,0x0b030014,0x0d03000d,0x10050011,0x0b010064,0x0b040009,0x0d020010,0x0d06000e,
0x00000000,0x01010001,0x1003000f,0x02020002,0x10020017,0x0b050007,0x0b040008,0x0d040027,
0x10010289,0x0b020033,0x10030010,0x0b050008,0x0d020017,0x0b04000a,0x0b040009,0x10060011,
0x00000000,0x01010001,0x0b050007,0x0b03001b,0x0b020026,0x0b050008,0x0b04000a,0x10050013,
0x00000000,0x0c020053,0x0b050008,0x10050012,0x0b010065,0x0b04000b,0x0b04000b,0x0b080009,
0x00000000,0x01010001,0x0d020014,0x02020002,0x10020018,0x0b040008,0x0b030009,0x03030003,
0x00000000,0x0d03000e,0x0d020015,0x10060010,0x0d020018,0x10050011,0x0b03000a,0x0d050016,
0x00000000,0x01010001,0x0b040008,0x0c03002b,0x0b020027,0x0b040009,0x0b03000b,0x10060011,
0x00000000,0x0b030015,0x0b040009,0x10050013,0x0b010066,0x10050012,0x0b03000c,0x0b07000a,
0x00000000,0x01010001,0x0d030009,0x02020002,0x10020019,0x10060010,0x10050011,0x0f050048,
0x00000000,0x0b020034,0x0d03000a,0x10060011,0x0d020019,0x10050013,0x10050012,0x0d06000b,
0x00000000,0x01010001,0x10060010,0x0d03004b,0x0b020028,0x10060011,0x10050013,0x10050014,
0x00000000,0x0c020054,0x10060011,0x10050014,0x0b010067,0x10050014,0x10050014,0x10090012,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d03000b,0x05020003,0x05010004,0x03030003,
0x0f02008b,0x0d03000f,0x04010003,0x1004002c,0x0d02001a,0x1003002d,0x05010005,0x04040004,
0x00000000,0x0f0200c6,0x05020003,0x0b04000d,0x0b020029,0x05020004,0x05010006,0x0b050008,
0x0f01018a,0x0b030016,0x05020004,0x0b040009,0x0b010068,0x1003002e,0x05010007,0x05050005,
0x00000000,0x01010001,0x0d040007,0x02020002,0x0d03000c,0x1004002c,0x1003002d,0x0b040010,
0x1001028a,0x0b020035,0x0d040008,0x1004002d,0x0d02001b,0x1003002f,0x1003002e,0x0d070009,
0x00000000,0x10020146,0x1004002c,0x0b03001c,0x0b02002a,0x1004002d,0x1003002f,0x0d06000a,
0x00000000,0x0c020055,0x1004002d,0x0b04000a,0x0b010069,0x10030030,0x10030030,0x1007002e,
0x00000000,0x01010001,0x0f030025,0x02020002,0x0d03000d,0x0d040008,0x0d030009,0x03030003,
0x00000000,0x0d030010,0x0f030026,0x0b050008,0x0d02001c,0x0b040009,0x0d03000a,0x0f060027,
0x00000000,0x01010001,0x0d040008,0x0c03002c,0x0b02002b,0x0d040009,0x0d03000b,0x0b050009,
0x00000000,0x0b030017,0x0d040009,0x0b04000b,0x0b01006a,0x0b04000a,0x0d03000c,0x0d07000a,
0x00000000,0x01010001,0x0d03000a,0x02020002,0x0d03000e,0x0b050008,0x0b040009,0x0c040018,
0x00000000,0x0b020036,0x0d03000b,0x0b050009,0x0d02001d,0x0b04000b,0x0b04000a,0x0d06000c,
0x00000000,0x01010001,0x0b050008,0x0d03004c,0x0b02002c,0x0b050009,0x0b04000b,0x0d06000b,
0x00000000,0x0c020056,0x0b050009,0x0b04000c,0x0b01006b,0x0b04000c,0x0b04000c,0x0b08000a,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d03000f,0x0f040015,0x0f030016,0x03030003,
0x0f02008c,0x0d030011,0x04010003,0x0b04000a,0x0d02001e,0x0b03000b,0x0f030017,0x04040004,
0x00000000,0x01010001,0x0f040015,0x0b04000e,0x0b02002d,0x0f040016,0x0f030018,0x0d070009,
0x0f01018b,0x0b030018,0x0f040016,0x0d06000a,0x0b01006c,0x0b03000c,0x0f030019,0x0f070017,
0x00000000,0x01010001,0x0d040008,0x02020002,0x0d030010,0x0b04000a,0x0b03000b,0x0d040028,
0x1001028b,0x0b020037,0x0d040009,0x0b04000b,0x0d02001f,0x0b03000d,0x0b03000c,0x0d07000a,
0x00000000,0x01010001,0x0b04000a,0x0b03001d,0x0b02002e,0x0b04000b,0x0b03000d,0x0d06000c,
0x00000000,0x0c020057,0x0b04000b,0x0d06000b,0x0b01006d,0x0b03000e,0x0b03000e,0x0b07000c,
0x00000000,0x01010001,0x0f020064,0x02020002,0x0d030011,0x0b040009,0x0b03000a,0x03030003,
0x00000000,0x0d030012,0x0f020065,0x0d070009,0x0d020020,0x0d06000a,0x0b03000b,0x0f050066,
0x00000000,0x01010001,0x0b040009,0x0c03002d,0x0b02002f,0x0b04000a,0x0b03000c,0x0d07000a,
0x00000000,0x0b030019,0x0b04000a,0x0d06000c,0x0b01006e,0x0d06000b,0x0b03000d,0x0b07000b,
0x00000000,0x01010001,0x0d03000b,0x02020002,0x0d030012,0x0d070009,0x0d06000a,0x0f050049,
0x00000000,0x0b020038,0x0d03000c,0x0d07000a,0x0d020021,0x0d06000c,0x0d06000b,0x0d06000d,
0x00000000,0x01010001,0x0d070009,0x0d03004d,0x0b020030,0x0d07000a,0x0d06000c,0x0d06000d,
0x00000000,0x0c020058,0x0d07000a,0x0d06000d,0x0b01006f,0x0d06000d,0x0d06000d,0x0d0a000b,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d02000c,0x05020003,0x05010004,0x03030003,
0x0f02008d,0x0b04000b,0x04010003,0x06030004,0x0b030012,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f0200c7,0x05020003,0x0b04000f,0x0b020031,0x05020004,0x05010006,0x10050019,
0x0f01018c,0x0b03001a,0x05020004,0x1004001a,0x0b010070,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0d030008,0x100300a5,0x0d02000d,0x06030004,0x06020005,0x0b040011,
0x1001028c,0x0b020039,0x0d030009,0x06030005,0x0b030013,0x06020007,0x06020006,0x0d06000a,
0x00000000,0x10020147,0x06030004,0x0b03001e,0x0b020032,0x06030005,0x06020007,0x0d05000b,
0x00000000,0x0c020059,0x06030005,0x1004001b,0x0b010071,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0c02000d,0x02020002,0x0d02000e,0x0b050007,0x0b040008,0x03030003,
0x00000000,0x0b04000c,0x0c02000e,0x10050019,0x0b030014,0x1004001a,0x0b040009,0x0c05000f,
0x00000000,0x01010001,0x0b050007,0x0c03002e,0x0b020033,0x0b050008,0x0b04000a,0x1005001a,
0x00000000,0x0b03001b,0x0b050008,0x1004001c,0x0b010072,0x1004001b,0x0b04000b,0x0b080009,
0x00000000,0x01010001,0x0b040008,0x02020002,0x0d02000f,0x10050019,0x1004001a,0x0c040019,
0x00000000,0x0b02003a,0x0b040009,0x1005001a,0x0b030015,0x1004001c,0x1004001b,0x0b07000a,
0x00000000,0x01010001,0x10050019,0x0d03004e,0x0b020034,0x1005001a,0x1004001c,0x0d05000c,
0x00000000,0x0c02005a,0x1005001a,0x1004001d,0x0b010073,0x1004001d,0x1004001d,0x1008001b,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d020010,0x0f030034,0x0f020035,0x03030003,
0x0f02008e,0x0b04000d,0x04010003,0x0b050008,0x0b030016,0x0b040009,0x0f020036,0x04040004,
0x00000000,0x01010001,0x0f030034,0x0b040010,0x0b020035,0x0f030035,0x0f020037,0x0d06000a,
0x0f01018d,0x0b03001c,0x0f030035,0x0d05000b,0x0b010074,0x0b04000a,0x0f020038,0x0f060036,
0x00000000,0x01010001,0x0d030009,0x02020002,0x0d020011,0x0b050008,0x0b040009,0x0d040029,
0x1001028d,0x0b02003b,0x0d03000a,0x0b050009,0x0b030017,0x0b04000b,0x0b04000a,0x0d06000b,
0x00000000,0x01010001,0x0b050008,0x0b03001f,0x0b020036,0x0b050009,0x0b04000b,0x0d05000d,
0x00000000,0x0c02005b,0x0b050009,0x0d05000c,0x0b010075,0x0b04000c,0x0b04000c,0x0b08000a,
0x00000000,0x01010001,0x0d020015,0x02020002,0x0d020012,0x0b04000a,0x0b03000b,0x03030003,
0x00000000,0x0b04000e,0x0d020016,0x0d06000a,0x0b030018,0x0d05000b,0x0b03000c,0x0d050017,
0x00000000,0x01010001,0x0b04000a,0x0c03002f,0x0b020037,0x0b04000b,0x0b03000d,0x0d06000b,
0x00000000,0x0b03001d,0x0b04000b,0x0d05000d,0x0b010076,0x0d05000c,0x0b03000e,0x0b07000c,
0x00000000,0x01010001,0x0b040009,0x02020002,0x0d020013,0x0d06000a,0x0d05000b,0x0f05004a,
0x00000000,0x0b02003c,0x0b04000a,0x0d06000b,0x0b030019,0x0d05000d,0x0d05000c,0x0b07000b,
0x00000000,0x01010001,0x0d06000a,0x0d03004f,0x0b020038,0x0d06000b,0x0d05000d,0x0d05000e,
0x00000000,0x0c02005c,0x0d06000b,0x0d05000e,0x0b010077,0x0d05000e,0x0d05000e,0x0d09000c,
0x00000000,0x01010001,0x04010002,0x02020002,0x0b04000b,0x05020003,0x05010004,0x03030003,
0x0f02008f,0x0b04000f,0x04010003,0x06030004,0x0b03001a,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f0200c8,0x05020003,0x0b040011,0x0b020039,0x05020004,0x05010006,0x0c05000a,
0x0f01018e,0x0b03001e,0x05020004,0x0c04000b,0x0b010078,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0b050007,0x02020002,0x0b04000c,0x06030004,0x06020005,0x0b040012,
0x1001028e,0x0b02003d,0x0b050008,0x06030005,0x0b03001b,0x06020007,0x06020006,0x0b080009,
0x00000000,0x10020148,0x06030004,0x0b030020,0x0b02003a,0x06030005,0x06020007,0x0b07000a,
0x00000000,0x0c02005d,0x06030005,0x0c04000c,0x0b010079,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0f030026,0x02020002,0x0b04000d,0x0b050008,0x0b040009,0x03030003,
0x00000000,0x0b040010,0x0f030027,0x0c05000a,0x0b03001c,0x0c04000b,0x0b04000a,0x0f060028,
0x00000000,0x01010001,0x0b050008,0x0c030030,0x0b02003b,0x0b050009,0x0b04000b,0x0c05000b,
0x00000000,0x0b03001f,0x0b050009,0x0c04000d,0x0b01007a,0x0c04000c,0x0b04000c,0x0b08000a,
0x00000000,0x01010001,0x0b04000a,0x02020002,0x0b04000e,0x0c05000a,0x0c04000b,0x0c04001a,
0x00000000,0x0b02003e,0x0b04000b,0x0c05000b,0x0b03001d,0x0c04000d,0x0c04000c,0x0b07000c,
0x00000000,0x01010001,0x0c05000a,0x0d030050,0x0b02003c,0x0c05000b,0x0c04000d,0x0b07000b,
0x00000000,0x0c02005e,0x0c05000b,0x0c04000e,0x0b01007b,0x0c04000e,0x0c04000e,0x0c08000c,
0x00000000,0x01010001,0x04010002,0x02020002,0x0b04000f,0x10030054,0x10020055,0x03030003,
0x0f020090,0x0b040011,0x04010003,0x0b04000b,0x0b03001e,0x0b03000c,0x10020056,0x04040004,
0x00000000,0x01010001,0x10030054,0x0b040012,0x0b02003d,0x10030055,0x10020057,0x0b080009,
0x0f01018f,0x0b030020,0x10030055,0x0b07000a,0x0b01007c,0x0b03000d,0x10020058,0x10060056,
0x00000000,0x01010001,0x0b050008,0x02020002,0x0b040010,0x0b04000b,0x0b03000c,0x0d04002a,
0x1001028f,0x0b02003f,0x0b050009,0x0b04000c,0x0b03001f,0x0b03000e,0x0b03000d,0x0b08000a,
0x00000000,0x01010001,0x0b04000b,0x0b030021,0x0b02003e,0x0b04000c,0x0b03000e,0x0b07000c,
0x00000000,0x0c02005f,0x0b04000c,0x0b07000b,0x0b01007d,0x0b03000f,0x0b03000f,0x0b07000d,
0x00000000,0x01010001,0x0f020065,0x02020002,0x0b040011,0x0b04000b,0x0b03000c,0x03030003,
0x00000000,0x0b040012,0x0f020066,0x0b080009,0x0b030020,0x0b07000a,0x0b03000d,0x0f050067,
0x00000000,0x01010001,0x0b04000b,0x0c030031,0x0b02003f,0x0b04000c,0x0b03000e,0x0b08000a,
0x00000000,0x0b030021,0x0b04000c,0x0b07000c,0x0b01007e,0x0b07000b,0x0b03000f,0x0b07000d,
0x00000000,0x01010001,0x0b04000b,0x02020002,0x0b040012,0x0b080009,0x0b07000a,0x0f05004b,
0x00000000,0x0b020040,0x0b04000c,0x0b08000a,0x0b030021,0x0b07000c,0x0b07000b,0x0b07000d,
0x00000000,0x01010001,0x0b080009,0x0d030051,0x0b020040,0x0b08000a,0x0b07000c,0x0b07000d,
0x00000000,0x0c020060,0x0b08000a,0x0b07000d,0x0b01007f,0x0b07000d,0x0b07000d,0x0b0b000b,
0x00000000,0x01010001,0x04010002,0x02020002,0x08010008,0x05020003,0x05010004,0x03030003,
0x0f020091,0x09020009,0x04010003,0x06030004,0x09010010,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f0200c9,0x05020003,0x0f05000d,0x0f030023,0x05020004,0x05010006,0x07040005,
0x0f010190,0x0f040014,0x05020004,0x07030006,0x0c020041,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x08020004,0x100300a6,0x08010009,0x06030004,0x06020005,0x0c05000c,
0x10010290,0x0c030022,0x08020005,0x06030005,0x09010011,0x06020007,0x06020006,0x08050006,
0x00000000,0x10020149,0x06030004,0x0c040013,0x0f030024,0x06030005,0x06020007,0x08040007,
0x00000000,0x0c020061,0x06030005,0x07030007,0x0c020042,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0c02000e,0x02020002,0x0801000a,0x09030005,0x09020006,0x03030003,
0x00000000,0x0902000a,0x0c02000f,0x07040005,0x09010012,0x07030006,0x09020007,0x0c050010,
0x00000000,0x01010001,0x09030005,0x0c030032,0x0f030025,0x09030006,0x09020008,0x07040006,
0x00000000,0x0f040015,0x09030006,0x07030008,0x0c020043,0x07030007,0x09020009,0x09060007,
0x00000000,0x01010001,0x09020006,0x02020002,0x0801000b,0x07040005,0x07030006,0x0c04001b,
0x00000000,0x0c030023,0x09020007,0x07040006,0x09010013,0x07030008,0x07030007,0x09050008,
0x00000000,0x01010001,0x07040005,0x0d030052,0x0f030026,0x07040006,0x07030008,0x08040008,
0x00000000,0x0c020062,0x07040006,0x07030009,0x0c020044,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0801000c,0x0d03000d,0x0d02000e,0x03030003,
0x0f020092,0x0902000b,0x04010003,0x0f060009,0x09010014,0x0f05000a,0x0d02000f,0x04040004,
0x00000000,0x01010001,0x0d03000d,0x0f05000e,0x0f030027,0x0d03000e,0x0d020010,0x08050006,
0x0f010191,0x0f040016,0x0d03000e,0x08040007,0x0c020045,0x0f05000b,0x0d020011,0x0d06000f,
0x00000000,0x01010001,0x08020005,0x02020002,0x0801000d,0x0f060009,0x0f05000a,0x0d04002b,
0x10010291,0x0c030024,0x08020006,0x0f06000a,0x09010015,0x0f05000c,0x0f05000b,0x08050007,
0x00000000,0x01010001,0x0f060009,0x0c040014,0x0f030028,0x0f06000a,0x0f05000c,0x08040009,
0x00000000,0x0c020063,0x0f06000a,0x08040008,0x0c020046,0x0f05000d,0x0f05000d,0x0f09000b,
0x00000000,0x01010001,0x0d020016,0x02020002,0x0801000e,0x0f05000a,0x0f04000b,0x03030003,
0x00000000,0x0902000c,0x0d020017,0x08050006,0x09010016,0x08040007,0x0f04000c,0x0d050018,
0x00000000,0x01010001,0x0f05000a,0x0c030033,0x0f030029,0x0f05000b,0x0f04000d,0x08050007,
0x00000000,0x0f040017,0x0f05000b,0x08040009,0x0c020047,0x08040008,0x0f04000e,0x0f08000c,
0x00000000,0x01010001,0x09020007,0x02020002,0x0801000f,0x08050006,0x08040007,0x0f05004c,
0x00000000,0x0c030025,0x09020008,0x08050007,0x09010017,0x08040009,0x08040008,0x09050009,
0x00000000,0x01010001,0x08050006,0x0d030053,0x0f03002a,0x08050007,0x08040009,0x0804000a,
0x00000000,0x0c020064,0x08050007,0x0804000a,0x0c020048,0x0804000a,0x0804000a,0x08080008,
0x00000000,0x01010001,0x04010002,0x02020002,0x09020009,0x05020003,0x05010004,0x03030003,
0x0f020093,0x0902000d,0x04010003,0x0f05000e,0x09010018,0x0f04000f,0x05010005,0x04040004,
0x00000000,0x0f0200ca,0x05020003,0x0f05000f,0x0f03002b,0x05020004,0x05010006,0x0c060008,
0x0f010192,0x0f040018,0x05020004,0x0c050009,0x0c020049,0x0f040010,0x05010007,0x05050005,
0x00000000,0x01010001,0x09030005,0x02020002,0x0902000a,0x0f05000e,0x0f04000f,0x0c05000d,
0x10010292,0x0c030026,0x09030006,0x0f05000f,0x09010019,0x0f040011,0x0f040010,0x09060007,
0x00000000,0x1002014a,0x0f05000e,0x0c040015,0x0f03002c,0x0f05000f,0x0f040011,0x09050008,
0x00000000,0x0c020065,0x0f05000f,0x0c05000a,0x0c02004a,0x0f040012,0x0f040012,0x0f080010,
0x00000000,0x01010001,0x0f030027,0x02020002,0x0902000b,0x09030006,0x09020007,0x03030003,
0x00000000,0x0902000e,0x0f030028,0x0c060008,0x0901001a,0x0c050009,0x09020008,0x0f060029,
0x00000000,0x01010001,0x09030006,0x0c030034,0x0f03002d,0x09030007,0x09020009,0x0c060009,
0x00000000,0x0f040019,0x09030007,0x0c05000b,0x0c02004b,0x0c05000a,0x0902000a,0x09060008,
0x00000000,0x01010001,0x09020008,0x02020002,0x0902000c,0x0c060008,0x0c050009,0x0c04001c,
0x00000000,0x0c030027,0x09020009,0x0c060009,0x0901001b,0x0c05000b,0x0c05000a,0x0905000a,
0x00000000,0x01010001,0x0c060008,0x0d030054,0x0f03002e,0x0c060009,0x0c05000b,0x09050009,
0x00000000,0x0c020066,0x0c060009,0x0c05000c,0x0c02004c,0x0c05000c,0x0c05000c,0x0c09000a,
0x00000000,0x01010001,0x04010002,0x02020002,0x0902000d,0x0f040016,0x0f030017,0x03030003,
0x0f020094,0x0902000f,0x04010003,0x0c050009,0x0901001c,0x0c04000a,0x0f030018,0x04040004,
0x00000000,0x01010001,0x0f040016,0x0f050010,0x0f03002f,0x0f040017,0x0f030019,0x09060007,
0x0f010193,0x0f04001a,0x0f040017,0x09050008,0x0c02004d,0x0c04000b,0x0f03001a,0x0f070018,
0x00000000,0x01010001,0x09030006,0x02020002,0x0902000e,0x0c050009,0x0c04000a,0x0d04002c,
0x10010293,0x0c030028,0x09030007,0x0c05000a,0x0901001d,0x0c04000c,0x0c04000b,0x09060008,
0x00000000,0x01010001,0x0c050009,0x0c040016,0x0f030030,0x0c05000a,0x0c04000c,0x0905000a,
0x00000000,0x0c020067,0x0c05000a,0x09050009,0x0c02004e,0x0c04000d,0x0c04000d,0x0c08000b,
0x00000000,0x01010001,0x0f020066,0x02020002,0x0902000f,0x0f05000b,0x0f04000c,0x03030003,
0x00000000,0x09020010,0x0f020067,0x09060007,0x0901001e,0x09050008,0x0f04000d,0x0f050068,
0x00000000,0x01010001,0x0f05000b,0x0c030035,0x0f030031,0x0f05000c,0x0f04000e,0x09060008,
0x00000000,0x0f04001b,0x0f05000c,0x0905000a,0x0c02004f,0x09050009,0x0f04000f,0x0f08000d,
0x00000000,0x01010001,0x09020009,0x02020002,0x09020010,0x09060007,0x09050008,0x0f05004d,
0x00000000,0x0c030029,0x0902000a,0x09060008,0x0901001f,0x0905000a,0x09050009,0x0905000b,
0x00000000,0x01010001,0x09060007,0x0d030055,0x0f030032,0x09060008,0x0905000a,0x0905000b,
0x00000000,0x0c020068,0x09060008,0x0905000b,0x0c020050,0x0905000b,0x0905000b,0x09090009,
0x00000000,0x01010001,0x04010002,0x02020002,0x0c02000b,0x05020003,0x05010004,0x03030003,
0x0f020095,0x0f05000d,0x04010003,0x06030004,0x0f040014,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f0200cb,0x05020003,0x0f050011,0x0f030033,0x05020004,0x05010006,0x07040005,
0x0f010194,0x0f04001c,0x05020004,0x07030006,0x0c020051,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0c030007,0x100300a7,0x0c02000c,0x06030004,0x06020005,0x0c05000e,
0x10010294,0x0c03002a,0x0c030008,0x06030005,0x0f040015,0x06020007,0x06020006,0x0c060009,
0x00000000,0x1002014b,0x06030004,0x0c040017,0x0f030034,0x06030005,0x06020007,0x0c05000a,
0x00000000,0x0c020069,0x06030005,0x07030007,0x0c020052,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0c02000f,0x02020002,0x0c02000d,0x0f060009,0x0f05000a,0x03030003,
0x00000000,0x0f05000e,0x0c020010,0x07040005,0x0f040016,0x07030006,0x0f05000b,0x0c050011,
0x00000000,0x01010001,0x0f060009,0x0c030036,0x0f030035,0x0f06000a,0x0f05000c,0x07040006,
0x00000000,0x0f04001d,0x0f06000a,0x07030008,0x0c020053,0x07030007,0x0f05000d,0x0f09000b,
0x00000000,0x01010001,0x0f05000a,0x02020002,0x0c02000e,0x07040005,0x07030006,0x0c04001d,
0x00000000,0x0c03002b,0x0f05000b,0x07040006,0x0f040017,0x07030008,0x07030007,0x0f08000c,
0x00000000,0x01010001,0x07040005,0x0d030056,0x0f030036,0x07040006,0x07030008,0x0c05000b,
0x00000000,0x0c02006a,0x07040006,0x07030009,0x0c020054,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0c02000f,0x0f030035,0x0f020036,0x03030003,
0x0f020096,0x0f05000f,0x04010003,0x0f06000a,0x0f040018,0x0f05000b,0x0f020037,0x04040004,
0x00000000,0x01010001,0x0f030035,0x0f050012,0x0f030037,0x0f030036,0x0f020038,0x0c060009,
0x0f010195,0x0f04001e,0x0f030036,0x0c05000a,0x0c020055,0x0f05000c,0x0f020039,0x0f060037,
0x00000000,0x01010001,0x0c030008,0x02020002,0x0c020010,0x0f06000a,0x0f05000b,0x0d04002d,
0x10010295,0x0c03002c,0x0c030009,0x0f06000b,0x0f040019,0x0f05000d,0x0f05000c,0x0c06000a,
0x00000000,0x01010001,0x0f06000a,0x0c040018,0x0f030038,0x0f06000b,0x0f05000d,0x0c05000c,
0x00000000,0x0c02006b,0x0f06000b,0x0c05000b,0x0c020056,0x0f05000e,0x0f05000e,0x0f09000c,
0x00000000,0x01010001,0x0d020017,0x02020002,0x0c020011,0x0f05000c,0x0f04000d,0x03030003,
0x00000000,0x0f050010,0x0d020018,0x0c060009,0x0f04001a,0x0c05000a,0x0f04000e,0x0d050019,
0x00000000,0x01010001,0x0f05000c,0x0c030037,0x0f030039,0x0f05000d,0x0f04000f,0x0c06000a,
0x00000000,0x0f04001f,0x0f05000d,0x0c05000c,0x0c020057,0x0c05000b,0x0f040010,0x0f08000e,
0x00000000,0x01010001,0x0f05000b,0x02020002,0x0c020012,0x0c060009,0x0c05000a,0x0f05004e,
0x00000000,0x0c03002d,0x0f05000c,0x0c06000a,0x0f04001b,0x0c05000c,0x0c05000b,0x0f08000d,
0x00000000,0x01010001,0x0c060009,0x0d030057,0x0f03003a,0x0c06000a,0x0c05000c,0x0c05000d,
0x00000000,0x0c02006c,0x0c06000a,0x0c05000d,0x0c020058,0x0c05000d,0x0c05000d,0x0c09000b,
0x00000000,0x01010001,0x04010002,0x02020002,0x0f05000d,0x05020003,0x05010004,0x03030003,
0x0f020097,0x0f050011,0x04010003,0x0f04001d,0x0f04001c,0x0f03001e,0x05010005,0x04040004,
0x00000000,0x0f0200cc,0x05020003,0x0f050013,0x0f03003b,0x05020004,0x05010006,0x0c05000b,
0x0f010196,0x0f040020,0x05020004,0x0c04000c,0x0c020059,0x0f03001f,0x05010007,0x05050005,
0x00000000,0x01010001,0x0f060009,0x02020002,0x0f05000e,0x0f04001d,0x0f03001e,0x0c05000f,
0x10010296,0x0c03002e,0x0f06000a,0x0f04001e,0x0f04001d,0x0f030020,0x0f03001f,0x0f09000b,
0x00000000,0x1002014c,0x0f04001d,0x0c040019,0x0f03003c,0x0f04001e,0x0f030020,0x0f08000c,
0x00000000,0x0c02006d,0x0f04001e,0x0c04000d,0x0c02005a,0x0f030021,0x0f030021,0x0f07001f,
0x00000000,0x01010001,0x0f030028,0x02020002,0x0f05000f,0x0f06000a,0x0f05000b,0x03030003,
0x00000000,0x0f050012,0x0f030029,0x0c05000b,0x0f04001e,0x0c04000c,0x0f05000c,0x0f06002a,
0x00000000,0x01010001,0x0f06000a,0x0c030038,0x0f03003d,0x0f06000b,0x0f05000d,0x0c05000c,
0x00000000,0x0f040021,0x0f06000b,0x0c04000e,0x0c02005b,0x0c04000d,0x0f05000e,0x0f09000c,
0x00000000,0x01010001,0x0f05000c,0x02020002,0x0f050010,0x0c05000b,0x0c04000c,0x0c04001e,
0x00000000,0x0c03002f,0x0f05000d,0x0c05000c,0x0f04001f,0x0c04000e,0x0c04000d,0x0f08000e,
0x00000000,0x01010001,0x0c05000b,0x0d030058,0x0f03003e,0x0c05000c,0x0c04000e,0x0f08000d,
0x00000000,0x0c02006e,0x0c05000c,0x0c04000f,0x0c02005c,0x0c04000f,0x0c04000f,0x0c08000d,
0x00000000,0x01010001,0x04010002,0x02020002,0x0f050011,0x10030055,0x10020056,0x03030003,
0x0f020098,0x0f050013,0x04010003,0x0c05000a,0x0f040020,0x0c04000b,0x10020057,0x04040004,
0x00000000,0x01010001,0x10030055,0x0f050014,0x0f03003f,0x10030056,0x10020058,0x0f09000b,
0x0f010197,0x0f040022,0x10030056,0x0f08000c,0x0c02005d,0x0c04000c,0x10020059,0x10060057,
0x00000000,0x01010001,0x0f06000a,0x02020002,0x0f050012,0x0c05000a,0x0c04000b,0x0d04002e,
0x10010297,0x0c030030,0x0f06000b,0x0c05000b,0x0f040021,0x0c04000d,0x0c04000c,0x0f09000c,
0x00000000,0x01010001,0x0c05000a,0x0c04001a,0x0f030040,0x0c05000b,0x0c04000d,0x0f08000e,
0x00000000,0x0c02006f,0x0c05000b,0x0f08000d,0x0c02005e,0x0c04000e,0x0c04000e,0x0c08000c,
0x00000000,0x01010001,0x0f020067,0x02020002,0x0f050013,0x0f05000d,0x0f04000e,0x03030003,
0x00000000,0x0f050014,0x0f020068,0x0f09000b,0x0f040022,0x0f08000c,0x0f04000f,0x0f050069,
0x00000000,0x01010001,0x0f05000d,0x0c030039,0x0f030041,0x0f05000e,0x0f040010,0x0f09000c,
0x00000000,0x0f040023,0x0f05000e,0x0f08000e,0x0c02005f,0x0f08000d,0x0f040011,0x0f08000f,
0x00000000,0x01010001,0x0f05000d,0x02020002,0x0f050014,0x0f09000b,0x0f08000c,0x0f05004f,
0x00000000,0x0c030031,0x0f05000e,0x0f09000c,0x0f040023,0x0f08000e,0x0f08000d,0x0f08000f,
0x00000000,0x01010001,0x0f09000b,0x0d030059,0x0f030042,0x0f09000c,0x0f08000e,0x0f08000f,
0x00000000,0x0c020070,0x0f09000c,0x0f08000f,0x0c020060,0x0f08000f,0x0f08000f,0x0f0c000d,
0x00000000,0x01010001,0x04010002,0x02020002,0x08010008,0x05020003,0x05010004,0x03030003,
0x0f020099,0x0f04000e,0x04010003,0x06030004,0x0f030015,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f0200cd,0x05020003,0x0c05000c,0x0c030022,0x05020004,0x05010006,0x07040005,
0x0f010198,0x0c040013,0x05020004,0x07030006,0x0c020061,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x08020004,0x100300a8,0x08010009,0x06030004,0x06020005,0x0c050010,
0x10010298,0x0c030032,0x08020005,0x06030005,0x0f030016,0x06020007,0x06020006,0x08050006,
0x00000000,0x1002014d,0x06030004,0x0c04001b,0x0c030023,0x06030005,0x06020007,0x08040007,
0x00000000,0x0c020071,0x06030005,0x07030007,0x0c020062,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0c020010,0x02020002,0x0801000a,0x0f05000a,0x0f04000b,0x03030003,
0x00000000,0x0f04000f,0x0c020011,0x07040005,0x0f030017,0x07030006,0x0f04000c,0x0c050012,
0x00000000,0x01010001,0x0f05000a,0x0c03003a,0x0c030024,0x0f05000b,0x0f04000d,0x07040006,
0x00000000,0x0c040014,0x0f05000b,0x07030008,0x0c020063,0x07030007,0x0f04000e,0x0f08000c,
0x00000000,0x01010001,0x0f04000b,0x02020002,0x0801000b,0x07040005,0x07030006,0x0c04001f,
0x00000000,0x0c030033,0x0f04000c,0x07040006,0x0f030018,0x07030008,0x07030007,0x0f07000d,
0x00000000,0x01010001,0x07040005,0x0d03005a,0x0c030025,0x07040006,0x07030008,0x08040008,
0x00000000,0x0c020072,0x07040006,0x07030009,0x0c020064,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0801000c,0x0d03000e,0x0d02000f,0x03030003,
0x0f02009a,0x0f040010,0x04010003,0x0c060008,0x0f030019,0x0c050009,0x0d020010,0x04040004,
0x00000000,0x01010001,0x0d03000e,0x0c05000d,0x0c030026,0x0d03000f,0x0d020011,0x08050006,
0x0f010199,0x0c040015,0x0d03000f,0x08040007,0x0c020065,0x0c05000a,0x0d020012,0x0d060010,
0x00000000,0x01010001,0x08020005,0x02020002,0x0801000d,0x0c060008,0x0c050009,0x0d04002f,
0x10010299,0x0c030034,0x08020006,0x0c060009,0x0f03001a,0x0c05000b,0x0c05000a,0x08050007,
0x00000000,0x01010001,0x0c060008,0x0c04001c,0x0c030027,0x0c060009,0x0c05000b,0x08040009,
0x00000000,0x0c020073,0x0c060009,0x08040008,0x0c020066,0x0c05000c,0x0c05000c,0x0c09000a,
0x00000000,0x01010001,0x0d020018,0x02020002,0x0801000e,0x0c050009,0x0c04000a,0x03030003,
0x00000000,0x0f040011,0x0d020019,0x08050006,0x0f03001b,0x08040007,0x0c04000b,0x0d05001a,
0x00000000,0x01010001,0x0c050009,0x0c03003b,0x0c030028,0x0c05000a,0x0c04000c,0x08050007,
0x00000000,0x0c040016,0x0c05000a,0x08040009,0x0c020067,0x08040008,0x0c04000d,0x0c08000b,
0x00000000,0x01010001,0x0f04000c,0x02020002,0x0801000f,0x08050006,0x08040007,0x0f050050,
0x00000000,0x0c030035,0x0f04000d,0x08050007,0x0f03001c,0x08040009,0x08040008,0x0f07000e,
0x00000000,0x01010001,0x08050006,0x0d03005b,0x0c030029,0x08050007,0x08040009,0x0804000a,
0x00000000,0x0c020074,0x08050007,0x0804000a,0x0c020068,0x0804000a,0x0804000a,0x08080008,
0x00000000,0x01010001,0x04010002,0x02020002,0x0f04000e,0x05020003,0x05010004,0x03030003,
0x0f02009b,0x0f040012,0x04010003,0x1004002d,0x0f03001d,0x1003002e,0x05010005,0x04040004,
0x00000000,0x0f0200ce,0x05020003,0x0c05000e,0x0c03002a,0x05020004,0x05010006,0x0c060009,
0x0f01019a,0x0c040017,0x05020004,0x0c05000a,0x0c020069,0x1003002f,0x05010007,0x05050005,
0x00000000,0x01010001,0x0f05000a,0x02020002,0x0f04000f,0x1004002d,0x1003002e,0x0c050011,
0x1001029a,0x0c030036,0x0f05000b,0x1004002e,0x0f03001e,0x10030030,0x1003002f,0x0f08000c,
0x00000000,0x1002014e,0x1004002d,0x0c04001d,0x0c03002b,0x1004002e,0x10030030,0x0f07000d,
0x00000000,0x0c020075,0x1004002e,0x0c05000b,0x0c02006a,0x10030031,0x10030031,0x1007002f,
0x00000000,0x01010001,0x0f030029,0x02020002,0x0f040010,0x0f05000b,0x0f04000c,0x03030003,
0x00000000,0x0f040013,0x0f03002a,0x0c060009,0x0f03001f,0x0c05000a,0x0f04000d,0x0f06002b,
0x00000000,0x01010001,0x0f05000b,0x0c03003c,0x0c03002c,0x0f05000c,0x0f04000e,0x0c06000a,
0x00000000,0x0c040018,0x0f05000c,0x0c05000c,0x0c02006b,0x0c05000b,0x0f04000f,0x0f08000d,
0x00000000,0x01010001,0x0f04000d,0x02020002,0x0f040011,0x0c060009,0x0c05000a,0x0c040020,
0x00000000,0x0c030037,0x0f04000e,0x0c06000a,0x0f030020,0x0c05000c,0x0c05000b,0x0f07000f,
0x00000000,0x01010001,0x0c060009,0x0d03005c,0x0c03002d,0x0c06000a,0x0c05000c,0x0f07000e,
0x00000000,0x0c020076,0x0c06000a,0x0c05000d,0x0c02006c,0x0c05000d,0x0c05000d,0x0c09000b,
0x00000000,0x01010001,0x04010002,0x02020002,0x0f040012,0x0f040017,0x0f030018,0x03030003,
0x0f02009c,0x0f040014,0x04010003,0x0c05000b,0x0f030021,0x0c04000c,0x0f030019,0x04040004,
0x00000000,0x01010001,0x0f040017,0x0c05000f,0x0c03002e,0x0f040018,0x0f03001a,0x0f08000c,
0x0f01019b,0x0c040019,0x0f040018,0x0f07000d,0x0c02006d,0x0c04000d,0x0f03001b,0x0f070019,
0x00000000,0x01010001,0x0f05000b,0x02020002,0x0f040013,0x0c05000b,0x0c04000c,0x0d040030,
0x1001029b,0x0c030038,0x0f05000c,0x0c05000c,0x0f030022,0x0c04000e,0x0c04000d,0x0f08000d,
0x00000000,0x01010001,0x0c05000b,0x0c04001e,0x0c03002f,0x0c05000c,0x0c04000e,0x0f07000f,
0x00000000,0x0c020077,0x0c05000c,0x0f07000e,0x0c02006e,0x0c04000f,0x0c04000f,0x0c08000d,
0x00000000,0x01010001,0x0f020068,0x02020002,0x0f040014,0x0c05000a,0x0c04000b,0x03030003,
0x00000000,0x0f040015,0x0f020069,0x0f08000c,0x0f030023,0x0f07000d,0x0c04000c,0x0f05006a,
0x00000000,0x01010001,0x0c05000a,0x0c03003d,0x0c030030,0x0c05000b,0x0c04000d,0x0f08000d,
0x00000000,0x0c04001a,0x0c05000b,0x0f07000f,0x0c02006f,0x0f07000e,0x0c04000e,0x0c08000c,
0x00000000,0x01010001,0x0f04000e,0x02020002,0x0f040015,0x0f08000c,0x0f07000d,0x0f050051,
0x00000000,0x0c030039,0x0f04000f,0x0f08000d,0x0f030024,0x0f07000f,0x0f07000e,0x0f070010,
0x00000000,0x01010001,0x0f08000c,0x0d03005d,0x0c030031,0x0f08000d,0x0f07000f,0x0f070010,
0x00000000,0x0c020078,0x0f08000d,0x0f070010,0x0c020070,0x0f070010,0x0f070010,0x0f0b000e,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d02000d,0x05020003,0x05010004,0x03030003,
0x0f02009d,0x0c05000c,0x04010003,0x06030004,0x0c040013,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f0200cf,0x05020003,0x0c050010,0x0c030032,0x05020004,0x05010006,0x07040005,
0x0f01019c,0x0c04001b,0x05020004,0x07030006,0x0c020071,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0d030009,0x100300a9,0x0d02000e,0x06030004,0x06020005,0x0c050012,
0x1001029c,0x0c03003a,0x0d03000a,0x06030005,0x0c040014,0x06020007,0x06020006,0x0d06000b,
0x00000000,0x1002014f,0x06030004,0x0c04001f,0x0c030033,0x06030005,0x06020007,0x0d05000c,
0x00000000,0x0c020079,0x06030005,0x07030007,0x0c020072,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0c020011,0x02020002,0x0d02000f,0x0c060008,0x0c050009,0x03030003,
0x00000000,0x0c05000d,0x0c020012,0x07040005,0x0c040015,0x07030006,0x0c05000a,0x0c050013,
0x00000000,0x01010001,0x0c060008,0x0c03003e,0x0c030034,0x0c060009,0x0c05000b,0x07040006,
0x00000000,0x0c04001c,0x0c060009,0x07030008,0x0c020073,0x07030007,0x0c05000c,0x0c09000a,
0x00000000,0x01010001,0x0c050009,0x02020002,0x0d020010,0x07040005,0x07030006,0x0c040021,
0x00000000,0x0c03003b,0x0c05000a,0x07040006,0x0c040016,0x07030008,0x07030007,0x0c08000b,
0x00000000,0x01010001,0x07040005,0x0d03005e,0x0c030035,0x07040006,0x07030008,0x0d05000d,
0x00000000,0x0c02007a,0x07040006,0x07030009,0x0c020074,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d020011,0x0f030036,0x0f020037,0x03030003,
0x0f02009e,0x0c05000e,0x04010003,0x0c060009,0x0c040017,0x0c05000a,0x0f020038,0x04040004,
0x00000000,0x01010001,0x0f030036,0x0c050011,0x0c030036,0x0f030037,0x0f020039,0x0d06000b,
0x0f01019d,0x0c04001d,0x0f030037,0x0d05000c,0x0c020075,0x0c05000b,0x0f02003a,0x0f060038,
0x00000000,0x01010001,0x0d03000a,0x02020002,0x0d020012,0x0c060009,0x0c05000a,0x0d040031,
0x1001029d,0x0c03003c,0x0d03000b,0x0c06000a,0x0c040018,0x0c05000c,0x0c05000b,0x0d06000c,
0x00000000,0x01010001,0x0c060009,0x0c040020,0x0c030037,0x0c06000a,0x0c05000c,0x0d05000e,
0x00000000,0x0c02007b,0x0c06000a,0x0d05000d,0x0c020076,0x0c05000d,0x0c05000d,0x0c09000b,
0x00000000,0x01010001,0x0d020019,0x02020002,0x0d020013,0x0c05000b,0x0c04000c,0x03030003,
0x00000000,0x0c05000f,0x0d02001a,0x0d06000b,0x0c040019,0x0d05000c,0x0c04000d,0x0d05001b,
0x00000000,0x01010001,0x0c05000b,0x0c03003f,0x0c030038,0x0c05000c,0x0c04000e,0x0d06000c,
0x00000000,0x0c04001e,0x0c05000c,0x0d05000e,0x0c020077,0x0d05000d,0x0c04000f,0x0c08000d,
0x00000000,0x01010001,0x0c05000a,0x02020002,0x0d020014,0x0d06000b,0x0d05000c,0x0f050052,
0x00000000,0x0c03003d,0x0c05000b,0x0d06000c,0x0c04001a,0x0d05000e,0x0d05000d,0x0c08000c,
0x00000000,0x01010001,0x0d06000b,0x0d03005f,0x0c030039,0x0d06000c,0x0d05000e,0x0d05000f,
0x00000000,0x0c02007c,0x0d06000c,0x0d05000f,0x0c020078,0x0d05000f,0x0d05000f,0x0d09000d,
0x00000000,0x01010001,0x04010002,0x02020002,0x0c05000c,0x05020003,0x05010004,0x03030003,
0x0f02009f,0x0c050010,0x04010003,0x06030004,0x0c04001b,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f0200d0,0x05020003,0x0c050012,0x0c03003a,0x05020004,0x05010006,0x0c05000c,
0x0f01019e,0x0c04001f,0x05020004,0x0c04000d,0x0c020079,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0c060008,0x02020002,0x0c05000d,0x06030004,0x06020005,0x0c050013,
0x1001029e,0x0c03003e,0x0c060009,0x06030005,0x0c04001c,0x06020007,0x06020006,0x0c09000a,
0x00000000,0x10020150,0x06030004,0x0c040021,0x0c03003b,0x06030005,0x06020007,0x0c08000b,
0x00000000,0x0c02007d,0x06030005,0x0c04000e,0x0c02007a,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0f03002a,0x02020002,0x0c05000e,0x0c060009,0x0c05000a,0x03030003,
0x00000000,0x0c050011,0x0f03002b,0x0c05000c,0x0c04001d,0x0c04000d,0x0c05000b,0x0f06002c,
0x00000000,0x01010001,0x0c060009,0x0c030040,0x0c03003c,0x0c06000a,0x0c05000c,0x0c05000d,
0x00000000,0x0c040020,0x0c06000a,0x0c04000f,0x0c02007b,0x0c04000e,0x0c05000d,0x0c09000b,
0x00000000,0x01010001,0x0c05000b,0x02020002,0x0c05000f,0x0c05000c,0x0c04000d,0x0c040022,
0x00000000,0x0c03003f,0x0c05000c,0x0c05000d,0x0c04001e,0x0c04000f,0x0c04000e,0x0c08000d,
0x00000000,0x01010001,0x0c05000c,0x0d030060,0x0c03003d,0x0c05000d,0x0c04000f,0x0c08000c,
0x00000000,0x0c02007e,0x0c05000d,0x0c040010,0x0c02007c,0x0c040010,0x0c040010,0x0c08000e,
0x00000000,0x01010001,0x04010002,0x02020002,0x0c050010,0x10030056,0x10020057,0x03030003,
0x0f0200a0,0x0c050012,0x04010003,0x0c05000c,0x0c04001f,0x0c04000d,0x10020058,0x04040004,
0x00000000,0x01010001,0x10030056,0x0c050013,0x0c03003e,0x10030057,0x10020059,0x0c09000a,
0x0f01019f,0x0c040021,0x10030057,0x0c08000b,0x0c02007d,0x0c04000e,0x1002005a,0x10060058,
0x00000000,0x01010001,0x0c060009,0x02020002,0x0c050011,0x0c05000c,0x0c04000d,0x0d040032,
0x1001029f,0x0c030040,0x0c06000a,0x0c05000d,0x0c040020,0x0c04000f,0x0c04000e,0x0c09000b,
0x00000000,0x01010001,0x0c05000c,0x0c040022,0x0c03003f,0x0c05000d,0x0c04000f,0x0c08000d,
0x00000000,0x0c02007f,0x0c05000d,0x0c08000c,0x0c02007e,0x0c040010,0x0c040010,0x0c08000e,
0x00000000,0x01010001,0x0f020069,0x02020002,0x0c050012,0x0c05000c,0x0c04000d,0x03030003,
0x00000000,0x0c050013,0x0f02006a,0x0c09000a,0x0c040021,0x0c08000b,0x0c04000e,0x0f05006b,
0x00000000,0x01010001,0x0c05000c,0x0c030041,0x0c030040,0x0c05000d,0x0c04000f,0x0c09000b,
0x00000000,0x0c040022,0x0c05000d,0x0c08000d,0x0c02007f,0x0c08000c,0x0c040010,0x0c08000e,
0x00000000,0x01010001,0x0c05000c,0x02020002,0x0c050013,0x0c09000a,0x0c08000b,0x0f050053,
0x00000000,0x0c030041,0x0c05000d,0x0c09000b,0x0c040022,0x0c08000d,0x0c08000c,0x0c08000e,
0x00000000,0x01010001,0x0c09000a,0x0d030061,0x0c030041,0x0c09000b,0x0c08000d,0x0c08000e,
0x00000000,0x0c020080,0x0c09000b,0x0c08000e,0x0c020080,0x0c08000e,0x0c08000e,0x0c0c000c,
0x00000000,0x01010001,0x04010002,0x02020002,0x08010008,0x05020003,0x05010004,0x03030003,
0x0f0200a1,0x09020009,0x04010003,0x06030004,0x09010010,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f0200d1,0x05020003,0x0a03000a,0x0a010020,0x05020004,0x05010006,0x07040005,
0x0f0101a0,0x0a020011,0x05020004,0x07030006,0x0f020043,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x08020004,0x100300aa,0x08010009,0x06030004,0x06020005,0x0f05000e,
0x100102a0,0x0f030024,0x08020005,0x06030005,0x09010011,0x06020007,0x06020006,0x08050006,
0x00000000,0x10020151,0x06030004,0x0f040015,0x0a010021,0x06030005,0x06020007,0x08040007,
0x00000000,0x0d030042,0x06030005,0x07030007,0x0f020044,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0d03000b,0x02020002,0x0801000a,0x09030005,0x09020006,0x03030003,
0x00000000,0x0902000a,0x0d03000c,0x07040005,0x09010012,0x07030006,0x09020007,0x0d06000d,
0x00000000,0x01010001,0x09030005,0x0d040023,0x0a010022,0x09030006,0x09020008,0x07040006,
0x00000000,0x0a020012,0x09030006,0x07030008,0x0f020045,0x07030007,0x09020009,0x09060007,
0x00000000,0x01010001,0x09020006,0x02020002,0x0801000b,0x07040005,0x07030006,0x0d050014,
0x00000000,0x0f030025,0x09020007,0x07040006,0x09010013,0x07030008,0x07030007,0x09050008,
0x00000000,0x01010001,0x07040005,0x0d030062,0x0a010023,0x07040006,0x07030008,0x08040008,
0x00000000,0x0d030043,0x07040006,0x07030009,0x0f020046,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0801000c,0x0d03000f,0x0d020010,0x03030003,
0x0f0200a2,0x0902000b,0x04010003,0x0a040006,0x09010014,0x0a030007,0x0d020011,0x04040004,
0x00000000,0x01010001,0x0d03000f,0x0a03000b,0x0a010024,0x0d030010,0x0d020012,0x08050006,
0x0f0101a1,0x0a020013,0x0d030010,0x08040007,0x0f020047,0x0a030008,0x0d020013,0x0d060011,
0x00000000,0x01010001,0x08020005,0x02020002,0x0801000d,0x0a040006,0x0a030007,0x0d040033,
0x100102a1,0x0f030026,0x08020006,0x0a040007,0x09010015,0x0a030009,0x0a030008,0x08050007,
0x00000000,0x01010001,0x0a040006,0x0f040016,0x0a010025,0x0a040007,0x0a030009,0x08040009,
0x00000000,0x0d030044,0x0a040007,0x08040008,0x0f020048,0x0a03000a,0x0a03000a,0x0a070008,
0x00000000,0x01010001,0x0d02001a,0x02020002,0x0801000e,0x0a030007,0x0a020008,0x03030003,
0x00000000,0x0902000c,0x0d02001b,0x08050006,0x09010016,0x08040007,0x0a020009,0x0d05001c,
0x00000000,0x01010001,0x0a030007,0x0d040024,0x0a010026,0x0a030008,0x0a02000a,0x08050007,
0x00000000,0x0a020014,0x0a030008,0x08040009,0x0f020049,0x08040008,0x0a02000b,0x0a060009,
0x00000000,0x01010001,0x09020007,0x02020002,0x0801000f,0x08050006,0x08040007,0x0f050054,
0x00000000,0x0f030027,0x09020008,0x08050007,0x09010017,0x08040009,0x08040008,0x09050009,
0x00000000,0x01010001,0x08050006,0x0d030063,0x0a010027,0x08050007,0x08040009,0x0804000a,
0x00000000,0x0d030045,0x08050007,0x0804000a,0x0f02004a,0x0804000a,0x0804000a,0x08080008,
0x00000000,0x01010001,0x04010002,0x02020002,0x09020009,0x05020003,0x05010004,0x03030003,
0x0f0200a3,0x0902000d,0x04010003,0x0f05000f,0x09010018,0x0f040010,0x05010005,0x04040004,
0x00000000,0x0f0200d2,0x05020003,0x0a03000c,0x0a010028,0x05020004,0x05010006,0x0f06000a,
0x0f0101a2,0x0a020015,0x05020004,0x0f05000b,0x0f02004b,0x0f040011,0x05010007,0x05050005,
0x00000000,0x01010001,0x09030005,0x02020002,0x0902000a,0x0f05000f,0x0f040010,0x0f05000f,
0x100102a2,0x0f030028,0x09030006,0x0f050010,0x09010019,0x0f040012,0x0f040011,0x09060007,
0x00000000,0x10020152,0x0f05000f,0x0f040017,0x0a010029,0x0f050010,0x0f040012,0x09050008,
0x00000000,0x0d030046,0x0f050010,0x0f05000c,0x0f02004c,0x0f040013,0x0f040013,0x0f080011,
0x00000000,0x01010001,0x0f03002b,0x02020002,0x0902000b,0x09030006,0x09020007,0x03030003,
0x00000000,0x0902000e,0x0f03002c,0x0f06000a,0x0901001a,0x0f05000b,0x09020008,0x0f06002d,
0x00000000,0x01010001,0x09030006,0x0d040025,0x0a01002a,0x09030007,0x09020009,0x0f06000b,
0x00000000,0x0a020016,0x09030007,0x0f05000d,0x0f02004d,0x0f05000c,0x0902000a,0x09060008,
0x00000000,0x01010001,0x09020008,0x02020002,0x0902000c,0x0f06000a,0x0f05000b,0x0d050015,
0x00000000,0x0f030029,0x09020009,0x0f06000b,0x0901001b,0x0f05000d,0x0f05000c,0x0905000a,
0x00000000,0x01010001,0x0f06000a,0x0d030064,0x0a01002b,0x0f06000b,0x0f05000d,0x09050009,
0x00000000,0x0d030047,0x0f06000b,0x0f05000e,0x0f02004e,0x0f05000e,0x0f05000e,0x0f09000c,
0x00000000,0x01010001,0x04010002,0x02020002,0x0902000d,0x0f040018,0x0f030019,0x03030003,
0x0f0200a4,0x0902000f,0x04010003,0x0f05000b,0x0901001c,0x0f04000c,0x0f03001a,0x04040004,
0x00000000,0x01010001,0x0f040018,0x0a03000d,0x0a01002c,0x0f040019,0x0f03001b,0x09060007,
0x0f0101a3,0x0a020017,0x0f040019,0x09050008,0x0f02004f,0x0f04000d,0x0f03001c,0x0f07001a,
0x00000000,0x01010001,0x09030006,0x02020002,0x0902000e,0x0f05000b,0x0f04000c,0x0d040034,
0x100102a3,0x0f03002a,0x09030007,0x0f05000c,0x0901001d,0x0f04000e,0x0f04000d,0x09060008,
0x00000000,0x01010001,0x0f05000b,0x0f040018,0x0a01002d,0x0f05000c,0x0f04000e,0x0905000a,
0x00000000,0x0d030048,0x0f05000c,0x09050009,0x0f020050,0x0f04000f,0x0f04000f,0x0f08000d,
0x00000000,0x01010001,0x0f02006a,0x02020002,0x0902000f,0x0a030008,0x0a020009,0x03030003,
0x00000000,0x09020010,0x0f02006b,0x09060007,0x0901001e,0x09050008,0x0a02000a,0x0f05006c,
0x00000000,0x01010001,0x0a030008,0x0d040026,0x0a01002e,0x0a030009,0x0a02000b,0x09060008,
0x00000000,0x0a020018,0x0a030009,0x0905000a,0x0f020051,0x09050009,0x0a02000c,0x0a06000a,
0x00000000,0x01010001,0x09020009,0x02020002,0x09020010,0x09060007,0x09050008,0x0f050055,
0x00000000,0x0f03002b,0x0902000a,0x09060008,0x0901001f,0x0905000a,0x09050009,0x0905000b,
0x00000000,0x01010001,0x09060007,0x0d030065,0x0a01002f,0x09060008,0x0905000a,0x0905000b,
0x00000000,0x0d030049,0x09060008,0x0905000b,0x0f020052,0x0905000b,0x0905000b,0x09090009,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d03000b,0x05020003,0x05010004,0x03030003,
0x0f0200a5,0x0a03000a,0x04010003,0x06030004,0x0a020011,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f0200d3,0x05020003,0x0a03000e,0x0a010030,0x05020004,0x05010006,0x0f050012,
0x0f0101a4,0x0a020019,0x05020004,0x0f040013,0x0f020053,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0d040007,0x100300ab,0x0d03000c,0x06030004,0x06020005,0x0f050010,
0x100102a4,0x0f03002c,0x0d040008,0x06030005,0x0a020012,0x06020007,0x06020006,0x0d070009,
0x00000000,0x10020153,0x06030004,0x0f040019,0x0a010031,0x06030005,0x06020007,0x0d06000a,
0x00000000,0x0d03004a,0x06030005,0x0f040014,0x0f020054,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0d03000c,0x02020002,0x0d03000d,0x0a040006,0x0a030007,0x03030003,
0x00000000,0x0a03000b,0x0d03000d,0x0f050012,0x0a020013,0x0f040013,0x0a030008,0x0d06000e,
0x00000000,0x01010001,0x0a040006,0x0d040027,0x0a010032,0x0a040007,0x0a030009,0x0f050013,
0x00000000,0x0a02001a,0x0a040007,0x0f040015,0x0f020055,0x0f040014,0x0a03000a,0x0a070008,
0x00000000,0x01010001,0x0a030007,0x02020002,0x0d03000e,0x0f050012,0x0f040013,0x0d050016,
0x00000000,0x0f03002d,0x0a030008,0x0f050013,0x0a020014,0x0f040015,0x0f040014,0x0a060009,
0x00000000,0x01010001,0x0f050012,0x0d030066,0x0a010033,0x0f050013,0x0f040015,0x0d06000b,
0x00000000,0x0d03004b,0x0f050013,0x0f040016,0x0f020056,0x0f040016,0x0f040016,0x0f080014,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d03000f,0x0f030037,0x0f020038,0x03030003,
0x0f0200a6,0x0a03000c,0x04010003,0x0a040007,0x0a020015,0x0a030008,0x0f020039,0x04040004,
0x00000000,0x01010001,0x0f030037,0x0a03000f,0x0a010034,0x0f030038,0x0f02003a,0x0d070009,
0x0f0101a5,0x0a02001b,0x0f030038,0x0d06000a,0x0f020057,0x0a030009,0x0f02003b,0x0f060039,
0x00000000,0x01010001,0x0d040008,0x02020002,0x0d030010,0x0a040007,0x0a030008,0x0d040035,
0x100102a5,0x0f03002e,0x0d040009,0x0a040008,0x0a020016,0x0a03000a,0x0a030009,0x0d07000a,
0x00000000,0x01010001,0x0a040007,0x0f04001a,0x0a010035,0x0a040008,0x0a03000a,0x0d06000c,
0x00000000,0x0d03004c,0x0a040008,0x0d06000b,0x0f020058,0x0a03000b,0x0a03000b,0x0a070009,
0x00000000,0x01010001,0x0d02001b,0x02020002,0x0d030011,0x0a030009,0x0a02000a,0x03030003,
0x00000000,0x0a03000d,0x0d02001c,0x0d070009,0x0a020017,0x0d06000a,0x0a02000b,0x0d05001d,
0x00000000,0x01010001,0x0a030009,0x0d040028,0x0a010036,0x0a03000a,0x0a02000c,0x0d07000a,
0x00000000,0x0a02001c,0x0a03000a,0x0d06000c,0x0f020059,0x0d06000b,0x0a02000d,0x0a06000b,
0x00000000,0x01010001,0x0a030008,0x02020002,0x0d030012,0x0d070009,0x0d06000a,0x0f050056,
0x00000000,0x0f03002f,0x0a030009,0x0d07000a,0x0a020018,0x0d06000c,0x0d06000b,0x0a06000a,
0x00000000,0x01010001,0x0d070009,0x0d030067,0x0a010037,0x0d07000a,0x0d06000c,0x0d06000d,
0x00000000,0x0d03004d,0x0d07000a,0x0d06000d,0x0f02005a,0x0d06000d,0x0d06000d,0x0d0a000b,
0x00000000,0x01010001,0x04010002,0x02020002,0x0a03000a,0x05020003,0x05010004,0x03030003,
0x0f0200a7,0x0a03000e,0x04010003,0x0f04001e,0x0a020019,0x0f03001f,0x05010005,0x04040004,
0x00000000,0x0f0200d4,0x05020003,0x0a030010,0x0a010038,0x05020004,0x05010006,0x0d06000a,
0x0f0101a6,0x0a02001d,0x05020004,0x0d05000b,0x0f02005b,0x0f030020,0x05010007,0x05050005,
0x00000000,0x01010001,0x0a040006,0x02020002,0x0a03000b,0x0f04001e,0x0f03001f,0x0f050011,
0x100102a6,0x0f030030,0x0a040007,0x0f04001f,0x0a02001a,0x0f030021,0x0f030020,0x0a070008,
0x00000000,0x10020154,0x0f04001e,0x0f04001b,0x0a010039,0x0f04001f,0x0f030021,0x0a060009,
0x00000000,0x0d03004e,0x0f04001f,0x0d05000c,0x0f02005c,0x0f030022,0x0f030022,0x0f070020,
0x00000000,0x01010001,0x0f03002c,0x02020002,0x0a03000c,0x0a040007,0x0a030008,0x03030003,
0x00000000,0x0a03000f,0x0f03002d,0x0d06000a,0x0a02001b,0x0d05000b,0x0a030009,0x0f06002e,
0x00000000,0x01010001,0x0a040007,0x0d040029,0x0a01003a,0x0a040008,0x0a03000a,0x0d06000b,
0x00000000,0x0a02001e,0x0a040008,0x0d05000d,0x0f02005d,0x0d05000c,0x0a03000b,0x0a070009,
0x00000000,0x01010001,0x0a030009,0x02020002,0x0a03000d,0x0d06000a,0x0d05000b,0x0d050017,
0x00000000,0x0f030031,0x0a03000a,0x0d06000b,0x0a02001c,0x0d05000d,0x0d05000c,0x0a06000b,
0x00000000,0x01010001,0x0d06000a,0x0d030068,0x0a01003b,0x0d06000b,0x0d05000d,0x0a06000a,
0x00000000,0x0d03004f,0x0d06000b,0x0d05000e,0x0f02005e,0x0d05000e,0x0d05000e,0x0d09000c,
0x00000000,0x01010001,0x04010002,0x02020002,0x0a03000e,0x10030057,0x10020058,0x03030003,
0x0f0200a8,0x0a030010,0x04010003,0x0f05000c,0x0a02001d,0x0f04000d,0x10020059,0x04040004,
0x00000000,0x01010001,0x10030057,0x0a030011,0x0a01003c,0x10030058,0x1002005a,0x0a070008,
0x0f0101a7,0x0a02001f,0x10030058,0x0a060009,0x0f02005f,0x0f04000e,0x1002005b,0x10060059,
0x00000000,0x01010001,0x0a040007,0x02020002,0x0a03000f,0x0f05000c,0x0f04000d,0x0d040036,
0x100102a7,0x0f030032,0x0a040008,0x0f05000d,0x0a02001e,0x0f04000f,0x0f04000e,0x0a070009,
0x00000000,0x01010001,0x0f05000c,0x0f04001c,0x0a01003d,0x0f05000d,0x0f04000f,0x0a06000b,
0x00000000,0x0d030050,0x0f05000d,0x0a06000a,0x0f020060,0x0f040010,0x0f040010,0x0f08000e,
0x00000000,0x01010001,0x0f02006b,0x02020002,0x0a030010,0x0a03000a,0x0a02000b,0x03030003,
0x00000000,0x0a030011,0x0f02006c,0x0a070008,0x0a02001f,0x0a060009,0x0a02000c,0x0f05006d,
0x00000000,0x01010001,0x0a03000a,0x0d04002a,0x0a01003e,0x0a03000b,0x0a02000d,0x0a070009,
0x00000000,0x0a020020,0x0a03000b,0x0a06000b,0x0f020061,0x0a06000a,0x0a02000e,0x0a06000c,
0x00000000,0x01010001,0x0a03000a,0x02020002,0x0a030011,0x0a070008,0x0a060009,0x0f050057,
0x00000000,0x0f030033,0x0a03000b,0x0a070009,0x0a020020,0x0a06000b,0x0a06000a,0x0a06000c,
0x00000000,0x01010001,0x0a070008,0x0d030069,0x0a01003f,0x0a070009,0x0a06000b,0x0a06000c,
0x00000000,0x0d030051,0x0a070009,0x0a06000c,0x0f020062,0x0a06000c,0x0a06000c,0x0a0a000a,
0x00000000,0x01010001,0x04010002,0x02020002,0x08010008,0x05020003,0x05010004,0x03030003,
0x0f0200a9,0x0d03000c,0x04010003,0x06030004,0x0d020013,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f0200d5,0x05020003,0x0f05000e,0x0f030024,0x05020004,0x05010006,0x07040005,
0x0f0101a8,0x0f040015,0x05020004,0x07030006,0x0f020063,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x08020004,0x100300ac,0x08010009,0x06030004,0x06020005,0x0f050012,
0x100102a8,0x0f030034,0x08020005,0x06030005,0x0d020014,0x06020007,0x06020006,0x08050006,
0x00000000,0x10020155,0x06030004,0x0f04001d,0x0f030025,0x06030005,0x06020007,0x08040007,
0x00000000,0x0d030052,0x06030005,0x07030007,0x0f020064,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0d03000d,0x02020002,0x0801000a,0x0d040008,0x0d030009,0x03030003,
0x00000000,0x0d03000d,0x0d03000e,0x07040005,0x0d020015,0x07030006,0x0d03000a,0x0d06000f,
0x00000000,0x01010001,0x0d040008,0x0d04002b,0x0f030026,0x0d040009,0x0d03000b,0x07040006,
0x00000000,0x0f040016,0x0d040009,0x07030008,0x0f020065,0x07030007,0x0d03000c,0x0d07000a,
0x00000000,0x01010001,0x0d030009,0x02020002,0x0801000b,0x07040005,0x07030006,0x0d050018,
0x00000000,0x0f030035,0x0d03000a,0x07040006,0x0d020016,0x07030008,0x07030007,0x0d06000b,
0x00000000,0x01010001,0x07040005,0x0d03006a,0x0f030027,0x07040006,0x07030008,0x08040008,
0x00000000,0x0d030053,0x07040006,0x07030009,0x0f020066,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0801000c,0x0d030010,0x0d020011,0x03030003,
0x0f0200aa,0x0d03000e,0x04010003,0x0f06000a,0x0d020017,0x0f05000b,0x0d020012,0x04040004,
0x00000000,0x01010001,0x0d030010,0x0f05000f,0x0f030028,0x0d030011,0x0d020013,0x08050006,
0x0f0101a9,0x0f040017,0x0d030011,0x08040007,0x0f020067,0x0f05000c,0x0d020014,0x0d060012,
0x00000000,0x01010001,0x08020005,0x02020002,0x0801000d,0x0f06000a,0x0f05000b,0x0d040037,
0x100102a9,0x0f030036,0x08020006,0x0f06000b,0x0d020018,0x0f05000d,0x0f05000c,0x08050007,
0x00000000,0x01010001,0x0f06000a,0x0f04001e,0x0f030029,0x0f06000b,0x0f05000d,0x08040009,
0x00000000,0x0d030054,0x0f06000b,0x08040008,0x0f020068,0x0f05000e,0x0f05000e,0x0f09000c,
0x00000000,0x01010001,0x0d02001c,0x02020002,0x0801000e,0x0f05000b,0x0f04000c,0x03030003,
0x00000000,0x0d03000f,0x0d02001d,0x08050006,0x0d020019,0x08040007,0x0f04000d,0x0d05001e,
0x00000000,0x01010001,0x0f05000b,0x0d04002c,0x0f03002a,0x0f05000c,0x0f04000e,0x08050007,
0x00000000,0x0f040018,0x0f05000c,0x08040009,0x0f020069,0x08040008,0x0f04000f,0x0f08000d,
0x00000000,0x01010001,0x0d03000a,0x02020002,0x0801000f,0x08050006,0x08040007,0x0f050058,
0x00000000,0x0f030037,0x0d03000b,0x08050007,0x0d02001a,0x08040009,0x08040008,0x0d06000c,
0x00000000,0x01010001,0x08050006,0x0d03006b,0x0f03002b,0x08050007,0x08040009,0x0804000a,
0x00000000,0x0d030055,0x08050007,0x0804000a,0x0f02006a,0x0804000a,0x0804000a,0x08080008,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d03000c,0x05020003,0x05010004,0x03030003,
0x0f0200ab,0x0d030010,0x04010003,0x1004002e,0x0d02001b,0x1003002f,0x05010005,0x04040004,
0x00000000,0x0f0200d6,0x05020003,0x0f050010,0x0f03002c,0x05020004,0x05010006,0x0f06000b,
0x0f0101aa,0x0f040019,0x05020004,0x0f05000c,0x0f02006b,0x10030030,0x05010007,0x05050005,
0x00000000,0x01010001,0x0d040008,0x02020002,0x0d03000d,0x1004002e,0x1003002f,0x0f050013,
0x100102aa,0x0f030038,0x0d040009,0x1004002f,0x0d02001c,0x10030031,0x10030030,0x0d07000a,
0x00000000,0x10020156,0x1004002e,0x0f04001f,0x0f03002d,0x1004002f,0x10030031,0x0d06000b,
0x00000000,0x0d030056,0x1004002f,0x0f05000d,0x0f02006c,0x10030032,0x10030032,0x10070030,
0x00000000,0x01010001,0x0f03002d,0x02020002,0x0d03000e,0x0d040009,0x0d03000a,0x03030003,
0x00000000,0x0d030011,0x0f03002e,0x0f06000b,0x0d02001d,0x0f05000c,0x0d03000b,0x0f06002f,
0x00000000,0x01010001,0x0d040009,0x0d04002d,0x0f03002e,0x0d04000a,0x0d03000c,0x0f06000c,
0x00000000,0x0f04001a,0x0d04000a,0x0f05000e,0x0f02006d,0x0f05000d,0x0d03000d,0x0d07000b,
0x00000000,0x01010001,0x0d03000b,0x02020002,0x0d03000f,0x0f06000b,0x0f05000c,0x0d050019,
0x00000000,0x0f030039,0x0d03000c,0x0f06000c,0x0d02001e,0x0f05000e,0x0f05000d,0x0d06000d,
0x00000000,0x01010001,0x0f06000b,0x0d03006c,0x0f03002f,0x0f06000c,0x0f05000e,0x0d06000c,
0x00000000,0x0d030057,0x0f06000c,0x0f05000f,0x0f02006e,0x0f05000f,0x0f05000f,0x0f09000d,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d030010,0x0f040019,0x0f03001a,0x03030003,
0x0f0200ac,0x0d030012,0x04010003,0x0f05000d,0x0d02001f,0x0f04000e,0x0f03001b,0x04040004,
0x00000000,0x01010001,0x0f040019,0x0f050011,0x0f030030,0x0f04001a,0x0f03001c,0x0d07000a,
0x0f0101ab,0x0f04001b,0x0f04001a,0x0d06000b,0x0f02006f,0x0f04000f,0x0f03001d,0x0f07001b,
0x00000000,0x01010001,0x0d040009,0x02020002,0x0d030011,0x0f05000d,0x0f04000e,0x0d040038,
0x100102ab,0x0f03003a,0x0d04000a,0x0f05000e,0x0d020020,0x0f040010,0x0f04000f,0x0d07000b,
0x00000000,0x01010001,0x0f05000d,0x0f040020,0x0f030031,0x0f05000e,0x0f040010,0x0d06000d,
0x00000000,0x0d030058,0x0f05000e,0x0d06000c,0x0f020070,0x0f040011,0x0f040011,0x0f08000f,
0x00000000,0x01010001,0x0f02006c,0x02020002,0x0d030012,0x0f05000c,0x0f04000d,0x03030003,
0x00000000,0x0d030013,0x0f02006d,0x0d07000a,0x0d020021,0x0d06000b,0x0f04000e,0x0f05006e,
0x00000000,0x01010001,0x0f05000c,0x0d04002e,0x0f030032,0x0f05000d,0x0f04000f,0x0d07000b,
0x00000000,0x0f04001c,0x0f05000d,0x0d06000d,0x0f020071,0x0d06000c,0x0f040010,0x0f08000e,
0x00000000,0x01010001,0x0d03000c,0x02020002,0x0d030013,0x0d07000a,0x0d06000b,0x0f050059,
0x00000000,0x0f03003b,0x0d03000d,0x0d07000b,0x0d020022,0x0d06000d,0x0d06000c,0x0d06000e,
0x00000000,0x01010001,0x0d07000a,0x0d03006d,0x0f030033,0x0d07000b,0x0d06000d,0x0d06000e,
0x00000000,0x0d030059,0x0d07000b,0x0d06000e,0x0f020072,0x0d06000e,0x0d06000e,0x0d0a000c,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d02000e,0x05020003,0x05010004,0x03030003,
0x0f0200ad,0x0f05000e,0x04010003,0x06030004,0x0f040015,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f0200d7,0x05020003,0x0f050012,0x0f030034,0x05020004,0x05010006,0x1005001a,
0x0f0101ac,0x0f04001d,0x05020004,0x1004001b,0x0f020073,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0d03000a,0x100300ad,0x0d02000f,0x06030004,0x06020005,0x0f050014,
0x100102ac,0x0f03003c,0x0d03000b,0x06030005,0x0f040016,0x06020007,0x06020006,0x0d06000c,
0x00000000,0x10020157,0x06030004,0x0f040021,0x0f030035,0x06030005,0x06020007,0x0d05000d,
0x00000000,0x0d03005a,0x06030005,0x1004001c,0x0f020074,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0d03000e,0x02020002,0x0d020010,0x0f06000a,0x0f05000b,0x03030003,
0x00000000,0x0f05000f,0x0d03000f,0x1005001a,0x0f040017,0x1004001b,0x0f05000c,0x0d060010,
0x00000000,0x01010001,0x0f06000a,0x0d04002f,0x0f030036,0x0f06000b,0x0f05000d,0x1005001b,
0x00000000,0x0f04001e,0x0f06000b,0x1004001d,0x0f020075,0x1004001c,0x0f05000e,0x0f09000c,
0x00000000,0x01010001,0x0f05000b,0x02020002,0x0d020011,0x1005001a,0x1004001b,0x0d05001a,
0x00000000,0x0f03003d,0x0f05000c,0x1005001b,0x0f040018,0x1004001d,0x1004001c,0x0f08000d,
0x00000000,0x01010001,0x1005001a,0x0d03006e,0x0f030037,0x1005001b,0x1004001d,0x0d05000e,
0x00000000,0x0d03005b,0x1005001b,0x1004001e,0x0f020076,0x1004001e,0x1004001e,0x1008001c,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d020012,0x0f030038,0x0f020039,0x03030003,
0x0f0200ae,0x0f050010,0x04010003,0x0f06000b,0x0f040019,0x0f05000c,0x0f02003a,0x04040004,
0x00000000,0x01010001,0x0f030038,0x0f050013,0x0f030038,0x0f030039,0x0f02003b,0x0d06000c,
0x0f0101ad,0x0f04001f,0x0f030039,0x0d05000d,0x0f020077,0x0f05000d,0x0f02003c,0x0f06003a,
0x00000000,0x01010001,0x0d03000b,0x02020002,0x0d020013,0x0f06000b,0x0f05000c,0x0d040039,
0x100102ad,0x0f03003e,0x0d03000c,0x0f06000c,0x0f04001a,0x0f05000e,0x0f05000d,0x0d06000d,
0x00000000,0x01010001,0x0f06000b,0x0f040022,0x0f030039,0x0f06000c,0x0f05000e,0x0d05000f,
0x00000000,0x0d03005c,0x0f06000c,0x0d05000e,0x0f020078,0x0f05000f,0x0f05000f,0x0f09000d,
0x00000000,0x01010001,0x0d02001d,0x02020002,0x0d020014,0x0f05000d,0x0f04000e,0x03030003,
0x00000000,0x0f050011,0x0d02001e,0x0d06000c,0x0f04001b,0x0d05000d,0x0f04000f,0x0d05001f,
0x00000000,0x01010001,0x0f05000d,0x0d040030,0x0f03003a,0x0f05000e,0x0f040010,0x0d06000d,
0x00000000,0x0f040020,0x0f05000e,0x0d05000f,0x0f020079,0x0d05000e,0x0f040011,0x0f08000f,
0x00000000,0x01010001,0x0f05000c,0x02020002,0x0d020015,0x0d06000c,0x0d05000d,0x0f05005a,
0x00000000,0x0f03003f,0x0f05000d,0x0d06000d,0x0f04001c,0x0d05000f,0x0d05000e,0x0f08000e,
0x00000000,0x01010001,0x0d06000c,0x0d03006f,0x0f03003b,0x0d06000d,0x0d05000f,0x0d050010,
0x00000000,0x0d03005d,0x0d06000d,0x0d050010,0x0f02007a,0x0d050010,0x0d050010,0x0d09000e,
0x00000000,0x01010001,0x04010002,0x02020002,0x0f05000e,0x05020003,0x05010004,0x03030003,
0x0f0200af,0x0f050012,0x04010003,0x06030004,0x0f04001d,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f0200d8,0x05020003,0x0f050014,0x0f03003c,0x05020004,0x05010006,0x0d06000b,
0x0f0101ae,0x0f040021,0x05020004,0x0d05000c,0x0f02007b,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0f06000a,0x02020002,0x0f05000f,0x06030004,0x06020005,0x0f050015,
0x100102ae,0x0f030040,0x0f06000b,0x06030005,0x0f04001e,0x06020007,0x06020006,0x0f09000c,
0x00000000,0x10020158,0x06030004,0x0f040023,0x0f03003d,0x06030005,0x06020007,0x0f08000d,
0x00000000,0x0d03005e,0x06030005,0x0d05000d,0x0f02007c,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0f03002e,0x02020002,0x0f050010,0x0f06000b,0x0f05000c,0x03030003,
0x00000000,0x0f050013,0x0f03002f,0x0d06000b,0x0f04001f,0x0d05000c,0x0f05000d,0x0f060030,
0x00000000,0x01010001,0x0f06000b,0x0d040031,0x0f03003e,0x0f06000c,0x0f05000e,0x0d06000c,
0x00000000,0x0f040022,0x0f06000c,0x0d05000e,0x0f02007d,0x0d05000d,0x0f05000f,0x0f09000d,
0x00000000,0x01010001,0x0f05000d,0x02020002,0x0f050011,0x0d06000b,0x0d05000c,0x0d05001b,
0x00000000,0x0f030041,0x0f05000e,0x0d06000c,0x0f040020,0x0d05000e,0x0d05000d,0x0f08000f,
0x00000000,0x01010001,0x0d06000b,0x0d030070,0x0f03003f,0x0d06000c,0x0d05000e,0x0f08000e,
0x00000000,0x0d03005f,0x0d06000c,0x0d05000f,0x0f02007e,0x0d05000f,0x0d05000f,0x0d09000d,
0x00000000,0x01010001,0x04010002,0x02020002,0x0f050012,0x10030058,0x10020059,0x03030003,
0x0f0200b0,0x0f050014,0x04010003,0x0f05000e,0x0f040021,0x0f04000f,0x1002005a,0x04040004,
0x00000000,0x01010001,0x10030058,0x0f050015,0x0f030040,0x10030059,0x1002005b,0x0f09000c,
0x0f0101af,0x0f040023,0x10030059,0x0f08000d,0x0f02007f,0x0f040010,0x1002005c,0x1006005a,
0x00000000,0x01010001,0x0f06000b,0x02020002,0x0f050013,0x0f05000e,0x0f04000f,0x0d04003a,
0x100102af,0x0f030042,0x0f06000c,0x0f05000f,0x0f040022,0x0f040011,0x0f040010,0x0f09000d,
0x00000000,0x01010001,0x0f05000e,0x0f040024,0x0f030041,0x0f05000f,0x0f040011,0x0f08000f,
0x00000000,0x0d030060,0x0f05000f,0x0f08000e,0x0f020080,0x0f040012,0x0f040012,0x0f080010,
0x00000000,0x01010001,0x0f02006d,0x02020002,0x0f050014,0x0f05000e,0x0f04000f,0x03030003,
0x00000000,0x0f050015,0x0f02006e,0x0f09000c,0x0f040023,0x0f08000d,0x0f040010,0x0f05006f,
0x00000000,0x01010001,0x0f05000e,0x0d040032,0x0f030042,0x0f05000f,0x0f040011,0x0f09000d,
0x00000000,0x0f040024,0x0f05000f,0x0f08000f,0x0f020081,0x0f08000e,0x0f040012,0x0f080010,
0x00000000,0x01010001,0x0f05000e,0x02020002,0x0f050015,0x0f09000c,0x0f08000d,0x0f05005b,
0x00000000,0x0f030043,0x0f05000f,0x0f09000d,0x0f040024,0x0f08000f,0x0f08000e,0x0f080010,
0x00000000,0x01010001,0x0f09000c,0x0d030071,0x0f030043,0x0f09000d,0x0f08000f,0x0f080010,
0x00000000,0x0d030061,0x0f09000d,0x0f080010,0x0f020082,0x0f080010,0x0f080010,0x0f0c000e,
0x00000000,0x01010001,0x04010002,0x02020002,0x08010008,0x05020003,0x05010004,0x03030003,
0x0f0200b1,0x09020009,0x04010003,0x06030004,0x09010010,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f0200d9,0x05020003,0x0f040010,0x0f020026,0x05020004,0x05010006,0x07040005,
0x0f0101b0,0x0f030017,0x05020004,0x07030006,0x0d030042,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x08020004,0x100300ae,0x08010009,0x06030004,0x06020005,0x0d06000d,
0x100102b0,0x0d040023,0x08020005,0x06030005,0x09010011,0x06020007,0x06020006,0x08050006,
0x00000000,0x10020159,0x06030004,0x0d050014,0x0f020027,0x06030005,0x06020007,0x08040007,
0x00000000,0x0d030062,0x06030005,0x07030007,0x0d030043,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0d03000f,0x02020002,0x0801000a,0x09030005,0x09020006,0x03030003,
0x00000000,0x0902000a,0x0d030010,0x07040005,0x09010012,0x07030006,0x09020007,0x0d060011,
0x00000000,0x01010001,0x09030005,0x0d040033,0x0f020028,0x09030006,0x09020008,0x07040006,
0x00000000,0x0f030018,0x09030006,0x07030008,0x0d030044,0x07030007,0x09020009,0x09060007,
0x00000000,0x01010001,0x09020006,0x02020002,0x0801000b,0x07040005,0x07030006,0x0d05001c,
0x00000000,0x0d040024,0x09020007,0x07040006,0x09010013,0x07030008,0x07030007,0x09050008,
0x00000000,0x01010001,0x07040005,0x0d030072,0x0f020029,0x07040006,0x07030008,0x08040008,
0x00000000,0x0d030063,0x07040006,0x07030009,0x0d030045,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0801000c,0x0d030011,0x0d020012,0x03030003,
0x0f0200b2,0x0902000b,0x04010003,0x0f05000c,0x09010014,0x0f04000d,0x0d020013,0x04040004,
0x00000000,0x01010001,0x0d030011,0x0f040011,0x0f02002a,0x0d030012,0x0d020014,0x08050006,
0x0f0101b1,0x0f030019,0x0d030012,0x08040007,0x0d030046,0x0f04000e,0x0d020015,0x0d060013,
0x00000000,0x01010001,0x08020005,0x02020002,0x0801000d,0x0f05000c,0x0f04000d,0x0d04003b,
0x100102b1,0x0d040025,0x08020006,0x0f05000d,0x09010015,0x0f04000f,0x0f04000e,0x08050007,
0x00000000,0x01010001,0x0f05000c,0x0d050015,0x0f02002b,0x0f05000d,0x0f04000f,0x08040009,
0x00000000,0x0d030064,0x0f05000d,0x08040008,0x0d030047,0x0f040010,0x0f040010,0x0f08000e,
0x00000000,0x01010001,0x0d02001e,0x02020002,0x0801000e,0x0f04000d,0x0f03000e,0x03030003,
0x00000000,0x0902000c,0x0d02001f,0x08050006,0x09010016,0x08040007,0x0f03000f,0x0d050020,
0x00000000,0x01010001,0x0f04000d,0x0d040034,0x0f02002c,0x0f04000e,0x0f030010,0x08050007,
0x00000000,0x0f03001a,0x0f04000e,0x08040009,0x0d030048,0x08040008,0x0f030011,0x0f07000f,
0x00000000,0x01010001,0x09020007,0x02020002,0x0801000f,0x08050006,0x08040007,0x0f05005c,
0x00000000,0x0d040026,0x09020008,0x08050007,0x09010017,0x08040009,0x08040008,0x09050009,
0x00000000,0x01010001,0x08050006,0x0d030073,0x0f02002d,0x08050007,0x08040009,0x0804000a,
0x00000000,0x0d030065,0x08050007,0x0804000a,0x0d030049,0x0804000a,0x0804000a,0x08080008,
0x00000000,0x01010001,0x04010002,0x02020002,0x09020009,0x05020003,0x05010004,0x03030003,
0x0f0200b3,0x0902000d,0x04010003,0x0f050010,0x09010018,0x0f040011,0x05010005,0x04040004,
0x00000000,0x0f0200da,0x05020003,0x0f040012,0x0f02002e,0x05020004,0x05010006,0x0d070009,
0x0f0101b2,0x0f03001b,0x05020004,0x0d06000a,0x0d03004a,0x0f040012,0x05010007,0x05050005,
0x00000000,0x01010001,0x09030005,0x02020002,0x0902000a,0x0f050010,0x0f040011,0x0d06000e,
0x100102b2,0x0d040027,0x09030006,0x0f050011,0x09010019,0x0f040013,0x0f040012,0x09060007,
0x00000000,0x1002015a,0x0f050010,0x0d050016,0x0f02002f,0x0f050011,0x0f040013,0x09050008,
0x00000000,0x0d030066,0x0f050011,0x0d06000b,0x0d03004b,0x0f040014,0x0f040014,0x0f080012,
0x00000000,0x01010001,0x0f03002f,0x02020002,0x0902000b,0x09030006,0x09020007,0x03030003,
0x00000000,0x0902000e,0x0f030030,0x0d070009,0x0901001a,0x0d06000a,0x09020008,0x0f060031,
0x00000000,0x01010001,0x09030006,0x0d040035,0x0f020030,0x09030007,0x09020009,0x0d07000a,
0x00000000,0x0f03001c,0x09030007,0x0d06000c,0x0d03004c,0x0d06000b,0x0902000a,0x09060008,
0x00000000,0x01010001,0x09020008,0x02020002,0x0902000c,0x0d070009,0x0d06000a,0x0d05001d,
0x00000000,0x0d040028,0x09020009,0x0d07000a,0x0901001b,0x0d06000c,0x0d06000b,0x0905000a,
0x00000000,0x01010001,0x0d070009,0x0d030074,0x0f020031,0x0d07000a,0x0d06000c,0x09050009,
0x00000000,0x0d030067,0x0d07000a,0x0d06000d,0x0d03004d,0x0d06000d,0x0d06000d,0x0d0a000b,
0x00000000,0x01010001,0x04010002,0x02020002,0x0902000d,0x0f04001a,0x0f03001b,0x03030003,
0x0f0200b4,0x0902000f,0x04010003,0x0d06000a,0x0901001c,0x0d05000b,0x0f03001c,0x04040004,
0x00000000,0x01010001,0x0f04001a,0x0f040013,0x0f020032,0x0f04001b,0x0f03001d,0x09060007,
0x0f0101b3,0x0f03001d,0x0f04001b,0x09050008,0x0d03004e,0x0d05000c,0x0f03001e,0x0f07001c,
0x00000000,0x01010001,0x09030006,0x02020002,0x0902000e,0x0d06000a,0x0d05000b,0x0d04003c,
0x100102b3,0x0d040029,0x09030007,0x0d06000b,0x0901001d,0x0d05000d,0x0d05000c,0x09060008,
0x00000000,0x01010001,0x0d06000a,0x0d050017,0x0f020033,0x0d06000b,0x0d05000d,0x0905000a,
0x00000000,0x0d030068,0x0d06000b,0x09050009,0x0d03004f,0x0d05000e,0x0d05000e,0x0d09000c,
0x00000000,0x01010001,0x0f02006e,0x02020002,0x0902000f,0x0f04000e,0x0f03000f,0x03030003,
0x00000000,0x09020010,0x0f02006f,0x09060007,0x0901001e,0x09050008,0x0f030010,0x0f050070,
0x00000000,0x01010001,0x0f04000e,0x0d040036,0x0f020034,0x0f04000f,0x0f030011,0x09060008,
0x00000000,0x0f03001e,0x0f04000f,0x0905000a,0x0d030050,0x09050009,0x0f030012,0x0f070010,
0x00000000,0x01010001,0x09020009,0x02020002,0x09020010,0x09060007,0x09050008,0x0f05005d,
0x00000000,0x0d04002a,0x0902000a,0x09060008,0x0901001f,0x0905000a,0x09050009,0x0905000b,
0x00000000,0x01010001,0x09060007,0x0d030075,0x0f020035,0x09060008,0x0905000a,0x0905000b,
0x00000000,0x0d030069,0x09060008,0x0905000b,0x0d030051,0x0905000b,0x0905000b,0x09090009,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d03000c,0x05020003,0x05010004,0x03030003,
0x0f0200b5,0x0f040010,0x04010003,0x06030004,0x0f030017,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f0200db,0x05020003,0x0f040014,0x0f020036,0x05020004,0x05010006,0x07040005,
0x0f0101b4,0x0f03001f,0x05020004,0x07030006,0x0d030052,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0d040008,0x100300af,0x0d03000d,0x06030004,0x06020005,0x0d06000f,
0x100102b4,0x0d04002b,0x0d040009,0x06030005,0x0f030018,0x06020007,0x06020006,0x0d07000a,
0x00000000,0x1002015b,0x06030004,0x0d050018,0x0f020037,0x06030005,0x06020007,0x0d06000b,
0x00000000,0x0d03006a,0x06030005,0x07030007,0x0d030053,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0d030010,0x02020002,0x0d03000e,0x0f05000c,0x0f04000d,0x03030003,
0x00000000,0x0f040011,0x0d030011,0x07040005,0x0f030019,0x07030006,0x0f04000e,0x0d060012,
0x00000000,0x01010001,0x0f05000c,0x0d040037,0x0f020038,0x0f05000d,0x0f04000f,0x07040006,
0x00000000,0x0f030020,0x0f05000d,0x07030008,0x0d030054,0x07030007,0x0f040010,0x0f08000e,
0x00000000,0x01010001,0x0f04000d,0x02020002,0x0d03000f,0x07040005,0x07030006,0x0d05001e,
0x00000000,0x0d04002c,0x0f04000e,0x07040006,0x0f03001a,0x07030008,0x07030007,0x0f07000f,
0x00000000,0x01010001,0x07040005,0x0d030076,0x0f020039,0x07040006,0x07030008,0x0d06000c,
0x00000000,0x0d03006b,0x07040006,0x07030009,0x0d030055,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d030010,0x0f030039,0x0f02003a,0x03030003,
0x0f0200b6,0x0f040012,0x04010003,0x0f05000d,0x0f03001b,0x0f04000e,0x0f02003b,0x04040004,
0x00000000,0x01010001,0x0f030039,0x0f040015,0x0f02003a,0x0f03003a,0x0f02003c,0x0d07000a,
0x0f0101b5,0x0f030021,0x0f03003a,0x0d06000b,0x0d030056,0x0f04000f,0x0f02003d,0x0f06003b,
0x00000000,0x01010001,0x0d040009,0x02020002,0x0d030011,0x0f05000d,0x0f04000e,0x0d04003d,
0x100102b5,0x0d04002d,0x0d04000a,0x0f05000e,0x0f03001c,0x0f040010,0x0f04000f,0x0d07000b,
0x00000000,0x01010001,0x0f05000d,0x0d050019,0x0f02003b,0x0f05000e,0x0f040010,0x0d06000d,
0x00000000,0x0d03006c,0x0f05000e,0x0d06000c,0x0d030057,0x0f040011,0x0f040011,0x0f08000f,
0x00000000,0x01010001,0x0d02001f,0x02020002,0x0d030012,0x0f04000f,0x0f030010,0x03030003,
0x00000000,0x0f040013,0x0d020020,0x0d07000a,0x0f03001d,0x0d06000b,0x0f030011,0x0d050021,
0x00000000,0x01010001,0x0f04000f,0x0d040038,0x0f02003c,0x0f040010,0x0f030012,0x0d07000b,
0x00000000,0x0f030022,0x0f040010,0x0d06000d,0x0d030058,0x0d06000c,0x0f030013,0x0f070011,
0x00000000,0x01010001,0x0f04000e,0x02020002,0x0d030013,0x0d07000a,0x0d06000b,0x0f05005e,
0x00000000,0x0d04002e,0x0f04000f,0x0d07000b,0x0f03001e,0x0d06000d,0x0d06000c,0x0f070010,
0x00000000,0x01010001,0x0d07000a,0x0d030077,0x0f02003d,0x0d07000b,0x0d06000d,0x0d06000e,
0x00000000,0x0d03006d,0x0d07000b,0x0d06000e,0x0d030059,0x0d06000e,0x0d06000e,0x0d0a000c,
0x00000000,0x01010001,0x04010002,0x02020002,0x0f040010,0x05020003,0x05010004,0x03030003,
0x0f0200b7,0x0f040014,0x04010003,0x0f04001f,0x0f03001f,0x0f030020,0x05010005,0x04040004,
0x00000000,0x0f0200dc,0x05020003,0x0f040016,0x0f02003e,0x05020004,0x05010006,0x0d06000c,
0x0f0101b6,0x0f030023,0x05020004,0x0d05000d,0x0d03005a,0x0f030021,0x05010007,0x05050005,
0x00000000,0x01010001,0x0f05000c,0x02020002,0x0f040011,0x0f04001f,0x0f030020,0x0d060010,
0x100102b6,0x0d04002f,0x0f05000d,0x0f040020,0x0f030020,0x0f030022,0x0f030021,0x0f08000e,
0x00000000,0x1002015c,0x0f04001f,0x0d05001a,0x0f02003f,0x0f040020,0x0f030022,0x0f07000f,
0x00000000,0x0d03006e,0x0f040020,0x0d05000e,0x0d03005b,0x0f030023,0x0f030023,0x0f070021,
0x00000000,0x01010001,0x0f030030,0x02020002,0x0f040012,0x0f05000d,0x0f04000e,0x03030003,
0x00000000,0x0f040015,0x0f030031,0x0d06000c,0x0f030021,0x0d05000d,0x0f04000f,0x0f060032,
0x00000000,0x01010001,0x0f05000d,0x0d040039,0x0f020040,0x0f05000e,0x0f040010,0x0d06000d,
0x00000000,0x0f030024,0x0f05000e,0x0d05000f,0x0d03005c,0x0d05000e,0x0f040011,0x0f08000f,
0x00000000,0x01010001,0x0f04000f,0x02020002,0x0f040013,0x0d06000c,0x0d05000d,0x0d05001f,
0x00000000,0x0d040030,0x0f040010,0x0d06000d,0x0f030022,0x0d05000f,0x0d05000e,0x0f070011,
0x00000000,0x01010001,0x0d06000c,0x0d030078,0x0f020041,0x0d06000d,0x0d05000f,0x0f070010,
0x00000000,0x0d03006f,0x0d06000d,0x0d050010,0x0d03005d,0x0d050010,0x0d050010,0x0d09000e,
0x00000000,0x01010001,0x04010002,0x02020002,0x0f040014,0x10030059,0x1002005a,0x03030003,
0x0f0200b8,0x0f040016,0x04010003,0x0d06000b,0x0f030023,0x0d05000c,0x1002005b,0x04040004,
0x00000000,0x01010001,0x10030059,0x0f040017,0x0f020042,0x1003005a,0x1002005c,0x0f08000e,
0x0f0101b7,0x0f030025,0x1003005a,0x0f07000f,0x0d03005e,0x0d05000d,0x1002005d,0x1006005b,
0x00000000,0x01010001,0x0f05000d,0x02020002,0x0f040015,0x0d06000b,0x0d05000c,0x0d04003e,
0x100102b7,0x0d040031,0x0f05000e,0x0d06000c,0x0f030024,0x0d05000e,0x0d05000d,0x0f08000f,
0x00000000,0x01010001,0x0d06000b,0x0d05001b,0x0f020043,0x0d06000c,0x0d05000e,0x0f070011,
0x00000000,0x0d030070,0x0d06000c,0x0f070010,0x0d03005f,0x0d05000f,0x0d05000f,0x0d09000d,
0x00000000,0x01010001,0x0f02006f,0x02020002,0x0f040016,0x0f040010,0x0f030011,0x03030003,
0x00000000,0x0f040017,0x0f020070,0x0f08000e,0x0f030025,0x0f07000f,0x0f030012,0x0f050071,
0x00000000,0x01010001,0x0f040010,0x0d04003a,0x0f020044,0x0f040011,0x0f030013,0x0f08000f,
0x00000000,0x0f030026,0x0f040011,0x0f070011,0x0d030060,0x0f070010,0x0f030014,0x0f070012,
0x00000000,0x01010001,0x0f040010,0x02020002,0x0f040017,0x0f08000e,0x0f07000f,0x0f05005f,
0x00000000,0x0d040032,0x0f040011,0x0f08000f,0x0f030026,0x0f070011,0x0f070010,0x0f070012,
0x00000000,0x01010001,0x0f08000e,0x0d030079,0x0f020045,0x0f08000f,0x0f070011,0x0f070012,
0x00000000,0x0d030071,0x0f08000f,0x0f070012,0x0d030061,0x0f070012,0x0f070012,0x0f0b0010,
0x00000000,0x01010001,0x04010002,0x02020002,0x08010008,0x05020003,0x05010004,0x03030003,
0x0f0200b9,0x0f04000f,0x04010003,0x06030004,0x0f030016,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f0200dd,0x05020003,0x0d06000d,0x0d040023,0x05020004,0x05010006,0x07040005,
0x0f0101b8,0x0d050014,0x05020004,0x07030006,0x0d030062,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x08020004,0x100300b0,0x08010009,0x06030004,0x06020005,0x0d060011,
0x100102b8,0x0d040033,0x08020005,0x06030005,0x0f030017,0x06020007,0x06020006,0x08050006,
0x00000000,0x1002015d,0x06030004,0x0d05001c,0x0d040024,0x06030005,0x06020007,0x08040007,
0x00000000,0x0d030072,0x06030005,0x07030007,0x0d030063,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0d030011,0x02020002,0x0801000a,0x0f05000b,0x0f04000c,0x03030003,
0x00000000,0x0f040010,0x0d030012,0x07040005,0x0f030018,0x07030006,0x0f04000d,0x0d060013,
0x00000000,0x01010001,0x0f05000b,0x0d04003b,0x0d040025,0x0f05000c,0x0f04000e,0x07040006,
0x00000000,0x0d050015,0x0f05000c,0x07030008,0x0d030064,0x07030007,0x0f04000f,0x0f08000d,
0x00000000,0x01010001,0x0f04000c,0x02020002,0x0801000b,0x07040005,0x07030006,0x0d050020,
0x00000000,0x0d040034,0x0f04000d,0x07040006,0x0f030019,0x07030008,0x07030007,0x0f07000e,
0x00000000,0x01010001,0x07040005,0x0d03007a,0x0d040026,0x07040006,0x07030008,0x08040008,
0x00000000,0x0d030073,0x07040006,0x07030009,0x0d030065,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0801000c,0x0d030012,0x0d020013,0x03030003,
0x0f0200ba,0x0f040011,0x04010003,0x0d070009,0x0f03001a,0x0d06000a,0x0d020014,0x04040004,
0x00000000,0x01010001,0x0d030012,0x0d06000e,0x0d040027,0x0d030013,0x0d020015,0x08050006,
0x0f0101b9,0x0d050016,0x0d030013,0x08040007,0x0d030066,0x0d06000b,0x0d020016,0x0d060014,
0x00000000,0x01010001,0x08020005,0x02020002,0x0801000d,0x0d070009,0x0d06000a,0x0d04003f,
0x100102b9,0x0d040035,0x08020006,0x0d07000a,0x0f03001b,0x0d06000c,0x0d06000b,0x08050007,
0x00000000,0x01010001,0x0d070009,0x0d05001d,0x0d040028,0x0d07000a,0x0d06000c,0x08040009,
0x00000000,0x0d030074,0x0d07000a,0x08040008,0x0d030067,0x0d06000d,0x0d06000d,0x0d0a000b,
0x00000000,0x01010001,0x0d020020,0x02020002,0x0801000e,0x0d06000a,0x0d05000b,0x03030003,
0x00000000,0x0f040012,0x0d020021,0x08050006,0x0f03001c,0x08040007,0x0d05000c,0x0d050022,
0x00000000,0x01010001,0x0d06000a,0x0d04003c,0x0d040029,0x0d06000b,0x0d05000d,0x08050007,
0x00000000,0x0d050017,0x0d06000b,0x08040009,0x0d030068,0x08040008,0x0d05000e,0x0d09000c,
0x00000000,0x01010001,0x0f04000d,0x02020002,0x0801000f,0x08050006,0x08040007,0x0f050060,
0x00000000,0x0d040036,0x0f04000e,0x08050007,0x0f03001d,0x08040009,0x08040008,0x0f07000f,
0x00000000,0x01010001,0x08050006,0x0d03007b,0x0d04002a,0x08050007,0x08040009,0x0804000a,
0x00000000,0x0d030075,0x08050007,0x0804000a,0x0d030069,0x0804000a,0x0804000a,0x08080008,
0x00000000,0x01010001,0x04010002,0x02020002,0x0f04000f,0x05020003,0x05010004,0x03030003,
0x0f0200bb,0x0f040013,0x04010003,0x1004002f,0x0f03001e,0x10030030,0x05010005,0x04040004,
0x00000000,0x0f0200de,0x05020003,0x0d06000f,0x0d04002b,0x05020004,0x05010006,0x0d07000a,
0x0f0101ba,0x0d050018,0x05020004,0x0d06000b,0x0d03006a,0x10030031,0x05010007,0x05050005,
0x00000000,0x01010001,0x0f05000b,0x02020002,0x0f040010,0x1004002f,0x10030030,0x0d060012,
0x100102ba,0x0d040037,0x0f05000c,0x10040030,0x0f03001f,0x10030032,0x10030031,0x0f08000d,
0x00000000,0x1002015e,0x1004002f,0x0d05001e,0x0d04002c,0x10040030,0x10030032,0x0f07000e,
0x00000000,0x0d030076,0x10040030,0x0d06000c,0x0d03006b,0x10030033,0x10030033,0x10070031,
0x00000000,0x01010001,0x0f030031,0x02020002,0x0f040011,0x0f05000c,0x0f04000d,0x03030003,
0x00000000,0x0f040014,0x0f030032,0x0d07000a,0x0f030020,0x0d06000b,0x0f04000e,0x0f060033,
0x00000000,0x01010001,0x0f05000c,0x0d04003d,0x0d04002d,0x0f05000d,0x0f04000f,0x0d07000b,
0x00000000,0x0d050019,0x0f05000d,0x0d06000d,0x0d03006c,0x0d06000c,0x0f040010,0x0f08000e,
0x00000000,0x01010001,0x0f04000e,0x02020002,0x0f040012,0x0d07000a,0x0d06000b,0x0d050021,
0x00000000,0x0d040038,0x0f04000f,0x0d07000b,0x0f030021,0x0d06000d,0x0d06000c,0x0f070010,
0x00000000,0x01010001,0x0d07000a,0x0d03007c,0x0d04002e,0x0d07000b,0x0d06000d,0x0f07000f,
0x00000000,0x0d030077,0x0d07000b,0x0d06000e,0x0d03006d,0x0d06000e,0x0d06000e,0x0d0a000c,
0x00000000,0x01010001,0x04010002,0x02020002,0x0f040013,0x0f04001b,0x0f03001c,0x03030003,
0x0f0200bc,0x0f040015,0x04010003,0x0d06000c,0x0f030022,0x0d05000d,0x0f03001d,0x04040004,
0x00000000,0x01010001,0x0f04001b,0x0d060010,0x0d04002f,0x0f04001c,0x0f03001e,0x0f08000d,
0x0f0101bb,0x0d05001a,0x0f04001c,0x0f07000e,0x0d03006e,0x0d05000e,0x0f03001f,0x0f07001d,
0x00000000,0x01010001,0x0f05000c,0x02020002,0x0f040014,0x0d06000c,0x0d05000d,0x0d040040,
0x100102bb,0x0d040039,0x0f05000d,0x0d06000d,0x0f030023,0x0d05000f,0x0d05000e,0x0f08000e,
0x00000000,0x01010001,0x0d06000c,0x0d05001f,0x0d040030,0x0d06000d,0x0d05000f,0x0f070010,
0x00000000,0x0d030078,0x0d06000d,0x0f07000f,0x0d03006f,0x0d050010,0x0d050010,0x0d09000e,
0x00000000,0x01010001,0x0f020070,0x02020002,0x0f040015,0x0d06000b,0x0d05000c,0x03030003,
0x00000000,0x0f040016,0x0f020071,0x0f08000d,0x0f030024,0x0f07000e,0x0d05000d,0x0f050072,
0x00000000,0x01010001,0x0d06000b,0x0d04003e,0x0d040031,0x0d06000c,0x0d05000e,0x0f08000e,
0x00000000,0x0d05001b,0x0d06000c,0x0f070010,0x0d030070,0x0f07000f,0x0d05000f,0x0d09000d,
0x00000000,0x01010001,0x0f04000f,0x02020002,0x0f040016,0x0f08000d,0x0f07000e,0x0f050061,
0x00000000,0x0d04003a,0x0f040010,0x0f08000e,0x0f030025,0x0f070010,0x0f07000f,0x0f070011,
0x00000000,0x01010001,0x0f08000d,0x0d03007d,0x0d040032,0x0f08000e,0x0f070010,0x0f070011,
0x00000000,0x0d030079,0x0f08000e,0x0f070011,0x0d030071,0x0f070011,0x0f070011,0x0f0b000f,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d02000f,0x05020003,0x05010004,0x03030003,
0x0f0200bd,0x0d06000d,0x04010003,0x06030004,0x0d050014,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f0200df,0x05020003,0x0d060011,0x0d040033,0x05020004,0x05010006,0x07040005,
0x0f0101bc,0x0d05001c,0x05020004,0x07030006,0x0d030072,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0d03000b,0x100300b1,0x0d020010,0x06030004,0x06020005,0x0d060013,
0x100102bc,0x0d04003b,0x0d03000c,0x06030005,0x0d050015,0x06020007,0x06020006,0x0d06000d,
0x00000000,0x1002015f,0x06030004,0x0d050020,0x0d040034,0x06030005,0x06020007,0x0d05000e,
0x00000000,0x0d03007a,0x06030005,0x07030007,0x0d030073,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0d030012,0x02020002,0x0d020011,0x0d070009,0x0d06000a,0x03030003,
0x00000000,0x0d06000e,0x0d030013,0x07040005,0x0d050016,0x07030006,0x0d06000b,0x0d060014,
0x00000000,0x01010001,0x0d070009,0x0d04003f,0x0d040035,0x0d07000a,0x0d06000c,0x07040006,
0x00000000,0x0d05001d,0x0d07000a,0x07030008,0x0d030074,0x07030007,0x0d06000d,0x0d0a000b,
0x00000000,0x01010001,0x0d06000a,0x02020002,0x0d020012,0x07040005,0x07030006,0x0d050022,
0x00000000,0x0d04003c,0x0d06000b,0x07040006,0x0d050017,0x07030008,0x07030007,0x0d09000c,
0x00000000,0x01010001,0x07040005,0x0d03007e,0x0d040036,0x07040006,0x07030008,0x0d05000f,
0x00000000,0x0d03007b,0x07040006,0x07030009,0x0d030075,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d020013,0x0f03003a,0x0f02003b,0x03030003,
0x0f0200be,0x0d06000f,0x04010003,0x0d07000a,0x0d050018,0x0d06000b,0x0f02003c,0x04040004,
0x00000000,0x01010001,0x0f03003a,0x0d060012,0x0d040037,0x0f03003b,0x0f02003d,0x0d06000d,
0x0f0101bd,0x0d05001e,0x0f03003b,0x0d05000e,0x0d030076,0x0d06000c,0x0f02003e,0x0f06003c,
0x00000000,0x01010001,0x0d03000c,0x02020002,0x0d020014,0x0d07000a,0x0d06000b,0x0d040041,
0x100102bd,0x0d04003d,0x0d03000d,0x0d07000b,0x0d050019,0x0d06000d,0x0d06000c,0x0d06000e,
0x00000000,0x01010001,0x0d07000a,0x0d050021,0x0d040038,0x0d07000b,0x0d06000d,0x0d050010,
0x00000000,0x0d03007c,0x0d07000b,0x0d05000f,0x0d030077,0x0d06000e,0x0d06000e,0x0d0a000c,
0x00000000,0x01010001,0x0d020021,0x02020002,0x0d020015,0x0d06000c,0x0d05000d,0x03030003,
0x00000000,0x0d060010,0x0d020022,0x0d06000d,0x0d05001a,0x0d05000e,0x0d05000e,0x0d050023,
0x00000000,0x01010001,0x0d06000c,0x0d040040,0x0d040039,0x0d06000d,0x0d05000f,0x0d06000e,
0x00000000,0x0d05001f,0x0d06000d,0x0d050010,0x0d030078,0x0d05000f,0x0d050010,0x0d09000e,
0x00000000,0x01010001,0x0d06000b,0x02020002,0x0d020016,0x0d06000d,0x0d05000e,0x0f050062,
0x00000000,0x0d04003e,0x0d06000c,0x0d06000e,0x0d05001b,0x0d050010,0x0d05000f,0x0d09000d,
0x00000000,0x01010001,0x0d06000d,0x0d03007f,0x0d04003a,0x0d06000e,0x0d050010,0x0d050011,
0x00000000,0x0d03007d,0x0d06000e,0x0d050011,0x0d030079,0x0d050011,0x0d050011,0x0d09000f,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d06000d,0x05020003,0x05010004,0x03030003,
0x0f0200bf,0x0d060011,0x04010003,0x06030004,0x0d05001c,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f0200e0,0x05020003,0x0d060013,0x0d04003b,0x05020004,0x05010006,0x0d06000d,
0x0f0101be,0x0d050020,0x05020004,0x0d05000e,0x0d03007a,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0d070009,0x02020002,0x0d06000e,0x06030004,0x06020005,0x0d060014,
0x100102be,0x0d04003f,0x0d07000a,0x06030005,0x0d05001d,0x06020007,0x06020006,0x0d0a000b,
0x00000000,0x10020160,0x06030004,0x0d050022,0x0d04003c,0x06030005,0x06020007,0x0d09000c,
0x00000000,0x0d03007e,0x06030005,0x0d05000f,0x0d03007b,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0f030032,0x02020002,0x0d06000f,0x0d07000a,0x0d06000b,0x03030003,
0x00000000,0x0d060012,0x0f030033,0x0d06000d,0x0d05001e,0x0d05000e,0x0d06000c,0x0f060034,
0x00000000,0x01010001,0x0d07000a,0x0d040041,0x0d04003d,0x0d07000b,0x0d06000d,0x0d06000e,
0x00000000,0x0d050021,0x0d07000b,0x0d050010,0x0d03007c,0x0d05000f,0x0d06000e,0x0d0a000c,
0x00000000,0x01010001,0x0d06000c,0x02020002,0x0d060010,0x0d06000d,0x0d05000e,0x0d050023,
0x00000000,0x0d040040,0x0d06000d,0x0d06000e,0x0d05001f,0x0d050010,0x0d05000f,0x0d09000e,
0x00000000,0x01010001,0x0d06000d,0x0d030080,0x0d04003e,0x0d06000e,0x0d050010,0x0d09000d,
0x00000000,0x0d03007f,0x0d06000e,0x0d050011,0x0d03007d,0x0d050011,0x0d050011,0x0d09000f,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d060011,0x1003005a,0x1002005b,0x03030003,
0x0f0200c0,0x0d060013,0x04010003,0x0d06000d,0x0d050020,0x0d05000e,0x1002005c,0x04040004,
0x00000000,0x01010001,0x1003005a,0x0d060014,0x0d04003f,0x1003005b,0x1002005d,0x0d0a000b,
0x0f0101bf,0x0d050022,0x1003005b,0x0d09000c,0x0d03007e,0x0d05000f,0x1002005e,0x1006005c,
0x00000000,0x01010001,0x0d07000a,0x02020002,0x0d060012,0x0d06000d,0x0d05000e,0x0d040042,
0x100102bf,0x0d040041,0x0d07000b,0x0d06000e,0x0d050021,0x0d050010,0x0d05000f,0x0d0a000c,
0x00000000,0x01010001,0x0d06000d,0x0d050023,0x0d040040,0x0d06000e,0x0d050010,0x0d09000e,
0x00000000,0x0d030080,0x0d06000e,0x0d09000d,0x0d03007f,0x0d050011,0x0d050011,0x0d09000f,
0x00000000,0x01010001,0x0f020071,0x02020002,0x0d060013,0x0d06000d,0x0d05000e,0x03030003,
0x00000000,0x0d060014,0x0f020072,0x0d0a000b,0x0d050022,0x0d09000c,0x0d05000f,0x0f050073,
0x00000000,0x01010001,0x0d06000d,0x0d040042,0x0d040041,0x0d06000e,0x0d050010,0x0d0a000c,
0x00000000,0x0d050023,0x0d06000e,0x0d09000e,0x0d030080,0x0d09000d,0x0d050011,0x0d09000f,
0x00000000,0x01010001,0x0d06000d,0x02020002,0x0d060014,0x0d0a000b,0x0d09000c,0x0f050063,
0x00000000,0x0d040042,0x0d06000e,0x0d0a000c,0x0d050023,0x0d09000e,0x0d09000d,0x0d09000f,
0x00000000,0x01010001,0x0d0a000b,0x0d030081,0x0d040042,0x0d0a000c,0x0d09000e,0x0d09000f,
0x00000000,0x0d030081,0x0d0a000c,0x0d09000f,0x0d030081,0x0d09000f,0x0d09000f,0x0d0d000d,
0x00000000,0x01010001,0x04010002,0x02020002,0x08010008,0x05020003,0x05010004,0x03030003,
0x0f0200c1,0x09020009,0x04010003,0x06030004,0x09010010,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f0200e1,0x05020003,0x0a03000a,0x0a010020,0x05020004,0x05010006,0x07040005,
0x0f0101c0,0x0a020011,0x05020004,0x07030006,0x0b010040,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x08020004,0x100300b2,0x08010009,0x06030004,0x06020005,0x0b04000b,
0x100102c0,0x0b020021,0x08020005,0x06030005,0x09010011,0x06020007,0x06020006,0x08050006,
0x00000000,0x10020161,0x06030004,0x0b030012,0x0a010021,0x06030005,0x06020007,0x08040007,
0x00000000,0x0c020041,0x06030005,0x07030007,0x0b010041,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0c02000a,0x02020002,0x0801000a,0x09030005,0x09020006,0x03030003,
0x00000000,0x0902000a,0x0c02000b,0x07040005,0x09010012,0x07030006,0x09020007,0x0c05000c,
0x00000000,0x01010001,0x09030005,0x0c030022,0x0a010022,0x09030006,0x09020008,0x07040006,
0x00000000,0x0a020012,0x09030006,0x07030008,0x0b010042,0x07030007,0x09020009,0x09060007,
0x00000000,0x01010001,0x09020006,0x02020002,0x0801000b,0x07040005,0x07030006,0x0c040013,
0x00000000,0x0b020022,0x09020007,0x07040006,0x09010013,0x07030008,0x07030007,0x09050008,
0x00000000,0x01010001,0x07040005,0x0f050044,0x0a010023,0x07040006,0x07030008,0x08040008,
0x00000000,0x0c020042,0x07040006,0x07030009,0x0b010043,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0801000c,0x0f05000d,0x0f04000e,0x03030003,
0x0f0200c2,0x0902000b,0x04010003,0x0a040006,0x09010014,0x0a030007,0x0f04000f,0x04040004,
0x00000000,0x01010001,0x0f05000d,0x0a03000b,0x0a010024,0x0f05000e,0x0f040010,0x08050006,
0x0f0101c1,0x0a020013,0x0f05000e,0x08040007,0x0b010044,0x0a030008,0x0f040011,0x0f08000f,
0x00000000,0x01010001,0x08020005,0x02020002,0x0801000d,0x0a040006,0x0a030007,0x0f060025,
0x100102c1,0x0b020023,0x08020006,0x0a040007,0x09010015,0x0a030009,0x0a030008,0x08050007,
0x00000000,0x01010001,0x0a040006,0x0b030013,0x0a010025,0x0a040007,0x0a030009,0x08040009,
0x00000000,0x0c020043,0x0a040007,0x08040008,0x0b010045,0x0a03000a,0x0a03000a,0x0a070008,
0x00000000,0x01010001,0x0f040014,0x02020002,0x0801000e,0x0a030007,0x0a020008,0x03030003,
0x00000000,0x0902000c,0x0f040015,0x08050006,0x09010016,0x08040007,0x0a020009,0x0f070016,
0x00000000,0x01010001,0x0a030007,0x0c030023,0x0a010026,0x0a030008,0x0a02000a,0x08050007,
0x00000000,0x0a020014,0x0a030008,0x08040009,0x0b010046,0x08040008,0x0a02000b,0x0a060009,
0x00000000,0x01010001,0x09020007,0x02020002,0x0801000f,0x08050006,0x08040007,0x0f050064,
0x00000000,0x0b020024,0x09020008,0x08050007,0x09010017,0x08040009,0x08040008,0x09050009,
0x00000000,0x01010001,0x08050006,0x0f050045,0x0a010027,0x08050007,0x08040009,0x0804000a,
0x00000000,0x0c020044,0x08050007,0x0804000a,0x0b010047,0x0804000a,0x0804000a,0x08080008,
0x00000000,0x01010001,0x04010002,0x02020002,0x09020009,0x05020003,0x05010004,0x03030003,
0x0f0200c3,0x0902000d,0x04010003,0x0f050011,0x09010018,0x0f040012,0x05010005,0x04040004,
0x00000000,0x0f0200e2,0x05020003,0x0a03000c,0x0a010028,0x05020004,0x05010006,0x0b050007,
0x0f0101c2,0x0a020015,0x05020004,0x0b040008,0x0b010048,0x0f040013,0x05010007,0x05050005,
0x00000000,0x01010001,0x09030005,0x02020002,0x0902000a,0x0f050011,0x0f040012,0x0b04000c,
0x100102c2,0x0b020025,0x09030006,0x0f050012,0x09010019,0x0f040014,0x0f040013,0x09060007,
0x00000000,0x10020162,0x0f050011,0x0b030014,0x0a010029,0x0f050012,0x0f040014,0x09050008,
0x00000000,0x0c020045,0x0f050012,0x0b040009,0x0b010049,0x0f040015,0x0f040015,0x0f080013,
0x00000000,0x01010001,0x0f030033,0x02020002,0x0902000b,0x09030006,0x09020007,0x03030003,
0x00000000,0x0902000e,0x0f030034,0x0b050007,0x0901001a,0x0b040008,0x09020008,0x0f060035,
0x00000000,0x01010001,0x09030006,0x0c030024,0x0a01002a,0x09030007,0x09020009,0x0b050008,
0x00000000,0x0a020016,0x09030007,0x0b04000a,0x0b01004a,0x0b040009,0x0902000a,0x09060008,
0x00000000,0x01010001,0x09020008,0x02020002,0x0902000c,0x0b050007,0x0b040008,0x0c040014,
0x00000000,0x0b020026,0x09020009,0x0b050008,0x0901001b,0x0b04000a,0x0b040009,0x0905000a,
0x00000000,0x01010001,0x0b050007,0x0f050046,0x0a01002b,0x0b050008,0x0b04000a,0x09050009,
0x00000000,0x0c020046,0x0b050008,0x0b04000b,0x0b01004b,0x0b04000b,0x0b04000b,0x0b080009,
0x00000000,0x01010001,0x04010002,0x02020002,0x0902000d,0x0f04001c,0x0f03001d,0x03030003,
0x0f0200c4,0x0902000f,0x04010003,0x0b040008,0x0901001c,0x0b030009,0x0f03001e,0x04040004,
0x00000000,0x01010001,0x0f04001c,0x0a03000d,0x0a01002c,0x0f04001d,0x0f03001f,0x09060007,
0x0f0101c3,0x0a020017,0x0f04001d,0x09050008,0x0b01004c,0x0b03000a,0x0f030020,0x0f07001e,
0x00000000,0x01010001,0x09030006,0x02020002,0x0902000e,0x0b040008,0x0b030009,0x0f060026,
0x100102c3,0x0b020027,0x09030007,0x0b040009,0x0901001d,0x0b03000b,0x0b03000a,0x09060008,
0x00000000,0x01010001,0x0b040008,0x0b030015,0x0a01002d,0x0b040009,0x0b03000b,0x0905000a,
0x00000000,0x0c020047,0x0b040009,0x09050009,0x0b01004d,0x0b03000c,0x0b03000c,0x0b07000a,
0x00000000,0x01010001,0x0f020072,0x02020002,0x0902000f,0x0a030008,0x0a020009,0x03030003,
0x00000000,0x09020010,0x0f020073,0x09060007,0x0901001e,0x09050008,0x0a02000a,0x0f050074,
0x00000000,0x01010001,0x0a030008,0x0c030025,0x0a01002e,0x0a030009,0x0a02000b,0x09060008,
0x00000000,0x0a020018,0x0a030009,0x0905000a,0x0b01004e,0x09050009,0x0a02000c,0x0a06000a,
0x00000000,0x01010001,0x09020009,0x02020002,0x09020010,0x09060007,0x09050008,0x0f050065,
0x00000000,0x0b020028,0x0902000a,0x09060008,0x0901001f,0x0905000a,0x09050009,0x0905000b,
0x00000000,0x01010001,0x09060007,0x0f050047,0x0a01002f,0x09060008,0x0905000a,0x0905000b,
0x00000000,0x0c020048,0x09060008,0x0905000b,0x0b01004f,0x0905000b,0x0905000b,0x09090009,
0x00000000,0x01010001,0x04010002,0x02020002,0x0c02000a,0x05020003,0x05010004,0x03030003,
0x0f0200c5,0x0a03000a,0x04010003,0x06030004,0x0a020011,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f0200e3,0x05020003,0x0a03000e,0x0a010030,0x05020004,0x05010006,0x0f050013,
0x0f0101c4,0x0a020019,0x05020004,0x0f040014,0x0b010050,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0c030006,0x100300b3,0x0c02000b,0x06030004,0x06020005,0x0b04000d,
0x100102c4,0x0b020029,0x0c030007,0x06030005,0x0a020012,0x06020007,0x06020006,0x0c060008,
0x00000000,0x10020163,0x06030004,0x0b030016,0x0a010031,0x06030005,0x06020007,0x0c050009,
0x00000000,0x0c020049,0x06030005,0x0f040015,0x0b010051,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0c02000b,0x02020002,0x0c02000c,0x0a040006,0x0a030007,0x03030003,
0x00000000,0x0a03000b,0x0c02000c,0x0f050013,0x0a020013,0x0f040014,0x0a030008,0x0c05000d,
0x00000000,0x01010001,0x0a040006,0x0c030026,0x0a010032,0x0a040007,0x0a030009,0x0f050014,
0x00000000,0x0a02001a,0x0a040007,0x0f040016,0x0b010052,0x0f040015,0x0a03000a,0x0a070008,
0x00000000,0x01010001,0x0a030007,0x02020002,0x0c02000d,0x0f050013,0x0f040014,0x0c040015,
0x00000000,0x0b02002a,0x0a030008,0x0f050014,0x0a020014,0x0f040016,0x0f040015,0x0a060009,
0x00000000,0x01010001,0x0f050013,0x0f050048,0x0a010033,0x0f050014,0x0f040016,0x0c05000a,
0x00000000,0x0c02004a,0x0f050014,0x0f040017,0x0b010053,0x0f040017,0x0f040017,0x0f080015,
0x00000000,0x01010001,0x04010002,0x02020002,0x0c02000e,0x0f03003b,0x0f02003c,0x03030003,
0x0f0200c6,0x0a03000c,0x04010003,0x0a040007,0x0a020015,0x0a030008,0x0f02003d,0x04040004,
0x00000000,0x01010001,0x0f03003b,0x0a03000f,0x0a010034,0x0f03003c,0x0f02003e,0x0c060008,
0x0f0101c5,0x0a02001b,0x0f03003c,0x0c050009,0x0b010054,0x0a030009,0x0f02003f,0x0f06003d,
0x00000000,0x01010001,0x0c030007,0x02020002,0x0c02000f,0x0a040007,0x0a030008,0x0f060027,
0x100102c5,0x0b02002b,0x0c030008,0x0a040008,0x0a020016,0x0a03000a,0x0a030009,0x0c060009,
0x00000000,0x01010001,0x0a040007,0x0b030017,0x0a010035,0x0a040008,0x0a03000a,0x0c05000b,
0x00000000,0x0c02004b,0x0a040008,0x0c05000a,0x0b010055,0x0a03000b,0x0a03000b,0x0a070009,
0x00000000,0x01010001,0x0f040015,0x02020002,0x0c020010,0x0a030009,0x0a02000a,0x03030003,
0x00000000,0x0a03000d,0x0f040016,0x0c060008,0x0a020017,0x0c050009,0x0a02000b,0x0f070017,
0x00000000,0x01010001,0x0a030009,0x0c030027,0x0a010036,0x0a03000a,0x0a02000c,0x0c060009,
0x00000000,0x0a02001c,0x0a03000a,0x0c05000b,0x0b010056,0x0c05000a,0x0a02000d,0x0a06000b,
0x00000000,0x01010001,0x0a030008,0x02020002,0x0c020011,0x0c060008,0x0c050009,0x0f050066,
0x00000000,0x0b02002c,0x0a030009,0x0c060009,0x0a020018,0x0c05000b,0x0c05000a,0x0a06000a,
0x00000000,0x01010001,0x0c060008,0x0f050049,0x0a010037,0x0c060009,0x0c05000b,0x0c05000c,
0x00000000,0x0c02004c,0x0c060009,0x0c05000c,0x0b010057,0x0c05000c,0x0c05000c,0x0c09000a,
0x00000000,0x01010001,0x04010002,0x02020002,0x0a03000a,0x05020003,0x05010004,0x03030003,
0x0f0200c7,0x0a03000e,0x04010003,0x0f040020,0x0a020019,0x0f030021,0x05010005,0x04040004,
0x00000000,0x0f0200e4,0x05020003,0x0a030010,0x0a010038,0x05020004,0x05010006,0x0c050009,
0x0f0101c6,0x0a02001d,0x05020004,0x0c04000a,0x0b010058,0x0f030022,0x05010007,0x05050005,
0x00000000,0x01010001,0x0a040006,0x02020002,0x0a03000b,0x0f040020,0x0f030021,0x0b04000e,
0x100102c6,0x0b02002d,0x0a040007,0x0f040021,0x0a02001a,0x0f030023,0x0f030022,0x0a070008,
0x00000000,0x10020164,0x0f040020,0x0b030018,0x0a010039,0x0f040021,0x0f030023,0x0a060009,
0x00000000,0x0c02004d,0x0f040021,0x0c04000b,0x0b010059,0x0f030024,0x0f030024,0x0f070022,
0x00000000,0x01010001,0x0f030034,0x02020002,0x0a03000c,0x0a040007,0x0a030008,0x03030003,
0x00000000,0x0a03000f,0x0f030035,0x0c050009,0x0a02001b,0x0c04000a,0x0a030009,0x0f060036,
0x00000000,0x01010001,0x0a040007,0x0c030028,0x0a01003a,0x0a040008,0x0a03000a,0x0c05000a,
0x00000000,0x0a02001e,0x0a040008,0x0c04000c,0x0b01005a,0x0c04000b,0x0a03000b,0x0a070009,
0x00000000,0x01010001,0x0a030009,0x02020002,0x0a03000d,0x0c050009,0x0c04000a,0x0c040016,
0x00000000,0x0b02002e,0x0a03000a,0x0c05000a,0x0a02001c,0x0c04000c,0x0c04000b,0x0a06000b,
0x00000000,0x01010001,0x0c050009,0x0f05004a,0x0a01003b,0x0c05000a,0x0c04000c,0x0a06000a,
0x00000000,0x0c02004e,0x0c05000a,0x0c04000d,0x0b01005b,0x0c04000d,0x0c04000d,0x0c08000b,
0x00000000,0x01010001,0x04010002,0x02020002,0x0a03000e,0x1003005b,0x1002005c,0x03030003,
0x0f0200c8,0x0a030010,0x04010003,0x0b040009,0x0a02001d,0x0b03000a,0x1002005d,0x04040004,
0x00000000,0x01010001,0x1003005b,0x0a030011,0x0a01003c,0x1003005c,0x1002005e,0x0a070008,
0x0f0101c7,0x0a02001f,0x1003005c,0x0a060009,0x0b01005c,0x0b03000b,0x1002005f,0x1006005d,
0x00000000,0x01010001,0x0a040007,0x02020002,0x0a03000f,0x0b040009,0x0b03000a,0x0f060028,
0x100102c7,0x0b02002f,0x0a040008,0x0b04000a,0x0a02001e,0x0b03000c,0x0b03000b,0x0a070009,
0x00000000,0x01010001,0x0b040009,0x0b030019,0x0a01003d,0x0b04000a,0x0b03000c,0x0a06000b,
0x00000000,0x0c02004f,0x0b04000a,0x0a06000a,0x0b01005d,0x0b03000d,0x0b03000d,0x0b07000b,
0x00000000,0x01010001,0x0f020073,0x02020002,0x0a030010,0x0a03000a,0x0a02000b,0x03030003,
0x00000000,0x0a030011,0x0f020074,0x0a070008,0x0a02001f,0x0a060009,0x0a02000c,0x0f050075,
0x00000000,0x01010001,0x0a03000a,0x0c030029,0x0a01003e,0x0a03000b,0x0a02000d,0x0a070009,
0x00000000,0x0a020020,0x0a03000b,0x0a06000b,0x0b01005e,0x0a06000a,0x0a02000e,0x0a06000c,
0x00000000,0x01010001,0x0a03000a,0x02020002,0x0a030011,0x0a070008,0x0a060009,0x0f050067,
0x00000000,0x0b020030,0x0a03000b,0x0a070009,0x0a020020,0x0a06000b,0x0a06000a,0x0a06000c,
0x00000000,0x01010001,0x0a070008,0x0f05004b,0x0a01003f,0x0a070009,0x0a06000b,0x0a06000c,
0x00000000,0x0c020050,0x0a070009,0x0a06000c,0x0b01005f,0x0a06000c,0x0a06000c,0x0a0a000a,
0x00000000,0x01010001,0x04010002,0x02020002,0x10020013,0x05020003,0x05010004,0x03030003,
0x0f0200c9,0x0f05000d,0x04010003,0x06030004,0x0f040014,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f0200e5,0x05020003,0x0b04000b,0x0b020021,0x05020004,0x05010006,0x07040005,
0x0f0101c8,0x0b030012,0x05020004,0x07030006,0x0b010060,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x1003000f,0x100300b4,0x10020014,0x06030004,0x06020005,0x0b04000f,
0x100102c8,0x0b020031,0x10030010,0x06030005,0x0f040015,0x06020007,0x06020006,0x10060011,
0x00000000,0x10020165,0x06030004,0x0b03001a,0x0b020022,0x06030005,0x06020007,0x10050012,
0x00000000,0x0c020051,0x06030005,0x07030007,0x0b010061,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0c02000c,0x02020002,0x10020015,0x0f060009,0x0f05000a,0x03030003,
0x00000000,0x0f05000e,0x0c02000d,0x07040005,0x0f040016,0x07030006,0x0f05000b,0x0c05000e,
0x00000000,0x01010001,0x0f060009,0x0c03002a,0x0b020023,0x0f06000a,0x0f05000c,0x07040006,
0x00000000,0x0b030013,0x0f06000a,0x07030008,0x0b010062,0x07030007,0x0f05000d,0x0f09000b,
0x00000000,0x01010001,0x0f05000a,0x02020002,0x10020016,0x07040005,0x07030006,0x0c040017,
0x00000000,0x0b020032,0x0f05000b,0x07040006,0x0f040017,0x07030008,0x07030007,0x0f08000c,
0x00000000,0x01010001,0x07040005,0x0f05004c,0x0b020024,0x07040006,0x07030008,0x10050013,
0x00000000,0x0c020052,0x07040006,0x07030009,0x0b010063,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x10020017,0x0f05000e,0x0f04000f,0x03030003,
0x0f0200ca,0x0f05000f,0x04010003,0x0b050007,0x0f040018,0x0b040008,0x0f040010,0x04040004,
0x00000000,0x01010001,0x0f05000e,0x0b04000c,0x0b020025,0x0f05000f,0x0f040011,0x10060011,
0x0f0101c9,0x0b030014,0x0f05000f,0x10050012,0x0b010064,0x0b040009,0x0f040012,0x0f080010,
0x00000000,0x01010001,0x10030010,0x02020002,0x10020018,0x0b050007,0x0b040008,0x0f060029,
0x100102c9,0x0b020033,0x10030011,0x0b050008,0x0f040019,0x0b04000a,0x0b040009,0x10060012,
0x00000000,0x01010001,0x0b050007,0x0b03001b,0x0b020026,0x0b050008,0x0b04000a,0x10050014,
0x00000000,0x0c020053,0x0b050008,0x10050013,0x0b010065,0x0b04000b,0x0b04000b,0x0b080009,
0x00000000,0x01010001,0x0f040016,0x02020002,0x10020019,0x0b040008,0x0b030009,0x03030003,
0x00000000,0x0f050010,0x0f040017,0x10060011,0x0f04001a,0x10050012,0x0b03000a,0x0f070018,
0x00000000,0x01010001,0x0b040008,0x0c03002b,0x0b020027,0x0b040009,0x0b03000b,0x10060012,
0x00000000,0x0b030015,0x0b040009,0x10050014,0x0b010066,0x10050013,0x0b03000c,0x0b07000a,
0x00000000,0x01010001,0x0f05000b,0x02020002,0x1002001a,0x10060011,0x10050012,0x0f050068,
0x00000000,0x0b020034,0x0f05000c,0x10060012,0x0f04001b,0x10050014,0x10050013,0x0f08000d,
0x00000000,0x01010001,0x10060011,0x0f05004d,0x0b020028,0x10060012,0x10050014,0x10050015,
0x00000000,0x0c020054,0x10060012,0x10050015,0x0b010067,0x10050015,0x10050015,0x10090013,
0x00000000,0x01010001,0x04010002,0x02020002,0x0f05000d,0x05020003,0x05010004,0x03030003,
0x0f0200cb,0x0f050011,0x04010003,0x10040030,0x0f04001c,0x10030031,0x05010005,0x04040004,
0x00000000,0x0f0200e6,0x05020003,0x0b04000d,0x0b020029,0x05020004,0x05010006,0x0b050008,
0x0f0101ca,0x0b030016,0x05020004,0x0b040009,0x0b010068,0x10030032,0x05010007,0x05050005,
0x00000000,0x01010001,0x0f060009,0x02020002,0x0f05000e,0x10040030,0x10030031,0x0b040010,
0x100102ca,0x0b020035,0x0f06000a,0x10040031,0x0f04001d,0x10030033,0x10030032,0x0f09000b,
0x00000000,0x10020166,0x10040030,0x0b03001c,0x0b02002a,0x10040031,0x10030033,0x0f08000c,
0x00000000,0x0c020055,0x10040031,0x0b04000a,0x0b010069,0x10030034,0x10030034,0x10070032,
0x00000000,0x01010001,0x0f030035,0x02020002,0x0f05000f,0x0f06000a,0x0f05000b,0x03030003,
0x00000000,0x0f050012,0x0f030036,0x0b050008,0x0f04001e,0x0b040009,0x0f05000c,0x0f060037,
0x00000000,0x01010001,0x0f06000a,0x0c03002c,0x0b02002b,0x0f06000b,0x0f05000d,0x0b050009,
0x00000000,0x0b030017,0x0f06000b,0x0b04000b,0x0b01006a,0x0b04000a,0x0f05000e,0x0f09000c,
0x00000000,0x01010001,0x0f05000c,0x02020002,0x0f050010,0x0b050008,0x0b040009,0x0c040018,
0x00000000,0x0b020036,0x0f05000d,0x0b050009,0x0f04001f,0x0b04000b,0x0b04000a,0x0f08000e,
0x00000000,0x01010001,0x0b050008,0x0f05004e,0x0b02002c,0x0b050009,0x0b04000b,0x0f08000d,
0x00000000,0x0c020056,0x0b050009,0x0b04000c,0x0b01006b,0x0b04000c,0x0b04000c,0x0b08000a,
0x00000000,0x01010001,0x04010002,0x02020002,0x0f050011,0x0f04001d,0x0f03001e,0x03030003,
0x0f0200cc,0x0f050013,0x04010003,0x0b04000a,0x0f040020,0x0b03000b,0x0f03001f,0x04040004,
0x00000000,0x01010001,0x0f04001d,0x0b04000e,0x0b02002d,0x0f04001e,0x0f030020,0x0f09000b,
0x0f0101cb,0x0b030018,0x0f04001e,0x0f08000c,0x0b01006c,0x0b03000c,0x0f030021,0x0f07001f,
0x00000000,0x01010001,0x0f06000a,0x02020002,0x0f050012,0x0b04000a,0x0b03000b,0x0f06002a,
0x100102cb,0x0b020037,0x0f06000b,0x0b04000b,0x0f040021,0x0b03000d,0x0b03000c,0x0f09000c,
0x00000000,0x01010001,0x0b04000a,0x0b03001d,0x0b02002e,0x0b04000b,0x0b03000d,0x0f08000e,
0x00000000,0x0c020057,0x0b04000b,0x0f08000d,0x0b01006d,0x0b03000e,0x0b03000e,0x0b07000c,
0x00000000,0x01010001,0x0f020074,0x02020002,0x0f050013,0x0b040009,0x0b03000a,0x03030003,
0x00000000,0x0f050014,0x0f020075,0x0f09000b,0x0f040022,0x0f08000c,0x0b03000b,0x0f050076,
0x00000000,0x01010001,0x0b040009,0x0c03002d,0x0b02002f,0x0b04000a,0x0b03000c,0x0f09000c,
0x00000000,0x0b030019,0x0b04000a,0x0f08000e,0x0b01006e,0x0f08000d,0x0b03000d,0x0b07000b,
0x00000000,0x01010001,0x0f05000d,0x02020002,0x0f050014,0x0f09000b,0x0f08000c,0x0f050069,
0x00000000,0x0b020038,0x0f05000e,0x0f09000c,0x0f040023,0x0f08000e,0x0f08000d,0x0f08000f,
0x00000000,0x01010001,0x0f09000b,0x0f05004f,0x0b020030,0x0f09000c,0x0f08000e,0x0f08000f,
0x00000000,0x0c020058,0x0f09000c,0x0f08000f,0x0b01006f,0x0f08000f,0x0f08000f,0x0f0c000d,
0x00000000,0x01010001,0x04010002,0x02020002,0x0f04000e,0x05020003,0x05010004,0x03030003,
0x0f0200cd,0x0b04000b,0x04010003,0x06030004,0x0b030012,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f0200e7,0x05020003,0x0b04000f,0x0b020031,0x05020004,0x05010006,0x1005001b,
0x0f0101cc,0x0b03001a,0x05020004,0x1004001c,0x0b010070,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0f05000a,0x100300b5,0x0f04000f,0x06030004,0x06020005,0x0b040011,
0x100102cc,0x0b020039,0x0f05000b,0x06030005,0x0b030013,0x06020007,0x06020006,0x0f08000c,
0x00000000,0x10020167,0x06030004,0x0b03001e,0x0b020032,0x06030005,0x06020007,0x0f07000d,
0x00000000,0x0c020059,0x06030005,0x1004001d,0x0b010071,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0c02000d,0x02020002,0x0f040010,0x0b050007,0x0b040008,0x03030003,
0x00000000,0x0b04000c,0x0c02000e,0x1005001b,0x0b030014,0x1004001c,0x0b040009,0x0c05000f,
0x00000000,0x01010001,0x0b050007,0x0c03002e,0x0b020033,0x0b050008,0x0b04000a,0x1005001c,
0x00000000,0x0b03001b,0x0b050008,0x1004001e,0x0b010072,0x1004001d,0x0b04000b,0x0b080009,
0x00000000,0x01010001,0x0b040008,0x02020002,0x0f040011,0x1005001b,0x1004001c,0x0c040019,
0x00000000,0x0b02003a,0x0b040009,0x1005001c,0x0b030015,0x1004001e,0x1004001d,0x0b07000a,
0x00000000,0x01010001,0x1005001b,0x0f050050,0x0b020034,0x1005001c,0x1004001e,0x0f07000e,
0x00000000,0x0c02005a,0x1005001c,0x1004001f,0x0b010073,0x1004001f,0x1004001f,0x1008001d,
0x00000000,0x01010001,0x04010002,0x02020002,0x0f040012,0x0f03003c,0x0f02003d,0x03030003,
0x0f0200ce,0x0b04000d,0x04010003,0x0b050008,0x0b030016,0x0b040009,0x0f02003e,0x04040004,
0x00000000,0x01010001,0x0f03003c,0x0b040010,0x0b020035,0x0f03003d,0x0f02003f,0x0f08000c,
0x0f0101cd,0x0b03001c,0x0f03003d,0x0f07000d,0x0b010074,0x0b04000a,0x0f020040,0x0f06003e,
0x00000000,0x01010001,0x0f05000b,0x02020002,0x0f040013,0x0b050008,0x0b040009,0x0f06002b,
0x100102cd,0x0b02003b,0x0f05000c,0x0b050009,0x0b030017,0x0b04000b,0x0b04000a,0x0f08000d,
0x00000000,0x01010001,0x0b050008,0x0b03001f,0x0b020036,0x0b050009,0x0b04000b,0x0f07000f,
0x00000000,0x0c02005b,0x0b050009,0x0f07000e,0x0b010075,0x0b04000c,0x0b04000c,0x0b08000a,
0x00000000,0x01010001,0x0f040017,0x02020002,0x0f040014,0x0b04000a,0x0b03000b,0x03030003,
0x00000000,0x0b04000e,0x0f040018,0x0f08000c,0x0b030018,0x0f07000d,0x0b03000c,0x0f070019,
0x00000000,0x01010001,0x0b04000a,0x0c03002f,0x0b020037,0x0b04000b,0x0b03000d,0x0f08000d,
0x00000000,0x0b03001d,0x0b04000b,0x0f07000f,0x0b010076,0x0f07000e,0x0b03000e,0x0b07000c,
0x00000000,0x01010001,0x0b040009,0x02020002,0x0f040015,0x0f08000c,0x0f07000d,0x0f05006a,
0x00000000,0x0b02003c,0x0b04000a,0x0f08000d,0x0b030019,0x0f07000f,0x0f07000e,0x0b07000b,
0x00000000,0x01010001,0x0f08000c,0x0f050051,0x0b020038,0x0f08000d,0x0f07000f,0x0f070010,
0x00000000,0x0c02005c,0x0f08000d,0x0f070010,0x0b010077,0x0f070010,0x0f070010,0x0f0b000e,
0x00000000,0x01010001,0x04010002,0x02020002,0x0b04000b,0x05020003,0x05010004,0x03030003,
0x0f0200cf,0x0b04000f,0x04010003,0x06030004,0x0b03001a,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f0200e8,0x05020003,0x0b040011,0x0b020039,0x05020004,0x05010006,0x0c05000a,
0x0f0101ce,0x0b03001e,0x05020004,0x0c04000b,0x0b010078,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0b050007,0x02020002,0x0b04000c,0x06030004,0x06020005,0x0b040012,
0x100102ce,0x0b02003d,0x0b050008,0x06030005,0x0b03001b,0x06020007,0x06020006,0x0b080009,
0x00000000,0x10020168,0x06030004,0x0b030020,0x0b02003a,0x06030005,0x06020007,0x0b07000a,
0x00000000,0x0c02005d,0x06030005,0x0c04000c,0x0b010079,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0f030036,0x02020002,0x0b04000d,0x0b050008,0x0b040009,0x03030003,
0x00000000,0x0b040010,0x0f030037,0x0c05000a,0x0b03001c,0x0c04000b,0x0b04000a,0x0f060038,
0x00000000,0x01010001,0x0b050008,0x0c030030,0x0b02003b,0x0b050009,0x0b04000b,0x0c05000b,
0x00000000,0x0b03001f,0x0b050009,0x0c04000d,0x0b01007a,0x0c04000c,0x0b04000c,0x0b08000a,
0x00000000,0x01010001,0x0b04000a,0x02020002,0x0b04000e,0x0c05000a,0x0c04000b,0x0c04001a,
0x00000000,0x0b02003e,0x0b04000b,0x0c05000b,0x0b03001d,0x0c04000d,0x0c04000c,0x0b07000c,
0x00000000,0x01010001,0x0c05000a,0x0f050052,0x0b02003c,0x0c05000b,0x0c04000d,0x0b07000b,
0x00000000,0x0c02005e,0x0c05000b,0x0c04000e,0x0b01007b,0x0c04000e,0x0c04000e,0x0c08000c,
0x00000000,0x01010001,0x04010002,0x02020002,0x0b04000f,0x1003005c,0x1002005d,0x03030003,
0x0f0200d0,0x0b040011,0x04010003,0x0b04000b,0x0b03001e,0x0b03000c,0x1002005e,0x04040004,
0x00000000,0x01010001,0x1003005c,0x0b040012,0x0b02003d,0x1003005d,0x1002005f,0x0b080009,
0x0f0101cf,0x0b030020,0x1003005d,0x0b07000a,0x0b01007c,0x0b03000d,0x10020060,0x1006005e,
0x00000000,0x01010001,0x0b050008,0x02020002,0x0b040010,0x0b04000b,0x0b03000c,0x0f06002c,
0x100102cf,0x0b02003f,0x0b050009,0x0b04000c,0x0b03001f,0x0b03000e,0x0b03000d,0x0b08000a,
0x00000000,0x01010001,0x0b04000b,0x0b030021,0x0b02003e,0x0b04000c,0x0b03000e,0x0b07000c,
0x00000000,0x0c02005f,0x0b04000c,0x0b07000b,0x0b01007d,0x0b03000f,0x0b03000f,0x0b07000d,
0x00000000,0x01010001,0x0f020075,0x02020002,0x0b040011,0x0b04000b,0x0b03000c,0x03030003,
0x00000000,0x0b040012,0x0f020076,0x0b080009,0x0b030020,0x0b07000a,0x0b03000d,0x0f050077,
0x00000000,0x01010001,0x0b04000b,0x0c030031,0x0b02003f,0x0b04000c,0x0b03000e,0x0b08000a,
0x00000000,0x0b030021,0x0b04000c,0x0b07000c,0x0b01007e,0x0b07000b,0x0b03000f,0x0b07000d,
0x00000000,0x01010001,0x0b04000b,0x02020002,0x0b040012,0x0b080009,0x0b07000a,0x0f05006b,
0x00000000,0x0b020040,0x0b04000c,0x0b08000a,0x0b030021,0x0b07000c,0x0b07000b,0x0b07000d,
0x00000000,0x01010001,0x0b080009,0x0f050053,0x0b020040,0x0b08000a,0x0b07000c,0x0b07000d,
0x00000000,0x0c020060,0x0b08000a,0x0b07000d,0x0b01007f,0x0b07000d,0x0b07000d,0x0b0b000b,
0x00000000,0x01010001,0x04010002,0x02020002,0x08010008,0x05020003,0x05010004,0x03030003,
0x0f0200d1,0x09020009,0x04010003,0x06030004,0x09010010,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f0200e9,0x05020003,0x0f05000e,0x0f030024,0x05020004,0x05010006,0x07040005,
0x0f0101d0,0x0f040015,0x05020004,0x07030006,0x0c020041,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x08020004,0x100300b6,0x08010009,0x06030004,0x06020005,0x0c05000c,
0x100102d0,0x0c030022,0x08020005,0x06030005,0x09010011,0x06020007,0x06020006,0x08050006,
0x00000000,0x10020169,0x06030004,0x0c040013,0x0f030025,0x06030005,0x06020007,0x08040007,
0x00000000,0x0c020061,0x06030005,0x07030007,0x0c020042,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0c02000e,0x02020002,0x0801000a,0x09030005,0x09020006,0x03030003,
0x00000000,0x0902000a,0x0c02000f,0x07040005,0x09010012,0x07030006,0x09020007,0x0c050010,
0x00000000,0x01010001,0x09030005,0x0c030032,0x0f030026,0x09030006,0x09020008,0x07040006,
0x00000000,0x0f040016,0x09030006,0x07030008,0x0c020043,0x07030007,0x09020009,0x09060007,
0x00000000,0x01010001,0x09020006,0x02020002,0x0801000b,0x07040005,0x07030006,0x0c04001b,
0x00000000,0x0c030023,0x09020007,0x07040006,0x09010013,0x07030008,0x07030007,0x09050008,
0x00000000,0x01010001,0x07040005,0x0f050054,0x0f030027,0x07040006,0x07030008,0x08040008,
0x00000000,0x0c020062,0x07040006,0x07030009,0x0c020044,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0801000c,0x0f05000f,0x0f040010,0x03030003,
0x0f0200d2,0x0902000b,0x04010003,0x0f06000a,0x09010014,0x0f05000b,0x0f040011,0x04040004,
0x00000000,0x01010001,0x0f05000f,0x0f05000f,0x0f030028,0x0f050010,0x0f040012,0x08050006,
0x0f0101d1,0x0f040017,0x0f050010,0x08040007,0x0c020045,0x0f05000c,0x0f040013,0x0f080011,
0x00000000,0x01010001,0x08020005,0x02020002,0x0801000d,0x0f06000a,0x0f05000b,0x0f06002d,
0x100102d1,0x0c030024,0x08020006,0x0f06000b,0x09010015,0x0f05000d,0x0f05000c,0x08050007,
0x00000000,0x01010001,0x0f06000a,0x0c040014,0x0f030029,0x0f06000b,0x0f05000d,0x08040009,
0x00000000,0x0c020063,0x0f06000b,0x08040008,0x0c020046,0x0f05000e,0x0f05000e,0x0f09000c,
0x00000000,0x01010001,0x0f040018,0x02020002,0x0801000e,0x0f05000b,0x0f04000c,0x03030003,
0x00000000,0x0902000c,0x0f040019,0x08050006,0x09010016,0x08040007,0x0f04000d,0x0f07001a,
0x00000000,0x01010001,0x0f05000b,0x0c030033,0x0f03002a,0x0f05000c,0x0f04000e,0x08050007,
0x00000000,0x0f040018,0x0f05000c,0x08040009,0x0c020047,0x08040008,0x0f04000f,0x0f08000d,
0x00000000,0x01010001,0x09020007,0x02020002,0x0801000f,0x08050006,0x08040007,0x0f05006c,
0x00000000,0x0c030025,0x09020008,0x08050007,0x09010017,0x08040009,0x08040008,0x09050009,
0x00000000,0x01010001,0x08050006,0x0f050055,0x0f03002b,0x08050007,0x08040009,0x0804000a,
0x00000000,0x0c020064,0x08050007,0x0804000a,0x0c020048,0x0804000a,0x0804000a,0x08080008,
0x00000000,0x01010001,0x04010002,0x02020002,0x09020009,0x05020003,0x05010004,0x03030003,
0x0f0200d3,0x0902000d,0x04010003,0x0f050012,0x09010018,0x0f040013,0x05010005,0x04040004,
0x00000000,0x0f0200ea,0x05020003,0x0f050010,0x0f03002c,0x05020004,0x05010006,0x0c060008,
0x0f0101d2,0x0f040019,0x05020004,0x0c050009,0x0c020049,0x0f040014,0x05010007,0x05050005,
0x00000000,0x01010001,0x09030005,0x02020002,0x0902000a,0x0f050012,0x0f040013,0x0c05000d,
0x100102d2,0x0c030026,0x09030006,0x0f050013,0x09010019,0x0f040015,0x0f040014,0x09060007,
0x00000000,0x1002016a,0x0f050012,0x0c040015,0x0f03002d,0x0f050013,0x0f040015,0x09050008,
0x00000000,0x0c020065,0x0f050013,0x0c05000a,0x0c02004a,0x0f040016,0x0f040016,0x0f080014,
0x00000000,0x01010001,0x0f030037,0x02020002,0x0902000b,0x09030006,0x09020007,0x03030003,
0x00000000,0x0902000e,0x0f030038,0x0c060008,0x0901001a,0x0c050009,0x09020008,0x0f060039,
0x00000000,0x01010001,0x09030006,0x0c030034,0x0f03002e,0x09030007,0x09020009,0x0c060009,
0x00000000,0x0f04001a,0x09030007,0x0c05000b,0x0c02004b,0x0c05000a,0x0902000a,0x09060008,
0x00000000,0x01010001,0x09020008,0x02020002,0x0902000c,0x0c060008,0x0c050009,0x0c04001c,
0x00000000,0x0c030027,0x09020009,0x0c060009,0x0901001b,0x0c05000b,0x0c05000a,0x0905000a,
0x00000000,0x01010001,0x0c060008,0x0f050056,0x0f03002f,0x0c060009,0x0c05000b,0x09050009,
0x00000000,0x0c020066,0x0c060009,0x0c05000c,0x0c02004c,0x0c05000c,0x0c05000c,0x0c09000a,
0x00000000,0x01010001,0x04010002,0x02020002,0x0902000d,0x0f04001e,0x0f03001f,0x03030003,
0x0f0200d4,0x0902000f,0x04010003,0x0c050009,0x0901001c,0x0c04000a,0x0f030020,0x04040004,
0x00000000,0x01010001,0x0f04001e,0x0f050011,0x0f030030,0x0f04001f,0x0f030021,0x09060007,
0x0f0101d3,0x0f04001b,0x0f04001f,0x09050008,0x0c02004d,0x0c04000b,0x0f030022,0x0f070020,
0x00000000,0x01010001,0x09030006,0x02020002,0x0902000e,0x0c050009,0x0c04000a,0x0f06002e,
0x100102d3,0x0c030028,0x09030007,0x0c05000a,0x0901001d,0x0c04000c,0x0c04000b,0x09060008,
0x00000000,0x01010001,0x0c050009,0x0c040016,0x0f030031,0x0c05000a,0x0c04000c,0x0905000a,
0x00000000,0x0c020067,0x0c05000a,0x09050009,0x0c02004e,0x0c04000d,0x0c04000d,0x0c08000b,
0x00000000,0x01010001,0x0f020076,0x02020002,0x0902000f,0x0f05000c,0x0f04000d,0x03030003,
0x00000000,0x09020010,0x0f020077,0x09060007,0x0901001e,0x09050008,0x0f04000e,0x0f050078,
0x00000000,0x01010001,0x0f05000c,0x0c030035,0x0f030032,0x0f05000d,0x0f04000f,0x09060008,
0x00000000,0x0f04001c,0x0f05000d,0x0905000a,0x0c02004f,0x09050009,0x0f040010,0x0f08000e,
0x00000000,0x01010001,0x09020009,0x02020002,0x09020010,0x09060007,0x09050008,0x0f05006d,
0x00000000,0x0c030029,0x0902000a,0x09060008,0x0901001f,0x0905000a,0x09050009,0x0905000b,
0x00000000,0x01010001,0x09060007,0x0f050057,0x0f030033,0x09060008,0x0905000a,0x0905000b,
0x00000000,0x0c020068,0x09060008,0x0905000b,0x0c020050,0x0905000b,0x0905000b,0x09090009,
0x00000000,0x01010001,0x04010002,0x02020002,0x0c02000b,0x05020003,0x05010004,0x03030003,
0x0f0200d5,0x0f05000e,0x04010003,0x06030004,0x0f040015,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f0200eb,0x05020003,0x0f050012,0x0f030034,0x05020004,0x05010006,0x07040005,
0x0f0101d4,0x0f04001d,0x05020004,0x07030006,0x0c020051,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0c030007,0x100300b7,0x0c02000c,0x06030004,0x06020005,0x0c05000e,
0x100102d4,0x0c03002a,0x0c030008,0x06030005,0x0f040016,0x06020007,0x06020006,0x0c060009,
0x00000000,0x1002016b,0x06030004,0x0c040017,0x0f030035,0x06030005,0x06020007,0x0c05000a,
0x00000000,0x0c020069,0x06030005,0x07030007,0x0c020052,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0c02000f,0x02020002,0x0c02000d,0x0f06000a,0x0f05000b,0x03030003,
0x00000000,0x0f05000f,0x0c020010,0x07040005,0x0f040017,0x07030006,0x0f05000c,0x0c050011,
0x00000000,0x01010001,0x0f06000a,0x0c030036,0x0f030036,0x0f06000b,0x0f05000d,0x07040006,
0x00000000,0x0f04001e,0x0f06000b,0x07030008,0x0c020053,0x07030007,0x0f05000e,0x0f09000c,
0x00000000,0x01010001,0x0f05000b,0x02020002,0x0c02000e,0x07040005,0x07030006,0x0c04001d,
0x00000000,0x0c03002b,0x0f05000c,0x07040006,0x0f040018,0x07030008,0x07030007,0x0f08000d,
0x00000000,0x01010001,0x07040005,0x0f050058,0x0f030037,0x07040006,0x07030008,0x0c05000b,
0x00000000,0x0c02006a,0x07040006,0x07030009,0x0c020054,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0c02000f,0x0f03003d,0x0f02003e,0x03030003,
0x0f0200d6,0x0f050010,0x04010003,0x0f06000b,0x0f040019,0x0f05000c,0x0f02003f,0x04040004,
0x00000000,0x01010001,0x0f03003d,0x0f050013,0x0f030038,0x0f03003e,0x0f020040,0x0c060009,
0x0f0101d5,0x0f04001f,0x0f03003e,0x0c05000a,0x0c020055,0x0f05000d,0x0f020041,0x0f06003f,
0x00000000,0x01010001,0x0c030008,0x02020002,0x0c020010,0x0f06000b,0x0f05000c,0x0f06002f,
0x100102d5,0x0c03002c,0x0c030009,0x0f06000c,0x0f04001a,0x0f05000e,0x0f05000d,0x0c06000a,
0x00000000,0x01010001,0x0f06000b,0x0c040018,0x0f030039,0x0f06000c,0x0f05000e,0x0c05000c,
0x00000000,0x0c02006b,0x0f06000c,0x0c05000b,0x0c020056,0x0f05000f,0x0f05000f,0x0f09000d,
0x00000000,0x01010001,0x0f040019,0x02020002,0x0c020011,0x0f05000d,0x0f04000e,0x03030003,
0x00000000,0x0f050011,0x0f04001a,0x0c060009,0x0f04001b,0x0c05000a,0x0f04000f,0x0f07001b,
0x00000000,0x01010001,0x0f05000d,0x0c030037,0x0f03003a,0x0f05000e,0x0f040010,0x0c06000a,
0x00000000,0x0f040020,0x0f05000e,0x0c05000c,0x0c020057,0x0c05000b,0x0f040011,0x0f08000f,
0x00000000,0x01010001,0x0f05000c,0x02020002,0x0c020012,0x0c060009,0x0c05000a,0x0f05006e,
0x00000000,0x0c03002d,0x0f05000d,0x0c06000a,0x0f04001c,0x0c05000c,0x0c05000b,0x0f08000e,
0x00000000,0x01010001,0x0c060009,0x0f050059,0x0f03003b,0x0c06000a,0x0c05000c,0x0c05000d,
0x00000000,0x0c02006c,0x0c06000a,0x0c05000d,0x0c020058,0x0c05000d,0x0c05000d,0x0c09000b,
0x00000000,0x01010001,0x04010002,0x02020002,0x0f05000e,0x05020003,0x05010004,0x03030003,
0x0f0200d7,0x0f050012,0x04010003,0x0f040021,0x0f04001d,0x0f030022,0x05010005,0x04040004,
0x00000000,0x0f0200ec,0x05020003,0x0f050014,0x0f03003c,0x05020004,0x05010006,0x0c05000b,
0x0f0101d6,0x0f040021,0x05020004,0x0c04000c,0x0c020059,0x0f030023,0x05010007,0x05050005,
0x00000000,0x01010001,0x0f06000a,0x02020002,0x0f05000f,0x0f040021,0x0f030022,0x0c05000f,
0x100102d6,0x0c03002e,0x0f06000b,0x0f040022,0x0f04001e,0x0f030024,0x0f030023,0x0f09000c,
0x00000000,0x1002016c,0x0f040021,0x0c040019,0x0f03003d,0x0f040022,0x0f030024,0x0f08000d,
0x00000000,0x0c02006d,0x0f040022,0x0c04000d,0x0c02005a,0x0f030025,0x0f030025,0x0f070023,
0x00000000,0x01010001,0x0f030038,0x02020002,0x0f050010,0x0f06000b,0x0f05000c,0x03030003,
0x00000000,0x0f050013,0x0f030039,0x0c05000b,0x0f04001f,0x0c04000c,0x0f05000d,0x0f06003a,
0x00000000,0x01010001,0x0f06000b,0x0c030038,0x0f03003e,0x0f06000c,0x0f05000e,0x0c05000c,
0x00000000,0x0f040022,0x0f06000c,0x0c04000e,0x0c02005b,0x0c04000d,0x0f05000f,0x0f09000d,
0x00000000,0x01010001,0x0f05000d,0x02020002,0x0f050011,0x0c05000b,0x0c04000c,0x0c04001e,
0x00000000,0x0c03002f,0x0f05000e,0x0c05000c,0x0f040020,0x0c04000e,0x0c04000d,0x0f08000f,
0x00000000,0x01010001,0x0c05000b,0x0f05005a,0x0f03003f,0x0c05000c,0x0c04000e,0x0f08000e,
0x00000000,0x0c02006e,0x0c05000c,0x0c04000f,0x0c02005c,0x0c04000f,0x0c04000f,0x0c08000d,
0x00000000,0x01010001,0x04010002,0x02020002,0x0f050012,0x1003005d,0x1002005e,0x03030003,
0x0f0200d8,0x0f050014,0x04010003,0x0c05000a,0x0f040021,0x0c04000b,0x1002005f,0x04040004,
0x00000000,0x01010001,0x1003005d,0x0f050015,0x0f030040,0x1003005e,0x10020060,0x0f09000c,
0x0f0101d7,0x0f040023,0x1003005e,0x0f08000d,0x0c02005d,0x0c04000c,0x10020061,0x1006005f,
0x00000000,0x01010001,0x0f06000b,0x02020002,0x0f050013,0x0c05000a,0x0c04000b,0x0f060030,
0x100102d7,0x0c030030,0x0f06000c,0x0c05000b,0x0f040022,0x0c04000d,0x0c04000c,0x0f09000d,
0x00000000,0x01010001,0x0c05000a,0x0c04001a,0x0f030041,0x0c05000b,0x0c04000d,0x0f08000f,
0x00000000,0x0c02006f,0x0c05000b,0x0f08000e,0x0c02005e,0x0c04000e,0x0c04000e,0x0c08000c,
0x00000000,0x01010001,0x0f020077,0x02020002,0x0f050014,0x0f05000e,0x0f04000f,0x03030003,
0x00000000,0x0f050015,0x0f020078,0x0f09000c,0x0f040023,0x0f08000d,0x0f040010,0x0f050079,
0x00000000,0x01010001,0x0f05000e,0x0c030039,0x0f030042,0x0f05000f,0x0f040011,0x0f09000d,
0x00000000,0x0f040024,0x0f05000f,0x0f08000f,0x0c02005f,0x0f08000e,0x0f040012,0x0f080010,
0x00000000,0x01010001,0x0f05000e,0x02020002,0x0f050015,0x0f09000c,0x0f08000d,0x0f05006f,
0x00000000,0x0c030031,0x0f05000f,0x0f09000d,0x0f040024,0x0f08000f,0x0f08000e,0x0f080010,
0x00000000,0x01010001,0x0f09000c,0x0f05005b,0x0f030043,0x0f09000d,0x0f08000f,0x0f080010,
0x00000000,0x0c020070,0x0f09000d,0x0f080010,0x0c020060,0x0f080010,0x0f080010,0x0f0c000e,
0x00000000,0x01010001,0x04010002,0x02020002,0x08010008,0x05020003,0x05010004,0x03030003,
0x0f0200d9,0x0f040010,0x04010003,0x06030004,0x0f030017,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f0200ed,0x05020003,0x0c05000c,0x0c030022,0x05020004,0x05010006,0x07040005,
0x0f0101d8,0x0c040013,0x05020004,0x07030006,0x0c020061,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x08020004,0x100300b8,0x08010009,0x06030004,0x06020005,0x0c050010,
0x100102d8,0x0c030032,0x08020005,0x06030005,0x0f030018,0x06020007,0x06020006,0x08050006,
0x00000000,0x1002016d,0x06030004,0x0c04001b,0x0c030023,0x06030005,0x06020007,0x08040007,
0x00000000,0x0c020071,0x06030005,0x07030007,0x0c020062,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0c020010,0x02020002,0x0801000a,0x0f05000c,0x0f04000d,0x03030003,
0x00000000,0x0f040011,0x0c020011,0x07040005,0x0f030019,0x07030006,0x0f04000e,0x0c050012,
0x00000000,0x01010001,0x0f05000c,0x0c03003a,0x0c030024,0x0f05000d,0x0f04000f,0x07040006,
0x00000000,0x0c040014,0x0f05000d,0x07030008,0x0c020063,0x07030007,0x0f040010,0x0f08000e,
0x00000000,0x01010001,0x0f04000d,0x02020002,0x0801000b,0x07040005,0x07030006,0x0c04001f,
0x00000000,0x0c030033,0x0f04000e,0x07040006,0x0f03001a,0x07030008,0x07030007,0x0f07000f,
0x00000000,0x01010001,0x07040005,0x0f05005c,0x0c030025,0x07040006,0x07030008,0x08040008,
0x00000000,0x0c020072,0x07040006,0x07030009,0x0c020064,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0801000c,0x0f050010,0x0f040011,0x03030003,
0x0f0200da,0x0f040012,0x04010003,0x0c060008,0x0f03001b,0x0c050009,0x0f040012,0x04040004,
0x00000000,0x01010001,0x0f050010,0x0c05000d,0x0c030026,0x0f050011,0x0f040013,0x08050006,
0x0f0101d9,0x0c040015,0x0f050011,0x08040007,0x0c020065,0x0c05000a,0x0f040014,0x0f080012,
0x00000000,0x01010001,0x08020005,0x02020002,0x0801000d,0x0c060008,0x0c050009,0x0f060031,
0x100102d9,0x0c030034,0x08020006,0x0c060009,0x0f03001c,0x0c05000b,0x0c05000a,0x08050007,
0x00000000,0x01010001,0x0c060008,0x0c04001c,0x0c030027,0x0c060009,0x0c05000b,0x08040009,
0x00000000,0x0c020073,0x0c060009,0x08040008,0x0c020066,0x0c05000c,0x0c05000c,0x0c09000a,
0x00000000,0x01010001,0x0f04001a,0x02020002,0x0801000e,0x0c050009,0x0c04000a,0x03030003,
0x00000000,0x0f040013,0x0f04001b,0x08050006,0x0f03001d,0x08040007,0x0c04000b,0x0f07001c,
0x00000000,0x01010001,0x0c050009,0x0c03003b,0x0c030028,0x0c05000a,0x0c04000c,0x08050007,
0x00000000,0x0c040016,0x0c05000a,0x08040009,0x0c020067,0x08040008,0x0c04000d,0x0c08000b,
0x00000000,0x01010001,0x0f04000e,0x02020002,0x0801000f,0x08050006,0x08040007,0x0f050070,
0x00000000,0x0c030035,0x0f04000f,0x08050007,0x0f03001e,0x08040009,0x08040008,0x0f070010,
0x00000000,0x01010001,0x08050006,0x0f05005d,0x0c030029,0x08050007,0x08040009,0x0804000a,
0x00000000,0x0c020074,0x08050007,0x0804000a,0x0c020068,0x0804000a,0x0804000a,0x08080008,
0x00000000,0x01010001,0x04010002,0x02020002,0x0f040010,0x05020003,0x05010004,0x03030003,
0x0f0200db,0x0f040014,0x04010003,0x10040031,0x0f03001f,0x10030032,0x05010005,0x04040004,
0x00000000,0x0f0200ee,0x05020003,0x0c05000e,0x0c03002a,0x05020004,0x05010006,0x0c060009,
0x0f0101da,0x0c040017,0x05020004,0x0c05000a,0x0c020069,0x10030033,0x05010007,0x05050005,
0x00000000,0x01010001,0x0f05000c,0x02020002,0x0f040011,0x10040031,0x10030032,0x0c050011,
0x100102da,0x0c030036,0x0f05000d,0x10040032,0x0f030020,0x10030034,0x10030033,0x0f08000e,
0x00000000,0x1002016e,0x10040031,0x0c04001d,0x0c03002b,0x10040032,0x10030034,0x0f07000f,
0x00000000,0x0c020075,0x10040032,0x0c05000b,0x0c02006a,0x10030035,0x10030035,0x10070033,
0x00000000,0x01010001,0x0f030039,0x02020002,0x0f040012,0x0f05000d,0x0f04000e,0x03030003,
0x00000000,0x0f040015,0x0f03003a,0x0c060009,0x0f030021,0x0c05000a,0x0f04000f,0x0f06003b,
0x00000000,0x01010001,0x0f05000d,0x0c03003c,0x0c03002c,0x0f05000e,0x0f040010,0x0c06000a,
0x00000000,0x0c040018,0x0f05000e,0x0c05000c,0x0c02006b,0x0c05000b,0x0f040011,0x0f08000f,
0x00000000,0x01010001,0x0f04000f,0x02020002,0x0f040013,0x0c060009,0x0c05000a,0x0c040020,
0x00000000,0x0c030037,0x0f040010,0x0c06000a,0x0f030022,0x0c05000c,0x0c05000b,0x0f070011,
0x00000000,0x01010001,0x0c060009,0x0f05005e,0x0c03002d,0x0c06000a,0x0c05000c,0x0f070010,
0x00000000,0x0c020076,0x0c06000a,0x0c05000d,0x0c02006c,0x0c05000d,0x0c05000d,0x0c09000b,
0x00000000,0x01010001,0x04010002,0x02020002,0x0f040014,0x0f04001f,0x0f030020,0x03030003,
0x0f0200dc,0x0f040016,0x04010003,0x0c05000b,0x0f030023,0x0c04000c,0x0f030021,0x04040004,
0x00000000,0x01010001,0x0f04001f,0x0c05000f,0x0c03002e,0x0f040020,0x0f030022,0x0f08000e,
0x0f0101db,0x0c040019,0x0f040020,0x0f07000f,0x0c02006d,0x0c04000d,0x0f030023,0x0f070021,
0x00000000,0x01010001,0x0f05000d,0x02020002,0x0f040015,0x0c05000b,0x0c04000c,0x0f060032,
0x100102db,0x0c030038,0x0f05000e,0x0c05000c,0x0f030024,0x0c04000e,0x0c04000d,0x0f08000f,
0x00000000,0x01010001,0x0c05000b,0x0c04001e,0x0c03002f,0x0c05000c,0x0c04000e,0x0f070011,
0x00000000,0x0c020077,0x0c05000c,0x0f070010,0x0c02006e,0x0c04000f,0x0c04000f,0x0c08000d,
0x00000000,0x01010001,0x0f020078,0x02020002,0x0f040016,0x0c05000a,0x0c04000b,0x03030003,
0x00000000,0x0f040017,0x0f020079,0x0f08000e,0x0f030025,0x0f07000f,0x0c04000c,0x0f05007a,
0x00000000,0x01010001,0x0c05000a,0x0c03003d,0x0c030030,0x0c05000b,0x0c04000d,0x0f08000f,
0x00000000,0x0c04001a,0x0c05000b,0x0f070011,0x0c02006f,0x0f070010,0x0c04000e,0x0c08000c,
0x00000000,0x01010001,0x0f040010,0x02020002,0x0f040017,0x0f08000e,0x0f07000f,0x0f050071,
0x00000000,0x0c030039,0x0f040011,0x0f08000f,0x0f030026,0x0f070011,0x0f070010,0x0f070012,
0x00000000,0x01010001,0x0f08000e,0x0f05005f,0x0c030031,0x0f08000f,0x0f070011,0x0f070012,
0x00000000,0x0c020078,0x0f08000f,0x0f070012,0x0c020070,0x0f070012,0x0f070012,0x0f0b0010,
0x00000000,0x01010001,0x04010002,0x02020002,0x0f04000f,0x05020003,0x05010004,0x03030003,
0x0f0200dd,0x0c05000c,0x04010003,0x06030004,0x0c040013,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f0200ef,0x05020003,0x0c050010,0x0c030032,0x05020004,0x05010006,0x07040005,
0x0f0101dc,0x0c04001b,0x05020004,0x07030006,0x0c020071,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0f05000b,0x100300b9,0x0f040010,0x06030004,0x06020005,0x0c050012,
0x100102dc,0x0c03003a,0x0f05000c,0x06030005,0x0c040014,0x06020007,0x06020006,0x0f08000d,
0x00000000,0x1002016f,0x06030004,0x0c04001f,0x0c030033,0x06030005,0x06020007,0x0f07000e,
0x00000000,0x0c020079,0x06030005,0x07030007,0x0c020072,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0c020011,0x02020002,0x0f040011,0x0c060008,0x0c050009,0x03030003,
0x00000000,0x0c05000d,0x0c020012,0x07040005,0x0c040015,0x07030006,0x0c05000a,0x0c050013,
0x00000000,0x01010001,0x0c060008,0x0c03003e,0x0c030034,0x0c060009,0x0c05000b,0x07040006,
0x00000000,0x0c04001c,0x0c060009,0x07030008,0x0c020073,0x07030007,0x0c05000c,0x0c09000a,
0x00000000,0x01010001,0x0c050009,0x02020002,0x0f040012,0x07040005,0x07030006,0x0c040021,
0x00000000,0x0c03003b,0x0c05000a,0x07040006,0x0c040016,0x07030008,0x07030007,0x0c08000b,
0x00000000,0x01010001,0x07040005,0x0f050060,0x0c030035,0x07040006,0x07030008,0x0f07000f,
0x00000000,0x0c02007a,0x07040006,0x07030009,0x0c020074,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0f040013,0x0f03003e,0x0f02003f,0x03030003,
0x0f0200de,0x0c05000e,0x04010003,0x0c060009,0x0c040017,0x0c05000a,0x0f020040,0x04040004,
0x00000000,0x01010001,0x0f03003e,0x0c050011,0x0c030036,0x0f03003f,0x0f020041,0x0f08000d,
0x0f0101dd,0x0c04001d,0x0f03003f,0x0f07000e,0x0c020075,0x0c05000b,0x0f020042,0x0f060040,
0x00000000,0x01010001,0x0f05000c,0x02020002,0x0f040014,0x0c060009,0x0c05000a,0x0f060033,
0x100102dd,0x0c03003c,0x0f05000d,0x0c06000a,0x0c040018,0x0c05000c,0x0c05000b,0x0f08000e,
0x00000000,0x01010001,0x0c060009,0x0c040020,0x0c030037,0x0c06000a,0x0c05000c,0x0f070010,
0x00000000,0x0c02007b,0x0c06000a,0x0f07000f,0x0c020076,0x0c05000d,0x0c05000d,0x0c09000b,
0x00000000,0x01010001,0x0f04001b,0x02020002,0x0f040015,0x0c05000b,0x0c04000c,0x03030003,
0x00000000,0x0c05000f,0x0f04001c,0x0f08000d,0x0c040019,0x0f07000e,0x0c04000d,0x0f07001d,
0x00000000,0x01010001,0x0c05000b,0x0c03003f,0x0c030038,0x0c05000c,0x0c04000e,0x0f08000e,
0x00000000,0x0c04001e,0x0c05000c,0x0f070010,0x0c020077,0x0f07000f,0x0c04000f,0x0c08000d,
0x00000000,0x01010001,0x0c05000a,0x02020002,0x0f040016,0x0f08000d,0x0f07000e,0x0f050072,
0x00000000,0x0c03003d,0x0c05000b,0x0f08000e,0x0c04001a,0x0f070010,0x0f07000f,0x0c08000c,
0x00000000,0x01010001,0x0f08000d,0x0f050061,0x0c030039,0x0f08000e,0x0f070010,0x0f070011,
0x00000000,0x0c02007c,0x0f08000e,0x0f070011,0x0c020078,0x0f070011,0x0f070011,0x0f0b000f,
0x00000000,0x01010001,0x04010002,0x02020002,0x0c05000c,0x05020003,0x05010004,0x03030003,
0x0f0200df,0x0c050010,0x04010003,0x06030004,0x0c04001b,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f0200f0,0x05020003,0x0c050012,0x0c03003a,0x05020004,0x05010006,0x0c05000c,
0x0f0101de,0x0c04001f,0x05020004,0x0c04000d,0x0c020079,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0c060008,0x02020002,0x0c05000d,0x06030004,0x06020005,0x0c050013,
0x100102de,0x0c03003e,0x0c060009,0x06030005,0x0c04001c,0x06020007,0x06020006,0x0c09000a,
0x00000000,0x10020170,0x06030004,0x0c040021,0x0c03003b,0x06030005,0x06020007,0x0c08000b,
0x00000000,0x0c02007d,0x06030005,0x0c04000e,0x0c02007a,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0f03003a,0x02020002,0x0c05000e,0x0c060009,0x0c05000a,0x03030003,
0x00000000,0x0c050011,0x0f03003b,0x0c05000c,0x0c04001d,0x0c04000d,0x0c05000b,0x0f06003c,
0x00000000,0x01010001,0x0c060009,0x0c030040,0x0c03003c,0x0c06000a,0x0c05000c,0x0c05000d,
0x00000000,0x0c040020,0x0c06000a,0x0c04000f,0x0c02007b,0x0c04000e,0x0c05000d,0x0c09000b,
0x00000000,0x01010001,0x0c05000b,0x02020002,0x0c05000f,0x0c05000c,0x0c04000d,0x0c040022,
0x00000000,0x0c03003f,0x0c05000c,0x0c05000d,0x0c04001e,0x0c04000f,0x0c04000e,0x0c08000d,
0x00000000,0x01010001,0x0c05000c,0x0f050062,0x0c03003d,0x0c05000d,0x0c04000f,0x0c08000c,
0x00000000,0x0c02007e,0x0c05000d,0x0c040010,0x0c02007c,0x0c040010,0x0c040010,0x0c08000e,
0x00000000,0x01010001,0x04010002,0x02020002,0x0c050010,0x1003005e,0x1002005f,0x03030003,
0x0f0200e0,0x0c050012,0x04010003,0x0c05000c,0x0c04001f,0x0c04000d,0x10020060,0x04040004,
0x00000000,0x01010001,0x1003005e,0x0c050013,0x0c03003e,0x1003005f,0x10020061,0x0c09000a,
0x0f0101df,0x0c040021,0x1003005f,0x0c08000b,0x0c02007d,0x0c04000e,0x10020062,0x10060060,
0x00000000,0x01010001,0x0c060009,0x02020002,0x0c050011,0x0c05000c,0x0c04000d,0x0f060034,
0x100102df,0x0c030040,0x0c06000a,0x0c05000d,0x0c040020,0x0c04000f,0x0c04000e,0x0c09000b,
0x00000000,0x01010001,0x0c05000c,0x0c040022,0x0c03003f,0x0c05000d,0x0c04000f,0x0c08000d,
0x00000000,0x0c02007f,0x0c05000d,0x0c08000c,0x0c02007e,0x0c040010,0x0c040010,0x0c08000e,
0x00000000,0x01010001,0x0f020079,0x02020002,0x0c050012,0x0c05000c,0x0c04000d,0x03030003,
0x00000000,0x0c050013,0x0f02007a,0x0c09000a,0x0c040021,0x0c08000b,0x0c04000e,0x0f05007b,
0x00000000,0x01010001,0x0c05000c,0x0c030041,0x0c030040,0x0c05000d,0x0c04000f,0x0c09000b,
0x00000000,0x0c040022,0x0c05000d,0x0c08000d,0x0c02007f,0x0c08000c,0x0c040010,0x0c08000e,
0x00000000,0x01010001,0x0c05000c,0x02020002,0x0c050013,0x0c09000a,0x0c08000b,0x0f050073,
0x00000000,0x0c030041,0x0c05000d,0x0c09000b,0x0c040022,0x0c08000d,0x0c08000c,0x0c08000e,
0x00000000,0x01010001,0x0c09000a,0x0f050063,0x0c030041,0x0c09000b,0x0c08000d,0x0c08000e,
0x00000000,0x0c020080,0x0c09000b,0x0c08000e,0x0c020080,0x0c08000e,0x0c08000e,0x0c0c000c,
0x00000000,0x01010001,0x04010002,0x02020002,0x08010008,0x05020003,0x05010004,0x03030003,
0x0f0200e1,0x09020009,0x04010003,0x06030004,0x09010010,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f0200f1,0x05020003,0x0a03000a,0x0a010020,0x05020004,0x05010006,0x07040005,
0x0f0101e0,0x0a020011,0x05020004,0x07030006,0x10020045,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x08020004,0x100300ba,0x08010009,0x06030004,0x06020005,0x10050010,
0x100102e0,0x10030026,0x08020005,0x06030005,0x09010011,0x06020007,0x06020006,0x08050006,
0x00000000,0x10020171,0x06030004,0x10040017,0x0a010021,0x06030005,0x06020007,0x08040007,
0x00000000,0x0f050044,0x06030005,0x07030007,0x10020046,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0f05000d,0x02020002,0x0801000a,0x09030005,0x09020006,0x03030003,
0x00000000,0x0902000a,0x0f05000e,0x07040005,0x09010012,0x07030006,0x09020007,0x0f08000f,
0x00000000,0x01010001,0x09030005,0x0f060025,0x0a010022,0x09030006,0x09020008,0x07040006,
0x00000000,0x0a020012,0x09030006,0x07030008,0x10020047,0x07030007,0x09020009,0x09060007,
0x00000000,0x01010001,0x09020006,0x02020002,0x0801000b,0x07040005,0x07030006,0x0f070016,
0x00000000,0x10030027,0x09020007,0x07040006,0x09010013,0x07030008,0x07030007,0x09050008,
0x00000000,0x01010001,0x07040005,0x0f050064,0x0a010023,0x07040006,0x07030008,0x08040008,
0x00000000,0x0f050045,0x07040006,0x07030009,0x10020048,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0801000c,0x0f050011,0x0f040012,0x03030003,
0x0f0200e2,0x0902000b,0x04010003,0x0a040006,0x09010014,0x0a030007,0x0f040013,0x04040004,
0x00000000,0x01010001,0x0f050011,0x0a03000b,0x0a010024,0x0f050012,0x0f040014,0x08050006,
0x0f0101e1,0x0a020013,0x0f050012,0x08040007,0x10020049,0x0a030008,0x0f040015,0x0f080013,
0x00000000,0x01010001,0x08020005,0x02020002,0x0801000d,0x0a040006,0x0a030007,0x0f060035,
0x100102e1,0x10030028,0x08020006,0x0a040007,0x09010015,0x0a030009,0x0a030008,0x08050007,
0x00000000,0x01010001,0x0a040006,0x10040018,0x0a010025,0x0a040007,0x0a030009,0x08040009,
0x00000000,0x0f050046,0x0a040007,0x08040008,0x1002004a,0x0a03000a,0x0a03000a,0x0a070008,
0x00000000,0x01010001,0x0f04001c,0x02020002,0x0801000e,0x0a030007,0x0a020008,0x03030003,
0x00000000,0x0902000c,0x0f04001d,0x08050006,0x09010016,0x08040007,0x0a020009,0x0f07001e,
0x00000000,0x01010001,0x0a030007,0x0f060026,0x0a010026,0x0a030008,0x0a02000a,0x08050007,
0x00000000,0x0a020014,0x0a030008,0x08040009,0x1002004b,0x08040008,0x0a02000b,0x0a060009,
0x00000000,0x01010001,0x09020007,0x02020002,0x0801000f,0x08050006,0x08040007,0x0f050074,
0x00000000,0x10030029,0x09020008,0x08050007,0x09010017,0x08040009,0x08040008,0x09050009,
0x00000000,0x01010001,0x08050006,0x0f050065,0x0a010027,0x08050007,0x08040009,0x0804000a,
0x00000000,0x0f050047,0x08050007,0x0804000a,0x1002004c,0x0804000a,0x0804000a,0x08080008,
0x00000000,0x01010001,0x04010002,0x02020002,0x09020009,0x05020003,0x05010004,0x03030003,
0x0f0200e3,0x0902000d,0x04010003,0x0f050013,0x09010018,0x0f040014,0x05010005,0x04040004,
0x00000000,0x0f0200f2,0x05020003,0x0a03000c,0x0a010028,0x05020004,0x05010006,0x1006000c,
0x0f0101e2,0x0a020015,0x05020004,0x1005000d,0x1002004d,0x0f040015,0x05010007,0x05050005,
0x00000000,0x01010001,0x09030005,0x02020002,0x0902000a,0x0f050013,0x0f040014,0x10050011,
0x100102e2,0x1003002a,0x09030006,0x0f050014,0x09010019,0x0f040016,0x0f040015,0x09060007,
0x00000000,0x10020172,0x0f050013,0x10040019,0x0a010029,0x0f050014,0x0f040016,0x09050008,
0x00000000,0x0f050048,0x0f050014,0x1005000e,0x1002004e,0x0f040017,0x0f040017,0x0f080015,
0x00000000,0x01010001,0x0f03003b,0x02020002,0x0902000b,0x09030006,0x09020007,0x03030003,
0x00000000,0x0902000e,0x0f03003c,0x1006000c,0x0901001a,0x1005000d,0x09020008,0x0f06003d,
0x00000000,0x01010001,0x09030006,0x0f060027,0x0a01002a,0x09030007,0x09020009,0x1006000d,
0x00000000,0x0a020016,0x09030007,0x1005000f,0x1002004f,0x1005000e,0x0902000a,0x09060008,
0x00000000,0x01010001,0x09020008,0x02020002,0x0902000c,0x1006000c,0x1005000d,0x0f070017,
0x00000000,0x1003002b,0x09020009,0x1006000d,0x0901001b,0x1005000f,0x1005000e,0x0905000a,
0x00000000,0x01010001,0x1006000c,0x0f050066,0x0a01002b,0x1006000d,0x1005000f,0x09050009,
0x00000000,0x0f050049,0x1006000d,0x10050010,0x10020050,0x10050010,0x10050010,0x1009000e,
0x00000000,0x01010001,0x04010002,0x02020002,0x0902000d,0x0f040020,0x0f030021,0x03030003,
0x0f0200e4,0x0902000f,0x04010003,0x1005000d,0x0901001c,0x1004000e,0x0f030022,0x04040004,
0x00000000,0x01010001,0x0f040020,0x0a03000d,0x0a01002c,0x0f040021,0x0f030023,0x09060007,
0x0f0101e3,0x0a020017,0x0f040021,0x09050008,0x10020051,0x1004000f,0x0f030024,0x0f070022,
0x00000000,0x01010001,0x09030006,0x02020002,0x0902000e,0x1005000d,0x1004000e,0x0f060036,
0x100102e3,0x1003002c,0x09030007,0x1005000e,0x0901001d,0x10040010,0x1004000f,0x09060008,
0x00000000,0x01010001,0x1005000d,0x1004001a,0x0a01002d,0x1005000e,0x10040010,0x0905000a,
0x00000000,0x0f05004a,0x1005000e,0x09050009,0x10020052,0x10040011,0x10040011,0x1008000f,
0x00000000,0x01010001,0x0f02007a,0x02020002,0x0902000f,0x0a030008,0x0a020009,0x03030003,
0x00000000,0x09020010,0x0f02007b,0x09060007,0x0901001e,0x09050008,0x0a02000a,0x0f05007c,
0x00000000,0x01010001,0x0a030008,0x0f060028,0x0a01002e,0x0a030009,0x0a02000b,0x09060008,
0x00000000,0x0a020018,0x0a030009,0x0905000a,0x10020053,0x09050009,0x0a02000c,0x0a06000a,
0x00000000,0x01010001,0x09020009,0x02020002,0x09020010,0x09060007,0x09050008,0x0f050075,
0x00000000,0x1003002d,0x0902000a,0x09060008,0x0901001f,0x0905000a,0x09050009,0x0905000b,
0x00000000,0x01010001,0x09060007,0x0f050067,0x0a01002f,0x09060008,0x0905000a,0x0905000b,
0x00000000,0x0f05004b,0x09060008,0x0905000b,0x10020054,0x0905000b,0x0905000b,0x09090009,
0x00000000,0x01010001,0x04010002,0x02020002,0x0f05000d,0x05020003,0x05010004,0x03030003,
0x0f0200e5,0x0a03000a,0x04010003,0x06030004,0x0a020011,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f0200f3,0x05020003,0x0a03000e,0x0a010030,0x05020004,0x05010006,0x0f050014,
0x0f0101e4,0x0a020019,0x05020004,0x0f040015,0x10020055,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0f060009,0x100300bb,0x0f05000e,0x06030004,0x06020005,0x10050012,
0x100102e4,0x1003002e,0x0f06000a,0x06030005,0x0a020012,0x06020007,0x06020006,0x0f09000b,
0x00000000,0x10020173,0x06030004,0x1004001b,0x0a010031,0x06030005,0x06020007,0x0f08000c,
0x00000000,0x0f05004c,0x06030005,0x0f040016,0x10020056,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0f05000e,0x02020002,0x0f05000f,0x0a040006,0x0a030007,0x03030003,
0x00000000,0x0a03000b,0x0f05000f,0x0f050014,0x0a020013,0x0f040015,0x0a030008,0x0f080010,
0x00000000,0x01010001,0x0a040006,0x0f060029,0x0a010032,0x0a040007,0x0a030009,0x0f050015,
0x00000000,0x0a02001a,0x0a040007,0x0f040017,0x10020057,0x0f040016,0x0a03000a,0x0a070008,
0x00000000,0x01010001,0x0a030007,0x02020002,0x0f050010,0x0f050014,0x0f040015,0x0f070018,
0x00000000,0x1003002f,0x0a030008,0x0f050015,0x0a020014,0x0f040017,0x0f040016,0x0a060009,
0x00000000,0x01010001,0x0f050014,0x0f050068,0x0a010033,0x0f050015,0x0f040017,0x0f08000d,
0x00000000,0x0f05004d,0x0f050015,0x0f040018,0x10020058,0x0f040018,0x0f040018,0x0f080016,
0x00000000,0x01010001,0x04010002,0x02020002,0x0f050011,0x0f03003f,0x0f020040,0x03030003,
0x0f0200e6,0x0a03000c,0x04010003,0x0a040007,0x0a020015,0x0a030008,0x0f020041,0x04040004,
0x00000000,0x01010001,0x0f03003f,0x0a03000f,0x0a010034,0x0f030040,0x0f020042,0x0f09000b,
0x0f0101e5,0x0a02001b,0x0f030040,0x0f08000c,0x10020059,0x0a030009,0x0f020043,0x0f060041,
0x00000000,0x01010001,0x0f06000a,0x02020002,0x0f050012,0x0a040007,0x0a030008,0x0f060037,
0x100102e5,0x10030030,0x0f06000b,0x0a040008,0x0a020016,0x0a03000a,0x0a030009,0x0f09000c,
0x00000000,0x01010001,0x0a040007,0x1004001c,0x0a010035,0x0a040008,0x0a03000a,0x0f08000e,
0x00000000,0x0f05004e,0x0a040008,0x0f08000d,0x1002005a,0x0a03000b,0x0a03000b,0x0a070009,
0x00000000,0x01010001,0x0f04001d,0x02020002,0x0f050013,0x0a030009,0x0a02000a,0x03030003,
0x00000000,0x0a03000d,0x0f04001e,0x0f09000b,0x0a020017,0x0f08000c,0x0a02000b,0x0f07001f,
0x00000000,0x01010001,0x0a030009,0x0f06002a,0x0a010036,0x0a03000a,0x0a02000c,0x0f09000c,
0x00000000,0x0a02001c,0x0a03000a,0x0f08000e,0x1002005b,0x0f08000d,0x0a02000d,0x0a06000b,
0x00000000,0x01010001,0x0a030008,0x02020002,0x0f050014,0x0f09000b,0x0f08000c,0x0f050076,
0x00000000,0x10030031,0x0a030009,0x0f09000c,0x0a020018,0x0f08000e,0x0f08000d,0x0a06000a,
0x00000000,0x01010001,0x0f09000b,0x0f050069,0x0a010037,0x0f09000c,0x0f08000e,0x0f08000f,
0x00000000,0x0f05004f,0x0f09000c,0x0f08000f,0x1002005c,0x0f08000f,0x0f08000f,0x0f0c000d,
0x00000000,0x01010001,0x04010002,0x02020002,0x0a03000a,0x05020003,0x05010004,0x03030003,
0x0f0200e7,0x0a03000e,0x04010003,0x0f040022,0x0a020019,0x0f030023,0x05010005,0x04040004,
0x00000000,0x0f0200f4,0x05020003,0x0a030010,0x0a010038,0x05020004,0x05010006,0x0f08000c,
0x0f0101e6,0x0a02001d,0x05020004,0x0f07000d,0x1002005d,0x0f030024,0x05010007,0x05050005,
0x00000000,0x01010001,0x0a040006,0x02020002,0x0a03000b,0x0f040022,0x0f030023,0x10050013,
0x100102e6,0x10030032,0x0a040007,0x0f040023,0x0a02001a,0x0f030025,0x0f030024,0x0a070008,
0x00000000,0x10020174,0x0f040022,0x1004001d,0x0a010039,0x0f040023,0x0f030025,0x0a060009,
0x00000000,0x0f050050,0x0f040023,0x0f07000e,0x1002005e,0x0f030026,0x0f030026,0x0f070024,
0x00000000,0x01010001,0x0f03003c,0x02020002,0x0a03000c,0x0a040007,0x0a030008,0x03030003,
0x00000000,0x0a03000f,0x0f03003d,0x0f08000c,0x0a02001b,0x0f07000d,0x0a030009,0x0f06003e,
0x00000000,0x01010001,0x0a040007,0x0f06002b,0x0a01003a,0x0a040008,0x0a03000a,0x0f08000d,
0x00000000,0x0a02001e,0x0a040008,0x0f07000f,0x1002005f,0x0f07000e,0x0a03000b,0x0a070009,
0x00000000,0x01010001,0x0a030009,0x02020002,0x0a03000d,0x0f08000c,0x0f07000d,0x0f070019,
0x00000000,0x10030033,0x0a03000a,0x0f08000d,0x0a02001c,0x0f07000f,0x0f07000e,0x0a06000b,
0x00000000,0x01010001,0x0f08000c,0x0f05006a,0x0a01003b,0x0f08000d,0x0f07000f,0x0a06000a,
0x00000000,0x0f050051,0x0f08000d,0x0f070010,0x10020060,0x0f070010,0x0f070010,0x0f0b000e,
0x00000000,0x01010001,0x04010002,0x02020002,0x0a03000e,0x1003005f,0x10020060,0x03030003,
0x0f0200e8,0x0a030010,0x04010003,0x1005000e,0x0a02001d,0x1004000f,0x10020061,0x04040004,
0x00000000,0x01010001,0x1003005f,0x0a030011,0x0a01003c,0x10030060,0x10020062,0x0a070008,
0x0f0101e7,0x0a02001f,0x10030060,0x0a060009,0x10020061,0x10040010,0x10020063,0x10060061,
0x00000000,0x01010001,0x0a040007,0x02020002,0x0a03000f,0x1005000e,0x1004000f,0x0f060038,
0x100102e7,0x10030034,0x0a040008,0x1005000f,0x0a02001e,0x10040011,0x10040010,0x0a070009,
0x00000000,0x01010001,0x1005000e,0x1004001e,0x0a01003d,0x1005000f,0x10040011,0x0a06000b,
0x00000000,0x0f050052,0x1005000f,0x0a06000a,0x10020062,0x10040012,0x10040012,0x10080010,
0x00000000,0x01010001,0x0f02007b,0x02020002,0x0a030010,0x0a03000a,0x0a02000b,0x03030003,
0x00000000,0x0a030011,0x0f02007c,0x0a070008,0x0a02001f,0x0a060009,0x0a02000c,0x0f05007d,
0x00000000,0x01010001,0x0a03000a,0x0f06002c,0x0a01003e,0x0a03000b,0x0a02000d,0x0a070009,
0x00000000,0x0a020020,0x0a03000b,0x0a06000b,0x10020063,0x0a06000a,0x0a02000e,0x0a06000c,
0x00000000,0x01010001,0x0a03000a,0x02020002,0x0a030011,0x0a070008,0x0a060009,0x0f050077,
0x00000000,0x10030035,0x0a03000b,0x0a070009,0x0a020020,0x0a06000b,0x0a06000a,0x0a06000c,
0x00000000,0x01010001,0x0a070008,0x0f05006b,0x0a01003f,0x0a070009,0x0a06000b,0x0a06000c,
0x00000000,0x0f050053,0x0a070009,0x0a06000c,0x10020064,0x0a06000c,0x0a06000c,0x0a0a000a,
0x00000000,0x01010001,0x04010002,0x02020002,0x08010008,0x05020003,0x05010004,0x03030003,
0x0f0200e9,0x0f05000e,0x04010003,0x06030004,0x0f040015,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f0200f5,0x05020003,0x10050010,0x10030026,0x05020004,0x05010006,0x07040005,
0x0f0101e8,0x10040017,0x05020004,0x07030006,0x10020065,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x08020004,0x100300bc,0x08010009,0x06030004,0x06020005,0x10050014,
0x100102e8,0x10030036,0x08020005,0x06030005,0x0f040016,0x06020007,0x06020006,0x08050006,
0x00000000,0x10020175,0x06030004,0x1004001f,0x10030027,0x06030005,0x06020007,0x08040007,
0x00000000,0x0f050054,0x06030005,0x07030007,0x10020066,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0f05000f,0x02020002,0x0801000a,0x0f06000a,0x0f05000b,0x03030003,
0x00000000,0x0f05000f,0x0f050010,0x07040005,0x0f040017,0x07030006,0x0f05000c,0x0f080011,
0x00000000,0x01010001,0x0f06000a,0x0f06002d,0x10030028,0x0f06000b,0x0f05000d,0x07040006,
0x00000000,0x10040018,0x0f06000b,0x07030008,0x10020067,0x07030007,0x0f05000e,0x0f09000c,
0x00000000,0x01010001,0x0f05000b,0x02020002,0x0801000b,0x07040005,0x07030006,0x0f07001a,
0x00000000,0x10030037,0x0f05000c,0x07040006,0x0f040018,0x07030008,0x07030007,0x0f08000d,
0x00000000,0x01010001,0x07040005,0x0f05006c,0x10030029,0x07040006,0x07030008,0x08040008,
0x00000000,0x0f050055,0x07040006,0x07030009,0x10020068,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0801000c,0x0f050012,0x0f040013,0x03030003,
0x0f0200ea,0x0f050010,0x04010003,0x1006000c,0x0f040019,0x1005000d,0x0f040014,0x04040004,
0x00000000,0x01010001,0x0f050012,0x10050011,0x1003002a,0x0f050013,0x0f040015,0x08050006,
0x0f0101e9,0x10040019,0x0f050013,0x08040007,0x10020069,0x1005000e,0x0f040016,0x0f080014,
0x00000000,0x01010001,0x08020005,0x02020002,0x0801000d,0x1006000c,0x1005000d,0x0f060039,
0x100102e9,0x10030038,0x08020006,0x1006000d,0x0f04001a,0x1005000f,0x1005000e,0x08050007,
0x00000000,0x01010001,0x1006000c,0x10040020,0x1003002b,0x1006000d,0x1005000f,0x08040009,
0x00000000,0x0f050056,0x1006000d,0x08040008,0x1002006a,0x10050010,0x10050010,0x1009000e,
0x00000000,0x01010001,0x0f04001e,0x02020002,0x0801000e,0x1005000d,0x1004000e,0x03030003,
0x00000000,0x0f050011,0x0f04001f,0x08050006,0x0f04001b,0x08040007,0x1004000f,0x0f070020,
0x00000000,0x01010001,0x1005000d,0x0f06002e,0x1003002c,0x1005000e,0x10040010,0x08050007,
0x00000000,0x1004001a,0x1005000e,0x08040009,0x1002006b,0x08040008,0x10040011,0x1008000f,
0x00000000,0x01010001,0x0f05000c,0x02020002,0x0801000f,0x08050006,0x08040007,0x0f050078,
0x00000000,0x10030039,0x0f05000d,0x08050007,0x0f04001c,0x08040009,0x08040008,0x0f08000e,
0x00000000,0x01010001,0x08050006,0x0f05006d,0x1003002d,0x08050007,0x08040009,0x0804000a,
0x00000000,0x0f050057,0x08050007,0x0804000a,0x1002006c,0x0804000a,0x0804000a,0x08080008,
0x00000000,0x01010001,0x04010002,0x02020002,0x0f05000e,0x05020003,0x05010004,0x03030003,
0x0f0200eb,0x0f050012,0x04010003,0x10040032,0x0f04001d,0x10030033,0x05010005,0x04040004,
0x00000000,0x0f0200f6,0x05020003,0x10050012,0x1003002e,0x05020004,0x05010006,0x1006000d,
0x0f0101ea,0x1004001b,0x05020004,0x1005000e,0x1002006d,0x10030034,0x05010007,0x05050005,
0x00000000,0x01010001,0x0f06000a,0x02020002,0x0f05000f,0x10040032,0x10030033,0x10050015,
0x100102ea,0x1003003a,0x0f06000b,0x10040033,0x0f04001e,0x10030035,0x10030034,0x0f09000c,
0x00000000,0x10020176,0x10040032,0x10040021,0x1003002f,0x10040033,0x10030035,0x0f08000d,
0x00000000,0x0f050058,0x10040033,0x1005000f,0x1002006e,0x10030036,0x10030036,0x10070034,
0x00000000,0x01010001,0x0f03003d,0x02020002,0x0f050010,0x0f06000b,0x0f05000c,0x03030003,
0x00000000,0x0f050013,0x0f03003e,0x1006000d,0x0f04001f,0x1005000e,0x0f05000d,0x0f06003f,
0x00000000,0x01010001,0x0f06000b,0x0f06002f,0x10030030,0x0f06000c,0x0f05000e,0x1006000e,
0x00000000,0x1004001c,0x0f06000c,0x10050010,0x1002006f,0x1005000f,0x0f05000f,0x0f09000d,
0x00000000,0x01010001,0x0f05000d,0x02020002,0x0f050011,0x1006000d,0x1005000e,0x0f07001b,
0x00000000,0x1003003b,0x0f05000e,0x1006000e,0x0f040020,0x10050010,0x1005000f,0x0f08000f,
0x00000000,0x01010001,0x1006000d,0x0f05006e,0x10030031,0x1006000e,0x10050010,0x0f08000e,
0x00000000,0x0f050059,0x1006000e,0x10050011,0x10020070,0x10050011,0x10050011,0x1009000f,
0x00000000,0x01010001,0x04010002,0x02020002,0x0f050012,0x0f040021,0x0f030022,0x03030003,
0x0f0200ec,0x0f050014,0x04010003,0x1005000f,0x0f040021,0x10040010,0x0f030023,0x04040004,
0x00000000,0x01010001,0x0f040021,0x10050013,0x10030032,0x0f040022,0x0f030024,0x0f09000c,
0x0f0101eb,0x1004001d,0x0f040022,0x0f08000d,0x10020071,0x10040011,0x0f030025,0x0f070023,
0x00000000,0x01010001,0x0f06000b,0x02020002,0x0f050013,0x1005000f,0x10040010,0x0f06003a,
0x100102eb,0x1003003c,0x0f06000c,0x10050010,0x0f040022,0x10040012,0x10040011,0x0f09000d,
0x00000000,0x01010001,0x1005000f,0x10040022,0x10030033,0x10050010,0x10040012,0x0f08000f,
0x00000000,0x0f05005a,0x10050010,0x0f08000e,0x10020072,0x10040013,0x10040013,0x10080011,
0x00000000,0x01010001,0x0f02007c,0x02020002,0x0f050014,0x1005000e,0x1004000f,0x03030003,
0x00000000,0x0f050015,0x0f02007d,0x0f09000c,0x0f040023,0x0f08000d,0x10040010,0x0f05007e,
0x00000000,0x01010001,0x1005000e,0x0f060030,0x10030034,0x1005000f,0x10040011,0x0f09000d,
0x00000000,0x1004001e,0x1005000f,0x0f08000f,0x10020073,0x0f08000e,0x10040012,0x10080010,
0x00000000,0x01010001,0x0f05000e,0x02020002,0x0f050015,0x0f09000c,0x0f08000d,0x0f050079,
0x00000000,0x1003003d,0x0f05000f,0x0f09000d,0x0f040024,0x0f08000f,0x0f08000e,0x0f080010,
0x00000000,0x01010001,0x0f09000c,0x0f05006f,0x10030035,0x0f09000d,0x0f08000f,0x0f080010,
0x00000000,0x0f05005b,0x0f09000d,0x0f080010,0x10020074,0x0f080010,0x0f080010,0x0f0c000e,
0x00000000,0x01010001,0x04010002,0x02020002,0x0f040010,0x05020003,0x05010004,0x03030003,
0x0f0200ed,0x10050010,0x04010003,0x06030004,0x10040017,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f0200f7,0x05020003,0x10050014,0x10030036,0x05020004,0x05010006,0x1005001c,
0x0f0101ec,0x1004001f,0x05020004,0x1004001d,0x10020075,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0f05000c,0x100300bd,0x0f040011,0x06030004,0x06020005,0x10050016,
0x100102ec,0x1003003e,0x0f05000d,0x06030005,0x10040018,0x06020007,0x06020006,0x0f08000e,
0x00000000,0x10020177,0x06030004,0x10040023,0x10030037,0x06030005,0x06020007,0x0f07000f,
0x00000000,0x0f05005c,0x06030005,0x1004001e,0x10020076,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0f050010,0x02020002,0x0f040012,0x1006000c,0x1005000d,0x03030003,
0x00000000,0x10050011,0x0f050011,0x1005001c,0x10040019,0x1004001d,0x1005000e,0x0f080012,
0x00000000,0x01010001,0x1006000c,0x0f060031,0x10030038,0x1006000d,0x1005000f,0x1005001d,
0x00000000,0x10040020,0x1006000d,0x1004001f,0x10020077,0x1004001e,0x10050010,0x1009000e,
0x00000000,0x01010001,0x1005000d,0x02020002,0x0f040013,0x1005001c,0x1004001d,0x0f07001c,
0x00000000,0x1003003f,0x1005000e,0x1005001d,0x1004001a,0x1004001f,0x1004001e,0x1008000f,
0x00000000,0x01010001,0x1005001c,0x0f050070,0x10030039,0x1005001d,0x1004001f,0x0f070010,
0x00000000,0x0f05005d,0x1005001d,0x10040020,0x10020078,0x10040020,0x10040020,0x1008001e,
0x00000000,0x01010001,0x04010002,0x02020002,0x0f040014,0x0f030040,0x0f020041,0x03030003,
0x0f0200ee,0x10050012,0x04010003,0x1006000d,0x1004001b,0x1005000e,0x0f020042,0x04040004,
0x00000000,0x01010001,0x0f030040,0x10050015,0x1003003a,0x0f030041,0x0f020043,0x0f08000e,
0x0f0101ed,0x10040021,0x0f030041,0x0f07000f,0x10020079,0x1005000f,0x0f020044,0x0f060042,
0x00000000,0x01010001,0x0f05000d,0x02020002,0x0f040015,0x1006000d,0x1005000e,0x0f06003b,
0x100102ed,0x10030040,0x0f05000e,0x1006000e,0x1004001c,0x10050010,0x1005000f,0x0f08000f,
0x00000000,0x01010001,0x1006000d,0x10040024,0x1003003b,0x1006000e,0x10050010,0x0f070011,
0x00000000,0x0f05005e,0x1006000e,0x0f070010,0x1002007a,0x10050011,0x10050011,0x1009000f,
0x00000000,0x01010001,0x0f04001f,0x02020002,0x0f040016,0x1005000f,0x10040010,0x03030003,
0x00000000,0x10050013,0x0f040020,0x0f08000e,0x1004001d,0x0f07000f,0x10040011,0x0f070021,
0x00000000,0x01010001,0x1005000f,0x0f060032,0x1003003c,0x10050010,0x10040012,0x0f08000f,
0x00000000,0x10040022,0x10050010,0x0f070011,0x1002007b,0x0f070010,0x10040013,0x10080011,
0x00000000,0x01010001,0x1005000e,0x02020002,0x0f040017,0x0f08000e,0x0f07000f,0x0f05007a,
0x00000000,0x10030041,0x1005000f,0x0f08000f,0x1004001e,0x0f070011,0x0f070010,0x10080010,
0x00000000,0x01010001,0x0f08000e,0x0f050071,0x1003003d,0x0f08000f,0x0f070011,0x0f070012,
0x00000000,0x0f05005f,0x0f08000f,0x0f070012,0x1002007c,0x0f070012,0x0f070012,0x0f0b0010,
0x00000000,0x01010001,0x04010002,0x02020002,0x10050010,0x05020003,0x05010004,0x03030003,
0x0f0200ef,0x10050014,0x04010003,0x06030004,0x1004001f,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f0200f8,0x05020003,0x10050016,0x1003003e,0x05020004,0x05010006,0x0f08000d,
0x0f0101ee,0x10040023,0x05020004,0x0f07000e,0x1002007d,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x1006000c,0x02020002,0x10050011,0x06030004,0x06020005,0x10050017,
0x100102ee,0x10030042,0x1006000d,0x06030005,0x10040020,0x06020007,0x06020006,0x1009000e,
0x00000000,0x10020178,0x06030004,0x10040025,0x1003003f,0x06030005,0x06020007,0x1008000f,
0x00000000,0x0f050060,0x06030005,0x0f07000f,0x1002007e,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0f03003e,0x02020002,0x10050012,0x1006000d,0x1005000e,0x03030003,
0x00000000,0x10050015,0x0f03003f,0x0f08000d,0x10040021,0x0f07000e,0x1005000f,0x0f060040,
0x00000000,0x01010001,0x1006000d,0x0f060033,0x10030040,0x1006000e,0x10050010,0x0f08000e,
0x00000000,0x10040024,0x1006000e,0x0f070010,0x1002007f,0x0f07000f,0x10050011,0x1009000f,
0x00000000,0x01010001,0x1005000f,0x02020002,0x10050013,0x0f08000d,0x0f07000e,0x0f07001d,
0x00000000,0x10030043,0x10050010,0x0f08000e,0x10040022,0x0f070010,0x0f07000f,0x10080011,
0x00000000,0x01010001,0x0f08000d,0x0f050072,0x10030041,0x0f08000e,0x0f070010,0x10080010,
0x00000000,0x0f050061,0x0f08000e,0x0f070011,0x10020080,0x0f070011,0x0f070011,0x0f0b000f,
0x00000000,0x01010001,0x04010002,0x02020002,0x10050014,0x10030060,0x10020061,0x03030003,
0x0f0200f0,0x10050016,0x04010003,0x10050010,0x10040023,0x10040011,0x10020062,0x04040004,
0x00000000,0x01010001,0x10030060,0x10050017,0x10030042,0x10030061,0x10020063,0x1009000e,
0x0f0101ef,0x10040025,0x10030061,0x1008000f,0x10020081,0x10040012,0x10020064,0x10060062,
0x00000000,0x01010001,0x1006000d,0x02020002,0x10050015,0x10050010,0x10040011,0x0f06003c,
0x100102ef,0x10030044,0x1006000e,0x10050011,0x10040024,0x10040013,0x10040012,0x1009000f,
0x00000000,0x01010001,0x10050010,0x10040026,0x10030043,0x10050011,0x10040013,0x10080011,
0x00000000,0x0f050062,0x10050011,0x10080010,0x10020082,0x10040014,0x10040014,0x10080012,
0x00000000,0x01010001,0x0f02007d,0x02020002,0x10050016,0x10050010,0x10040011,0x03030003,
0x00000000,0x10050017,0x0f02007e,0x1009000e,0x10040025,0x1008000f,0x10040012,0x0f05007f,
0x00000000,0x01010001,0x10050010,0x0f060034,0x10030044,0x10050011,0x10040013,0x1009000f,
0x00000000,0x10040026,0x10050011,0x10080011,0x10020083,0x10080010,0x10040014,0x10080012,
0x00000000,0x01010001,0x10050010,0x02020002,0x10050017,0x1009000e,0x1008000f,0x0f05007b,
0x00000000,0x10030045,0x10050011,0x1009000f,0x10040026,0x10080011,0x10080010,0x10080012,
0x00000000,0x01010001,0x1009000e,0x0f050073,0x10030045,0x1009000f,0x10080011,0x10080012,
0x00000000,0x0f050063,0x1009000f,0x10080012,0x10020084,0x10080012,0x10080012,0x100c0010,
0x00000000,0x01010001,0x04010002,0x02020002,0x08010008,0x05020003,0x05010004,0x03030003,
0x0f0200f1,0x09020009,0x04010003,0x06030004,0x09010010,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f0200f9,0x05020003,0x0f040011,0x0f020027,0x05020004,0x05010006,0x07040005,
0x0f0101f0,0x0f030018,0x05020004,0x07030006,0x0f050044,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x08020004,0x100300be,0x08010009,0x06030004,0x06020005,0x0f08000f,
0x100102f0,0x0f060025,0x08020005,0x06030005,0x09010011,0x06020007,0x06020006,0x08050006,
0x00000000,0x10020179,0x06030004,0x0f070016,0x0f020028,0x06030005,0x06020007,0x08040007,
0x00000000,0x0f050064,0x06030005,0x07030007,0x0f050045,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0f050011,0x02020002,0x0801000a,0x09030005,0x09020006,0x03030003,
0x00000000,0x0902000a,0x0f050012,0x07040005,0x09010012,0x07030006,0x09020007,0x0f080013,
0x00000000,0x01010001,0x09030005,0x0f060035,0x0f020029,0x09030006,0x09020008,0x07040006,
0x00000000,0x0f030019,0x09030006,0x07030008,0x0f050046,0x07030007,0x09020009,0x09060007,
0x00000000,0x01010001,0x09020006,0x02020002,0x0801000b,0x07040005,0x07030006,0x0f07001e,
0x00000000,0x0f060026,0x09020007,0x07040006,0x09010013,0x07030008,0x07030007,0x09050008,
0x00000000,0x01010001,0x07040005,0x0f050074,0x0f02002a,0x07040006,0x07030008,0x08040008,
0x00000000,0x0f050065,0x07040006,0x07030009,0x0f050047,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0801000c,0x0f050013,0x0f040014,0x03030003,
0x0f0200f2,0x0902000b,0x04010003,0x0f05000d,0x09010014,0x0f04000e,0x0f040015,0x04040004,
0x00000000,0x01010001,0x0f050013,0x0f040012,0x0f02002b,0x0f050014,0x0f040016,0x08050006,
0x0f0101f1,0x0f03001a,0x0f050014,0x08040007,0x0f050048,0x0f04000f,0x0f040017,0x0f080015,
0x00000000,0x01010001,0x08020005,0x02020002,0x0801000d,0x0f05000d,0x0f04000e,0x0f06003d,
0x100102f1,0x0f060027,0x08020006,0x0f05000e,0x09010015,0x0f040010,0x0f04000f,0x08050007,
0x00000000,0x01010001,0x0f05000d,0x0f070017,0x0f02002c,0x0f05000e,0x0f040010,0x08040009,
0x00000000,0x0f050066,0x0f05000e,0x08040008,0x0f050049,0x0f040011,0x0f040011,0x0f08000f,
0x00000000,0x01010001,0x0f040020,0x02020002,0x0801000e,0x0f04000e,0x0f03000f,0x03030003,
0x00000000,0x0902000c,0x0f040021,0x08050006,0x09010016,0x08040007,0x0f030010,0x0f070022,
0x00000000,0x01010001,0x0f04000e,0x0f060036,0x0f02002d,0x0f04000f,0x0f030011,0x08050007,
0x00000000,0x0f03001b,0x0f04000f,0x08040009,0x0f05004a,0x08040008,0x0f030012,0x0f070010,
0x00000000,0x01010001,0x09020007,0x02020002,0x0801000f,0x08050006,0x08040007,0x0f05007c,
0x00000000,0x0f060028,0x09020008,0x08050007,0x09010017,0x08040009,0x08040008,0x09050009,
0x00000000,0x01010001,0x08050006,0x0f050075,0x0f02002e,0x08050007,0x08040009,0x0804000a,
0x00000000,0x0f050067,0x08050007,0x0804000a,0x0f05004b,0x0804000a,0x0804000a,0x08080008,
0x00000000,0x01010001,0x04010002,0x02020002,0x09020009,0x05020003,0x05010004,0x03030003,
0x0f0200f3,0x0902000d,0x04010003,0x0f050014,0x09010018,0x0f040015,0x05010005,0x04040004,
0x00000000,0x0f0200fa,0x05020003,0x0f040013,0x0f02002f,0x05020004,0x05010006,0x0f09000b,
0x0f0101f2,0x0f03001c,0x05020004,0x0f08000c,0x0f05004c,0x0f040016,0x05010007,0x05050005,
0x00000000,0x01010001,0x09030005,0x02020002,0x0902000a,0x0f050014,0x0f040015,0x0f080010,
0x100102f2,0x0f060029,0x09030006,0x0f050015,0x09010019,0x0f040017,0x0f040016,0x09060007,
0x00000000,0x1002017a,0x0f050014,0x0f070018,0x0f020030,0x0f050015,0x0f040017,0x09050008,
0x00000000,0x0f050068,0x0f050015,0x0f08000d,0x0f05004d,0x0f040018,0x0f040018,0x0f080016,
0x00000000,0x01010001,0x0f03003f,0x02020002,0x0902000b,0x09030006,0x09020007,0x03030003,
0x00000000,0x0902000e,0x0f030040,0x0f09000b,0x0901001a,0x0f08000c,0x09020008,0x0f060041,
0x00000000,0x01010001,0x09030006,0x0f060037,0x0f020031,0x09030007,0x09020009,0x0f09000c,
0x00000000,0x0f03001d,0x09030007,0x0f08000e,0x0f05004e,0x0f08000d,0x0902000a,0x09060008,
0x00000000,0x01010001,0x09020008,0x02020002,0x0902000c,0x0f09000b,0x0f08000c,0x0f07001f,
0x00000000,0x0f06002a,0x09020009,0x0f09000c,0x0901001b,0x0f08000e,0x0f08000d,0x0905000a,
0x00000000,0x01010001,0x0f09000b,0x0f050076,0x0f020032,0x0f09000c,0x0f08000e,0x09050009,
0x00000000,0x0f050069,0x0f09000c,0x0f08000f,0x0f05004f,0x0f08000f,0x0f08000f,0x0f0c000d,
0x00000000,0x01010001,0x04010002,0x02020002,0x0902000d,0x0f040022,0x0f030023,0x03030003,
0x0f0200f4,0x0902000f,0x04010003,0x0f08000c,0x0901001c,0x0f07000d,0x0f030024,0x04040004,
0x00000000,0x01010001,0x0f040022,0x0f040014,0x0f020033,0x0f040023,0x0f030025,0x09060007,
0x0f0101f3,0x0f03001e,0x0f040023,0x09050008,0x0f050050,0x0f07000e,0x0f030026,0x0f070024,
0x00000000,0x01010001,0x09030006,0x02020002,0x0902000e,0x0f08000c,0x0f07000d,0x0f06003e,
0x100102f3,0x0f06002b,0x09030007,0x0f08000d,0x0901001d,0x0f07000f,0x0f07000e,0x09060008,
0x00000000,0x01010001,0x0f08000c,0x0f070019,0x0f020034,0x0f08000d,0x0f07000f,0x0905000a,
0x00000000,0x0f05006a,0x0f08000d,0x09050009,0x0f050051,0x0f070010,0x0f070010,0x0f0b000e,
0x00000000,0x01010001,0x0f02007e,0x02020002,0x0902000f,0x0f04000f,0x0f030010,0x03030003,
0x00000000,0x09020010,0x0f02007f,0x09060007,0x0901001e,0x09050008,0x0f030011,0x0f050080,
0x00000000,0x01010001,0x0f04000f,0x0f060038,0x0f020035,0x0f040010,0x0f030012,0x09060008,
0x00000000,0x0f03001f,0x0f040010,0x0905000a,0x0f050052,0x09050009,0x0f030013,0x0f070011,
0x00000000,0x01010001,0x09020009,0x02020002,0x09020010,0x09060007,0x09050008,0x0f05007d,
0x00000000,0x0f06002c,0x0902000a,0x09060008,0x0901001f,0x0905000a,0x09050009,0x0905000b,
0x00000000,0x01010001,0x09060007,0x0f050077,0x0f020036,0x09060008,0x0905000a,0x0905000b,
0x00000000,0x0f05006b,0x09060008,0x0905000b,0x0f050053,0x0905000b,0x0905000b,0x09090009,
0x00000000,0x01010001,0x04010002,0x02020002,0x0f05000e,0x05020003,0x05010004,0x03030003,
0x0f0200f5,0x0f040011,0x04010003,0x06030004,0x0f030018,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f0200fb,0x05020003,0x0f040015,0x0f020037,0x05020004,0x05010006,0x07040005,
0x0f0101f4,0x0f030020,0x05020004,0x07030006,0x0f050054,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0f06000a,0x100300bf,0x0f05000f,0x06030004,0x06020005,0x0f080011,
0x100102f4,0x0f06002d,0x0f06000b,0x06030005,0x0f030019,0x06020007,0x06020006,0x0f09000c,
0x00000000,0x1002017b,0x06030004,0x0f07001a,0x0f020038,0x06030005,0x06020007,0x0f08000d,
0x00000000,0x0f05006c,0x06030005,0x07030007,0x0f050055,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0f050012,0x02020002,0x0f050010,0x0f05000d,0x0f04000e,0x03030003,
0x00000000,0x0f040012,0x0f050013,0x07040005,0x0f03001a,0x07030006,0x0f04000f,0x0f080014,
0x00000000,0x01010001,0x0f05000d,0x0f060039,0x0f020039,0x0f05000e,0x0f040010,0x07040006,
0x00000000,0x0f030021,0x0f05000e,0x07030008,0x0f050056,0x07030007,0x0f040011,0x0f08000f,
0x00000000,0x01010001,0x0f04000e,0x02020002,0x0f050011,0x07040005,0x07030006,0x0f070020,
0x00000000,0x0f06002e,0x0f04000f,0x07040006,0x0f03001b,0x07030008,0x07030007,0x0f070010,
0x00000000,0x01010001,0x07040005,0x0f050078,0x0f02003a,0x07040006,0x07030008,0x0f08000e,
0x00000000,0x0f05006d,0x07040006,0x07030009,0x0f050057,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0f050012,0x0f030041,0x0f020042,0x03030003,
0x0f0200f6,0x0f040013,0x04010003,0x0f05000e,0x0f03001c,0x0f04000f,0x0f020043,0x04040004,
0x00000000,0x01010001,0x0f030041,0x0f040016,0x0f02003b,0x0f030042,0x0f020044,0x0f09000c,
0x0f0101f5,0x0f030022,0x0f030042,0x0f08000d,0x0f050058,0x0f040010,0x0f020045,0x0f060043,
0x00000000,0x01010001,0x0f06000b,0x02020002,0x0f050013,0x0f05000e,0x0f04000f,0x0f06003f,
0x100102f5,0x0f06002f,0x0f06000c,0x0f05000f,0x0f03001d,0x0f040011,0x0f040010,0x0f09000d,
0x00000000,0x01010001,0x0f05000e,0x0f07001b,0x0f02003c,0x0f05000f,0x0f040011,0x0f08000f,
0x00000000,0x0f05006e,0x0f05000f,0x0f08000e,0x0f050059,0x0f040012,0x0f040012,0x0f080010,
0x00000000,0x01010001,0x0f040021,0x02020002,0x0f050014,0x0f040010,0x0f030011,0x03030003,
0x00000000,0x0f040014,0x0f040022,0x0f09000c,0x0f03001e,0x0f08000d,0x0f030012,0x0f070023,
0x00000000,0x01010001,0x0f040010,0x0f06003a,0x0f02003d,0x0f040011,0x0f030013,0x0f09000d,
0x00000000,0x0f030023,0x0f040011,0x0f08000f,0x0f05005a,0x0f08000e,0x0f030014,0x0f070012,
0x00000000,0x01010001,0x0f04000f,0x02020002,0x0f050015,0x0f09000c,0x0f08000d,0x0f05007e,
0x00000000,0x0f060030,0x0f040010,0x0f09000d,0x0f03001f,0x0f08000f,0x0f08000e,0x0f070011,
0x00000000,0x01010001,0x0f09000c,0x0f050079,0x0f02003e,0x0f09000d,0x0f08000f,0x0f080010,
0x00000000,0x0f05006f,0x0f09000d,0x0f080010,0x0f05005b,0x0f080010,0x0f080010,0x0f0c000e,
0x00000000,0x01010001,0x04010002,0x02020002,0x0f040011,0x05020003,0x05010004,0x03030003,
0x0f0200f7,0x0f040015,0x04010003,0x0f040023,0x0f030020,0x0f030024,0x05010005,0x04040004,
0x00000000,0x0f0200fc,0x05020003,0x0f040017,0x0f02003f,0x05020004,0x05010006,0x0f08000e,
0x0f0101f6,0x0f030024,0x05020004,0x0f07000f,0x0f05005c,0x0f030025,0x05010007,0x05050005,
0x00000000,0x01010001,0x0f05000d,0x02020002,0x0f040012,0x0f040023,0x0f030024,0x0f080012,
0x100102f6,0x0f060031,0x0f05000e,0x0f040024,0x0f030021,0x0f030026,0x0f030025,0x0f08000f,
0x00000000,0x1002017c,0x0f040023,0x0f07001c,0x0f020040,0x0f040024,0x0f030026,0x0f070010,
0x00000000,0x0f050070,0x0f040024,0x0f070010,0x0f05005d,0x0f030027,0x0f030027,0x0f070025,
0x00000000,0x01010001,0x0f030040,0x02020002,0x0f040013,0x0f05000e,0x0f04000f,0x03030003,
0x00000000,0x0f040016,0x0f030041,0x0f08000e,0x0f030022,0x0f07000f,0x0f040010,0x0f060042,
0x00000000,0x01010001,0x0f05000e,0x0f06003b,0x0f020041,0x0f05000f,0x0f040011,0x0f08000f,
0x00000000,0x0f030025,0x0f05000f,0x0f070011,0x0f05005e,0x0f070010,0x0f040012,0x0f080010,
0x00000000,0x01010001,0x0f040010,0x02020002,0x0f040014,0x0f08000e,0x0f07000f,0x0f070021,
0x00000000,0x0f060032,0x0f040011,0x0f08000f,0x0f030023,0x0f070011,0x0f070010,0x0f070012,
0x00000000,0x01010001,0x0f08000e,0x0f05007a,0x0f020042,0x0f08000f,0x0f070011,0x0f070011,
0x00000000,0x0f050071,0x0f08000f,0x0f070012,0x0f05005f,0x0f070012,0x0f070012,0x0f0b0010,
0x00000000,0x01010001,0x04010002,0x02020002,0x0f040015,0x10030061,0x10020062,0x03030003,
0x0f0200f8,0x0f040017,0x04010003,0x0f08000d,0x0f030024,0x0f07000e,0x10020063,0x04040004,
0x00000000,0x01010001,0x10030061,0x0f040018,0x0f020043,0x10030062,0x10020064,0x0f08000f,
0x0f0101f7,0x0f030026,0x10030062,0x0f070010,0x0f050060,0x0f07000f,0x10020065,0x10060063,
0x00000000,0x01010001,0x0f05000e,0x02020002,0x0f040016,0x0f08000d,0x0f07000e,0x0f060040,
0x100102f7,0x0f060033,0x0f05000f,0x0f08000e,0x0f030025,0x0f070010,0x0f07000f,0x0f080010,
0x00000000,0x01010001,0x0f08000d,0x0f07001d,0x0f020044,0x0f08000e,0x0f070010,0x0f070012,
0x00000000,0x0f050072,0x0f08000e,0x0f070011,0x0f050061,0x0f070011,0x0f070011,0x0f0b000f,
0x00000000,0x01010001,0x0f02007f,0x02020002,0x0f040017,0x0f040011,0x0f030012,0x03030003,
0x00000000,0x0f040018,0x0f020080,0x0f08000f,0x0f030026,0x0f070010,0x0f030013,0x0f050081,
0x00000000,0x01010001,0x0f040011,0x0f06003c,0x0f020045,0x0f040012,0x0f030014,0x0f080010,
0x00000000,0x0f030027,0x0f040012,0x0f070012,0x0f050062,0x0f070011,0x0f030015,0x0f070013,
0x00000000,0x01010001,0x0f040011,0x02020002,0x0f040018,0x0f08000f,0x0f070010,0x0f05007f,
0x00000000,0x0f060034,0x0f040012,0x0f080010,0x0f030027,0x0f070012,0x0f070011,0x0f070013,
0x00000000,0x01010001,0x0f08000f,0x0f05007b,0x0f020046,0x0f080010,0x0f070012,0x0f070013,
0x00000000,0x0f050073,0x0f080010,0x0f070013,0x0f050063,0x0f070013,0x0f070013,0x0f0b0011,
0x00000000,0x01010001,0x04010002,0x02020002,0x08010008,0x05020003,0x05010004,0x03030003,
0x0f0200f9,0x0f040011,0x04010003,0x06030004,0x0f030018,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f0200fd,0x05020003,0x0f08000f,0x0f060025,0x05020004,0x05010006,0x07040005,
0x0f0101f8,0x0f070016,0x05020004,0x07030006,0x0f050064,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x08020004,0x100300c0,0x08010009,0x06030004,0x06020005,0x0f080013,
0x100102f8,0x0f060035,0x08020005,0x06030005,0x0f030019,0x06020007,0x06020006,0x08050006,
0x00000000,0x1002017d,0x06030004,0x0f07001e,0x0f060026,0x06030005,0x06020007,0x08040007,
0x00000000,0x0f050074,0x06030005,0x07030007,0x0f050065,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0f050013,0x02020002,0x0801000a,0x0f05000d,0x0f04000e,0x03030003,
0x00000000,0x0f040012,0x0f050014,0x07040005,0x0f03001a,0x07030006,0x0f04000f,0x0f080015,
0x00000000,0x01010001,0x0f05000d,0x0f06003d,0x0f060027,0x0f05000e,0x0f040010,0x07040006,
0x00000000,0x0f070017,0x0f05000e,0x07030008,0x0f050066,0x07030007,0x0f040011,0x0f08000f,
0x00000000,0x01010001,0x0f04000e,0x02020002,0x0801000b,0x07040005,0x07030006,0x0f070022,
0x00000000,0x0f060036,0x0f04000f,0x07040006,0x0f03001b,0x07030008,0x07030007,0x0f070010,
0x00000000,0x01010001,0x07040005,0x0f05007c,0x0f060028,0x07040006,0x07030008,0x08040008,
0x00000000,0x0f050075,0x07040006,0x07030009,0x0f050067,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0801000c,0x0f050014,0x0f040015,0x03030003,
0x0f0200fa,0x0f040013,0x04010003,0x0f09000b,0x0f03001c,0x0f08000c,0x0f040016,0x04040004,
0x00000000,0x01010001,0x0f050014,0x0f080010,0x0f060029,0x0f050015,0x0f040017,0x08050006,
0x0f0101f9,0x0f070018,0x0f050015,0x08040007,0x0f050068,0x0f08000d,0x0f040018,0x0f080016,
0x00000000,0x01010001,0x08020005,0x02020002,0x0801000d,0x0f09000b,0x0f08000c,0x0f060041,
0x100102f9,0x0f060037,0x08020006,0x0f09000c,0x0f03001d,0x0f08000e,0x0f08000d,0x08050007,
0x00000000,0x01010001,0x0f09000b,0x0f07001f,0x0f06002a,0x0f09000c,0x0f08000e,0x08040009,
0x00000000,0x0f050076,0x0f09000c,0x08040008,0x0f050069,0x0f08000f,0x0f08000f,0x0f0c000d,
0x00000000,0x01010001,0x0f040022,0x02020002,0x0801000e,0x0f08000c,0x0f07000d,0x03030003,
0x00000000,0x0f040014,0x0f040023,0x08050006,0x0f03001e,0x08040007,0x0f07000e,0x0f070024,
0x00000000,0x01010001,0x0f08000c,0x0f06003e,0x0f06002b,0x0f08000d,0x0f07000f,0x08050007,
0x00000000,0x0f070019,0x0f08000d,0x08040009,0x0f05006a,0x08040008,0x0f070010,0x0f0b000e,
0x00000000,0x01010001,0x0f04000f,0x02020002,0x0801000f,0x08050006,0x08040007,0x0f050080,
0x00000000,0x0f060038,0x0f040010,0x08050007,0x0f03001f,0x08040009,0x08040008,0x0f070011,
0x00000000,0x01010001,0x08050006,0x0f05007d,0x0f06002c,0x08050007,0x08040009,0x0804000a,
0x00000000,0x0f050077,0x08050007,0x0804000a,0x0f05006b,0x0804000a,0x0804000a,0x08080008,
0x00000000,0x01010001,0x04010002,0x02020002,0x0f040011,0x05020003,0x05010004,0x03030003,
0x0f0200fb,0x0f040015,0x04010003,0x10040033,0x0f030020,0x10030034,0x05010005,0x04040004,
0x00000000,0x0f0200fe,0x05020003,0x0f080011,0x0f06002d,0x05020004,0x05010006,0x0f09000c,
0x0f0101fa,0x0f07001a,0x05020004,0x0f08000d,0x0f05006c,0x10030035,0x05010007,0x05050005,
0x00000000,0x01010001,0x0f05000d,0x02020002,0x0f040012,0x10040033,0x10030034,0x0f080014,
0x100102fa,0x0f060039,0x0f05000e,0x10040034,0x0f030021,0x10030036,0x10030035,0x0f08000f,
0x00000000,0x1002017e,0x10040033,0x0f070020,0x0f06002e,0x10040034,0x10030036,0x0f070010,
0x00000000,0x0f050078,0x10040034,0x0f08000e,0x0f05006d,0x10030037,0x10030037,0x10070035,
0x00000000,0x01010001,0x0f030041,0x02020002,0x0f040013,0x0f05000e,0x0f04000f,0x03030003,
0x00000000,0x0f040016,0x0f030042,0x0f09000c,0x0f030022,0x0f08000d,0x0f040010,0x0f060043,
0x00000000,0x01010001,0x0f05000e,0x0f06003f,0x0f06002f,0x0f05000f,0x0f040011,0x0f09000d,
0x00000000,0x0f07001b,0x0f05000f,0x0f08000f,0x0f05006e,0x0f08000e,0x0f040012,0x0f080010,
0x00000000,0x01010001,0x0f040010,0x02020002,0x0f040014,0x0f09000c,0x0f08000d,0x0f070023,
0x00000000,0x0f06003a,0x0f040011,0x0f09000d,0x0f030023,0x0f08000f,0x0f08000e,0x0f070012,
0x00000000,0x01010001,0x0f09000c,0x0f05007e,0x0f060030,0x0f09000d,0x0f08000f,0x0f070011,
0x00000000,0x0f050079,0x0f09000d,0x0f080010,0x0f05006f,0x0f080010,0x0f080010,0x0f0c000e,
0x00000000,0x01010001,0x04010002,0x02020002,0x0f040015,0x0f040023,0x0f030024,0x03030003,
0x0f0200fc,0x0f040017,0x04010003,0x0f08000e,0x0f030024,0x0f07000f,0x0f030025,0x04040004,
0x00000000,0x01010001,0x0f040023,0x0f080012,0x0f060031,0x0f040024,0x0f030026,0x0f08000f,
0x0f0101fb,0x0f07001c,0x0f040024,0x0f070010,0x0f050070,0x0f070010,0x0f030027,0x0f070025,
0x00000000,0x01010001,0x0f05000e,0x02020002,0x0f040016,0x0f08000e,0x0f07000f,0x0f060042,
0x100102fb,0x0f06003b,0x0f05000f,0x0f08000f,0x0f030025,0x0f070011,0x0f070010,0x0f080010,
0x00000000,0x01010001,0x0f08000e,0x0f070021,0x0f060032,0x0f08000f,0x0f070011,0x0f070012,
0x00000000,0x0f05007a,0x0f08000f,0x0f070011,0x0f050071,0x0f070012,0x0f070012,0x0f0b0010,
0x00000000,0x01010001,0x0f020080,0x02020002,0x0f040017,0x0f08000d,0x0f07000e,0x03030003,
0x00000000,0x0f040018,0x0f020081,0x0f08000f,0x0f030026,0x0f070010,0x0f07000f,0x0f050082,
0x00000000,0x01010001,0x0f08000d,0x0f060040,0x0f060033,0x0f08000e,0x0f070010,0x0f080010,
0x00000000,0x0f07001d,0x0f08000e,0x0f070012,0x0f050072,0x0f070011,0x0f070011,0x0f0b000f,
0x00000000,0x01010001,0x0f040011,0x02020002,0x0f040018,0x0f08000f,0x0f070010,0x0f050081,
0x00000000,0x0f06003c,0x0f040012,0x0f080010,0x0f030027,0x0f070012,0x0f070011,0x0f070013,
0x00000000,0x01010001,0x0f08000f,0x0f05007f,0x0f060034,0x0f080010,0x0f070012,0x0f070013,
0x00000000,0x0f05007b,0x0f080010,0x0f070013,0x0f050073,0x0f070013,0x0f070013,0x0f0b0011,
0x00000000,0x01010001,0x04010002,0x02020002,0x0f040011,0x05020003,0x05010004,0x03030003,
0x0f0200fd,0x0f08000f,0x04010003,0x06030004,0x0f070016,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f0200ff,0x05020003,0x0f080013,0x0f060035,0x05020004,0x05010006,0x07040005,
0x0f0101fc,0x0f07001e,0x05020004,0x07030006,0x0f050074,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0f05000d,0x100300c1,0x0f040012,0x06030004,0x06020005,0x0f080015,
0x100102fc,0x0f06003d,0x0f05000e,0x06030005,0x0f070017,0x06020007,0x06020006,0x0f08000f,
0x00000000,0x1002017f,0x06030004,0x0f070022,0x0f060036,0x06030005,0x06020007,0x0f070010,
0x00000000,0x0f05007c,0x06030005,0x07030007,0x0f050075,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0f050014,0x02020002,0x0f040013,0x0f09000b,0x0f08000c,0x03030003,
0x00000000,0x0f080010,0x0f050015,0x07040005,0x0f070018,0x07030006,0x0f08000d,0x0f080016,
0x00000000,0x01010001,0x0f09000b,0x0f060041,0x0f060037,0x0f09000c,0x0f08000e,0x07040006,
0x00000000,0x0f07001f,0x0f09000c,0x07030008,0x0f050076,0x07030007,0x0f08000f,0x0f0c000d,
0x00000000,0x01010001,0x0f08000c,0x02020002,0x0f040014,0x07040005,0x07030006,0x0f070024,
0x00000000,0x0f06003e,0x0f08000d,0x07040006,0x0f070019,0x07030008,0x07030007,0x0f0b000e,
0x00000000,0x01010001,0x07040005,0x0f050080,0x0f060038,0x07040006,0x07030008,0x0f070011,
0x00000000,0x0f05007d,0x07040006,0x07030009,0x0f050077,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0f040015,0x0f030042,0x0f020043,0x03030003,
0x0f0200fe,0x0f080011,0x04010003,0x0f09000c,0x0f07001a,0x0f08000d,0x0f020044,0x04040004,
0x00000000,0x01010001,0x0f030042,0x0f080014,0x0f060039,0x0f030043,0x0f020045,0x0f08000f,
0x0f0101fd,0x0f070020,0x0f030043,0x0f070010,0x0f050078,0x0f08000e,0x0f020046,0x0f060044,
0x00000000,0x01010001,0x0f05000e,0x02020002,0x0f040016,0x0f09000c,0x0f08000d,0x0f060043,
0x100102fd,0x0f06003f,0x0f05000f,0x0f09000d,0x0f07001b,0x0f08000f,0x0f08000e,0x0f080010,
0x00000000,0x01010001,0x0f09000c,0x0f070023,0x0f06003a,0x0f09000d,0x0f08000f,0x0f070012,
0x00000000,0x0f05007e,0x0f09000d,0x0f070011,0x0f050079,0x0f080010,0x0f080010,0x0f0c000e,
0x00000000,0x01010001,0x0f040023,0x02020002,0x0f040017,0x0f08000e,0x0f07000f,0x03030003,
0x00000000,0x0f080012,0x0f040024,0x0f08000f,0x0f07001c,0x0f070010,0x0f070010,0x0f070025,
0x00000000,0x01010001,0x0f08000e,0x0f060042,0x0f06003b,0x0f08000f,0x0f070011,0x0f080010,
0x00000000,0x0f070021,0x0f08000f,0x0f070012,0x0f05007a,0x0f070011,0x0f070012,0x0f0b0010,
0x00000000,0x01010001,0x0f08000d,0x02020002,0x0f040018,0x0f08000f,0x0f070010,0x0f050082,
0x00000000,0x0f060040,0x0f08000e,0x0f080010,0x0f07001d,0x0f070012,0x0f070011,0x0f0b000f,
0x00000000,0x01010001,0x0f08000f,0x0f050081,0x0f06003c,0x0f080010,0x0f070012,0x0f070013,
0x00000000,0x0f05007f,0x0f080010,0x0f070013,0x0f05007b,0x0f070013,0x0f070013,0x0f0b0011,
0x00000000,0x01010001,0x04010002,0x02020002,0x0f08000f,0x05020003,0x05010004,0x03030003,
0x0f0200ff,0x0f080013,0x04010003,0x06030004,0x0f07001e,0x06020005,0x05010005,0x04040004,
0x00000000,0x0f020100,0x05020003,0x0f080015,0x0f06003d,0x05020004,0x05010006,0x0f08000f,
0x0f0101fe,0x0f070022,0x05020004,0x0f070010,0x0f05007c,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0f09000b,0x02020002,0x0f080010,0x06030004,0x06020005,0x0f080016,
0x100102fe,0x0f060041,0x0f09000c,0x06030005,0x0f07001f,0x06020007,0x06020006,0x0f0c000d,
0x00000000,0x10020180,0x06030004,0x0f070024,0x0f06003e,0x06030005,0x06020007,0x0f0b000e,
0x00000000,0x0f050080,0x06030005,0x0f070011,0x0f05007d,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0f030042,0x02020002,0x0f080011,0x0f09000c,0x0f08000d,0x03030003,
0x00000000,0x0f080014,0x0f030043,0x0f08000f,0x0f070020,0x0f070010,0x0f08000e,0x0f060044,
0x00000000,0x01010001,0x0f09000c,0x0f060043,0x0f06003f,0x0f09000d,0x0f08000f,0x0f080010,
0x00000000,0x0f070023,0x0f09000d,0x0f070012,0x0f05007e,0x0f070011,0x0f080010,0x0f0c000e,
0x00000000,0x01010001,0x0f08000e,0x02020002,0x0f080012,0x0f08000f,0x0f070010,0x0f070025,
0x00000000,0x0f060042,0x0f08000f,0x0f080010,0x0f070021,0x0f070012,0x0f070011,0x0f0b0010,
0x00000000,0x01010001,0x0f08000f,0x0f050082,0x0f060040,0x0f080010,0x0f070012,0x0f0b000f,
0x00000000,0x0f050081,0x0f080010,0x0f070013,0x0f05007f,0x0f070013,0x0f070013,0x0f0b0011,
0x00000000,0x01010001,0x04010002,0x02020002,0x0f080013,0x10030062,0x10020063,0x03030003,
0x0f020100,0x0f080015,0x04010003,0x0f08000f,0x0f070022,0x0f070010,0x10020064,0x04040004,
0x00000000,0x01010001,0x10030062,0x0f080016,0x0f060041,0x10030063,0x10020065,0x0f0c000d,
0x0f0101ff,0x0f070024,0x10030063,0x0f0b000e,0x0f050080,0x0f070011,0x10020066,0x10060064,
0x00000000,0x01010001,0x0f09000c,0x02020002,0x0f080014,0x0f08000f,0x0f070010,0x0f060044,
0x100102ff,0x0f060043,0x0f09000d,0x0f080010,0x0f070023,0x0f070012,0x0f070011,0x0f0c000e,
0x00000000,0x01010001,0x0f08000f,0x0f070025,0x0f060042,0x0f080010,0x0f070012,0x0f0b0010,
0x00000000,0x0f050082,0x0f080010,0x0f0b000f,0x0f050081,0x0f070013,0x0f070013,0x0f0b0011,
0x00000000,0x01010001,0x0f020081,0x02020002,0x0f080015,0x0f08000f,0x0f070010,0x03030003,
0x00000000,0x0f080016,0x0f020082,0x0f0c000d,0x0f070024,0x0f0b000e,0x0f070011,0x0f050083,
0x00000000,0x01010001,0x0f08000f,0x0f060044,0x0f060043,0x0f080010,0x0f070012,0x0f0c000e,
0x00000000,0x0f070025,0x0f080010,0x0f0b0010,0x0f050082,0x0f0b000f,0x0f070013,0x0f0b0011,
0x00000000,0x01010001,0x0f08000f,0x02020002,0x0f080016,0x0f0c000d,0x0f0b000e,0x0f050083,
0x00000000,0x0f060044,0x0f080010,0x0f0c000e,0x0f070025,0x0f0b0010,0x0f0b000f,0x0f0b0011,
0x00000000,0x01010001,0x0f0c000d,0x0f050083,0x0f060044,0x0f0c000e,0x0f0b0010,0x0f0b0011,
0x00000000,0x0f050083,0x0f0c000e,0x0f0b0011,0x0f050083,0x0f0b0011,0x0f0b0011,0x0f0f000f,
0x00000000,0x01010001,0x04010002,0x02020002,0x08010008,0x05020003,0x05010004,0x03030003,
0x0e010080,0x09020009,0x04010003,0x06030004,0x09010010,0x06020005,0x05010005,0x04040004,
0x00000000,0x10030082,0x05020003,0x0a03000a,0x0a010020,0x05020004,0x05010006,0x07040005,
0x10020101,0x0a020011,0x05020004,0x07030006,0x0b010040,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x08020004,0x100300c2,0x08010009,0x06030004,0x06020005,0x0b04000b,
0x10010300,0x0b020021,0x08020005,0x06030005,0x09010011,0x06020007,0x06020006,0x08050006,
0x00000000,0x10020181,0x06030004,0x0b030012,0x0a010021,0x06030005,0x06020007,0x08040007,
0x00000000,0x0c020041,0x06030005,0x07030007,0x0b010041,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0c02000a,0x02020002,0x0801000a,0x09030005,0x09020006,0x03030003,
0x00000000,0x0902000a,0x0c02000b,0x07040005,0x09010012,0x07030006,0x09020007,0x0c05000c,
0x00000000,0x01010001,0x09030005,0x0c030022,0x0a010022,0x09030006,0x09020008,0x07040006,
0x00000000,0x0a020012,0x09030006,0x07030008,0x0b010042,0x07030007,0x09020009,0x09060007,
0x00000000,0x01010001,0x09020006,0x02020002,0x0801000b,0x07040005,0x07030006,0x0c040013,
0x00000000,0x0b020022,0x09020007,0x07040006,0x09010013,0x07030008,0x07030007,0x09050008,
0x00000000,0x01010001,0x07040005,0x0d030042,0x0a010023,0x07040006,0x07030008,0x08040008,
0x00000000,0x0c020042,0x07040006,0x07030009,0x0b010043,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0801000c,0x0d03000b,0x0d02000c,0x03030003,
0x0e010081,0x0902000b,0x04010003,0x0a040006,0x09010014,0x0a030007,0x0d02000d,0x04040004,
0x00000000,0x01010001,0x0d03000b,0x0a03000b,0x0a010024,0x0d03000c,0x0d02000e,0x08050006,
0x10020102,0x0a020013,0x0d03000c,0x08040007,0x0b010044,0x0a030008,0x0d02000f,0x0d06000d,
0x00000000,0x01010001,0x08020005,0x02020002,0x0801000d,0x0a040006,0x0a030007,0x0d040023,
0x10010301,0x0b020023,0x08020006,0x0a040007,0x09010015,0x0a030009,0x0a030008,0x08050007,
0x00000000,0x01010001,0x0a040006,0x0b030013,0x0a010025,0x0a040007,0x0a030009,0x08040009,
0x00000000,0x0c020043,0x0a040007,0x08040008,0x0b010045,0x0a03000a,0x0a03000a,0x0a070008,
0x00000000,0x01010001,0x0d020012,0x02020002,0x0801000e,0x0a030007,0x0a020008,0x03030003,
0x00000000,0x0902000c,0x0d020013,0x08050006,0x09010016,0x08040007,0x0a020009,0x0d050014,
0x00000000,0x01010001,0x0a030007,0x0c030023,0x0a010026,0x0a030008,0x0a02000a,0x08050007,
0x00000000,0x0a020014,0x0a030008,0x08040009,0x0b010046,0x08040008,0x0a02000b,0x0a060009,
0x00000000,0x01010001,0x09020007,0x02020002,0x0801000f,0x08050006,0x08040007,0x0e040043,
0x00000000,0x0b020024,0x09020008,0x08050007,0x09010017,0x08040009,0x08040008,0x09050009,
0x00000000,0x01010001,0x08050006,0x0d030043,0x0a010027,0x08050007,0x08040009,0x0804000a,
0x00000000,0x0c020044,0x08050007,0x0804000a,0x0b010047,0x0804000a,0x0804000a,0x08080008,
0x00000000,0x01010001,0x04010002,0x02020002,0x09020009,0x05020003,0x05010004,0x03030003,
0x0e010082,0x0902000d,0x04010003,0x0e04000c,0x09010018,0x0e03000d,0x05010005,0x04040004,
0x00000000,0x10030083,0x05020003,0x0a03000c,0x0a010028,0x05020004,0x05010006,0x0b050007,
0x10020103,0x0a020015,0x05020004,0x0b040008,0x0b010048,0x0e03000e,0x05010007,0x05050005,
0x00000000,0x01010001,0x09030005,0x02020002,0x0902000a,0x0e04000c,0x0e03000d,0x0b04000c,
0x10010302,0x0b020025,0x09030006,0x0e04000d,0x09010019,0x0e03000f,0x0e03000e,0x09060007,
0x00000000,0x10020182,0x0e04000c,0x0b030014,0x0a010029,0x0e04000d,0x0e03000f,0x09050008,
0x00000000,0x0c020045,0x0e04000d,0x0b040009,0x0b010049,0x0e030010,0x0e030010,0x0e07000e,
0x00000000,0x01010001,0x0e020022,0x02020002,0x0902000b,0x09030006,0x09020007,0x03030003,
0x00000000,0x0902000e,0x0e020023,0x0b050007,0x0901001a,0x0b040008,0x09020008,0x0e050024,
0x00000000,0x01010001,0x09030006,0x0c030024,0x0a01002a,0x09030007,0x09020009,0x0b050008,
0x00000000,0x0a020016,0x09030007,0x0b04000a,0x0b01004a,0x0b040009,0x0902000a,0x09060008,
0x00000000,0x01010001,0x09020008,0x02020002,0x0902000c,0x0b050007,0x0b040008,0x0c040014,
0x00000000,0x0b020026,0x09020009,0x0b050008,0x0901001b,0x0b04000a,0x0b040009,0x0905000a,
0x00000000,0x01010001,0x0b050007,0x0d030044,0x0a01002b,0x0b050008,0x0b04000a,0x09050009,
0x00000000,0x0c020046,0x0b050008,0x0b04000b,0x0b01004b,0x0b04000b,0x0b04000b,0x0b080009,
0x00000000,0x01010001,0x04010002,0x02020002,0x0902000d,0x0e030013,0x0e020014,0x03030003,
0x0e010083,0x0902000f,0x04010003,0x0b040008,0x0901001c,0x0b030009,0x0e020015,0x04040004,
0x00000000,0x01010001,0x0e030013,0x0a03000d,0x0a01002c,0x0e030014,0x0e020016,0x09060007,
0x10020104,0x0a020017,0x0e030014,0x09050008,0x0b01004c,0x0b03000a,0x0e020017,0x0e060015,
0x00000000,0x01010001,0x09030006,0x02020002,0x0902000e,0x0b040008,0x0b030009,0x0d040024,
0x10010303,0x0b020027,0x09030007,0x0b040009,0x0901001d,0x0b03000b,0x0b03000a,0x09060008,
0x00000000,0x01010001,0x0b040008,0x0b030015,0x0a01002d,0x0b040009,0x0b03000b,0x0905000a,
0x00000000,0x0c020047,0x0b040009,0x09050009,0x0b01004d,0x0b03000c,0x0b03000c,0x0b07000a,
0x00000000,0x01010001,0x10030043,0x02020002,0x0902000f,0x0a030008,0x0a020009,0x03030003,
0x00000000,0x09020010,0x10030044,0x09060007,0x0901001e,0x09050008,0x0a02000a,0x10060045,
0x00000000,0x01010001,0x0a030008,0x0c030025,0x0a01002e,0x0a030009,0x0a02000b,0x09060008,
0x00000000,0x0a020018,0x0a030009,0x0905000a,0x0b01004e,0x09050009,0x0a02000c,0x0a06000a,
0x00000000,0x01010001,0x09020009,0x02020002,0x09020010,0x09060007,0x09050008,0x0e040044,
0x00000000,0x0b020028,0x0902000a,0x09060008,0x0901001f,0x0905000a,0x09050009,0x0905000b,
0x00000000,0x01010001,0x09060007,0x0d030045,0x0a01002f,0x09060008,0x0905000a,0x0905000b,
0x00000000,0x0c020048,0x09060008,0x0905000b,0x0b01004f,0x0905000b,0x0905000b,0x09090009,
0x00000000,0x01010001,0x04010002,0x02020002,0x0c02000a,0x05020003,0x05010004,0x03030003,
0x0e010084,0x0a03000a,0x04010003,0x06030004,0x0a020011,0x06020005,0x05010005,0x04040004,
0x00000000,0x10030084,0x05020003,0x0a03000e,0x0a010030,0x05020004,0x05010006,0x1006000e,
0x10020105,0x0a020019,0x05020004,0x1005000f,0x0b010050,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0c030006,0x100300c3,0x0c02000b,0x06030004,0x06020005,0x0b04000d,
0x10010304,0x0b020029,0x0c030007,0x06030005,0x0a020012,0x06020007,0x06020006,0x0c060008,
0x00000000,0x10020183,0x06030004,0x0b030016,0x0a010031,0x06030005,0x06020007,0x0c050009,
0x00000000,0x0c020049,0x06030005,0x10050010,0x0b010051,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0c02000b,0x02020002,0x0c02000c,0x0a040006,0x0a030007,0x03030003,
0x00000000,0x0a03000b,0x0c02000c,0x1006000e,0x0a020013,0x1005000f,0x0a030008,0x0c05000d,
0x00000000,0x01010001,0x0a040006,0x0c030026,0x0a010032,0x0a040007,0x0a030009,0x1006000f,
0x00000000,0x0a02001a,0x0a040007,0x10050011,0x0b010052,0x10050010,0x0a03000a,0x0a070008,
0x00000000,0x01010001,0x0a030007,0x02020002,0x0c02000d,0x1006000e,0x1005000f,0x0c040015,
0x00000000,0x0b02002a,0x0a030008,0x1006000f,0x0a020014,0x10050011,0x10050010,0x0a060009,
0x00000000,0x01010001,0x1006000e,0x0d030046,0x0a010033,0x1006000f,0x10050011,0x0c05000a,
0x00000000,0x0c02004a,0x1006000f,0x10050012,0x0b010053,0x10050012,0x10050012,0x10090010,
0x00000000,0x01010001,0x04010002,0x02020002,0x0c02000e,0x10040024,0x10030025,0x03030003,
0x0e010085,0x0a03000c,0x04010003,0x0a040007,0x0a020015,0x0a030008,0x10030026,0x04040004,
0x00000000,0x01010001,0x10040024,0x0a03000f,0x0a010034,0x10040025,0x10030027,0x0c060008,
0x10020106,0x0a02001b,0x10040025,0x0c050009,0x0b010054,0x0a030009,0x10030028,0x10070026,
0x00000000,0x01010001,0x0c030007,0x02020002,0x0c02000f,0x0a040007,0x0a030008,0x0d040025,
0x10010305,0x0b02002b,0x0c030008,0x0a040008,0x0a020016,0x0a03000a,0x0a030009,0x0c060009,
0x00000000,0x01010001,0x0a040007,0x0b030017,0x0a010035,0x0a040008,0x0a03000a,0x0c05000b,
0x00000000,0x0c02004b,0x0a040008,0x0c05000a,0x0b010055,0x0a03000b,0x0a03000b,0x0a070009,
0x00000000,0x01010001,0x0d020013,0x02020002,0x0c020010,0x0a030009,0x0a02000a,0x03030003,
0x00000000,0x0a03000d,0x0d020014,0x0c060008,0x0a020017,0x0c050009,0x0a02000b,0x0d050015,
0x00000000,0x01010001,0x0a030009,0x0c030027,0x0a010036,0x0a03000a,0x0a02000c,0x0c060009,
0x00000000,0x0a02001c,0x0a03000a,0x0c05000b,0x0b010056,0x0c05000a,0x0a02000d,0x0a06000b,
0x00000000,0x01010001,0x0a030008,0x02020002,0x0c020011,0x0c060008,0x0c050009,0x0e040045,
0x00000000,0x0b02002c,0x0a030009,0x0c060009,0x0a020018,0x0c05000b,0x0c05000a,0x0a06000a,
0x00000000,0x01010001,0x0c060008,0x0d030047,0x0a010037,0x0c060009,0x0c05000b,0x0c05000c,
0x00000000,0x0c02004c,0x0c060009,0x0c05000c,0x0b010057,0x0c05000c,0x0c05000c,0x0c09000a,
0x00000000,0x01010001,0x04010002,0x02020002,0x0a03000a,0x05020003,0x05010004,0x03030003,
0x0e010086,0x0a03000e,0x04010003,0x10050015,0x0a020019,0x10040016,0x05010005,0x04040004,
0x00000000,0x10030085,0x05020003,0x0a030010,0x0a010038,0x05020004,0x05010006,0x0c050009,
0x10020107,0x0a02001d,0x05020004,0x0c04000a,0x0b010058,0x10040017,0x05010007,0x05050005,
0x00000000,0x01010001,0x0a040006,0x02020002,0x0a03000b,0x10050015,0x10040016,0x0b04000e,
0x10010306,0x0b02002d,0x0a040007,0x10050016,0x0a02001a,0x10040018,0x10040017,0x0a070008,
0x00000000,0x10020184,0x10050015,0x0b030018,0x0a010039,0x10050016,0x10040018,0x0a060009,
0x00000000,0x0c02004d,0x10050016,0x0c04000b,0x0b010059,0x10040019,0x10040019,0x10080017,
0x00000000,0x01010001,0x0e020023,0x02020002,0x0a03000c,0x0a040007,0x0a030008,0x03030003,
0x00000000,0x0a03000f,0x0e020024,0x0c050009,0x0a02001b,0x0c04000a,0x0a030009,0x0e050025,
0x00000000,0x01010001,0x0a040007,0x0c030028,0x0a01003a,0x0a040008,0x0a03000a,0x0c05000a,
0x00000000,0x0a02001e,0x0a040008,0x0c04000c,0x0b01005a,0x0c04000b,0x0a03000b,0x0a070009,
0x00000000,0x01010001,0x0a030009,0x02020002,0x0a03000d,0x0c050009,0x0c04000a,0x0c040016,
0x00000000,0x0b02002e,0x0a03000a,0x0c05000a,0x0a02001c,0x0c04000c,0x0c04000b,0x0a06000b,
0x00000000,0x01010001,0x0c050009,0x0d030048,0x0a01003b,0x0c05000a,0x0c04000c,0x0a06000a,
0x00000000,0x0c02004e,0x0c05000a,0x0c04000d,0x0b01005b,0x0c04000d,0x0c04000d,0x0c08000b,
0x00000000,0x01010001,0x04010002,0x02020002,0x0a03000e,0x10030063,0x10020064,0x03030003,
0x0e010087,0x0a030010,0x04010003,0x0b040009,0x0a02001d,0x0b03000a,0x10020065,0x04040004,
0x00000000,0x01010001,0x10030063,0x0a030011,0x0a01003c,0x10030064,0x10020066,0x0a070008,
0x10020108,0x0a02001f,0x10030064,0x0a060009,0x0b01005c,0x0b03000b,0x10020067,0x10060065,
0x00000000,0x01010001,0x0a040007,0x02020002,0x0a03000f,0x0b040009,0x0b03000a,0x0d040026,
0x10010307,0x0b02002f,0x0a040008,0x0b04000a,0x0a02001e,0x0b03000c,0x0b03000b,0x0a070009,
0x00000000,0x01010001,0x0b040009,0x0b030019,0x0a01003d,0x0b04000a,0x0b03000c,0x0a06000b,
0x00000000,0x0c02004f,0x0b04000a,0x0a06000a,0x0b01005d,0x0b03000d,0x0b03000d,0x0b07000b,
0x00000000,0x01010001,0x10030044,0x02020002,0x0a030010,0x0a03000a,0x0a02000b,0x03030003,
0x00000000,0x0a030011,0x10030045,0x0a070008,0x0a02001f,0x0a060009,0x0a02000c,0x10060046,
0x00000000,0x01010001,0x0a03000a,0x0c030029,0x0a01003e,0x0a03000b,0x0a02000d,0x0a070009,
0x00000000,0x0a020020,0x0a03000b,0x0a06000b,0x0b01005e,0x0a06000a,0x0a02000e,0x0a06000c,
0x00000000,0x01010001,0x0a03000a,0x02020002,0x0a030011,0x0a070008,0x0a060009,0x0e040046,
0x00000000,0x0b020030,0x0a03000b,0x0a070009,0x0a020020,0x0a06000b,0x0a06000a,0x0a06000c,
0x00000000,0x01010001,0x0a070008,0x0d030049,0x0a01003f,0x0a070009,0x0a06000b,0x0a06000c,
0x00000000,0x0c020050,0x0a070009,0x0a06000c,0x0b01005f,0x0a06000c,0x0a06000c,0x0a0a000a,
0x00000000,0x01010001,0x04010002,0x02020002,0x10020014,0x05020003,0x05010004,0x03030003,
0x0e010088,0x0d03000b,0x04010003,0x06030004,0x0d020012,0x06020005,0x05010005,0x04040004,
0x00000000,0x10030086,0x05020003,0x0b04000b,0x0b020021,0x05020004,0x05010006,0x07040005,
0x10020109,0x0b030012,0x05020004,0x07030006,0x0b010060,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x10030010,0x100300c4,0x10020015,0x06030004,0x06020005,0x0b04000f,
0x10010308,0x0b020031,0x10030011,0x06030005,0x0d020013,0x06020007,0x06020006,0x10060012,
0x00000000,0x10020185,0x06030004,0x0b03001a,0x0b020022,0x06030005,0x06020007,0x10050013,
0x00000000,0x0c020051,0x06030005,0x07030007,0x0b010061,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0c02000c,0x02020002,0x10020016,0x0d040007,0x0d030008,0x03030003,
0x00000000,0x0d03000c,0x0c02000d,0x07040005,0x0d020014,0x07030006,0x0d030009,0x0c05000e,
0x00000000,0x01010001,0x0d040007,0x0c03002a,0x0b020023,0x0d040008,0x0d03000a,0x07040006,
0x00000000,0x0b030013,0x0d040008,0x07030008,0x0b010062,0x07030007,0x0d03000b,0x0d070009,
0x00000000,0x01010001,0x0d030008,0x02020002,0x10020017,0x07040005,0x07030006,0x0c040017,
0x00000000,0x0b020032,0x0d030009,0x07040006,0x0d020015,0x07030008,0x07030007,0x0d06000a,
0x00000000,0x01010001,0x07040005,0x0d03004a,0x0b020024,0x07040006,0x07030008,0x10050014,
0x00000000,0x0c020052,0x07040006,0x07030009,0x0b010063,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x10020018,0x0d03000c,0x0d02000d,0x03030003,
0x0e010089,0x0d03000d,0x04010003,0x0b050007,0x0d020016,0x0b040008,0x0d02000e,0x04040004,
0x00000000,0x01010001,0x0d03000c,0x0b04000c,0x0b020025,0x0d03000d,0x0d02000f,0x10060012,
0x1002010a,0x0b030014,0x0d03000d,0x10050013,0x0b010064,0x0b040009,0x0d020010,0x0d06000e,
0x00000000,0x01010001,0x10030011,0x02020002,0x10020019,0x0b050007,0x0b040008,0x0d040027,
0x10010309,0x0b020033,0x10030012,0x0b050008,0x0d020017,0x0b04000a,0x0b040009,0x10060013,
0x00000000,0x01010001,0x0b050007,0x0b03001b,0x0b020026,0x0b050008,0x0b04000a,0x10050015,
0x00000000,0x0c020053,0x0b050008,0x10050014,0x0b010065,0x0b04000b,0x0b04000b,0x0b080009,
0x00000000,0x01010001,0x0d020014,0x02020002,0x1002001a,0x0b040008,0x0b030009,0x03030003,
0x00000000,0x0d03000e,0x0d020015,0x10060012,0x0d020018,0x10050013,0x0b03000a,0x0d050016,
0x00000000,0x01010001,0x0b040008,0x0c03002b,0x0b020027,0x0b040009,0x0b03000b,0x10060013,
0x00000000,0x0b030015,0x0b040009,0x10050015,0x0b010066,0x10050014,0x0b03000c,0x0b07000a,
0x00000000,0x01010001,0x0d030009,0x02020002,0x1002001b,0x10060012,0x10050013,0x0e040047,
0x00000000,0x0b020034,0x0d03000a,0x10060013,0x0d020019,0x10050015,0x10050014,0x0d06000b,
0x00000000,0x01010001,0x10060012,0x0d03004b,0x0b020028,0x10060013,0x10050015,0x10050016,
0x00000000,0x0c020054,0x10060013,0x10050016,0x0b010067,0x10050016,0x10050016,0x10090014,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d03000b,0x05020003,0x05010004,0x03030003,
0x0e01008a,0x0d03000f,0x04010003,0x10040034,0x0d02001a,0x10030035,0x05010005,0x04040004,
0x00000000,0x10030087,0x05020003,0x0b04000d,0x0b020029,0x05020004,0x05010006,0x0b050008,
0x1002010b,0x0b030016,0x05020004,0x0b040009,0x0b010068,0x10030036,0x05010007,0x05050005,
0x00000000,0x01010001,0x0d040007,0x02020002,0x0d03000c,0x10040034,0x10030035,0x0b040010,
0x1001030a,0x0b020035,0x0d040008,0x10040035,0x0d02001b,0x10030037,0x10030036,0x0d070009,
0x00000000,0x10020186,0x10040034,0x0b03001c,0x0b02002a,0x10040035,0x10030037,0x0d06000a,
0x00000000,0x0c020055,0x10040035,0x0b04000a,0x0b010069,0x10030038,0x10030038,0x10070036,
0x00000000,0x01010001,0x0e020024,0x02020002,0x0d03000d,0x0d040008,0x0d030009,0x03030003,
0x00000000,0x0d030010,0x0e020025,0x0b050008,0x0d02001c,0x0b040009,0x0d03000a,0x0e050026,
0x00000000,0x01010001,0x0d040008,0x0c03002c,0x0b02002b,0x0d040009,0x0d03000b,0x0b050009,
0x00000000,0x0b030017,0x0d040009,0x0b04000b,0x0b01006a,0x0b04000a,0x0d03000c,0x0d07000a,
0x00000000,0x01010001,0x0d03000a,0x02020002,0x0d03000e,0x0b050008,0x0b040009,0x0c040018,
0x00000000,0x0b020036,0x0d03000b,0x0b050009,0x0d02001d,0x0b04000b,0x0b04000a,0x0d06000c,
0x00000000,0x01010001,0x0b050008,0x0d03004c,0x0b02002c,0x0b050009,0x0b04000b,0x0d06000b,
0x00000000,0x0c020056,0x0b050009,0x0b04000c,0x0b01006b,0x0b04000c,0x0b04000c,0x0b08000a,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d03000f,0x0e030014,0x0e020015,0x03030003,
0x0e01008b,0x0d030011,0x04010003,0x0b04000a,0x0d02001e,0x0b03000b,0x0e020016,0x04040004,
0x00000000,0x01010001,0x0e030014,0x0b04000e,0x0b02002d,0x0e030015,0x0e020017,0x0d070009,
0x1002010c,0x0b030018,0x0e030015,0x0d06000a,0x0b01006c,0x0b03000c,0x0e020018,0x0e060016,
0x00000000,0x01010001,0x0d040008,0x02020002,0x0d030010,0x0b04000a,0x0b03000b,0x0d040028,
0x1001030b,0x0b020037,0x0d040009,0x0b04000b,0x0d02001f,0x0b03000d,0x0b03000c,0x0d07000a,
0x00000000,0x01010001,0x0b04000a,0x0b03001d,0x0b02002e,0x0b04000b,0x0b03000d,0x0d06000c,
0x00000000,0x0c020057,0x0b04000b,0x0d06000b,0x0b01006d,0x0b03000e,0x0b03000e,0x0b07000c,
0x00000000,0x01010001,0x10030045,0x02020002,0x0d030011,0x0b040009,0x0b03000a,0x03030003,
0x00000000,0x0d030012,0x10030046,0x0d070009,0x0d020020,0x0d06000a,0x0b03000b,0x10060047,
0x00000000,0x01010001,0x0b040009,0x0c03002d,0x0b02002f,0x0b04000a,0x0b03000c,0x0d07000a,
0x00000000,0x0b030019,0x0b04000a,0x0d06000c,0x0b01006e,0x0d06000b,0x0b03000d,0x0b07000b,
0x00000000,0x01010001,0x0d03000b,0x02020002,0x0d030012,0x0d070009,0x0d06000a,0x0e040048,
0x00000000,0x0b020038,0x0d03000c,0x0d07000a,0x0d020021,0x0d06000c,0x0d06000b,0x0d06000d,
0x00000000,0x01010001,0x0d070009,0x0d03004d,0x0b020030,0x0d07000a,0x0d06000c,0x0d06000d,
0x00000000,0x0c020058,0x0d07000a,0x0d06000d,0x0b01006f,0x0d06000d,0x0d06000d,0x0d0a000b,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d02000c,0x05020003,0x05010004,0x03030003,
0x0e01008c,0x0b04000b,0x04010003,0x06030004,0x0b030012,0x06020005,0x05010005,0x04040004,
0x00000000,0x10030088,0x05020003,0x0b04000f,0x0b020031,0x05020004,0x05010006,0x1005001d,
0x1002010d,0x0b03001a,0x05020004,0x1004001e,0x0b010070,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0d030008,0x100300c5,0x0d02000d,0x06030004,0x06020005,0x0b040011,
0x1001030c,0x0b020039,0x0d030009,0x06030005,0x0b030013,0x06020007,0x06020006,0x0d06000a,
0x00000000,0x10020187,0x06030004,0x0b03001e,0x0b020032,0x06030005,0x06020007,0x0d05000b,
0x00000000,0x0c020059,0x06030005,0x1004001f,0x0b010071,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0c02000d,0x02020002,0x0d02000e,0x0b050007,0x0b040008,0x03030003,
0x00000000,0x0b04000c,0x0c02000e,0x1005001d,0x0b030014,0x1004001e,0x0b040009,0x0c05000f,
0x00000000,0x01010001,0x0b050007,0x0c03002e,0x0b020033,0x0b050008,0x0b04000a,0x1005001e,
0x00000000,0x0b03001b,0x0b050008,0x10040020,0x0b010072,0x1004001f,0x0b04000b,0x0b080009,
0x00000000,0x01010001,0x0b040008,0x02020002,0x0d02000f,0x1005001d,0x1004001e,0x0c040019,
0x00000000,0x0b02003a,0x0b040009,0x1005001e,0x0b030015,0x10040020,0x1004001f,0x0b07000a,
0x00000000,0x01010001,0x1005001d,0x0d03004e,0x0b020034,0x1005001e,0x10040020,0x0d05000c,
0x00000000,0x0c02005a,0x1005001e,0x10040021,0x0b010073,0x10040021,0x10040021,0x1008001f,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d020010,0x10040025,0x10030026,0x03030003,
0x0e01008d,0x0b04000d,0x04010003,0x0b050008,0x0b030016,0x0b040009,0x10030027,0x04040004,
0x00000000,0x01010001,0x10040025,0x0b040010,0x0b020035,0x10040026,0x10030028,0x0d06000a,
0x1002010e,0x0b03001c,0x10040026,0x0d05000b,0x0b010074,0x0b04000a,0x10030029,0x10070027,
0x00000000,0x01010001,0x0d030009,0x02020002,0x0d020011,0x0b050008,0x0b040009,0x0d040029,
0x1001030d,0x0b02003b,0x0d03000a,0x0b050009,0x0b030017,0x0b04000b,0x0b04000a,0x0d06000b,
0x00000000,0x01010001,0x0b050008,0x0b03001f,0x0b020036,0x0b050009,0x0b04000b,0x0d05000d,
0x00000000,0x0c02005b,0x0b050009,0x0d05000c,0x0b010075,0x0b04000c,0x0b04000c,0x0b08000a,
0x00000000,0x01010001,0x0d020015,0x02020002,0x0d020012,0x0b04000a,0x0b03000b,0x03030003,
0x00000000,0x0b04000e,0x0d020016,0x0d06000a,0x0b030018,0x0d05000b,0x0b03000c,0x0d050017,
0x00000000,0x01010001,0x0b04000a,0x0c03002f,0x0b020037,0x0b04000b,0x0b03000d,0x0d06000b,
0x00000000,0x0b03001d,0x0b04000b,0x0d05000d,0x0b010076,0x0d05000c,0x0b03000e,0x0b07000c,
0x00000000,0x01010001,0x0b040009,0x02020002,0x0d020013,0x0d06000a,0x0d05000b,0x0e040049,
0x00000000,0x0b02003c,0x0b04000a,0x0d06000b,0x0b030019,0x0d05000d,0x0d05000c,0x0b07000b,
0x00000000,0x01010001,0x0d06000a,0x0d03004f,0x0b020038,0x0d06000b,0x0d05000d,0x0d05000e,
0x00000000,0x0c02005c,0x0d06000b,0x0d05000e,0x0b010077,0x0d05000e,0x0d05000e,0x0d09000c,
0x00000000,0x01010001,0x04010002,0x02020002,0x0b04000b,0x05020003,0x05010004,0x03030003,
0x0e01008e,0x0b04000f,0x04010003,0x06030004,0x0b03001a,0x06020005,0x05010005,0x04040004,
0x00000000,0x10030089,0x05020003,0x0b040011,0x0b020039,0x05020004,0x05010006,0x0c05000a,
0x1002010f,0x0b03001e,0x05020004,0x0c04000b,0x0b010078,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0b050007,0x02020002,0x0b04000c,0x06030004,0x06020005,0x0b040012,
0x1001030e,0x0b02003d,0x0b050008,0x06030005,0x0b03001b,0x06020007,0x06020006,0x0b080009,
0x00000000,0x10020188,0x06030004,0x0b030020,0x0b02003a,0x06030005,0x06020007,0x0b07000a,
0x00000000,0x0c02005d,0x06030005,0x0c04000c,0x0b010079,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0e020025,0x02020002,0x0b04000d,0x0b050008,0x0b040009,0x03030003,
0x00000000,0x0b040010,0x0e020026,0x0c05000a,0x0b03001c,0x0c04000b,0x0b04000a,0x0e050027,
0x00000000,0x01010001,0x0b050008,0x0c030030,0x0b02003b,0x0b050009,0x0b04000b,0x0c05000b,
0x00000000,0x0b03001f,0x0b050009,0x0c04000d,0x0b01007a,0x0c04000c,0x0b04000c,0x0b08000a,
0x00000000,0x01010001,0x0b04000a,0x02020002,0x0b04000e,0x0c05000a,0x0c04000b,0x0c04001a,
0x00000000,0x0b02003e,0x0b04000b,0x0c05000b,0x0b03001d,0x0c04000d,0x0c04000c,0x0b07000c,
0x00000000,0x01010001,0x0c05000a,0x0d030050,0x0b02003c,0x0c05000b,0x0c04000d,0x0b07000b,
0x00000000,0x0c02005e,0x0c05000b,0x0c04000e,0x0b01007b,0x0c04000e,0x0c04000e,0x0c08000c,
0x00000000,0x01010001,0x04010002,0x02020002,0x0b04000f,0x10030064,0x10020065,0x03030003,
0x0e01008f,0x0b040011,0x04010003,0x0b04000b,0x0b03001e,0x0b03000c,0x10020066,0x04040004,
0x00000000,0x01010001,0x10030064,0x0b040012,0x0b02003d,0x10030065,0x10020067,0x0b080009,
0x10020110,0x0b030020,0x10030065,0x0b07000a,0x0b01007c,0x0b03000d,0x10020068,0x10060066,
0x00000000,0x01010001,0x0b050008,0x02020002,0x0b040010,0x0b04000b,0x0b03000c,0x0d04002a,
0x1001030f,0x0b02003f,0x0b050009,0x0b04000c,0x0b03001f,0x0b03000e,0x0b03000d,0x0b08000a,
0x00000000,0x01010001,0x0b04000b,0x0b030021,0x0b02003e,0x0b04000c,0x0b03000e,0x0b07000c,
0x00000000,0x0c02005f,0x0b04000c,0x0b07000b,0x0b01007d,0x0b03000f,0x0b03000f,0x0b07000d,
0x00000000,0x01010001,0x10030046,0x02020002,0x0b040011,0x0b04000b,0x0b03000c,0x03030003,
0x00000000,0x0b040012,0x10030047,0x0b080009,0x0b030020,0x0b07000a,0x0b03000d,0x10060048,
0x00000000,0x01010001,0x0b04000b,0x0c030031,0x0b02003f,0x0b04000c,0x0b03000e,0x0b08000a,
0x00000000,0x0b030021,0x0b04000c,0x0b07000c,0x0b01007e,0x0b07000b,0x0b03000f,0x0b07000d,
0x00000000,0x01010001,0x0b04000b,0x02020002,0x0b040012,0x0b080009,0x0b07000a,0x0e04004a,
0x00000000,0x0b020040,0x0b04000c,0x0b08000a,0x0b030021,0x0b07000c,0x0b07000b,0x0b07000d,
0x00000000,0x01010001,0x0b080009,0x0d030051,0x0b020040,0x0b08000a,0x0b07000c,0x0b07000d,
0x00000000,0x0c020060,0x0b08000a,0x0b07000d,0x0b01007f,0x0b07000d,0x0b07000d,0x0b0b000b,
0x00000000,0x01010001,0x04010002,0x02020002,0x08010008,0x05020003,0x05010004,0x03030003,
0x0e010090,0x09020009,0x04010003,0x06030004,0x09010010,0x06020005,0x05010005,0x04040004,
0x00000000,0x1003008a,0x05020003,0x0e04000c,0x0e020022,0x05020004,0x05010006,0x07040005,
0x10020111,0x0e030013,0x05020004,0x07030006,0x0c020041,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x08020004,0x100300c6,0x08010009,0x06030004,0x06020005,0x0c05000c,
0x10010310,0x0c030022,0x08020005,0x06030005,0x09010011,0x06020007,0x06020006,0x08050006,
0x00000000,0x10020189,0x06030004,0x0c040013,0x0e020023,0x06030005,0x06020007,0x08040007,
0x00000000,0x0c020061,0x06030005,0x07030007,0x0c020042,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0c02000e,0x02020002,0x0801000a,0x09030005,0x09020006,0x03030003,
0x00000000,0x0902000a,0x0c02000f,0x07040005,0x09010012,0x07030006,0x09020007,0x0c050010,
0x00000000,0x01010001,0x09030005,0x0c030032,0x0e020024,0x09030006,0x09020008,0x07040006,
0x00000000,0x0e030014,0x09030006,0x07030008,0x0c020043,0x07030007,0x09020009,0x09060007,
0x00000000,0x01010001,0x09020006,0x02020002,0x0801000b,0x07040005,0x07030006,0x0c04001b,
0x00000000,0x0c030023,0x09020007,0x07040006,0x09010013,0x07030008,0x07030007,0x09050008,
0x00000000,0x01010001,0x07040005,0x0d030052,0x0e020025,0x07040006,0x07030008,0x08040008,
0x00000000,0x0c020062,0x07040006,0x07030009,0x0c020044,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0801000c,0x0d03000d,0x0d02000e,0x03030003,
0x0e010091,0x0902000b,0x04010003,0x0e050008,0x09010014,0x0e040009,0x0d02000f,0x04040004,
0x00000000,0x01010001,0x0d03000d,0x0e04000d,0x0e020026,0x0d03000e,0x0d020010,0x08050006,
0x10020112,0x0e030015,0x0d03000e,0x08040007,0x0c020045,0x0e04000a,0x0d020011,0x0d06000f,
0x00000000,0x01010001,0x08020005,0x02020002,0x0801000d,0x0e050008,0x0e040009,0x0d04002b,
0x10010311,0x0c030024,0x08020006,0x0e050009,0x09010015,0x0e04000b,0x0e04000a,0x08050007,
0x00000000,0x01010001,0x0e050008,0x0c040014,0x0e020027,0x0e050009,0x0e04000b,0x08040009,
0x00000000,0x0c020063,0x0e050009,0x08040008,0x0c020046,0x0e04000c,0x0e04000c,0x0e08000a,
0x00000000,0x01010001,0x0d020016,0x02020002,0x0801000e,0x0e040009,0x0e03000a,0x03030003,
0x00000000,0x0902000c,0x0d020017,0x08050006,0x09010016,0x08040007,0x0e03000b,0x0d050018,
0x00000000,0x01010001,0x0e040009,0x0c030033,0x0e020028,0x0e04000a,0x0e03000c,0x08050007,
0x00000000,0x0e030016,0x0e04000a,0x08040009,0x0c020047,0x08040008,0x0e03000d,0x0e07000b,
0x00000000,0x01010001,0x09020007,0x02020002,0x0801000f,0x08050006,0x08040007,0x0e04004b,
0x00000000,0x0c030025,0x09020008,0x08050007,0x09010017,0x08040009,0x08040008,0x09050009,
0x00000000,0x01010001,0x08050006,0x0d030053,0x0e020029,0x08050007,0x08040009,0x0804000a,
0x00000000,0x0c020064,0x08050007,0x0804000a,0x0c020048,0x0804000a,0x0804000a,0x08080008,
0x00000000,0x01010001,0x04010002,0x02020002,0x09020009,0x05020003,0x05010004,0x03030003,
0x0e010092,0x0902000d,0x04010003,0x0e04000d,0x09010018,0x0e03000e,0x05010005,0x04040004,
0x00000000,0x1003008b,0x05020003,0x0e04000e,0x0e02002a,0x05020004,0x05010006,0x0c060008,
0x10020113,0x0e030017,0x05020004,0x0c050009,0x0c020049,0x0e03000f,0x05010007,0x05050005,
0x00000000,0x01010001,0x09030005,0x02020002,0x0902000a,0x0e04000d,0x0e03000e,0x0c05000d,
0x10010312,0x0c030026,0x09030006,0x0e04000e,0x09010019,0x0e030010,0x0e03000f,0x09060007,
0x00000000,0x1002018a,0x0e04000d,0x0c040015,0x0e02002b,0x0e04000e,0x0e030010,0x09050008,
0x00000000,0x0c020065,0x0e04000e,0x0c05000a,0x0c02004a,0x0e030011,0x0e030011,0x0e07000f,
0x00000000,0x01010001,0x0e020026,0x02020002,0x0902000b,0x09030006,0x09020007,0x03030003,
0x00000000,0x0902000e,0x0e020027,0x0c060008,0x0901001a,0x0c050009,0x09020008,0x0e050028,
0x00000000,0x01010001,0x09030006,0x0c030034,0x0e02002c,0x09030007,0x09020009,0x0c060009,
0x00000000,0x0e030018,0x09030007,0x0c05000b,0x0c02004b,0x0c05000a,0x0902000a,0x09060008,
0x00000000,0x01010001,0x09020008,0x02020002,0x0902000c,0x0c060008,0x0c050009,0x0c04001c,
0x00000000,0x0c030027,0x09020009,0x0c060009,0x0901001b,0x0c05000b,0x0c05000a,0x0905000a,
0x00000000,0x01010001,0x0c060008,0x0d030054,0x0e02002d,0x0c060009,0x0c05000b,0x09050009,
0x00000000,0x0c020066,0x0c060009,0x0c05000c,0x0c02004c,0x0c05000c,0x0c05000c,0x0c09000a,
0x00000000,0x01010001,0x04010002,0x02020002,0x0902000d,0x0e030015,0x0e020016,0x03030003,
0x0e010093,0x0902000f,0x04010003,0x0c050009,0x0901001c,0x0c04000a,0x0e020017,0x04040004,
0x00000000,0x01010001,0x0e030015,0x0e04000f,0x0e02002e,0x0e030016,0x0e020018,0x09060007,
0x10020114,0x0e030019,0x0e030016,0x09050008,0x0c02004d,0x0c04000b,0x0e020019,0x0e060017,
0x00000000,0x01010001,0x09030006,0x02020002,0x0902000e,0x0c050009,0x0c04000a,0x0d04002c,
0x10010313,0x0c030028,0x09030007,0x0c05000a,0x0901001d,0x0c04000c,0x0c04000b,0x09060008,
0x00000000,0x01010001,0x0c050009,0x0c040016,0x0e02002f,0x0c05000a,0x0c04000c,0x0905000a,
0x00000000,0x0c020067,0x0c05000a,0x09050009,0x0c02004e,0x0c04000d,0x0c04000d,0x0c08000b,
0x00000000,0x01010001,0x10030047,0x02020002,0x0902000f,0x0e04000a,0x0e03000b,0x03030003,
0x00000000,0x09020010,0x10030048,0x09060007,0x0901001e,0x09050008,0x0e03000c,0x10060049,
0x00000000,0x01010001,0x0e04000a,0x0c030035,0x0e020030,0x0e04000b,0x0e03000d,0x09060008,
0x00000000,0x0e03001a,0x0e04000b,0x0905000a,0x0c02004f,0x09050009,0x0e03000e,0x0e07000c,
0x00000000,0x01010001,0x09020009,0x02020002,0x09020010,0x09060007,0x09050008,0x0e04004c,
0x00000000,0x0c030029,0x0902000a,0x09060008,0x0901001f,0x0905000a,0x09050009,0x0905000b,
0x00000000,0x01010001,0x09060007,0x0d030055,0x0e020031,0x09060008,0x0905000a,0x0905000b,
0x00000000,0x0c020068,0x09060008,0x0905000b,0x0c020050,0x0905000b,0x0905000b,0x09090009,
0x00000000,0x01010001,0x04010002,0x02020002,0x0c02000b,0x05020003,0x05010004,0x03030003,
0x0e010094,0x0e04000c,0x04010003,0x06030004,0x0e030013,0x06020005,0x05010005,0x04040004,
0x00000000,0x1003008c,0x05020003,0x0e040010,0x0e020032,0x05020004,0x05010006,0x07040005,
0x10020115,0x0e03001b,0x05020004,0x07030006,0x0c020051,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0c030007,0x100300c7,0x0c02000c,0x06030004,0x06020005,0x0c05000e,
0x10010314,0x0c03002a,0x0c030008,0x06030005,0x0e030014,0x06020007,0x06020006,0x0c060009,
0x00000000,0x1002018b,0x06030004,0x0c040017,0x0e020033,0x06030005,0x06020007,0x0c05000a,
0x00000000,0x0c020069,0x06030005,0x07030007,0x0c020052,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0c02000f,0x02020002,0x0c02000d,0x0e050008,0x0e040009,0x03030003,
0x00000000,0x0e04000d,0x0c020010,0x07040005,0x0e030015,0x07030006,0x0e04000a,0x0c050011,
0x00000000,0x01010001,0x0e050008,0x0c030036,0x0e020034,0x0e050009,0x0e04000b,0x07040006,
0x00000000,0x0e03001c,0x0e050009,0x07030008,0x0c020053,0x07030007,0x0e04000c,0x0e08000a,
0x00000000,0x01010001,0x0e040009,0x02020002,0x0c02000e,0x07040005,0x07030006,0x0c04001d,
0x00000000,0x0c03002b,0x0e04000a,0x07040006,0x0e030016,0x07030008,0x07030007,0x0e07000b,
0x00000000,0x01010001,0x07040005,0x0d030056,0x0e020035,0x07040006,0x07030008,0x0c05000b,
0x00000000,0x0c02006a,0x07040006,0x07030009,0x0c020054,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0c02000f,0x10040026,0x10030027,0x03030003,
0x0e010095,0x0e04000e,0x04010003,0x0e050009,0x0e030017,0x0e04000a,0x10030028,0x04040004,
0x00000000,0x01010001,0x10040026,0x0e040011,0x0e020036,0x10040027,0x10030029,0x0c060009,
0x10020116,0x0e03001d,0x10040027,0x0c05000a,0x0c020055,0x0e04000b,0x1003002a,0x10070028,
0x00000000,0x01010001,0x0c030008,0x02020002,0x0c020010,0x0e050009,0x0e04000a,0x0d04002d,
0x10010315,0x0c03002c,0x0c030009,0x0e05000a,0x0e030018,0x0e04000c,0x0e04000b,0x0c06000a,
0x00000000,0x01010001,0x0e050009,0x0c040018,0x0e020037,0x0e05000a,0x0e04000c,0x0c05000c,
0x00000000,0x0c02006b,0x0e05000a,0x0c05000b,0x0c020056,0x0e04000d,0x0e04000d,0x0e08000b,
0x00000000,0x01010001,0x0d020017,0x02020002,0x0c020011,0x0e04000b,0x0e03000c,0x03030003,
0x00000000,0x0e04000f,0x0d020018,0x0c060009,0x0e030019,0x0c05000a,0x0e03000d,0x0d050019,
0x00000000,0x01010001,0x0e04000b,0x0c030037,0x0e020038,0x0e04000c,0x0e03000e,0x0c06000a,
0x00000000,0x0e03001e,0x0e04000c,0x0c05000c,0x0c020057,0x0c05000b,0x0e03000f,0x0e07000d,
0x00000000,0x01010001,0x0e04000a,0x02020002,0x0c020012,0x0c060009,0x0c05000a,0x0e04004d,
0x00000000,0x0c03002d,0x0e04000b,0x0c06000a,0x0e03001a,0x0c05000c,0x0c05000b,0x0e07000c,
0x00000000,0x01010001,0x0c060009,0x0d030057,0x0e020039,0x0c06000a,0x0c05000c,0x0c05000d,
0x00000000,0x0c02006c,0x0c06000a,0x0c05000d,0x0c020058,0x0c05000d,0x0c05000d,0x0c09000b,
0x00000000,0x01010001,0x04010002,0x02020002,0x0e04000c,0x05020003,0x05010004,0x03030003,
0x0e010096,0x0e040010,0x04010003,0x10050016,0x0e03001b,0x10040017,0x05010005,0x04040004,
0x00000000,0x1003008d,0x05020003,0x0e040012,0x0e02003a,0x05020004,0x05010006,0x0c05000b,
0x10020117,0x0e03001f,0x05020004,0x0c04000c,0x0c020059,0x10040018,0x05010007,0x05050005,
0x00000000,0x01010001,0x0e050008,0x02020002,0x0e04000d,0x10050016,0x10040017,0x0c05000f,
0x10010316,0x0c03002e,0x0e050009,0x10050017,0x0e03001c,0x10040019,0x10040018,0x0e08000a,
0x00000000,0x1002018c,0x10050016,0x0c040019,0x0e02003b,0x10050017,0x10040019,0x0e07000b,
0x00000000,0x0c02006d,0x10050017,0x0c04000d,0x0c02005a,0x1004001a,0x1004001a,0x10080018,
0x00000000,0x01010001,0x0e020027,0x02020002,0x0e04000e,0x0e050009,0x0e04000a,0x03030003,
0x00000000,0x0e040011,0x0e020028,0x0c05000b,0x0e03001d,0x0c04000c,0x0e04000b,0x0e050029,
0x00000000,0x01010001,0x0e050009,0x0c030038,0x0e02003c,0x0e05000a,0x0e04000c,0x0c05000c,
0x00000000,0x0e030020,0x0e05000a,0x0c04000e,0x0c02005b,0x0c04000d,0x0e04000d,0x0e08000b,
0x00000000,0x01010001,0x0e04000b,0x02020002,0x0e04000f,0x0c05000b,0x0c04000c,0x0c04001e,
0x00000000,0x0c03002f,0x0e04000c,0x0c05000c,0x0e03001e,0x0c04000e,0x0c04000d,0x0e07000d,
0x00000000,0x01010001,0x0c05000b,0x0d030058,0x0e02003d,0x0c05000c,0x0c04000e,0x0e07000c,
0x00000000,0x0c02006e,0x0c05000c,0x0c04000f,0x0c02005c,0x0c04000f,0x0c04000f,0x0c08000d,
0x00000000,0x01010001,0x04010002,0x02020002,0x0e040010,0x10030065,0x10020066,0x03030003,
0x0e010097,0x0e040012,0x04010003,0x0c05000a,0x0e03001f,0x0c04000b,0x10020067,0x04040004,
0x00000000,0x01010001,0x10030065,0x0e040013,0x0e02003e,0x10030066,0x10020068,0x0e08000a,
0x10020118,0x0e030021,0x10030066,0x0e07000b,0x0c02005d,0x0c04000c,0x10020069,0x10060067,
0x00000000,0x01010001,0x0e050009,0x02020002,0x0e040011,0x0c05000a,0x0c04000b,0x0d04002e,
0x10010317,0x0c030030,0x0e05000a,0x0c05000b,0x0e030020,0x0c04000d,0x0c04000c,0x0e08000b,
0x00000000,0x01010001,0x0c05000a,0x0c04001a,0x0e02003f,0x0c05000b,0x0c04000d,0x0e07000d,
0x00000000,0x0c02006f,0x0c05000b,0x0e07000c,0x0c02005e,0x0c04000e,0x0c04000e,0x0c08000c,
0x00000000,0x01010001,0x10030048,0x02020002,0x0e040012,0x0e04000c,0x0e03000d,0x03030003,
0x00000000,0x0e040013,0x10030049,0x0e08000a,0x0e030021,0x0e07000b,0x0e03000e,0x1006004a,
0x00000000,0x01010001,0x0e04000c,0x0c030039,0x0e020040,0x0e04000d,0x0e03000f,0x0e08000b,
0x00000000,0x0e030022,0x0e04000d,0x0e07000d,0x0c02005f,0x0e07000c,0x0e030010,0x0e07000e,
0x00000000,0x01010001,0x0e04000c,0x02020002,0x0e040013,0x0e08000a,0x0e07000b,0x0e04004e,
0x00000000,0x0c030031,0x0e04000d,0x0e08000b,0x0e030022,0x0e07000d,0x0e07000c,0x0e07000e,
0x00000000,0x01010001,0x0e08000a,0x0d030059,0x0e020041,0x0e08000b,0x0e07000d,0x0e07000e,
0x00000000,0x0c020070,0x0e08000b,0x0e07000e,0x0c020060,0x0e07000e,0x0e07000e,0x0e0b000c,
0x00000000,0x01010001,0x04010002,0x02020002,0x08010008,0x05020003,0x05010004,0x03030003,
0x0e010098,0x0e03000d,0x04010003,0x06030004,0x0e020014,0x06020005,0x05010005,0x04040004,
0x00000000,0x1003008e,0x05020003,0x0c05000c,0x0c030022,0x05020004,0x05010006,0x07040005,
0x10020119,0x0c040013,0x05020004,0x07030006,0x0c020061,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x08020004,0x100300c8,0x08010009,0x06030004,0x06020005,0x0c050010,
0x10010318,0x0c030032,0x08020005,0x06030005,0x0e020015,0x06020007,0x06020006,0x08050006,
0x00000000,0x1002018d,0x06030004,0x0c04001b,0x0c030023,0x06030005,0x06020007,0x08040007,
0x00000000,0x0c020071,0x06030005,0x07030007,0x0c020062,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0c020010,0x02020002,0x0801000a,0x0e040009,0x0e03000a,0x03030003,
0x00000000,0x0e03000e,0x0c020011,0x07040005,0x0e020016,0x07030006,0x0e03000b,0x0c050012,
0x00000000,0x01010001,0x0e040009,0x0c03003a,0x0c030024,0x0e04000a,0x0e03000c,0x07040006,
0x00000000,0x0c040014,0x0e04000a,0x07030008,0x0c020063,0x07030007,0x0e03000d,0x0e07000b,
0x00000000,0x01010001,0x0e03000a,0x02020002,0x0801000b,0x07040005,0x07030006,0x0c04001f,
0x00000000,0x0c030033,0x0e03000b,0x07040006,0x0e020017,0x07030008,0x07030007,0x0e06000c,
0x00000000,0x01010001,0x07040005,0x0d03005a,0x0c030025,0x07040006,0x07030008,0x08040008,
0x00000000,0x0c020072,0x07040006,0x07030009,0x0c020064,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0801000c,0x0d03000e,0x0d02000f,0x03030003,
0x0e010099,0x0e03000f,0x04010003,0x0c060008,0x0e020018,0x0c050009,0x0d020010,0x04040004,
0x00000000,0x01010001,0x0d03000e,0x0c05000d,0x0c030026,0x0d03000f,0x0d020011,0x08050006,
0x1002011a,0x0c040015,0x0d03000f,0x08040007,0x0c020065,0x0c05000a,0x0d020012,0x0d060010,
0x00000000,0x01010001,0x08020005,0x02020002,0x0801000d,0x0c060008,0x0c050009,0x0d04002f,
0x10010319,0x0c030034,0x08020006,0x0c060009,0x0e020019,0x0c05000b,0x0c05000a,0x08050007,
0x00000000,0x01010001,0x0c060008,0x0c04001c,0x0c030027,0x0c060009,0x0c05000b,0x08040009,
0x00000000,0x0c020073,0x0c060009,0x08040008,0x0c020066,0x0c05000c,0x0c05000c,0x0c09000a,
0x00000000,0x01010001,0x0d020018,0x02020002,0x0801000e,0x0c050009,0x0c04000a,0x03030003,
0x00000000,0x0e030010,0x0d020019,0x08050006,0x0e02001a,0x08040007,0x0c04000b,0x0d05001a,
0x00000000,0x01010001,0x0c050009,0x0c03003b,0x0c030028,0x0c05000a,0x0c04000c,0x08050007,
0x00000000,0x0c040016,0x0c05000a,0x08040009,0x0c020067,0x08040008,0x0c04000d,0x0c08000b,
0x00000000,0x01010001,0x0e03000b,0x02020002,0x0801000f,0x08050006,0x08040007,0x0e04004f,
0x00000000,0x0c030035,0x0e03000c,0x08050007,0x0e02001b,0x08040009,0x08040008,0x0e06000d,
0x00000000,0x01010001,0x08050006,0x0d03005b,0x0c030029,0x08050007,0x08040009,0x0804000a,
0x00000000,0x0c020074,0x08050007,0x0804000a,0x0c020068,0x0804000a,0x0804000a,0x08080008,
0x00000000,0x01010001,0x04010002,0x02020002,0x0e03000d,0x05020003,0x05010004,0x03030003,
0x0e01009a,0x0e030011,0x04010003,0x10040035,0x0e02001c,0x10030036,0x05010005,0x04040004,
0x00000000,0x1003008f,0x05020003,0x0c05000e,0x0c03002a,0x05020004,0x05010006,0x0c060009,
0x1002011b,0x0c040017,0x05020004,0x0c05000a,0x0c020069,0x10030037,0x05010007,0x05050005,
0x00000000,0x01010001,0x0e040009,0x02020002,0x0e03000e,0x10040035,0x10030036,0x0c050011,
0x1001031a,0x0c030036,0x0e04000a,0x10040036,0x0e02001d,0x10030038,0x10030037,0x0e07000b,
0x00000000,0x1002018e,0x10040035,0x0c04001d,0x0c03002b,0x10040036,0x10030038,0x0e06000c,
0x00000000,0x0c020075,0x10040036,0x0c05000b,0x0c02006a,0x10030039,0x10030039,0x10070037,
0x00000000,0x01010001,0x0e020028,0x02020002,0x0e03000f,0x0e04000a,0x0e03000b,0x03030003,
0x00000000,0x0e030012,0x0e020029,0x0c060009,0x0e02001e,0x0c05000a,0x0e03000c,0x0e05002a,
0x00000000,0x01010001,0x0e04000a,0x0c03003c,0x0c03002c,0x0e04000b,0x0e03000d,0x0c06000a,
0x00000000,0x0c040018,0x0e04000b,0x0c05000c,0x0c02006b,0x0c05000b,0x0e03000e,0x0e07000c,
0x00000000,0x01010001,0x0e03000c,0x02020002,0x0e030010,0x0c060009,0x0c05000a,0x0c040020,
0x00000000,0x0c030037,0x0e03000d,0x0c06000a,0x0e02001f,0x0c05000c,0x0c05000b,0x0e06000e,
0x00000000,0x01010001,0x0c060009,0x0d03005c,0x0c03002d,0x0c06000a,0x0c05000c,0x0e06000d,
0x00000000,0x0c020076,0x0c06000a,0x0c05000d,0x0c02006c,0x0c05000d,0x0c05000d,0x0c09000b,
0x00000000,0x01010001,0x04010002,0x02020002,0x0e030011,0x0e030016,0x0e020017,0x03030003,
0x0e01009b,0x0e030013,0x04010003,0x0c05000b,0x0e020020,0x0c04000c,0x0e020018,0x04040004,
0x00000000,0x01010001,0x0e030016,0x0c05000f,0x0c03002e,0x0e030017,0x0e020019,0x0e07000b,
0x1002011c,0x0c040019,0x0e030017,0x0e06000c,0x0c02006d,0x0c04000d,0x0e02001a,0x0e060018,
0x00000000,0x01010001,0x0e04000a,0x02020002,0x0e030012,0x0c05000b,0x0c04000c,0x0d040030,
0x1001031b,0x0c030038,0x0e04000b,0x0c05000c,0x0e020021,0x0c04000e,0x0c04000d,0x0e07000c,
0x00000000,0x01010001,0x0c05000b,0x0c04001e,0x0c03002f,0x0c05000c,0x0c04000e,0x0e06000e,
0x00000000,0x0c020077,0x0c05000c,0x0e06000d,0x0c02006e,0x0c04000f,0x0c04000f,0x0c08000d,
0x00000000,0x01010001,0x10030049,0x02020002,0x0e030013,0x0c05000a,0x0c04000b,0x03030003,
0x00000000,0x0e030014,0x1003004a,0x0e07000b,0x0e020022,0x0e06000c,0x0c04000c,0x1006004b,
0x00000000,0x01010001,0x0c05000a,0x0c03003d,0x0c030030,0x0c05000b,0x0c04000d,0x0e07000c,
0x00000000,0x0c04001a,0x0c05000b,0x0e06000e,0x0c02006f,0x0e06000d,0x0c04000e,0x0c08000c,
0x00000000,0x01010001,0x0e03000d,0x02020002,0x0e030014,0x0e07000b,0x0e06000c,0x0e040050,
0x00000000,0x0c030039,0x0e03000e,0x0e07000c,0x0e020023,0x0e06000e,0x0e06000d,0x0e06000f,
0x00000000,0x01010001,0x0e07000b,0x0d03005d,0x0c030031,0x0e07000c,0x0e06000e,0x0e06000f,
0x00000000,0x0c020078,0x0e07000c,0x0e06000f,0x0c020070,0x0e06000f,0x0e06000f,0x0e0a000d,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d02000d,0x05020003,0x05010004,0x03030003,
0x0e01009c,0x0c05000c,0x04010003,0x06030004,0x0c040013,0x06020005,0x05010005,0x04040004,
0x00000000,0x10030090,0x05020003,0x0c050010,0x0c030032,0x05020004,0x05010006,0x07040005,
0x1002011d,0x0c04001b,0x05020004,0x07030006,0x0c020071,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0d030009,0x100300c9,0x0d02000e,0x06030004,0x06020005,0x0c050012,
0x1001031c,0x0c03003a,0x0d03000a,0x06030005,0x0c040014,0x06020007,0x06020006,0x0d06000b,
0x00000000,0x1002018f,0x06030004,0x0c04001f,0x0c030033,0x06030005,0x06020007,0x0d05000c,
0x00000000,0x0c020079,0x06030005,0x07030007,0x0c020072,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0c020011,0x02020002,0x0d02000f,0x0c060008,0x0c050009,0x03030003,
0x00000000,0x0c05000d,0x0c020012,0x07040005,0x0c040015,0x07030006,0x0c05000a,0x0c050013,
0x00000000,0x01010001,0x0c060008,0x0c03003e,0x0c030034,0x0c060009,0x0c05000b,0x07040006,
0x00000000,0x0c04001c,0x0c060009,0x07030008,0x0c020073,0x07030007,0x0c05000c,0x0c09000a,
0x00000000,0x01010001,0x0c050009,0x02020002,0x0d020010,0x07040005,0x07030006,0x0c040021,
0x00000000,0x0c03003b,0x0c05000a,0x07040006,0x0c040016,0x07030008,0x07030007,0x0c08000b,
0x00000000,0x01010001,0x07040005,0x0d03005e,0x0c030035,0x07040006,0x07030008,0x0d05000d,
0x00000000,0x0c02007a,0x07040006,0x07030009,0x0c020074,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d020011,0x10040027,0x10030028,0x03030003,
0x0e01009d,0x0c05000e,0x04010003,0x0c060009,0x0c040017,0x0c05000a,0x10030029,0x04040004,
0x00000000,0x01010001,0x10040027,0x0c050011,0x0c030036,0x10040028,0x1003002a,0x0d06000b,
0x1002011e,0x0c04001d,0x10040028,0x0d05000c,0x0c020075,0x0c05000b,0x1003002b,0x10070029,
0x00000000,0x01010001,0x0d03000a,0x02020002,0x0d020012,0x0c060009,0x0c05000a,0x0d040031,
0x1001031d,0x0c03003c,0x0d03000b,0x0c06000a,0x0c040018,0x0c05000c,0x0c05000b,0x0d06000c,
0x00000000,0x01010001,0x0c060009,0x0c040020,0x0c030037,0x0c06000a,0x0c05000c,0x0d05000e,
0x00000000,0x0c02007b,0x0c06000a,0x0d05000d,0x0c020076,0x0c05000d,0x0c05000d,0x0c09000b,
0x00000000,0x01010001,0x0d020019,0x02020002,0x0d020013,0x0c05000b,0x0c04000c,0x03030003,
0x00000000,0x0c05000f,0x0d02001a,0x0d06000b,0x0c040019,0x0d05000c,0x0c04000d,0x0d05001b,
0x00000000,0x01010001,0x0c05000b,0x0c03003f,0x0c030038,0x0c05000c,0x0c04000e,0x0d06000c,
0x00000000,0x0c04001e,0x0c05000c,0x0d05000e,0x0c020077,0x0d05000d,0x0c04000f,0x0c08000d,
0x00000000,0x01010001,0x0c05000a,0x02020002,0x0d020014,0x0d06000b,0x0d05000c,0x0e040051,
0x00000000,0x0c03003d,0x0c05000b,0x0d06000c,0x0c04001a,0x0d05000e,0x0d05000d,0x0c08000c,
0x00000000,0x01010001,0x0d06000b,0x0d03005f,0x0c030039,0x0d06000c,0x0d05000e,0x0d05000f,
0x00000000,0x0c02007c,0x0d06000c,0x0d05000f,0x0c020078,0x0d05000f,0x0d05000f,0x0d09000d,
0x00000000,0x01010001,0x04010002,0x02020002,0x0c05000c,0x05020003,0x05010004,0x03030003,
0x0e01009e,0x0c050010,0x04010003,0x06030004,0x0c04001b,0x06020005,0x05010005,0x04040004,
0x00000000,0x10030091,0x05020003,0x0c050012,0x0c03003a,0x05020004,0x05010006,0x0c05000c,
0x1002011f,0x0c04001f,0x05020004,0x0c04000d,0x0c020079,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0c060008,0x02020002,0x0c05000d,0x06030004,0x06020005,0x0c050013,
0x1001031e,0x0c03003e,0x0c060009,0x06030005,0x0c04001c,0x06020007,0x06020006,0x0c09000a,
0x00000000,0x10020190,0x06030004,0x0c040021,0x0c03003b,0x06030005,0x06020007,0x0c08000b,
0x00000000,0x0c02007d,0x06030005,0x0c04000e,0x0c02007a,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0e020029,0x02020002,0x0c05000e,0x0c060009,0x0c05000a,0x03030003,
0x00000000,0x0c050011,0x0e02002a,0x0c05000c,0x0c04001d,0x0c04000d,0x0c05000b,0x0e05002b,
0x00000000,0x01010001,0x0c060009,0x0c030040,0x0c03003c,0x0c06000a,0x0c05000c,0x0c05000d,
0x00000000,0x0c040020,0x0c06000a,0x0c04000f,0x0c02007b,0x0c04000e,0x0c05000d,0x0c09000b,
0x00000000,0x01010001,0x0c05000b,0x02020002,0x0c05000f,0x0c05000c,0x0c04000d,0x0c040022,
0x00000000,0x0c03003f,0x0c05000c,0x0c05000d,0x0c04001e,0x0c04000f,0x0c04000e,0x0c08000d,
0x00000000,0x01010001,0x0c05000c,0x0d030060,0x0c03003d,0x0c05000d,0x0c04000f,0x0c08000c,
0x00000000,0x0c02007e,0x0c05000d,0x0c040010,0x0c02007c,0x0c040010,0x0c040010,0x0c08000e,
0x00000000,0x01010001,0x04010002,0x02020002,0x0c050010,0x10030066,0x10020067,0x03030003,
0x0e01009f,0x0c050012,0x04010003,0x0c05000c,0x0c04001f,0x0c04000d,0x10020068,0x04040004,
0x00000000,0x01010001,0x10030066,0x0c050013,0x0c03003e,0x10030067,0x10020069,0x0c09000a,
0x10020120,0x0c040021,0x10030067,0x0c08000b,0x0c02007d,0x0c04000e,0x1002006a,0x10060068,
0x00000000,0x01010001,0x0c060009,0x02020002,0x0c050011,0x0c05000c,0x0c04000d,0x0d040032,
0x1001031f,0x0c030040,0x0c06000a,0x0c05000d,0x0c040020,0x0c04000f,0x0c04000e,0x0c09000b,
0x00000000,0x01010001,0x0c05000c,0x0c040022,0x0c03003f,0x0c05000d,0x0c04000f,0x0c08000d,
0x00000000,0x0c02007f,0x0c05000d,0x0c08000c,0x0c02007e,0x0c040010,0x0c040010,0x0c08000e,
0x00000000,0x01010001,0x1003004a,0x02020002,0x0c050012,0x0c05000c,0x0c04000d,0x03030003,
0x00000000,0x0c050013,0x1003004b,0x0c09000a,0x0c040021,0x0c08000b,0x0c04000e,0x1006004c,
0x00000000,0x01010001,0x0c05000c,0x0c030041,0x0c030040,0x0c05000d,0x0c04000f,0x0c09000b,
0x00000000,0x0c040022,0x0c05000d,0x0c08000d,0x0c02007f,0x0c08000c,0x0c040010,0x0c08000e,
0x00000000,0x01010001,0x0c05000c,0x02020002,0x0c050013,0x0c09000a,0x0c08000b,0x0e040052,
0x00000000,0x0c030041,0x0c05000d,0x0c09000b,0x0c040022,0x0c08000d,0x0c08000c,0x0c08000e,
0x00000000,0x01010001,0x0c09000a,0x0d030061,0x0c030041,0x0c09000b,0x0c08000d,0x0c08000e,
0x00000000,0x0c020080,0x0c09000b,0x0c08000e,0x0c020080,0x0c08000e,0x0c08000e,0x0c0c000c,
0x00000000,0x01010001,0x04010002,0x02020002,0x08010008,0x05020003,0x05010004,0x03030003,
0x0e0100a0,0x09020009,0x04010003,0x06030004,0x09010010,0x06020005,0x05010005,0x04040004,
0x00000000,0x10030092,0x05020003,0x0a03000a,0x0a010020,0x05020004,0x05010006,0x07040005,
0x10020121,0x0a020011,0x05020004,0x07030006,0x10030043,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x08020004,0x100300ca,0x08010009,0x06030004,0x06020005,0x1006000e,
0x10010320,0x10040024,0x08020005,0x06030005,0x09010011,0x06020007,0x06020006,0x08050006,
0x00000000,0x10020191,0x06030004,0x10050015,0x0a010021,0x06030005,0x06020007,0x08040007,
0x00000000,0x0d030042,0x06030005,0x07030007,0x10030044,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0d03000b,0x02020002,0x0801000a,0x09030005,0x09020006,0x03030003,
0x00000000,0x0902000a,0x0d03000c,0x07040005,0x09010012,0x07030006,0x09020007,0x0d06000d,
0x00000000,0x01010001,0x09030005,0x0d040023,0x0a010022,0x09030006,0x09020008,0x07040006,
0x00000000,0x0a020012,0x09030006,0x07030008,0x10030045,0x07030007,0x09020009,0x09060007,
0x00000000,0x01010001,0x09020006,0x02020002,0x0801000b,0x07040005,0x07030006,0x0d050014,
0x00000000,0x10040025,0x09020007,0x07040006,0x09010013,0x07030008,0x07030007,0x09050008,
0x00000000,0x01010001,0x07040005,0x0d030062,0x0a010023,0x07040006,0x07030008,0x08040008,
0x00000000,0x0d030043,0x07040006,0x07030009,0x10030046,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0801000c,0x0d03000f,0x0d020010,0x03030003,
0x0e0100a1,0x0902000b,0x04010003,0x0a040006,0x09010014,0x0a030007,0x0d020011,0x04040004,
0x00000000,0x01010001,0x0d03000f,0x0a03000b,0x0a010024,0x0d030010,0x0d020012,0x08050006,
0x10020122,0x0a020013,0x0d030010,0x08040007,0x10030047,0x0a030008,0x0d020013,0x0d060011,
0x00000000,0x01010001,0x08020005,0x02020002,0x0801000d,0x0a040006,0x0a030007,0x0d040033,
0x10010321,0x10040026,0x08020006,0x0a040007,0x09010015,0x0a030009,0x0a030008,0x08050007,
0x00000000,0x01010001,0x0a040006,0x10050016,0x0a010025,0x0a040007,0x0a030009,0x08040009,
0x00000000,0x0d030044,0x0a040007,0x08040008,0x10030048,0x0a03000a,0x0a03000a,0x0a070008,
0x00000000,0x01010001,0x0d02001a,0x02020002,0x0801000e,0x0a030007,0x0a020008,0x03030003,
0x00000000,0x0902000c,0x0d02001b,0x08050006,0x09010016,0x08040007,0x0a020009,0x0d05001c,
0x00000000,0x01010001,0x0a030007,0x0d040024,0x0a010026,0x0a030008,0x0a02000a,0x08050007,
0x00000000,0x0a020014,0x0a030008,0x08040009,0x10030049,0x08040008,0x0a02000b,0x0a060009,
0x00000000,0x01010001,0x09020007,0x02020002,0x0801000f,0x08050006,0x08040007,0x0e040053,
0x00000000,0x10040027,0x09020008,0x08050007,0x09010017,0x08040009,0x08040008,0x09050009,
0x00000000,0x01010001,0x08050006,0x0d030063,0x0a010027,0x08050007,0x08040009,0x0804000a,
0x00000000,0x0d030045,0x08050007,0x0804000a,0x1003004a,0x0804000a,0x0804000a,0x08080008,
0x00000000,0x01010001,0x04010002,0x02020002,0x09020009,0x05020003,0x05010004,0x03030003,
0x0e0100a2,0x0902000d,0x04010003,0x0e04000e,0x09010018,0x0e03000f,0x05010005,0x04040004,
0x00000000,0x10030093,0x05020003,0x0a03000c,0x0a010028,0x05020004,0x05010006,0x1007000a,
0x10020123,0x0a020015,0x05020004,0x1006000b,0x1003004b,0x0e030010,0x05010007,0x05050005,
0x00000000,0x01010001,0x09030005,0x02020002,0x0902000a,0x0e04000e,0x0e03000f,0x1006000f,
0x10010322,0x10040028,0x09030006,0x0e04000f,0x09010019,0x0e030011,0x0e030010,0x09060007,
0x00000000,0x10020192,0x0e04000e,0x10050017,0x0a010029,0x0e04000f,0x0e030011,0x09050008,
0x00000000,0x0d030046,0x0e04000f,0x1006000c,0x1003004c,0x0e030012,0x0e030012,0x0e070010,
0x00000000,0x01010001,0x0e02002a,0x02020002,0x0902000b,0x09030006,0x09020007,0x03030003,
0x00000000,0x0902000e,0x0e02002b,0x1007000a,0x0901001a,0x1006000b,0x09020008,0x0e05002c,
0x00000000,0x01010001,0x09030006,0x0d040025,0x0a01002a,0x09030007,0x09020009,0x1007000b,
0x00000000,0x0a020016,0x09030007,0x1006000d,0x1003004d,0x1006000c,0x0902000a,0x09060008,
0x00000000,0x01010001,0x09020008,0x02020002,0x0902000c,0x1007000a,0x1006000b,0x0d050015,
0x00000000,0x10040029,0x09020009,0x1007000b,0x0901001b,0x1006000d,0x1006000c,0x0905000a,
0x00000000,0x01010001,0x1007000a,0x0d030064,0x0a01002b,0x1007000b,0x1006000d,0x09050009,
0x00000000,0x0d030047,0x1007000b,0x1006000e,0x1003004e,0x1006000e,0x1006000e,0x100a000c,
0x00000000,0x01010001,0x04010002,0x02020002,0x0902000d,0x0e030017,0x0e020018,0x03030003,
0x0e0100a3,0x0902000f,0x04010003,0x1006000b,0x0901001c,0x1005000c,0x0e020019,0x04040004,
0x00000000,0x01010001,0x0e030017,0x0a03000d,0x0a01002c,0x0e030018,0x0e02001a,0x09060007,
0x10020124,0x0a020017,0x0e030018,0x09050008,0x1003004f,0x1005000d,0x0e02001b,0x0e060019,
0x00000000,0x01010001,0x09030006,0x02020002,0x0902000e,0x1006000b,0x1005000c,0x0d040034,
0x10010323,0x1004002a,0x09030007,0x1006000c,0x0901001d,0x1005000e,0x1005000d,0x09060008,
0x00000000,0x01010001,0x1006000b,0x10050018,0x0a01002d,0x1006000c,0x1005000e,0x0905000a,
0x00000000,0x0d030048,0x1006000c,0x09050009,0x10030050,0x1005000f,0x1005000f,0x1009000d,
0x00000000,0x01010001,0x1003004b,0x02020002,0x0902000f,0x0a030008,0x0a020009,0x03030003,
0x00000000,0x09020010,0x1003004c,0x09060007,0x0901001e,0x09050008,0x0a02000a,0x1006004d,
0x00000000,0x01010001,0x0a030008,0x0d040026,0x0a01002e,0x0a030009,0x0a02000b,0x09060008,
0x00000000,0x0a020018,0x0a030009,0x0905000a,0x10030051,0x09050009,0x0a02000c,0x0a06000a,
0x00000000,0x01010001,0x09020009,0x02020002,0x09020010,0x09060007,0x09050008,0x0e040054,
0x00000000,0x1004002b,0x0902000a,0x09060008,0x0901001f,0x0905000a,0x09050009,0x0905000b,
0x00000000,0x01010001,0x09060007,0x0d030065,0x0a01002f,0x09060008,0x0905000a,0x0905000b,
0x00000000,0x0d030049,0x09060008,0x0905000b,0x10030052,0x0905000b,0x0905000b,0x09090009,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d03000b,0x05020003,0x05010004,0x03030003,
0x0e0100a4,0x0a03000a,0x04010003,0x06030004,0x0a020011,0x06020005,0x05010005,0x04040004,
0x00000000,0x10030094,0x05020003,0x0a03000e,0x0a010030,0x05020004,0x05010006,0x1006000f,
0x10020125,0x0a020019,0x05020004,0x10050010,0x10030053,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0d040007,0x100300cb,0x0d03000c,0x06030004,0x06020005,0x10060010,
0x10010324,0x1004002c,0x0d040008,0x06030005,0x0a020012,0x06020007,0x06020006,0x0d070009,
0x00000000,0x10020193,0x06030004,0x10050019,0x0a010031,0x06030005,0x06020007,0x0d06000a,
0x00000000,0x0d03004a,0x06030005,0x10050011,0x10030054,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0d03000c,0x02020002,0x0d03000d,0x0a040006,0x0a030007,0x03030003,
0x00000000,0x0a03000b,0x0d03000d,0x1006000f,0x0a020013,0x10050010,0x0a030008,0x0d06000e,
0x00000000,0x01010001,0x0a040006,0x0d040027,0x0a010032,0x0a040007,0x0a030009,0x10060010,
0x00000000,0x0a02001a,0x0a040007,0x10050012,0x10030055,0x10050011,0x0a03000a,0x0a070008,
0x00000000,0x01010001,0x0a030007,0x02020002,0x0d03000e,0x1006000f,0x10050010,0x0d050016,
0x00000000,0x1004002d,0x0a030008,0x10060010,0x0a020014,0x10050012,0x10050011,0x0a060009,
0x00000000,0x01010001,0x1006000f,0x0d030066,0x0a010033,0x10060010,0x10050012,0x0d06000b,
0x00000000,0x0d03004b,0x10060010,0x10050013,0x10030056,0x10050013,0x10050013,0x10090011,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d03000f,0x10040028,0x10030029,0x03030003,
0x0e0100a5,0x0a03000c,0x04010003,0x0a040007,0x0a020015,0x0a030008,0x1003002a,0x04040004,
0x00000000,0x01010001,0x10040028,0x0a03000f,0x0a010034,0x10040029,0x1003002b,0x0d070009,
0x10020126,0x0a02001b,0x10040029,0x0d06000a,0x10030057,0x0a030009,0x1003002c,0x1007002a,
0x00000000,0x01010001,0x0d040008,0x02020002,0x0d030010,0x0a040007,0x0a030008,0x0d040035,
0x10010325,0x1004002e,0x0d040009,0x0a040008,0x0a020016,0x0a03000a,0x0a030009,0x0d07000a,
0x00000000,0x01010001,0x0a040007,0x1005001a,0x0a010035,0x0a040008,0x0a03000a,0x0d06000c,
0x00000000,0x0d03004c,0x0a040008,0x0d06000b,0x10030058,0x0a03000b,0x0a03000b,0x0a070009,
0x00000000,0x01010001,0x0d02001b,0x02020002,0x0d030011,0x0a030009,0x0a02000a,0x03030003,
0x00000000,0x0a03000d,0x0d02001c,0x0d070009,0x0a020017,0x0d06000a,0x0a02000b,0x0d05001d,
0x00000000,0x01010001,0x0a030009,0x0d040028,0x0a010036,0x0a03000a,0x0a02000c,0x0d07000a,
0x00000000,0x0a02001c,0x0a03000a,0x0d06000c,0x10030059,0x0d06000b,0x0a02000d,0x0a06000b,
0x00000000,0x01010001,0x0a030008,0x02020002,0x0d030012,0x0d070009,0x0d06000a,0x0e040055,
0x00000000,0x1004002f,0x0a030009,0x0d07000a,0x0a020018,0x0d06000c,0x0d06000b,0x0a06000a,
0x00000000,0x01010001,0x0d070009,0x0d030067,0x0a010037,0x0d07000a,0x0d06000c,0x0d06000d,
0x00000000,0x0d03004d,0x0d07000a,0x0d06000d,0x1003005a,0x0d06000d,0x0d06000d,0x0d0a000b,
0x00000000,0x01010001,0x04010002,0x02020002,0x0a03000a,0x05020003,0x05010004,0x03030003,
0x0e0100a6,0x0a03000e,0x04010003,0x10050017,0x0a020019,0x10040018,0x05010005,0x04040004,
0x00000000,0x10030095,0x05020003,0x0a030010,0x0a010038,0x05020004,0x05010006,0x0d06000a,
0x10020127,0x0a02001d,0x05020004,0x0d05000b,0x1003005b,0x10040019,0x05010007,0x05050005,
0x00000000,0x01010001,0x0a040006,0x02020002,0x0a03000b,0x10050017,0x10040018,0x10060011,
0x10010326,0x10040030,0x0a040007,0x10050018,0x0a02001a,0x1004001a,0x10040019,0x0a070008,
0x00000000,0x10020194,0x10050017,0x1005001b,0x0a010039,0x10050018,0x1004001a,0x0a060009,
0x00000000,0x0d03004e,0x10050018,0x0d05000c,0x1003005c,0x1004001b,0x1004001b,0x10080019,
0x00000000,0x01010001,0x0e02002b,0x02020002,0x0a03000c,0x0a040007,0x0a030008,0x03030003,
0x00000000,0x0a03000f,0x0e02002c,0x0d06000a,0x0a02001b,0x0d05000b,0x0a030009,0x0e05002d,
0x00000000,0x01010001,0x0a040007,0x0d040029,0x0a01003a,0x0a040008,0x0a03000a,0x0d06000b,
0x00000000,0x0a02001e,0x0a040008,0x0d05000d,0x1003005d,0x0d05000c,0x0a03000b,0x0a070009,
0x00000000,0x01010001,0x0a030009,0x02020002,0x0a03000d,0x0d06000a,0x0d05000b,0x0d050017,
0x00000000,0x10040031,0x0a03000a,0x0d06000b,0x0a02001c,0x0d05000d,0x0d05000c,0x0a06000b,
0x00000000,0x01010001,0x0d06000a,0x0d030068,0x0a01003b,0x0d06000b,0x0d05000d,0x0a06000a,
0x00000000,0x0d03004f,0x0d06000b,0x0d05000e,0x1003005e,0x0d05000e,0x0d05000e,0x0d09000c,
0x00000000,0x01010001,0x04010002,0x02020002,0x0a03000e,0x10030067,0x10020068,0x03030003,
0x0e0100a7,0x0a030010,0x04010003,0x1006000c,0x0a02001d,0x1005000d,0x10020069,0x04040004,
0x00000000,0x01010001,0x10030067,0x0a030011,0x0a01003c,0x10030068,0x1002006a,0x0a070008,
0x10020128,0x0a02001f,0x10030068,0x0a060009,0x1003005f,0x1005000e,0x1002006b,0x10060069,
0x00000000,0x01010001,0x0a040007,0x02020002,0x0a03000f,0x1006000c,0x1005000d,0x0d040036,
0x10010327,0x10040032,0x0a040008,0x1006000d,0x0a02001e,0x1005000f,0x1005000e,0x0a070009,
0x00000000,0x01010001,0x1006000c,0x1005001c,0x0a01003d,0x1006000d,0x1005000f,0x0a06000b,
0x00000000,0x0d030050,0x1006000d,0x0a06000a,0x10030060,0x10050010,0x10050010,0x1009000e,
0x00000000,0x01010001,0x1003004c,0x02020002,0x0a030010,0x0a03000a,0x0a02000b,0x03030003,
0x00000000,0x0a030011,0x1003004d,0x0a070008,0x0a02001f,0x0a060009,0x0a02000c,0x1006004e,
0x00000000,0x01010001,0x0a03000a,0x0d04002a,0x0a01003e,0x0a03000b,0x0a02000d,0x0a070009,
0x00000000,0x0a020020,0x0a03000b,0x0a06000b,0x10030061,0x0a06000a,0x0a02000e,0x0a06000c,
0x00000000,0x01010001,0x0a03000a,0x02020002,0x0a030011,0x0a070008,0x0a060009,0x0e040056,
0x00000000,0x10040033,0x0a03000b,0x0a070009,0x0a020020,0x0a06000b,0x0a06000a,0x0a06000c,
0x00000000,0x01010001,0x0a070008,0x0d030069,0x0a01003f,0x0a070009,0x0a06000b,0x0a06000c,
0x00000000,0x0d030051,0x0a070009,0x0a06000c,0x10030062,0x0a06000c,0x0a06000c,0x0a0a000a,
0x00000000,0x01010001,0x04010002,0x02020002,0x08010008,0x05020003,0x05010004,0x03030003,
0x0e0100a8,0x0d03000c,0x04010003,0x06030004,0x0d020013,0x06020005,0x05010005,0x04040004,
0x00000000,0x10030096,0x05020003,0x1006000e,0x10040024,0x05020004,0x05010006,0x07040005,
0x10020129,0x10050015,0x05020004,0x07030006,0x10030063,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x08020004,0x100300cc,0x08010009,0x06030004,0x06020005,0x10060012,
0x10010328,0x10040034,0x08020005,0x06030005,0x0d020014,0x06020007,0x06020006,0x08050006,
0x00000000,0x10020195,0x06030004,0x1005001d,0x10040025,0x06030005,0x06020007,0x08040007,
0x00000000,0x0d030052,0x06030005,0x07030007,0x10030064,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0d03000d,0x02020002,0x0801000a,0x0d040008,0x0d030009,0x03030003,
0x00000000,0x0d03000d,0x0d03000e,0x07040005,0x0d020015,0x07030006,0x0d03000a,0x0d06000f,
0x00000000,0x01010001,0x0d040008,0x0d04002b,0x10040026,0x0d040009,0x0d03000b,0x07040006,
0x00000000,0x10050016,0x0d040009,0x07030008,0x10030065,0x07030007,0x0d03000c,0x0d07000a,
0x00000000,0x01010001,0x0d030009,0x02020002,0x0801000b,0x07040005,0x07030006,0x0d050018,
0x00000000,0x10040035,0x0d03000a,0x07040006,0x0d020016,0x07030008,0x07030007,0x0d06000b,
0x00000000,0x01010001,0x07040005,0x0d03006a,0x10040027,0x07040006,0x07030008,0x08040008,
0x00000000,0x0d030053,0x07040006,0x07030009,0x10030066,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0801000c,0x0d030010,0x0d020011,0x03030003,
0x0e0100a9,0x0d03000e,0x04010003,0x1007000a,0x0d020017,0x1006000b,0x0d020012,0x04040004,
0x00000000,0x01010001,0x0d030010,0x1006000f,0x10040028,0x0d030011,0x0d020013,0x08050006,
0x1002012a,0x10050017,0x0d030011,0x08040007,0x10030067,0x1006000c,0x0d020014,0x0d060012,
0x00000000,0x01010001,0x08020005,0x02020002,0x0801000d,0x1007000a,0x1006000b,0x0d040037,
0x10010329,0x10040036,0x08020006,0x1007000b,0x0d020018,0x1006000d,0x1006000c,0x08050007,
0x00000000,0x01010001,0x1007000a,0x1005001e,0x10040029,0x1007000b,0x1006000d,0x08040009,
0x00000000,0x0d030054,0x1007000b,0x08040008,0x10030068,0x1006000e,0x1006000e,0x100a000c,
0x00000000,0x01010001,0x0d02001c,0x02020002,0x0801000e,0x1006000b,0x1005000c,0x03030003,
0x00000000,0x0d03000f,0x0d02001d,0x08050006,0x0d020019,0x08040007,0x1005000d,0x0d05001e,
0x00000000,0x01010001,0x1006000b,0x0d04002c,0x1004002a,0x1006000c,0x1005000e,0x08050007,
0x00000000,0x10050018,0x1006000c,0x08040009,0x10030069,0x08040008,0x1005000f,0x1009000d,
0x00000000,0x01010001,0x0d03000a,0x02020002,0x0801000f,0x08050006,0x08040007,0x0e040057,
0x00000000,0x10040037,0x0d03000b,0x08050007,0x0d02001a,0x08040009,0x08040008,0x0d06000c,
0x00000000,0x01010001,0x08050006,0x0d03006b,0x1004002b,0x08050007,0x08040009,0x0804000a,
0x00000000,0x0d030055,0x08050007,0x0804000a,0x1003006a,0x0804000a,0x0804000a,0x08080008,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d03000c,0x05020003,0x05010004,0x03030003,
0x0e0100aa,0x0d030010,0x04010003,0x10040036,0x0d02001b,0x10030037,0x05010005,0x04040004,
0x00000000,0x10030097,0x05020003,0x10060010,0x1004002c,0x05020004,0x05010006,0x1007000b,
0x1002012b,0x10050019,0x05020004,0x1006000c,0x1003006b,0x10030038,0x05010007,0x05050005,
0x00000000,0x01010001,0x0d040008,0x02020002,0x0d03000d,0x10040036,0x10030037,0x10060013,
0x1001032a,0x10040038,0x0d040009,0x10040037,0x0d02001c,0x10030039,0x10030038,0x0d07000a,
0x00000000,0x10020196,0x10040036,0x1005001f,0x1004002d,0x10040037,0x10030039,0x0d06000b,
0x00000000,0x0d030056,0x10040037,0x1006000d,0x1003006c,0x1003003a,0x1003003a,0x10070038,
0x00000000,0x01010001,0x0e02002c,0x02020002,0x0d03000e,0x0d040009,0x0d03000a,0x03030003,
0x00000000,0x0d030011,0x0e02002d,0x1007000b,0x0d02001d,0x1006000c,0x0d03000b,0x0e05002e,
0x00000000,0x01010001,0x0d040009,0x0d04002d,0x1004002e,0x0d04000a,0x0d03000c,0x1007000c,
0x00000000,0x1005001a,0x0d04000a,0x1006000e,0x1003006d,0x1006000d,0x0d03000d,0x0d07000b,
0x00000000,0x01010001,0x0d03000b,0x02020002,0x0d03000f,0x1007000b,0x1006000c,0x0d050019,
0x00000000,0x10040039,0x0d03000c,0x1007000c,0x0d02001e,0x1006000e,0x1006000d,0x0d06000d,
0x00000000,0x01010001,0x1007000b,0x0d03006c,0x1004002f,0x1007000c,0x1006000e,0x0d06000c,
0x00000000,0x0d030057,0x1007000c,0x1006000f,0x1003006e,0x1006000f,0x1006000f,0x100a000d,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d030010,0x0e030018,0x0e020019,0x03030003,
0x0e0100ab,0x0d030012,0x04010003,0x1006000d,0x0d02001f,0x1005000e,0x0e02001a,0x04040004,
0x00000000,0x01010001,0x0e030018,0x10060011,0x10040030,0x0e030019,0x0e02001b,0x0d07000a,
0x1002012c,0x1005001b,0x0e030019,0x0d06000b,0x1003006f,0x1005000f,0x0e02001c,0x0e06001a,
0x00000000,0x01010001,0x0d040009,0x02020002,0x0d030011,0x1006000d,0x1005000e,0x0d040038,
0x1001032b,0x1004003a,0x0d04000a,0x1006000e,0x0d020020,0x10050010,0x1005000f,0x0d07000b,
0x00000000,0x01010001,0x1006000d,0x10050020,0x10040031,0x1006000e,0x10050010,0x0d06000d,
0x00000000,0x0d030058,0x1006000e,0x0d06000c,0x10030070,0x10050011,0x10050011,0x1009000f,
0x00000000,0x01010001,0x1003004d,0x02020002,0x0d030012,0x1006000c,0x1005000d,0x03030003,
0x00000000,0x0d030013,0x1003004e,0x0d07000a,0x0d020021,0x0d06000b,0x1005000e,0x1006004f,
0x00000000,0x01010001,0x1006000c,0x0d04002e,0x10040032,0x1006000d,0x1005000f,0x0d07000b,
0x00000000,0x1005001c,0x1006000d,0x0d06000d,0x10030071,0x0d06000c,0x10050010,0x1009000e,
0x00000000,0x01010001,0x0d03000c,0x02020002,0x0d030013,0x0d07000a,0x0d06000b,0x0e040058,
0x00000000,0x1004003b,0x0d03000d,0x0d07000b,0x0d020022,0x0d06000d,0x0d06000c,0x0d06000e,
0x00000000,0x01010001,0x0d07000a,0x0d03006d,0x10040033,0x0d07000b,0x0d06000d,0x0d06000e,
0x00000000,0x0d030059,0x0d07000b,0x0d06000e,0x10030072,0x0d06000e,0x0d06000e,0x0d0a000c,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d02000e,0x05020003,0x05010004,0x03030003,
0x0e0100ac,0x1006000e,0x04010003,0x06030004,0x10050015,0x06020005,0x05010005,0x04040004,
0x00000000,0x10030098,0x05020003,0x10060012,0x10040034,0x05020004,0x05010006,0x1005001e,
0x1002012d,0x1005001d,0x05020004,0x1004001f,0x10030073,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0d03000a,0x100300cd,0x0d02000f,0x06030004,0x06020005,0x10060014,
0x1001032c,0x1004003c,0x0d03000b,0x06030005,0x10050016,0x06020007,0x06020006,0x0d06000c,
0x00000000,0x10020197,0x06030004,0x10050021,0x10040035,0x06030005,0x06020007,0x0d05000d,
0x00000000,0x0d03005a,0x06030005,0x10040020,0x10030074,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0d03000e,0x02020002,0x0d020010,0x1007000a,0x1006000b,0x03030003,
0x00000000,0x1006000f,0x0d03000f,0x1005001e,0x10050017,0x1004001f,0x1006000c,0x0d060010,
0x00000000,0x01010001,0x1007000a,0x0d04002f,0x10040036,0x1007000b,0x1006000d,0x1005001f,
0x00000000,0x1005001e,0x1007000b,0x10040021,0x10030075,0x10040020,0x1006000e,0x100a000c,
0x00000000,0x01010001,0x1006000b,0x02020002,0x0d020011,0x1005001e,0x1004001f,0x0d05001a,
0x00000000,0x1004003d,0x1006000c,0x1005001f,0x10050018,0x10040021,0x10040020,0x1009000d,
0x00000000,0x01010001,0x1005001e,0x0d03006e,0x10040037,0x1005001f,0x10040021,0x0d05000e,
0x00000000,0x0d03005b,0x1005001f,0x10040022,0x10030076,0x10040022,0x10040022,0x10080020,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d020012,0x10040029,0x1003002a,0x03030003,
0x0e0100ad,0x10060010,0x04010003,0x1007000b,0x10050019,0x1006000c,0x1003002b,0x04040004,
0x00000000,0x01010001,0x10040029,0x10060013,0x10040038,0x1004002a,0x1003002c,0x0d06000c,
0x1002012e,0x1005001f,0x1004002a,0x0d05000d,0x10030077,0x1006000d,0x1003002d,0x1007002b,
0x00000000,0x01010001,0x0d03000b,0x02020002,0x0d020013,0x1007000b,0x1006000c,0x0d040039,
0x1001032d,0x1004003e,0x0d03000c,0x1007000c,0x1005001a,0x1006000e,0x1006000d,0x0d06000d,
0x00000000,0x01010001,0x1007000b,0x10050022,0x10040039,0x1007000c,0x1006000e,0x0d05000f,
0x00000000,0x0d03005c,0x1007000c,0x0d05000e,0x10030078,0x1006000f,0x1006000f,0x100a000d,
0x00000000,0x01010001,0x0d02001d,0x02020002,0x0d020014,0x1006000d,0x1005000e,0x03030003,
0x00000000,0x10060011,0x0d02001e,0x0d06000c,0x1005001b,0x0d05000d,0x1005000f,0x0d05001f,
0x00000000,0x01010001,0x1006000d,0x0d040030,0x1004003a,0x1006000e,0x10050010,0x0d06000d,
0x00000000,0x10050020,0x1006000e,0x0d05000f,0x10030079,0x0d05000e,0x10050011,0x1009000f,
0x00000000,0x01010001,0x1006000c,0x02020002,0x0d020015,0x0d06000c,0x0d05000d,0x0e040059,
0x00000000,0x1004003f,0x1006000d,0x0d06000d,0x1005001c,0x0d05000f,0x0d05000e,0x1009000e,
0x00000000,0x01010001,0x0d06000c,0x0d03006f,0x1004003b,0x0d06000d,0x0d05000f,0x0d050010,
0x00000000,0x0d03005d,0x0d06000d,0x0d050010,0x1003007a,0x0d050010,0x0d050010,0x0d09000e,
0x00000000,0x01010001,0x04010002,0x02020002,0x1006000e,0x05020003,0x05010004,0x03030003,
0x0e0100ae,0x10060012,0x04010003,0x06030004,0x1005001d,0x06020005,0x05010005,0x04040004,
0x00000000,0x10030099,0x05020003,0x10060014,0x1004003c,0x05020004,0x05010006,0x0d06000b,
0x1002012f,0x10050021,0x05020004,0x0d05000c,0x1003007b,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x1007000a,0x02020002,0x1006000f,0x06030004,0x06020005,0x10060015,
0x1001032e,0x10040040,0x1007000b,0x06030005,0x1005001e,0x06020007,0x06020006,0x100a000c,
0x00000000,0x10020198,0x06030004,0x10050023,0x1004003d,0x06030005,0x06020007,0x1009000d,
0x00000000,0x0d03005e,0x06030005,0x0d05000d,0x1003007c,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0e02002d,0x02020002,0x10060010,0x1007000b,0x1006000c,0x03030003,
0x00000000,0x10060013,0x0e02002e,0x0d06000b,0x1005001f,0x0d05000c,0x1006000d,0x0e05002f,
0x00000000,0x01010001,0x1007000b,0x0d040031,0x1004003e,0x1007000c,0x1006000e,0x0d06000c,
0x00000000,0x10050022,0x1007000c,0x0d05000e,0x1003007d,0x0d05000d,0x1006000f,0x100a000d,
0x00000000,0x01010001,0x1006000d,0x02020002,0x10060011,0x0d06000b,0x0d05000c,0x0d05001b,
0x00000000,0x10040041,0x1006000e,0x0d06000c,0x10050020,0x0d05000e,0x0d05000d,0x1009000f,
0x00000000,0x01010001,0x0d06000b,0x0d030070,0x1004003f,0x0d06000c,0x0d05000e,0x1009000e,
0x00000000,0x0d03005f,0x0d06000c,0x0d05000f,0x1003007e,0x0d05000f,0x0d05000f,0x0d09000d,
0x00000000,0x01010001,0x04010002,0x02020002,0x10060012,0x10030068,0x10020069,0x03030003,
0x0e0100af,0x10060014,0x04010003,0x1006000e,0x10050021,0x1005000f,0x1002006a,0x04040004,
0x00000000,0x01010001,0x10030068,0x10060015,0x10040040,0x10030069,0x1002006b,0x100a000c,
0x10020130,0x10050023,0x10030069,0x1009000d,0x1003007f,0x10050010,0x1002006c,0x1006006a,
0x00000000,0x01010001,0x1007000b,0x02020002,0x10060013,0x1006000e,0x1005000f,0x0d04003a,
0x1001032f,0x10040042,0x1007000c,0x1006000f,0x10050022,0x10050011,0x10050010,0x100a000d,
0x00000000,0x01010001,0x1006000e,0x10050024,0x10040041,0x1006000f,0x10050011,0x1009000f,
0x00000000,0x0d030060,0x1006000f,0x1009000e,0x10030080,0x10050012,0x10050012,0x10090010,
0x00000000,0x01010001,0x1003004e,0x02020002,0x10060014,0x1006000e,0x1005000f,0x03030003,
0x00000000,0x10060015,0x1003004f,0x100a000c,0x10050023,0x1009000d,0x10050010,0x10060050,
0x00000000,0x01010001,0x1006000e,0x0d040032,0x10040042,0x1006000f,0x10050011,0x100a000d,
0x00000000,0x10050024,0x1006000f,0x1009000f,0x10030081,0x1009000e,0x10050012,0x10090010,
0x00000000,0x01010001,0x1006000e,0x02020002,0x10060015,0x100a000c,0x1009000d,0x0e04005a,
0x00000000,0x10040043,0x1006000f,0x100a000d,0x10050024,0x1009000f,0x1009000e,0x10090010,
0x00000000,0x01010001,0x100a000c,0x0d030071,0x10040043,0x100a000d,0x1009000f,0x10090010,
0x00000000,0x0d030061,0x100a000d,0x10090010,0x10030082,0x10090010,0x10090010,0x100d000e,
0x00000000,0x01010001,0x04010002,0x02020002,0x08010008,0x05020003,0x05010004,0x03030003,
0x0e0100b0,0x09020009,0x04010003,0x06030004,0x09010010,0x06020005,0x05010005,0x04040004,
0x00000000,0x1003009a,0x05020003,0x1005000f,0x10030025,0x05020004,0x05010006,0x07040005,
0x10020131,0x10040016,0x05020004,0x07030006,0x0d030042,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x08020004,0x100300ce,0x08010009,0x06030004,0x06020005,0x0d06000d,
0x10010330,0x0d040023,0x08020005,0x06030005,0x09010011,0x06020007,0x06020006,0x08050006,
0x00000000,0x10020199,0x06030004,0x0d050014,0x10030026,0x06030005,0x06020007,0x08040007,
0x00000000,0x0d030062,0x06030005,0x07030007,0x0d030043,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0d03000f,0x02020002,0x0801000a,0x09030005,0x09020006,0x03030003,
0x00000000,0x0902000a,0x0d030010,0x07040005,0x09010012,0x07030006,0x09020007,0x0d060011,
0x00000000,0x01010001,0x09030005,0x0d040033,0x10030027,0x09030006,0x09020008,0x07040006,
0x00000000,0x10040017,0x09030006,0x07030008,0x0d030044,0x07030007,0x09020009,0x09060007,
0x00000000,0x01010001,0x09020006,0x02020002,0x0801000b,0x07040005,0x07030006,0x0d05001c,
0x00000000,0x0d040024,0x09020007,0x07040006,0x09010013,0x07030008,0x07030007,0x09050008,
0x00000000,0x01010001,0x07040005,0x0d030072,0x10030028,0x07040006,0x07030008,0x08040008,
0x00000000,0x0d030063,0x07040006,0x07030009,0x0d030045,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0801000c,0x0d030011,0x0d020012,0x03030003,
0x0e0100b1,0x0902000b,0x04010003,0x1006000b,0x09010014,0x1005000c,0x0d020013,0x04040004,
0x00000000,0x01010001,0x0d030011,0x10050010,0x10030029,0x0d030012,0x0d020014,0x08050006,
0x10020132,0x10040018,0x0d030012,0x08040007,0x0d030046,0x1005000d,0x0d020015,0x0d060013,
0x00000000,0x01010001,0x08020005,0x02020002,0x0801000d,0x1006000b,0x1005000c,0x0d04003b,
0x10010331,0x0d040025,0x08020006,0x1006000c,0x09010015,0x1005000e,0x1005000d,0x08050007,
0x00000000,0x01010001,0x1006000b,0x0d050015,0x1003002a,0x1006000c,0x1005000e,0x08040009,
0x00000000,0x0d030064,0x1006000c,0x08040008,0x0d030047,0x1005000f,0x1005000f,0x1009000d,
0x00000000,0x01010001,0x0d02001e,0x02020002,0x0801000e,0x1005000c,0x1004000d,0x03030003,
0x00000000,0x0902000c,0x0d02001f,0x08050006,0x09010016,0x08040007,0x1004000e,0x0d050020,
0x00000000,0x01010001,0x1005000c,0x0d040034,0x1003002b,0x1005000d,0x1004000f,0x08050007,
0x00000000,0x10040019,0x1005000d,0x08040009,0x0d030048,0x08040008,0x10040010,0x1008000e,
0x00000000,0x01010001,0x09020007,0x02020002,0x0801000f,0x08050006,0x08040007,0x0e04005b,
0x00000000,0x0d040026,0x09020008,0x08050007,0x09010017,0x08040009,0x08040008,0x09050009,
0x00000000,0x01010001,0x08050006,0x0d030073,0x1003002c,0x08050007,0x08040009,0x0804000a,
0x00000000,0x0d030065,0x08050007,0x0804000a,0x0d030049,0x0804000a,0x0804000a,0x08080008,
0x00000000,0x01010001,0x04010002,0x02020002,0x09020009,0x05020003,0x05010004,0x03030003,
0x0e0100b2,0x0902000d,0x04010003,0x0e04000f,0x09010018,0x0e030010,0x05010005,0x04040004,
0x00000000,0x1003009b,0x05020003,0x10050011,0x1003002d,0x05020004,0x05010006,0x0d070009,
0x10020133,0x1004001a,0x05020004,0x0d06000a,0x0d03004a,0x0e030011,0x05010007,0x05050005,
0x00000000,0x01010001,0x09030005,0x02020002,0x0902000a,0x0e04000f,0x0e030010,0x0d06000e,
0x10010332,0x0d040027,0x09030006,0x0e040010,0x09010019,0x0e030012,0x0e030011,0x09060007,
0x00000000,0x1002019a,0x0e04000f,0x0d050016,0x1003002e,0x0e040010,0x0e030012,0x09050008,
0x00000000,0x0d030066,0x0e040010,0x0d06000b,0x0d03004b,0x0e030013,0x0e030013,0x0e070011,
0x00000000,0x01010001,0x0e02002e,0x02020002,0x0902000b,0x09030006,0x09020007,0x03030003,
0x00000000,0x0902000e,0x0e02002f,0x0d070009,0x0901001a,0x0d06000a,0x09020008,0x0e050030,
0x00000000,0x01010001,0x09030006,0x0d040035,0x1003002f,0x09030007,0x09020009,0x0d07000a,
0x00000000,0x1004001b,0x09030007,0x0d06000c,0x0d03004c,0x0d06000b,0x0902000a,0x09060008,
0x00000000,0x01010001,0x09020008,0x02020002,0x0902000c,0x0d070009,0x0d06000a,0x0d05001d,
0x00000000,0x0d040028,0x09020009,0x0d07000a,0x0901001b,0x0d06000c,0x0d06000b,0x0905000a,
0x00000000,0x01010001,0x0d070009,0x0d030074,0x10030030,0x0d07000a,0x0d06000c,0x09050009,
0x00000000,0x0d030067,0x0d07000a,0x0d06000d,0x0d03004d,0x0d06000d,0x0d06000d,0x0d0a000b,
0x00000000,0x01010001,0x04010002,0x02020002,0x0902000d,0x0e030019,0x0e02001a,0x03030003,
0x0e0100b3,0x0902000f,0x04010003,0x0d06000a,0x0901001c,0x0d05000b,0x0e02001b,0x04040004,
0x00000000,0x01010001,0x0e030019,0x10050012,0x10030031,0x0e03001a,0x0e02001c,0x09060007,
0x10020134,0x1004001c,0x0e03001a,0x09050008,0x0d03004e,0x0d05000c,0x0e02001d,0x0e06001b,
0x00000000,0x01010001,0x09030006,0x02020002,0x0902000e,0x0d06000a,0x0d05000b,0x0d04003c,
0x10010333,0x0d040029,0x09030007,0x0d06000b,0x0901001d,0x0d05000d,0x0d05000c,0x09060008,
0x00000000,0x01010001,0x0d06000a,0x0d050017,0x10030032,0x0d06000b,0x0d05000d,0x0905000a,
0x00000000,0x0d030068,0x0d06000b,0x09050009,0x0d03004f,0x0d05000e,0x0d05000e,0x0d09000c,
0x00000000,0x01010001,0x1003004f,0x02020002,0x0902000f,0x1005000d,0x1004000e,0x03030003,
0x00000000,0x09020010,0x10030050,0x09060007,0x0901001e,0x09050008,0x1004000f,0x10060051,
0x00000000,0x01010001,0x1005000d,0x0d040036,0x10030033,0x1005000e,0x10040010,0x09060008,
0x00000000,0x1004001d,0x1005000e,0x0905000a,0x0d030050,0x09050009,0x10040011,0x1008000f,
0x00000000,0x01010001,0x09020009,0x02020002,0x09020010,0x09060007,0x09050008,0x0e04005c,
0x00000000,0x0d04002a,0x0902000a,0x09060008,0x0901001f,0x0905000a,0x09050009,0x0905000b,
0x00000000,0x01010001,0x09060007,0x0d030075,0x10030034,0x09060008,0x0905000a,0x0905000b,
0x00000000,0x0d030069,0x09060008,0x0905000b,0x0d030051,0x0905000b,0x0905000b,0x09090009,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d03000c,0x05020003,0x05010004,0x03030003,
0x0e0100b4,0x1005000f,0x04010003,0x06030004,0x10040016,0x06020005,0x05010005,0x04040004,
0x00000000,0x1003009c,0x05020003,0x10050013,0x10030035,0x05020004,0x05010006,0x07040005,
0x10020135,0x1004001e,0x05020004,0x07030006,0x0d030052,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0d040008,0x100300cf,0x0d03000d,0x06030004,0x06020005,0x0d06000f,
0x10010334,0x0d04002b,0x0d040009,0x06030005,0x10040017,0x06020007,0x06020006,0x0d07000a,
0x00000000,0x1002019b,0x06030004,0x0d050018,0x10030036,0x06030005,0x06020007,0x0d06000b,
0x00000000,0x0d03006a,0x06030005,0x07030007,0x0d030053,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0d030010,0x02020002,0x0d03000e,0x1006000b,0x1005000c,0x03030003,
0x00000000,0x10050010,0x0d030011,0x07040005,0x10040018,0x07030006,0x1005000d,0x0d060012,
0x00000000,0x01010001,0x1006000b,0x0d040037,0x10030037,0x1006000c,0x1005000e,0x07040006,
0x00000000,0x1004001f,0x1006000c,0x07030008,0x0d030054,0x07030007,0x1005000f,0x1009000d,
0x00000000,0x01010001,0x1005000c,0x02020002,0x0d03000f,0x07040005,0x07030006,0x0d05001e,
0x00000000,0x0d04002c,0x1005000d,0x07040006,0x10040019,0x07030008,0x07030007,0x1008000e,
0x00000000,0x01010001,0x07040005,0x0d030076,0x10030038,0x07040006,0x07030008,0x0d06000c,
0x00000000,0x0d03006b,0x07040006,0x07030009,0x0d030055,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d030010,0x1004002a,0x1003002b,0x03030003,
0x0e0100b5,0x10050011,0x04010003,0x1006000c,0x1004001a,0x1005000d,0x1003002c,0x04040004,
0x00000000,0x01010001,0x1004002a,0x10050014,0x10030039,0x1004002b,0x1003002d,0x0d07000a,
0x10020136,0x10040020,0x1004002b,0x0d06000b,0x0d030056,0x1005000e,0x1003002e,0x1007002c,
0x00000000,0x01010001,0x0d040009,0x02020002,0x0d030011,0x1006000c,0x1005000d,0x0d04003d,
0x10010335,0x0d04002d,0x0d04000a,0x1006000d,0x1004001b,0x1005000f,0x1005000e,0x0d07000b,
0x00000000,0x01010001,0x1006000c,0x0d050019,0x1003003a,0x1006000d,0x1005000f,0x0d06000d,
0x00000000,0x0d03006c,0x1006000d,0x0d06000c,0x0d030057,0x10050010,0x10050010,0x1009000e,
0x00000000,0x01010001,0x0d02001f,0x02020002,0x0d030012,0x1005000e,0x1004000f,0x03030003,
0x00000000,0x10050012,0x0d020020,0x0d07000a,0x1004001c,0x0d06000b,0x10040010,0x0d050021,
0x00000000,0x01010001,0x1005000e,0x0d040038,0x1003003b,0x1005000f,0x10040011,0x0d07000b,
0x00000000,0x10040021,0x1005000f,0x0d06000d,0x0d030058,0x0d06000c,0x10040012,0x10080010,
0x00000000,0x01010001,0x1005000d,0x02020002,0x0d030013,0x0d07000a,0x0d06000b,0x0e04005d,
0x00000000,0x0d04002e,0x1005000e,0x0d07000b,0x1004001d,0x0d06000d,0x0d06000c,0x1008000f,
0x00000000,0x01010001,0x0d07000a,0x0d030077,0x1003003c,0x0d07000b,0x0d06000d,0x0d06000e,
0x00000000,0x0d03006d,0x0d07000b,0x0d06000e,0x0d030059,0x0d06000e,0x0d06000e,0x0d0a000c,
0x00000000,0x01010001,0x04010002,0x02020002,0x1005000f,0x05020003,0x05010004,0x03030003,
0x0e0100b6,0x10050013,0x04010003,0x10050018,0x1004001e,0x10040019,0x05010005,0x04040004,
0x00000000,0x1003009d,0x05020003,0x10050015,0x1003003d,0x05020004,0x05010006,0x0d06000c,
0x10020137,0x10040022,0x05020004,0x0d05000d,0x0d03005a,0x1004001a,0x05010007,0x05050005,
0x00000000,0x01010001,0x1006000b,0x02020002,0x10050010,0x10050018,0x10040019,0x0d060010,
0x10010336,0x0d04002f,0x1006000c,0x10050019,0x1004001f,0x1004001b,0x1004001a,0x1009000d,
0x00000000,0x1002019c,0x10050018,0x0d05001a,0x1003003e,0x10050019,0x1004001b,0x1008000e,
0x00000000,0x0d03006e,0x10050019,0x0d05000e,0x0d03005b,0x1004001c,0x1004001c,0x1008001a,
0x00000000,0x01010001,0x0e02002f,0x02020002,0x10050011,0x1006000c,0x1005000d,0x03030003,
0x00000000,0x10050014,0x0e020030,0x0d06000c,0x10040020,0x0d05000d,0x1005000e,0x0e050031,
0x00000000,0x01010001,0x1006000c,0x0d040039,0x1003003f,0x1006000d,0x1005000f,0x0d06000d,
0x00000000,0x10040023,0x1006000d,0x0d05000f,0x0d03005c,0x0d05000e,0x10050010,0x1009000e,
0x00000000,0x01010001,0x1005000e,0x02020002,0x10050012,0x0d06000c,0x0d05000d,0x0d05001f,
0x00000000,0x0d040030,0x1005000f,0x0d06000d,0x10040021,0x0d05000f,0x0d05000e,0x10080010,
0x00000000,0x01010001,0x0d06000c,0x0d030078,0x10030040,0x0d06000d,0x0d05000f,0x1008000f,
0x00000000,0x0d03006f,0x0d06000d,0x0d050010,0x0d03005d,0x0d050010,0x0d050010,0x0d09000e,
0x00000000,0x01010001,0x04010002,0x02020002,0x10050013,0x10030069,0x1002006a,0x03030003,
0x0e0100b7,0x10050015,0x04010003,0x0d06000b,0x10040022,0x0d05000c,0x1002006b,0x04040004,
0x00000000,0x01010001,0x10030069,0x10050016,0x10030041,0x1003006a,0x1002006c,0x1009000d,
0x10020138,0x10040024,0x1003006a,0x1008000e,0x0d03005e,0x0d05000d,0x1002006d,0x1006006b,
0x00000000,0x01010001,0x1006000c,0x02020002,0x10050014,0x0d06000b,0x0d05000c,0x0d04003e,
0x10010337,0x0d040031,0x1006000d,0x0d06000c,0x10040023,0x0d05000e,0x0d05000d,0x1009000e,
0x00000000,0x01010001,0x0d06000b,0x0d05001b,0x10030042,0x0d06000c,0x0d05000e,0x10080010,
0x00000000,0x0d030070,0x0d06000c,0x1008000f,0x0d03005f,0x0d05000f,0x0d05000f,0x0d09000d,
0x00000000,0x01010001,0x10030050,0x02020002,0x10050015,0x1005000f,0x10040010,0x03030003,
0x00000000,0x10050016,0x10030051,0x1009000d,0x10040024,0x1008000e,0x10040011,0x10060052,
0x00000000,0x01010001,0x1005000f,0x0d04003a,0x10030043,0x10050010,0x10040012,0x1009000e,
0x00000000,0x10040025,0x10050010,0x10080010,0x0d030060,0x1008000f,0x10040013,0x10080011,
0x00000000,0x01010001,0x1005000f,0x02020002,0x10050016,0x1009000d,0x1008000e,0x0e04005e,
0x00000000,0x0d040032,0x10050010,0x1009000e,0x10040025,0x10080010,0x1008000f,0x10080011,
0x00000000,0x01010001,0x1009000d,0x0d030079,0x10030044,0x1009000e,0x10080010,0x10080011,
0x00000000,0x0d030071,0x1009000e,0x10080011,0x0d030061,0x10080011,0x10080011,0x100c000f,
0x00000000,0x01010001,0x04010002,0x02020002,0x08010008,0x05020003,0x05010004,0x03030003,
0x0e0100b8,0x0e03000e,0x04010003,0x06030004,0x0e020015,0x06020005,0x05010005,0x04040004,
0x00000000,0x1003009e,0x05020003,0x0d06000d,0x0d040023,0x05020004,0x05010006,0x07040005,
0x10020139,0x0d050014,0x05020004,0x07030006,0x0d030062,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x08020004,0x100300d0,0x08010009,0x06030004,0x06020005,0x0d060011,
0x10010338,0x0d040033,0x08020005,0x06030005,0x0e020016,0x06020007,0x06020006,0x08050006,
0x00000000,0x1002019d,0x06030004,0x0d05001c,0x0d040024,0x06030005,0x06020007,0x08040007,
0x00000000,0x0d030072,0x06030005,0x07030007,0x0d030063,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0d030011,0x02020002,0x0801000a,0x0e04000a,0x0e03000b,0x03030003,
0x00000000,0x0e03000f,0x0d030012,0x07040005,0x0e020017,0x07030006,0x0e03000c,0x0d060013,
0x00000000,0x01010001,0x0e04000a,0x0d04003b,0x0d040025,0x0e04000b,0x0e03000d,0x07040006,
0x00000000,0x0d050015,0x0e04000b,0x07030008,0x0d030064,0x07030007,0x0e03000e,0x0e07000c,
0x00000000,0x01010001,0x0e03000b,0x02020002,0x0801000b,0x07040005,0x07030006,0x0d050020,
0x00000000,0x0d040034,0x0e03000c,0x07040006,0x0e020018,0x07030008,0x07030007,0x0e06000d,
0x00000000,0x01010001,0x07040005,0x0d03007a,0x0d040026,0x07040006,0x07030008,0x08040008,
0x00000000,0x0d030073,0x07040006,0x07030009,0x0d030065,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0801000c,0x0d030012,0x0d020013,0x03030003,
0x0e0100b9,0x0e030010,0x04010003,0x0d070009,0x0e020019,0x0d06000a,0x0d020014,0x04040004,
0x00000000,0x01010001,0x0d030012,0x0d06000e,0x0d040027,0x0d030013,0x0d020015,0x08050006,
0x1002013a,0x0d050016,0x0d030013,0x08040007,0x0d030066,0x0d06000b,0x0d020016,0x0d060014,
0x00000000,0x01010001,0x08020005,0x02020002,0x0801000d,0x0d070009,0x0d06000a,0x0d04003f,
0x10010339,0x0d040035,0x08020006,0x0d07000a,0x0e02001a,0x0d06000c,0x0d06000b,0x08050007,
0x00000000,0x01010001,0x0d070009,0x0d05001d,0x0d040028,0x0d07000a,0x0d06000c,0x08040009,
0x00000000,0x0d030074,0x0d07000a,0x08040008,0x0d030067,0x0d06000d,0x0d06000d,0x0d0a000b,
0x00000000,0x01010001,0x0d020020,0x02020002,0x0801000e,0x0d06000a,0x0d05000b,0x03030003,
0x00000000,0x0e030011,0x0d020021,0x08050006,0x0e02001b,0x08040007,0x0d05000c,0x0d050022,
0x00000000,0x01010001,0x0d06000a,0x0d04003c,0x0d040029,0x0d06000b,0x0d05000d,0x08050007,
0x00000000,0x0d050017,0x0d06000b,0x08040009,0x0d030068,0x08040008,0x0d05000e,0x0d09000c,
0x00000000,0x01010001,0x0e03000c,0x02020002,0x0801000f,0x08050006,0x08040007,0x0e04005f,
0x00000000,0x0d040036,0x0e03000d,0x08050007,0x0e02001c,0x08040009,0x08040008,0x0e06000e,
0x00000000,0x01010001,0x08050006,0x0d03007b,0x0d04002a,0x08050007,0x08040009,0x0804000a,
0x00000000,0x0d030075,0x08050007,0x0804000a,0x0d030069,0x0804000a,0x0804000a,0x08080008,
0x00000000,0x01010001,0x04010002,0x02020002,0x0e03000e,0x05020003,0x05010004,0x03030003,
0x0e0100ba,0x0e030012,0x04010003,0x10040037,0x0e02001d,0x10030038,0x05010005,0x04040004,
0x00000000,0x1003009f,0x05020003,0x0d06000f,0x0d04002b,0x05020004,0x05010006,0x0d07000a,
0x1002013b,0x0d050018,0x05020004,0x0d06000b,0x0d03006a,0x10030039,0x05010007,0x05050005,
0x00000000,0x01010001,0x0e04000a,0x02020002,0x0e03000f,0x10040037,0x10030038,0x0d060012,
0x1001033a,0x0d040037,0x0e04000b,0x10040038,0x0e02001e,0x1003003a,0x10030039,0x0e07000c,
0x00000000,0x1002019e,0x10040037,0x0d05001e,0x0d04002c,0x10040038,0x1003003a,0x0e06000d,
0x00000000,0x0d030076,0x10040038,0x0d06000c,0x0d03006b,0x1003003b,0x1003003b,0x10070039,
0x00000000,0x01010001,0x0e020030,0x02020002,0x0e030010,0x0e04000b,0x0e03000c,0x03030003,
0x00000000,0x0e030013,0x0e020031,0x0d07000a,0x0e02001f,0x0d06000b,0x0e03000d,0x0e050032,
0x00000000,0x01010001,0x0e04000b,0x0d04003d,0x0d04002d,0x0e04000c,0x0e03000e,0x0d07000b,
0x00000000,0x0d050019,0x0e04000c,0x0d06000d,0x0d03006c,0x0d06000c,0x0e03000f,0x0e07000d,
0x00000000,0x01010001,0x0e03000d,0x02020002,0x0e030011,0x0d07000a,0x0d06000b,0x0d050021,
0x00000000,0x0d040038,0x0e03000e,0x0d07000b,0x0e020020,0x0d06000d,0x0d06000c,0x0e06000f,
0x00000000,0x01010001,0x0d07000a,0x0d03007c,0x0d04002e,0x0d07000b,0x0d06000d,0x0e06000e,
0x00000000,0x0d030077,0x0d07000b,0x0d06000e,0x0d03006d,0x0d06000e,0x0d06000e,0x0d0a000c,
0x00000000,0x01010001,0x04010002,0x02020002,0x0e030012,0x0e03001a,0x0e02001b,0x03030003,
0x0e0100bb,0x0e030014,0x04010003,0x0d06000c,0x0e020021,0x0d05000d,0x0e02001c,0x04040004,
0x00000000,0x01010001,0x0e03001a,0x0d060010,0x0d04002f,0x0e03001b,0x0e02001d,0x0e07000c,
0x1002013c,0x0d05001a,0x0e03001b,0x0e06000d,0x0d03006e,0x0d05000e,0x0e02001e,0x0e06001c,
0x00000000,0x01010001,0x0e04000b,0x02020002,0x0e030013,0x0d06000c,0x0d05000d,0x0d040040,
0x1001033b,0x0d040039,0x0e04000c,0x0d06000d,0x0e020022,0x0d05000f,0x0d05000e,0x0e07000d,
0x00000000,0x01010001,0x0d06000c,0x0d05001f,0x0d040030,0x0d06000d,0x0d05000f,0x0e06000f,
0x00000000,0x0d030078,0x0d06000d,0x0e06000e,0x0d03006f,0x0d050010,0x0d050010,0x0d09000e,
0x00000000,0x01010001,0x10030051,0x02020002,0x0e030014,0x0d06000b,0x0d05000c,0x03030003,
0x00000000,0x0e030015,0x10030052,0x0e07000c,0x0e020023,0x0e06000d,0x0d05000d,0x10060053,
0x00000000,0x01010001,0x0d06000b,0x0d04003e,0x0d040031,0x0d06000c,0x0d05000e,0x0e07000d,
0x00000000,0x0d05001b,0x0d06000c,0x0e06000f,0x0d030070,0x0e06000e,0x0d05000f,0x0d09000d,
0x00000000,0x01010001,0x0e03000e,0x02020002,0x0e030015,0x0e07000c,0x0e06000d,0x0e040060,
0x00000000,0x0d04003a,0x0e03000f,0x0e07000d,0x0e020024,0x0e06000f,0x0e06000e,0x0e060010,
0x00000000,0x01010001,0x0e07000c,0x0d03007d,0x0d040032,0x0e07000d,0x0e06000f,0x0e060010,
0x00000000,0x0d030079,0x0e07000d,0x0e060010,0x0d030071,0x0e060010,0x0e060010,0x0e0a000e,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d02000f,0x05020003,0x05010004,0x03030003,
0x0e0100bc,0x0d06000d,0x04010003,0x06030004,0x0d050014,0x06020005,0x05010005,0x04040004,
0x00000000,0x100300a0,0x05020003,0x0d060011,0x0d040033,0x05020004,0x05010006,0x07040005,
0x1002013d,0x0d05001c,0x05020004,0x07030006,0x0d030072,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0d03000b,0x100300d1,0x0d020010,0x06030004,0x06020005,0x0d060013,
0x1001033c,0x0d04003b,0x0d03000c,0x06030005,0x0d050015,0x06020007,0x06020006,0x0d06000d,
0x00000000,0x1002019f,0x06030004,0x0d050020,0x0d040034,0x06030005,0x06020007,0x0d05000e,
0x00000000,0x0d03007a,0x06030005,0x07030007,0x0d030073,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0d030012,0x02020002,0x0d020011,0x0d070009,0x0d06000a,0x03030003,
0x00000000,0x0d06000e,0x0d030013,0x07040005,0x0d050016,0x07030006,0x0d06000b,0x0d060014,
0x00000000,0x01010001,0x0d070009,0x0d04003f,0x0d040035,0x0d07000a,0x0d06000c,0x07040006,
0x00000000,0x0d05001d,0x0d07000a,0x07030008,0x0d030074,0x07030007,0x0d06000d,0x0d0a000b,
0x00000000,0x01010001,0x0d06000a,0x02020002,0x0d020012,0x07040005,0x07030006,0x0d050022,
0x00000000,0x0d04003c,0x0d06000b,0x07040006,0x0d050017,0x07030008,0x07030007,0x0d09000c,
0x00000000,0x01010001,0x07040005,0x0d03007e,0x0d040036,0x07040006,0x07030008,0x0d05000f,
0x00000000,0x0d03007b,0x07040006,0x07030009,0x0d030075,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d020013,0x1004002b,0x1003002c,0x03030003,
0x0e0100bd,0x0d06000f,0x04010003,0x0d07000a,0x0d050018,0x0d06000b,0x1003002d,0x04040004,
0x00000000,0x01010001,0x1004002b,0x0d060012,0x0d040037,0x1004002c,0x1003002e,0x0d06000d,
0x1002013e,0x0d05001e,0x1004002c,0x0d05000e,0x0d030076,0x0d06000c,0x1003002f,0x1007002d,
0x00000000,0x01010001,0x0d03000c,0x02020002,0x0d020014,0x0d07000a,0x0d06000b,0x0d040041,
0x1001033d,0x0d04003d,0x0d03000d,0x0d07000b,0x0d050019,0x0d06000d,0x0d06000c,0x0d06000e,
0x00000000,0x01010001,0x0d07000a,0x0d050021,0x0d040038,0x0d07000b,0x0d06000d,0x0d050010,
0x00000000,0x0d03007c,0x0d07000b,0x0d05000f,0x0d030077,0x0d06000e,0x0d06000e,0x0d0a000c,
0x00000000,0x01010001,0x0d020021,0x02020002,0x0d020015,0x0d06000c,0x0d05000d,0x03030003,
0x00000000,0x0d060010,0x0d020022,0x0d06000d,0x0d05001a,0x0d05000e,0x0d05000e,0x0d050023,
0x00000000,0x01010001,0x0d06000c,0x0d040040,0x0d040039,0x0d06000d,0x0d05000f,0x0d06000e,
0x00000000,0x0d05001f,0x0d06000d,0x0d050010,0x0d030078,0x0d05000f,0x0d050010,0x0d09000e,
0x00000000,0x01010001,0x0d06000b,0x02020002,0x0d020016,0x0d06000d,0x0d05000e,0x0e040061,
0x00000000,0x0d04003e,0x0d06000c,0x0d06000e,0x0d05001b,0x0d050010,0x0d05000f,0x0d09000d,
0x00000000,0x01010001,0x0d06000d,0x0d03007f,0x0d04003a,0x0d06000e,0x0d050010,0x0d050011,
0x00000000,0x0d03007d,0x0d06000e,0x0d050011,0x0d030079,0x0d050011,0x0d050011,0x0d09000f,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d06000d,0x05020003,0x05010004,0x03030003,
0x0e0100be,0x0d060011,0x04010003,0x06030004,0x0d05001c,0x06020005,0x05010005,0x04040004,
0x00000000,0x100300a1,0x05020003,0x0d060013,0x0d04003b,0x05020004,0x05010006,0x0d06000d,
0x1002013f,0x0d050020,0x05020004,0x0d05000e,0x0d03007a,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0d070009,0x02020002,0x0d06000e,0x06030004,0x06020005,0x0d060014,
0x1001033e,0x0d04003f,0x0d07000a,0x06030005,0x0d05001d,0x06020007,0x06020006,0x0d0a000b,
0x00000000,0x100201a0,0x06030004,0x0d050022,0x0d04003c,0x06030005,0x06020007,0x0d09000c,
0x00000000,0x0d03007e,0x06030005,0x0d05000f,0x0d03007b,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0e020031,0x02020002,0x0d06000f,0x0d07000a,0x0d06000b,0x03030003,
0x00000000,0x0d060012,0x0e020032,0x0d06000d,0x0d05001e,0x0d05000e,0x0d06000c,0x0e050033,
0x00000000,0x01010001,0x0d07000a,0x0d040041,0x0d04003d,0x0d07000b,0x0d06000d,0x0d06000e,
0x00000000,0x0d050021,0x0d07000b,0x0d050010,0x0d03007c,0x0d05000f,0x0d06000e,0x0d0a000c,
0x00000000,0x01010001,0x0d06000c,0x02020002,0x0d060010,0x0d06000d,0x0d05000e,0x0d050023,
0x00000000,0x0d040040,0x0d06000d,0x0d06000e,0x0d05001f,0x0d050010,0x0d05000f,0x0d09000e,
0x00000000,0x01010001,0x0d06000d,0x0d030080,0x0d04003e,0x0d06000e,0x0d050010,0x0d09000d,
0x00000000,0x0d03007f,0x0d06000e,0x0d050011,0x0d03007d,0x0d050011,0x0d050011,0x0d09000f,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d060011,0x1003006a,0x1002006b,0x03030003,
0x0e0100bf,0x0d060013,0x04010003,0x0d06000d,0x0d050020,0x0d05000e,0x1002006c,0x04040004,
0x00000000,0x01010001,0x1003006a,0x0d060014,0x0d04003f,0x1003006b,0x1002006d,0x0d0a000b,
0x10020140,0x0d050022,0x1003006b,0x0d09000c,0x0d03007e,0x0d05000f,0x1002006e,0x1006006c,
0x00000000,0x01010001,0x0d07000a,0x02020002,0x0d060012,0x0d06000d,0x0d05000e,0x0d040042,
0x1001033f,0x0d040041,0x0d07000b,0x0d06000e,0x0d050021,0x0d050010,0x0d05000f,0x0d0a000c,
0x00000000,0x01010001,0x0d06000d,0x0d050023,0x0d040040,0x0d06000e,0x0d050010,0x0d09000e,
0x00000000,0x0d030080,0x0d06000e,0x0d09000d,0x0d03007f,0x0d050011,0x0d050011,0x0d09000f,
0x00000000,0x01010001,0x10030052,0x02020002,0x0d060013,0x0d06000d,0x0d05000e,0x03030003,
0x00000000,0x0d060014,0x10030053,0x0d0a000b,0x0d050022,0x0d09000c,0x0d05000f,0x10060054,
0x00000000,0x01010001,0x0d06000d,0x0d040042,0x0d040041,0x0d06000e,0x0d050010,0x0d0a000c,
0x00000000,0x0d050023,0x0d06000e,0x0d09000e,0x0d030080,0x0d09000d,0x0d050011,0x0d09000f,
0x00000000,0x01010001,0x0d06000d,0x02020002,0x0d060014,0x0d0a000b,0x0d09000c,0x0e040062,
0x00000000,0x0d040042,0x0d06000e,0x0d0a000c,0x0d050023,0x0d09000e,0x0d09000d,0x0d09000f,
0x00000000,0x01010001,0x0d0a000b,0x0d030081,0x0d040042,0x0d0a000c,0x0d09000e,0x0d09000f,
0x00000000,0x0d030081,0x0d0a000c,0x0d09000f,0x0d030081,0x0d09000f,0x0d09000f,0x0d0d000d,
0x00000000,0x01010001,0x04010002,0x02020002,0x08010008,0x05020003,0x05010004,0x03030003,
0x0e0100c0,0x09020009,0x04010003,0x06030004,0x09010010,0x06020005,0x05010005,0x04040004,
0x00000000,0x100300a2,0x05020003,0x0a03000a,0x0a010020,0x05020004,0x05010006,0x07040005,
0x10020141,0x0a020011,0x05020004,0x07030006,0x0b010040,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x08020004,0x100300d2,0x08010009,0x06030004,0x06020005,0x0b04000b,
0x10010340,0x0b020021,0x08020005,0x06030005,0x09010011,0x06020007,0x06020006,0x08050006,
0x00000000,0x100201a1,0x06030004,0x0b030012,0x0a010021,0x06030005,0x06020007,0x08040007,
0x00000000,0x10030044,0x06030005,0x07030007,0x0b010041,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x1003000d,0x02020002,0x0801000a,0x09030005,0x09020006,0x03030003,
0x00000000,0x0902000a,0x1003000e,0x07040005,0x09010012,0x07030006,0x09020007,0x1006000f,
0x00000000,0x01010001,0x09030005,0x10040025,0x0a010022,0x09030006,0x09020008,0x07040006,
0x00000000,0x0a020012,0x09030006,0x07030008,0x0b010042,0x07030007,0x09020009,0x09060007,
0x00000000,0x01010001,0x09020006,0x02020002,0x0801000b,0x07040005,0x07030006,0x10050016,
0x00000000,0x0b020022,0x09020007,0x07040006,0x09010013,0x07030008,0x07030007,0x09050008,
0x00000000,0x01010001,0x07040005,0x0e040043,0x0a010023,0x07040006,0x07030008,0x08040008,
0x00000000,0x10030045,0x07040006,0x07030009,0x0b010043,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0801000c,0x0e04000c,0x0e03000d,0x03030003,
0x0e0100c1,0x0902000b,0x04010003,0x0a040006,0x09010014,0x0a030007,0x0e03000e,0x04040004,
0x00000000,0x01010001,0x0e04000c,0x0a03000b,0x0a010024,0x0e04000d,0x0e03000f,0x08050006,
0x10020142,0x0a020013,0x0e04000d,0x08040007,0x0b010044,0x0a030008,0x0e030010,0x0e07000e,
0x00000000,0x01010001,0x08020005,0x02020002,0x0801000d,0x0a040006,0x0a030007,0x0e050024,
0x10010341,0x0b020023,0x08020006,0x0a040007,0x09010015,0x0a030009,0x0a030008,0x08050007,
0x00000000,0x01010001,0x0a040006,0x0b030013,0x0a010025,0x0a040007,0x0a030009,0x08040009,
0x00000000,0x10030046,0x0a040007,0x08040008,0x0b010045,0x0a03000a,0x0a03000a,0x0a070008,
0x00000000,0x01010001,0x0e030013,0x02020002,0x0801000e,0x0a030007,0x0a020008,0x03030003,
0x00000000,0x0902000c,0x0e030014,0x08050006,0x09010016,0x08040007,0x0a020009,0x0e060015,
0x00000000,0x01010001,0x0a030007,0x10040026,0x0a010026,0x0a030008,0x0a02000a,0x08050007,
0x00000000,0x0a020014,0x0a030008,0x08040009,0x0b010046,0x08040008,0x0a02000b,0x0a060009,
0x00000000,0x01010001,0x09020007,0x02020002,0x0801000f,0x08050006,0x08040007,0x0e040063,
0x00000000,0x0b020024,0x09020008,0x08050007,0x09010017,0x08040009,0x08040008,0x09050009,
0x00000000,0x01010001,0x08050006,0x0e040044,0x0a010027,0x08050007,0x08040009,0x0804000a,
0x00000000,0x10030047,0x08050007,0x0804000a,0x0b010047,0x0804000a,0x0804000a,0x08080008,
0x00000000,0x01010001,0x04010002,0x02020002,0x09020009,0x05020003,0x05010004,0x03030003,
0x0e0100c2,0x0902000d,0x04010003,0x0e040010,0x09010018,0x0e030011,0x05010005,0x04040004,
0x00000000,0x100300a3,0x05020003,0x0a03000c,0x0a010028,0x05020004,0x05010006,0x0b050007,
0x10020143,0x0a020015,0x05020004,0x0b040008,0x0b010048,0x0e030012,0x05010007,0x05050005,
0x00000000,0x01010001,0x09030005,0x02020002,0x0902000a,0x0e040010,0x0e030011,0x0b04000c,
0x10010342,0x0b020025,0x09030006,0x0e040011,0x09010019,0x0e030013,0x0e030012,0x09060007,
0x00000000,0x100201a2,0x0e040010,0x0b030014,0x0a010029,0x0e040011,0x0e030013,0x09050008,
0x00000000,0x10030048,0x0e040011,0x0b040009,0x0b010049,0x0e030014,0x0e030014,0x0e070012,
0x00000000,0x01010001,0x0e020032,0x02020002,0x0902000b,0x09030006,0x09020007,0x03030003,
0x00000000,0x0902000e,0x0e020033,0x0b050007,0x0901001a,0x0b040008,0x09020008,0x0e050034,
0x00000000,0x01010001,0x09030006,0x10040027,0x0a01002a,0x09030007,0x09020009,0x0b050008,
0x00000000,0x0a020016,0x09030007,0x0b04000a,0x0b01004a,0x0b040009,0x0902000a,0x09060008,
0x00000000,0x01010001,0x09020008,0x02020002,0x0902000c,0x0b050007,0x0b040008,0x10050017,
0x00000000,0x0b020026,0x09020009,0x0b050008,0x0901001b,0x0b04000a,0x0b040009,0x0905000a,
0x00000000,0x01010001,0x0b050007,0x0e040045,0x0a01002b,0x0b050008,0x0b04000a,0x09050009,
0x00000000,0x10030049,0x0b050008,0x0b04000b,0x0b01004b,0x0b04000b,0x0b04000b,0x0b080009,
0x00000000,0x01010001,0x04010002,0x02020002,0x0902000d,0x0e03001b,0x0e02001c,0x03030003,
0x0e0100c3,0x0902000f,0x04010003,0x0b040008,0x0901001c,0x0b030009,0x0e02001d,0x04040004,
0x00000000,0x01010001,0x0e03001b,0x0a03000d,0x0a01002c,0x0e03001c,0x0e02001e,0x09060007,
0x10020144,0x0a020017,0x0e03001c,0x09050008,0x0b01004c,0x0b03000a,0x0e02001f,0x0e06001d,
0x00000000,0x01010001,0x09030006,0x02020002,0x0902000e,0x0b040008,0x0b030009,0x0e050025,
0x10010343,0x0b020027,0x09030007,0x0b040009,0x0901001d,0x0b03000b,0x0b03000a,0x09060008,
0x00000000,0x01010001,0x0b040008,0x0b030015,0x0a01002d,0x0b040009,0x0b03000b,0x0905000a,
0x00000000,0x1003004a,0x0b040009,0x09050009,0x0b01004d,0x0b03000c,0x0b03000c,0x0b07000a,
0x00000000,0x01010001,0x10030053,0x02020002,0x0902000f,0x0a030008,0x0a020009,0x03030003,
0x00000000,0x09020010,0x10030054,0x09060007,0x0901001e,0x09050008,0x0a02000a,0x10060055,
0x00000000,0x01010001,0x0a030008,0x10040028,0x0a01002e,0x0a030009,0x0a02000b,0x09060008,
0x00000000,0x0a020018,0x0a030009,0x0905000a,0x0b01004e,0x09050009,0x0a02000c,0x0a06000a,
0x00000000,0x01010001,0x09020009,0x02020002,0x09020010,0x09060007,0x09050008,0x0e040064,
0x00000000,0x0b020028,0x0902000a,0x09060008,0x0901001f,0x0905000a,0x09050009,0x0905000b,
0x00000000,0x01010001,0x09060007,0x0e040046,0x0a01002f,0x09060008,0x0905000a,0x0905000b,
0x00000000,0x1003004b,0x09060008,0x0905000b,0x0b01004f,0x0905000b,0x0905000b,0x09090009,
0x00000000,0x01010001,0x04010002,0x02020002,0x1003000d,0x05020003,0x05010004,0x03030003,
0x0e0100c4,0x0a03000a,0x04010003,0x06030004,0x0a020011,0x06020005,0x05010005,0x04040004,
0x00000000,0x100300a4,0x05020003,0x0a03000e,0x0a010030,0x05020004,0x05010006,0x10060010,
0x10020145,0x0a020019,0x05020004,0x10050011,0x0b010050,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x10040009,0x100300d3,0x1003000e,0x06030004,0x06020005,0x0b04000d,
0x10010344,0x0b020029,0x1004000a,0x06030005,0x0a020012,0x06020007,0x06020006,0x1007000b,
0x00000000,0x100201a3,0x06030004,0x0b030016,0x0a010031,0x06030005,0x06020007,0x1006000c,
0x00000000,0x1003004c,0x06030005,0x10050012,0x0b010051,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x1003000e,0x02020002,0x1003000f,0x0a040006,0x0a030007,0x03030003,
0x00000000,0x0a03000b,0x1003000f,0x10060010,0x0a020013,0x10050011,0x0a030008,0x10060010,
0x00000000,0x01010001,0x0a040006,0x10040029,0x0a010032,0x0a040007,0x0a030009,0x10060011,
0x00000000,0x0a02001a,0x0a040007,0x10050013,0x0b010052,0x10050012,0x0a03000a,0x0a070008,
0x00000000,0x01010001,0x0a030007,0x02020002,0x10030010,0x10060010,0x10050011,0x10050018,
0x00000000,0x0b02002a,0x0a030008,0x10060011,0x0a020014,0x10050013,0x10050012,0x0a060009,
0x00000000,0x01010001,0x10060010,0x0e040047,0x0a010033,0x10060011,0x10050013,0x1006000d,
0x00000000,0x1003004d,0x10060011,0x10050014,0x0b010053,0x10050014,0x10050014,0x10090012,
0x00000000,0x01010001,0x04010002,0x02020002,0x10030011,0x1004002c,0x1003002d,0x03030003,
0x0e0100c5,0x0a03000c,0x04010003,0x0a040007,0x0a020015,0x0a030008,0x1003002e,0x04040004,
0x00000000,0x01010001,0x1004002c,0x0a03000f,0x0a010034,0x1004002d,0x1003002f,0x1007000b,
0x10020146,0x0a02001b,0x1004002d,0x1006000c,0x0b010054,0x0a030009,0x10030030,0x1007002e,
0x00000000,0x01010001,0x1004000a,0x02020002,0x10030012,0x0a040007,0x0a030008,0x0e050026,
0x10010345,0x0b02002b,0x1004000b,0x0a040008,0x0a020016,0x0a03000a,0x0a030009,0x1007000c,
0x00000000,0x01010001,0x0a040007,0x0b030017,0x0a010035,0x0a040008,0x0a03000a,0x1006000e,
0x00000000,0x1003004e,0x0a040008,0x1006000d,0x0b010055,0x0a03000b,0x0a03000b,0x0a070009,
0x00000000,0x01010001,0x0e030014,0x02020002,0x10030013,0x0a030009,0x0a02000a,0x03030003,
0x00000000,0x0a03000d,0x0e030015,0x1007000b,0x0a020017,0x1006000c,0x0a02000b,0x0e060016,
0x00000000,0x01010001,0x0a030009,0x1004002a,0x0a010036,0x0a03000a,0x0a02000c,0x1007000c,
0x00000000,0x0a02001c,0x0a03000a,0x1006000e,0x0b010056,0x1006000d,0x0a02000d,0x0a06000b,
0x00000000,0x01010001,0x0a030008,0x02020002,0x10030014,0x1007000b,0x1006000c,0x0e040065,
0x00000000,0x0b02002c,0x0a030009,0x1007000c,0x0a020018,0x1006000e,0x1006000d,0x0a06000a,
0x00000000,0x01010001,0x1007000b,0x0e040048,0x0a010037,0x1007000c,0x1006000e,0x1006000f,
0x00000000,0x1003004f,0x1007000c,0x1006000f,0x0b010057,0x1006000f,0x1006000f,0x100a000d,
0x00000000,0x01010001,0x04010002,0x02020002,0x0a03000a,0x05020003,0x05010004,0x03030003,
0x0e0100c6,0x0a03000e,0x04010003,0x10050019,0x0a020019,0x1004001a,0x05010005,0x04040004,
0x00000000,0x100300a5,0x05020003,0x0a030010,0x0a010038,0x05020004,0x05010006,0x1006000c,
0x10020147,0x0a02001d,0x05020004,0x1005000d,0x0b010058,0x1004001b,0x05010007,0x05050005,
0x00000000,0x01010001,0x0a040006,0x02020002,0x0a03000b,0x10050019,0x1004001a,0x0b04000e,
0x10010346,0x0b02002d,0x0a040007,0x1005001a,0x0a02001a,0x1004001c,0x1004001b,0x0a070008,
0x00000000,0x100201a4,0x10050019,0x0b030018,0x0a010039,0x1005001a,0x1004001c,0x0a060009,
0x00000000,0x10030050,0x1005001a,0x1005000e,0x0b010059,0x1004001d,0x1004001d,0x1008001b,
0x00000000,0x01010001,0x0e020033,0x02020002,0x0a03000c,0x0a040007,0x0a030008,0x03030003,
0x00000000,0x0a03000f,0x0e020034,0x1006000c,0x0a02001b,0x1005000d,0x0a030009,0x0e050035,
0x00000000,0x01010001,0x0a040007,0x1004002b,0x0a01003a,0x0a040008,0x0a03000a,0x1006000d,
0x00000000,0x0a02001e,0x0a040008,0x1005000f,0x0b01005a,0x1005000e,0x0a03000b,0x0a070009,
0x00000000,0x01010001,0x0a030009,0x02020002,0x0a03000d,0x1006000c,0x1005000d,0x10050019,
0x00000000,0x0b02002e,0x0a03000a,0x1006000d,0x0a02001c,0x1005000f,0x1005000e,0x0a06000b,
0x00000000,0x01010001,0x1006000c,0x0e040049,0x0a01003b,0x1006000d,0x1005000f,0x0a06000a,
0x00000000,0x10030051,0x1006000d,0x10050010,0x0b01005b,0x10050010,0x10050010,0x1009000e,
0x00000000,0x01010001,0x04010002,0x02020002,0x0a03000e,0x1003006b,0x1002006c,0x03030003,
0x0e0100c7,0x0a030010,0x04010003,0x0b040009,0x0a02001d,0x0b03000a,0x1002006d,0x04040004,
0x00000000,0x01010001,0x1003006b,0x0a030011,0x0a01003c,0x1003006c,0x1002006e,0x0a070008,
0x10020148,0x0a02001f,0x1003006c,0x0a060009,0x0b01005c,0x0b03000b,0x1002006f,0x1006006d,
0x00000000,0x01010001,0x0a040007,0x02020002,0x0a03000f,0x0b040009,0x0b03000a,0x0e050027,
0x10010347,0x0b02002f,0x0a040008,0x0b04000a,0x0a02001e,0x0b03000c,0x0b03000b,0x0a070009,
0x00000000,0x01010001,0x0b040009,0x0b030019,0x0a01003d,0x0b04000a,0x0b03000c,0x0a06000b,
0x00000000,0x10030052,0x0b04000a,0x0a06000a,0x0b01005d,0x0b03000d,0x0b03000d,0x0b07000b,
0x00000000,0x01010001,0x10030054,0x02020002,0x0a030010,0x0a03000a,0x0a02000b,0x03030003,
0x00000000,0x0a030011,0x10030055,0x0a070008,0x0a02001f,0x0a060009,0x0a02000c,0x10060056,
0x00000000,0x01010001,0x0a03000a,0x1004002c,0x0a01003e,0x0a03000b,0x0a02000d,0x0a070009,
0x00000000,0x0a020020,0x0a03000b,0x0a06000b,0x0b01005e,0x0a06000a,0x0a02000e,0x0a06000c,
0x00000000,0x01010001,0x0a03000a,0x02020002,0x0a030011,0x0a070008,0x0a060009,0x0e040066,
0x00000000,0x0b020030,0x0a03000b,0x0a070009,0x0a020020,0x0a06000b,0x0a06000a,0x0a06000c,
0x00000000,0x01010001,0x0a070008,0x0e04004a,0x0a01003f,0x0a070009,0x0a06000b,0x0a06000c,
0x00000000,0x10030053,0x0a070009,0x0a06000c,0x0b01005f,0x0a06000c,0x0a06000c,0x0a0a000a,
0x00000000,0x01010001,0x04010002,0x02020002,0x10020015,0x05020003,0x05010004,0x03030003,
0x0e0100c8,0x0e04000c,0x04010003,0x06030004,0x0e030013,0x06020005,0x05010005,0x04040004,
0x00000000,0x100300a6,0x05020003,0x0b04000b,0x0b020021,0x05020004,0x05010006,0x07040005,
0x10020149,0x0b030012,0x05020004,0x07030006,0x0b010060,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x10030011,0x100300d4,0x10020016,0x06030004,0x06020005,0x0b04000f,
0x10010348,0x0b020031,0x10030012,0x06030005,0x0e030014,0x06020007,0x06020006,0x10060013,
0x00000000,0x100201a5,0x06030004,0x0b03001a,0x0b020022,0x06030005,0x06020007,0x10050014,
0x00000000,0x10030054,0x06030005,0x07030007,0x0b010061,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x1003000f,0x02020002,0x10020017,0x0e050008,0x0e040009,0x03030003,
0x00000000,0x0e04000d,0x10030010,0x07040005,0x0e030015,0x07030006,0x0e04000a,0x10060011,
0x00000000,0x01010001,0x0e050008,0x1004002d,0x0b020023,0x0e050009,0x0e04000b,0x07040006,
0x00000000,0x0b030013,0x0e050009,0x07030008,0x0b010062,0x07030007,0x0e04000c,0x0e08000a,
0x00000000,0x01010001,0x0e040009,0x02020002,0x10020018,0x07040005,0x07030006,0x1005001a,
0x00000000,0x0b020032,0x0e04000a,0x07040006,0x0e030016,0x07030008,0x07030007,0x0e07000b,
0x00000000,0x01010001,0x07040005,0x0e04004b,0x0b020024,0x07040006,0x07030008,0x10050015,
0x00000000,0x10030055,0x07040006,0x07030009,0x0b010063,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x10020019,0x0e04000d,0x0e03000e,0x03030003,
0x0e0100c9,0x0e04000e,0x04010003,0x0b050007,0x0e030017,0x0b040008,0x0e03000f,0x04040004,
0x00000000,0x01010001,0x0e04000d,0x0b04000c,0x0b020025,0x0e04000e,0x0e030010,0x10060013,
0x1002014a,0x0b030014,0x0e04000e,0x10050014,0x0b010064,0x0b040009,0x0e030011,0x0e07000f,
0x00000000,0x01010001,0x10030012,0x02020002,0x1002001a,0x0b050007,0x0b040008,0x0e050028,
0x10010349,0x0b020033,0x10030013,0x0b050008,0x0e030018,0x0b04000a,0x0b040009,0x10060014,
0x00000000,0x01010001,0x0b050007,0x0b03001b,0x0b020026,0x0b050008,0x0b04000a,0x10050016,
0x00000000,0x10030056,0x0b050008,0x10050015,0x0b010065,0x0b04000b,0x0b04000b,0x0b080009,
0x00000000,0x01010001,0x0e030015,0x02020002,0x1002001b,0x0b040008,0x0b030009,0x03030003,
0x00000000,0x0e04000f,0x0e030016,0x10060013,0x0e030019,0x10050014,0x0b03000a,0x0e060017,
0x00000000,0x01010001,0x0b040008,0x1004002e,0x0b020027,0x0b040009,0x0b03000b,0x10060014,
0x00000000,0x0b030015,0x0b040009,0x10050016,0x0b010066,0x10050015,0x0b03000c,0x0b07000a,
0x00000000,0x01010001,0x0e04000a,0x02020002,0x1002001c,0x10060013,0x10050014,0x0e040067,
0x00000000,0x0b020034,0x0e04000b,0x10060014,0x0e03001a,0x10050016,0x10050015,0x0e07000c,
0x00000000,0x01010001,0x10060013,0x0e04004c,0x0b020028,0x10060014,0x10050016,0x10050017,
0x00000000,0x10030057,0x10060014,0x10050017,0x0b010067,0x10050017,0x10050017,0x10090015,
0x00000000,0x01010001,0x04010002,0x02020002,0x0e04000c,0x05020003,0x05010004,0x03030003,
0x0e0100ca,0x0e040010,0x04010003,0x10040038,0x0e03001b,0x10030039,0x05010005,0x04040004,
0x00000000,0x100300a7,0x05020003,0x0b04000d,0x0b020029,0x05020004,0x05010006,0x0b050008,
0x1002014b,0x0b030016,0x05020004,0x0b040009,0x0b010068,0x1003003a,0x05010007,0x05050005,
0x00000000,0x01010001,0x0e050008,0x02020002,0x0e04000d,0x10040038,0x10030039,0x0b040010,
0x1001034a,0x0b020035,0x0e050009,0x10040039,0x0e03001c,0x1003003b,0x1003003a,0x0e08000a,
0x00000000,0x100201a6,0x10040038,0x0b03001c,0x0b02002a,0x10040039,0x1003003b,0x0e07000b,
0x00000000,0x10030058,0x10040039,0x0b04000a,0x0b010069,0x1003003c,0x1003003c,0x1007003a,
0x00000000,0x01010001,0x0e020034,0x02020002,0x0e04000e,0x0e050009,0x0e04000a,0x03030003,
0x00000000,0x0e040011,0x0e020035,0x0b050008,0x0e03001d,0x0b040009,0x0e04000b,0x0e050036,
0x00000000,0x01010001,0x0e050009,0x1004002f,0x0b02002b,0x0e05000a,0x0e04000c,0x0b050009,
0x00000000,0x0b030017,0x0e05000a,0x0b04000b,0x0b01006a,0x0b04000a,0x0e04000d,0x0e08000b,
0x00000000,0x01010001,0x0e04000b,0x02020002,0x0e04000f,0x0b050008,0x0b040009,0x1005001b,
0x00000000,0x0b020036,0x0e04000c,0x0b050009,0x0e03001e,0x0b04000b,0x0b04000a,0x0e07000d,
0x00000000,0x01010001,0x0b050008,0x0e04004d,0x0b02002c,0x0b050009,0x0b04000b,0x0e07000c,
0x00000000,0x10030059,0x0b050009,0x0b04000c,0x0b01006b,0x0b04000c,0x0b04000c,0x0b08000a,
0x00000000,0x01010001,0x04010002,0x02020002,0x0e040010,0x0e03001c,0x0e02001d,0x03030003,
0x0e0100cb,0x0e040012,0x04010003,0x0b04000a,0x0e03001f,0x0b03000b,0x0e02001e,0x04040004,
0x00000000,0x01010001,0x0e03001c,0x0b04000e,0x0b02002d,0x0e03001d,0x0e02001f,0x0e08000a,
0x1002014c,0x0b030018,0x0e03001d,0x0e07000b,0x0b01006c,0x0b03000c,0x0e020020,0x0e06001e,
0x00000000,0x01010001,0x0e050009,0x02020002,0x0e040011,0x0b04000a,0x0b03000b,0x0e050029,
0x1001034b,0x0b020037,0x0e05000a,0x0b04000b,0x0e030020,0x0b03000d,0x0b03000c,0x0e08000b,
0x00000000,0x01010001,0x0b04000a,0x0b03001d,0x0b02002e,0x0b04000b,0x0b03000d,0x0e07000d,
0x00000000,0x1003005a,0x0b04000b,0x0e07000c,0x0b01006d,0x0b03000e,0x0b03000e,0x0b07000c,
0x00000000,0x01010001,0x10030055,0x02020002,0x0e040012,0x0b040009,0x0b03000a,0x03030003,
0x00000000,0x0e040013,0x10030056,0x0e08000a,0x0e030021,0x0e07000b,0x0b03000b,0x10060057,
0x00000000,0x01010001,0x0b040009,0x10040030,0x0b02002f,0x0b04000a,0x0b03000c,0x0e08000b,
0x00000000,0x0b030019,0x0b04000a,0x0e07000d,0x0b01006e,0x0e07000c,0x0b03000d,0x0b07000b,
0x00000000,0x01010001,0x0e04000c,0x02020002,0x0e040013,0x0e08000a,0x0e07000b,0x0e040068,
0x00000000,0x0b020038,0x0e04000d,0x0e08000b,0x0e030022,0x0e07000d,0x0e07000c,0x0e07000e,
0x00000000,0x01010001,0x0e08000a,0x0e04004e,0x0b020030,0x0e08000b,0x0e07000d,0x0e07000e,
0x00000000,0x1003005b,0x0e08000b,0x0e07000e,0x0b01006f,0x0e07000e,0x0e07000e,0x0e0b000c,
0x00000000,0x01010001,0x04010002,0x02020002,0x0e03000d,0x05020003,0x05010004,0x03030003,
0x0e0100cc,0x0b04000b,0x04010003,0x06030004,0x0b030012,0x06020005,0x05010005,0x04040004,
0x00000000,0x100300a8,0x05020003,0x0b04000f,0x0b020031,0x05020004,0x05010006,0x1005001f,
0x1002014d,0x0b03001a,0x05020004,0x10040020,0x0b010070,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0e040009,0x100300d5,0x0e03000e,0x06030004,0x06020005,0x0b040011,
0x1001034c,0x0b020039,0x0e04000a,0x06030005,0x0b030013,0x06020007,0x06020006,0x0e07000b,
0x00000000,0x100201a7,0x06030004,0x0b03001e,0x0b020032,0x06030005,0x06020007,0x0e06000c,
0x00000000,0x1003005c,0x06030005,0x10040021,0x0b010071,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x10030010,0x02020002,0x0e03000f,0x0b050007,0x0b040008,0x03030003,
0x00000000,0x0b04000c,0x10030011,0x1005001f,0x0b030014,0x10040020,0x0b040009,0x10060012,
0x00000000,0x01010001,0x0b050007,0x10040031,0x0b020033,0x0b050008,0x0b04000a,0x10050020,
0x00000000,0x0b03001b,0x0b050008,0x10040022,0x0b010072,0x10040021,0x0b04000b,0x0b080009,
0x00000000,0x01010001,0x0b040008,0x02020002,0x0e030010,0x1005001f,0x10040020,0x1005001c,
0x00000000,0x0b02003a,0x0b040009,0x10050020,0x0b030015,0x10040022,0x10040021,0x0b07000a,
0x00000000,0x01010001,0x1005001f,0x0e04004f,0x0b020034,0x10050020,0x10040022,0x0e06000d,
0x00000000,0x1003005d,0x10050020,0x10040023,0x0b010073,0x10040023,0x10040023,0x10080021,
0x00000000,0x01010001,0x04010002,0x02020002,0x0e030011,0x1004002d,0x1003002e,0x03030003,
0x0e0100cd,0x0b04000d,0x04010003,0x0b050008,0x0b030016,0x0b040009,0x1003002f,0x04040004,
0x00000000,0x01010001,0x1004002d,0x0b040010,0x0b020035,0x1004002e,0x10030030,0x0e07000b,
0x1002014e,0x0b03001c,0x1004002e,0x0e06000c,0x0b010074,0x0b04000a,0x10030031,0x1007002f,
0x00000000,0x01010001,0x0e04000a,0x02020002,0x0e030012,0x0b050008,0x0b040009,0x0e05002a,
0x1001034d,0x0b02003b,0x0e04000b,0x0b050009,0x0b030017,0x0b04000b,0x0b04000a,0x0e07000c,
0x00000000,0x01010001,0x0b050008,0x0b03001f,0x0b020036,0x0b050009,0x0b04000b,0x0e06000e,
0x00000000,0x1003005e,0x0b050009,0x0e06000d,0x0b010075,0x0b04000c,0x0b04000c,0x0b08000a,
0x00000000,0x01010001,0x0e030016,0x02020002,0x0e030013,0x0b04000a,0x0b03000b,0x03030003,
0x00000000,0x0b04000e,0x0e030017,0x0e07000b,0x0b030018,0x0e06000c,0x0b03000c,0x0e060018,
0x00000000,0x01010001,0x0b04000a,0x10040032,0x0b020037,0x0b04000b,0x0b03000d,0x0e07000c,
0x00000000,0x0b03001d,0x0b04000b,0x0e06000e,0x0b010076,0x0e06000d,0x0b03000e,0x0b07000c,
0x00000000,0x01010001,0x0b040009,0x02020002,0x0e030014,0x0e07000b,0x0e06000c,0x0e040069,
0x00000000,0x0b02003c,0x0b04000a,0x0e07000c,0x0b030019,0x0e06000e,0x0e06000d,0x0b07000b,
0x00000000,0x01010001,0x0e07000b,0x0e040050,0x0b020038,0x0e07000c,0x0e06000e,0x0e06000f,
0x00000000,0x1003005f,0x0e07000c,0x0e06000f,0x0b010077,0x0e06000f,0x0e06000f,0x0e0a000d,
0x00000000,0x01010001,0x04010002,0x02020002,0x0b04000b,0x05020003,0x05010004,0x03030003,
0x0e0100ce,0x0b04000f,0x04010003,0x06030004,0x0b03001a,0x06020005,0x05010005,0x04040004,
0x00000000,0x100300a9,0x05020003,0x0b040011,0x0b020039,0x05020004,0x05010006,0x1006000d,
0x1002014f,0x0b03001e,0x05020004,0x1005000e,0x0b010078,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0b050007,0x02020002,0x0b04000c,0x06030004,0x06020005,0x0b040012,
0x1001034e,0x0b02003d,0x0b050008,0x06030005,0x0b03001b,0x06020007,0x06020006,0x0b080009,
0x00000000,0x100201a8,0x06030004,0x0b030020,0x0b02003a,0x06030005,0x06020007,0x0b07000a,
0x00000000,0x10030060,0x06030005,0x1005000f,0x0b010079,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0e020035,0x02020002,0x0b04000d,0x0b050008,0x0b040009,0x03030003,
0x00000000,0x0b040010,0x0e020036,0x1006000d,0x0b03001c,0x1005000e,0x0b04000a,0x0e050037,
0x00000000,0x01010001,0x0b050008,0x10040033,0x0b02003b,0x0b050009,0x0b04000b,0x1006000e,
0x00000000,0x0b03001f,0x0b050009,0x10050010,0x0b01007a,0x1005000f,0x0b04000c,0x0b08000a,
0x00000000,0x01010001,0x0b04000a,0x02020002,0x0b04000e,0x1006000d,0x1005000e,0x1005001d,
0x00000000,0x0b02003e,0x0b04000b,0x1006000e,0x0b03001d,0x10050010,0x1005000f,0x0b07000c,
0x00000000,0x01010001,0x1006000d,0x0e040051,0x0b02003c,0x1006000e,0x10050010,0x0b07000b,
0x00000000,0x10030061,0x1006000e,0x10050011,0x0b01007b,0x10050011,0x10050011,0x1009000f,
0x00000000,0x01010001,0x04010002,0x02020002,0x0b04000f,0x1003006c,0x1002006d,0x03030003,
0x0e0100cf,0x0b040011,0x04010003,0x0b04000b,0x0b03001e,0x0b03000c,0x1002006e,0x04040004,
0x00000000,0x01010001,0x1003006c,0x0b040012,0x0b02003d,0x1003006d,0x1002006f,0x0b080009,
0x10020150,0x0b030020,0x1003006d,0x0b07000a,0x0b01007c,0x0b03000d,0x10020070,0x1006006e,
0x00000000,0x01010001,0x0b050008,0x02020002,0x0b040010,0x0b04000b,0x0b03000c,0x0e05002b,
0x1001034f,0x0b02003f,0x0b050009,0x0b04000c,0x0b03001f,0x0b03000e,0x0b03000d,0x0b08000a,
0x00000000,0x01010001,0x0b04000b,0x0b030021,0x0b02003e,0x0b04000c,0x0b03000e,0x0b07000c,
0x00000000,0x10030062,0x0b04000c,0x0b07000b,0x0b01007d,0x0b03000f,0x0b03000f,0x0b07000d,
0x00000000,0x01010001,0x10030056,0x02020002,0x0b040011,0x0b04000b,0x0b03000c,0x03030003,
0x00000000,0x0b040012,0x10030057,0x0b080009,0x0b030020,0x0b07000a,0x0b03000d,0x10060058,
0x00000000,0x01010001,0x0b04000b,0x10040034,0x0b02003f,0x0b04000c,0x0b03000e,0x0b08000a,
0x00000000,0x0b030021,0x0b04000c,0x0b07000c,0x0b01007e,0x0b07000b,0x0b03000f,0x0b07000d,
0x00000000,0x01010001,0x0b04000b,0x02020002,0x0b040012,0x0b080009,0x0b07000a,0x0e04006a,
0x00000000,0x0b020040,0x0b04000c,0x0b08000a,0x0b030021,0x0b07000c,0x0b07000b,0x0b07000d,
0x00000000,0x01010001,0x0b080009,0x0e040052,0x0b020040,0x0b08000a,0x0b07000c,0x0b07000d,
0x00000000,0x10030063,0x0b08000a,0x0b07000d,0x0b01007f,0x0b07000d,0x0b07000d,0x0b0b000b,
0x00000000,0x01010001,0x04010002,0x02020002,0x08010008,0x05020003,0x05010004,0x03030003,
0x0e0100d0,0x09020009,0x04010003,0x06030004,0x09010010,0x06020005,0x05010005,0x04040004,
0x00000000,0x100300aa,0x05020003,0x0e04000d,0x0e020023,0x05020004,0x05010006,0x07040005,
0x10020151,0x0e030014,0x05020004,0x07030006,0x10030044,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x08020004,0x100300d6,0x08010009,0x06030004,0x06020005,0x1006000f,
0x10010350,0x10040025,0x08020005,0x06030005,0x09010011,0x06020007,0x06020006,0x08050006,
0x00000000,0x100201a9,0x06030004,0x10050016,0x0e020024,0x06030005,0x06020007,0x08040007,
0x00000000,0x10030064,0x06030005,0x07030007,0x10030045,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x10030011,0x02020002,0x0801000a,0x09030005,0x09020006,0x03030003,
0x00000000,0x0902000a,0x10030012,0x07040005,0x09010012,0x07030006,0x09020007,0x10060013,
0x00000000,0x01010001,0x09030005,0x10040035,0x0e020025,0x09030006,0x09020008,0x07040006,
0x00000000,0x0e030015,0x09030006,0x07030008,0x10030046,0x07030007,0x09020009,0x09060007,
0x00000000,0x01010001,0x09020006,0x02020002,0x0801000b,0x07040005,0x07030006,0x1005001e,
0x00000000,0x10040026,0x09020007,0x07040006,0x09010013,0x07030008,0x07030007,0x09050008,
0x00000000,0x01010001,0x07040005,0x0e040053,0x0e020026,0x07040006,0x07030008,0x08040008,
0x00000000,0x10030065,0x07040006,0x07030009,0x10030047,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0801000c,0x0e04000e,0x0e03000f,0x03030003,
0x0e0100d1,0x0902000b,0x04010003,0x0e050009,0x09010014,0x0e04000a,0x0e030010,0x04040004,
0x00000000,0x01010001,0x0e04000e,0x0e04000e,0x0e020027,0x0e04000f,0x0e030011,0x08050006,
0x10020152,0x0e030016,0x0e04000f,0x08040007,0x10030048,0x0e04000b,0x0e030012,0x0e070010,
0x00000000,0x01010001,0x08020005,0x02020002,0x0801000d,0x0e050009,0x0e04000a,0x0e05002c,
0x10010351,0x10040027,0x08020006,0x0e05000a,0x09010015,0x0e04000c,0x0e04000b,0x08050007,
0x00000000,0x01010001,0x0e050009,0x10050017,0x0e020028,0x0e05000a,0x0e04000c,0x08040009,
0x00000000,0x10030066,0x0e05000a,0x08040008,0x10030049,0x0e04000d,0x0e04000d,0x0e08000b,
0x00000000,0x01010001,0x0e030017,0x02020002,0x0801000e,0x0e04000a,0x0e03000b,0x03030003,
0x00000000,0x0902000c,0x0e030018,0x08050006,0x09010016,0x08040007,0x0e03000c,0x0e060019,
0x00000000,0x01010001,0x0e04000a,0x10040036,0x0e020029,0x0e04000b,0x0e03000d,0x08050007,
0x00000000,0x0e030017,0x0e04000b,0x08040009,0x1003004a,0x08040008,0x0e03000e,0x0e07000c,
0x00000000,0x01010001,0x09020007,0x02020002,0x0801000f,0x08050006,0x08040007,0x0e04006b,
0x00000000,0x10040028,0x09020008,0x08050007,0x09010017,0x08040009,0x08040008,0x09050009,
0x00000000,0x01010001,0x08050006,0x0e040054,0x0e02002a,0x08050007,0x08040009,0x0804000a,
0x00000000,0x10030067,0x08050007,0x0804000a,0x1003004b,0x0804000a,0x0804000a,0x08080008,
0x00000000,0x01010001,0x04010002,0x02020002,0x09020009,0x05020003,0x05010004,0x03030003,
0x0e0100d2,0x0902000d,0x04010003,0x0e040011,0x09010018,0x0e030012,0x05010005,0x04040004,
0x00000000,0x100300ab,0x05020003,0x0e04000f,0x0e02002b,0x05020004,0x05010006,0x1007000b,
0x10020153,0x0e030018,0x05020004,0x1006000c,0x1003004c,0x0e030013,0x05010007,0x05050005,
0x00000000,0x01010001,0x09030005,0x02020002,0x0902000a,0x0e040011,0x0e030012,0x10060010,
0x10010352,0x10040029,0x09030006,0x0e040012,0x09010019,0x0e030014,0x0e030013,0x09060007,
0x00000000,0x100201aa,0x0e040011,0x10050018,0x0e02002c,0x0e040012,0x0e030014,0x09050008,
0x00000000,0x10030068,0x0e040012,0x1006000d,0x1003004d,0x0e030015,0x0e030015,0x0e070013,
0x00000000,0x01010001,0x0e020036,0x02020002,0x0902000b,0x09030006,0x09020007,0x03030003,
0x00000000,0x0902000e,0x0e020037,0x1007000b,0x0901001a,0x1006000c,0x09020008,0x0e050038,
0x00000000,0x01010001,0x09030006,0x10040037,0x0e02002d,0x09030007,0x09020009,0x1007000c,
0x00000000,0x0e030019,0x09030007,0x1006000e,0x1003004e,0x1006000d,0x0902000a,0x09060008,
0x00000000,0x01010001,0x09020008,0x02020002,0x0902000c,0x1007000b,0x1006000c,0x1005001f,
0x00000000,0x1004002a,0x09020009,0x1007000c,0x0901001b,0x1006000e,0x1006000d,0x0905000a,
0x00000000,0x01010001,0x1007000b,0x0e040055,0x0e02002e,0x1007000c,0x1006000e,0x09050009,
0x00000000,0x10030069,0x1007000c,0x1006000f,0x1003004f,0x1006000f,0x1006000f,0x100a000d,
0x00000000,0x01010001,0x04010002,0x02020002,0x0902000d,0x0e03001d,0x0e02001e,0x03030003,
0x0e0100d3,0x0902000f,0x04010003,0x1006000c,0x0901001c,0x1005000d,0x0e02001f,0x04040004,
0x00000000,0x01010001,0x0e03001d,0x0e040010,0x0e02002f,0x0e03001e,0x0e020020,0x09060007,
0x10020154,0x0e03001a,0x0e03001e,0x09050008,0x10030050,0x1005000e,0x0e020021,0x0e06001f,
0x00000000,0x01010001,0x09030006,0x02020002,0x0902000e,0x1006000c,0x1005000d,0x0e05002d,
0x10010353,0x1004002b,0x09030007,0x1006000d,0x0901001d,0x1005000f,0x1005000e,0x09060008,
0x00000000,0x01010001,0x1006000c,0x10050019,0x0e020030,0x1006000d,0x1005000f,0x0905000a,
0x00000000,0x1003006a,0x1006000d,0x09050009,0x10030051,0x10050010,0x10050010,0x1009000e,
0x00000000,0x01010001,0x10030057,0x02020002,0x0902000f,0x0e04000b,0x0e03000c,0x03030003,
0x00000000,0x09020010,0x10030058,0x09060007,0x0901001e,0x09050008,0x0e03000d,0x10060059,
0x00000000,0x01010001,0x0e04000b,0x10040038,0x0e020031,0x0e04000c,0x0e03000e,0x09060008,
0x00000000,0x0e03001b,0x0e04000c,0x0905000a,0x10030052,0x09050009,0x0e03000f,0x0e07000d,
0x00000000,0x01010001,0x09020009,0x02020002,0x09020010,0x09060007,0x09050008,0x0e04006c,
0x00000000,0x1004002c,0x0902000a,0x09060008,0x0901001f,0x0905000a,0x09050009,0x0905000b,
0x00000000,0x01010001,0x09060007,0x0e040056,0x0e020032,0x09060008,0x0905000a,0x0905000b,
0x00000000,0x1003006b,0x09060008,0x0905000b,0x10030053,0x0905000b,0x0905000b,0x09090009,
0x00000000,0x01010001,0x04010002,0x02020002,0x1003000e,0x05020003,0x05010004,0x03030003,
0x0e0100d4,0x0e04000d,0x04010003,0x06030004,0x0e030014,0x06020005,0x05010005,0x04040004,
0x00000000,0x100300ac,0x05020003,0x0e040011,0x0e020033,0x05020004,0x05010006,0x07040005,
0x10020155,0x0e03001c,0x05020004,0x07030006,0x10030054,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x1004000a,0x100300d7,0x1003000f,0x06030004,0x06020005,0x10060011,
0x10010354,0x1004002d,0x1004000b,0x06030005,0x0e030015,0x06020007,0x06020006,0x1007000c,
0x00000000,0x100201ab,0x06030004,0x1005001a,0x0e020034,0x06030005,0x06020007,0x1006000d,
0x00000000,0x1003006c,0x06030005,0x07030007,0x10030055,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x10030012,0x02020002,0x10030010,0x0e050009,0x0e04000a,0x03030003,
0x00000000,0x0e04000e,0x10030013,0x07040005,0x0e030016,0x07030006,0x0e04000b,0x10060014,
0x00000000,0x01010001,0x0e050009,0x10040039,0x0e020035,0x0e05000a,0x0e04000c,0x07040006,
0x00000000,0x0e03001d,0x0e05000a,0x07030008,0x10030056,0x07030007,0x0e04000d,0x0e08000b,
0x00000000,0x01010001,0x0e04000a,0x02020002,0x10030011,0x07040005,0x07030006,0x10050020,
0x00000000,0x1004002e,0x0e04000b,0x07040006,0x0e030017,0x07030008,0x07030007,0x0e07000c,
0x00000000,0x01010001,0x07040005,0x0e040057,0x0e020036,0x07040006,0x07030008,0x1006000e,
0x00000000,0x1003006d,0x07040006,0x07030009,0x10030057,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x10030012,0x1004002e,0x1003002f,0x03030003,
0x0e0100d5,0x0e04000f,0x04010003,0x0e05000a,0x0e030018,0x0e04000b,0x10030030,0x04040004,
0x00000000,0x01010001,0x1004002e,0x0e040012,0x0e020037,0x1004002f,0x10030031,0x1007000c,
0x10020156,0x0e03001e,0x1004002f,0x1006000d,0x10030058,0x0e04000c,0x10030032,0x10070030,
0x00000000,0x01010001,0x1004000b,0x02020002,0x10030013,0x0e05000a,0x0e04000b,0x0e05002e,
0x10010355,0x1004002f,0x1004000c,0x0e05000b,0x0e030019,0x0e04000d,0x0e04000c,0x1007000d,
0x00000000,0x01010001,0x0e05000a,0x1005001b,0x0e020038,0x0e05000b,0x0e04000d,0x1006000f,
0x00000000,0x1003006e,0x0e05000b,0x1006000e,0x10030059,0x0e04000e,0x0e04000e,0x0e08000c,
0x00000000,0x01010001,0x0e030018,0x02020002,0x10030014,0x0e04000c,0x0e03000d,0x03030003,
0x00000000,0x0e040010,0x0e030019,0x1007000c,0x0e03001a,0x1006000d,0x0e03000e,0x0e06001a,
0x00000000,0x01010001,0x0e04000c,0x1004003a,0x0e020039,0x0e04000d,0x0e03000f,0x1007000d,
0x00000000,0x0e03001f,0x0e04000d,0x1006000f,0x1003005a,0x1006000e,0x0e030010,0x0e07000e,
0x00000000,0x01010001,0x0e04000b,0x02020002,0x10030015,0x1007000c,0x1006000d,0x0e04006d,
0x00000000,0x10040030,0x0e04000c,0x1007000d,0x0e03001b,0x1006000f,0x1006000e,0x0e07000d,
0x00000000,0x01010001,0x1007000c,0x0e040058,0x0e02003a,0x1007000d,0x1006000f,0x10060010,
0x00000000,0x1003006f,0x1007000d,0x10060010,0x1003005b,0x10060010,0x10060010,0x100a000e,
0x00000000,0x01010001,0x04010002,0x02020002,0x0e04000d,0x05020003,0x05010004,0x03030003,
0x0e0100d6,0x0e040011,0x04010003,0x1005001a,0x0e03001c,0x1004001b,0x05010005,0x04040004,
0x00000000,0x100300ad,0x05020003,0x0e040013,0x0e02003b,0x05020004,0x05010006,0x1006000e,
0x10020157,0x0e030020,0x05020004,0x1005000f,0x1003005c,0x1004001c,0x05010007,0x05050005,
0x00000000,0x01010001,0x0e050009,0x02020002,0x0e04000e,0x1005001a,0x1004001b,0x10060012,
0x10010356,0x10040031,0x0e05000a,0x1005001b,0x0e03001d,0x1004001d,0x1004001c,0x0e08000b,
0x00000000,0x100201ac,0x1005001a,0x1005001c,0x0e02003c,0x1005001b,0x1004001d,0x0e07000c,
0x00000000,0x10030070,0x1005001b,0x10050010,0x1003005d,0x1004001e,0x1004001e,0x1008001c,
0x00000000,0x01010001,0x0e020037,0x02020002,0x0e04000f,0x0e05000a,0x0e04000b,0x03030003,
0x00000000,0x0e040012,0x0e020038,0x1006000e,0x0e03001e,0x1005000f,0x0e04000c,0x0e050039,
0x00000000,0x01010001,0x0e05000a,0x1004003b,0x0e02003d,0x0e05000b,0x0e04000d,0x1006000f,
0x00000000,0x0e030021,0x0e05000b,0x10050011,0x1003005e,0x10050010,0x0e04000e,0x0e08000c,
0x00000000,0x01010001,0x0e04000c,0x02020002,0x0e040010,0x1006000e,0x1005000f,0x10050021,
0x00000000,0x10040032,0x0e04000d,0x1006000f,0x0e03001f,0x10050011,0x10050010,0x0e07000e,
0x00000000,0x01010001,0x1006000e,0x0e040059,0x0e02003e,0x1006000f,0x10050011,0x0e07000d,
0x00000000,0x10030071,0x1006000f,0x10050012,0x1003005f,0x10050012,0x10050012,0x10090010,
0x00000000,0x01010001,0x04010002,0x02020002,0x0e040011,0x1003006d,0x1002006e,0x03030003,
0x0e0100d7,0x0e040013,0x04010003,0x1006000d,0x0e030020,0x1005000e,0x1002006f,0x04040004,
0x00000000,0x01010001,0x1003006d,0x0e040014,0x0e02003f,0x1003006e,0x10020070,0x0e08000b,
0x10020158,0x0e030022,0x1003006e,0x0e07000c,0x10030060,0x1005000f,0x10020071,0x1006006f,
0x00000000,0x01010001,0x0e05000a,0x02020002,0x0e040012,0x1006000d,0x1005000e,0x0e05002f,
0x10010357,0x10040033,0x0e05000b,0x1006000e,0x0e030021,0x10050010,0x1005000f,0x0e08000c,
0x00000000,0x01010001,0x1006000d,0x1005001d,0x0e020040,0x1006000e,0x10050010,0x0e07000e,
0x00000000,0x10030072,0x1006000e,0x0e07000d,0x10030061,0x10050011,0x10050011,0x1009000f,
0x00000000,0x01010001,0x10030058,0x02020002,0x0e040013,0x0e04000d,0x0e03000e,0x03030003,
0x00000000,0x0e040014,0x10030059,0x0e08000b,0x0e030022,0x0e07000c,0x0e03000f,0x1006005a,
0x00000000,0x01010001,0x0e04000d,0x1004003c,0x0e020041,0x0e04000e,0x0e030010,0x0e08000c,
0x00000000,0x0e030023,0x0e04000e,0x0e07000e,0x10030062,0x0e07000d,0x0e030011,0x0e07000f,
0x00000000,0x01010001,0x0e04000d,0x02020002,0x0e040014,0x0e08000b,0x0e07000c,0x0e04006e,
0x00000000,0x10040034,0x0e04000e,0x0e08000c,0x0e030023,0x0e07000e,0x0e07000d,0x0e07000f,
0x00000000,0x01010001,0x0e08000b,0x0e04005a,0x0e020042,0x0e08000c,0x0e07000e,0x0e07000f,
0x00000000,0x10030073,0x0e08000c,0x0e07000f,0x10030063,0x0e07000f,0x0e07000f,0x0e0b000d,
0x00000000,0x01010001,0x04010002,0x02020002,0x08010008,0x05020003,0x05010004,0x03030003,
0x0e0100d8,0x0e03000f,0x04010003,0x06030004,0x0e020016,0x06020005,0x05010005,0x04040004,
0x00000000,0x100300ae,0x05020003,0x1006000f,0x10040025,0x05020004,0x05010006,0x07040005,
0x10020159,0x10050016,0x05020004,0x07030006,0x10030064,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x08020004,0x100300d8,0x08010009,0x06030004,0x06020005,0x10060013,
0x10010358,0x10040035,0x08020005,0x06030005,0x0e020017,0x06020007,0x06020006,0x08050006,
0x00000000,0x100201ad,0x06030004,0x1005001e,0x10040026,0x06030005,0x06020007,0x08040007,
0x00000000,0x10030074,0x06030005,0x07030007,0x10030065,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x10030013,0x02020002,0x0801000a,0x0e04000b,0x0e03000c,0x03030003,
0x00000000,0x0e030010,0x10030014,0x07040005,0x0e020018,0x07030006,0x0e03000d,0x10060015,
0x00000000,0x01010001,0x0e04000b,0x1004003d,0x10040027,0x0e04000c,0x0e03000e,0x07040006,
0x00000000,0x10050017,0x0e04000c,0x07030008,0x10030066,0x07030007,0x0e03000f,0x0e07000d,
0x00000000,0x01010001,0x0e03000c,0x02020002,0x0801000b,0x07040005,0x07030006,0x10050022,
0x00000000,0x10040036,0x0e03000d,0x07040006,0x0e020019,0x07030008,0x07030007,0x0e06000e,
0x00000000,0x01010001,0x07040005,0x0e04005b,0x10040028,0x07040006,0x07030008,0x08040008,
0x00000000,0x10030075,0x07040006,0x07030009,0x10030067,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0801000c,0x0e04000f,0x0e030010,0x03030003,
0x0e0100d9,0x0e030011,0x04010003,0x1007000b,0x0e02001a,0x1006000c,0x0e030011,0x04040004,
0x00000000,0x01010001,0x0e04000f,0x10060010,0x10040029,0x0e040010,0x0e030012,0x08050006,
0x1002015a,0x10050018,0x0e040010,0x08040007,0x10030068,0x1006000d,0x0e030013,0x0e070011,
0x00000000,0x01010001,0x08020005,0x02020002,0x0801000d,0x1007000b,0x1006000c,0x0e050030,
0x10010359,0x10040037,0x08020006,0x1007000c,0x0e02001b,0x1006000e,0x1006000d,0x08050007,
0x00000000,0x01010001,0x1007000b,0x1005001f,0x1004002a,0x1007000c,0x1006000e,0x08040009,
0x00000000,0x10030076,0x1007000c,0x08040008,0x10030069,0x1006000f,0x1006000f,0x100a000d,
0x00000000,0x01010001,0x0e030019,0x02020002,0x0801000e,0x1006000c,0x1005000d,0x03030003,
0x00000000,0x0e030012,0x0e03001a,0x08050006,0x0e02001c,0x08040007,0x1005000e,0x0e06001b,
0x00000000,0x01010001,0x1006000c,0x1004003e,0x1004002b,0x1006000d,0x1005000f,0x08050007,
0x00000000,0x10050019,0x1006000d,0x08040009,0x1003006a,0x08040008,0x10050010,0x1009000e,
0x00000000,0x01010001,0x0e03000d,0x02020002,0x0801000f,0x08050006,0x08040007,0x0e04006f,
0x00000000,0x10040038,0x0e03000e,0x08050007,0x0e02001d,0x08040009,0x08040008,0x0e06000f,
0x00000000,0x01010001,0x08050006,0x0e04005c,0x1004002c,0x08050007,0x08040009,0x0804000a,
0x00000000,0x10030077,0x08050007,0x0804000a,0x1003006b,0x0804000a,0x0804000a,0x08080008,
0x00000000,0x01010001,0x04010002,0x02020002,0x0e03000f,0x05020003,0x05010004,0x03030003,
0x0e0100da,0x0e030013,0x04010003,0x10040039,0x0e02001e,0x1003003a,0x05010005,0x04040004,
0x00000000,0x100300af,0x05020003,0x10060011,0x1004002d,0x05020004,0x05010006,0x1007000c,
0x1002015b,0x1005001a,0x05020004,0x1006000d,0x1003006c,0x1003003b,0x05010007,0x05050005,
0x00000000,0x01010001,0x0e04000b,0x02020002,0x0e030010,0x10040039,0x1003003a,0x10060014,
0x1001035a,0x10040039,0x0e04000c,0x1004003a,0x0e02001f,0x1003003c,0x1003003b,0x0e07000d,
0x00000000,0x100201ae,0x10040039,0x10050020,0x1004002e,0x1004003a,0x1003003c,0x0e06000e,
0x00000000,0x10030078,0x1004003a,0x1006000e,0x1003006d,0x1003003d,0x1003003d,0x1007003b,
0x00000000,0x01010001,0x0e020038,0x02020002,0x0e030011,0x0e04000c,0x0e03000d,0x03030003,
0x00000000,0x0e030014,0x0e020039,0x1007000c,0x0e020020,0x1006000d,0x0e03000e,0x0e05003a,
0x00000000,0x01010001,0x0e04000c,0x1004003f,0x1004002f,0x0e04000d,0x0e03000f,0x1007000d,
0x00000000,0x1005001b,0x0e04000d,0x1006000f,0x1003006e,0x1006000e,0x0e030010,0x0e07000e,
0x00000000,0x01010001,0x0e03000e,0x02020002,0x0e030012,0x1007000c,0x1006000d,0x10050023,
0x00000000,0x1004003a,0x0e03000f,0x1007000d,0x0e020021,0x1006000f,0x1006000e,0x0e060010,
0x00000000,0x01010001,0x1007000c,0x0e04005d,0x10040030,0x1007000d,0x1006000f,0x0e06000f,
0x00000000,0x10030079,0x1007000d,0x10060010,0x1003006f,0x10060010,0x10060010,0x100a000e,
0x00000000,0x01010001,0x04010002,0x02020002,0x0e030013,0x0e03001e,0x0e02001f,0x03030003,
0x0e0100db,0x0e030015,0x04010003,0x1006000e,0x0e020022,0x1005000f,0x0e020020,0x04040004,
0x00000000,0x01010001,0x0e03001e,0x10060012,0x10040031,0x0e03001f,0x0e020021,0x0e07000d,
0x1002015c,0x1005001c,0x0e03001f,0x0e06000e,0x10030070,0x10050010,0x0e020022,0x0e060020,
0x00000000,0x01010001,0x0e04000c,0x02020002,0x0e030014,0x1006000e,0x1005000f,0x0e050031,
0x1001035b,0x1004003b,0x0e04000d,0x1006000f,0x0e020023,0x10050011,0x10050010,0x0e07000e,
0x00000000,0x01010001,0x1006000e,0x10050021,0x10040032,0x1006000f,0x10050011,0x0e060010,
0x00000000,0x1003007a,0x1006000f,0x0e06000f,0x10030071,0x10050012,0x10050012,0x10090010,
0x00000000,0x01010001,0x10030059,0x02020002,0x0e030015,0x1006000d,0x1005000e,0x03030003,
0x00000000,0x0e030016,0x1003005a,0x0e07000d,0x0e020024,0x0e06000e,0x1005000f,0x1006005b,
0x00000000,0x01010001,0x1006000d,0x10040040,0x10040033,0x1006000e,0x10050010,0x0e07000e,
0x00000000,0x1005001d,0x1006000e,0x0e060010,0x10030072,0x0e06000f,0x10050011,0x1009000f,
0x00000000,0x01010001,0x0e03000f,0x02020002,0x0e030016,0x0e07000d,0x0e06000e,0x0e040070,
0x00000000,0x1004003c,0x0e030010,0x0e07000e,0x0e020025,0x0e060010,0x0e06000f,0x0e060011,
0x00000000,0x01010001,0x0e07000d,0x0e04005e,0x10040034,0x0e07000e,0x0e060010,0x0e060011,
0x00000000,0x1003007b,0x0e07000e,0x0e060011,0x10030073,0x0e060011,0x0e060011,0x0e0a000f,
0x00000000,0x01010001,0x04010002,0x02020002,0x0e03000e,0x05020003,0x05010004,0x03030003,
0x0e0100dc,0x1006000f,0x04010003,0x06030004,0x10050016,0x06020005,0x05010005,0x04040004,
0x00000000,0x100300b0,0x05020003,0x10060013,0x10040035,0x05020004,0x05010006,0x07040005,
0x1002015d,0x1005001e,0x05020004,0x07030006,0x10030074,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0e04000a,0x100300d9,0x0e03000f,0x06030004,0x06020005,0x10060015,
0x1001035c,0x1004003d,0x0e04000b,0x06030005,0x10050017,0x06020007,0x06020006,0x0e07000c,
0x00000000,0x100201af,0x06030004,0x10050022,0x10040036,0x06030005,0x06020007,0x0e06000d,
0x00000000,0x1003007c,0x06030005,0x07030007,0x10030075,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x10030014,0x02020002,0x0e030010,0x1007000b,0x1006000c,0x03030003,
0x00000000,0x10060010,0x10030015,0x07040005,0x10050018,0x07030006,0x1006000d,0x10060016,
0x00000000,0x01010001,0x1007000b,0x10040041,0x10040037,0x1007000c,0x1006000e,0x07040006,
0x00000000,0x1005001f,0x1007000c,0x07030008,0x10030076,0x07030007,0x1006000f,0x100a000d,
0x00000000,0x01010001,0x1006000c,0x02020002,0x0e030011,0x07040005,0x07030006,0x10050024,
0x00000000,0x1004003e,0x1006000d,0x07040006,0x10050019,0x07030008,0x07030007,0x1009000e,
0x00000000,0x01010001,0x07040005,0x0e04005f,0x10040038,0x07040006,0x07030008,0x0e06000e,
0x00000000,0x1003007d,0x07040006,0x07030009,0x10030077,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0e030012,0x1004002f,0x10030030,0x03030003,
0x0e0100dd,0x10060011,0x04010003,0x1007000c,0x1005001a,0x1006000d,0x10030031,0x04040004,
0x00000000,0x01010001,0x1004002f,0x10060014,0x10040039,0x10040030,0x10030032,0x0e07000c,
0x1002015e,0x10050020,0x10040030,0x0e06000d,0x10030078,0x1006000e,0x10030033,0x10070031,
0x00000000,0x01010001,0x0e04000b,0x02020002,0x0e030013,0x1007000c,0x1006000d,0x0e050032,
0x1001035d,0x1004003f,0x0e04000c,0x1007000d,0x1005001b,0x1006000f,0x1006000e,0x0e07000d,
0x00000000,0x01010001,0x1007000c,0x10050023,0x1004003a,0x1007000d,0x1006000f,0x0e06000f,
0x00000000,0x1003007e,0x1007000d,0x0e06000e,0x10030079,0x10060010,0x10060010,0x100a000e,
0x00000000,0x01010001,0x0e03001a,0x02020002,0x0e030014,0x1006000e,0x1005000f,0x03030003,
0x00000000,0x10060012,0x0e03001b,0x0e07000c,0x1005001c,0x0e06000d,0x10050010,0x0e06001c,
0x00000000,0x01010001,0x1006000e,0x10040042,0x1004003b,0x1006000f,0x10050011,0x0e07000d,
0x00000000,0x10050021,0x1006000f,0x0e06000f,0x1003007a,0x0e06000e,0x10050012,0x10090010,
0x00000000,0x01010001,0x1006000d,0x02020002,0x0e030015,0x0e07000c,0x0e06000d,0x0e040071,
0x00000000,0x10040040,0x1006000e,0x0e07000d,0x1005001d,0x0e06000f,0x0e06000e,0x1009000f,
0x00000000,0x01010001,0x0e07000c,0x0e040060,0x1004003c,0x0e07000d,0x0e06000f,0x0e060010,
0x00000000,0x1003007f,0x0e07000d,0x0e060010,0x1003007b,0x0e060010,0x0e060010,0x0e0a000e,
0x00000000,0x01010001,0x04010002,0x02020002,0x1006000f,0x05020003,0x05010004,0x03030003,
0x0e0100de,0x10060013,0x04010003,0x06030004,0x1005001e,0x06020005,0x05010005,0x04040004,
0x00000000,0x100300b1,0x05020003,0x10060015,0x1004003d,0x05020004,0x05010006,0x1006000f,
0x1002015f,0x10050022,0x05020004,0x10050010,0x1003007c,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x1007000b,0x02020002,0x10060010,0x06030004,0x06020005,0x10060016,
0x1001035e,0x10040041,0x1007000c,0x06030005,0x1005001f,0x06020007,0x06020006,0x100a000d,
0x00000000,0x100201b0,0x06030004,0x10050024,0x1004003e,0x06030005,0x06020007,0x1009000e,
0x00000000,0x10030080,0x06030005,0x10050011,0x1003007d,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0e020039,0x02020002,0x10060011,0x1007000c,0x1006000d,0x03030003,
0x00000000,0x10060014,0x0e02003a,0x1006000f,0x10050020,0x10050010,0x1006000e,0x0e05003b,
0x00000000,0x01010001,0x1007000c,0x10040043,0x1004003f,0x1007000d,0x1006000f,0x10060010,
0x00000000,0x10050023,0x1007000d,0x10050012,0x1003007e,0x10050011,0x10060010,0x100a000e,
0x00000000,0x01010001,0x1006000e,0x02020002,0x10060012,0x1006000f,0x10050010,0x10050025,
0x00000000,0x10040042,0x1006000f,0x10060010,0x10050021,0x10050012,0x10050011,0x10090010,
0x00000000,0x01010001,0x1006000f,0x0e040061,0x10040040,0x10060010,0x10050012,0x1009000f,
0x00000000,0x10030081,0x10060010,0x10050013,0x1003007f,0x10050013,0x10050013,0x10090011,
0x00000000,0x01010001,0x04010002,0x02020002,0x10060013,0x1003006e,0x1002006f,0x03030003,
0x0e0100df,0x10060015,0x04010003,0x1006000f,0x10050022,0x10050010,0x10020070,0x04040004,
0x00000000,0x01010001,0x1003006e,0x10060016,0x10040041,0x1003006f,0x10020071,0x100a000d,
0x10020160,0x10050024,0x1003006f,0x1009000e,0x10030080,0x10050011,0x10020072,0x10060070,
0x00000000,0x01010001,0x1007000c,0x02020002,0x10060014,0x1006000f,0x10050010,0x0e050033,
0x1001035f,0x10040043,0x1007000d,0x10060010,0x10050023,0x10050012,0x10050011,0x100a000e,
0x00000000,0x01010001,0x1006000f,0x10050025,0x10040042,0x10060010,0x10050012,0x10090010,
0x00000000,0x10030082,0x10060010,0x1009000f,0x10030081,0x10050013,0x10050013,0x10090011,
0x00000000,0x01010001,0x1003005a,0x02020002,0x10060015,0x1006000f,0x10050010,0x03030003,
0x00000000,0x10060016,0x1003005b,0x100a000d,0x10050024,0x1009000e,0x10050011,0x1006005c,
0x00000000,0x01010001,0x1006000f,0x10040044,0x10040043,0x10060010,0x10050012,0x100a000e,
0x00000000,0x10050025,0x10060010,0x10090010,0x10030082,0x1009000f,0x10050013,0x10090011,
0x00000000,0x01010001,0x1006000f,0x02020002,0x10060016,0x100a000d,0x1009000e,0x0e040072,
0x00000000,0x10040044,0x10060010,0x100a000e,0x10050025,0x10090010,0x1009000f,0x10090011,
0x00000000,0x01010001,0x100a000d,0x0e040062,0x10040044,0x100a000e,0x10090010,0x10090011,
0x00000000,0x10030083,0x100a000e,0x10090011,0x10030083,0x10090011,0x10090011,0x100d000f,
0x00000000,0x01010001,0x04010002,0x02020002,0x08010008,0x05020003,0x05010004,0x03030003,
0x0e0100e0,0x09020009,0x04010003,0x06030004,0x09010010,0x06020005,0x05010005,0x04040004,
0x00000000,0x100300b2,0x05020003,0x0a03000a,0x0a010020,0x05020004,0x05010006,0x07040005,
0x10020161,0x0a020011,0x05020004,0x07030006,0x10020046,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x08020004,0x100300da,0x08010009,0x06030004,0x06020005,0x10050011,
0x10010360,0x10030027,0x08020005,0x06030005,0x09010011,0x06020007,0x06020006,0x08050006,
0x00000000,0x100201b1,0x06030004,0x10040018,0x0a010021,0x06030005,0x06020007,0x08040007,
0x00000000,0x0e040043,0x06030005,0x07030007,0x10020047,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0e04000c,0x02020002,0x0801000a,0x09030005,0x09020006,0x03030003,
0x00000000,0x0902000a,0x0e04000d,0x07040005,0x09010012,0x07030006,0x09020007,0x0e07000e,
0x00000000,0x01010001,0x09030005,0x0e050024,0x0a010022,0x09030006,0x09020008,0x07040006,
0x00000000,0x0a020012,0x09030006,0x07030008,0x10020048,0x07030007,0x09020009,0x09060007,
0x00000000,0x01010001,0x09020006,0x02020002,0x0801000b,0x07040005,0x07030006,0x0e060015,
0x00000000,0x10030028,0x09020007,0x07040006,0x09010013,0x07030008,0x07030007,0x09050008,
0x00000000,0x01010001,0x07040005,0x0e040063,0x0a010023,0x07040006,0x07030008,0x08040008,
0x00000000,0x0e040044,0x07040006,0x07030009,0x10020049,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0801000c,0x0e040010,0x0e030011,0x03030003,
0x0e0100e1,0x0902000b,0x04010003,0x0a040006,0x09010014,0x0a030007,0x0e030012,0x04040004,
0x00000000,0x01010001,0x0e040010,0x0a03000b,0x0a010024,0x0e040011,0x0e030013,0x08050006,
0x10020162,0x0a020013,0x0e040011,0x08040007,0x1002004a,0x0a030008,0x0e030014,0x0e070012,
0x00000000,0x01010001,0x08020005,0x02020002,0x0801000d,0x0a040006,0x0a030007,0x0e050034,
0x10010361,0x10030029,0x08020006,0x0a040007,0x09010015,0x0a030009,0x0a030008,0x08050007,
0x00000000,0x01010001,0x0a040006,0x10040019,0x0a010025,0x0a040007,0x0a030009,0x08040009,
0x00000000,0x0e040045,0x0a040007,0x08040008,0x1002004b,0x0a03000a,0x0a03000a,0x0a070008,
0x00000000,0x01010001,0x0e03001b,0x02020002,0x0801000e,0x0a030007,0x0a020008,0x03030003,
0x00000000,0x0902000c,0x0e03001c,0x08050006,0x09010016,0x08040007,0x0a020009,0x0e06001d,
0x00000000,0x01010001,0x0a030007,0x0e050025,0x0a010026,0x0a030008,0x0a02000a,0x08050007,
0x00000000,0x0a020014,0x0a030008,0x08040009,0x1002004c,0x08040008,0x0a02000b,0x0a060009,
0x00000000,0x01010001,0x09020007,0x02020002,0x0801000f,0x08050006,0x08040007,0x0e040073,
0x00000000,0x1003002a,0x09020008,0x08050007,0x09010017,0x08040009,0x08040008,0x09050009,
0x00000000,0x01010001,0x08050006,0x0e040064,0x0a010027,0x08050007,0x08040009,0x0804000a,
0x00000000,0x0e040046,0x08050007,0x0804000a,0x1002004d,0x0804000a,0x0804000a,0x08080008,
0x00000000,0x01010001,0x04010002,0x02020002,0x09020009,0x05020003,0x05010004,0x03030003,
0x0e0100e2,0x0902000d,0x04010003,0x0e040012,0x09010018,0x0e030013,0x05010005,0x04040004,
0x00000000,0x100300b3,0x05020003,0x0a03000c,0x0a010028,0x05020004,0x05010006,0x1006000d,
0x10020163,0x0a020015,0x05020004,0x1005000e,0x1002004e,0x0e030014,0x05010007,0x05050005,
0x00000000,0x01010001,0x09030005,0x02020002,0x0902000a,0x0e040012,0x0e030013,0x10050012,
0x10010362,0x1003002b,0x09030006,0x0e040013,0x09010019,0x0e030015,0x0e030014,0x09060007,
0x00000000,0x100201b2,0x0e040012,0x1004001a,0x0a010029,0x0e040013,0x0e030015,0x09050008,
0x00000000,0x0e040047,0x0e040013,0x1005000f,0x1002004f,0x0e030016,0x0e030016,0x0e070014,
0x00000000,0x01010001,0x0e02003a,0x02020002,0x0902000b,0x09030006,0x09020007,0x03030003,
0x00000000,0x0902000e,0x0e02003b,0x1006000d,0x0901001a,0x1005000e,0x09020008,0x0e05003c,
0x00000000,0x01010001,0x09030006,0x0e050026,0x0a01002a,0x09030007,0x09020009,0x1006000e,
0x00000000,0x0a020016,0x09030007,0x10050010,0x10020050,0x1005000f,0x0902000a,0x09060008,
0x00000000,0x01010001,0x09020008,0x02020002,0x0902000c,0x1006000d,0x1005000e,0x0e060016,
0x00000000,0x1003002c,0x09020009,0x1006000e,0x0901001b,0x10050010,0x1005000f,0x0905000a,
0x00000000,0x01010001,0x1006000d,0x0e040065,0x0a01002b,0x1006000e,0x10050010,0x09050009,
0x00000000,0x0e040048,0x1006000e,0x10050011,0x10020051,0x10050011,0x10050011,0x1009000f,
0x00000000,0x01010001,0x04010002,0x02020002,0x0902000d,0x0e03001f,0x0e020020,0x03030003,
0x0e0100e3,0x0902000f,0x04010003,0x1005000e,0x0901001c,0x1004000f,0x0e020021,0x04040004,
0x00000000,0x01010001,0x0e03001f,0x0a03000d,0x0a01002c,0x0e030020,0x0e020022,0x09060007,
0x10020164,0x0a020017,0x0e030020,0x09050008,0x10020052,0x10040010,0x0e020023,0x0e060021,
0x00000000,0x01010001,0x09030006,0x02020002,0x0902000e,0x1005000e,0x1004000f,0x0e050035,
0x10010363,0x1003002d,0x09030007,0x1005000f,0x0901001d,0x10040011,0x10040010,0x09060008,
0x00000000,0x01010001,0x1005000e,0x1004001b,0x0a01002d,0x1005000f,0x10040011,0x0905000a,
0x00000000,0x0e040049,0x1005000f,0x09050009,0x10020053,0x10040012,0x10040012,0x10080010,
0x00000000,0x01010001,0x1003005b,0x02020002,0x0902000f,0x0a030008,0x0a020009,0x03030003,
0x00000000,0x09020010,0x1003005c,0x09060007,0x0901001e,0x09050008,0x0a02000a,0x1006005d,
0x00000000,0x01010001,0x0a030008,0x0e050027,0x0a01002e,0x0a030009,0x0a02000b,0x09060008,
0x00000000,0x0a020018,0x0a030009,0x0905000a,0x10020054,0x09050009,0x0a02000c,0x0a06000a,
0x00000000,0x01010001,0x09020009,0x02020002,0x09020010,0x09060007,0x09050008,0x0e040074,
0x00000000,0x1003002e,0x0902000a,0x09060008,0x0901001f,0x0905000a,0x09050009,0x0905000b,
0x00000000,0x01010001,0x09060007,0x0e040066,0x0a01002f,0x09060008,0x0905000a,0x0905000b,
0x00000000,0x0e04004a,0x09060008,0x0905000b,0x10020055,0x0905000b,0x0905000b,0x09090009,
0x00000000,0x01010001,0x04010002,0x02020002,0x0e04000c,0x05020003,0x05010004,0x03030003,
0x0e0100e4,0x0a03000a,0x04010003,0x06030004,0x0a020011,0x06020005,0x05010005,0x04040004,
0x00000000,0x100300b4,0x05020003,0x0a03000e,0x0a010030,0x05020004,0x05010006,0x10060011,
0x10020165,0x0a020019,0x05020004,0x10050012,0x10020056,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0e050008,0x100300db,0x0e04000d,0x06030004,0x06020005,0x10050013,
0x10010364,0x1003002f,0x0e050009,0x06030005,0x0a020012,0x06020007,0x06020006,0x0e08000a,
0x00000000,0x100201b3,0x06030004,0x1004001c,0x0a010031,0x06030005,0x06020007,0x0e07000b,
0x00000000,0x0e04004b,0x06030005,0x10050013,0x10020057,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0e04000d,0x02020002,0x0e04000e,0x0a040006,0x0a030007,0x03030003,
0x00000000,0x0a03000b,0x0e04000e,0x10060011,0x0a020013,0x10050012,0x0a030008,0x0e07000f,
0x00000000,0x01010001,0x0a040006,0x0e050028,0x0a010032,0x0a040007,0x0a030009,0x10060012,
0x00000000,0x0a02001a,0x0a040007,0x10050014,0x10020058,0x10050013,0x0a03000a,0x0a070008,
0x00000000,0x01010001,0x0a030007,0x02020002,0x0e04000f,0x10060011,0x10050012,0x0e060017,
0x00000000,0x10030030,0x0a030008,0x10060012,0x0a020014,0x10050014,0x10050013,0x0a060009,
0x00000000,0x01010001,0x10060011,0x0e040067,0x0a010033,0x10060012,0x10050014,0x0e07000c,
0x00000000,0x0e04004c,0x10060012,0x10050015,0x10020059,0x10050015,0x10050015,0x10090013,
0x00000000,0x01010001,0x04010002,0x02020002,0x0e040010,0x10040030,0x10030031,0x03030003,
0x0e0100e5,0x0a03000c,0x04010003,0x0a040007,0x0a020015,0x0a030008,0x10030032,0x04040004,
0x00000000,0x01010001,0x10040030,0x0a03000f,0x0a010034,0x10040031,0x10030033,0x0e08000a,
0x10020166,0x0a02001b,0x10040031,0x0e07000b,0x1002005a,0x0a030009,0x10030034,0x10070032,
0x00000000,0x01010001,0x0e050009,0x02020002,0x0e040011,0x0a040007,0x0a030008,0x0e050036,
0x10010365,0x10030031,0x0e05000a,0x0a040008,0x0a020016,0x0a03000a,0x0a030009,0x0e08000b,
0x00000000,0x01010001,0x0a040007,0x1004001d,0x0a010035,0x0a040008,0x0a03000a,0x0e07000d,
0x00000000,0x0e04004d,0x0a040008,0x0e07000c,0x1002005b,0x0a03000b,0x0a03000b,0x0a070009,
0x00000000,0x01010001,0x0e03001c,0x02020002,0x0e040012,0x0a030009,0x0a02000a,0x03030003,
0x00000000,0x0a03000d,0x0e03001d,0x0e08000a,0x0a020017,0x0e07000b,0x0a02000b,0x0e06001e,
0x00000000,0x01010001,0x0a030009,0x0e050029,0x0a010036,0x0a03000a,0x0a02000c,0x0e08000b,
0x00000000,0x0a02001c,0x0a03000a,0x0e07000d,0x1002005c,0x0e07000c,0x0a02000d,0x0a06000b,
0x00000000,0x01010001,0x0a030008,0x02020002,0x0e040013,0x0e08000a,0x0e07000b,0x0e040075,
0x00000000,0x10030032,0x0a030009,0x0e08000b,0x0a020018,0x0e07000d,0x0e07000c,0x0a06000a,
0x00000000,0x01010001,0x0e08000a,0x0e040068,0x0a010037,0x0e08000b,0x0e07000d,0x0e07000e,
0x00000000,0x0e04004e,0x0e08000b,0x0e07000e,0x1002005d,0x0e07000e,0x0e07000e,0x0e0b000c,
0x00000000,0x01010001,0x04010002,0x02020002,0x0a03000a,0x05020003,0x05010004,0x03030003,
0x0e0100e6,0x0a03000e,0x04010003,0x1005001b,0x0a020019,0x1004001c,0x05010005,0x04040004,
0x00000000,0x100300b5,0x05020003,0x0a030010,0x0a010038,0x05020004,0x05010006,0x0e07000b,
0x10020167,0x0a02001d,0x05020004,0x0e06000c,0x1002005e,0x1004001d,0x05010007,0x05050005,
0x00000000,0x01010001,0x0a040006,0x02020002,0x0a03000b,0x1005001b,0x1004001c,0x10050014,
0x10010366,0x10030033,0x0a040007,0x1005001c,0x0a02001a,0x1004001e,0x1004001d,0x0a070008,
0x00000000,0x100201b4,0x1005001b,0x1004001e,0x0a010039,0x1005001c,0x1004001e,0x0a060009,
0x00000000,0x0e04004f,0x1005001c,0x0e06000d,0x1002005f,0x1004001f,0x1004001f,0x1008001d,
0x00000000,0x01010001,0x0e02003b,0x02020002,0x0a03000c,0x0a040007,0x0a030008,0x03030003,
0x00000000,0x0a03000f,0x0e02003c,0x0e07000b,0x0a02001b,0x0e06000c,0x0a030009,0x0e05003d,
0x00000000,0x01010001,0x0a040007,0x0e05002a,0x0a01003a,0x0a040008,0x0a03000a,0x0e07000c,
0x00000000,0x0a02001e,0x0a040008,0x0e06000e,0x10020060,0x0e06000d,0x0a03000b,0x0a070009,
0x00000000,0x01010001,0x0a030009,0x02020002,0x0a03000d,0x0e07000b,0x0e06000c,0x0e060018,
0x00000000,0x10030034,0x0a03000a,0x0e07000c,0x0a02001c,0x0e06000e,0x0e06000d,0x0a06000b,
0x00000000,0x01010001,0x0e07000b,0x0e040069,0x0a01003b,0x0e07000c,0x0e06000e,0x0a06000a,
0x00000000,0x0e040050,0x0e07000c,0x0e06000f,0x10020061,0x0e06000f,0x0e06000f,0x0e0a000d,
0x00000000,0x01010001,0x04010002,0x02020002,0x0a03000e,0x1003006f,0x10020070,0x03030003,
0x0e0100e7,0x0a030010,0x04010003,0x1005000f,0x0a02001d,0x10040010,0x10020071,0x04040004,
0x00000000,0x01010001,0x1003006f,0x0a030011,0x0a01003c,0x10030070,0x10020072,0x0a070008,
0x10020168,0x0a02001f,0x10030070,0x0a060009,0x10020062,0x10040011,0x10020073,0x10060071,
0x00000000,0x01010001,0x0a040007,0x02020002,0x0a03000f,0x1005000f,0x10040010,0x0e050037,
0x10010367,0x10030035,0x0a040008,0x10050010,0x0a02001e,0x10040012,0x10040011,0x0a070009,
0x00000000,0x01010001,0x1005000f,0x1004001f,0x0a01003d,0x10050010,0x10040012,0x0a06000b,
0x00000000,0x0e040051,0x10050010,0x0a06000a,0x10020063,0x10040013,0x10040013,0x10080011,
0x00000000,0x01010001,0x1003005c,0x02020002,0x0a030010,0x0a03000a,0x0a02000b,0x03030003,
0x00000000,0x0a030011,0x1003005d,0x0a070008,0x0a02001f,0x0a060009,0x0a02000c,0x1006005e,
0x00000000,0x01010001,0x0a03000a,0x0e05002b,0x0a01003e,0x0a03000b,0x0a02000d,0x0a070009,
0x00000000,0x0a020020,0x0a03000b,0x0a06000b,0x10020064,0x0a06000a,0x0a02000e,0x0a06000c,
0x00000000,0x01010001,0x0a03000a,0x02020002,0x0a030011,0x0a070008,0x0a060009,0x0e040076,
0x00000000,0x10030036,0x0a03000b,0x0a070009,0x0a020020,0x0a06000b,0x0a06000a,0x0a06000c,
0x00000000,0x01010001,0x0a070008,0x0e04006a,0x0a01003f,0x0a070009,0x0a06000b,0x0a06000c,
0x00000000,0x0e040052,0x0a070009,0x0a06000c,0x10020065,0x0a06000c,0x0a06000c,0x0a0a000a,
0x00000000,0x01010001,0x04010002,0x02020002,0x08010008,0x05020003,0x05010004,0x03030003,
0x0e0100e8,0x0e04000d,0x04010003,0x06030004,0x0e030014,0x06020005,0x05010005,0x04040004,
0x00000000,0x100300b6,0x05020003,0x10050011,0x10030027,0x05020004,0x05010006,0x07040005,
0x10020169,0x10040018,0x05020004,0x07030006,0x10020066,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x08020004,0x100300dc,0x08010009,0x06030004,0x06020005,0x10050015,
0x10010368,0x10030037,0x08020005,0x06030005,0x0e030015,0x06020007,0x06020006,0x08050006,
0x00000000,0x100201b5,0x06030004,0x10040020,0x10030028,0x06030005,0x06020007,0x08040007,
0x00000000,0x0e040053,0x06030005,0x07030007,0x10020067,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0e04000e,0x02020002,0x0801000a,0x0e050009,0x0e04000a,0x03030003,
0x00000000,0x0e04000e,0x0e04000f,0x07040005,0x0e030016,0x07030006,0x0e04000b,0x0e070010,
0x00000000,0x01010001,0x0e050009,0x0e05002c,0x10030029,0x0e05000a,0x0e04000c,0x07040006,
0x00000000,0x10040019,0x0e05000a,0x07030008,0x10020068,0x07030007,0x0e04000d,0x0e08000b,
0x00000000,0x01010001,0x0e04000a,0x02020002,0x0801000b,0x07040005,0x07030006,0x0e060019,
0x00000000,0x10030038,0x0e04000b,0x07040006,0x0e030017,0x07030008,0x07030007,0x0e07000c,
0x00000000,0x01010001,0x07040005,0x0e04006b,0x1003002a,0x07040006,0x07030008,0x08040008,
0x00000000,0x0e040054,0x07040006,0x07030009,0x10020069,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0801000c,0x0e040011,0x0e030012,0x03030003,
0x0e0100e9,0x0e04000f,0x04010003,0x1006000d,0x0e030018,0x1005000e,0x0e030013,0x04040004,
0x00000000,0x01010001,0x0e040011,0x10050012,0x1003002b,0x0e040012,0x0e030014,0x08050006,
0x1002016a,0x1004001a,0x0e040012,0x08040007,0x1002006a,0x1005000f,0x0e030015,0x0e070013,
0x00000000,0x01010001,0x08020005,0x02020002,0x0801000d,0x1006000d,0x1005000e,0x0e050038,
0x10010369,0x10030039,0x08020006,0x1006000e,0x0e030019,0x10050010,0x1005000f,0x08050007,
0x00000000,0x01010001,0x1006000d,0x10040021,0x1003002c,0x1006000e,0x10050010,0x08040009,
0x00000000,0x0e040055,0x1006000e,0x08040008,0x1002006b,0x10050011,0x10050011,0x1009000f,
0x00000000,0x01010001,0x0e03001d,0x02020002,0x0801000e,0x1005000e,0x1004000f,0x03030003,
0x00000000,0x0e040010,0x0e03001e,0x08050006,0x0e03001a,0x08040007,0x10040010,0x0e06001f,
0x00000000,0x01010001,0x1005000e,0x0e05002d,0x1003002d,0x1005000f,0x10040011,0x08050007,
0x00000000,0x1004001b,0x1005000f,0x08040009,0x1002006c,0x08040008,0x10040012,0x10080010,
0x00000000,0x01010001,0x0e04000b,0x02020002,0x0801000f,0x08050006,0x08040007,0x0e040077,
0x00000000,0x1003003a,0x0e04000c,0x08050007,0x0e03001b,0x08040009,0x08040008,0x0e07000d,
0x00000000,0x01010001,0x08050006,0x0e04006c,0x1003002e,0x08050007,0x08040009,0x0804000a,
0x00000000,0x0e040056,0x08050007,0x0804000a,0x1002006d,0x0804000a,0x0804000a,0x08080008,
0x00000000,0x01010001,0x04010002,0x02020002,0x0e04000d,0x05020003,0x05010004,0x03030003,
0x0e0100ea,0x0e040011,0x04010003,0x1004003a,0x0e03001c,0x1003003b,0x05010005,0x04040004,
0x00000000,0x100300b7,0x05020003,0x10050013,0x1003002f,0x05020004,0x05010006,0x1006000e,
0x1002016b,0x1004001c,0x05020004,0x1005000f,0x1002006e,0x1003003c,0x05010007,0x05050005,
0x00000000,0x01010001,0x0e050009,0x02020002,0x0e04000e,0x1004003a,0x1003003b,0x10050016,
0x1001036a,0x1003003b,0x0e05000a,0x1004003b,0x0e03001d,0x1003003d,0x1003003c,0x0e08000b,
0x00000000,0x100201b6,0x1004003a,0x10040022,0x10030030,0x1004003b,0x1003003d,0x0e07000c,
0x00000000,0x0e040057,0x1004003b,0x10050010,0x1002006f,0x1003003e,0x1003003e,0x1007003c,
0x00000000,0x01010001,0x0e02003c,0x02020002,0x0e04000f,0x0e05000a,0x0e04000b,0x03030003,
0x00000000,0x0e040012,0x0e02003d,0x1006000e,0x0e03001e,0x1005000f,0x0e04000c,0x0e05003e,
0x00000000,0x01010001,0x0e05000a,0x0e05002e,0x10030031,0x0e05000b,0x0e04000d,0x1006000f,
0x00000000,0x1004001d,0x0e05000b,0x10050011,0x10020070,0x10050010,0x0e04000e,0x0e08000c,
0x00000000,0x01010001,0x0e04000c,0x02020002,0x0e040010,0x1006000e,0x1005000f,0x0e06001a,
0x00000000,0x1003003c,0x0e04000d,0x1006000f,0x0e03001f,0x10050011,0x10050010,0x0e07000e,
0x00000000,0x01010001,0x1006000e,0x0e04006d,0x10030032,0x1006000f,0x10050011,0x0e07000d,
0x00000000,0x0e040058,0x1006000f,0x10050012,0x10020071,0x10050012,0x10050012,0x10090010,
0x00000000,0x01010001,0x04010002,0x02020002,0x0e040011,0x0e030020,0x0e020021,0x03030003,
0x0e0100eb,0x0e040013,0x04010003,0x10050010,0x0e030020,0x10040011,0x0e020022,0x04040004,
0x00000000,0x01010001,0x0e030020,0x10050014,0x10030033,0x0e030021,0x0e020023,0x0e08000b,
0x1002016c,0x1004001e,0x0e030021,0x0e07000c,0x10020072,0x10040012,0x0e020024,0x0e060022,
0x00000000,0x01010001,0x0e05000a,0x02020002,0x0e040012,0x10050010,0x10040011,0x0e050039,
0x1001036b,0x1003003d,0x0e05000b,0x10050011,0x0e030021,0x10040013,0x10040012,0x0e08000c,
0x00000000,0x01010001,0x10050010,0x10040023,0x10030034,0x10050011,0x10040013,0x0e07000e,
0x00000000,0x0e040059,0x10050011,0x0e07000d,0x10020073,0x10040014,0x10040014,0x10080012,
0x00000000,0x01010001,0x1003005d,0x02020002,0x0e040013,0x1005000f,0x10040010,0x03030003,
0x00000000,0x0e040014,0x1003005e,0x0e08000b,0x0e030022,0x0e07000c,0x10040011,0x1006005f,
0x00000000,0x01010001,0x1005000f,0x0e05002f,0x10030035,0x10050010,0x10040012,0x0e08000c,
0x00000000,0x1004001f,0x10050010,0x0e07000e,0x10020074,0x0e07000d,0x10040013,0x10080011,
0x00000000,0x01010001,0x0e04000d,0x02020002,0x0e040014,0x0e08000b,0x0e07000c,0x0e040078,
0x00000000,0x1003003e,0x0e04000e,0x0e08000c,0x0e030023,0x0e07000e,0x0e07000d,0x0e07000f,
0x00000000,0x01010001,0x0e08000b,0x0e04006e,0x10030036,0x0e08000c,0x0e07000e,0x0e07000f,
0x00000000,0x0e04005a,0x0e08000c,0x0e07000f,0x10020075,0x0e07000f,0x0e07000f,0x0e0b000d,
0x00000000,0x01010001,0x04010002,0x02020002,0x0e03000f,0x05020003,0x05010004,0x03030003,
0x0e0100ec,0x10050011,0x04010003,0x06030004,0x10040018,0x06020005,0x05010005,0x04040004,
0x00000000,0x100300b8,0x05020003,0x10050015,0x10030037,0x05020004,0x05010006,0x10050020,
0x1002016d,0x10040020,0x05020004,0x10040021,0x10020076,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0e04000b,0x100300dd,0x0e030010,0x06030004,0x06020005,0x10050017,
0x1001036c,0x1003003f,0x0e04000c,0x06030005,0x10040019,0x06020007,0x06020006,0x0e07000d,
0x00000000,0x100201b7,0x06030004,0x10040024,0x10030038,0x06030005,0x06020007,0x0e06000e,
0x00000000,0x0e04005b,0x06030005,0x10040022,0x10020077,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0e04000f,0x02020002,0x0e030011,0x1006000d,0x1005000e,0x03030003,
0x00000000,0x10050012,0x0e040010,0x10050020,0x1004001a,0x10040021,0x1005000f,0x0e070011,
0x00000000,0x01010001,0x1006000d,0x0e050030,0x10030039,0x1006000e,0x10050010,0x10050021,
0x00000000,0x10040021,0x1006000e,0x10040023,0x10020078,0x10040022,0x10050011,0x1009000f,
0x00000000,0x01010001,0x1005000e,0x02020002,0x0e030012,0x10050020,0x10040021,0x0e06001b,
0x00000000,0x10030040,0x1005000f,0x10050021,0x1004001b,0x10040023,0x10040022,0x10080010,
0x00000000,0x01010001,0x10050020,0x0e04006f,0x1003003a,0x10050021,0x10040023,0x0e06000f,
0x00000000,0x0e04005c,0x10050021,0x10040024,0x10020079,0x10040024,0x10040024,0x10080022,
0x00000000,0x01010001,0x04010002,0x02020002,0x0e030013,0x10040031,0x10030032,0x03030003,
0x0e0100ed,0x10050013,0x04010003,0x1006000e,0x1004001c,0x1005000f,0x10030033,0x04040004,
0x00000000,0x01010001,0x10040031,0x10050016,0x1003003b,0x10040032,0x10030034,0x0e07000d,
0x1002016e,0x10040022,0x10040032,0x0e06000e,0x1002007a,0x10050010,0x10030035,0x10070033,
0x00000000,0x01010001,0x0e04000c,0x02020002,0x0e030014,0x1006000e,0x1005000f,0x0e05003a,
0x1001036d,0x10030041,0x0e04000d,0x1006000f,0x1004001d,0x10050011,0x10050010,0x0e07000e,
0x00000000,0x01010001,0x1006000e,0x10040025,0x1003003c,0x1006000f,0x10050011,0x0e060010,
0x00000000,0x0e04005d,0x1006000f,0x0e06000f,0x1002007b,0x10050012,0x10050012,0x10090010,
0x00000000,0x01010001,0x0e03001e,0x02020002,0x0e030015,0x10050010,0x10040011,0x03030003,
0x00000000,0x10050014,0x0e03001f,0x0e07000d,0x1004001e,0x0e06000e,0x10040012,0x0e060020,
0x00000000,0x01010001,0x10050010,0x0e050031,0x1003003d,0x10050011,0x10040013,0x0e07000e,
0x00000000,0x10040023,0x10050011,0x0e060010,0x1002007c,0x0e06000f,0x10040014,0x10080012,
0x00000000,0x01010001,0x1005000f,0x02020002,0x0e030016,0x0e07000d,0x0e06000e,0x0e040079,
0x00000000,0x10030042,0x10050010,0x0e07000e,0x1004001f,0x0e060010,0x0e06000f,0x10080011,
0x00000000,0x01010001,0x0e07000d,0x0e040070,0x1003003e,0x0e07000e,0x0e060010,0x0e060011,
0x00000000,0x0e04005e,0x0e07000e,0x0e060011,0x1002007d,0x0e060011,0x0e060011,0x0e0a000f,
0x00000000,0x01010001,0x04010002,0x02020002,0x10050011,0x05020003,0x05010004,0x03030003,
0x0e0100ee,0x10050015,0x04010003,0x06030004,0x10040020,0x06020005,0x05010005,0x04040004,
0x00000000,0x100300b9,0x05020003,0x10050017,0x1003003f,0x05020004,0x05010006,0x0e07000c,
0x1002016f,0x10040024,0x05020004,0x0e06000d,0x1002007e,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x1006000d,0x02020002,0x10050012,0x06030004,0x06020005,0x10050018,
0x1001036e,0x10030043,0x1006000e,0x06030005,0x10040021,0x06020007,0x06020006,0x1009000f,
0x00000000,0x100201b8,0x06030004,0x10040026,0x10030040,0x06030005,0x06020007,0x10080010,
0x00000000,0x0e04005f,0x06030005,0x0e06000e,0x1002007f,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0e02003d,0x02020002,0x10050013,0x1006000e,0x1005000f,0x03030003,
0x00000000,0x10050016,0x0e02003e,0x0e07000c,0x10040022,0x0e06000d,0x10050010,0x0e05003f,
0x00000000,0x01010001,0x1006000e,0x0e050032,0x10030041,0x1006000f,0x10050011,0x0e07000d,
0x00000000,0x10040025,0x1006000f,0x0e06000f,0x10020080,0x0e06000e,0x10050012,0x10090010,
0x00000000,0x01010001,0x10050010,0x02020002,0x10050014,0x0e07000c,0x0e06000d,0x0e06001c,
0x00000000,0x10030044,0x10050011,0x0e07000d,0x10040023,0x0e06000f,0x0e06000e,0x10080012,
0x00000000,0x01010001,0x0e07000c,0x0e040071,0x10030042,0x0e07000d,0x0e06000f,0x10080011,
0x00000000,0x0e040060,0x0e07000d,0x0e060010,0x10020081,0x0e060010,0x0e060010,0x0e0a000e,
0x00000000,0x01010001,0x04010002,0x02020002,0x10050015,0x10030070,0x10020071,0x03030003,
0x0e0100ef,0x10050017,0x04010003,0x10050011,0x10040024,0x10040012,0x10020072,0x04040004,
0x00000000,0x01010001,0x10030070,0x10050018,0x10030043,0x10030071,0x10020073,0x1009000f,
0x10020170,0x10040026,0x10030071,0x10080010,0x10020082,0x10040013,0x10020074,0x10060072,
0x00000000,0x01010001,0x1006000e,0x02020002,0x10050016,0x10050011,0x10040012,0x0e05003b,
0x1001036f,0x10030045,0x1006000f,0x10050012,0x10040025,0x10040014,0x10040013,0x10090010,
0x00000000,0x01010001,0x10050011,0x10040027,0x10030044,0x10050012,0x10040014,0x10080012,
0x00000000,0x0e040061,0x10050012,0x10080011,0x10020083,0x10040015,0x10040015,0x10080013,
0x00000000,0x01010001,0x1003005e,0x02020002,0x10050017,0x10050011,0x10040012,0x03030003,
0x00000000,0x10050018,0x1003005f,0x1009000f,0x10040026,0x10080010,0x10040013,0x10060060,
0x00000000,0x01010001,0x10050011,0x0e050033,0x10030045,0x10050012,0x10040014,0x10090010,
0x00000000,0x10040027,0x10050012,0x10080012,0x10020084,0x10080011,0x10040015,0x10080013,
0x00000000,0x01010001,0x10050011,0x02020002,0x10050018,0x1009000f,0x10080010,0x0e04007a,
0x00000000,0x10030046,0x10050012,0x10090010,0x10040027,0x10080012,0x10080011,0x10080013,
0x00000000,0x01010001,0x1009000f,0x0e040072,0x10030046,0x10090010,0x10080012,0x10080013,
0x00000000,0x0e040062,0x10090010,0x10080013,0x10020085,0x10080013,0x10080013,0x100c0011,
0x00000000,0x01010001,0x04010002,0x02020002,0x08010008,0x05020003,0x05010004,0x03030003,
0x0e0100f0,0x09020009,0x04010003,0x06030004,0x09010010,0x06020005,0x05010005,0x04040004,
0x00000000,0x100300ba,0x05020003,0x10050010,0x10030026,0x05020004,0x05010006,0x07040005,
0x10020171,0x10040017,0x05020004,0x07030006,0x0e040043,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x08020004,0x100300de,0x08010009,0x06030004,0x06020005,0x0e07000e,
0x10010370,0x0e050024,0x08020005,0x06030005,0x09010011,0x06020007,0x06020006,0x08050006,
0x00000000,0x100201b9,0x06030004,0x0e060015,0x10030027,0x06030005,0x06020007,0x08040007,
0x00000000,0x0e040063,0x06030005,0x07030007,0x0e040044,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0e040010,0x02020002,0x0801000a,0x09030005,0x09020006,0x03030003,
0x00000000,0x0902000a,0x0e040011,0x07040005,0x09010012,0x07030006,0x09020007,0x0e070012,
0x00000000,0x01010001,0x09030005,0x0e050034,0x10030028,0x09030006,0x09020008,0x07040006,
0x00000000,0x10040018,0x09030006,0x07030008,0x0e040045,0x07030007,0x09020009,0x09060007,
0x00000000,0x01010001,0x09020006,0x02020002,0x0801000b,0x07040005,0x07030006,0x0e06001d,
0x00000000,0x0e050025,0x09020007,0x07040006,0x09010013,0x07030008,0x07030007,0x09050008,
0x00000000,0x01010001,0x07040005,0x0e040073,0x10030029,0x07040006,0x07030008,0x08040008,
0x00000000,0x0e040064,0x07040006,0x07030009,0x0e040046,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0801000c,0x0e040012,0x0e030013,0x03030003,
0x0e0100f1,0x0902000b,0x04010003,0x1006000c,0x09010014,0x1005000d,0x0e030014,0x04040004,
0x00000000,0x01010001,0x0e040012,0x10050011,0x1003002a,0x0e040013,0x0e030015,0x08050006,
0x10020172,0x10040019,0x0e040013,0x08040007,0x0e040047,0x1005000e,0x0e030016,0x0e070014,
0x00000000,0x01010001,0x08020005,0x02020002,0x0801000d,0x1006000c,0x1005000d,0x0e05003c,
0x10010371,0x0e050026,0x08020006,0x1006000d,0x09010015,0x1005000f,0x1005000e,0x08050007,
0x00000000,0x01010001,0x1006000c,0x0e060016,0x1003002b,0x1006000d,0x1005000f,0x08040009,
0x00000000,0x0e040065,0x1006000d,0x08040008,0x0e040048,0x10050010,0x10050010,0x1009000e,
0x00000000,0x01010001,0x0e03001f,0x02020002,0x0801000e,0x1005000d,0x1004000e,0x03030003,
0x00000000,0x0902000c,0x0e030020,0x08050006,0x09010016,0x08040007,0x1004000f,0x0e060021,
0x00000000,0x01010001,0x1005000d,0x0e050035,0x1003002c,0x1005000e,0x10040010,0x08050007,
0x00000000,0x1004001a,0x1005000e,0x08040009,0x0e040049,0x08040008,0x10040011,0x1008000f,
0x00000000,0x01010001,0x09020007,0x02020002,0x0801000f,0x08050006,0x08040007,0x0e04007b,
0x00000000,0x0e050027,0x09020008,0x08050007,0x09010017,0x08040009,0x08040008,0x09050009,
0x00000000,0x01010001,0x08050006,0x0e040074,0x1003002d,0x08050007,0x08040009,0x0804000a,
0x00000000,0x0e040066,0x08050007,0x0804000a,0x0e04004a,0x0804000a,0x0804000a,0x08080008,
0x00000000,0x01010001,0x04010002,0x02020002,0x09020009,0x05020003,0x05010004,0x03030003,
0x0e0100f2,0x0902000d,0x04010003,0x0e040013,0x09010018,0x0e030014,0x05010005,0x04040004,
0x00000000,0x100300bb,0x05020003,0x10050012,0x1003002e,0x05020004,0x05010006,0x0e08000a,
0x10020173,0x1004001b,0x05020004,0x0e07000b,0x0e04004b,0x0e030015,0x05010007,0x05050005,
0x00000000,0x01010001,0x09030005,0x02020002,0x0902000a,0x0e040013,0x0e030014,0x0e07000f,
0x10010372,0x0e050028,0x09030006,0x0e040014,0x09010019,0x0e030016,0x0e030015,0x09060007,
0x00000000,0x100201ba,0x0e040013,0x0e060017,0x1003002f,0x0e040014,0x0e030016,0x09050008,
0x00000000,0x0e040067,0x0e040014,0x0e07000c,0x0e04004c,0x0e030017,0x0e030017,0x0e070015,
0x00000000,0x01010001,0x0e02003e,0x02020002,0x0902000b,0x09030006,0x09020007,0x03030003,
0x00000000,0x0902000e,0x0e02003f,0x0e08000a,0x0901001a,0x0e07000b,0x09020008,0x0e050040,
0x00000000,0x01010001,0x09030006,0x0e050036,0x10030030,0x09030007,0x09020009,0x0e08000b,
0x00000000,0x1004001c,0x09030007,0x0e07000d,0x0e04004d,0x0e07000c,0x0902000a,0x09060008,
0x00000000,0x01010001,0x09020008,0x02020002,0x0902000c,0x0e08000a,0x0e07000b,0x0e06001e,
0x00000000,0x0e050029,0x09020009,0x0e08000b,0x0901001b,0x0e07000d,0x0e07000c,0x0905000a,
0x00000000,0x01010001,0x0e08000a,0x0e040075,0x10030031,0x0e08000b,0x0e07000d,0x09050009,
0x00000000,0x0e040068,0x0e08000b,0x0e07000e,0x0e04004e,0x0e07000e,0x0e07000e,0x0e0b000c,
0x00000000,0x01010001,0x04010002,0x02020002,0x0902000d,0x0e030021,0x0e020022,0x03030003,
0x0e0100f3,0x0902000f,0x04010003,0x0e07000b,0x0901001c,0x0e06000c,0x0e020023,0x04040004,
0x00000000,0x01010001,0x0e030021,0x10050013,0x10030032,0x0e030022,0x0e020024,0x09060007,
0x10020174,0x1004001d,0x0e030022,0x09050008,0x0e04004f,0x0e06000d,0x0e020025,0x0e060023,
0x00000000,0x01010001,0x09030006,0x02020002,0x0902000e,0x0e07000b,0x0e06000c,0x0e05003d,
0x10010373,0x0e05002a,0x09030007,0x0e07000c,0x0901001d,0x0e06000e,0x0e06000d,0x09060008,
0x00000000,0x01010001,0x0e07000b,0x0e060018,0x10030033,0x0e07000c,0x0e06000e,0x0905000a,
0x00000000,0x0e040069,0x0e07000c,0x09050009,0x0e040050,0x0e06000f,0x0e06000f,0x0e0a000d,
0x00000000,0x01010001,0x1003005f,0x02020002,0x0902000f,0x1005000e,0x1004000f,0x03030003,
0x00000000,0x09020010,0x10030060,0x09060007,0x0901001e,0x09050008,0x10040010,0x10060061,
0x00000000,0x01010001,0x1005000e,0x0e050037,0x10030034,0x1005000f,0x10040011,0x09060008,
0x00000000,0x1004001e,0x1005000f,0x0905000a,0x0e040051,0x09050009,0x10040012,0x10080010,
0x00000000,0x01010001,0x09020009,0x02020002,0x09020010,0x09060007,0x09050008,0x0e04007c,
0x00000000,0x0e05002b,0x0902000a,0x09060008,0x0901001f,0x0905000a,0x09050009,0x0905000b,
0x00000000,0x01010001,0x09060007,0x0e040076,0x10030035,0x09060008,0x0905000a,0x0905000b,
0x00000000,0x0e04006a,0x09060008,0x0905000b,0x0e040052,0x0905000b,0x0905000b,0x09090009,
0x00000000,0x01010001,0x04010002,0x02020002,0x0e04000d,0x05020003,0x05010004,0x03030003,
0x0e0100f4,0x10050010,0x04010003,0x06030004,0x10040017,0x06020005,0x05010005,0x04040004,
0x00000000,0x100300bc,0x05020003,0x10050014,0x10030036,0x05020004,0x05010006,0x07040005,
0x10020175,0x1004001f,0x05020004,0x07030006,0x0e040053,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0e050009,0x100300df,0x0e04000e,0x06030004,0x06020005,0x0e070010,
0x10010374,0x0e05002c,0x0e05000a,0x06030005,0x10040018,0x06020007,0x06020006,0x0e08000b,
0x00000000,0x100201bb,0x06030004,0x0e060019,0x10030037,0x06030005,0x06020007,0x0e07000c,
0x00000000,0x0e04006b,0x06030005,0x07030007,0x0e040054,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0e040011,0x02020002,0x0e04000f,0x1006000c,0x1005000d,0x03030003,
0x00000000,0x10050011,0x0e040012,0x07040005,0x10040019,0x07030006,0x1005000e,0x0e070013,
0x00000000,0x01010001,0x1006000c,0x0e050038,0x10030038,0x1006000d,0x1005000f,0x07040006,
0x00000000,0x10040020,0x1006000d,0x07030008,0x0e040055,0x07030007,0x10050010,0x1009000e,
0x00000000,0x01010001,0x1005000d,0x02020002,0x0e040010,0x07040005,0x07030006,0x0e06001f,
0x00000000,0x0e05002d,0x1005000e,0x07040006,0x1004001a,0x07030008,0x07030007,0x1008000f,
0x00000000,0x01010001,0x07040005,0x0e040077,0x10030039,0x07040006,0x07030008,0x0e07000d,
0x00000000,0x0e04006c,0x07040006,0x07030009,0x0e040056,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0e040011,0x10040032,0x10030033,0x03030003,
0x0e0100f5,0x10050012,0x04010003,0x1006000d,0x1004001b,0x1005000e,0x10030034,0x04040004,
0x00000000,0x01010001,0x10040032,0x10050015,0x1003003a,0x10040033,0x10030035,0x0e08000b,
0x10020176,0x10040021,0x10040033,0x0e07000c,0x0e040057,0x1005000f,0x10030036,0x10070034,
0x00000000,0x01010001,0x0e05000a,0x02020002,0x0e040012,0x1006000d,0x1005000e,0x0e05003e,
0x10010375,0x0e05002e,0x0e05000b,0x1006000e,0x1004001c,0x10050010,0x1005000f,0x0e08000c,
0x00000000,0x01010001,0x1006000d,0x0e06001a,0x1003003b,0x1006000e,0x10050010,0x0e07000e,
0x00000000,0x0e04006d,0x1006000e,0x0e07000d,0x0e040058,0x10050011,0x10050011,0x1009000f,
0x00000000,0x01010001,0x0e030020,0x02020002,0x0e040013,0x1005000f,0x10040010,0x03030003,
0x00000000,0x10050013,0x0e030021,0x0e08000b,0x1004001d,0x0e07000c,0x10040011,0x0e060022,
0x00000000,0x01010001,0x1005000f,0x0e050039,0x1003003c,0x10050010,0x10040012,0x0e08000c,
0x00000000,0x10040022,0x10050010,0x0e07000e,0x0e040059,0x0e07000d,0x10040013,0x10080011,
0x00000000,0x01010001,0x1005000e,0x02020002,0x0e040014,0x0e08000b,0x0e07000c,0x0e04007d,
0x00000000,0x0e05002f,0x1005000f,0x0e08000c,0x1004001e,0x0e07000e,0x0e07000d,0x10080010,
0x00000000,0x01010001,0x0e08000b,0x0e040078,0x1003003d,0x0e08000c,0x0e07000e,0x0e07000f,
0x00000000,0x0e04006e,0x0e08000c,0x0e07000f,0x0e04005a,0x0e07000f,0x0e07000f,0x0e0b000d,
0x00000000,0x01010001,0x04010002,0x02020002,0x10050010,0x05020003,0x05010004,0x03030003,
0x0e0100f6,0x10050014,0x04010003,0x1005001c,0x1004001f,0x1004001d,0x05010005,0x04040004,
0x00000000,0x100300bd,0x05020003,0x10050016,0x1003003e,0x05020004,0x05010006,0x0e07000d,
0x10020177,0x10040023,0x05020004,0x0e06000e,0x0e04005b,0x1004001e,0x05010007,0x05050005,
0x00000000,0x01010001,0x1006000c,0x02020002,0x10050011,0x1005001c,0x1004001d,0x0e070011,
0x10010376,0x0e050030,0x1006000d,0x1005001d,0x10040020,0x1004001f,0x1004001e,0x1009000e,
0x00000000,0x100201bc,0x1005001c,0x0e06001b,0x1003003f,0x1005001d,0x1004001f,0x1008000f,
0x00000000,0x0e04006f,0x1005001d,0x0e06000f,0x0e04005c,0x10040020,0x10040020,0x1008001e,
0x00000000,0x01010001,0x0e02003f,0x02020002,0x10050012,0x1006000d,0x1005000e,0x03030003,
0x00000000,0x10050015,0x0e020040,0x0e07000d,0x10040021,0x0e06000e,0x1005000f,0x0e050041,
0x00000000,0x01010001,0x1006000d,0x0e05003a,0x10030040,0x1006000e,0x10050010,0x0e07000e,
0x00000000,0x10040024,0x1006000e,0x0e060010,0x0e04005d,0x0e06000f,0x10050011,0x1009000f,
0x00000000,0x01010001,0x1005000f,0x02020002,0x10050013,0x0e07000d,0x0e06000e,0x0e060020,
0x00000000,0x0e050031,0x10050010,0x0e07000e,0x10040022,0x0e060010,0x0e06000f,0x10080011,
0x00000000,0x01010001,0x0e07000d,0x0e040079,0x10030041,0x0e07000e,0x0e060010,0x10080010,
0x00000000,0x0e040070,0x0e07000e,0x0e060011,0x0e04005e,0x0e060011,0x0e060011,0x0e0a000f,
0x00000000,0x01010001,0x04010002,0x02020002,0x10050014,0x10030071,0x10020072,0x03030003,
0x0e0100f7,0x10050016,0x04010003,0x0e07000c,0x10040023,0x0e06000d,0x10020073,0x04040004,
0x00000000,0x01010001,0x10030071,0x10050017,0x10030042,0x10030072,0x10020074,0x1009000e,
0x10020178,0x10040025,0x10030072,0x1008000f,0x0e04005f,0x0e06000e,0x10020075,0x10060073,
0x00000000,0x01010001,0x1006000d,0x02020002,0x10050015,0x0e07000c,0x0e06000d,0x0e05003f,
0x10010377,0x0e050032,0x1006000e,0x0e07000d,0x10040024,0x0e06000f,0x0e06000e,0x1009000f,
0x00000000,0x01010001,0x0e07000c,0x0e06001c,0x10030043,0x0e07000d,0x0e06000f,0x10080011,
0x00000000,0x0e040071,0x0e07000d,0x10080010,0x0e040060,0x0e060010,0x0e060010,0x0e0a000e,
0x00000000,0x01010001,0x10030060,0x02020002,0x10050016,0x10050010,0x10040011,0x03030003,
0x00000000,0x10050017,0x10030061,0x1009000e,0x10040025,0x1008000f,0x10040012,0x10060062,
0x00000000,0x01010001,0x10050010,0x0e05003b,0x10030044,0x10050011,0x10040013,0x1009000f,
0x00000000,0x10040026,0x10050011,0x10080011,0x0e040061,0x10080010,0x10040014,0x10080012,
0x00000000,0x01010001,0x10050010,0x02020002,0x10050017,0x1009000e,0x1008000f,0x0e04007e,
0x00000000,0x0e050033,0x10050011,0x1009000f,0x10040026,0x10080011,0x10080010,0x10080012,
0x00000000,0x01010001,0x1009000e,0x0e04007a,0x10030045,0x1009000f,0x10080011,0x10080012,
0x00000000,0x0e040072,0x1009000f,0x10080012,0x0e040062,0x10080012,0x10080012,0x100c0010,
0x00000000,0x01010001,0x04010002,0x02020002,0x08010008,0x05020003,0x05010004,0x03030003,
0x0e0100f8,0x0e030010,0x04010003,0x06030004,0x0e020017,0x06020005,0x05010005,0x04040004,
0x00000000,0x100300be,0x05020003,0x0e07000e,0x0e050024,0x05020004,0x05010006,0x07040005,
0x10020179,0x0e060015,0x05020004,0x07030006,0x0e040063,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x08020004,0x100300e0,0x08010009,0x06030004,0x06020005,0x0e070012,
0x10010378,0x0e050034,0x08020005,0x06030005,0x0e020018,0x06020007,0x06020006,0x08050006,
0x00000000,0x100201bd,0x06030004,0x0e06001d,0x0e050025,0x06030005,0x06020007,0x08040007,
0x00000000,0x0e040073,0x06030005,0x07030007,0x0e040064,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0e040012,0x02020002,0x0801000a,0x0e04000c,0x0e03000d,0x03030003,
0x00000000,0x0e030011,0x0e040013,0x07040005,0x0e020019,0x07030006,0x0e03000e,0x0e070014,
0x00000000,0x01010001,0x0e04000c,0x0e05003c,0x0e050026,0x0e04000d,0x0e03000f,0x07040006,
0x00000000,0x0e060016,0x0e04000d,0x07030008,0x0e040065,0x07030007,0x0e030010,0x0e07000e,
0x00000000,0x01010001,0x0e03000d,0x02020002,0x0801000b,0x07040005,0x07030006,0x0e060021,
0x00000000,0x0e050035,0x0e03000e,0x07040006,0x0e02001a,0x07030008,0x07030007,0x0e06000f,
0x00000000,0x01010001,0x07040005,0x0e04007b,0x0e050027,0x07040006,0x07030008,0x08040008,
0x00000000,0x0e040074,0x07040006,0x07030009,0x0e040066,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0801000c,0x0e040013,0x0e030014,0x03030003,
0x0e0100f9,0x0e030012,0x04010003,0x0e08000a,0x0e02001b,0x0e07000b,0x0e030015,0x04040004,
0x00000000,0x01010001,0x0e040013,0x0e07000f,0x0e050028,0x0e040014,0x0e030016,0x08050006,
0x1002017a,0x0e060017,0x0e040014,0x08040007,0x0e040067,0x0e07000c,0x0e030017,0x0e070015,
0x00000000,0x01010001,0x08020005,0x02020002,0x0801000d,0x0e08000a,0x0e07000b,0x0e050040,
0x10010379,0x0e050036,0x08020006,0x0e08000b,0x0e02001c,0x0e07000d,0x0e07000c,0x08050007,
0x00000000,0x01010001,0x0e08000a,0x0e06001e,0x0e050029,0x0e08000b,0x0e07000d,0x08040009,
0x00000000,0x0e040075,0x0e08000b,0x08040008,0x0e040068,0x0e07000e,0x0e07000e,0x0e0b000c,
0x00000000,0x01010001,0x0e030021,0x02020002,0x0801000e,0x0e07000b,0x0e06000c,0x03030003,
0x00000000,0x0e030013,0x0e030022,0x08050006,0x0e02001d,0x08040007,0x0e06000d,0x0e060023,
0x00000000,0x01010001,0x0e07000b,0x0e05003d,0x0e05002a,0x0e07000c,0x0e06000e,0x08050007,
0x00000000,0x0e060018,0x0e07000c,0x08040009,0x0e040069,0x08040008,0x0e06000f,0x0e0a000d,
0x00000000,0x01010001,0x0e03000e,0x02020002,0x0801000f,0x08050006,0x08040007,0x0e04007f,
0x00000000,0x0e050037,0x0e03000f,0x08050007,0x0e02001e,0x08040009,0x08040008,0x0e060010,
0x00000000,0x01010001,0x08050006,0x0e04007c,0x0e05002b,0x08050007,0x08040009,0x0804000a,
0x00000000,0x0e040076,0x08050007,0x0804000a,0x0e04006a,0x0804000a,0x0804000a,0x08080008,
0x00000000,0x01010001,0x04010002,0x02020002,0x0e030010,0x05020003,0x05010004,0x03030003,
0x0e0100fa,0x0e030014,0x04010003,0x1004003b,0x0e02001f,0x1003003c,0x05010005,0x04040004,
0x00000000,0x100300bf,0x05020003,0x0e070010,0x0e05002c,0x05020004,0x05010006,0x0e08000b,
0x1002017b,0x0e060019,0x05020004,0x0e07000c,0x0e04006b,0x1003003d,0x05010007,0x05050005,
0x00000000,0x01010001,0x0e04000c,0x02020002,0x0e030011,0x1004003b,0x1003003c,0x0e070013,
0x1001037a,0x0e050038,0x0e04000d,0x1004003c,0x0e020020,0x1003003e,0x1003003d,0x0e07000e,
0x00000000,0x100201be,0x1004003b,0x0e06001f,0x0e05002d,0x1004003c,0x1003003e,0x0e06000f,
0x00000000,0x0e040077,0x1004003c,0x0e07000d,0x0e04006c,0x1003003f,0x1003003f,0x1007003d,
0x00000000,0x01010001,0x0e020040,0x02020002,0x0e030012,0x0e04000d,0x0e03000e,0x03030003,
0x00000000,0x0e030015,0x0e020041,0x0e08000b,0x0e020021,0x0e07000c,0x0e03000f,0x0e050042,
0x00000000,0x01010001,0x0e04000d,0x0e05003e,0x0e05002e,0x0e04000e,0x0e030010,0x0e08000c,
0x00000000,0x0e06001a,0x0e04000e,0x0e07000e,0x0e04006d,0x0e07000d,0x0e030011,0x0e07000f,
0x00000000,0x01010001,0x0e03000f,0x02020002,0x0e030013,0x0e08000b,0x0e07000c,0x0e060022,
0x00000000,0x0e050039,0x0e030010,0x0e08000c,0x0e020022,0x0e07000e,0x0e07000d,0x0e060011,
0x00000000,0x01010001,0x0e08000b,0x0e04007d,0x0e05002f,0x0e08000c,0x0e07000e,0x0e060010,
0x00000000,0x0e040078,0x0e08000c,0x0e07000f,0x0e04006e,0x0e07000f,0x0e07000f,0x0e0b000d,
0x00000000,0x01010001,0x04010002,0x02020002,0x0e030014,0x0e030022,0x0e020023,0x03030003,
0x0e0100fb,0x0e030016,0x04010003,0x0e07000d,0x0e020023,0x0e06000e,0x0e020024,0x04040004,
0x00000000,0x01010001,0x0e030022,0x0e070011,0x0e050030,0x0e030023,0x0e020025,0x0e07000e,
0x1002017c,0x0e06001b,0x0e030023,0x0e06000f,0x0e04006f,0x0e06000f,0x0e020026,0x0e060024,
0x00000000,0x01010001,0x0e04000d,0x02020002,0x0e030015,0x0e07000d,0x0e06000e,0x0e050041,
0x1001037b,0x0e05003a,0x0e04000e,0x0e07000e,0x0e020024,0x0e060010,0x0e06000f,0x0e07000f,
0x00000000,0x01010001,0x0e07000d,0x0e060020,0x0e050031,0x0e07000e,0x0e060010,0x0e060011,
0x00000000,0x0e040079,0x0e07000e,0x0e060010,0x0e040070,0x0e060011,0x0e060011,0x0e0a000f,
0x00000000,0x01010001,0x10030061,0x02020002,0x0e030016,0x0e07000c,0x0e06000d,0x03030003,
0x00000000,0x0e030017,0x10030062,0x0e07000e,0x0e020025,0x0e06000f,0x0e06000e,0x10060063,
0x00000000,0x01010001,0x0e07000c,0x0e05003f,0x0e050032,0x0e07000d,0x0e06000f,0x0e07000f,
0x00000000,0x0e06001c,0x0e07000d,0x0e060011,0x0e040071,0x0e060010,0x0e060010,0x0e0a000e,
0x00000000,0x01010001,0x0e030010,0x02020002,0x0e030017,0x0e07000e,0x0e06000f,0x0e040080,
0x00000000,0x0e05003b,0x0e030011,0x0e07000f,0x0e020026,0x0e060011,0x0e060010,0x0e060012,
0x00000000,0x01010001,0x0e07000e,0x0e04007e,0x0e050033,0x0e07000f,0x0e060011,0x0e060012,
0x00000000,0x0e04007a,0x0e07000f,0x0e060012,0x0e040072,0x0e060012,0x0e060012,0x0e0a0010,
0x00000000,0x01010001,0x04010002,0x02020002,0x0e030010,0x05020003,0x05010004,0x03030003,
0x0e0100fc,0x0e07000e,0x04010003,0x06030004,0x0e060015,0x06020005,0x05010005,0x04040004,
0x00000000,0x100300c0,0x05020003,0x0e070012,0x0e050034,0x05020004,0x05010006,0x07040005,
0x1002017d,0x0e06001d,0x05020004,0x07030006,0x0e040073,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0e04000c,0x100300e1,0x0e030011,0x06030004,0x06020005,0x0e070014,
0x1001037c,0x0e05003c,0x0e04000d,0x06030005,0x0e060016,0x06020007,0x06020006,0x0e07000e,
0x00000000,0x100201bf,0x06030004,0x0e060021,0x0e050035,0x06030005,0x06020007,0x0e06000f,
0x00000000,0x0e04007b,0x06030005,0x07030007,0x0e040074,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0e040013,0x02020002,0x0e030012,0x0e08000a,0x0e07000b,0x03030003,
0x00000000,0x0e07000f,0x0e040014,0x07040005,0x0e060017,0x07030006,0x0e07000c,0x0e070015,
0x00000000,0x01010001,0x0e08000a,0x0e050040,0x0e050036,0x0e08000b,0x0e07000d,0x07040006,
0x00000000,0x0e06001e,0x0e08000b,0x07030008,0x0e040075,0x07030007,0x0e07000e,0x0e0b000c,
0x00000000,0x01010001,0x0e07000b,0x02020002,0x0e030013,0x07040005,0x07030006,0x0e060023,
0x00000000,0x0e05003d,0x0e07000c,0x07040006,0x0e060018,0x07030008,0x07030007,0x0e0a000d,
0x00000000,0x01010001,0x07040005,0x0e04007f,0x0e050037,0x07040006,0x07030008,0x0e060010,
0x00000000,0x0e04007c,0x07040006,0x07030009,0x0e040076,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0e030014,0x10040033,0x10030034,0x03030003,
0x0e0100fd,0x0e070010,0x04010003,0x0e08000b,0x0e060019,0x0e07000c,0x10030035,0x04040004,
0x00000000,0x01010001,0x10040033,0x0e070013,0x0e050038,0x10040034,0x10030036,0x0e07000e,
0x1002017e,0x0e06001f,0x10040034,0x0e06000f,0x0e040077,0x0e07000d,0x10030037,0x10070035,
0x00000000,0x01010001,0x0e04000d,0x02020002,0x0e030015,0x0e08000b,0x0e07000c,0x0e050042,
0x1001037d,0x0e05003e,0x0e04000e,0x0e08000c,0x0e06001a,0x0e07000e,0x0e07000d,0x0e07000f,
0x00000000,0x01010001,0x0e08000b,0x0e060022,0x0e050039,0x0e08000c,0x0e07000e,0x0e060011,
0x00000000,0x0e04007d,0x0e08000c,0x0e060010,0x0e040078,0x0e07000f,0x0e07000f,0x0e0b000d,
0x00000000,0x01010001,0x0e030022,0x02020002,0x0e030016,0x0e07000d,0x0e06000e,0x03030003,
0x00000000,0x0e070011,0x0e030023,0x0e07000e,0x0e06001b,0x0e06000f,0x0e06000f,0x0e060024,
0x00000000,0x01010001,0x0e07000d,0x0e050041,0x0e05003a,0x0e07000e,0x0e060010,0x0e07000f,
0x00000000,0x0e060020,0x0e07000e,0x0e060011,0x0e040079,0x0e060010,0x0e060011,0x0e0a000f,
0x00000000,0x01010001,0x0e07000c,0x02020002,0x0e030017,0x0e07000e,0x0e06000f,0x0e040081,
0x00000000,0x0e05003f,0x0e07000d,0x0e07000f,0x0e06001c,0x0e060011,0x0e060010,0x0e0a000e,
0x00000000,0x01010001,0x0e07000e,0x0e040080,0x0e05003b,0x0e07000f,0x0e060011,0x0e060012,
0x00000000,0x0e04007e,0x0e07000f,0x0e060012,0x0e04007a,0x0e060012,0x0e060012,0x0e0a0010,
0x00000000,0x01010001,0x04010002,0x02020002,0x0e07000e,0x05020003,0x05010004,0x03030003,
0x0e0100fe,0x0e070012,0x04010003,0x06030004,0x0e06001d,0x06020005,0x05010005,0x04040004,
0x00000000,0x100300c1,0x05020003,0x0e070014,0x0e05003c,0x05020004,0x05010006,0x0e07000e,
0x1002017f,0x0e060021,0x05020004,0x0e06000f,0x0e04007b,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0e08000a,0x02020002,0x0e07000f,0x06030004,0x06020005,0x0e070015,
0x1001037e,0x0e050040,0x0e08000b,0x06030005,0x0e06001e,0x06020007,0x06020006,0x0e0b000c,
0x00000000,0x100201c0,0x06030004,0x0e060023,0x0e05003d,0x06030005,0x06020007,0x0e0a000d,
0x00000000,0x0e04007f,0x06030005,0x0e060010,0x0e04007c,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0e020041,0x02020002,0x0e070010,0x0e08000b,0x0e07000c,0x03030003,
0x00000000,0x0e070013,0x0e020042,0x0e07000e,0x0e06001f,0x0e06000f,0x0e07000d,0x0e050043,
0x00000000,0x01010001,0x0e08000b,0x0e050042,0x0e05003e,0x0e08000c,0x0e07000e,0x0e07000f,
0x00000000,0x0e060022,0x0e08000c,0x0e060011,0x0e04007d,0x0e060010,0x0e07000f,0x0e0b000d,
0x00000000,0x01010001,0x0e07000d,0x02020002,0x0e070011,0x0e07000e,0x0e06000f,0x0e060024,
0x00000000,0x0e050041,0x0e07000e,0x0e07000f,0x0e060020,0x0e060011,0x0e060010,0x0e0a000f,
0x00000000,0x01010001,0x0e07000e,0x0e040081,0x0e05003f,0x0e07000f,0x0e060011,0x0e0a000e,
0x00000000,0x0e040080,0x0e07000f,0x0e060012,0x0e04007e,0x0e060012,0x0e060012,0x0e0a0010,
0x00000000,0x01010001,0x04010002,0x02020002,0x0e070012,0x10030072,0x10020073,0x03030003,
0x0e0100ff,0x0e070014,0x04010003,0x0e07000e,0x0e060021,0x0e06000f,0x10020074,0x04040004,
0x00000000,0x01010001,0x10030072,0x0e070015,0x0e050040,0x10030073,0x10020075,0x0e0b000c,
0x10020180,0x0e060023,0x10030073,0x0e0a000d,0x0e04007f,0x0e060010,0x10020076,0x10060074,
0x00000000,0x01010001,0x0e08000b,0x02020002,0x0e070013,0x0e07000e,0x0e06000f,0x0e050043,
0x1001037f,0x0e050042,0x0e08000c,0x0e07000f,0x0e060022,0x0e060011,0x0e060010,0x0e0b000d,
0x00000000,0x01010001,0x0e07000e,0x0e060024,0x0e050041,0x0e07000f,0x0e060011,0x0e0a000f,
0x00000000,0x0e040081,0x0e07000f,0x0e0a000e,0x0e040080,0x0e060012,0x0e060012,0x0e0a0010,
0x00000000,0x01010001,0x10030062,0x02020002,0x0e070014,0x0e07000e,0x0e06000f,0x03030003,
0x00000000,0x0e070015,0x10030063,0x0e0b000c,0x0e060023,0x0e0a000d,0x0e060010,0x10060064,
0x00000000,0x01010001,0x0e07000e,0x0e050043,0x0e050042,0x0e07000f,0x0e060011,0x0e0b000d,
0x00000000,0x0e060024,0x0e07000f,0x0e0a000f,0x0e040081,0x0e0a000e,0x0e060012,0x0e0a0010,
0x00000000,0x01010001,0x0e07000e,0x02020002,0x0e070015,0x0e0b000c,0x0e0a000d,0x0e040082,
0x00000000,0x0e050043,0x0e07000f,0x0e0b000d,0x0e060024,0x0e0a000f,0x0e0a000e,0x0e0a0010,
0x00000000,0x01010001,0x0e0b000c,0x0e040082,0x0e050043,0x0e0b000d,0x0e0a000f,0x0e0a0010,
0x00000000,0x0e040082,0x0e0b000d,0x0e0a0010,0x0e040082,0x0e0a0010,0x0e0a0010,0x0e0e000e,
0x00000000,0x01010001,0x04010002,0x02020002,0x08010008,0x05020003,0x05010004,0x03030003,
0x10030082,0x09020009,0x04010003,0x06030004,0x09010010,0x06020005,0x05010005,0x04040004,
0x00000000,0x100300c2,0x05020003,0x0a03000a,0x0a010020,0x05020004,0x05010006,0x07040005,
0x10020181,0x0a020011,0x05020004,0x07030006,0x0b010040,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x08020004,0x100300e2,0x08010009,0x06030004,0x06020005,0x0b04000b,
0x10010380,0x0b020021,0x08020005,0x06030005,0x09010011,0x06020007,0x06020006,0x08050006,
0x00000000,0x100201c1,0x06030004,0x0b030012,0x0a010021,0x06030005,0x06020007,0x08040007,
0x00000000,0x0c020041,0x06030005,0x07030007,0x0b010041,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0c02000a,0x02020002,0x0801000a,0x09030005,0x09020006,0x03030003,
0x00000000,0x0902000a,0x0c02000b,0x07040005,0x09010012,0x07030006,0x09020007,0x0c05000c,
0x00000000,0x01010001,0x09030005,0x0c030022,0x0a010022,0x09030006,0x09020008,0x07040006,
0x00000000,0x0a020012,0x09030006,0x07030008,0x0b010042,0x07030007,0x09020009,0x09060007,
0x00000000,0x01010001,0x09020006,0x02020002,0x0801000b,0x07040005,0x07030006,0x0c040013,
0x00000000,0x0b020022,0x09020007,0x07040006,0x09010013,0x07030008,0x07030007,0x09050008,
0x00000000,0x01010001,0x07040005,0x0d030042,0x0a010023,0x07040006,0x07030008,0x08040008,
0x00000000,0x0c020042,0x07040006,0x07030009,0x0b010043,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0801000c,0x0d03000b,0x0d02000c,0x03030003,
0x10030083,0x0902000b,0x04010003,0x0a040006,0x09010014,0x0a030007,0x0d02000d,0x04040004,
0x00000000,0x01010001,0x0d03000b,0x0a03000b,0x0a010024,0x0d03000c,0x0d02000e,0x08050006,
0x10020182,0x0a020013,0x0d03000c,0x08040007,0x0b010044,0x0a030008,0x0d02000f,0x0d06000d,
0x00000000,0x01010001,0x08020005,0x02020002,0x0801000d,0x0a040006,0x0a030007,0x0d040023,
0x10010381,0x0b020023,0x08020006,0x0a040007,0x09010015,0x0a030009,0x0a030008,0x08050007,
0x00000000,0x01010001,0x0a040006,0x0b030013,0x0a010025,0x0a040007,0x0a030009,0x08040009,
0x00000000,0x0c020043,0x0a040007,0x08040008,0x0b010045,0x0a03000a,0x0a03000a,0x0a070008,
0x00000000,0x01010001,0x0d020012,0x02020002,0x0801000e,0x0a030007,0x0a020008,0x03030003,
0x00000000,0x0902000c,0x0d020013,0x08050006,0x09010016,0x08040007,0x0a020009,0x0d050014,
0x00000000,0x01010001,0x0a030007,0x0c030023,0x0a010026,0x0a030008,0x0a02000a,0x08050007,
0x00000000,0x0a020014,0x0a030008,0x08040009,0x0b010046,0x08040008,0x0a02000b,0x0a060009,
0x00000000,0x01010001,0x09020007,0x02020002,0x0801000f,0x08050006,0x08040007,0x10060045,
0x00000000,0x0b020024,0x09020008,0x08050007,0x09010017,0x08040009,0x08040008,0x09050009,
0x00000000,0x01010001,0x08050006,0x0d030043,0x0a010027,0x08050007,0x08040009,0x0804000a,
0x00000000,0x0c020044,0x08050007,0x0804000a,0x0b010047,0x0804000a,0x0804000a,0x08080008,
0x00000000,0x01010001,0x04010002,0x02020002,0x09020009,0x05020003,0x05010004,0x03030003,
0x10030084,0x0902000d,0x04010003,0x1006000e,0x09010018,0x1005000f,0x05010005,0x04040004,
0x00000000,0x100300c3,0x05020003,0x0a03000c,0x0a010028,0x05020004,0x05010006,0x0b050007,
0x10020183,0x0a020015,0x05020004,0x0b040008,0x0b010048,0x10050010,0x05010007,0x05050005,
0x00000000,0x01010001,0x09030005,0x02020002,0x0902000a,0x1006000e,0x1005000f,0x0b04000c,
0x10010382,0x0b020025,0x09030006,0x1006000f,0x09010019,0x10050011,0x10050010,0x09060007,
0x00000000,0x100201c2,0x1006000e,0x0b030014,0x0a010029,0x1006000f,0x10050011,0x09050008,
0x00000000,0x0c020045,0x1006000f,0x0b040009,0x0b010049,0x10050012,0x10050012,0x10090010,
0x00000000,0x01010001,0x10040024,0x02020002,0x0902000b,0x09030006,0x09020007,0x03030003,
0x00000000,0x0902000e,0x10040025,0x0b050007,0x0901001a,0x0b040008,0x09020008,0x10070026,
0x00000000,0x01010001,0x09030006,0x0c030024,0x0a01002a,0x09030007,0x09020009,0x0b050008,
0x00000000,0x0a020016,0x09030007,0x0b04000a,0x0b01004a,0x0b040009,0x0902000a,0x09060008,
0x00000000,0x01010001,0x09020008,0x02020002,0x0902000c,0x0b050007,0x0b040008,0x0c040014,
0x00000000,0x0b020026,0x09020009,0x0b050008,0x0901001b,0x0b04000a,0x0b040009,0x0905000a,
0x00000000,0x01010001,0x0b050007,0x0d030044,0x0a01002b,0x0b050008,0x0b04000a,0x09050009,
0x00000000,0x0c020046,0x0b050008,0x0b04000b,0x0b01004b,0x0b04000b,0x0b04000b,0x0b080009,
0x00000000,0x01010001,0x04010002,0x02020002,0x0902000d,0x10050015,0x10040016,0x03030003,
0x10030085,0x0902000f,0x04010003,0x0b040008,0x0901001c,0x0b030009,0x10040017,0x04040004,
0x00000000,0x01010001,0x10050015,0x0a03000d,0x0a01002c,0x10050016,0x10040018,0x09060007,
0x10020184,0x0a020017,0x10050016,0x09050008,0x0b01004c,0x0b03000a,0x10040019,0x10080017,
0x00000000,0x01010001,0x09030006,0x02020002,0x0902000e,0x0b040008,0x0b030009,0x0d040024,
0x10010383,0x0b020027,0x09030007,0x0b040009,0x0901001d,0x0b03000b,0x0b03000a,0x09060008,
0x00000000,0x01010001,0x0b040008,0x0b030015,0x0a01002d,0x0b040009,0x0b03000b,0x0905000a,
0x00000000,0x0c020047,0x0b040009,0x09050009,0x0b01004d,0x0b03000c,0x0b03000c,0x0b07000a,
0x00000000,0x01010001,0x10030063,0x02020002,0x0902000f,0x0a030008,0x0a020009,0x03030003,
0x00000000,0x09020010,0x10030064,0x09060007,0x0901001e,0x09050008,0x0a02000a,0x10060065,
0x00000000,0x01010001,0x0a030008,0x0c030025,0x0a01002e,0x0a030009,0x0a02000b,0x09060008,
0x00000000,0x0a020018,0x0a030009,0x0905000a,0x0b01004e,0x09050009,0x0a02000c,0x0a06000a,
0x00000000,0x01010001,0x09020009,0x02020002,0x09020010,0x09060007,0x09050008,0x10060046,
0x00000000,0x0b020028,0x0902000a,0x09060008,0x0901001f,0x0905000a,0x09050009,0x0905000b,
0x00000000,0x01010001,0x09060007,0x0d030045,0x0a01002f,0x09060008,0x0905000a,0x0905000b,
0x00000000,0x0c020048,0x09060008,0x0905000b,0x0b01004f,0x0905000b,0x0905000b,0x09090009,
0x00000000,0x01010001,0x04010002,0x02020002,0x0c02000a,0x05020003,0x05010004,0x03030003,
0x10030086,0x0a03000a,0x04010003,0x06030004,0x0a020011,0x06020005,0x05010005,0x04040004,
0x00000000,0x100300c4,0x05020003,0x0a03000e,0x0a010030,0x05020004,0x05010006,0x10060012,
0x10020185,0x0a020019,0x05020004,0x10050013,0x0b010050,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0c030006,0x100300e3,0x0c02000b,0x06030004,0x06020005,0x0b04000d,
0x10010384,0x0b020029,0x0c030007,0x06030005,0x0a020012,0x06020007,0x06020006,0x0c060008,
0x00000000,0x100201c3,0x06030004,0x0b030016,0x0a010031,0x06030005,0x06020007,0x0c050009,
0x00000000,0x0c020049,0x06030005,0x10050014,0x0b010051,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0c02000b,0x02020002,0x0c02000c,0x0a040006,0x0a030007,0x03030003,
0x00000000,0x0a03000b,0x0c02000c,0x10060012,0x0a020013,0x10050013,0x0a030008,0x0c05000d,
0x00000000,0x01010001,0x0a040006,0x0c030026,0x0a010032,0x0a040007,0x0a030009,0x10060013,
0x00000000,0x0a02001a,0x0a040007,0x10050015,0x0b010052,0x10050014,0x0a03000a,0x0a070008,
0x00000000,0x01010001,0x0a030007,0x02020002,0x0c02000d,0x10060012,0x10050013,0x0c040015,
0x00000000,0x0b02002a,0x0a030008,0x10060013,0x0a020014,0x10050015,0x10050014,0x0a060009,
0x00000000,0x01010001,0x10060012,0x0d030046,0x0a010033,0x10060013,0x10050015,0x0c05000a,
0x00000000,0x0c02004a,0x10060013,0x10050016,0x0b010053,0x10050016,0x10050016,0x10090014,
0x00000000,0x01010001,0x04010002,0x02020002,0x0c02000e,0x10040034,0x10030035,0x03030003,
0x10030087,0x0a03000c,0x04010003,0x0a040007,0x0a020015,0x0a030008,0x10030036,0x04040004,
0x00000000,0x01010001,0x10040034,0x0a03000f,0x0a010034,0x10040035,0x10030037,0x0c060008,
0x10020186,0x0a02001b,0x10040035,0x0c050009,0x0b010054,0x0a030009,0x10030038,0x10070036,
0x00000000,0x01010001,0x0c030007,0x02020002,0x0c02000f,0x0a040007,0x0a030008,0x0d040025,
0x10010385,0x0b02002b,0x0c030008,0x0a040008,0x0a020016,0x0a03000a,0x0a030009,0x0c060009,
0x00000000,0x01010001,0x0a040007,0x0b030017,0x0a010035,0x0a040008,0x0a03000a,0x0c05000b,
0x00000000,0x0c02004b,0x0a040008,0x0c05000a,0x0b010055,0x0a03000b,0x0a03000b,0x0a070009,
0x00000000,0x01010001,0x0d020013,0x02020002,0x0c020010,0x0a030009,0x0a02000a,0x03030003,
0x00000000,0x0a03000d,0x0d020014,0x0c060008,0x0a020017,0x0c050009,0x0a02000b,0x0d050015,
0x00000000,0x01010001,0x0a030009,0x0c030027,0x0a010036,0x0a03000a,0x0a02000c,0x0c060009,
0x00000000,0x0a02001c,0x0a03000a,0x0c05000b,0x0b010056,0x0c05000a,0x0a02000d,0x0a06000b,
0x00000000,0x01010001,0x0a030008,0x02020002,0x0c020011,0x0c060008,0x0c050009,0x10060047,
0x00000000,0x0b02002c,0x0a030009,0x0c060009,0x0a020018,0x0c05000b,0x0c05000a,0x0a06000a,
0x00000000,0x01010001,0x0c060008,0x0d030047,0x0a010037,0x0c060009,0x0c05000b,0x0c05000c,
0x00000000,0x0c02004c,0x0c060009,0x0c05000c,0x0b010057,0x0c05000c,0x0c05000c,0x0c09000a,
0x00000000,0x01010001,0x04010002,0x02020002,0x0a03000a,0x05020003,0x05010004,0x03030003,
0x10030088,0x0a03000e,0x04010003,0x1005001d,0x0a020019,0x1004001e,0x05010005,0x04040004,
0x00000000,0x100300c5,0x05020003,0x0a030010,0x0a010038,0x05020004,0x05010006,0x0c050009,
0x10020187,0x0a02001d,0x05020004,0x0c04000a,0x0b010058,0x1004001f,0x05010007,0x05050005,
0x00000000,0x01010001,0x0a040006,0x02020002,0x0a03000b,0x1005001d,0x1004001e,0x0b04000e,
0x10010386,0x0b02002d,0x0a040007,0x1005001e,0x0a02001a,0x10040020,0x1004001f,0x0a070008,
0x00000000,0x100201c4,0x1005001d,0x0b030018,0x0a010039,0x1005001e,0x10040020,0x0a060009,
0x00000000,0x0c02004d,0x1005001e,0x0c04000b,0x0b010059,0x10040021,0x10040021,0x1008001f,
0x00000000,0x01010001,0x10040025,0x02020002,0x0a03000c,0x0a040007,0x0a030008,0x03030003,
0x00000000,0x0a03000f,0x10040026,0x0c050009,0x0a02001b,0x0c04000a,0x0a030009,0x10070027,
0x00000000,0x01010001,0x0a040007,0x0c030028,0x0a01003a,0x0a040008,0x0a03000a,0x0c05000a,
0x00000000,0x0a02001e,0x0a040008,0x0c04000c,0x0b01005a,0x0c04000b,0x0a03000b,0x0a070009,
0x00000000,0x01010001,0x0a030009,0x02020002,0x0a03000d,0x0c050009,0x0c04000a,0x0c040016,
0x00000000,0x0b02002e,0x0a03000a,0x0c05000a,0x0a02001c,0x0c04000c,0x0c04000b,0x0a06000b,
0x00000000,0x01010001,0x0c050009,0x0d030048,0x0a01003b,0x0c05000a,0x0c04000c,0x0a06000a,
0x00000000,0x0c02004e,0x0c05000a,0x0c04000d,0x0b01005b,0x0c04000d,0x0c04000d,0x0c08000b,
0x00000000,0x01010001,0x04010002,0x02020002,0x0a03000e,0x10030073,0x10020074,0x03030003,
0x10030089,0x0a030010,0x04010003,0x0b040009,0x0a02001d,0x0b03000a,0x10020075,0x04040004,
0x00000000,0x01010001,0x10030073,0x0a030011,0x0a01003c,0x10030074,0x10020076,0x0a070008,
0x10020188,0x0a02001f,0x10030074,0x0a060009,0x0b01005c,0x0b03000b,0x10020077,0x10060075,
0x00000000,0x01010001,0x0a040007,0x02020002,0x0a03000f,0x0b040009,0x0b03000a,0x0d040026,
0x10010387,0x0b02002f,0x0a040008,0x0b04000a,0x0a02001e,0x0b03000c,0x0b03000b,0x0a070009,
0x00000000,0x01010001,0x0b040009,0x0b030019,0x0a01003d,0x0b04000a,0x0b03000c,0x0a06000b,
0x00000000,0x0c02004f,0x0b04000a,0x0a06000a,0x0b01005d,0x0b03000d,0x0b03000d,0x0b07000b,
0x00000000,0x01010001,0x10030064,0x02020002,0x0a030010,0x0a03000a,0x0a02000b,0x03030003,
0x00000000,0x0a030011,0x10030065,0x0a070008,0x0a02001f,0x0a060009,0x0a02000c,0x10060066,
0x00000000,0x01010001,0x0a03000a,0x0c030029,0x0a01003e,0x0a03000b,0x0a02000d,0x0a070009,
0x00000000,0x0a020020,0x0a03000b,0x0a06000b,0x0b01005e,0x0a06000a,0x0a02000e,0x0a06000c,
0x00000000,0x01010001,0x0a03000a,0x02020002,0x0a030011,0x0a070008,0x0a060009,0x10060048,
0x00000000,0x0b020030,0x0a03000b,0x0a070009,0x0a020020,0x0a06000b,0x0a06000a,0x0a06000c,
0x00000000,0x01010001,0x0a070008,0x0d030049,0x0a01003f,0x0a070009,0x0a06000b,0x0a06000c,
0x00000000,0x0c020050,0x0a070009,0x0a06000c,0x0b01005f,0x0a06000c,0x0a06000c,0x0a0a000a,
0x00000000,0x01010001,0x04010002,0x02020002,0x10020016,0x05020003,0x05010004,0x03030003,
0x1003008a,0x0d03000b,0x04010003,0x06030004,0x0d020012,0x06020005,0x05010005,0x04040004,
0x00000000,0x100300c6,0x05020003,0x0b04000b,0x0b020021,0x05020004,0x05010006,0x07040005,
0x10020189,0x0b030012,0x05020004,0x07030006,0x0b010060,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x10030012,0x100300e4,0x10020017,0x06030004,0x06020005,0x0b04000f,
0x10010388,0x0b020031,0x10030013,0x06030005,0x0d020013,0x06020007,0x06020006,0x10060014,
0x00000000,0x100201c5,0x06030004,0x0b03001a,0x0b020022,0x06030005,0x06020007,0x10050015,
0x00000000,0x0c020051,0x06030005,0x07030007,0x0b010061,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0c02000c,0x02020002,0x10020018,0x0d040007,0x0d030008,0x03030003,
0x00000000,0x0d03000c,0x0c02000d,0x07040005,0x0d020014,0x07030006,0x0d030009,0x0c05000e,
0x00000000,0x01010001,0x0d040007,0x0c03002a,0x0b020023,0x0d040008,0x0d03000a,0x07040006,
0x00000000,0x0b030013,0x0d040008,0x07030008,0x0b010062,0x07030007,0x0d03000b,0x0d070009,
0x00000000,0x01010001,0x0d030008,0x02020002,0x10020019,0x07040005,0x07030006,0x0c040017,
0x00000000,0x0b020032,0x0d030009,0x07040006,0x0d020015,0x07030008,0x07030007,0x0d06000a,
0x00000000,0x01010001,0x07040005,0x0d03004a,0x0b020024,0x07040006,0x07030008,0x10050016,
0x00000000,0x0c020052,0x07040006,0x07030009,0x0b010063,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x1002001a,0x0d03000c,0x0d02000d,0x03030003,
0x1003008b,0x0d03000d,0x04010003,0x0b050007,0x0d020016,0x0b040008,0x0d02000e,0x04040004,
0x00000000,0x01010001,0x0d03000c,0x0b04000c,0x0b020025,0x0d03000d,0x0d02000f,0x10060014,
0x1002018a,0x0b030014,0x0d03000d,0x10050015,0x0b010064,0x0b040009,0x0d020010,0x0d06000e,
0x00000000,0x01010001,0x10030013,0x02020002,0x1002001b,0x0b050007,0x0b040008,0x0d040027,
0x10010389,0x0b020033,0x10030014,0x0b050008,0x0d020017,0x0b04000a,0x0b040009,0x10060015,
0x00000000,0x01010001,0x0b050007,0x0b03001b,0x0b020026,0x0b050008,0x0b04000a,0x10050017,
0x00000000,0x0c020053,0x0b050008,0x10050016,0x0b010065,0x0b04000b,0x0b04000b,0x0b080009,
0x00000000,0x01010001,0x0d020014,0x02020002,0x1002001c,0x0b040008,0x0b030009,0x03030003,
0x00000000,0x0d03000e,0x0d020015,0x10060014,0x0d020018,0x10050015,0x0b03000a,0x0d050016,
0x00000000,0x01010001,0x0b040008,0x0c03002b,0x0b020027,0x0b040009,0x0b03000b,0x10060015,
0x00000000,0x0b030015,0x0b040009,0x10050017,0x0b010066,0x10050016,0x0b03000c,0x0b07000a,
0x00000000,0x01010001,0x0d030009,0x02020002,0x1002001d,0x10060014,0x10050015,0x10060049,
0x00000000,0x0b020034,0x0d03000a,0x10060015,0x0d020019,0x10050017,0x10050016,0x0d06000b,
0x00000000,0x01010001,0x10060014,0x0d03004b,0x0b020028,0x10060015,0x10050017,0x10050018,
0x00000000,0x0c020054,0x10060015,0x10050018,0x0b010067,0x10050018,0x10050018,0x10090016,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d03000b,0x05020003,0x05010004,0x03030003,
0x1003008c,0x0d03000f,0x04010003,0x1004003c,0x0d02001a,0x1003003d,0x05010005,0x04040004,
0x00000000,0x100300c7,0x05020003,0x0b04000d,0x0b020029,0x05020004,0x05010006,0x0b050008,
0x1002018b,0x0b030016,0x05020004,0x0b040009,0x0b010068,0x1003003e,0x05010007,0x05050005,
0x00000000,0x01010001,0x0d040007,0x02020002,0x0d03000c,0x1004003c,0x1003003d,0x0b040010,
0x1001038a,0x0b020035,0x0d040008,0x1004003d,0x0d02001b,0x1003003f,0x1003003e,0x0d070009,
0x00000000,0x100201c6,0x1004003c,0x0b03001c,0x0b02002a,0x1004003d,0x1003003f,0x0d06000a,
0x00000000,0x0c020055,0x1004003d,0x0b04000a,0x0b010069,0x10030040,0x10030040,0x1007003e,
0x00000000,0x01010001,0x10040026,0x02020002,0x0d03000d,0x0d040008,0x0d030009,0x03030003,
0x00000000,0x0d030010,0x10040027,0x0b050008,0x0d02001c,0x0b040009,0x0d03000a,0x10070028,
0x00000000,0x01010001,0x0d040008,0x0c03002c,0x0b02002b,0x0d040009,0x0d03000b,0x0b050009,
0x00000000,0x0b030017,0x0d040009,0x0b04000b,0x0b01006a,0x0b04000a,0x0d03000c,0x0d07000a,
0x00000000,0x01010001,0x0d03000a,0x02020002,0x0d03000e,0x0b050008,0x0b040009,0x0c040018,
0x00000000,0x0b020036,0x0d03000b,0x0b050009,0x0d02001d,0x0b04000b,0x0b04000a,0x0d06000c,
0x00000000,0x01010001,0x0b050008,0x0d03004c,0x0b02002c,0x0b050009,0x0b04000b,0x0d06000b,
0x00000000,0x0c020056,0x0b050009,0x0b04000c,0x0b01006b,0x0b04000c,0x0b04000c,0x0b08000a,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d03000f,0x10050016,0x10040017,0x03030003,
0x1003008d,0x0d030011,0x04010003,0x0b04000a,0x0d02001e,0x0b03000b,0x10040018,0x04040004,
0x00000000,0x01010001,0x10050016,0x0b04000e,0x0b02002d,0x10050017,0x10040019,0x0d070009,
0x1002018c,0x0b030018,0x10050017,0x0d06000a,0x0b01006c,0x0b03000c,0x1004001a,0x10080018,
0x00000000,0x01010001,0x0d040008,0x02020002,0x0d030010,0x0b04000a,0x0b03000b,0x0d040028,
0x1001038b,0x0b020037,0x0d040009,0x0b04000b,0x0d02001f,0x0b03000d,0x0b03000c,0x0d07000a,
0x00000000,0x01010001,0x0b04000a,0x0b03001d,0x0b02002e,0x0b04000b,0x0b03000d,0x0d06000c,
0x00000000,0x0c020057,0x0b04000b,0x0d06000b,0x0b01006d,0x0b03000e,0x0b03000e,0x0b07000c,
0x00000000,0x01010001,0x10030065,0x02020002,0x0d030011,0x0b040009,0x0b03000a,0x03030003,
0x00000000,0x0d030012,0x10030066,0x0d070009,0x0d020020,0x0d06000a,0x0b03000b,0x10060067,
0x00000000,0x01010001,0x0b040009,0x0c03002d,0x0b02002f,0x0b04000a,0x0b03000c,0x0d07000a,
0x00000000,0x0b030019,0x0b04000a,0x0d06000c,0x0b01006e,0x0d06000b,0x0b03000d,0x0b07000b,
0x00000000,0x01010001,0x0d03000b,0x02020002,0x0d030012,0x0d070009,0x0d06000a,0x1006004a,
0x00000000,0x0b020038,0x0d03000c,0x0d07000a,0x0d020021,0x0d06000c,0x0d06000b,0x0d06000d,
0x00000000,0x01010001,0x0d070009,0x0d03004d,0x0b020030,0x0d07000a,0x0d06000c,0x0d06000d,
0x00000000,0x0c020058,0x0d07000a,0x0d06000d,0x0b01006f,0x0d06000d,0x0d06000d,0x0d0a000b,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d02000c,0x05020003,0x05010004,0x03030003,
0x1003008e,0x0b04000b,0x04010003,0x06030004,0x0b030012,0x06020005,0x05010005,0x04040004,
0x00000000,0x100300c8,0x05020003,0x0b04000f,0x0b020031,0x05020004,0x05010006,0x10050021,
0x1002018d,0x0b03001a,0x05020004,0x10040022,0x0b010070,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0d030008,0x100300e5,0x0d02000d,0x06030004,0x06020005,0x0b040011,
0x1001038c,0x0b020039,0x0d030009,0x06030005,0x0b030013,0x06020007,0x06020006,0x0d06000a,
0x00000000,0x100201c7,0x06030004,0x0b03001e,0x0b020032,0x06030005,0x06020007,0x0d05000b,
0x00000000,0x0c020059,0x06030005,0x10040023,0x0b010071,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0c02000d,0x02020002,0x0d02000e,0x0b050007,0x0b040008,0x03030003,
0x00000000,0x0b04000c,0x0c02000e,0x10050021,0x0b030014,0x10040022,0x0b040009,0x0c05000f,
0x00000000,0x01010001,0x0b050007,0x0c03002e,0x0b020033,0x0b050008,0x0b04000a,0x10050022,
0x00000000,0x0b03001b,0x0b050008,0x10040024,0x0b010072,0x10040023,0x0b04000b,0x0b080009,
0x00000000,0x01010001,0x0b040008,0x02020002,0x0d02000f,0x10050021,0x10040022,0x0c040019,
0x00000000,0x0b02003a,0x0b040009,0x10050022,0x0b030015,0x10040024,0x10040023,0x0b07000a,
0x00000000,0x01010001,0x10050021,0x0d03004e,0x0b020034,0x10050022,0x10040024,0x0d05000c,
0x00000000,0x0c02005a,0x10050022,0x10040025,0x0b010073,0x10040025,0x10040025,0x10080023,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d020010,0x10040035,0x10030036,0x03030003,
0x1003008f,0x0b04000d,0x04010003,0x0b050008,0x0b030016,0x0b040009,0x10030037,0x04040004,
0x00000000,0x01010001,0x10040035,0x0b040010,0x0b020035,0x10040036,0x10030038,0x0d06000a,
0x1002018e,0x0b03001c,0x10040036,0x0d05000b,0x0b010074,0x0b04000a,0x10030039,0x10070037,
0x00000000,0x01010001,0x0d030009,0x02020002,0x0d020011,0x0b050008,0x0b040009,0x0d040029,
0x1001038d,0x0b02003b,0x0d03000a,0x0b050009,0x0b030017,0x0b04000b,0x0b04000a,0x0d06000b,
0x00000000,0x01010001,0x0b050008,0x0b03001f,0x0b020036,0x0b050009,0x0b04000b,0x0d05000d,
0x00000000,0x0c02005b,0x0b050009,0x0d05000c,0x0b010075,0x0b04000c,0x0b04000c,0x0b08000a,
0x00000000,0x01010001,0x0d020015,0x02020002,0x0d020012,0x0b04000a,0x0b03000b,0x03030003,
0x00000000,0x0b04000e,0x0d020016,0x0d06000a,0x0b030018,0x0d05000b,0x0b03000c,0x0d050017,
0x00000000,0x01010001,0x0b04000a,0x0c03002f,0x0b020037,0x0b04000b,0x0b03000d,0x0d06000b,
0x00000000,0x0b03001d,0x0b04000b,0x0d05000d,0x0b010076,0x0d05000c,0x0b03000e,0x0b07000c,
0x00000000,0x01010001,0x0b040009,0x02020002,0x0d020013,0x0d06000a,0x0d05000b,0x1006004b,
0x00000000,0x0b02003c,0x0b04000a,0x0d06000b,0x0b030019,0x0d05000d,0x0d05000c,0x0b07000b,
0x00000000,0x01010001,0x0d06000a,0x0d03004f,0x0b020038,0x0d06000b,0x0d05000d,0x0d05000e,
0x00000000,0x0c02005c,0x0d06000b,0x0d05000e,0x0b010077,0x0d05000e,0x0d05000e,0x0d09000c,
0x00000000,0x01010001,0x04010002,0x02020002,0x0b04000b,0x05020003,0x05010004,0x03030003,
0x10030090,0x0b04000f,0x04010003,0x06030004,0x0b03001a,0x06020005,0x05010005,0x04040004,
0x00000000,0x100300c9,0x05020003,0x0b040011,0x0b020039,0x05020004,0x05010006,0x0c05000a,
0x1002018f,0x0b03001e,0x05020004,0x0c04000b,0x0b010078,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0b050007,0x02020002,0x0b04000c,0x06030004,0x06020005,0x0b040012,
0x1001038e,0x0b02003d,0x0b050008,0x06030005,0x0b03001b,0x06020007,0x06020006,0x0b080009,
0x00000000,0x100201c8,0x06030004,0x0b030020,0x0b02003a,0x06030005,0x06020007,0x0b07000a,
0x00000000,0x0c02005d,0x06030005,0x0c04000c,0x0b010079,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x10040027,0x02020002,0x0b04000d,0x0b050008,0x0b040009,0x03030003,
0x00000000,0x0b040010,0x10040028,0x0c05000a,0x0b03001c,0x0c04000b,0x0b04000a,0x10070029,
0x00000000,0x01010001,0x0b050008,0x0c030030,0x0b02003b,0x0b050009,0x0b04000b,0x0c05000b,
0x00000000,0x0b03001f,0x0b050009,0x0c04000d,0x0b01007a,0x0c04000c,0x0b04000c,0x0b08000a,
0x00000000,0x01010001,0x0b04000a,0x02020002,0x0b04000e,0x0c05000a,0x0c04000b,0x0c04001a,
0x00000000,0x0b02003e,0x0b04000b,0x0c05000b,0x0b03001d,0x0c04000d,0x0c04000c,0x0b07000c,
0x00000000,0x01010001,0x0c05000a,0x0d030050,0x0b02003c,0x0c05000b,0x0c04000d,0x0b07000b,
0x00000000,0x0c02005e,0x0c05000b,0x0c04000e,0x0b01007b,0x0c04000e,0x0c04000e,0x0c08000c,
0x00000000,0x01010001,0x04010002,0x02020002,0x0b04000f,0x10030074,0x10020075,0x03030003,
0x10030091,0x0b040011,0x04010003,0x0b04000b,0x0b03001e,0x0b03000c,0x10020076,0x04040004,
0x00000000,0x01010001,0x10030074,0x0b040012,0x0b02003d,0x10030075,0x10020077,0x0b080009,
0x10020190,0x0b030020,0x10030075,0x0b07000a,0x0b01007c,0x0b03000d,0x10020078,0x10060076,
0x00000000,0x01010001,0x0b050008,0x02020002,0x0b040010,0x0b04000b,0x0b03000c,0x0d04002a,
0x1001038f,0x0b02003f,0x0b050009,0x0b04000c,0x0b03001f,0x0b03000e,0x0b03000d,0x0b08000a,
0x00000000,0x01010001,0x0b04000b,0x0b030021,0x0b02003e,0x0b04000c,0x0b03000e,0x0b07000c,
0x00000000,0x0c02005f,0x0b04000c,0x0b07000b,0x0b01007d,0x0b03000f,0x0b03000f,0x0b07000d,
0x00000000,0x01010001,0x10030066,0x02020002,0x0b040011,0x0b04000b,0x0b03000c,0x03030003,
0x00000000,0x0b040012,0x10030067,0x0b080009,0x0b030020,0x0b07000a,0x0b03000d,0x10060068,
0x00000000,0x01010001,0x0b04000b,0x0c030031,0x0b02003f,0x0b04000c,0x0b03000e,0x0b08000a,
0x00000000,0x0b030021,0x0b04000c,0x0b07000c,0x0b01007e,0x0b07000b,0x0b03000f,0x0b07000d,
0x00000000,0x01010001,0x0b04000b,0x02020002,0x0b040012,0x0b080009,0x0b07000a,0x1006004c,
0x00000000,0x0b020040,0x0b04000c,0x0b08000a,0x0b030021,0x0b07000c,0x0b07000b,0x0b07000d,
0x00000000,0x01010001,0x0b080009,0x0d030051,0x0b020040,0x0b08000a,0x0b07000c,0x0b07000d,
0x00000000,0x0c020060,0x0b08000a,0x0b07000d,0x0b01007f,0x0b07000d,0x0b07000d,0x0b0b000b,
0x00000000,0x01010001,0x04010002,0x02020002,0x08010008,0x05020003,0x05010004,0x03030003,
0x10030092,0x09020009,0x04010003,0x06030004,0x09010010,0x06020005,0x05010005,0x04040004,
0x00000000,0x100300ca,0x05020003,0x1006000e,0x10040024,0x05020004,0x05010006,0x07040005,
0x10020191,0x10050015,0x05020004,0x07030006,0x0c020041,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x08020004,0x100300e6,0x08010009,0x06030004,0x06020005,0x0c05000c,
0x10010390,0x0c030022,0x08020005,0x06030005,0x09010011,0x06020007,0x06020006,0x08050006,
0x00000000,0x100201c9,0x06030004,0x0c040013,0x10040025,0x06030005,0x06020007,0x08040007,
0x00000000,0x0c020061,0x06030005,0x07030007,0x0c020042,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0c02000e,0x02020002,0x0801000a,0x09030005,0x09020006,0x03030003,
0x00000000,0x0902000a,0x0c02000f,0x07040005,0x09010012,0x07030006,0x09020007,0x0c050010,
0x00000000,0x01010001,0x09030005,0x0c030032,0x10040026,0x09030006,0x09020008,0x07040006,
0x00000000,0x10050016,0x09030006,0x07030008,0x0c020043,0x07030007,0x09020009,0x09060007,
0x00000000,0x01010001,0x09020006,0x02020002,0x0801000b,0x07040005,0x07030006,0x0c04001b,
0x00000000,0x0c030023,0x09020007,0x07040006,0x09010013,0x07030008,0x07030007,0x09050008,
0x00000000,0x01010001,0x07040005,0x0d030052,0x10040027,0x07040006,0x07030008,0x08040008,
0x00000000,0x0c020062,0x07040006,0x07030009,0x0c020044,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0801000c,0x0d03000d,0x0d02000e,0x03030003,
0x10030093,0x0902000b,0x04010003,0x1007000a,0x09010014,0x1006000b,0x0d02000f,0x04040004,
0x00000000,0x01010001,0x0d03000d,0x1006000f,0x10040028,0x0d03000e,0x0d020010,0x08050006,
0x10020192,0x10050017,0x0d03000e,0x08040007,0x0c020045,0x1006000c,0x0d020011,0x0d06000f,
0x00000000,0x01010001,0x08020005,0x02020002,0x0801000d,0x1007000a,0x1006000b,0x0d04002b,
0x10010391,0x0c030024,0x08020006,0x1007000b,0x09010015,0x1006000d,0x1006000c,0x08050007,
0x00000000,0x01010001,0x1007000a,0x0c040014,0x10040029,0x1007000b,0x1006000d,0x08040009,
0x00000000,0x0c020063,0x1007000b,0x08040008,0x0c020046,0x1006000e,0x1006000e,0x100a000c,
0x00000000,0x01010001,0x0d020016,0x02020002,0x0801000e,0x1006000b,0x1005000c,0x03030003,
0x00000000,0x0902000c,0x0d020017,0x08050006,0x09010016,0x08040007,0x1005000d,0x0d050018,
0x00000000,0x01010001,0x1006000b,0x0c030033,0x1004002a,0x1006000c,0x1005000e,0x08050007,
0x00000000,0x10050018,0x1006000c,0x08040009,0x0c020047,0x08040008,0x1005000f,0x1009000d,
0x00000000,0x01010001,0x09020007,0x02020002,0x0801000f,0x08050006,0x08040007,0x1006004d,
0x00000000,0x0c030025,0x09020008,0x08050007,0x09010017,0x08040009,0x08040008,0x09050009,
0x00000000,0x01010001,0x08050006,0x0d030053,0x1004002b,0x08050007,0x08040009,0x0804000a,
0x00000000,0x0c020064,0x08050007,0x0804000a,0x0c020048,0x0804000a,0x0804000a,0x08080008,
0x00000000,0x01010001,0x04010002,0x02020002,0x09020009,0x05020003,0x05010004,0x03030003,
0x10030094,0x0902000d,0x04010003,0x1006000f,0x09010018,0x10050010,0x05010005,0x04040004,
0x00000000,0x100300cb,0x05020003,0x10060010,0x1004002c,0x05020004,0x05010006,0x0c060008,
0x10020193,0x10050019,0x05020004,0x0c050009,0x0c020049,0x10050011,0x05010007,0x05050005,
0x00000000,0x01010001,0x09030005,0x02020002,0x0902000a,0x1006000f,0x10050010,0x0c05000d,
0x10010392,0x0c030026,0x09030006,0x10060010,0x09010019,0x10050012,0x10050011,0x09060007,
0x00000000,0x100201ca,0x1006000f,0x0c040015,0x1004002d,0x10060010,0x10050012,0x09050008,
0x00000000,0x0c020065,0x10060010,0x0c05000a,0x0c02004a,0x10050013,0x10050013,0x10090011,
0x00000000,0x01010001,0x10040028,0x02020002,0x0902000b,0x09030006,0x09020007,0x03030003,
0x00000000,0x0902000e,0x10040029,0x0c060008,0x0901001a,0x0c050009,0x09020008,0x1007002a,
0x00000000,0x01010001,0x09030006,0x0c030034,0x1004002e,0x09030007,0x09020009,0x0c060009,
0x00000000,0x1005001a,0x09030007,0x0c05000b,0x0c02004b,0x0c05000a,0x0902000a,0x09060008,
0x00000000,0x01010001,0x09020008,0x02020002,0x0902000c,0x0c060008,0x0c050009,0x0c04001c,
0x00000000,0x0c030027,0x09020009,0x0c060009,0x0901001b,0x0c05000b,0x0c05000a,0x0905000a,
0x00000000,0x01010001,0x0c060008,0x0d030054,0x1004002f,0x0c060009,0x0c05000b,0x09050009,
0x00000000,0x0c020066,0x0c060009,0x0c05000c,0x0c02004c,0x0c05000c,0x0c05000c,0x0c09000a,
0x00000000,0x01010001,0x04010002,0x02020002,0x0902000d,0x10050017,0x10040018,0x03030003,
0x10030095,0x0902000f,0x04010003,0x0c050009,0x0901001c,0x0c04000a,0x10040019,0x04040004,
0x00000000,0x01010001,0x10050017,0x10060011,0x10040030,0x10050018,0x1004001a,0x09060007,
0x10020194,0x1005001b,0x10050018,0x09050008,0x0c02004d,0x0c04000b,0x1004001b,0x10080019,
0x00000000,0x01010001,0x09030006,0x02020002,0x0902000e,0x0c050009,0x0c04000a,0x0d04002c,
0x10010393,0x0c030028,0x09030007,0x0c05000a,0x0901001d,0x0c04000c,0x0c04000b,0x09060008,
0x00000000,0x01010001,0x0c050009,0x0c040016,0x10040031,0x0c05000a,0x0c04000c,0x0905000a,
0x00000000,0x0c020067,0x0c05000a,0x09050009,0x0c02004e,0x0c04000d,0x0c04000d,0x0c08000b,
0x00000000,0x01010001,0x10030067,0x02020002,0x0902000f,0x1006000c,0x1005000d,0x03030003,
0x00000000,0x09020010,0x10030068,0x09060007,0x0901001e,0x09050008,0x1005000e,0x10060069,
0x00000000,0x01010001,0x1006000c,0x0c030035,0x10040032,0x1006000d,0x1005000f,0x09060008,
0x00000000,0x1005001c,0x1006000d,0x0905000a,0x0c02004f,0x09050009,0x10050010,0x1009000e,
0x00000000,0x01010001,0x09020009,0x02020002,0x09020010,0x09060007,0x09050008,0x1006004e,
0x00000000,0x0c030029,0x0902000a,0x09060008,0x0901001f,0x0905000a,0x09050009,0x0905000b,
0x00000000,0x01010001,0x09060007,0x0d030055,0x10040033,0x09060008,0x0905000a,0x0905000b,
0x00000000,0x0c020068,0x09060008,0x0905000b,0x0c020050,0x0905000b,0x0905000b,0x09090009,
0x00000000,0x01010001,0x04010002,0x02020002,0x0c02000b,0x05020003,0x05010004,0x03030003,
0x10030096,0x1006000e,0x04010003,0x06030004,0x10050015,0x06020005,0x05010005,0x04040004,
0x00000000,0x100300cc,0x05020003,0x10060012,0x10040034,0x05020004,0x05010006,0x07040005,
0x10020195,0x1005001d,0x05020004,0x07030006,0x0c020051,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0c030007,0x100300e7,0x0c02000c,0x06030004,0x06020005,0x0c05000e,
0x10010394,0x0c03002a,0x0c030008,0x06030005,0x10050016,0x06020007,0x06020006,0x0c060009,
0x00000000,0x100201cb,0x06030004,0x0c040017,0x10040035,0x06030005,0x06020007,0x0c05000a,
0x00000000,0x0c020069,0x06030005,0x07030007,0x0c020052,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0c02000f,0x02020002,0x0c02000d,0x1007000a,0x1006000b,0x03030003,
0x00000000,0x1006000f,0x0c020010,0x07040005,0x10050017,0x07030006,0x1006000c,0x0c050011,
0x00000000,0x01010001,0x1007000a,0x0c030036,0x10040036,0x1007000b,0x1006000d,0x07040006,
0x00000000,0x1005001e,0x1007000b,0x07030008,0x0c020053,0x07030007,0x1006000e,0x100a000c,
0x00000000,0x01010001,0x1006000b,0x02020002,0x0c02000e,0x07040005,0x07030006,0x0c04001d,
0x00000000,0x0c03002b,0x1006000c,0x07040006,0x10050018,0x07030008,0x07030007,0x1009000d,
0x00000000,0x01010001,0x07040005,0x0d030056,0x10040037,0x07040006,0x07030008,0x0c05000b,
0x00000000,0x0c02006a,0x07040006,0x07030009,0x0c020054,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0c02000f,0x10040036,0x10030037,0x03030003,
0x10030097,0x10060010,0x04010003,0x1007000b,0x10050019,0x1006000c,0x10030038,0x04040004,
0x00000000,0x01010001,0x10040036,0x10060013,0x10040038,0x10040037,0x10030039,0x0c060009,
0x10020196,0x1005001f,0x10040037,0x0c05000a,0x0c020055,0x1006000d,0x1003003a,0x10070038,
0x00000000,0x01010001,0x0c030008,0x02020002,0x0c020010,0x1007000b,0x1006000c,0x0d04002d,
0x10010395,0x0c03002c,0x0c030009,0x1007000c,0x1005001a,0x1006000e,0x1006000d,0x0c06000a,
0x00000000,0x01010001,0x1007000b,0x0c040018,0x10040039,0x1007000c,0x1006000e,0x0c05000c,
0x00000000,0x0c02006b,0x1007000c,0x0c05000b,0x0c020056,0x1006000f,0x1006000f,0x100a000d,
0x00000000,0x01010001,0x0d020017,0x02020002,0x0c020011,0x1006000d,0x1005000e,0x03030003,
0x00000000,0x10060011,0x0d020018,0x0c060009,0x1005001b,0x0c05000a,0x1005000f,0x0d050019,
0x00000000,0x01010001,0x1006000d,0x0c030037,0x1004003a,0x1006000e,0x10050010,0x0c06000a,
0x00000000,0x10050020,0x1006000e,0x0c05000c,0x0c020057,0x0c05000b,0x10050011,0x1009000f,
0x00000000,0x01010001,0x1006000c,0x02020002,0x0c020012,0x0c060009,0x0c05000a,0x1006004f,
0x00000000,0x0c03002d,0x1006000d,0x0c06000a,0x1005001c,0x0c05000c,0x0c05000b,0x1009000e,
0x00000000,0x01010001,0x0c060009,0x0d030057,0x1004003b,0x0c06000a,0x0c05000c,0x0c05000d,
0x00000000,0x0c02006c,0x0c06000a,0x0c05000d,0x0c020058,0x0c05000d,0x0c05000d,0x0c09000b,
0x00000000,0x01010001,0x04010002,0x02020002,0x1006000e,0x05020003,0x05010004,0x03030003,
0x10030098,0x10060012,0x04010003,0x1005001e,0x1005001d,0x1004001f,0x05010005,0x04040004,
0x00000000,0x100300cd,0x05020003,0x10060014,0x1004003c,0x05020004,0x05010006,0x0c05000b,
0x10020197,0x10050021,0x05020004,0x0c04000c,0x0c020059,0x10040020,0x05010007,0x05050005,
0x00000000,0x01010001,0x1007000a,0x02020002,0x1006000f,0x1005001e,0x1004001f,0x0c05000f,
0x10010396,0x0c03002e,0x1007000b,0x1005001f,0x1005001e,0x10040021,0x10040020,0x100a000c,
0x00000000,0x100201cc,0x1005001e,0x0c040019,0x1004003d,0x1005001f,0x10040021,0x1009000d,
0x00000000,0x0c02006d,0x1005001f,0x0c04000d,0x0c02005a,0x10040022,0x10040022,0x10080020,
0x00000000,0x01010001,0x10040029,0x02020002,0x10060010,0x1007000b,0x1006000c,0x03030003,
0x00000000,0x10060013,0x1004002a,0x0c05000b,0x1005001f,0x0c04000c,0x1006000d,0x1007002b,
0x00000000,0x01010001,0x1007000b,0x0c030038,0x1004003e,0x1007000c,0x1006000e,0x0c05000c,
0x00000000,0x10050022,0x1007000c,0x0c04000e,0x0c02005b,0x0c04000d,0x1006000f,0x100a000d,
0x00000000,0x01010001,0x1006000d,0x02020002,0x10060011,0x0c05000b,0x0c04000c,0x0c04001e,
0x00000000,0x0c03002f,0x1006000e,0x0c05000c,0x10050020,0x0c04000e,0x0c04000d,0x1009000f,
0x00000000,0x01010001,0x0c05000b,0x0d030058,0x1004003f,0x0c05000c,0x0c04000e,0x1009000e,
0x00000000,0x0c02006e,0x0c05000c,0x0c04000f,0x0c02005c,0x0c04000f,0x0c04000f,0x0c08000d,
0x00000000,0x01010001,0x04010002,0x02020002,0x10060012,0x10030075,0x10020076,0x03030003,
0x10030099,0x10060014,0x04010003,0x0c05000a,0x10050021,0x0c04000b,0x10020077,0x04040004,
0x00000000,0x01010001,0x10030075,0x10060015,0x10040040,0x10030076,0x10020078,0x100a000c,
0x10020198,0x10050023,0x10030076,0x1009000d,0x0c02005d,0x0c04000c,0x10020079,0x10060077,
0x00000000,0x01010001,0x1007000b,0x02020002,0x10060013,0x0c05000a,0x0c04000b,0x0d04002e,
0x10010397,0x0c030030,0x1007000c,0x0c05000b,0x10050022,0x0c04000d,0x0c04000c,0x100a000d,
0x00000000,0x01010001,0x0c05000a,0x0c04001a,0x10040041,0x0c05000b,0x0c04000d,0x1009000f,
0x00000000,0x0c02006f,0x0c05000b,0x1009000e,0x0c02005e,0x0c04000e,0x0c04000e,0x0c08000c,
0x00000000,0x01010001,0x10030068,0x02020002,0x10060014,0x1006000e,0x1005000f,0x03030003,
0x00000000,0x10060015,0x10030069,0x100a000c,0x10050023,0x1009000d,0x10050010,0x1006006a,
0x00000000,0x01010001,0x1006000e,0x0c030039,0x10040042,0x1006000f,0x10050011,0x100a000d,
0x00000000,0x10050024,0x1006000f,0x1009000f,0x0c02005f,0x1009000e,0x10050012,0x10090010,
0x00000000,0x01010001,0x1006000e,0x02020002,0x10060015,0x100a000c,0x1009000d,0x10060050,
0x00000000,0x0c030031,0x1006000f,0x100a000d,0x10050024,0x1009000f,0x1009000e,0x10090010,
0x00000000,0x01010001,0x100a000c,0x0d030059,0x10040043,0x100a000d,0x1009000f,0x10090010,
0x00000000,0x0c020070,0x100a000d,0x10090010,0x0c020060,0x10090010,0x10090010,0x100d000e,
0x00000000,0x01010001,0x04010002,0x02020002,0x08010008,0x05020003,0x05010004,0x03030003,
0x1003009a,0x1005000f,0x04010003,0x06030004,0x10040016,0x06020005,0x05010005,0x04040004,
0x00000000,0x100300ce,0x05020003,0x0c05000c,0x0c030022,0x05020004,0x05010006,0x07040005,
0x10020199,0x0c040013,0x05020004,0x07030006,0x0c020061,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x08020004,0x100300e8,0x08010009,0x06030004,0x06020005,0x0c050010,
0x10010398,0x0c030032,0x08020005,0x06030005,0x10040017,0x06020007,0x06020006,0x08050006,
0x00000000,0x100201cd,0x06030004,0x0c04001b,0x0c030023,0x06030005,0x06020007,0x08040007,
0x00000000,0x0c020071,0x06030005,0x07030007,0x0c020062,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0c020010,0x02020002,0x0801000a,0x1006000b,0x1005000c,0x03030003,
0x00000000,0x10050010,0x0c020011,0x07040005,0x10040018,0x07030006,0x1005000d,0x0c050012,
0x00000000,0x01010001,0x1006000b,0x0c03003a,0x0c030024,0x1006000c,0x1005000e,0x07040006,
0x00000000,0x0c040014,0x1006000c,0x07030008,0x0c020063,0x07030007,0x1005000f,0x1009000d,
0x00000000,0x01010001,0x1005000c,0x02020002,0x0801000b,0x07040005,0x07030006,0x0c04001f,
0x00000000,0x0c030033,0x1005000d,0x07040006,0x10040019,0x07030008,0x07030007,0x1008000e,
0x00000000,0x01010001,0x07040005,0x0d03005a,0x0c030025,0x07040006,0x07030008,0x08040008,
0x00000000,0x0c020072,0x07040006,0x07030009,0x0c020064,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0801000c,0x0d03000e,0x0d02000f,0x03030003,
0x1003009b,0x10050011,0x04010003,0x0c060008,0x1004001a,0x0c050009,0x0d020010,0x04040004,
0x00000000,0x01010001,0x0d03000e,0x0c05000d,0x0c030026,0x0d03000f,0x0d020011,0x08050006,
0x1002019a,0x0c040015,0x0d03000f,0x08040007,0x0c020065,0x0c05000a,0x0d020012,0x0d060010,
0x00000000,0x01010001,0x08020005,0x02020002,0x0801000d,0x0c060008,0x0c050009,0x0d04002f,
0x10010399,0x0c030034,0x08020006,0x0c060009,0x1004001b,0x0c05000b,0x0c05000a,0x08050007,
0x00000000,0x01010001,0x0c060008,0x0c04001c,0x0c030027,0x0c060009,0x0c05000b,0x08040009,
0x00000000,0x0c020073,0x0c060009,0x08040008,0x0c020066,0x0c05000c,0x0c05000c,0x0c09000a,
0x00000000,0x01010001,0x0d020018,0x02020002,0x0801000e,0x0c050009,0x0c04000a,0x03030003,
0x00000000,0x10050012,0x0d020019,0x08050006,0x1004001c,0x08040007,0x0c04000b,0x0d05001a,
0x00000000,0x01010001,0x0c050009,0x0c03003b,0x0c030028,0x0c05000a,0x0c04000c,0x08050007,
0x00000000,0x0c040016,0x0c05000a,0x08040009,0x0c020067,0x08040008,0x0c04000d,0x0c08000b,
0x00000000,0x01010001,0x1005000d,0x02020002,0x0801000f,0x08050006,0x08040007,0x10060051,
0x00000000,0x0c030035,0x1005000e,0x08050007,0x1004001d,0x08040009,0x08040008,0x1008000f,
0x00000000,0x01010001,0x08050006,0x0d03005b,0x0c030029,0x08050007,0x08040009,0x0804000a,
0x00000000,0x0c020074,0x08050007,0x0804000a,0x0c020068,0x0804000a,0x0804000a,0x08080008,
0x00000000,0x01010001,0x04010002,0x02020002,0x1005000f,0x05020003,0x05010004,0x03030003,
0x1003009c,0x10050013,0x04010003,0x1004003d,0x1004001e,0x1003003e,0x05010005,0x04040004,
0x00000000,0x100300cf,0x05020003,0x0c05000e,0x0c03002a,0x05020004,0x05010006,0x0c060009,
0x1002019b,0x0c040017,0x05020004,0x0c05000a,0x0c020069,0x1003003f,0x05010007,0x05050005,
0x00000000,0x01010001,0x1006000b,0x02020002,0x10050010,0x1004003d,0x1003003e,0x0c050011,
0x1001039a,0x0c030036,0x1006000c,0x1004003e,0x1004001f,0x10030040,0x1003003f,0x1009000d,
0x00000000,0x100201ce,0x1004003d,0x0c04001d,0x0c03002b,0x1004003e,0x10030040,0x1008000e,
0x00000000,0x0c020075,0x1004003e,0x0c05000b,0x0c02006a,0x10030041,0x10030041,0x1007003f,
0x00000000,0x01010001,0x1004002a,0x02020002,0x10050011,0x1006000c,0x1005000d,0x03030003,
0x00000000,0x10050014,0x1004002b,0x0c060009,0x10040020,0x0c05000a,0x1005000e,0x1007002c,
0x00000000,0x01010001,0x1006000c,0x0c03003c,0x0c03002c,0x1006000d,0x1005000f,0x0c06000a,
0x00000000,0x0c040018,0x1006000d,0x0c05000c,0x0c02006b,0x0c05000b,0x10050010,0x1009000e,
0x00000000,0x01010001,0x1005000e,0x02020002,0x10050012,0x0c060009,0x0c05000a,0x0c040020,
0x00000000,0x0c030037,0x1005000f,0x0c06000a,0x10040021,0x0c05000c,0x0c05000b,0x10080010,
0x00000000,0x01010001,0x0c060009,0x0d03005c,0x0c03002d,0x0c06000a,0x0c05000c,0x1008000f,
0x00000000,0x0c020076,0x0c06000a,0x0c05000d,0x0c02006c,0x0c05000d,0x0c05000d,0x0c09000b,
0x00000000,0x01010001,0x04010002,0x02020002,0x10050013,0x10050018,0x10040019,0x03030003,
0x1003009d,0x10050015,0x04010003,0x0c05000b,0x10040022,0x0c04000c,0x1004001a,0x04040004,
0x00000000,0x01010001,0x10050018,0x0c05000f,0x0c03002e,0x10050019,0x1004001b,0x1009000d,
0x1002019c,0x0c040019,0x10050019,0x1008000e,0x0c02006d,0x0c04000d,0x1004001c,0x1008001a,
0x00000000,0x01010001,0x1006000c,0x02020002,0x10050014,0x0c05000b,0x0c04000c,0x0d040030,
0x1001039b,0x0c030038,0x1006000d,0x0c05000c,0x10040023,0x0c04000e,0x0c04000d,0x1009000e,
0x00000000,0x01010001,0x0c05000b,0x0c04001e,0x0c03002f,0x0c05000c,0x0c04000e,0x10080010,
0x00000000,0x0c020077,0x0c05000c,0x1008000f,0x0c02006e,0x0c04000f,0x0c04000f,0x0c08000d,
0x00000000,0x01010001,0x10030069,0x02020002,0x10050015,0x0c05000a,0x0c04000b,0x03030003,
0x00000000,0x10050016,0x1003006a,0x1009000d,0x10040024,0x1008000e,0x0c04000c,0x1006006b,
0x00000000,0x01010001,0x0c05000a,0x0c03003d,0x0c030030,0x0c05000b,0x0c04000d,0x1009000e,
0x00000000,0x0c04001a,0x0c05000b,0x10080010,0x0c02006f,0x1008000f,0x0c04000e,0x0c08000c,
0x00000000,0x01010001,0x1005000f,0x02020002,0x10050016,0x1009000d,0x1008000e,0x10060052,
0x00000000,0x0c030039,0x10050010,0x1009000e,0x10040025,0x10080010,0x1008000f,0x10080011,
0x00000000,0x01010001,0x1009000d,0x0d03005d,0x0c030031,0x1009000e,0x10080010,0x10080011,
0x00000000,0x0c020078,0x1009000e,0x10080011,0x0c020070,0x10080011,0x10080011,0x100c000f,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d02000d,0x05020003,0x05010004,0x03030003,
0x1003009e,0x0c05000c,0x04010003,0x06030004,0x0c040013,0x06020005,0x05010005,0x04040004,
0x00000000,0x100300d0,0x05020003,0x0c050010,0x0c030032,0x05020004,0x05010006,0x07040005,
0x1002019d,0x0c04001b,0x05020004,0x07030006,0x0c020071,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0d030009,0x100300e9,0x0d02000e,0x06030004,0x06020005,0x0c050012,
0x1001039c,0x0c03003a,0x0d03000a,0x06030005,0x0c040014,0x06020007,0x06020006,0x0d06000b,
0x00000000,0x100201cf,0x06030004,0x0c04001f,0x0c030033,0x06030005,0x06020007,0x0d05000c,
0x00000000,0x0c020079,0x06030005,0x07030007,0x0c020072,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0c020011,0x02020002,0x0d02000f,0x0c060008,0x0c050009,0x03030003,
0x00000000,0x0c05000d,0x0c020012,0x07040005,0x0c040015,0x07030006,0x0c05000a,0x0c050013,
0x00000000,0x01010001,0x0c060008,0x0c03003e,0x0c030034,0x0c060009,0x0c05000b,0x07040006,
0x00000000,0x0c04001c,0x0c060009,0x07030008,0x0c020073,0x07030007,0x0c05000c,0x0c09000a,
0x00000000,0x01010001,0x0c050009,0x02020002,0x0d020010,0x07040005,0x07030006,0x0c040021,
0x00000000,0x0c03003b,0x0c05000a,0x07040006,0x0c040016,0x07030008,0x07030007,0x0c08000b,
0x00000000,0x01010001,0x07040005,0x0d03005e,0x0c030035,0x07040006,0x07030008,0x0d05000d,
0x00000000,0x0c02007a,0x07040006,0x07030009,0x0c020074,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d020011,0x10040037,0x10030038,0x03030003,
0x1003009f,0x0c05000e,0x04010003,0x0c060009,0x0c040017,0x0c05000a,0x10030039,0x04040004,
0x00000000,0x01010001,0x10040037,0x0c050011,0x0c030036,0x10040038,0x1003003a,0x0d06000b,
0x1002019e,0x0c04001d,0x10040038,0x0d05000c,0x0c020075,0x0c05000b,0x1003003b,0x10070039,
0x00000000,0x01010001,0x0d03000a,0x02020002,0x0d020012,0x0c060009,0x0c05000a,0x0d040031,
0x1001039d,0x0c03003c,0x0d03000b,0x0c06000a,0x0c040018,0x0c05000c,0x0c05000b,0x0d06000c,
0x00000000,0x01010001,0x0c060009,0x0c040020,0x0c030037,0x0c06000a,0x0c05000c,0x0d05000e,
0x00000000,0x0c02007b,0x0c06000a,0x0d05000d,0x0c020076,0x0c05000d,0x0c05000d,0x0c09000b,
0x00000000,0x01010001,0x0d020019,0x02020002,0x0d020013,0x0c05000b,0x0c04000c,0x03030003,
0x00000000,0x0c05000f,0x0d02001a,0x0d06000b,0x0c040019,0x0d05000c,0x0c04000d,0x0d05001b,
0x00000000,0x01010001,0x0c05000b,0x0c03003f,0x0c030038,0x0c05000c,0x0c04000e,0x0d06000c,
0x00000000,0x0c04001e,0x0c05000c,0x0d05000e,0x0c020077,0x0d05000d,0x0c04000f,0x0c08000d,
0x00000000,0x01010001,0x0c05000a,0x02020002,0x0d020014,0x0d06000b,0x0d05000c,0x10060053,
0x00000000,0x0c03003d,0x0c05000b,0x0d06000c,0x0c04001a,0x0d05000e,0x0d05000d,0x0c08000c,
0x00000000,0x01010001,0x0d06000b,0x0d03005f,0x0c030039,0x0d06000c,0x0d05000e,0x0d05000f,
0x00000000,0x0c02007c,0x0d06000c,0x0d05000f,0x0c020078,0x0d05000f,0x0d05000f,0x0d09000d,
0x00000000,0x01010001,0x04010002,0x02020002,0x0c05000c,0x05020003,0x05010004,0x03030003,
0x100300a0,0x0c050010,0x04010003,0x06030004,0x0c04001b,0x06020005,0x05010005,0x04040004,
0x00000000,0x100300d1,0x05020003,0x0c050012,0x0c03003a,0x05020004,0x05010006,0x0c05000c,
0x1002019f,0x0c04001f,0x05020004,0x0c04000d,0x0c020079,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0c060008,0x02020002,0x0c05000d,0x06030004,0x06020005,0x0c050013,
0x1001039e,0x0c03003e,0x0c060009,0x06030005,0x0c04001c,0x06020007,0x06020006,0x0c09000a,
0x00000000,0x100201d0,0x06030004,0x0c040021,0x0c03003b,0x06030005,0x06020007,0x0c08000b,
0x00000000,0x0c02007d,0x06030005,0x0c04000e,0x0c02007a,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x1004002b,0x02020002,0x0c05000e,0x0c060009,0x0c05000a,0x03030003,
0x00000000,0x0c050011,0x1004002c,0x0c05000c,0x0c04001d,0x0c04000d,0x0c05000b,0x1007002d,
0x00000000,0x01010001,0x0c060009,0x0c030040,0x0c03003c,0x0c06000a,0x0c05000c,0x0c05000d,
0x00000000,0x0c040020,0x0c06000a,0x0c04000f,0x0c02007b,0x0c04000e,0x0c05000d,0x0c09000b,
0x00000000,0x01010001,0x0c05000b,0x02020002,0x0c05000f,0x0c05000c,0x0c04000d,0x0c040022,
0x00000000,0x0c03003f,0x0c05000c,0x0c05000d,0x0c04001e,0x0c04000f,0x0c04000e,0x0c08000d,
0x00000000,0x01010001,0x0c05000c,0x0d030060,0x0c03003d,0x0c05000d,0x0c04000f,0x0c08000c,
0x00000000,0x0c02007e,0x0c05000d,0x0c040010,0x0c02007c,0x0c040010,0x0c040010,0x0c08000e,
0x00000000,0x01010001,0x04010002,0x02020002,0x0c050010,0x10030076,0x10020077,0x03030003,
0x100300a1,0x0c050012,0x04010003,0x0c05000c,0x0c04001f,0x0c04000d,0x10020078,0x04040004,
0x00000000,0x01010001,0x10030076,0x0c050013,0x0c03003e,0x10030077,0x10020079,0x0c09000a,
0x100201a0,0x0c040021,0x10030077,0x0c08000b,0x0c02007d,0x0c04000e,0x1002007a,0x10060078,
0x00000000,0x01010001,0x0c060009,0x02020002,0x0c050011,0x0c05000c,0x0c04000d,0x0d040032,
0x1001039f,0x0c030040,0x0c06000a,0x0c05000d,0x0c040020,0x0c04000f,0x0c04000e,0x0c09000b,
0x00000000,0x01010001,0x0c05000c,0x0c040022,0x0c03003f,0x0c05000d,0x0c04000f,0x0c08000d,
0x00000000,0x0c02007f,0x0c05000d,0x0c08000c,0x0c02007e,0x0c040010,0x0c040010,0x0c08000e,
0x00000000,0x01010001,0x1003006a,0x02020002,0x0c050012,0x0c05000c,0x0c04000d,0x03030003,
0x00000000,0x0c050013,0x1003006b,0x0c09000a,0x0c040021,0x0c08000b,0x0c04000e,0x1006006c,
0x00000000,0x01010001,0x0c05000c,0x0c030041,0x0c030040,0x0c05000d,0x0c04000f,0x0c09000b,
0x00000000,0x0c040022,0x0c05000d,0x0c08000d,0x0c02007f,0x0c08000c,0x0c040010,0x0c08000e,
0x00000000,0x01010001,0x0c05000c,0x02020002,0x0c050013,0x0c09000a,0x0c08000b,0x10060054,
0x00000000,0x0c030041,0x0c05000d,0x0c09000b,0x0c040022,0x0c08000d,0x0c08000c,0x0c08000e,
0x00000000,0x01010001,0x0c09000a,0x0d030061,0x0c030041,0x0c09000b,0x0c08000d,0x0c08000e,
0x00000000,0x0c020080,0x0c09000b,0x0c08000e,0x0c020080,0x0c08000e,0x0c08000e,0x0c0c000c,
0x00000000,0x01010001,0x04010002,0x02020002,0x08010008,0x05020003,0x05010004,0x03030003,
0x100300a2,0x09020009,0x04010003,0x06030004,0x09010010,0x06020005,0x05010005,0x04040004,
0x00000000,0x100300d2,0x05020003,0x0a03000a,0x0a010020,0x05020004,0x05010006,0x07040005,
0x100201a1,0x0a020011,0x05020004,0x07030006,0x10030044,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x08020004,0x100300ea,0x08010009,0x06030004,0x06020005,0x1006000f,
0x100103a0,0x10040025,0x08020005,0x06030005,0x09010011,0x06020007,0x06020006,0x08050006,
0x00000000,0x100201d1,0x06030004,0x10050016,0x0a010021,0x06030005,0x06020007,0x08040007,
0x00000000,0x0d030042,0x06030005,0x07030007,0x10030045,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0d03000b,0x02020002,0x0801000a,0x09030005,0x09020006,0x03030003,
0x00000000,0x0902000a,0x0d03000c,0x07040005,0x09010012,0x07030006,0x09020007,0x0d06000d,
0x00000000,0x01010001,0x09030005,0x0d040023,0x0a010022,0x09030006,0x09020008,0x07040006,
0x00000000,0x0a020012,0x09030006,0x07030008,0x10030046,0x07030007,0x09020009,0x09060007,
0x00000000,0x01010001,0x09020006,0x02020002,0x0801000b,0x07040005,0x07030006,0x0d050014,
0x00000000,0x10040026,0x09020007,0x07040006,0x09010013,0x07030008,0x07030007,0x09050008,
0x00000000,0x01010001,0x07040005,0x0d030062,0x0a010023,0x07040006,0x07030008,0x08040008,
0x00000000,0x0d030043,0x07040006,0x07030009,0x10030047,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0801000c,0x0d03000f,0x0d020010,0x03030003,
0x100300a3,0x0902000b,0x04010003,0x0a040006,0x09010014,0x0a030007,0x0d020011,0x04040004,
0x00000000,0x01010001,0x0d03000f,0x0a03000b,0x0a010024,0x0d030010,0x0d020012,0x08050006,
0x100201a2,0x0a020013,0x0d030010,0x08040007,0x10030048,0x0a030008,0x0d020013,0x0d060011,
0x00000000,0x01010001,0x08020005,0x02020002,0x0801000d,0x0a040006,0x0a030007,0x0d040033,
0x100103a1,0x10040027,0x08020006,0x0a040007,0x09010015,0x0a030009,0x0a030008,0x08050007,
0x00000000,0x01010001,0x0a040006,0x10050017,0x0a010025,0x0a040007,0x0a030009,0x08040009,
0x00000000,0x0d030044,0x0a040007,0x08040008,0x10030049,0x0a03000a,0x0a03000a,0x0a070008,
0x00000000,0x01010001,0x0d02001a,0x02020002,0x0801000e,0x0a030007,0x0a020008,0x03030003,
0x00000000,0x0902000c,0x0d02001b,0x08050006,0x09010016,0x08040007,0x0a020009,0x0d05001c,
0x00000000,0x01010001,0x0a030007,0x0d040024,0x0a010026,0x0a030008,0x0a02000a,0x08050007,
0x00000000,0x0a020014,0x0a030008,0x08040009,0x1003004a,0x08040008,0x0a02000b,0x0a060009,
0x00000000,0x01010001,0x09020007,0x02020002,0x0801000f,0x08050006,0x08040007,0x10060055,
0x00000000,0x10040028,0x09020008,0x08050007,0x09010017,0x08040009,0x08040008,0x09050009,
0x00000000,0x01010001,0x08050006,0x0d030063,0x0a010027,0x08050007,0x08040009,0x0804000a,
0x00000000,0x0d030045,0x08050007,0x0804000a,0x1003004b,0x0804000a,0x0804000a,0x08080008,
0x00000000,0x01010001,0x04010002,0x02020002,0x09020009,0x05020003,0x05010004,0x03030003,
0x100300a4,0x0902000d,0x04010003,0x10060010,0x09010018,0x10050011,0x05010005,0x04040004,
0x00000000,0x100300d3,0x05020003,0x0a03000c,0x0a010028,0x05020004,0x05010006,0x1007000b,
0x100201a3,0x0a020015,0x05020004,0x1006000c,0x1003004c,0x10050012,0x05010007,0x05050005,
0x00000000,0x01010001,0x09030005,0x02020002,0x0902000a,0x10060010,0x10050011,0x10060010,
0x100103a2,0x10040029,0x09030006,0x10060011,0x09010019,0x10050013,0x10050012,0x09060007,
0x00000000,0x100201d2,0x10060010,0x10050018,0x0a010029,0x10060011,0x10050013,0x09050008,
0x00000000,0x0d030046,0x10060011,0x1006000d,0x1003004d,0x10050014,0x10050014,0x10090012,
0x00000000,0x01010001,0x1004002c,0x02020002,0x0902000b,0x09030006,0x09020007,0x03030003,
0x00000000,0x0902000e,0x1004002d,0x1007000b,0x0901001a,0x1006000c,0x09020008,0x1007002e,
0x00000000,0x01010001,0x09030006,0x0d040025,0x0a01002a,0x09030007,0x09020009,0x1007000c,
0x00000000,0x0a020016,0x09030007,0x1006000e,0x1003004e,0x1006000d,0x0902000a,0x09060008,
0x00000000,0x01010001,0x09020008,0x02020002,0x0902000c,0x1007000b,0x1006000c,0x0d050015,
0x00000000,0x1004002a,0x09020009,0x1007000c,0x0901001b,0x1006000e,0x1006000d,0x0905000a,
0x00000000,0x01010001,0x1007000b,0x0d030064,0x0a01002b,0x1007000c,0x1006000e,0x09050009,
0x00000000,0x0d030047,0x1007000c,0x1006000f,0x1003004f,0x1006000f,0x1006000f,0x100a000d,
0x00000000,0x01010001,0x04010002,0x02020002,0x0902000d,0x10050019,0x1004001a,0x03030003,
0x100300a5,0x0902000f,0x04010003,0x1006000c,0x0901001c,0x1005000d,0x1004001b,0x04040004,
0x00000000,0x01010001,0x10050019,0x0a03000d,0x0a01002c,0x1005001a,0x1004001c,0x09060007,
0x100201a4,0x0a020017,0x1005001a,0x09050008,0x10030050,0x1005000e,0x1004001d,0x1008001b,
0x00000000,0x01010001,0x09030006,0x02020002,0x0902000e,0x1006000c,0x1005000d,0x0d040034,
0x100103a3,0x1004002b,0x09030007,0x1006000d,0x0901001d,0x1005000f,0x1005000e,0x09060008,
0x00000000,0x01010001,0x1006000c,0x10050019,0x0a01002d,0x1006000d,0x1005000f,0x0905000a,
0x00000000,0x0d030048,0x1006000d,0x09050009,0x10030051,0x10050010,0x10050010,0x1009000e,
0x00000000,0x01010001,0x1003006b,0x02020002,0x0902000f,0x0a030008,0x0a020009,0x03030003,
0x00000000,0x09020010,0x1003006c,0x09060007,0x0901001e,0x09050008,0x0a02000a,0x1006006d,
0x00000000,0x01010001,0x0a030008,0x0d040026,0x0a01002e,0x0a030009,0x0a02000b,0x09060008,
0x00000000,0x0a020018,0x0a030009,0x0905000a,0x10030052,0x09050009,0x0a02000c,0x0a06000a,
0x00000000,0x01010001,0x09020009,0x02020002,0x09020010,0x09060007,0x09050008,0x10060056,
0x00000000,0x1004002c,0x0902000a,0x09060008,0x0901001f,0x0905000a,0x09050009,0x0905000b,
0x00000000,0x01010001,0x09060007,0x0d030065,0x0a01002f,0x09060008,0x0905000a,0x0905000b,
0x00000000,0x0d030049,0x09060008,0x0905000b,0x10030053,0x0905000b,0x0905000b,0x09090009,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d03000b,0x05020003,0x05010004,0x03030003,
0x100300a6,0x0a03000a,0x04010003,0x06030004,0x0a020011,0x06020005,0x05010005,0x04040004,
0x00000000,0x100300d4,0x05020003,0x0a03000e,0x0a010030,0x05020004,0x05010006,0x10060013,
0x100201a5,0x0a020019,0x05020004,0x10050014,0x10030054,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0d040007,0x100300eb,0x0d03000c,0x06030004,0x06020005,0x10060011,
0x100103a4,0x1004002d,0x0d040008,0x06030005,0x0a020012,0x06020007,0x06020006,0x0d070009,
0x00000000,0x100201d3,0x06030004,0x1005001a,0x0a010031,0x06030005,0x06020007,0x0d06000a,
0x00000000,0x0d03004a,0x06030005,0x10050015,0x10030055,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0d03000c,0x02020002,0x0d03000d,0x0a040006,0x0a030007,0x03030003,
0x00000000,0x0a03000b,0x0d03000d,0x10060013,0x0a020013,0x10050014,0x0a030008,0x0d06000e,
0x00000000,0x01010001,0x0a040006,0x0d040027,0x0a010032,0x0a040007,0x0a030009,0x10060014,
0x00000000,0x0a02001a,0x0a040007,0x10050016,0x10030056,0x10050015,0x0a03000a,0x0a070008,
0x00000000,0x01010001,0x0a030007,0x02020002,0x0d03000e,0x10060013,0x10050014,0x0d050016,
0x00000000,0x1004002e,0x0a030008,0x10060014,0x0a020014,0x10050016,0x10050015,0x0a060009,
0x00000000,0x01010001,0x10060013,0x0d030066,0x0a010033,0x10060014,0x10050016,0x0d06000b,
0x00000000,0x0d03004b,0x10060014,0x10050017,0x10030057,0x10050017,0x10050017,0x10090015,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d03000f,0x10040038,0x10030039,0x03030003,
0x100300a7,0x0a03000c,0x04010003,0x0a040007,0x0a020015,0x0a030008,0x1003003a,0x04040004,
0x00000000,0x01010001,0x10040038,0x0a03000f,0x0a010034,0x10040039,0x1003003b,0x0d070009,
0x100201a6,0x0a02001b,0x10040039,0x0d06000a,0x10030058,0x0a030009,0x1003003c,0x1007003a,
0x00000000,0x01010001,0x0d040008,0x02020002,0x0d030010,0x0a040007,0x0a030008,0x0d040035,
0x100103a5,0x1004002f,0x0d040009,0x0a040008,0x0a020016,0x0a03000a,0x0a030009,0x0d07000a,
0x00000000,0x01010001,0x0a040007,0x1005001b,0x0a010035,0x0a040008,0x0a03000a,0x0d06000c,
0x00000000,0x0d03004c,0x0a040008,0x0d06000b,0x10030059,0x0a03000b,0x0a03000b,0x0a070009,
0x00000000,0x01010001,0x0d02001b,0x02020002,0x0d030011,0x0a030009,0x0a02000a,0x03030003,
0x00000000,0x0a03000d,0x0d02001c,0x0d070009,0x0a020017,0x0d06000a,0x0a02000b,0x0d05001d,
0x00000000,0x01010001,0x0a030009,0x0d040028,0x0a010036,0x0a03000a,0x0a02000c,0x0d07000a,
0x00000000,0x0a02001c,0x0a03000a,0x0d06000c,0x1003005a,0x0d06000b,0x0a02000d,0x0a06000b,
0x00000000,0x01010001,0x0a030008,0x02020002,0x0d030012,0x0d070009,0x0d06000a,0x10060057,
0x00000000,0x10040030,0x0a030009,0x0d07000a,0x0a020018,0x0d06000c,0x0d06000b,0x0a06000a,
0x00000000,0x01010001,0x0d070009,0x0d030067,0x0a010037,0x0d07000a,0x0d06000c,0x0d06000d,
0x00000000,0x0d03004d,0x0d07000a,0x0d06000d,0x1003005b,0x0d06000d,0x0d06000d,0x0d0a000b,
0x00000000,0x01010001,0x04010002,0x02020002,0x0a03000a,0x05020003,0x05010004,0x03030003,
0x100300a8,0x0a03000e,0x04010003,0x1005001f,0x0a020019,0x10040020,0x05010005,0x04040004,
0x00000000,0x100300d5,0x05020003,0x0a030010,0x0a010038,0x05020004,0x05010006,0x0d06000a,
0x100201a7,0x0a02001d,0x05020004,0x0d05000b,0x1003005c,0x10040021,0x05010007,0x05050005,
0x00000000,0x01010001,0x0a040006,0x02020002,0x0a03000b,0x1005001f,0x10040020,0x10060012,
0x100103a6,0x10040031,0x0a040007,0x10050020,0x0a02001a,0x10040022,0x10040021,0x0a070008,
0x00000000,0x100201d4,0x1005001f,0x1005001c,0x0a010039,0x10050020,0x10040022,0x0a060009,
0x00000000,0x0d03004e,0x10050020,0x0d05000c,0x1003005d,0x10040023,0x10040023,0x10080021,
0x00000000,0x01010001,0x1004002d,0x02020002,0x0a03000c,0x0a040007,0x0a030008,0x03030003,
0x00000000,0x0a03000f,0x1004002e,0x0d06000a,0x0a02001b,0x0d05000b,0x0a030009,0x1007002f,
0x00000000,0x01010001,0x0a040007,0x0d040029,0x0a01003a,0x0a040008,0x0a03000a,0x0d06000b,
0x00000000,0x0a02001e,0x0a040008,0x0d05000d,0x1003005e,0x0d05000c,0x0a03000b,0x0a070009,
0x00000000,0x01010001,0x0a030009,0x02020002,0x0a03000d,0x0d06000a,0x0d05000b,0x0d050017,
0x00000000,0x10040032,0x0a03000a,0x0d06000b,0x0a02001c,0x0d05000d,0x0d05000c,0x0a06000b,
0x00000000,0x01010001,0x0d06000a,0x0d030068,0x0a01003b,0x0d06000b,0x0d05000d,0x0a06000a,
0x00000000,0x0d03004f,0x0d06000b,0x0d05000e,0x1003005f,0x0d05000e,0x0d05000e,0x0d09000c,
0x00000000,0x01010001,0x04010002,0x02020002,0x0a03000e,0x10030077,0x10020078,0x03030003,
0x100300a9,0x0a030010,0x04010003,0x1006000d,0x0a02001d,0x1005000e,0x10020079,0x04040004,
0x00000000,0x01010001,0x10030077,0x0a030011,0x0a01003c,0x10030078,0x1002007a,0x0a070008,
0x100201a8,0x0a02001f,0x10030078,0x0a060009,0x10030060,0x1005000f,0x1002007b,0x10060079,
0x00000000,0x01010001,0x0a040007,0x02020002,0x0a03000f,0x1006000d,0x1005000e,0x0d040036,
0x100103a7,0x10040033,0x0a040008,0x1006000e,0x0a02001e,0x10050010,0x1005000f,0x0a070009,
0x00000000,0x01010001,0x1006000d,0x1005001d,0x0a01003d,0x1006000e,0x10050010,0x0a06000b,
0x00000000,0x0d030050,0x1006000e,0x0a06000a,0x10030061,0x10050011,0x10050011,0x1009000f,
0x00000000,0x01010001,0x1003006c,0x02020002,0x0a030010,0x0a03000a,0x0a02000b,0x03030003,
0x00000000,0x0a030011,0x1003006d,0x0a070008,0x0a02001f,0x0a060009,0x0a02000c,0x1006006e,
0x00000000,0x01010001,0x0a03000a,0x0d04002a,0x0a01003e,0x0a03000b,0x0a02000d,0x0a070009,
0x00000000,0x0a020020,0x0a03000b,0x0a06000b,0x10030062,0x0a06000a,0x0a02000e,0x0a06000c,
0x00000000,0x01010001,0x0a03000a,0x02020002,0x0a030011,0x0a070008,0x0a060009,0x10060058,
0x00000000,0x10040034,0x0a03000b,0x0a070009,0x0a020020,0x0a06000b,0x0a06000a,0x0a06000c,
0x00000000,0x01010001,0x0a070008,0x0d030069,0x0a01003f,0x0a070009,0x0a06000b,0x0a06000c,
0x00000000,0x0d030051,0x0a070009,0x0a06000c,0x10030063,0x0a06000c,0x0a06000c,0x0a0a000a,
0x00000000,0x01010001,0x04010002,0x02020002,0x08010008,0x05020003,0x05010004,0x03030003,
0x100300aa,0x0d03000c,0x04010003,0x06030004,0x0d020013,0x06020005,0x05010005,0x04040004,
0x00000000,0x100300d6,0x05020003,0x1006000f,0x10040025,0x05020004,0x05010006,0x07040005,
0x100201a9,0x10050016,0x05020004,0x07030006,0x10030064,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x08020004,0x100300ec,0x08010009,0x06030004,0x06020005,0x10060013,
0x100103a8,0x10040035,0x08020005,0x06030005,0x0d020014,0x06020007,0x06020006,0x08050006,
0x00000000,0x100201d5,0x06030004,0x1005001e,0x10040026,0x06030005,0x06020007,0x08040007,
0x00000000,0x0d030052,0x06030005,0x07030007,0x10030065,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0d03000d,0x02020002,0x0801000a,0x0d040008,0x0d030009,0x03030003,
0x00000000,0x0d03000d,0x0d03000e,0x07040005,0x0d020015,0x07030006,0x0d03000a,0x0d06000f,
0x00000000,0x01010001,0x0d040008,0x0d04002b,0x10040027,0x0d040009,0x0d03000b,0x07040006,
0x00000000,0x10050017,0x0d040009,0x07030008,0x10030066,0x07030007,0x0d03000c,0x0d07000a,
0x00000000,0x01010001,0x0d030009,0x02020002,0x0801000b,0x07040005,0x07030006,0x0d050018,
0x00000000,0x10040036,0x0d03000a,0x07040006,0x0d020016,0x07030008,0x07030007,0x0d06000b,
0x00000000,0x01010001,0x07040005,0x0d03006a,0x10040028,0x07040006,0x07030008,0x08040008,
0x00000000,0x0d030053,0x07040006,0x07030009,0x10030067,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0801000c,0x0d030010,0x0d020011,0x03030003,
0x100300ab,0x0d03000e,0x04010003,0x1007000b,0x0d020017,0x1006000c,0x0d020012,0x04040004,
0x00000000,0x01010001,0x0d030010,0x10060010,0x10040029,0x0d030011,0x0d020013,0x08050006,
0x100201aa,0x10050018,0x0d030011,0x08040007,0x10030068,0x1006000d,0x0d020014,0x0d060012,
0x00000000,0x01010001,0x08020005,0x02020002,0x0801000d,0x1007000b,0x1006000c,0x0d040037,
0x100103a9,0x10040037,0x08020006,0x1007000c,0x0d020018,0x1006000e,0x1006000d,0x08050007,
0x00000000,0x01010001,0x1007000b,0x1005001f,0x1004002a,0x1007000c,0x1006000e,0x08040009,
0x00000000,0x0d030054,0x1007000c,0x08040008,0x10030069,0x1006000f,0x1006000f,0x100a000d,
0x00000000,0x01010001,0x0d02001c,0x02020002,0x0801000e,0x1006000c,0x1005000d,0x03030003,
0x00000000,0x0d03000f,0x0d02001d,0x08050006,0x0d020019,0x08040007,0x1005000e,0x0d05001e,
0x00000000,0x01010001,0x1006000c,0x0d04002c,0x1004002b,0x1006000d,0x1005000f,0x08050007,
0x00000000,0x10050019,0x1006000d,0x08040009,0x1003006a,0x08040008,0x10050010,0x1009000e,
0x00000000,0x01010001,0x0d03000a,0x02020002,0x0801000f,0x08050006,0x08040007,0x10060059,
0x00000000,0x10040038,0x0d03000b,0x08050007,0x0d02001a,0x08040009,0x08040008,0x0d06000c,
0x00000000,0x01010001,0x08050006,0x0d03006b,0x1004002c,0x08050007,0x08040009,0x0804000a,
0x00000000,0x0d030055,0x08050007,0x0804000a,0x1003006b,0x0804000a,0x0804000a,0x08080008,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d03000c,0x05020003,0x05010004,0x03030003,
0x100300ac,0x0d030010,0x04010003,0x1004003e,0x0d02001b,0x1003003f,0x05010005,0x04040004,
0x00000000,0x100300d7,0x05020003,0x10060011,0x1004002d,0x05020004,0x05010006,0x1007000c,
0x100201ab,0x1005001a,0x05020004,0x1006000d,0x1003006c,0x10030040,0x05010007,0x05050005,
0x00000000,0x01010001,0x0d040008,0x02020002,0x0d03000d,0x1004003e,0x1003003f,0x10060014,
0x100103aa,0x10040039,0x0d040009,0x1004003f,0x0d02001c,0x10030041,0x10030040,0x0d07000a,
0x00000000,0x100201d6,0x1004003e,0x10050020,0x1004002e,0x1004003f,0x10030041,0x0d06000b,
0x00000000,0x0d030056,0x1004003f,0x1006000e,0x1003006d,0x10030042,0x10030042,0x10070040,
0x00000000,0x01010001,0x1004002e,0x02020002,0x0d03000e,0x0d040009,0x0d03000a,0x03030003,
0x00000000,0x0d030011,0x1004002f,0x1007000c,0x0d02001d,0x1006000d,0x0d03000b,0x10070030,
0x00000000,0x01010001,0x0d040009,0x0d04002d,0x1004002f,0x0d04000a,0x0d03000c,0x1007000d,
0x00000000,0x1005001b,0x0d04000a,0x1006000f,0x1003006e,0x1006000e,0x0d03000d,0x0d07000b,
0x00000000,0x01010001,0x0d03000b,0x02020002,0x0d03000f,0x1007000c,0x1006000d,0x0d050019,
0x00000000,0x1004003a,0x0d03000c,0x1007000d,0x0d02001e,0x1006000f,0x1006000e,0x0d06000d,
0x00000000,0x01010001,0x1007000c,0x0d03006c,0x10040030,0x1007000d,0x1006000f,0x0d06000c,
0x00000000,0x0d030057,0x1007000d,0x10060010,0x1003006f,0x10060010,0x10060010,0x100a000e,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d030010,0x1005001a,0x1004001b,0x03030003,
0x100300ad,0x0d030012,0x04010003,0x1006000e,0x0d02001f,0x1005000f,0x1004001c,0x04040004,
0x00000000,0x01010001,0x1005001a,0x10060012,0x10040031,0x1005001b,0x1004001d,0x0d07000a,
0x100201ac,0x1005001c,0x1005001b,0x0d06000b,0x10030070,0x10050010,0x1004001e,0x1008001c,
0x00000000,0x01010001,0x0d040009,0x02020002,0x0d030011,0x1006000e,0x1005000f,0x0d040038,
0x100103ab,0x1004003b,0x0d04000a,0x1006000f,0x0d020020,0x10050011,0x10050010,0x0d07000b,
0x00000000,0x01010001,0x1006000e,0x10050021,0x10040032,0x1006000f,0x10050011,0x0d06000d,
0x00000000,0x0d030058,0x1006000f,0x0d06000c,0x10030071,0x10050012,0x10050012,0x10090010,
0x00000000,0x01010001,0x1003006d,0x02020002,0x0d030012,0x1006000d,0x1005000e,0x03030003,
0x00000000,0x0d030013,0x1003006e,0x0d07000a,0x0d020021,0x0d06000b,0x1005000f,0x1006006f,
0x00000000,0x01010001,0x1006000d,0x0d04002e,0x10040033,0x1006000e,0x10050010,0x0d07000b,
0x00000000,0x1005001d,0x1006000e,0x0d06000d,0x10030072,0x0d06000c,0x10050011,0x1009000f,
0x00000000,0x01010001,0x0d03000c,0x02020002,0x0d030013,0x0d07000a,0x0d06000b,0x1006005a,
0x00000000,0x1004003c,0x0d03000d,0x0d07000b,0x0d020022,0x0d06000d,0x0d06000c,0x0d06000e,
0x00000000,0x01010001,0x0d07000a,0x0d03006d,0x10040034,0x0d07000b,0x0d06000d,0x0d06000e,
0x00000000,0x0d030059,0x0d07000b,0x0d06000e,0x10030073,0x0d06000e,0x0d06000e,0x0d0a000c,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d02000e,0x05020003,0x05010004,0x03030003,
0x100300ae,0x1006000f,0x04010003,0x06030004,0x10050016,0x06020005,0x05010005,0x04040004,
0x00000000,0x100300d8,0x05020003,0x10060013,0x10040035,0x05020004,0x05010006,0x10050022,
0x100201ad,0x1005001e,0x05020004,0x10040023,0x10030074,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0d03000a,0x100300ed,0x0d02000f,0x06030004,0x06020005,0x10060015,
0x100103ac,0x1004003d,0x0d03000b,0x06030005,0x10050017,0x06020007,0x06020006,0x0d06000c,
0x00000000,0x100201d7,0x06030004,0x10050022,0x10040036,0x06030005,0x06020007,0x0d05000d,
0x00000000,0x0d03005a,0x06030005,0x10040024,0x10030075,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0d03000e,0x02020002,0x0d020010,0x1007000b,0x1006000c,0x03030003,
0x00000000,0x10060010,0x0d03000f,0x10050022,0x10050018,0x10040023,0x1006000d,0x0d060010,
0x00000000,0x01010001,0x1007000b,0x0d04002f,0x10040037,0x1007000c,0x1006000e,0x10050023,
0x00000000,0x1005001f,0x1007000c,0x10040025,0x10030076,0x10040024,0x1006000f,0x100a000d,
0x00000000,0x01010001,0x1006000c,0x02020002,0x0d020011,0x10050022,0x10040023,0x0d05001a,
0x00000000,0x1004003e,0x1006000d,0x10050023,0x10050019,0x10040025,0x10040024,0x1009000e,
0x00000000,0x01010001,0x10050022,0x0d03006e,0x10040038,0x10050023,0x10040025,0x0d05000e,
0x00000000,0x0d03005b,0x10050023,0x10040026,0x10030077,0x10040026,0x10040026,0x10080024,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d020012,0x10040039,0x1003003a,0x03030003,
0x100300af,0x10060011,0x04010003,0x1007000c,0x1005001a,0x1006000d,0x1003003b,0x04040004,
0x00000000,0x01010001,0x10040039,0x10060014,0x10040039,0x1004003a,0x1003003c,0x0d06000c,
0x100201ae,0x10050020,0x1004003a,0x0d05000d,0x10030078,0x1006000e,0x1003003d,0x1007003b,
0x00000000,0x01010001,0x0d03000b,0x02020002,0x0d020013,0x1007000c,0x1006000d,0x0d040039,
0x100103ad,0x1004003f,0x0d03000c,0x1007000d,0x1005001b,0x1006000f,0x1006000e,0x0d06000d,
0x00000000,0x01010001,0x1007000c,0x10050023,0x1004003a,0x1007000d,0x1006000f,0x0d05000f,
0x00000000,0x0d03005c,0x1007000d,0x0d05000e,0x10030079,0x10060010,0x10060010,0x100a000e,
0x00000000,0x01010001,0x0d02001d,0x02020002,0x0d020014,0x1006000e,0x1005000f,0x03030003,
0x00000000,0x10060012,0x0d02001e,0x0d06000c,0x1005001c,0x0d05000d,0x10050010,0x0d05001f,
0x00000000,0x01010001,0x1006000e,0x0d040030,0x1004003b,0x1006000f,0x10050011,0x0d06000d,
0x00000000,0x10050021,0x1006000f,0x0d05000f,0x1003007a,0x0d05000e,0x10050012,0x10090010,
0x00000000,0x01010001,0x1006000d,0x02020002,0x0d020015,0x0d06000c,0x0d05000d,0x1006005b,
0x00000000,0x10040040,0x1006000e,0x0d06000d,0x1005001d,0x0d05000f,0x0d05000e,0x1009000f,
0x00000000,0x01010001,0x0d06000c,0x0d03006f,0x1004003c,0x0d06000d,0x0d05000f,0x0d050010,
0x00000000,0x0d03005d,0x0d06000d,0x0d050010,0x1003007b,0x0d050010,0x0d050010,0x0d09000e,
0x00000000,0x01010001,0x04010002,0x02020002,0x1006000f,0x05020003,0x05010004,0x03030003,
0x100300b0,0x10060013,0x04010003,0x06030004,0x1005001e,0x06020005,0x05010005,0x04040004,
0x00000000,0x100300d9,0x05020003,0x10060015,0x1004003d,0x05020004,0x05010006,0x0d06000b,
0x100201af,0x10050022,0x05020004,0x0d05000c,0x1003007c,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x1007000b,0x02020002,0x10060010,0x06030004,0x06020005,0x10060016,
0x100103ae,0x10040041,0x1007000c,0x06030005,0x1005001f,0x06020007,0x06020006,0x100a000d,
0x00000000,0x100201d8,0x06030004,0x10050024,0x1004003e,0x06030005,0x06020007,0x1009000e,
0x00000000,0x0d03005e,0x06030005,0x0d05000d,0x1003007d,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x1004002f,0x02020002,0x10060011,0x1007000c,0x1006000d,0x03030003,
0x00000000,0x10060014,0x10040030,0x0d06000b,0x10050020,0x0d05000c,0x1006000e,0x10070031,
0x00000000,0x01010001,0x1007000c,0x0d040031,0x1004003f,0x1007000d,0x1006000f,0x0d06000c,
0x00000000,0x10050023,0x1007000d,0x0d05000e,0x1003007e,0x0d05000d,0x10060010,0x100a000e,
0x00000000,0x01010001,0x1006000e,0x02020002,0x10060012,0x0d06000b,0x0d05000c,0x0d05001b,
0x00000000,0x10040042,0x1006000f,0x0d06000c,0x10050021,0x0d05000e,0x0d05000d,0x10090010,
0x00000000,0x01010001,0x0d06000b,0x0d030070,0x10040040,0x0d06000c,0x0d05000e,0x1009000f,
0x00000000,0x0d03005f,0x0d06000c,0x0d05000f,0x1003007f,0x0d05000f,0x0d05000f,0x0d09000d,
0x00000000,0x01010001,0x04010002,0x02020002,0x10060013,0x10030078,0x10020079,0x03030003,
0x100300b1,0x10060015,0x04010003,0x1006000f,0x10050022,0x10050010,0x1002007a,0x04040004,
0x00000000,0x01010001,0x10030078,0x10060016,0x10040041,0x10030079,0x1002007b,0x100a000d,
0x100201b0,0x10050024,0x10030079,0x1009000e,0x10030080,0x10050011,0x1002007c,0x1006007a,
0x00000000,0x01010001,0x1007000c,0x02020002,0x10060014,0x1006000f,0x10050010,0x0d04003a,
0x100103af,0x10040043,0x1007000d,0x10060010,0x10050023,0x10050012,0x10050011,0x100a000e,
0x00000000,0x01010001,0x1006000f,0x10050025,0x10040042,0x10060010,0x10050012,0x10090010,
0x00000000,0x0d030060,0x10060010,0x1009000f,0x10030081,0x10050013,0x10050013,0x10090011,
0x00000000,0x01010001,0x1003006e,0x02020002,0x10060015,0x1006000f,0x10050010,0x03030003,
0x00000000,0x10060016,0x1003006f,0x100a000d,0x10050024,0x1009000e,0x10050011,0x10060070,
0x00000000,0x01010001,0x1006000f,0x0d040032,0x10040043,0x10060010,0x10050012,0x100a000e,
0x00000000,0x10050025,0x10060010,0x10090010,0x10030082,0x1009000f,0x10050013,0x10090011,
0x00000000,0x01010001,0x1006000f,0x02020002,0x10060016,0x100a000d,0x1009000e,0x1006005c,
0x00000000,0x10040044,0x10060010,0x100a000e,0x10050025,0x10090010,0x1009000f,0x10090011,
0x00000000,0x01010001,0x100a000d,0x0d030071,0x10040044,0x100a000e,0x10090010,0x10090011,
0x00000000,0x0d030061,0x100a000e,0x10090011,0x10030083,0x10090011,0x10090011,0x100d000f,
0x00000000,0x01010001,0x04010002,0x02020002,0x08010008,0x05020003,0x05010004,0x03030003,
0x100300b2,0x09020009,0x04010003,0x06030004,0x09010010,0x06020005,0x05010005,0x04040004,
0x00000000,0x100300da,0x05020003,0x10050011,0x10030027,0x05020004,0x05010006,0x07040005,
0x100201b1,0x10040018,0x05020004,0x07030006,0x0d030042,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x08020004,0x100300ee,0x08010009,0x06030004,0x06020005,0x0d06000d,
0x100103b0,0x0d040023,0x08020005,0x06030005,0x09010011,0x06020007,0x06020006,0x08050006,
0x00000000,0x100201d9,0x06030004,0x0d050014,0x10030028,0x06030005,0x06020007,0x08040007,
0x00000000,0x0d030062,0x06030005,0x07030007,0x0d030043,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0d03000f,0x02020002,0x0801000a,0x09030005,0x09020006,0x03030003,
0x00000000,0x0902000a,0x0d030010,0x07040005,0x09010012,0x07030006,0x09020007,0x0d060011,
0x00000000,0x01010001,0x09030005,0x0d040033,0x10030029,0x09030006,0x09020008,0x07040006,
0x00000000,0x10040019,0x09030006,0x07030008,0x0d030044,0x07030007,0x09020009,0x09060007,
0x00000000,0x01010001,0x09020006,0x02020002,0x0801000b,0x07040005,0x07030006,0x0d05001c,
0x00000000,0x0d040024,0x09020007,0x07040006,0x09010013,0x07030008,0x07030007,0x09050008,
0x00000000,0x01010001,0x07040005,0x0d030072,0x1003002a,0x07040006,0x07030008,0x08040008,
0x00000000,0x0d030063,0x07040006,0x07030009,0x0d030045,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0801000c,0x0d030011,0x0d020012,0x03030003,
0x100300b3,0x0902000b,0x04010003,0x1006000d,0x09010014,0x1005000e,0x0d020013,0x04040004,
0x00000000,0x01010001,0x0d030011,0x10050012,0x1003002b,0x0d030012,0x0d020014,0x08050006,
0x100201b2,0x1004001a,0x0d030012,0x08040007,0x0d030046,0x1005000f,0x0d020015,0x0d060013,
0x00000000,0x01010001,0x08020005,0x02020002,0x0801000d,0x1006000d,0x1005000e,0x0d04003b,
0x100103b1,0x0d040025,0x08020006,0x1006000e,0x09010015,0x10050010,0x1005000f,0x08050007,
0x00000000,0x01010001,0x1006000d,0x0d050015,0x1003002c,0x1006000e,0x10050010,0x08040009,
0x00000000,0x0d030064,0x1006000e,0x08040008,0x0d030047,0x10050011,0x10050011,0x1009000f,
0x00000000,0x01010001,0x0d02001e,0x02020002,0x0801000e,0x1005000e,0x1004000f,0x03030003,
0x00000000,0x0902000c,0x0d02001f,0x08050006,0x09010016,0x08040007,0x10040010,0x0d050020,
0x00000000,0x01010001,0x1005000e,0x0d040034,0x1003002d,0x1005000f,0x10040011,0x08050007,
0x00000000,0x1004001b,0x1005000f,0x08040009,0x0d030048,0x08040008,0x10040012,0x10080010,
0x00000000,0x01010001,0x09020007,0x02020002,0x0801000f,0x08050006,0x08040007,0x1006005d,
0x00000000,0x0d040026,0x09020008,0x08050007,0x09010017,0x08040009,0x08040008,0x09050009,
0x00000000,0x01010001,0x08050006,0x0d030073,0x1003002e,0x08050007,0x08040009,0x0804000a,
0x00000000,0x0d030065,0x08050007,0x0804000a,0x0d030049,0x0804000a,0x0804000a,0x08080008,
0x00000000,0x01010001,0x04010002,0x02020002,0x09020009,0x05020003,0x05010004,0x03030003,
0x100300b4,0x0902000d,0x04010003,0x10060011,0x09010018,0x10050012,0x05010005,0x04040004,
0x00000000,0x100300db,0x05020003,0x10050013,0x1003002f,0x05020004,0x05010006,0x0d070009,
0x100201b3,0x1004001c,0x05020004,0x0d06000a,0x0d03004a,0x10050013,0x05010007,0x05050005,
0x00000000,0x01010001,0x09030005,0x02020002,0x0902000a,0x10060011,0x10050012,0x0d06000e,
0x100103b2,0x0d040027,0x09030006,0x10060012,0x09010019,0x10050014,0x10050013,0x09060007,
0x00000000,0x100201da,0x10060011,0x0d050016,0x10030030,0x10060012,0x10050014,0x09050008,
0x00000000,0x0d030066,0x10060012,0x0d06000b,0x0d03004b,0x10050015,0x10050015,0x10090013,
0x00000000,0x01010001,0x10040030,0x02020002,0x0902000b,0x09030006,0x09020007,0x03030003,
0x00000000,0x0902000e,0x10040031,0x0d070009,0x0901001a,0x0d06000a,0x09020008,0x10070032,
0x00000000,0x01010001,0x09030006,0x0d040035,0x10030031,0x09030007,0x09020009,0x0d07000a,
0x00000000,0x1004001d,0x09030007,0x0d06000c,0x0d03004c,0x0d06000b,0x0902000a,0x09060008,
0x00000000,0x01010001,0x09020008,0x02020002,0x0902000c,0x0d070009,0x0d06000a,0x0d05001d,
0x00000000,0x0d040028,0x09020009,0x0d07000a,0x0901001b,0x0d06000c,0x0d06000b,0x0905000a,
0x00000000,0x01010001,0x0d070009,0x0d030074,0x10030032,0x0d07000a,0x0d06000c,0x09050009,
0x00000000,0x0d030067,0x0d07000a,0x0d06000d,0x0d03004d,0x0d06000d,0x0d06000d,0x0d0a000b,
0x00000000,0x01010001,0x04010002,0x02020002,0x0902000d,0x1005001b,0x1004001c,0x03030003,
0x100300b5,0x0902000f,0x04010003,0x0d06000a,0x0901001c,0x0d05000b,0x1004001d,0x04040004,
0x00000000,0x01010001,0x1005001b,0x10050014,0x10030033,0x1005001c,0x1004001e,0x09060007,
0x100201b4,0x1004001e,0x1005001c,0x09050008,0x0d03004e,0x0d05000c,0x1004001f,0x1008001d,
0x00000000,0x01010001,0x09030006,0x02020002,0x0902000e,0x0d06000a,0x0d05000b,0x0d04003c,
0x100103b3,0x0d040029,0x09030007,0x0d06000b,0x0901001d,0x0d05000d,0x0d05000c,0x09060008,
0x00000000,0x01010001,0x0d06000a,0x0d050017,0x10030034,0x0d06000b,0x0d05000d,0x0905000a,
0x00000000,0x0d030068,0x0d06000b,0x09050009,0x0d03004f,0x0d05000e,0x0d05000e,0x0d09000c,
0x00000000,0x01010001,0x1003006f,0x02020002,0x0902000f,0x1005000f,0x10040010,0x03030003,
0x00000000,0x09020010,0x10030070,0x09060007,0x0901001e,0x09050008,0x10040011,0x10060071,
0x00000000,0x01010001,0x1005000f,0x0d040036,0x10030035,0x10050010,0x10040012,0x09060008,
0x00000000,0x1004001f,0x10050010,0x0905000a,0x0d030050,0x09050009,0x10040013,0x10080011,
0x00000000,0x01010001,0x09020009,0x02020002,0x09020010,0x09060007,0x09050008,0x1006005e,
0x00000000,0x0d04002a,0x0902000a,0x09060008,0x0901001f,0x0905000a,0x09050009,0x0905000b,
0x00000000,0x01010001,0x09060007,0x0d030075,0x10030036,0x09060008,0x0905000a,0x0905000b,
0x00000000,0x0d030069,0x09060008,0x0905000b,0x0d030051,0x0905000b,0x0905000b,0x09090009,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d03000c,0x05020003,0x05010004,0x03030003,
0x100300b6,0x10050011,0x04010003,0x06030004,0x10040018,0x06020005,0x05010005,0x04040004,
0x00000000,0x100300dc,0x05020003,0x10050015,0x10030037,0x05020004,0x05010006,0x07040005,
0x100201b5,0x10040020,0x05020004,0x07030006,0x0d030052,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0d040008,0x100300ef,0x0d03000d,0x06030004,0x06020005,0x0d06000f,
0x100103b4,0x0d04002b,0x0d040009,0x06030005,0x10040019,0x06020007,0x06020006,0x0d07000a,
0x00000000,0x100201db,0x06030004,0x0d050018,0x10030038,0x06030005,0x06020007,0x0d06000b,
0x00000000,0x0d03006a,0x06030005,0x07030007,0x0d030053,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0d030010,0x02020002,0x0d03000e,0x1006000d,0x1005000e,0x03030003,
0x00000000,0x10050012,0x0d030011,0x07040005,0x1004001a,0x07030006,0x1005000f,0x0d060012,
0x00000000,0x01010001,0x1006000d,0x0d040037,0x10030039,0x1006000e,0x10050010,0x07040006,
0x00000000,0x10040021,0x1006000e,0x07030008,0x0d030054,0x07030007,0x10050011,0x1009000f,
0x00000000,0x01010001,0x1005000e,0x02020002,0x0d03000f,0x07040005,0x07030006,0x0d05001e,
0x00000000,0x0d04002c,0x1005000f,0x07040006,0x1004001b,0x07030008,0x07030007,0x10080010,
0x00000000,0x01010001,0x07040005,0x0d030076,0x1003003a,0x07040006,0x07030008,0x0d06000c,
0x00000000,0x0d03006b,0x07040006,0x07030009,0x0d030055,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d030010,0x1004003a,0x1003003b,0x03030003,
0x100300b7,0x10050013,0x04010003,0x1006000e,0x1004001c,0x1005000f,0x1003003c,0x04040004,
0x00000000,0x01010001,0x1004003a,0x10050016,0x1003003b,0x1004003b,0x1003003d,0x0d07000a,
0x100201b6,0x10040022,0x1004003b,0x0d06000b,0x0d030056,0x10050010,0x1003003e,0x1007003c,
0x00000000,0x01010001,0x0d040009,0x02020002,0x0d030011,0x1006000e,0x1005000f,0x0d04003d,
0x100103b5,0x0d04002d,0x0d04000a,0x1006000f,0x1004001d,0x10050011,0x10050010,0x0d07000b,
0x00000000,0x01010001,0x1006000e,0x0d050019,0x1003003c,0x1006000f,0x10050011,0x0d06000d,
0x00000000,0x0d03006c,0x1006000f,0x0d06000c,0x0d030057,0x10050012,0x10050012,0x10090010,
0x00000000,0x01010001,0x0d02001f,0x02020002,0x0d030012,0x10050010,0x10040011,0x03030003,
0x00000000,0x10050014,0x0d020020,0x0d07000a,0x1004001e,0x0d06000b,0x10040012,0x0d050021,
0x00000000,0x01010001,0x10050010,0x0d040038,0x1003003d,0x10050011,0x10040013,0x0d07000b,
0x00000000,0x10040023,0x10050011,0x0d06000d,0x0d030058,0x0d06000c,0x10040014,0x10080012,
0x00000000,0x01010001,0x1005000f,0x02020002,0x0d030013,0x0d07000a,0x0d06000b,0x1006005f,
0x00000000,0x0d04002e,0x10050010,0x0d07000b,0x1004001f,0x0d06000d,0x0d06000c,0x10080011,
0x00000000,0x01010001,0x0d07000a,0x0d030077,0x1003003e,0x0d07000b,0x0d06000d,0x0d06000e,
0x00000000,0x0d03006d,0x0d07000b,0x0d06000e,0x0d030059,0x0d06000e,0x0d06000e,0x0d0a000c,
0x00000000,0x01010001,0x04010002,0x02020002,0x10050011,0x05020003,0x05010004,0x03030003,
0x100300b8,0x10050015,0x04010003,0x10050020,0x10040020,0x10040021,0x05010005,0x04040004,
0x00000000,0x100300dd,0x05020003,0x10050017,0x1003003f,0x05020004,0x05010006,0x0d06000c,
0x100201b7,0x10040024,0x05020004,0x0d05000d,0x0d03005a,0x10040022,0x05010007,0x05050005,
0x00000000,0x01010001,0x1006000d,0x02020002,0x10050012,0x10050020,0x10040021,0x0d060010,
0x100103b6,0x0d04002f,0x1006000e,0x10050021,0x10040021,0x10040023,0x10040022,0x1009000f,
0x00000000,0x100201dc,0x10050020,0x0d05001a,0x10030040,0x10050021,0x10040023,0x10080010,
0x00000000,0x0d03006e,0x10050021,0x0d05000e,0x0d03005b,0x10040024,0x10040024,0x10080022,
0x00000000,0x01010001,0x10040031,0x02020002,0x10050013,0x1006000e,0x1005000f,0x03030003,
0x00000000,0x10050016,0x10040032,0x0d06000c,0x10040022,0x0d05000d,0x10050010,0x10070033,
0x00000000,0x01010001,0x1006000e,0x0d040039,0x10030041,0x1006000f,0x10050011,0x0d06000d,
0x00000000,0x10040025,0x1006000f,0x0d05000f,0x0d03005c,0x0d05000e,0x10050012,0x10090010,
0x00000000,0x01010001,0x10050010,0x02020002,0x10050014,0x0d06000c,0x0d05000d,0x0d05001f,
0x00000000,0x0d040030,0x10050011,0x0d06000d,0x10040023,0x0d05000f,0x0d05000e,0x10080012,
0x00000000,0x01010001,0x0d06000c,0x0d030078,0x10030042,0x0d06000d,0x0d05000f,0x10080011,
0x00000000,0x0d03006f,0x0d06000d,0x0d050010,0x0d03005d,0x0d050010,0x0d050010,0x0d09000e,
0x00000000,0x01010001,0x04010002,0x02020002,0x10050015,0x10030079,0x1002007a,0x03030003,
0x100300b9,0x10050017,0x04010003,0x0d06000b,0x10040024,0x0d05000c,0x1002007b,0x04040004,
0x00000000,0x01010001,0x10030079,0x10050018,0x10030043,0x1003007a,0x1002007c,0x1009000f,
0x100201b8,0x10040026,0x1003007a,0x10080010,0x0d03005e,0x0d05000d,0x1002007d,0x1006007b,
0x00000000,0x01010001,0x1006000e,0x02020002,0x10050016,0x0d06000b,0x0d05000c,0x0d04003e,
0x100103b7,0x0d040031,0x1006000f,0x0d06000c,0x10040025,0x0d05000e,0x0d05000d,0x10090010,
0x00000000,0x01010001,0x0d06000b,0x0d05001b,0x10030044,0x0d06000c,0x0d05000e,0x10080012,
0x00000000,0x0d030070,0x0d06000c,0x10080011,0x0d03005f,0x0d05000f,0x0d05000f,0x0d09000d,
0x00000000,0x01010001,0x10030070,0x02020002,0x10050017,0x10050011,0x10040012,0x03030003,
0x00000000,0x10050018,0x10030071,0x1009000f,0x10040026,0x10080010,0x10040013,0x10060072,
0x00000000,0x01010001,0x10050011,0x0d04003a,0x10030045,0x10050012,0x10040014,0x10090010,
0x00000000,0x10040027,0x10050012,0x10080012,0x0d030060,0x10080011,0x10040015,0x10080013,
0x00000000,0x01010001,0x10050011,0x02020002,0x10050018,0x1009000f,0x10080010,0x10060060,
0x00000000,0x0d040032,0x10050012,0x10090010,0x10040027,0x10080012,0x10080011,0x10080013,
0x00000000,0x01010001,0x1009000f,0x0d030079,0x10030046,0x10090010,0x10080012,0x10080013,
0x00000000,0x0d030071,0x10090010,0x10080013,0x0d030061,0x10080013,0x10080013,0x100c0011,
0x00000000,0x01010001,0x04010002,0x02020002,0x08010008,0x05020003,0x05010004,0x03030003,
0x100300ba,0x10050010,0x04010003,0x06030004,0x10040017,0x06020005,0x05010005,0x04040004,
0x00000000,0x100300de,0x05020003,0x0d06000d,0x0d040023,0x05020004,0x05010006,0x07040005,
0x100201b9,0x0d050014,0x05020004,0x07030006,0x0d030062,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x08020004,0x100300f0,0x08010009,0x06030004,0x06020005,0x0d060011,
0x100103b8,0x0d040033,0x08020005,0x06030005,0x10040018,0x06020007,0x06020006,0x08050006,
0x00000000,0x100201dd,0x06030004,0x0d05001c,0x0d040024,0x06030005,0x06020007,0x08040007,
0x00000000,0x0d030072,0x06030005,0x07030007,0x0d030063,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0d030011,0x02020002,0x0801000a,0x1006000c,0x1005000d,0x03030003,
0x00000000,0x10050011,0x0d030012,0x07040005,0x10040019,0x07030006,0x1005000e,0x0d060013,
0x00000000,0x01010001,0x1006000c,0x0d04003b,0x0d040025,0x1006000d,0x1005000f,0x07040006,
0x00000000,0x0d050015,0x1006000d,0x07030008,0x0d030064,0x07030007,0x10050010,0x1009000e,
0x00000000,0x01010001,0x1005000d,0x02020002,0x0801000b,0x07040005,0x07030006,0x0d050020,
0x00000000,0x0d040034,0x1005000e,0x07040006,0x1004001a,0x07030008,0x07030007,0x1008000f,
0x00000000,0x01010001,0x07040005,0x0d03007a,0x0d040026,0x07040006,0x07030008,0x08040008,
0x00000000,0x0d030073,0x07040006,0x07030009,0x0d030065,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0801000c,0x0d030012,0x0d020013,0x03030003,
0x100300bb,0x10050012,0x04010003,0x0d070009,0x1004001b,0x0d06000a,0x0d020014,0x04040004,
0x00000000,0x01010001,0x0d030012,0x0d06000e,0x0d040027,0x0d030013,0x0d020015,0x08050006,
0x100201ba,0x0d050016,0x0d030013,0x08040007,0x0d030066,0x0d06000b,0x0d020016,0x0d060014,
0x00000000,0x01010001,0x08020005,0x02020002,0x0801000d,0x0d070009,0x0d06000a,0x0d04003f,
0x100103b9,0x0d040035,0x08020006,0x0d07000a,0x1004001c,0x0d06000c,0x0d06000b,0x08050007,
0x00000000,0x01010001,0x0d070009,0x0d05001d,0x0d040028,0x0d07000a,0x0d06000c,0x08040009,
0x00000000,0x0d030074,0x0d07000a,0x08040008,0x0d030067,0x0d06000d,0x0d06000d,0x0d0a000b,
0x00000000,0x01010001,0x0d020020,0x02020002,0x0801000e,0x0d06000a,0x0d05000b,0x03030003,
0x00000000,0x10050013,0x0d020021,0x08050006,0x1004001d,0x08040007,0x0d05000c,0x0d050022,
0x00000000,0x01010001,0x0d06000a,0x0d04003c,0x0d040029,0x0d06000b,0x0d05000d,0x08050007,
0x00000000,0x0d050017,0x0d06000b,0x08040009,0x0d030068,0x08040008,0x0d05000e,0x0d09000c,
0x00000000,0x01010001,0x1005000e,0x02020002,0x0801000f,0x08050006,0x08040007,0x10060061,
0x00000000,0x0d040036,0x1005000f,0x08050007,0x1004001e,0x08040009,0x08040008,0x10080010,
0x00000000,0x01010001,0x08050006,0x0d03007b,0x0d04002a,0x08050007,0x08040009,0x0804000a,
0x00000000,0x0d030075,0x08050007,0x0804000a,0x0d030069,0x0804000a,0x0804000a,0x08080008,
0x00000000,0x01010001,0x04010002,0x02020002,0x10050010,0x05020003,0x05010004,0x03030003,
0x100300bc,0x10050014,0x04010003,0x1004003f,0x1004001f,0x10030040,0x05010005,0x04040004,
0x00000000,0x100300df,0x05020003,0x0d06000f,0x0d04002b,0x05020004,0x05010006,0x0d07000a,
0x100201bb,0x0d050018,0x05020004,0x0d06000b,0x0d03006a,0x10030041,0x05010007,0x05050005,
0x00000000,0x01010001,0x1006000c,0x02020002,0x10050011,0x1004003f,0x10030040,0x0d060012,
0x100103ba,0x0d040037,0x1006000d,0x10040040,0x10040020,0x10030042,0x10030041,0x1009000e,
0x00000000,0x100201de,0x1004003f,0x0d05001e,0x0d04002c,0x10040040,0x10030042,0x1008000f,
0x00000000,0x0d030076,0x10040040,0x0d06000c,0x0d03006b,0x10030043,0x10030043,0x10070041,
0x00000000,0x01010001,0x10040032,0x02020002,0x10050012,0x1006000d,0x1005000e,0x03030003,
0x00000000,0x10050015,0x10040033,0x0d07000a,0x10040021,0x0d06000b,0x1005000f,0x10070034,
0x00000000,0x01010001,0x1006000d,0x0d04003d,0x0d04002d,0x1006000e,0x10050010,0x0d07000b,
0x00000000,0x0d050019,0x1006000e,0x0d06000d,0x0d03006c,0x0d06000c,0x10050011,0x1009000f,
0x00000000,0x01010001,0x1005000f,0x02020002,0x10050013,0x0d07000a,0x0d06000b,0x0d050021,
0x00000000,0x0d040038,0x10050010,0x0d07000b,0x10040022,0x0d06000d,0x0d06000c,0x10080011,
0x00000000,0x01010001,0x0d07000a,0x0d03007c,0x0d04002e,0x0d07000b,0x0d06000d,0x10080010,
0x00000000,0x0d030077,0x0d07000b,0x0d06000e,0x0d03006d,0x0d06000e,0x0d06000e,0x0d0a000c,
0x00000000,0x01010001,0x04010002,0x02020002,0x10050014,0x1005001c,0x1004001d,0x03030003,
0x100300bd,0x10050016,0x04010003,0x0d06000c,0x10040023,0x0d05000d,0x1004001e,0x04040004,
0x00000000,0x01010001,0x1005001c,0x0d060010,0x0d04002f,0x1005001d,0x1004001f,0x1009000e,
0x100201bc,0x0d05001a,0x1005001d,0x1008000f,0x0d03006e,0x0d05000e,0x10040020,0x1008001e,
0x00000000,0x01010001,0x1006000d,0x02020002,0x10050015,0x0d06000c,0x0d05000d,0x0d040040,
0x100103bb,0x0d040039,0x1006000e,0x0d06000d,0x10040024,0x0d05000f,0x0d05000e,0x1009000f,
0x00000000,0x01010001,0x0d06000c,0x0d05001f,0x0d040030,0x0d06000d,0x0d05000f,0x10080011,
0x00000000,0x0d030078,0x0d06000d,0x10080010,0x0d03006f,0x0d050010,0x0d050010,0x0d09000e,
0x00000000,0x01010001,0x10030071,0x02020002,0x10050016,0x0d06000b,0x0d05000c,0x03030003,
0x00000000,0x10050017,0x10030072,0x1009000e,0x10040025,0x1008000f,0x0d05000d,0x10060073,
0x00000000,0x01010001,0x0d06000b,0x0d04003e,0x0d040031,0x0d06000c,0x0d05000e,0x1009000f,
0x00000000,0x0d05001b,0x0d06000c,0x10080011,0x0d030070,0x10080010,0x0d05000f,0x0d09000d,
0x00000000,0x01010001,0x10050010,0x02020002,0x10050017,0x1009000e,0x1008000f,0x10060062,
0x00000000,0x0d04003a,0x10050011,0x1009000f,0x10040026,0x10080011,0x10080010,0x10080012,
0x00000000,0x01010001,0x1009000e,0x0d03007d,0x0d040032,0x1009000f,0x10080011,0x10080012,
0x00000000,0x0d030079,0x1009000f,0x10080012,0x0d030071,0x10080012,0x10080012,0x100c0010,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d02000f,0x05020003,0x05010004,0x03030003,
0x100300be,0x0d06000d,0x04010003,0x06030004,0x0d050014,0x06020005,0x05010005,0x04040004,
0x00000000,0x100300e0,0x05020003,0x0d060011,0x0d040033,0x05020004,0x05010006,0x07040005,
0x100201bd,0x0d05001c,0x05020004,0x07030006,0x0d030072,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0d03000b,0x100300f1,0x0d020010,0x06030004,0x06020005,0x0d060013,
0x100103bc,0x0d04003b,0x0d03000c,0x06030005,0x0d050015,0x06020007,0x06020006,0x0d06000d,
0x00000000,0x100201df,0x06030004,0x0d050020,0x0d040034,0x06030005,0x06020007,0x0d05000e,
0x00000000,0x0d03007a,0x06030005,0x07030007,0x0d030073,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0d030012,0x02020002,0x0d020011,0x0d070009,0x0d06000a,0x03030003,
0x00000000,0x0d06000e,0x0d030013,0x07040005,0x0d050016,0x07030006,0x0d06000b,0x0d060014,
0x00000000,0x01010001,0x0d070009,0x0d04003f,0x0d040035,0x0d07000a,0x0d06000c,0x07040006,
0x00000000,0x0d05001d,0x0d07000a,0x07030008,0x0d030074,0x07030007,0x0d06000d,0x0d0a000b,
0x00000000,0x01010001,0x0d06000a,0x02020002,0x0d020012,0x07040005,0x07030006,0x0d050022,
0x00000000,0x0d04003c,0x0d06000b,0x07040006,0x0d050017,0x07030008,0x07030007,0x0d09000c,
0x00000000,0x01010001,0x07040005,0x0d03007e,0x0d040036,0x07040006,0x07030008,0x0d05000f,
0x00000000,0x0d03007b,0x07040006,0x07030009,0x0d030075,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d020013,0x1004003b,0x1003003c,0x03030003,
0x100300bf,0x0d06000f,0x04010003,0x0d07000a,0x0d050018,0x0d06000b,0x1003003d,0x04040004,
0x00000000,0x01010001,0x1004003b,0x0d060012,0x0d040037,0x1004003c,0x1003003e,0x0d06000d,
0x100201be,0x0d05001e,0x1004003c,0x0d05000e,0x0d030076,0x0d06000c,0x1003003f,0x1007003d,
0x00000000,0x01010001,0x0d03000c,0x02020002,0x0d020014,0x0d07000a,0x0d06000b,0x0d040041,
0x100103bd,0x0d04003d,0x0d03000d,0x0d07000b,0x0d050019,0x0d06000d,0x0d06000c,0x0d06000e,
0x00000000,0x01010001,0x0d07000a,0x0d050021,0x0d040038,0x0d07000b,0x0d06000d,0x0d050010,
0x00000000,0x0d03007c,0x0d07000b,0x0d05000f,0x0d030077,0x0d06000e,0x0d06000e,0x0d0a000c,
0x00000000,0x01010001,0x0d020021,0x02020002,0x0d020015,0x0d06000c,0x0d05000d,0x03030003,
0x00000000,0x0d060010,0x0d020022,0x0d06000d,0x0d05001a,0x0d05000e,0x0d05000e,0x0d050023,
0x00000000,0x01010001,0x0d06000c,0x0d040040,0x0d040039,0x0d06000d,0x0d05000f,0x0d06000e,
0x00000000,0x0d05001f,0x0d06000d,0x0d050010,0x0d030078,0x0d05000f,0x0d050010,0x0d09000e,
0x00000000,0x01010001,0x0d06000b,0x02020002,0x0d020016,0x0d06000d,0x0d05000e,0x10060063,
0x00000000,0x0d04003e,0x0d06000c,0x0d06000e,0x0d05001b,0x0d050010,0x0d05000f,0x0d09000d,
0x00000000,0x01010001,0x0d06000d,0x0d03007f,0x0d04003a,0x0d06000e,0x0d050010,0x0d050011,
0x00000000,0x0d03007d,0x0d06000e,0x0d050011,0x0d030079,0x0d050011,0x0d050011,0x0d09000f,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d06000d,0x05020003,0x05010004,0x03030003,
0x100300c0,0x0d060011,0x04010003,0x06030004,0x0d05001c,0x06020005,0x05010005,0x04040004,
0x00000000,0x100300e1,0x05020003,0x0d060013,0x0d04003b,0x05020004,0x05010006,0x0d06000d,
0x100201bf,0x0d050020,0x05020004,0x0d05000e,0x0d03007a,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0d070009,0x02020002,0x0d06000e,0x06030004,0x06020005,0x0d060014,
0x100103be,0x0d04003f,0x0d07000a,0x06030005,0x0d05001d,0x06020007,0x06020006,0x0d0a000b,
0x00000000,0x100201e0,0x06030004,0x0d050022,0x0d04003c,0x06030005,0x06020007,0x0d09000c,
0x00000000,0x0d03007e,0x06030005,0x0d05000f,0x0d03007b,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x10040033,0x02020002,0x0d06000f,0x0d07000a,0x0d06000b,0x03030003,
0x00000000,0x0d060012,0x10040034,0x0d06000d,0x0d05001e,0x0d05000e,0x0d06000c,0x10070035,
0x00000000,0x01010001,0x0d07000a,0x0d040041,0x0d04003d,0x0d07000b,0x0d06000d,0x0d06000e,
0x00000000,0x0d050021,0x0d07000b,0x0d050010,0x0d03007c,0x0d05000f,0x0d06000e,0x0d0a000c,
0x00000000,0x01010001,0x0d06000c,0x02020002,0x0d060010,0x0d06000d,0x0d05000e,0x0d050023,
0x00000000,0x0d040040,0x0d06000d,0x0d06000e,0x0d05001f,0x0d050010,0x0d05000f,0x0d09000e,
0x00000000,0x01010001,0x0d06000d,0x0d030080,0x0d04003e,0x0d06000e,0x0d050010,0x0d09000d,
0x00000000,0x0d03007f,0x0d06000e,0x0d050011,0x0d03007d,0x0d050011,0x0d050011,0x0d09000f,
0x00000000,0x01010001,0x04010002,0x02020002,0x0d060011,0x1003007a,0x1002007b,0x03030003,
0x100300c1,0x0d060013,0x04010003,0x0d06000d,0x0d050020,0x0d05000e,0x1002007c,0x04040004,
0x00000000,0x01010001,0x1003007a,0x0d060014,0x0d04003f,0x1003007b,0x1002007d,0x0d0a000b,
0x100201c0,0x0d050022,0x1003007b,0x0d09000c,0x0d03007e,0x0d05000f,0x1002007e,0x1006007c,
0x00000000,0x01010001,0x0d07000a,0x02020002,0x0d060012,0x0d06000d,0x0d05000e,0x0d040042,
0x100103bf,0x0d040041,0x0d07000b,0x0d06000e,0x0d050021,0x0d050010,0x0d05000f,0x0d0a000c,
0x00000000,0x01010001,0x0d06000d,0x0d050023,0x0d040040,0x0d06000e,0x0d050010,0x0d09000e,
0x00000000,0x0d030080,0x0d06000e,0x0d09000d,0x0d03007f,0x0d050011,0x0d050011,0x0d09000f,
0x00000000,0x01010001,0x10030072,0x02020002,0x0d060013,0x0d06000d,0x0d05000e,0x03030003,
0x00000000,0x0d060014,0x10030073,0x0d0a000b,0x0d050022,0x0d09000c,0x0d05000f,0x10060074,
0x00000000,0x01010001,0x0d06000d,0x0d040042,0x0d040041,0x0d06000e,0x0d050010,0x0d0a000c,
0x00000000,0x0d050023,0x0d06000e,0x0d09000e,0x0d030080,0x0d09000d,0x0d050011,0x0d09000f,
0x00000000,0x01010001,0x0d06000d,0x02020002,0x0d060014,0x0d0a000b,0x0d09000c,0x10060064,
0x00000000,0x0d040042,0x0d06000e,0x0d0a000c,0x0d050023,0x0d09000e,0x0d09000d,0x0d09000f,
0x00000000,0x01010001,0x0d0a000b,0x0d030081,0x0d040042,0x0d0a000c,0x0d09000e,0x0d09000f,
0x00000000,0x0d030081,0x0d0a000c,0x0d09000f,0x0d030081,0x0d09000f,0x0d09000f,0x0d0d000d,
0x00000000,0x01010001,0x04010002,0x02020002,0x08010008,0x05020003,0x05010004,0x03030003,
0x100300c2,0x09020009,0x04010003,0x06030004,0x09010010,0x06020005,0x05010005,0x04040004,
0x00000000,0x100300e2,0x05020003,0x0a03000a,0x0a010020,0x05020004,0x05010006,0x07040005,
0x100201c1,0x0a020011,0x05020004,0x07030006,0x0b010040,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x08020004,0x100300f2,0x08010009,0x06030004,0x06020005,0x0b04000b,
0x100103c0,0x0b020021,0x08020005,0x06030005,0x09010011,0x06020007,0x06020006,0x08050006,
0x00000000,0x100201e1,0x06030004,0x0b030012,0x0a010021,0x06030005,0x06020007,0x08040007,
0x00000000,0x0c020041,0x06030005,0x07030007,0x0b010041,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0c02000a,0x02020002,0x0801000a,0x09030005,0x09020006,0x03030003,
0x00000000,0x0902000a,0x0c02000b,0x07040005,0x09010012,0x07030006,0x09020007,0x0c05000c,
0x00000000,0x01010001,0x09030005,0x0c030022,0x0a010022,0x09030006,0x09020008,0x07040006,
0x00000000,0x0a020012,0x09030006,0x07030008,0x0b010042,0x07030007,0x09020009,0x09060007,
0x00000000,0x01010001,0x09020006,0x02020002,0x0801000b,0x07040005,0x07030006,0x0c040013,
0x00000000,0x0b020022,0x09020007,0x07040006,0x09010013,0x07030008,0x07030007,0x09050008,
0x00000000,0x01010001,0x07040005,0x10060045,0x0a010023,0x07040006,0x07030008,0x08040008,
0x00000000,0x0c020042,0x07040006,0x07030009,0x0b010043,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0801000c,0x1006000e,0x1005000f,0x03030003,
0x100300c3,0x0902000b,0x04010003,0x0a040006,0x09010014,0x0a030007,0x10050010,0x04040004,
0x00000000,0x01010001,0x1006000e,0x0a03000b,0x0a010024,0x1006000f,0x10050011,0x08050006,
0x100201c2,0x0a020013,0x1006000f,0x08040007,0x0b010044,0x0a030008,0x10050012,0x10090010,
0x00000000,0x01010001,0x08020005,0x02020002,0x0801000d,0x0a040006,0x0a030007,0x10070026,
0x100103c1,0x0b020023,0x08020006,0x0a040007,0x09010015,0x0a030009,0x0a030008,0x08050007,
0x00000000,0x01010001,0x0a040006,0x0b030013,0x0a010025,0x0a040007,0x0a030009,0x08040009,
0x00000000,0x0c020043,0x0a040007,0x08040008,0x0b010045,0x0a03000a,0x0a03000a,0x0a070008,
0x00000000,0x01010001,0x10050015,0x02020002,0x0801000e,0x0a030007,0x0a020008,0x03030003,
0x00000000,0x0902000c,0x10050016,0x08050006,0x09010016,0x08040007,0x0a020009,0x10080017,
0x00000000,0x01010001,0x0a030007,0x0c030023,0x0a010026,0x0a030008,0x0a02000a,0x08050007,
0x00000000,0x0a020014,0x0a030008,0x08040009,0x0b010046,0x08040008,0x0a02000b,0x0a060009,
0x00000000,0x01010001,0x09020007,0x02020002,0x0801000f,0x08050006,0x08040007,0x10060065,
0x00000000,0x0b020024,0x09020008,0x08050007,0x09010017,0x08040009,0x08040008,0x09050009,
0x00000000,0x01010001,0x08050006,0x10060046,0x0a010027,0x08050007,0x08040009,0x0804000a,
0x00000000,0x0c020044,0x08050007,0x0804000a,0x0b010047,0x0804000a,0x0804000a,0x08080008,
0x00000000,0x01010001,0x04010002,0x02020002,0x09020009,0x05020003,0x05010004,0x03030003,
0x100300c4,0x0902000d,0x04010003,0x10060012,0x09010018,0x10050013,0x05010005,0x04040004,
0x00000000,0x100300e3,0x05020003,0x0a03000c,0x0a010028,0x05020004,0x05010006,0x0b050007,
0x100201c3,0x0a020015,0x05020004,0x0b040008,0x0b010048,0x10050014,0x05010007,0x05050005,
0x00000000,0x01010001,0x09030005,0x02020002,0x0902000a,0x10060012,0x10050013,0x0b04000c,
0x100103c2,0x0b020025,0x09030006,0x10060013,0x09010019,0x10050015,0x10050014,0x09060007,
0x00000000,0x100201e2,0x10060012,0x0b030014,0x0a010029,0x10060013,0x10050015,0x09050008,
0x00000000,0x0c020045,0x10060013,0x0b040009,0x0b010049,0x10050016,0x10050016,0x10090014,
0x00000000,0x01010001,0x10040034,0x02020002,0x0902000b,0x09030006,0x09020007,0x03030003,
0x00000000,0x0902000e,0x10040035,0x0b050007,0x0901001a,0x0b040008,0x09020008,0x10070036,
0x00000000,0x01010001,0x09030006,0x0c030024,0x0a01002a,0x09030007,0x09020009,0x0b050008,
0x00000000,0x0a020016,0x09030007,0x0b04000a,0x0b01004a,0x0b040009,0x0902000a,0x09060008,
0x00000000,0x01010001,0x09020008,0x02020002,0x0902000c,0x0b050007,0x0b040008,0x0c040014,
0x00000000,0x0b020026,0x09020009,0x0b050008,0x0901001b,0x0b04000a,0x0b040009,0x0905000a,
0x00000000,0x01010001,0x0b050007,0x10060047,0x0a01002b,0x0b050008,0x0b04000a,0x09050009,
0x00000000,0x0c020046,0x0b050008,0x0b04000b,0x0b01004b,0x0b04000b,0x0b04000b,0x0b080009,
0x00000000,0x01010001,0x04010002,0x02020002,0x0902000d,0x1005001d,0x1004001e,0x03030003,
0x100300c5,0x0902000f,0x04010003,0x0b040008,0x0901001c,0x0b030009,0x1004001f,0x04040004,
0x00000000,0x01010001,0x1005001d,0x0a03000d,0x0a01002c,0x1005001e,0x10040020,0x09060007,
0x100201c4,0x0a020017,0x1005001e,0x09050008,0x0b01004c,0x0b03000a,0x10040021,0x1008001f,
0x00000000,0x01010001,0x09030006,0x02020002,0x0902000e,0x0b040008,0x0b030009,0x10070027,
0x100103c3,0x0b020027,0x09030007,0x0b040009,0x0901001d,0x0b03000b,0x0b03000a,0x09060008,
0x00000000,0x01010001,0x0b040008,0x0b030015,0x0a01002d,0x0b040009,0x0b03000b,0x0905000a,
0x00000000,0x0c020047,0x0b040009,0x09050009,0x0b01004d,0x0b03000c,0x0b03000c,0x0b07000a,
0x00000000,0x01010001,0x10030073,0x02020002,0x0902000f,0x0a030008,0x0a020009,0x03030003,
0x00000000,0x09020010,0x10030074,0x09060007,0x0901001e,0x09050008,0x0a02000a,0x10060075,
0x00000000,0x01010001,0x0a030008,0x0c030025,0x0a01002e,0x0a030009,0x0a02000b,0x09060008,
0x00000000,0x0a020018,0x0a030009,0x0905000a,0x0b01004e,0x09050009,0x0a02000c,0x0a06000a,
0x00000000,0x01010001,0x09020009,0x02020002,0x09020010,0x09060007,0x09050008,0x10060066,
0x00000000,0x0b020028,0x0902000a,0x09060008,0x0901001f,0x0905000a,0x09050009,0x0905000b,
0x00000000,0x01010001,0x09060007,0x10060048,0x0a01002f,0x09060008,0x0905000a,0x0905000b,
0x00000000,0x0c020048,0x09060008,0x0905000b,0x0b01004f,0x0905000b,0x0905000b,0x09090009,
0x00000000,0x01010001,0x04010002,0x02020002,0x0c02000a,0x05020003,0x05010004,0x03030003,
0x100300c6,0x0a03000a,0x04010003,0x06030004,0x0a020011,0x06020005,0x05010005,0x04040004,
0x00000000,0x100300e4,0x05020003,0x0a03000e,0x0a010030,0x05020004,0x05010006,0x10060014,
0x100201c5,0x0a020019,0x05020004,0x10050015,0x0b010050,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0c030006,0x100300f3,0x0c02000b,0x06030004,0x06020005,0x0b04000d,
0x100103c4,0x0b020029,0x0c030007,0x06030005,0x0a020012,0x06020007,0x06020006,0x0c060008,
0x00000000,0x100201e3,0x06030004,0x0b030016,0x0a010031,0x06030005,0x06020007,0x0c050009,
0x00000000,0x0c020049,0x06030005,0x10050016,0x0b010051,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0c02000b,0x02020002,0x0c02000c,0x0a040006,0x0a030007,0x03030003,
0x00000000,0x0a03000b,0x0c02000c,0x10060014,0x0a020013,0x10050015,0x0a030008,0x0c05000d,
0x00000000,0x01010001,0x0a040006,0x0c030026,0x0a010032,0x0a040007,0x0a030009,0x10060015,
0x00000000,0x0a02001a,0x0a040007,0x10050017,0x0b010052,0x10050016,0x0a03000a,0x0a070008,
0x00000000,0x01010001,0x0a030007,0x02020002,0x0c02000d,0x10060014,0x10050015,0x0c040015,
0x00000000,0x0b02002a,0x0a030008,0x10060015,0x0a020014,0x10050017,0x10050016,0x0a060009,
0x00000000,0x01010001,0x10060014,0x10060049,0x0a010033,0x10060015,0x10050017,0x0c05000a,
0x00000000,0x0c02004a,0x10060015,0x10050018,0x0b010053,0x10050018,0x10050018,0x10090016,
0x00000000,0x01010001,0x04010002,0x02020002,0x0c02000e,0x1004003c,0x1003003d,0x03030003,
0x100300c7,0x0a03000c,0x04010003,0x0a040007,0x0a020015,0x0a030008,0x1003003e,0x04040004,
0x00000000,0x01010001,0x1004003c,0x0a03000f,0x0a010034,0x1004003d,0x1003003f,0x0c060008,
0x100201c6,0x0a02001b,0x1004003d,0x0c050009,0x0b010054,0x0a030009,0x10030040,0x1007003e,
0x00000000,0x01010001,0x0c030007,0x02020002,0x0c02000f,0x0a040007,0x0a030008,0x10070028,
0x100103c5,0x0b02002b,0x0c030008,0x0a040008,0x0a020016,0x0a03000a,0x0a030009,0x0c060009,
0x00000000,0x01010001,0x0a040007,0x0b030017,0x0a010035,0x0a040008,0x0a03000a,0x0c05000b,
0x00000000,0x0c02004b,0x0a040008,0x0c05000a,0x0b010055,0x0a03000b,0x0a03000b,0x0a070009,
0x00000000,0x01010001,0x10050016,0x02020002,0x0c020010,0x0a030009,0x0a02000a,0x03030003,
0x00000000,0x0a03000d,0x10050017,0x0c060008,0x0a020017,0x0c050009,0x0a02000b,0x10080018,
0x00000000,0x01010001,0x0a030009,0x0c030027,0x0a010036,0x0a03000a,0x0a02000c,0x0c060009,
0x00000000,0x0a02001c,0x0a03000a,0x0c05000b,0x0b010056,0x0c05000a,0x0a02000d,0x0a06000b,
0x00000000,0x01010001,0x0a030008,0x02020002,0x0c020011,0x0c060008,0x0c050009,0x10060067,
0x00000000,0x0b02002c,0x0a030009,0x0c060009,0x0a020018,0x0c05000b,0x0c05000a,0x0a06000a,
0x00000000,0x01010001,0x0c060008,0x1006004a,0x0a010037,0x0c060009,0x0c05000b,0x0c05000c,
0x00000000,0x0c02004c,0x0c060009,0x0c05000c,0x0b010057,0x0c05000c,0x0c05000c,0x0c09000a,
0x00000000,0x01010001,0x04010002,0x02020002,0x0a03000a,0x05020003,0x05010004,0x03030003,
0x100300c8,0x0a03000e,0x04010003,0x10050021,0x0a020019,0x10040022,0x05010005,0x04040004,
0x00000000,0x100300e5,0x05020003,0x0a030010,0x0a010038,0x05020004,0x05010006,0x0c050009,
0x100201c7,0x0a02001d,0x05020004,0x0c04000a,0x0b010058,0x10040023,0x05010007,0x05050005,
0x00000000,0x01010001,0x0a040006,0x02020002,0x0a03000b,0x10050021,0x10040022,0x0b04000e,
0x100103c6,0x0b02002d,0x0a040007,0x10050022,0x0a02001a,0x10040024,0x10040023,0x0a070008,
0x00000000,0x100201e4,0x10050021,0x0b030018,0x0a010039,0x10050022,0x10040024,0x0a060009,
0x00000000,0x0c02004d,0x10050022,0x0c04000b,0x0b010059,0x10040025,0x10040025,0x10080023,
0x00000000,0x01010001,0x10040035,0x02020002,0x0a03000c,0x0a040007,0x0a030008,0x03030003,
0x00000000,0x0a03000f,0x10040036,0x0c050009,0x0a02001b,0x0c04000a,0x0a030009,0x10070037,
0x00000000,0x01010001,0x0a040007,0x0c030028,0x0a01003a,0x0a040008,0x0a03000a,0x0c05000a,
0x00000000,0x0a02001e,0x0a040008,0x0c04000c,0x0b01005a,0x0c04000b,0x0a03000b,0x0a070009,
0x00000000,0x01010001,0x0a030009,0x02020002,0x0a03000d,0x0c050009,0x0c04000a,0x0c040016,
0x00000000,0x0b02002e,0x0a03000a,0x0c05000a,0x0a02001c,0x0c04000c,0x0c04000b,0x0a06000b,
0x00000000,0x01010001,0x0c050009,0x1006004b,0x0a01003b,0x0c05000a,0x0c04000c,0x0a06000a,
0x00000000,0x0c02004e,0x0c05000a,0x0c04000d,0x0b01005b,0x0c04000d,0x0c04000d,0x0c08000b,
0x00000000,0x01010001,0x04010002,0x02020002,0x0a03000e,0x1003007b,0x1002007c,0x03030003,
0x100300c9,0x0a030010,0x04010003,0x0b040009,0x0a02001d,0x0b03000a,0x1002007d,0x04040004,
0x00000000,0x01010001,0x1003007b,0x0a030011,0x0a01003c,0x1003007c,0x1002007e,0x0a070008,
0x100201c8,0x0a02001f,0x1003007c,0x0a060009,0x0b01005c,0x0b03000b,0x1002007f,0x1006007d,
0x00000000,0x01010001,0x0a040007,0x02020002,0x0a03000f,0x0b040009,0x0b03000a,0x10070029,
0x100103c7,0x0b02002f,0x0a040008,0x0b04000a,0x0a02001e,0x0b03000c,0x0b03000b,0x0a070009,
0x00000000,0x01010001,0x0b040009,0x0b030019,0x0a01003d,0x0b04000a,0x0b03000c,0x0a06000b,
0x00000000,0x0c02004f,0x0b04000a,0x0a06000a,0x0b01005d,0x0b03000d,0x0b03000d,0x0b07000b,
0x00000000,0x01010001,0x10030074,0x02020002,0x0a030010,0x0a03000a,0x0a02000b,0x03030003,
0x00000000,0x0a030011,0x10030075,0x0a070008,0x0a02001f,0x0a060009,0x0a02000c,0x10060076,
0x00000000,0x01010001,0x0a03000a,0x0c030029,0x0a01003e,0x0a03000b,0x0a02000d,0x0a070009,
0x00000000,0x0a020020,0x0a03000b,0x0a06000b,0x0b01005e,0x0a06000a,0x0a02000e,0x0a06000c,
0x00000000,0x01010001,0x0a03000a,0x02020002,0x0a030011,0x0a070008,0x0a060009,0x10060068,
0x00000000,0x0b020030,0x0a03000b,0x0a070009,0x0a020020,0x0a06000b,0x0a06000a,0x0a06000c,
0x00000000,0x01010001,0x0a070008,0x1006004c,0x0a01003f,0x0a070009,0x0a06000b,0x0a06000c,
0x00000000,0x0c020050,0x0a070009,0x0a06000c,0x0b01005f,0x0a06000c,0x0a06000c,0x0a0a000a,
0x00000000,0x01010001,0x04010002,0x02020002,0x10020017,0x05020003,0x05010004,0x03030003,
0x100300ca,0x1006000e,0x04010003,0x06030004,0x10050015,0x06020005,0x05010005,0x04040004,
0x00000000,0x100300e6,0x05020003,0x0b04000b,0x0b020021,0x05020004,0x05010006,0x07040005,
0x100201c9,0x0b030012,0x05020004,0x07030006,0x0b010060,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x10030013,0x100300f4,0x10020018,0x06030004,0x06020005,0x0b04000f,
0x100103c8,0x0b020031,0x10030014,0x06030005,0x10050016,0x06020007,0x06020006,0x10060015,
0x00000000,0x100201e5,0x06030004,0x0b03001a,0x0b020022,0x06030005,0x06020007,0x10050016,
0x00000000,0x0c020051,0x06030005,0x07030007,0x0b010061,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0c02000c,0x02020002,0x10020019,0x1007000a,0x1006000b,0x03030003,
0x00000000,0x1006000f,0x0c02000d,0x07040005,0x10050017,0x07030006,0x1006000c,0x0c05000e,
0x00000000,0x01010001,0x1007000a,0x0c03002a,0x0b020023,0x1007000b,0x1006000d,0x07040006,
0x00000000,0x0b030013,0x1007000b,0x07030008,0x0b010062,0x07030007,0x1006000e,0x100a000c,
0x00000000,0x01010001,0x1006000b,0x02020002,0x1002001a,0x07040005,0x07030006,0x0c040017,
0x00000000,0x0b020032,0x1006000c,0x07040006,0x10050018,0x07030008,0x07030007,0x1009000d,
0x00000000,0x01010001,0x07040005,0x1006004d,0x0b020024,0x07040006,0x07030008,0x10050017,
0x00000000,0x0c020052,0x07040006,0x07030009,0x0b010063,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x1002001b,0x1006000f,0x10050010,0x03030003,
0x100300cb,0x10060010,0x04010003,0x0b050007,0x10050019,0x0b040008,0x10050011,0x04040004,
0x00000000,0x01010001,0x1006000f,0x0b04000c,0x0b020025,0x10060010,0x10050012,0x10060015,
0x100201ca,0x0b030014,0x10060010,0x10050016,0x0b010064,0x0b040009,0x10050013,0x10090011,
0x00000000,0x01010001,0x10030014,0x02020002,0x1002001c,0x0b050007,0x0b040008,0x1007002a,
0x100103c9,0x0b020033,0x10030015,0x0b050008,0x1005001a,0x0b04000a,0x0b040009,0x10060016,
0x00000000,0x01010001,0x0b050007,0x0b03001b,0x0b020026,0x0b050008,0x0b04000a,0x10050018,
0x00000000,0x0c020053,0x0b050008,0x10050017,0x0b010065,0x0b04000b,0x0b04000b,0x0b080009,
0x00000000,0x01010001,0x10050017,0x02020002,0x1002001d,0x0b040008,0x0b030009,0x03030003,
0x00000000,0x10060011,0x10050018,0x10060015,0x1005001b,0x10050016,0x0b03000a,0x10080019,
0x00000000,0x01010001,0x0b040008,0x0c03002b,0x0b020027,0x0b040009,0x0b03000b,0x10060016,
0x00000000,0x0b030015,0x0b040009,0x10050018,0x0b010066,0x10050017,0x0b03000c,0x0b07000a,
0x00000000,0x01010001,0x1006000c,0x02020002,0x1002001e,0x10060015,0x10050016,0x10060069,
0x00000000,0x0b020034,0x1006000d,0x10060016,0x1005001c,0x10050018,0x10050017,0x1009000e,
0x00000000,0x01010001,0x10060015,0x1006004e,0x0b020028,0x10060016,0x10050018,0x10050019,
0x00000000,0x0c020054,0x10060016,0x10050019,0x0b010067,0x10050019,0x10050019,0x10090017,
0x00000000,0x01010001,0x04010002,0x02020002,0x1006000e,0x05020003,0x05010004,0x03030003,
0x100300cc,0x10060012,0x04010003,0x10040040,0x1005001d,0x10030041,0x05010005,0x04040004,
0x00000000,0x100300e7,0x05020003,0x0b04000d,0x0b020029,0x05020004,0x05010006,0x0b050008,
0x100201cb,0x0b030016,0x05020004,0x0b040009,0x0b010068,0x10030042,0x05010007,0x05050005,
0x00000000,0x01010001,0x1007000a,0x02020002,0x1006000f,0x10040040,0x10030041,0x0b040010,
0x100103ca,0x0b020035,0x1007000b,0x10040041,0x1005001e,0x10030043,0x10030042,0x100a000c,
0x00000000,0x100201e6,0x10040040,0x0b03001c,0x0b02002a,0x10040041,0x10030043,0x1009000d,
0x00000000,0x0c020055,0x10040041,0x0b04000a,0x0b010069,0x10030044,0x10030044,0x10070042,
0x00000000,0x01010001,0x10040036,0x02020002,0x10060010,0x1007000b,0x1006000c,0x03030003,
0x00000000,0x10060013,0x10040037,0x0b050008,0x1005001f,0x0b040009,0x1006000d,0x10070038,
0x00000000,0x01010001,0x1007000b,0x0c03002c,0x0b02002b,0x1007000c,0x1006000e,0x0b050009,
0x00000000,0x0b030017,0x1007000c,0x0b04000b,0x0b01006a,0x0b04000a,0x1006000f,0x100a000d,
0x00000000,0x01010001,0x1006000d,0x02020002,0x10060011,0x0b050008,0x0b040009,0x0c040018,
0x00000000,0x0b020036,0x1006000e,0x0b050009,0x10050020,0x0b04000b,0x0b04000a,0x1009000f,
0x00000000,0x01010001,0x0b050008,0x1006004f,0x0b02002c,0x0b050009,0x0b04000b,0x1009000e,
0x00000000,0x0c020056,0x0b050009,0x0b04000c,0x0b01006b,0x0b04000c,0x0b04000c,0x0b08000a,
0x00000000,0x01010001,0x04010002,0x02020002,0x10060012,0x1005001e,0x1004001f,0x03030003,
0x100300cd,0x10060014,0x04010003,0x0b04000a,0x10050021,0x0b03000b,0x10040020,0x04040004,
0x00000000,0x01010001,0x1005001e,0x0b04000e,0x0b02002d,0x1005001f,0x10040021,0x100a000c,
0x100201cc,0x0b030018,0x1005001f,0x1009000d,0x0b01006c,0x0b03000c,0x10040022,0x10080020,
0x00000000,0x01010001,0x1007000b,0x02020002,0x10060013,0x0b04000a,0x0b03000b,0x1007002b,
0x100103cb,0x0b020037,0x1007000c,0x0b04000b,0x10050022,0x0b03000d,0x0b03000c,0x100a000d,
0x00000000,0x01010001,0x0b04000a,0x0b03001d,0x0b02002e,0x0b04000b,0x0b03000d,0x1009000f,
0x00000000,0x0c020057,0x0b04000b,0x1009000e,0x0b01006d,0x0b03000e,0x0b03000e,0x0b07000c,
0x00000000,0x01010001,0x10030075,0x02020002,0x10060014,0x0b040009,0x0b03000a,0x03030003,
0x00000000,0x10060015,0x10030076,0x100a000c,0x10050023,0x1009000d,0x0b03000b,0x10060077,
0x00000000,0x01010001,0x0b040009,0x0c03002d,0x0b02002f,0x0b04000a,0x0b03000c,0x100a000d,
0x00000000,0x0b030019,0x0b04000a,0x1009000f,0x0b01006e,0x1009000e,0x0b03000d,0x0b07000b,
0x00000000,0x01010001,0x1006000e,0x02020002,0x10060015,0x100a000c,0x1009000d,0x1006006a,
0x00000000,0x0b020038,0x1006000f,0x100a000d,0x10050024,0x1009000f,0x1009000e,0x10090010,
0x00000000,0x01010001,0x100a000c,0x10060050,0x0b020030,0x100a000d,0x1009000f,0x10090010,
0x00000000,0x0c020058,0x100a000d,0x10090010,0x0b01006f,0x10090010,0x10090010,0x100d000e,
0x00000000,0x01010001,0x04010002,0x02020002,0x1005000f,0x05020003,0x05010004,0x03030003,
0x100300ce,0x0b04000b,0x04010003,0x06030004,0x0b030012,0x06020005,0x05010005,0x04040004,
0x00000000,0x100300e8,0x05020003,0x0b04000f,0x0b020031,0x05020004,0x05010006,0x10050023,
0x100201cd,0x0b03001a,0x05020004,0x10040024,0x0b010070,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x1006000b,0x100300f5,0x10050010,0x06030004,0x06020005,0x0b040011,
0x100103cc,0x0b020039,0x1006000c,0x06030005,0x0b030013,0x06020007,0x06020006,0x1009000d,
0x00000000,0x100201e7,0x06030004,0x0b03001e,0x0b020032,0x06030005,0x06020007,0x1008000e,
0x00000000,0x0c020059,0x06030005,0x10040025,0x0b010071,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0c02000d,0x02020002,0x10050011,0x0b050007,0x0b040008,0x03030003,
0x00000000,0x0b04000c,0x0c02000e,0x10050023,0x0b030014,0x10040024,0x0b040009,0x0c05000f,
0x00000000,0x01010001,0x0b050007,0x0c03002e,0x0b020033,0x0b050008,0x0b04000a,0x10050024,
0x00000000,0x0b03001b,0x0b050008,0x10040026,0x0b010072,0x10040025,0x0b04000b,0x0b080009,
0x00000000,0x01010001,0x0b040008,0x02020002,0x10050012,0x10050023,0x10040024,0x0c040019,
0x00000000,0x0b02003a,0x0b040009,0x10050024,0x0b030015,0x10040026,0x10040025,0x0b07000a,
0x00000000,0x01010001,0x10050023,0x10060051,0x0b020034,0x10050024,0x10040026,0x1008000f,
0x00000000,0x0c02005a,0x10050024,0x10040027,0x0b010073,0x10040027,0x10040027,0x10080025,
0x00000000,0x01010001,0x04010002,0x02020002,0x10050013,0x1004003d,0x1003003e,0x03030003,
0x100300cf,0x0b04000d,0x04010003,0x0b050008,0x0b030016,0x0b040009,0x1003003f,0x04040004,
0x00000000,0x01010001,0x1004003d,0x0b040010,0x0b020035,0x1004003e,0x10030040,0x1009000d,
0x100201ce,0x0b03001c,0x1004003e,0x1008000e,0x0b010074,0x0b04000a,0x10030041,0x1007003f,
0x00000000,0x01010001,0x1006000c,0x02020002,0x10050014,0x0b050008,0x0b040009,0x1007002c,
0x100103cd,0x0b02003b,0x1006000d,0x0b050009,0x0b030017,0x0b04000b,0x0b04000a,0x1009000e,
0x00000000,0x01010001,0x0b050008,0x0b03001f,0x0b020036,0x0b050009,0x0b04000b,0x10080010,
0x00000000,0x0c02005b,0x0b050009,0x1008000f,0x0b010075,0x0b04000c,0x0b04000c,0x0b08000a,
0x00000000,0x01010001,0x10050018,0x02020002,0x10050015,0x0b04000a,0x0b03000b,0x03030003,
0x00000000,0x0b04000e,0x10050019,0x1009000d,0x0b030018,0x1008000e,0x0b03000c,0x1008001a,
0x00000000,0x01010001,0x0b04000a,0x0c03002f,0x0b020037,0x0b04000b,0x0b03000d,0x1009000e,
0x00000000,0x0b03001d,0x0b04000b,0x10080010,0x0b010076,0x1008000f,0x0b03000e,0x0b07000c,
0x00000000,0x01010001,0x0b040009,0x02020002,0x10050016,0x1009000d,0x1008000e,0x1006006b,
0x00000000,0x0b02003c,0x0b04000a,0x1009000e,0x0b030019,0x10080010,0x1008000f,0x0b07000b,
0x00000000,0x01010001,0x1009000d,0x10060052,0x0b020038,0x1009000e,0x10080010,0x10080011,
0x00000000,0x0c02005c,0x1009000e,0x10080011,0x0b010077,0x10080011,0x10080011,0x100c000f,
0x00000000,0x01010001,0x04010002,0x02020002,0x0b04000b,0x05020003,0x05010004,0x03030003,
0x100300d0,0x0b04000f,0x04010003,0x06030004,0x0b03001a,0x06020005,0x05010005,0x04040004,
0x00000000,0x100300e9,0x05020003,0x0b040011,0x0b020039,0x05020004,0x05010006,0x0c05000a,
0x100201cf,0x0b03001e,0x05020004,0x0c04000b,0x0b010078,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0b050007,0x02020002,0x0b04000c,0x06030004,0x06020005,0x0b040012,
0x100103ce,0x0b02003d,0x0b050008,0x06030005,0x0b03001b,0x06020007,0x06020006,0x0b080009,
0x00000000,0x100201e8,0x06030004,0x0b030020,0x0b02003a,0x06030005,0x06020007,0x0b07000a,
0x00000000,0x0c02005d,0x06030005,0x0c04000c,0x0b010079,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x10040037,0x02020002,0x0b04000d,0x0b050008,0x0b040009,0x03030003,
0x00000000,0x0b040010,0x10040038,0x0c05000a,0x0b03001c,0x0c04000b,0x0b04000a,0x10070039,
0x00000000,0x01010001,0x0b050008,0x0c030030,0x0b02003b,0x0b050009,0x0b04000b,0x0c05000b,
0x00000000,0x0b03001f,0x0b050009,0x0c04000d,0x0b01007a,0x0c04000c,0x0b04000c,0x0b08000a,
0x00000000,0x01010001,0x0b04000a,0x02020002,0x0b04000e,0x0c05000a,0x0c04000b,0x0c04001a,
0x00000000,0x0b02003e,0x0b04000b,0x0c05000b,0x0b03001d,0x0c04000d,0x0c04000c,0x0b07000c,
0x00000000,0x01010001,0x0c05000a,0x10060053,0x0b02003c,0x0c05000b,0x0c04000d,0x0b07000b,
0x00000000,0x0c02005e,0x0c05000b,0x0c04000e,0x0b01007b,0x0c04000e,0x0c04000e,0x0c08000c,
0x00000000,0x01010001,0x04010002,0x02020002,0x0b04000f,0x1003007c,0x1002007d,0x03030003,
0x100300d1,0x0b040011,0x04010003,0x0b04000b,0x0b03001e,0x0b03000c,0x1002007e,0x04040004,
0x00000000,0x01010001,0x1003007c,0x0b040012,0x0b02003d,0x1003007d,0x1002007f,0x0b080009,
0x100201d0,0x0b030020,0x1003007d,0x0b07000a,0x0b01007c,0x0b03000d,0x10020080,0x1006007e,
0x00000000,0x01010001,0x0b050008,0x02020002,0x0b040010,0x0b04000b,0x0b03000c,0x1007002d,
0x100103cf,0x0b02003f,0x0b050009,0x0b04000c,0x0b03001f,0x0b03000e,0x0b03000d,0x0b08000a,
0x00000000,0x01010001,0x0b04000b,0x0b030021,0x0b02003e,0x0b04000c,0x0b03000e,0x0b07000c,
0x00000000,0x0c02005f,0x0b04000c,0x0b07000b,0x0b01007d,0x0b03000f,0x0b03000f,0x0b07000d,
0x00000000,0x01010001,0x10030076,0x02020002,0x0b040011,0x0b04000b,0x0b03000c,0x03030003,
0x00000000,0x0b040012,0x10030077,0x0b080009,0x0b030020,0x0b07000a,0x0b03000d,0x10060078,
0x00000000,0x01010001,0x0b04000b,0x0c030031,0x0b02003f,0x0b04000c,0x0b03000e,0x0b08000a,
0x00000000,0x0b030021,0x0b04000c,0x0b07000c,0x0b01007e,0x0b07000b,0x0b03000f,0x0b07000d,
0x00000000,0x01010001,0x0b04000b,0x02020002,0x0b040012,0x0b080009,0x0b07000a,0x1006006c,
0x00000000,0x0b020040,0x0b04000c,0x0b08000a,0x0b030021,0x0b07000c,0x0b07000b,0x0b07000d,
0x00000000,0x01010001,0x0b080009,0x10060054,0x0b020040,0x0b08000a,0x0b07000c,0x0b07000d,
0x00000000,0x0c020060,0x0b08000a,0x0b07000d,0x0b01007f,0x0b07000d,0x0b07000d,0x0b0b000b,
0x00000000,0x01010001,0x04010002,0x02020002,0x08010008,0x05020003,0x05010004,0x03030003,
0x100300d2,0x09020009,0x04010003,0x06030004,0x09010010,0x06020005,0x05010005,0x04040004,
0x00000000,0x100300ea,0x05020003,0x1006000f,0x10040025,0x05020004,0x05010006,0x07040005,
0x100201d1,0x10050016,0x05020004,0x07030006,0x0c020041,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x08020004,0x100300f6,0x08010009,0x06030004,0x06020005,0x0c05000c,
0x100103d0,0x0c030022,0x08020005,0x06030005,0x09010011,0x06020007,0x06020006,0x08050006,
0x00000000,0x100201e9,0x06030004,0x0c040013,0x10040026,0x06030005,0x06020007,0x08040007,
0x00000000,0x0c020061,0x06030005,0x07030007,0x0c020042,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0c02000e,0x02020002,0x0801000a,0x09030005,0x09020006,0x03030003,
0x00000000,0x0902000a,0x0c02000f,0x07040005,0x09010012,0x07030006,0x09020007,0x0c050010,
0x00000000,0x01010001,0x09030005,0x0c030032,0x10040027,0x09030006,0x09020008,0x07040006,
0x00000000,0x10050017,0x09030006,0x07030008,0x0c020043,0x07030007,0x09020009,0x09060007,
0x00000000,0x01010001,0x09020006,0x02020002,0x0801000b,0x07040005,0x07030006,0x0c04001b,
0x00000000,0x0c030023,0x09020007,0x07040006,0x09010013,0x07030008,0x07030007,0x09050008,
0x00000000,0x01010001,0x07040005,0x10060055,0x10040028,0x07040006,0x07030008,0x08040008,
0x00000000,0x0c020062,0x07040006,0x07030009,0x0c020044,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0801000c,0x10060010,0x10050011,0x03030003,
0x100300d3,0x0902000b,0x04010003,0x1007000b,0x09010014,0x1006000c,0x10050012,0x04040004,
0x00000000,0x01010001,0x10060010,0x10060010,0x10040029,0x10060011,0x10050013,0x08050006,
0x100201d2,0x10050018,0x10060011,0x08040007,0x0c020045,0x1006000d,0x10050014,0x10090012,
0x00000000,0x01010001,0x08020005,0x02020002,0x0801000d,0x1007000b,0x1006000c,0x1007002e,
0x100103d1,0x0c030024,0x08020006,0x1007000c,0x09010015,0x1006000e,0x1006000d,0x08050007,
0x00000000,0x01010001,0x1007000b,0x0c040014,0x1004002a,0x1007000c,0x1006000e,0x08040009,
0x00000000,0x0c020063,0x1007000c,0x08040008,0x0c020046,0x1006000f,0x1006000f,0x100a000d,
0x00000000,0x01010001,0x10050019,0x02020002,0x0801000e,0x1006000c,0x1005000d,0x03030003,
0x00000000,0x0902000c,0x1005001a,0x08050006,0x09010016,0x08040007,0x1005000e,0x1008001b,
0x00000000,0x01010001,0x1006000c,0x0c030033,0x1004002b,0x1006000d,0x1005000f,0x08050007,
0x00000000,0x10050019,0x1006000d,0x08040009,0x0c020047,0x08040008,0x10050010,0x1009000e,
0x00000000,0x01010001,0x09020007,0x02020002,0x0801000f,0x08050006,0x08040007,0x1006006d,
0x00000000,0x0c030025,0x09020008,0x08050007,0x09010017,0x08040009,0x08040008,0x09050009,
0x00000000,0x01010001,0x08050006,0x10060056,0x1004002c,0x08050007,0x08040009,0x0804000a,
0x00000000,0x0c020064,0x08050007,0x0804000a,0x0c020048,0x0804000a,0x0804000a,0x08080008,
0x00000000,0x01010001,0x04010002,0x02020002,0x09020009,0x05020003,0x05010004,0x03030003,
0x100300d4,0x0902000d,0x04010003,0x10060013,0x09010018,0x10050014,0x05010005,0x04040004,
0x00000000,0x100300eb,0x05020003,0x10060011,0x1004002d,0x05020004,0x05010006,0x0c060008,
0x100201d3,0x1005001a,0x05020004,0x0c050009,0x0c020049,0x10050015,0x05010007,0x05050005,
0x00000000,0x01010001,0x09030005,0x02020002,0x0902000a,0x10060013,0x10050014,0x0c05000d,
0x100103d2,0x0c030026,0x09030006,0x10060014,0x09010019,0x10050016,0x10050015,0x09060007,
0x00000000,0x100201ea,0x10060013,0x0c040015,0x1004002e,0x10060014,0x10050016,0x09050008,
0x00000000,0x0c020065,0x10060014,0x0c05000a,0x0c02004a,0x10050017,0x10050017,0x10090015,
0x00000000,0x01010001,0x10040038,0x02020002,0x0902000b,0x09030006,0x09020007,0x03030003,
0x00000000,0x0902000e,0x10040039,0x0c060008,0x0901001a,0x0c050009,0x09020008,0x1007003a,
0x00000000,0x01010001,0x09030006,0x0c030034,0x1004002f,0x09030007,0x09020009,0x0c060009,
0x00000000,0x1005001b,0x09030007,0x0c05000b,0x0c02004b,0x0c05000a,0x0902000a,0x09060008,
0x00000000,0x01010001,0x09020008,0x02020002,0x0902000c,0x0c060008,0x0c050009,0x0c04001c,
0x00000000,0x0c030027,0x09020009,0x0c060009,0x0901001b,0x0c05000b,0x0c05000a,0x0905000a,
0x00000000,0x01010001,0x0c060008,0x10060057,0x10040030,0x0c060009,0x0c05000b,0x09050009,
0x00000000,0x0c020066,0x0c060009,0x0c05000c,0x0c02004c,0x0c05000c,0x0c05000c,0x0c09000a,
0x00000000,0x01010001,0x04010002,0x02020002,0x0902000d,0x1005001f,0x10040020,0x03030003,
0x100300d5,0x0902000f,0x04010003,0x0c050009,0x0901001c,0x0c04000a,0x10040021,0x04040004,
0x00000000,0x01010001,0x1005001f,0x10060012,0x10040031,0x10050020,0x10040022,0x09060007,
0x100201d4,0x1005001c,0x10050020,0x09050008,0x0c02004d,0x0c04000b,0x10040023,0x10080021,
0x00000000,0x01010001,0x09030006,0x02020002,0x0902000e,0x0c050009,0x0c04000a,0x1007002f,
0x100103d3,0x0c030028,0x09030007,0x0c05000a,0x0901001d,0x0c04000c,0x0c04000b,0x09060008,
0x00000000,0x01010001,0x0c050009,0x0c040016,0x10040032,0x0c05000a,0x0c04000c,0x0905000a,
0x00000000,0x0c020067,0x0c05000a,0x09050009,0x0c02004e,0x0c04000d,0x0c04000d,0x0c08000b,
0x00000000,0x01010001,0x10030077,0x02020002,0x0902000f,0x1006000d,0x1005000e,0x03030003,
0x00000000,0x09020010,0x10030078,0x09060007,0x0901001e,0x09050008,0x1005000f,0x10060079,
0x00000000,0x01010001,0x1006000d,0x0c030035,0x10040033,0x1006000e,0x10050010,0x09060008,
0x00000000,0x1005001d,0x1006000e,0x0905000a,0x0c02004f,0x09050009,0x10050011,0x1009000f,
0x00000000,0x01010001,0x09020009,0x02020002,0x09020010,0x09060007,0x09050008,0x1006006e,
0x00000000,0x0c030029,0x0902000a,0x09060008,0x0901001f,0x0905000a,0x09050009,0x0905000b,
0x00000000,0x01010001,0x09060007,0x10060058,0x10040034,0x09060008,0x0905000a,0x0905000b,
0x00000000,0x0c020068,0x09060008,0x0905000b,0x0c020050,0x0905000b,0x0905000b,0x09090009,
0x00000000,0x01010001,0x04010002,0x02020002,0x0c02000b,0x05020003,0x05010004,0x03030003,
0x100300d6,0x1006000f,0x04010003,0x06030004,0x10050016,0x06020005,0x05010005,0x04040004,
0x00000000,0x100300ec,0x05020003,0x10060013,0x10040035,0x05020004,0x05010006,0x07040005,
0x100201d5,0x1005001e,0x05020004,0x07030006,0x0c020051,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0c030007,0x100300f7,0x0c02000c,0x06030004,0x06020005,0x0c05000e,
0x100103d4,0x0c03002a,0x0c030008,0x06030005,0x10050017,0x06020007,0x06020006,0x0c060009,
0x00000000,0x100201eb,0x06030004,0x0c040017,0x10040036,0x06030005,0x06020007,0x0c05000a,
0x00000000,0x0c020069,0x06030005,0x07030007,0x0c020052,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0c02000f,0x02020002,0x0c02000d,0x1007000b,0x1006000c,0x03030003,
0x00000000,0x10060010,0x0c020010,0x07040005,0x10050018,0x07030006,0x1006000d,0x0c050011,
0x00000000,0x01010001,0x1007000b,0x0c030036,0x10040037,0x1007000c,0x1006000e,0x07040006,
0x00000000,0x1005001f,0x1007000c,0x07030008,0x0c020053,0x07030007,0x1006000f,0x100a000d,
0x00000000,0x01010001,0x1006000c,0x02020002,0x0c02000e,0x07040005,0x07030006,0x0c04001d,
0x00000000,0x0c03002b,0x1006000d,0x07040006,0x10050019,0x07030008,0x07030007,0x1009000e,
0x00000000,0x01010001,0x07040005,0x10060059,0x10040038,0x07040006,0x07030008,0x0c05000b,
0x00000000,0x0c02006a,0x07040006,0x07030009,0x0c020054,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0c02000f,0x1004003e,0x1003003f,0x03030003,
0x100300d7,0x10060011,0x04010003,0x1007000c,0x1005001a,0x1006000d,0x10030040,0x04040004,
0x00000000,0x01010001,0x1004003e,0x10060014,0x10040039,0x1004003f,0x10030041,0x0c060009,
0x100201d6,0x10050020,0x1004003f,0x0c05000a,0x0c020055,0x1006000e,0x10030042,0x10070040,
0x00000000,0x01010001,0x0c030008,0x02020002,0x0c020010,0x1007000c,0x1006000d,0x10070030,
0x100103d5,0x0c03002c,0x0c030009,0x1007000d,0x1005001b,0x1006000f,0x1006000e,0x0c06000a,
0x00000000,0x01010001,0x1007000c,0x0c040018,0x1004003a,0x1007000d,0x1006000f,0x0c05000c,
0x00000000,0x0c02006b,0x1007000d,0x0c05000b,0x0c020056,0x10060010,0x10060010,0x100a000e,
0x00000000,0x01010001,0x1005001a,0x02020002,0x0c020011,0x1006000e,0x1005000f,0x03030003,
0x00000000,0x10060012,0x1005001b,0x0c060009,0x1005001c,0x0c05000a,0x10050010,0x1008001c,
0x00000000,0x01010001,0x1006000e,0x0c030037,0x1004003b,0x1006000f,0x10050011,0x0c06000a,
0x00000000,0x10050021,0x1006000f,0x0c05000c,0x0c020057,0x0c05000b,0x10050012,0x10090010,
0x00000000,0x01010001,0x1006000d,0x02020002,0x0c020012,0x0c060009,0x0c05000a,0x1006006f,
0x00000000,0x0c03002d,0x1006000e,0x0c06000a,0x1005001d,0x0c05000c,0x0c05000b,0x1009000f,
0x00000000,0x01010001,0x0c060009,0x1006005a,0x1004003c,0x0c06000a,0x0c05000c,0x0c05000d,
0x00000000,0x0c02006c,0x0c06000a,0x0c05000d,0x0c020058,0x0c05000d,0x0c05000d,0x0c09000b,
0x00000000,0x01010001,0x04010002,0x02020002,0x1006000f,0x05020003,0x05010004,0x03030003,
0x100300d8,0x10060013,0x04010003,0x10050022,0x1005001e,0x10040023,0x05010005,0x04040004,
0x00000000,0x100300ed,0x05020003,0x10060015,0x1004003d,0x05020004,0x05010006,0x0c05000b,
0x100201d7,0x10050022,0x05020004,0x0c04000c,0x0c020059,0x10040024,0x05010007,0x05050005,
0x00000000,0x01010001,0x1007000b,0x02020002,0x10060010,0x10050022,0x10040023,0x0c05000f,
0x100103d6,0x0c03002e,0x1007000c,0x10050023,0x1005001f,0x10040025,0x10040024,0x100a000d,
0x00000000,0x100201ec,0x10050022,0x0c040019,0x1004003e,0x10050023,0x10040025,0x1009000e,
0x00000000,0x0c02006d,0x10050023,0x0c04000d,0x0c02005a,0x10040026,0x10040026,0x10080024,
0x00000000,0x01010001,0x10040039,0x02020002,0x10060011,0x1007000c,0x1006000d,0x03030003,
0x00000000,0x10060014,0x1004003a,0x0c05000b,0x10050020,0x0c04000c,0x1006000e,0x1007003b,
0x00000000,0x01010001,0x1007000c,0x0c030038,0x1004003f,0x1007000d,0x1006000f,0x0c05000c,
0x00000000,0x10050023,0x1007000d,0x0c04000e,0x0c02005b,0x0c04000d,0x10060010,0x100a000e,
0x00000000,0x01010001,0x1006000e,0x02020002,0x10060012,0x0c05000b,0x0c04000c,0x0c04001e,
0x00000000,0x0c03002f,0x1006000f,0x0c05000c,0x10050021,0x0c04000e,0x0c04000d,0x10090010,
0x00000000,0x01010001,0x0c05000b,0x1006005b,0x10040040,0x0c05000c,0x0c04000e,0x1009000f,
0x00000000,0x0c02006e,0x0c05000c,0x0c04000f,0x0c02005c,0x0c04000f,0x0c04000f,0x0c08000d,
0x00000000,0x01010001,0x04010002,0x02020002,0x10060013,0x1003007d,0x1002007e,0x03030003,
0x100300d9,0x10060015,0x04010003,0x0c05000a,0x10050022,0x0c04000b,0x1002007f,0x04040004,
0x00000000,0x01010001,0x1003007d,0x10060016,0x10040041,0x1003007e,0x10020080,0x100a000d,
0x100201d8,0x10050024,0x1003007e,0x1009000e,0x0c02005d,0x0c04000c,0x10020081,0x1006007f,
0x00000000,0x01010001,0x1007000c,0x02020002,0x10060014,0x0c05000a,0x0c04000b,0x10070031,
0x100103d7,0x0c030030,0x1007000d,0x0c05000b,0x10050023,0x0c04000d,0x0c04000c,0x100a000e,
0x00000000,0x01010001,0x0c05000a,0x0c04001a,0x10040042,0x0c05000b,0x0c04000d,0x10090010,
0x00000000,0x0c02006f,0x0c05000b,0x1009000f,0x0c02005e,0x0c04000e,0x0c04000e,0x0c08000c,
0x00000000,0x01010001,0x10030078,0x02020002,0x10060015,0x1006000f,0x10050010,0x03030003,
0x00000000,0x10060016,0x10030079,0x100a000d,0x10050024,0x1009000e,0x10050011,0x1006007a,
0x00000000,0x01010001,0x1006000f,0x0c030039,0x10040043,0x10060010,0x10050012,0x100a000e,
0x00000000,0x10050025,0x10060010,0x10090010,0x0c02005f,0x1009000f,0x10050013,0x10090011,
0x00000000,0x01010001,0x1006000f,0x02020002,0x10060016,0x100a000d,0x1009000e,0x10060070,
0x00000000,0x0c030031,0x10060010,0x100a000e,0x10050025,0x10090010,0x1009000f,0x10090011,
0x00000000,0x01010001,0x100a000d,0x1006005c,0x10040044,0x100a000e,0x10090010,0x10090011,
0x00000000,0x0c020070,0x100a000e,0x10090011,0x0c020060,0x10090011,0x10090011,0x100d000f,
0x00000000,0x01010001,0x04010002,0x02020002,0x08010008,0x05020003,0x05010004,0x03030003,
0x100300da,0x10050011,0x04010003,0x06030004,0x10040018,0x06020005,0x05010005,0x04040004,
0x00000000,0x100300ee,0x05020003,0x0c05000c,0x0c030022,0x05020004,0x05010006,0x07040005,
0x100201d9,0x0c040013,0x05020004,0x07030006,0x0c020061,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x08020004,0x100300f8,0x08010009,0x06030004,0x06020005,0x0c050010,
0x100103d8,0x0c030032,0x08020005,0x06030005,0x10040019,0x06020007,0x06020006,0x08050006,
0x00000000,0x100201ed,0x06030004,0x0c04001b,0x0c030023,0x06030005,0x06020007,0x08040007,
0x00000000,0x0c020071,0x06030005,0x07030007,0x0c020062,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0c020010,0x02020002,0x0801000a,0x1006000d,0x1005000e,0x03030003,
0x00000000,0x10050012,0x0c020011,0x07040005,0x1004001a,0x07030006,0x1005000f,0x0c050012,
0x00000000,0x01010001,0x1006000d,0x0c03003a,0x0c030024,0x1006000e,0x10050010,0x07040006,
0x00000000,0x0c040014,0x1006000e,0x07030008,0x0c020063,0x07030007,0x10050011,0x1009000f,
0x00000000,0x01010001,0x1005000e,0x02020002,0x0801000b,0x07040005,0x07030006,0x0c04001f,
0x00000000,0x0c030033,0x1005000f,0x07040006,0x1004001b,0x07030008,0x07030007,0x10080010,
0x00000000,0x01010001,0x07040005,0x1006005d,0x0c030025,0x07040006,0x07030008,0x08040008,
0x00000000,0x0c020072,0x07040006,0x07030009,0x0c020064,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0801000c,0x10060011,0x10050012,0x03030003,
0x100300db,0x10050013,0x04010003,0x0c060008,0x1004001c,0x0c050009,0x10050013,0x04040004,
0x00000000,0x01010001,0x10060011,0x0c05000d,0x0c030026,0x10060012,0x10050014,0x08050006,
0x100201da,0x0c040015,0x10060012,0x08040007,0x0c020065,0x0c05000a,0x10050015,0x10090013,
0x00000000,0x01010001,0x08020005,0x02020002,0x0801000d,0x0c060008,0x0c050009,0x10070032,
0x100103d9,0x0c030034,0x08020006,0x0c060009,0x1004001d,0x0c05000b,0x0c05000a,0x08050007,
0x00000000,0x01010001,0x0c060008,0x0c04001c,0x0c030027,0x0c060009,0x0c05000b,0x08040009,
0x00000000,0x0c020073,0x0c060009,0x08040008,0x0c020066,0x0c05000c,0x0c05000c,0x0c09000a,
0x00000000,0x01010001,0x1005001b,0x02020002,0x0801000e,0x0c050009,0x0c04000a,0x03030003,
0x00000000,0x10050014,0x1005001c,0x08050006,0x1004001e,0x08040007,0x0c04000b,0x1008001d,
0x00000000,0x01010001,0x0c050009,0x0c03003b,0x0c030028,0x0c05000a,0x0c04000c,0x08050007,
0x00000000,0x0c040016,0x0c05000a,0x08040009,0x0c020067,0x08040008,0x0c04000d,0x0c08000b,
0x00000000,0x01010001,0x1005000f,0x02020002,0x0801000f,0x08050006,0x08040007,0x10060071,
0x00000000,0x0c030035,0x10050010,0x08050007,0x1004001f,0x08040009,0x08040008,0x10080011,
0x00000000,0x01010001,0x08050006,0x1006005e,0x0c030029,0x08050007,0x08040009,0x0804000a,
0x00000000,0x0c020074,0x08050007,0x0804000a,0x0c020068,0x0804000a,0x0804000a,0x08080008,
0x00000000,0x01010001,0x04010002,0x02020002,0x10050011,0x05020003,0x05010004,0x03030003,
0x100300dc,0x10050015,0x04010003,0x10040041,0x10040020,0x10030042,0x05010005,0x04040004,
0x00000000,0x100300ef,0x05020003,0x0c05000e,0x0c03002a,0x05020004,0x05010006,0x0c060009,
0x100201db,0x0c040017,0x05020004,0x0c05000a,0x0c020069,0x10030043,0x05010007,0x05050005,
0x00000000,0x01010001,0x1006000d,0x02020002,0x10050012,0x10040041,0x10030042,0x0c050011,
0x100103da,0x0c030036,0x1006000e,0x10040042,0x10040021,0x10030044,0x10030043,0x1009000f,
0x00000000,0x100201ee,0x10040041,0x0c04001d,0x0c03002b,0x10040042,0x10030044,0x10080010,
0x00000000,0x0c020075,0x10040042,0x0c05000b,0x0c02006a,0x10030045,0x10030045,0x10070043,
0x00000000,0x01010001,0x1004003a,0x02020002,0x10050013,0x1006000e,0x1005000f,0x03030003,
0x00000000,0x10050016,0x1004003b,0x0c060009,0x10040022,0x0c05000a,0x10050010,0x1007003c,
0x00000000,0x01010001,0x1006000e,0x0c03003c,0x0c03002c,0x1006000f,0x10050011,0x0c06000a,
0x00000000,0x0c040018,0x1006000f,0x0c05000c,0x0c02006b,0x0c05000b,0x10050012,0x10090010,
0x00000000,0x01010001,0x10050010,0x02020002,0x10050014,0x0c060009,0x0c05000a,0x0c040020,
0x00000000,0x0c030037,0x10050011,0x0c06000a,0x10040023,0x0c05000c,0x0c05000b,0x10080012,
0x00000000,0x01010001,0x0c060009,0x1006005f,0x0c03002d,0x0c06000a,0x0c05000c,0x10080011,
0x00000000,0x0c020076,0x0c06000a,0x0c05000d,0x0c02006c,0x0c05000d,0x0c05000d,0x0c09000b,
0x00000000,0x01010001,0x04010002,0x02020002,0x10050015,0x10050020,0x10040021,0x03030003,
0x100300dd,0x10050017,0x04010003,0x0c05000b,0x10040024,0x0c04000c,0x10040022,0x04040004,
0x00000000,0x01010001,0x10050020,0x0c05000f,0x0c03002e,0x10050021,0x10040023,0x1009000f,
0x100201dc,0x0c040019,0x10050021,0x10080010,0x0c02006d,0x0c04000d,0x10040024,0x10080022,
0x00000000,0x01010001,0x1006000e,0x02020002,0x10050016,0x0c05000b,0x0c04000c,0x10070033,
0x100103db,0x0c030038,0x1006000f,0x0c05000c,0x10040025,0x0c04000e,0x0c04000d,0x10090010,
0x00000000,0x01010001,0x0c05000b,0x0c04001e,0x0c03002f,0x0c05000c,0x0c04000e,0x10080012,
0x00000000,0x0c020077,0x0c05000c,0x10080011,0x0c02006e,0x0c04000f,0x0c04000f,0x0c08000d,
0x00000000,0x01010001,0x10030079,0x02020002,0x10050017,0x0c05000a,0x0c04000b,0x03030003,
0x00000000,0x10050018,0x1003007a,0x1009000f,0x10040026,0x10080010,0x0c04000c,0x1006007b,
0x00000000,0x01010001,0x0c05000a,0x0c03003d,0x0c030030,0x0c05000b,0x0c04000d,0x10090010,
0x00000000,0x0c04001a,0x0c05000b,0x10080012,0x0c02006f,0x10080011,0x0c04000e,0x0c08000c,
0x00000000,0x01010001,0x10050011,0x02020002,0x10050018,0x1009000f,0x10080010,0x10060072,
0x00000000,0x0c030039,0x10050012,0x10090010,0x10040027,0x10080012,0x10080011,0x10080013,
0x00000000,0x01010001,0x1009000f,0x10060060,0x0c030031,0x10090010,0x10080012,0x10080013,
0x00000000,0x0c020078,0x10090010,0x10080013,0x0c020070,0x10080013,0x10080013,0x100c0011,
0x00000000,0x01010001,0x04010002,0x02020002,0x10050010,0x05020003,0x05010004,0x03030003,
0x100300de,0x0c05000c,0x04010003,0x06030004,0x0c040013,0x06020005,0x05010005,0x04040004,
0x00000000,0x100300f0,0x05020003,0x0c050010,0x0c030032,0x05020004,0x05010006,0x07040005,
0x100201dd,0x0c04001b,0x05020004,0x07030006,0x0c020071,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x1006000c,0x100300f9,0x10050011,0x06030004,0x06020005,0x0c050012,
0x100103dc,0x0c03003a,0x1006000d,0x06030005,0x0c040014,0x06020007,0x06020006,0x1009000e,
0x00000000,0x100201ef,0x06030004,0x0c04001f,0x0c030033,0x06030005,0x06020007,0x1008000f,
0x00000000,0x0c020079,0x06030005,0x07030007,0x0c020072,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x0c020011,0x02020002,0x10050012,0x0c060008,0x0c050009,0x03030003,
0x00000000,0x0c05000d,0x0c020012,0x07040005,0x0c040015,0x07030006,0x0c05000a,0x0c050013,
0x00000000,0x01010001,0x0c060008,0x0c03003e,0x0c030034,0x0c060009,0x0c05000b,0x07040006,
0x00000000,0x0c04001c,0x0c060009,0x07030008,0x0c020073,0x07030007,0x0c05000c,0x0c09000a,
0x00000000,0x01010001,0x0c050009,0x02020002,0x10050013,0x07040005,0x07030006,0x0c040021,
0x00000000,0x0c03003b,0x0c05000a,0x07040006,0x0c040016,0x07030008,0x07030007,0x0c08000b,
0x00000000,0x01010001,0x07040005,0x10060061,0x0c030035,0x07040006,0x07030008,0x10080010,
0x00000000,0x0c02007a,0x07040006,0x07030009,0x0c020074,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x10050014,0x1004003f,0x10030040,0x03030003,
0x100300df,0x0c05000e,0x04010003,0x0c060009,0x0c040017,0x0c05000a,0x10030041,0x04040004,
0x00000000,0x01010001,0x1004003f,0x0c050011,0x0c030036,0x10040040,0x10030042,0x1009000e,
0x100201de,0x0c04001d,0x10040040,0x1008000f,0x0c020075,0x0c05000b,0x10030043,0x10070041,
0x00000000,0x01010001,0x1006000d,0x02020002,0x10050015,0x0c060009,0x0c05000a,0x10070034,
0x100103dd,0x0c03003c,0x1006000e,0x0c06000a,0x0c040018,0x0c05000c,0x0c05000b,0x1009000f,
0x00000000,0x01010001,0x0c060009,0x0c040020,0x0c030037,0x0c06000a,0x0c05000c,0x10080011,
0x00000000,0x0c02007b,0x0c06000a,0x10080010,0x0c020076,0x0c05000d,0x0c05000d,0x0c09000b,
0x00000000,0x01010001,0x1005001c,0x02020002,0x10050016,0x0c05000b,0x0c04000c,0x03030003,
0x00000000,0x0c05000f,0x1005001d,0x1009000e,0x0c040019,0x1008000f,0x0c04000d,0x1008001e,
0x00000000,0x01010001,0x0c05000b,0x0c03003f,0x0c030038,0x0c05000c,0x0c04000e,0x1009000f,
0x00000000,0x0c04001e,0x0c05000c,0x10080011,0x0c020077,0x10080010,0x0c04000f,0x0c08000d,
0x00000000,0x01010001,0x0c05000a,0x02020002,0x10050017,0x1009000e,0x1008000f,0x10060073,
0x00000000,0x0c03003d,0x0c05000b,0x1009000f,0x0c04001a,0x10080011,0x10080010,0x0c08000c,
0x00000000,0x01010001,0x1009000e,0x10060062,0x0c030039,0x1009000f,0x10080011,0x10080012,
0x00000000,0x0c02007c,0x1009000f,0x10080012,0x0c020078,0x10080012,0x10080012,0x100c0010,
0x00000000,0x01010001,0x04010002,0x02020002,0x0c05000c,0x05020003,0x05010004,0x03030003,
0x100300e0,0x0c050010,0x04010003,0x06030004,0x0c04001b,0x06020005,0x05010005,0x04040004,
0x00000000,0x100300f1,0x05020003,0x0c050012,0x0c03003a,0x05020004,0x05010006,0x0c05000c,
0x100201df,0x0c04001f,0x05020004,0x0c04000d,0x0c020079,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x0c060008,0x02020002,0x0c05000d,0x06030004,0x06020005,0x0c050013,
0x100103de,0x0c03003e,0x0c060009,0x06030005,0x0c04001c,0x06020007,0x06020006,0x0c09000a,
0x00000000,0x100201f0,0x06030004,0x0c040021,0x0c03003b,0x06030005,0x06020007,0x0c08000b,
0x00000000,0x0c02007d,0x06030005,0x0c04000e,0x0c02007a,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x1004003b,0x02020002,0x0c05000e,0x0c060009,0x0c05000a,0x03030003,
0x00000000,0x0c050011,0x1004003c,0x0c05000c,0x0c04001d,0x0c04000d,0x0c05000b,0x1007003d,
0x00000000,0x01010001,0x0c060009,0x0c030040,0x0c03003c,0x0c06000a,0x0c05000c,0x0c05000d,
0x00000000,0x0c040020,0x0c06000a,0x0c04000f,0x0c02007b,0x0c04000e,0x0c05000d,0x0c09000b,
0x00000000,0x01010001,0x0c05000b,0x02020002,0x0c05000f,0x0c05000c,0x0c04000d,0x0c040022,
0x00000000,0x0c03003f,0x0c05000c,0x0c05000d,0x0c04001e,0x0c04000f,0x0c04000e,0x0c08000d,
0x00000000,0x01010001,0x0c05000c,0x10060063,0x0c03003d,0x0c05000d,0x0c04000f,0x0c08000c,
0x00000000,0x0c02007e,0x0c05000d,0x0c040010,0x0c02007c,0x0c040010,0x0c040010,0x0c08000e,
0x00000000,0x01010001,0x04010002,0x02020002,0x0c050010,0x1003007e,0x1002007f,0x03030003,
0x100300e1,0x0c050012,0x04010003,0x0c05000c,0x0c04001f,0x0c04000d,0x10020080,0x04040004,
0x00000000,0x01010001,0x1003007e,0x0c050013,0x0c03003e,0x1003007f,0x10020081,0x0c09000a,
0x100201e0,0x0c040021,0x1003007f,0x0c08000b,0x0c02007d,0x0c04000e,0x10020082,0x10060080,
0x00000000,0x01010001,0x0c060009,0x02020002,0x0c050011,0x0c05000c,0x0c04000d,0x10070035,
0x100103df,0x0c030040,0x0c06000a,0x0c05000d,0x0c040020,0x0c04000f,0x0c04000e,0x0c09000b,
0x00000000,0x01010001,0x0c05000c,0x0c040022,0x0c03003f,0x0c05000d,0x0c04000f,0x0c08000d,
0x00000000,0x0c02007f,0x0c05000d,0x0c08000c,0x0c02007e,0x0c040010,0x0c040010,0x0c08000e,
0x00000000,0x01010001,0x1003007a,0x02020002,0x0c050012,0x0c05000c,0x0c04000d,0x03030003,
0x00000000,0x0c050013,0x1003007b,0x0c09000a,0x0c040021,0x0c08000b,0x0c04000e,0x1006007c,
0x00000000,0x01010001,0x0c05000c,0x0c030041,0x0c030040,0x0c05000d,0x0c04000f,0x0c09000b,
0x00000000,0x0c040022,0x0c05000d,0x0c08000d,0x0c02007f,0x0c08000c,0x0c040010,0x0c08000e,
0x00000000,0x01010001,0x0c05000c,0x02020002,0x0c050013,0x0c09000a,0x0c08000b,0x10060074,
0x00000000,0x0c030041,0x0c05000d,0x0c09000b,0x0c040022,0x0c08000d,0x0c08000c,0x0c08000e,
0x00000000,0x01010001,0x0c09000a,0x10060064,0x0c030041,0x0c09000b,0x0c08000d,0x0c08000e,
0x00000000,0x0c020080,0x0c09000b,0x0c08000e,0x0c020080,0x0c08000e,0x0c08000e,0x0c0c000c,
0x00000000,0x01010001,0x04010002,0x02020002,0x08010008,0x05020003,0x05010004,0x03030003,
0x100300e2,0x09020009,0x04010003,0x06030004,0x09010010,0x06020005,0x05010005,0x04040004,
0x00000000,0x100300f2,0x05020003,0x0a03000a,0x0a010020,0x05020004,0x05010006,0x07040005,
0x100201e1,0x0a020011,0x05020004,0x07030006,0x10020047,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x08020004,0x100300fa,0x08010009,0x06030004,0x06020005,0x10050012,
0x100103e0,0x10030028,0x08020005,0x06030005,0x09010011,0x06020007,0x06020006,0x08050006,
0x00000000,0x100201f1,0x06030004,0x10040019,0x0a010021,0x06030005,0x06020007,0x08040007,
0x00000000,0x10060045,0x06030005,0x07030007,0x10020048,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x1006000e,0x02020002,0x0801000a,0x09030005,0x09020006,0x03030003,
0x00000000,0x0902000a,0x1006000f,0x07040005,0x09010012,0x07030006,0x09020007,0x10090010,
0x00000000,0x01010001,0x09030005,0x10070026,0x0a010022,0x09030006,0x09020008,0x07040006,
0x00000000,0x0a020012,0x09030006,0x07030008,0x10020049,0x07030007,0x09020009,0x09060007,
0x00000000,0x01010001,0x09020006,0x02020002,0x0801000b,0x07040005,0x07030006,0x10080017,
0x00000000,0x10030029,0x09020007,0x07040006,0x09010013,0x07030008,0x07030007,0x09050008,
0x00000000,0x01010001,0x07040005,0x10060065,0x0a010023,0x07040006,0x07030008,0x08040008,
0x00000000,0x10060046,0x07040006,0x07030009,0x1002004a,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0801000c,0x10060012,0x10050013,0x03030003,
0x100300e3,0x0902000b,0x04010003,0x0a040006,0x09010014,0x0a030007,0x10050014,0x04040004,
0x00000000,0x01010001,0x10060012,0x0a03000b,0x0a010024,0x10060013,0x10050015,0x08050006,
0x100201e2,0x0a020013,0x10060013,0x08040007,0x1002004b,0x0a030008,0x10050016,0x10090014,
0x00000000,0x01010001,0x08020005,0x02020002,0x0801000d,0x0a040006,0x0a030007,0x10070036,
0x100103e1,0x1003002a,0x08020006,0x0a040007,0x09010015,0x0a030009,0x0a030008,0x08050007,
0x00000000,0x01010001,0x0a040006,0x1004001a,0x0a010025,0x0a040007,0x0a030009,0x08040009,
0x00000000,0x10060047,0x0a040007,0x08040008,0x1002004c,0x0a03000a,0x0a03000a,0x0a070008,
0x00000000,0x01010001,0x1005001d,0x02020002,0x0801000e,0x0a030007,0x0a020008,0x03030003,
0x00000000,0x0902000c,0x1005001e,0x08050006,0x09010016,0x08040007,0x0a020009,0x1008001f,
0x00000000,0x01010001,0x0a030007,0x10070027,0x0a010026,0x0a030008,0x0a02000a,0x08050007,
0x00000000,0x0a020014,0x0a030008,0x08040009,0x1002004d,0x08040008,0x0a02000b,0x0a060009,
0x00000000,0x01010001,0x09020007,0x02020002,0x0801000f,0x08050006,0x08040007,0x10060075,
0x00000000,0x1003002b,0x09020008,0x08050007,0x09010017,0x08040009,0x08040008,0x09050009,
0x00000000,0x01010001,0x08050006,0x10060066,0x0a010027,0x08050007,0x08040009,0x0804000a,
0x00000000,0x10060048,0x08050007,0x0804000a,0x1002004e,0x0804000a,0x0804000a,0x08080008,
0x00000000,0x01010001,0x04010002,0x02020002,0x09020009,0x05020003,0x05010004,0x03030003,
0x100300e4,0x0902000d,0x04010003,0x10060014,0x09010018,0x10050015,0x05010005,0x04040004,
0x00000000,0x100300f3,0x05020003,0x0a03000c,0x0a010028,0x05020004,0x05010006,0x1006000e,
0x100201e3,0x0a020015,0x05020004,0x1005000f,0x1002004f,0x10050016,0x05010007,0x05050005,
0x00000000,0x01010001,0x09030005,0x02020002,0x0902000a,0x10060014,0x10050015,0x10050013,
0x100103e2,0x1003002c,0x09030006,0x10060015,0x09010019,0x10050017,0x10050016,0x09060007,
0x00000000,0x100201f2,0x10060014,0x1004001b,0x0a010029,0x10060015,0x10050017,0x09050008,
0x00000000,0x10060049,0x10060015,0x10050010,0x10020050,0x10050018,0x10050018,0x10090016,
0x00000000,0x01010001,0x1004003c,0x02020002,0x0902000b,0x09030006,0x09020007,0x03030003,
0x00000000,0x0902000e,0x1004003d,0x1006000e,0x0901001a,0x1005000f,0x09020008,0x1007003e,
0x00000000,0x01010001,0x09030006,0x10070028,0x0a01002a,0x09030007,0x09020009,0x1006000f,
0x00000000,0x0a020016,0x09030007,0x10050011,0x10020051,0x10050010,0x0902000a,0x09060008,
0x00000000,0x01010001,0x09020008,0x02020002,0x0902000c,0x1006000e,0x1005000f,0x10080018,
0x00000000,0x1003002d,0x09020009,0x1006000f,0x0901001b,0x10050011,0x10050010,0x0905000a,
0x00000000,0x01010001,0x1006000e,0x10060067,0x0a01002b,0x1006000f,0x10050011,0x09050009,
0x00000000,0x1006004a,0x1006000f,0x10050012,0x10020052,0x10050012,0x10050012,0x10090010,
0x00000000,0x01010001,0x04010002,0x02020002,0x0902000d,0x10050021,0x10040022,0x03030003,
0x100300e5,0x0902000f,0x04010003,0x1005000f,0x0901001c,0x10040010,0x10040023,0x04040004,
0x00000000,0x01010001,0x10050021,0x0a03000d,0x0a01002c,0x10050022,0x10040024,0x09060007,
0x100201e4,0x0a020017,0x10050022,0x09050008,0x10020053,0x10040011,0x10040025,0x10080023,
0x00000000,0x01010001,0x09030006,0x02020002,0x0902000e,0x1005000f,0x10040010,0x10070037,
0x100103e3,0x1003002e,0x09030007,0x10050010,0x0901001d,0x10040012,0x10040011,0x09060008,
0x00000000,0x01010001,0x1005000f,0x1004001c,0x0a01002d,0x10050010,0x10040012,0x0905000a,
0x00000000,0x1006004b,0x10050010,0x09050009,0x10020054,0x10040013,0x10040013,0x10080011,
0x00000000,0x01010001,0x1003007b,0x02020002,0x0902000f,0x0a030008,0x0a020009,0x03030003,
0x00000000,0x09020010,0x1003007c,0x09060007,0x0901001e,0x09050008,0x0a02000a,0x1006007d,
0x00000000,0x01010001,0x0a030008,0x10070029,0x0a01002e,0x0a030009,0x0a02000b,0x09060008,
0x00000000,0x0a020018,0x0a030009,0x0905000a,0x10020055,0x09050009,0x0a02000c,0x0a06000a,
0x00000000,0x01010001,0x09020009,0x02020002,0x09020010,0x09060007,0x09050008,0x10060076,
0x00000000,0x1003002f,0x0902000a,0x09060008,0x0901001f,0x0905000a,0x09050009,0x0905000b,
0x00000000,0x01010001,0x09060007,0x10060068,0x0a01002f,0x09060008,0x0905000a,0x0905000b,
0x00000000,0x1006004c,0x09060008,0x0905000b,0x10020056,0x0905000b,0x0905000b,0x09090009,
0x00000000,0x01010001,0x04010002,0x02020002,0x1006000e,0x05020003,0x05010004,0x03030003,
0x100300e6,0x0a03000a,0x04010003,0x06030004,0x0a020011,0x06020005,0x05010005,0x04040004,
0x00000000,0x100300f4,0x05020003,0x0a03000e,0x0a010030,0x05020004,0x05010006,0x10060015,
0x100201e5,0x0a020019,0x05020004,0x10050016,0x10020057,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x1007000a,0x100300fb,0x1006000f,0x06030004,0x06020005,0x10050014,
0x100103e4,0x10030030,0x1007000b,0x06030005,0x0a020012,0x06020007,0x06020006,0x100a000c,
0x00000000,0x100201f3,0x06030004,0x1004001d,0x0a010031,0x06030005,0x06020007,0x1009000d,
0x00000000,0x1006004d,0x06030005,0x10050017,0x10020058,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x1006000f,0x02020002,0x10060010,0x0a040006,0x0a030007,0x03030003,
0x00000000,0x0a03000b,0x10060010,0x10060015,0x0a020013,0x10050016,0x0a030008,0x10090011,
0x00000000,0x01010001,0x0a040006,0x1007002a,0x0a010032,0x0a040007,0x0a030009,0x10060016,
0x00000000,0x0a02001a,0x0a040007,0x10050018,0x10020059,0x10050017,0x0a03000a,0x0a070008,
0x00000000,0x01010001,0x0a030007,0x02020002,0x10060011,0x10060015,0x10050016,0x10080019,
0x00000000,0x10030031,0x0a030008,0x10060016,0x0a020014,0x10050018,0x10050017,0x0a060009,
0x00000000,0x01010001,0x10060015,0x10060069,0x0a010033,0x10060016,0x10050018,0x1009000e,
0x00000000,0x1006004e,0x10060016,0x10050019,0x1002005a,0x10050019,0x10050019,0x10090017,
0x00000000,0x01010001,0x04010002,0x02020002,0x10060012,0x10040040,0x10030041,0x03030003,
0x100300e7,0x0a03000c,0x04010003,0x0a040007,0x0a020015,0x0a030008,0x10030042,0x04040004,
0x00000000,0x01010001,0x10040040,0x0a03000f,0x0a010034,0x10040041,0x10030043,0x100a000c,
0x100201e6,0x0a02001b,0x10040041,0x1009000d,0x1002005b,0x0a030009,0x10030044,0x10070042,
0x00000000,0x01010001,0x1007000b,0x02020002,0x10060013,0x0a040007,0x0a030008,0x10070038,
0x100103e5,0x10030032,0x1007000c,0x0a040008,0x0a020016,0x0a03000a,0x0a030009,0x100a000d,
0x00000000,0x01010001,0x0a040007,0x1004001e,0x0a010035,0x0a040008,0x0a03000a,0x1009000f,
0x00000000,0x1006004f,0x0a040008,0x1009000e,0x1002005c,0x0a03000b,0x0a03000b,0x0a070009,
0x00000000,0x01010001,0x1005001e,0x02020002,0x10060014,0x0a030009,0x0a02000a,0x03030003,
0x00000000,0x0a03000d,0x1005001f,0x100a000c,0x0a020017,0x1009000d,0x0a02000b,0x10080020,
0x00000000,0x01010001,0x0a030009,0x1007002b,0x0a010036,0x0a03000a,0x0a02000c,0x100a000d,
0x00000000,0x0a02001c,0x0a03000a,0x1009000f,0x1002005d,0x1009000e,0x0a02000d,0x0a06000b,
0x00000000,0x01010001,0x0a030008,0x02020002,0x10060015,0x100a000c,0x1009000d,0x10060077,
0x00000000,0x10030033,0x0a030009,0x100a000d,0x0a020018,0x1009000f,0x1009000e,0x0a06000a,
0x00000000,0x01010001,0x100a000c,0x1006006a,0x0a010037,0x100a000d,0x1009000f,0x10090010,
0x00000000,0x10060050,0x100a000d,0x10090010,0x1002005e,0x10090010,0x10090010,0x100d000e,
0x00000000,0x01010001,0x04010002,0x02020002,0x0a03000a,0x05020003,0x05010004,0x03030003,
0x100300e8,0x0a03000e,0x04010003,0x10050023,0x0a020019,0x10040024,0x05010005,0x04040004,
0x00000000,0x100300f5,0x05020003,0x0a030010,0x0a010038,0x05020004,0x05010006,0x1009000d,
0x100201e7,0x0a02001d,0x05020004,0x1008000e,0x1002005f,0x10040025,0x05010007,0x05050005,
0x00000000,0x01010001,0x0a040006,0x02020002,0x0a03000b,0x10050023,0x10040024,0x10050015,
0x100103e6,0x10030034,0x0a040007,0x10050024,0x0a02001a,0x10040026,0x10040025,0x0a070008,
0x00000000,0x100201f4,0x10050023,0x1004001f,0x0a010039,0x10050024,0x10040026,0x0a060009,
0x00000000,0x10060051,0x10050024,0x1008000f,0x10020060,0x10040027,0x10040027,0x10080025,
0x00000000,0x01010001,0x1004003d,0x02020002,0x0a03000c,0x0a040007,0x0a030008,0x03030003,
0x00000000,0x0a03000f,0x1004003e,0x1009000d,0x0a02001b,0x1008000e,0x0a030009,0x1007003f,
0x00000000,0x01010001,0x0a040007,0x1007002c,0x0a01003a,0x0a040008,0x0a03000a,0x1009000e,
0x00000000,0x0a02001e,0x0a040008,0x10080010,0x10020061,0x1008000f,0x0a03000b,0x0a070009,
0x00000000,0x01010001,0x0a030009,0x02020002,0x0a03000d,0x1009000d,0x1008000e,0x1008001a,
0x00000000,0x10030035,0x0a03000a,0x1009000e,0x0a02001c,0x10080010,0x1008000f,0x0a06000b,
0x00000000,0x01010001,0x1009000d,0x1006006b,0x0a01003b,0x1009000e,0x10080010,0x0a06000a,
0x00000000,0x10060052,0x1009000e,0x10080011,0x10020062,0x10080011,0x10080011,0x100c000f,
0x00000000,0x01010001,0x04010002,0x02020002,0x0a03000e,0x1003007f,0x10020080,0x03030003,
0x100300e9,0x0a030010,0x04010003,0x10050010,0x0a02001d,0x10040011,0x10020081,0x04040004,
0x00000000,0x01010001,0x1003007f,0x0a030011,0x0a01003c,0x10030080,0x10020082,0x0a070008,
0x100201e8,0x0a02001f,0x10030080,0x0a060009,0x10020063,0x10040012,0x10020083,0x10060081,
0x00000000,0x01010001,0x0a040007,0x02020002,0x0a03000f,0x10050010,0x10040011,0x10070039,
0x100103e7,0x10030036,0x0a040008,0x10050011,0x0a02001e,0x10040013,0x10040012,0x0a070009,
0x00000000,0x01010001,0x10050010,0x10040020,0x0a01003d,0x10050011,0x10040013,0x0a06000b,
0x00000000,0x10060053,0x10050011,0x0a06000a,0x10020064,0x10040014,0x10040014,0x10080012,
0x00000000,0x01010001,0x1003007c,0x02020002,0x0a030010,0x0a03000a,0x0a02000b,0x03030003,
0x00000000,0x0a030011,0x1003007d,0x0a070008,0x0a02001f,0x0a060009,0x0a02000c,0x1006007e,
0x00000000,0x01010001,0x0a03000a,0x1007002d,0x0a01003e,0x0a03000b,0x0a02000d,0x0a070009,
0x00000000,0x0a020020,0x0a03000b,0x0a06000b,0x10020065,0x0a06000a,0x0a02000e,0x0a06000c,
0x00000000,0x01010001,0x0a03000a,0x02020002,0x0a030011,0x0a070008,0x0a060009,0x10060078,
0x00000000,0x10030037,0x0a03000b,0x0a070009,0x0a020020,0x0a06000b,0x0a06000a,0x0a06000c,
0x00000000,0x01010001,0x0a070008,0x1006006c,0x0a01003f,0x0a070009,0x0a06000b,0x0a06000c,
0x00000000,0x10060054,0x0a070009,0x0a06000c,0x10020066,0x0a06000c,0x0a06000c,0x0a0a000a,
0x00000000,0x01010001,0x04010002,0x02020002,0x08010008,0x05020003,0x05010004,0x03030003,
0x100300ea,0x1006000f,0x04010003,0x06030004,0x10050016,0x06020005,0x05010005,0x04040004,
0x00000000,0x100300f6,0x05020003,0x10050012,0x10030028,0x05020004,0x05010006,0x07040005,
0x100201e9,0x10040019,0x05020004,0x07030006,0x10020067,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x08020004,0x100300fc,0x08010009,0x06030004,0x06020005,0x10050016,
0x100103e8,0x10030038,0x08020005,0x06030005,0x10050017,0x06020007,0x06020006,0x08050006,
0x00000000,0x100201f5,0x06030004,0x10040021,0x10030029,0x06030005,0x06020007,0x08040007,
0x00000000,0x10060055,0x06030005,0x07030007,0x10020068,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x10060010,0x02020002,0x0801000a,0x1007000b,0x1006000c,0x03030003,
0x00000000,0x10060010,0x10060011,0x07040005,0x10050018,0x07030006,0x1006000d,0x10090012,
0x00000000,0x01010001,0x1007000b,0x1007002e,0x1003002a,0x1007000c,0x1006000e,0x07040006,
0x00000000,0x1004001a,0x1007000c,0x07030008,0x10020069,0x07030007,0x1006000f,0x100a000d,
0x00000000,0x01010001,0x1006000c,0x02020002,0x0801000b,0x07040005,0x07030006,0x1008001b,
0x00000000,0x10030039,0x1006000d,0x07040006,0x10050019,0x07030008,0x07030007,0x1009000e,
0x00000000,0x01010001,0x07040005,0x1006006d,0x1003002b,0x07040006,0x07030008,0x08040008,
0x00000000,0x10060056,0x07040006,0x07030009,0x1002006a,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0801000c,0x10060013,0x10050014,0x03030003,
0x100300eb,0x10060011,0x04010003,0x1006000e,0x1005001a,0x1005000f,0x10050015,0x04040004,
0x00000000,0x01010001,0x10060013,0x10050013,0x1003002c,0x10060014,0x10050016,0x08050006,
0x100201ea,0x1004001b,0x10060014,0x08040007,0x1002006b,0x10050010,0x10050017,0x10090015,
0x00000000,0x01010001,0x08020005,0x02020002,0x0801000d,0x1006000e,0x1005000f,0x1007003a,
0x100103e9,0x1003003a,0x08020006,0x1006000f,0x1005001b,0x10050011,0x10050010,0x08050007,
0x00000000,0x01010001,0x1006000e,0x10040022,0x1003002d,0x1006000f,0x10050011,0x08040009,
0x00000000,0x10060057,0x1006000f,0x08040008,0x1002006c,0x10050012,0x10050012,0x10090010,
0x00000000,0x01010001,0x1005001f,0x02020002,0x0801000e,0x1005000f,0x10040010,0x03030003,
0x00000000,0x10060012,0x10050020,0x08050006,0x1005001c,0x08040007,0x10040011,0x10080021,
0x00000000,0x01010001,0x1005000f,0x1007002f,0x1003002e,0x10050010,0x10040012,0x08050007,
0x00000000,0x1004001c,0x10050010,0x08040009,0x1002006d,0x08040008,0x10040013,0x10080011,
0x00000000,0x01010001,0x1006000d,0x02020002,0x0801000f,0x08050006,0x08040007,0x10060079,
0x00000000,0x1003003b,0x1006000e,0x08050007,0x1005001d,0x08040009,0x08040008,0x1009000f,
0x00000000,0x01010001,0x08050006,0x1006006e,0x1003002f,0x08050007,0x08040009,0x0804000a,
0x00000000,0x10060058,0x08050007,0x0804000a,0x1002006e,0x0804000a,0x0804000a,0x08080008,
0x00000000,0x01010001,0x04010002,0x02020002,0x1006000f,0x05020003,0x05010004,0x03030003,
0x100300ec,0x10060013,0x04010003,0x10040042,0x1005001e,0x10030043,0x05010005,0x04040004,
0x00000000,0x100300f7,0x05020003,0x10050014,0x10030030,0x05020004,0x05010006,0x1006000f,
0x100201eb,0x1004001d,0x05020004,0x10050010,0x1002006f,0x10030044,0x05010007,0x05050005,
0x00000000,0x01010001,0x1007000b,0x02020002,0x10060010,0x10040042,0x10030043,0x10050017,
0x100103ea,0x1003003c,0x1007000c,0x10040043,0x1005001f,0x10030045,0x10030044,0x100a000d,
0x00000000,0x100201f6,0x10040042,0x10040023,0x10030031,0x10040043,0x10030045,0x1009000e,
0x00000000,0x10060059,0x10040043,0x10050011,0x10020070,0x10030046,0x10030046,0x10070044,
0x00000000,0x01010001,0x1004003e,0x02020002,0x10060011,0x1007000c,0x1006000d,0x03030003,
0x00000000,0x10060014,0x1004003f,0x1006000f,0x10050020,0x10050010,0x1006000e,0x10070040,
0x00000000,0x01010001,0x1007000c,0x10070030,0x10030032,0x1007000d,0x1006000f,0x10060010,
0x00000000,0x1004001e,0x1007000d,0x10050012,0x10020071,0x10050011,0x10060010,0x100a000e,
0x00000000,0x01010001,0x1006000e,0x02020002,0x10060012,0x1006000f,0x10050010,0x1008001c,
0x00000000,0x1003003d,0x1006000f,0x10060010,0x10050021,0x10050012,0x10050011,0x10090010,
0x00000000,0x01010001,0x1006000f,0x1006006f,0x10030033,0x10060010,0x10050012,0x1009000f,
0x00000000,0x1006005a,0x10060010,0x10050013,0x10020072,0x10050013,0x10050013,0x10090011,
0x00000000,0x01010001,0x04010002,0x02020002,0x10060013,0x10050022,0x10040023,0x03030003,
0x100300ed,0x10060015,0x04010003,0x10050011,0x10050022,0x10040012,0x10040024,0x04040004,
0x00000000,0x01010001,0x10050022,0x10050015,0x10030034,0x10050023,0x10040025,0x100a000d,
0x100201ec,0x1004001f,0x10050023,0x1009000e,0x10020073,0x10040013,0x10040026,0x10080024,
0x00000000,0x01010001,0x1007000c,0x02020002,0x10060014,0x10050011,0x10040012,0x1007003b,
0x100103eb,0x1003003e,0x1007000d,0x10050012,0x10050023,0x10040014,0x10040013,0x100a000e,
0x00000000,0x01010001,0x10050011,0x10040024,0x10030035,0x10050012,0x10040014,0x10090010,
0x00000000,0x1006005b,0x10050012,0x1009000f,0x10020074,0x10040015,0x10040015,0x10080013,
0x00000000,0x01010001,0x1003007d,0x02020002,0x10060015,0x10050010,0x10040011,0x03030003,
0x00000000,0x10060016,0x1003007e,0x100a000d,0x10050024,0x1009000e,0x10040012,0x1006007f,
0x00000000,0x01010001,0x10050010,0x10070031,0x10030036,0x10050011,0x10040013,0x100a000e,
0x00000000,0x10040020,0x10050011,0x10090010,0x10020075,0x1009000f,0x10040014,0x10080012,
0x00000000,0x01010001,0x1006000f,0x02020002,0x10060016,0x100a000d,0x1009000e,0x1006007a,
0x00000000,0x1003003f,0x10060010,0x100a000e,0x10050025,0x10090010,0x1009000f,0x10090011,
0x00000000,0x01010001,0x100a000d,0x10060070,0x10030037,0x100a000e,0x10090010,0x10090011,
0x00000000,0x1006005c,0x100a000e,0x10090011,0x10020076,0x10090011,0x10090011,0x100d000f,
0x00000000,0x01010001,0x04010002,0x02020002,0x10050011,0x05020003,0x05010004,0x03030003,
0x100300ee,0x10050012,0x04010003,0x06030004,0x10040019,0x06020005,0x05010005,0x04040004,
0x00000000,0x100300f8,0x05020003,0x10050016,0x10030038,0x05020004,0x05010006,0x10050024,
0x100201ed,0x10040021,0x05020004,0x10040025,0x10020077,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x1006000d,0x100300fd,0x10050012,0x06030004,0x06020005,0x10050018,
0x100103ec,0x10030040,0x1006000e,0x06030005,0x1004001a,0x06020007,0x06020006,0x1009000f,
0x00000000,0x100201f7,0x06030004,0x10040025,0x10030039,0x06030005,0x06020007,0x10080010,
0x00000000,0x1006005d,0x06030005,0x10040026,0x10020078,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x10060011,0x02020002,0x10050013,0x1006000e,0x1005000f,0x03030003,
0x00000000,0x10050013,0x10060012,0x10050024,0x1004001b,0x10040025,0x10050010,0x10090013,
0x00000000,0x01010001,0x1006000e,0x10070032,0x1003003a,0x1006000f,0x10050011,0x10050025,
0x00000000,0x10040022,0x1006000f,0x10040027,0x10020079,0x10040026,0x10050012,0x10090010,
0x00000000,0x01010001,0x1005000f,0x02020002,0x10050014,0x10050024,0x10040025,0x1008001d,
0x00000000,0x10030041,0x10050010,0x10050025,0x1004001c,0x10040027,0x10040026,0x10080011,
0x00000000,0x01010001,0x10050024,0x10060071,0x1003003b,0x10050025,0x10040027,0x10080011,
0x00000000,0x1006005e,0x10050025,0x10040028,0x1002007a,0x10040028,0x10040028,0x10080026,
0x00000000,0x01010001,0x04010002,0x02020002,0x10050015,0x10040041,0x10030042,0x03030003,
0x100300ef,0x10050014,0x04010003,0x1006000f,0x1004001d,0x10050010,0x10030043,0x04040004,
0x00000000,0x01010001,0x10040041,0x10050017,0x1003003c,0x10040042,0x10030044,0x1009000f,
0x100201ee,0x10040023,0x10040042,0x10080010,0x1002007b,0x10050011,0x10030045,0x10070043,
0x00000000,0x01010001,0x1006000e,0x02020002,0x10050016,0x1006000f,0x10050010,0x1007003c,
0x100103ed,0x10030042,0x1006000f,0x10060010,0x1004001e,0x10050012,0x10050011,0x10090010,
0x00000000,0x01010001,0x1006000f,0x10040026,0x1003003d,0x10060010,0x10050012,0x10080012,
0x00000000,0x1006005f,0x10060010,0x10080011,0x1002007c,0x10050013,0x10050013,0x10090011,
0x00000000,0x01010001,0x10050020,0x02020002,0x10050017,0x10050011,0x10040012,0x03030003,
0x00000000,0x10050015,0x10050021,0x1009000f,0x1004001f,0x10080010,0x10040013,0x10080022,
0x00000000,0x01010001,0x10050011,0x10070033,0x1003003e,0x10050012,0x10040014,0x10090010,
0x00000000,0x10040024,0x10050012,0x10080012,0x1002007d,0x10080011,0x10040015,0x10080013,
0x00000000,0x01010001,0x10050010,0x02020002,0x10050018,0x1009000f,0x10080010,0x1006007b,
0x00000000,0x10030043,0x10050011,0x10090010,0x10040020,0x10080012,0x10080011,0x10080012,
0x00000000,0x01010001,0x1009000f,0x10060072,0x1003003f,0x10090010,0x10080012,0x10080013,
0x00000000,0x10060060,0x10090010,0x10080013,0x1002007e,0x10080013,0x10080013,0x100c0011,
0x00000000,0x01010001,0x04010002,0x02020002,0x10050012,0x05020003,0x05010004,0x03030003,
0x100300f0,0x10050016,0x04010003,0x06030004,0x10040021,0x06020005,0x05010005,0x04040004,
0x00000000,0x100300f9,0x05020003,0x10050018,0x10030040,0x05020004,0x05010006,0x1009000e,
0x100201ef,0x10040025,0x05020004,0x1008000f,0x1002007f,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x1006000e,0x02020002,0x10050013,0x06030004,0x06020005,0x10050019,
0x100103ee,0x10030044,0x1006000f,0x06030005,0x10040022,0x06020007,0x06020006,0x10090010,
0x00000000,0x100201f8,0x06030004,0x10040027,0x10030041,0x06030005,0x06020007,0x10080011,
0x00000000,0x10060061,0x06030005,0x10080010,0x10020080,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x1004003f,0x02020002,0x10050014,0x1006000f,0x10050010,0x03030003,
0x00000000,0x10050017,0x10040040,0x1009000e,0x10040023,0x1008000f,0x10050011,0x10070041,
0x00000000,0x01010001,0x1006000f,0x10070034,0x10030042,0x10060010,0x10050012,0x1009000f,
0x00000000,0x10040026,0x10060010,0x10080011,0x10020081,0x10080010,0x10050013,0x10090011,
0x00000000,0x01010001,0x10050011,0x02020002,0x10050015,0x1009000e,0x1008000f,0x1008001e,
0x00000000,0x10030045,0x10050012,0x1009000f,0x10040024,0x10080011,0x10080010,0x10080013,
0x00000000,0x01010001,0x1009000e,0x10060073,0x10030043,0x1009000f,0x10080011,0x10080012,
0x00000000,0x10060062,0x1009000f,0x10080012,0x10020082,0x10080012,0x10080012,0x100c0010,
0x00000000,0x01010001,0x04010002,0x02020002,0x10050016,0x10030080,0x10020081,0x03030003,
0x100300f1,0x10050018,0x04010003,0x10050012,0x10040025,0x10040013,0x10020082,0x04040004,
0x00000000,0x01010001,0x10030080,0x10050019,0x10030044,0x10030081,0x10020083,0x10090010,
0x100201f0,0x10040027,0x10030081,0x10080011,0x10020083,0x10040014,0x10020084,0x10060082,
0x00000000,0x01010001,0x1006000f,0x02020002,0x10050017,0x10050012,0x10040013,0x1007003d,
0x100103ef,0x10030046,0x10060010,0x10050013,0x10040026,0x10040015,0x10040014,0x10090011,
0x00000000,0x01010001,0x10050012,0x10040028,0x10030045,0x10050013,0x10040015,0x10080013,
0x00000000,0x10060063,0x10050013,0x10080012,0x10020084,0x10040016,0x10040016,0x10080014,
0x00000000,0x01010001,0x1003007e,0x02020002,0x10050018,0x10050012,0x10040013,0x03030003,
0x00000000,0x10050019,0x1003007f,0x10090010,0x10040027,0x10080011,0x10040014,0x10060080,
0x00000000,0x01010001,0x10050012,0x10070035,0x10030046,0x10050013,0x10040015,0x10090011,
0x00000000,0x10040028,0x10050013,0x10080013,0x10020085,0x10080012,0x10040016,0x10080014,
0x00000000,0x01010001,0x10050012,0x02020002,0x10050019,0x10090010,0x10080011,0x1006007c,
0x00000000,0x10030047,0x10050013,0x10090011,0x10040028,0x10080013,0x10080012,0x10080014,
0x00000000,0x01010001,0x10090010,0x10060074,0x10030047,0x10090011,0x10080013,0x10080014,
0x00000000,0x10060064,0x10090011,0x10080014,0x10020086,0x10080014,0x10080014,0x100c0012,
0x00000000,0x01010001,0x04010002,0x02020002,0x08010008,0x05020003,0x05010004,0x03030003,
0x100300f2,0x09020009,0x04010003,0x06030004,0x09010010,0x06020005,0x05010005,0x04040004,
0x00000000,0x100300fa,0x05020003,0x10050012,0x10030028,0x05020004,0x05010006,0x07040005,
0x100201f1,0x10040019,0x05020004,0x07030006,0x10060045,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x08020004,0x100300fe,0x08010009,0x06030004,0x06020005,0x10090010,
0x100103f0,0x10070026,0x08020005,0x06030005,0x09010011,0x06020007,0x06020006,0x08050006,
0x00000000,0x100201f9,0x06030004,0x10080017,0x10030029,0x06030005,0x06020007,0x08040007,
0x00000000,0x10060065,0x06030005,0x07030007,0x10060046,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x10060012,0x02020002,0x0801000a,0x09030005,0x09020006,0x03030003,
0x00000000,0x0902000a,0x10060013,0x07040005,0x09010012,0x07030006,0x09020007,0x10090014,
0x00000000,0x01010001,0x09030005,0x10070036,0x1003002a,0x09030006,0x09020008,0x07040006,
0x00000000,0x1004001a,0x09030006,0x07030008,0x10060047,0x07030007,0x09020009,0x09060007,
0x00000000,0x01010001,0x09020006,0x02020002,0x0801000b,0x07040005,0x07030006,0x1008001f,
0x00000000,0x10070027,0x09020007,0x07040006,0x09010013,0x07030008,0x07030007,0x09050008,
0x00000000,0x01010001,0x07040005,0x10060075,0x1003002b,0x07040006,0x07030008,0x08040008,
0x00000000,0x10060066,0x07040006,0x07030009,0x10060048,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0801000c,0x10060014,0x10050015,0x03030003,
0x100300f3,0x0902000b,0x04010003,0x1006000e,0x09010014,0x1005000f,0x10050016,0x04040004,
0x00000000,0x01010001,0x10060014,0x10050013,0x1003002c,0x10060015,0x10050017,0x08050006,
0x100201f2,0x1004001b,0x10060015,0x08040007,0x10060049,0x10050010,0x10050018,0x10090016,
0x00000000,0x01010001,0x08020005,0x02020002,0x0801000d,0x1006000e,0x1005000f,0x1007003e,
0x100103f1,0x10070028,0x08020006,0x1006000f,0x09010015,0x10050011,0x10050010,0x08050007,
0x00000000,0x01010001,0x1006000e,0x10080018,0x1003002d,0x1006000f,0x10050011,0x08040009,
0x00000000,0x10060067,0x1006000f,0x08040008,0x1006004a,0x10050012,0x10050012,0x10090010,
0x00000000,0x01010001,0x10050021,0x02020002,0x0801000e,0x1005000f,0x10040010,0x03030003,
0x00000000,0x0902000c,0x10050022,0x08050006,0x09010016,0x08040007,0x10040011,0x10080023,
0x00000000,0x01010001,0x1005000f,0x10070037,0x1003002e,0x10050010,0x10040012,0x08050007,
0x00000000,0x1004001c,0x10050010,0x08040009,0x1006004b,0x08040008,0x10040013,0x10080011,
0x00000000,0x01010001,0x09020007,0x02020002,0x0801000f,0x08050006,0x08040007,0x1006007d,
0x00000000,0x10070029,0x09020008,0x08050007,0x09010017,0x08040009,0x08040008,0x09050009,
0x00000000,0x01010001,0x08050006,0x10060076,0x1003002f,0x08050007,0x08040009,0x0804000a,
0x00000000,0x10060068,0x08050007,0x0804000a,0x1006004c,0x0804000a,0x0804000a,0x08080008,
0x00000000,0x01010001,0x04010002,0x02020002,0x09020009,0x05020003,0x05010004,0x03030003,
0x100300f4,0x0902000d,0x04010003,0x10060015,0x09010018,0x10050016,0x05010005,0x04040004,
0x00000000,0x100300fb,0x05020003,0x10050014,0x10030030,0x05020004,0x05010006,0x100a000c,
0x100201f3,0x1004001d,0x05020004,0x1009000d,0x1006004d,0x10050017,0x05010007,0x05050005,
0x00000000,0x01010001,0x09030005,0x02020002,0x0902000a,0x10060015,0x10050016,0x10090011,
0x100103f2,0x1007002a,0x09030006,0x10060016,0x09010019,0x10050018,0x10050017,0x09060007,
0x00000000,0x100201fa,0x10060015,0x10080019,0x10030031,0x10060016,0x10050018,0x09050008,
0x00000000,0x10060069,0x10060016,0x1009000e,0x1006004e,0x10050019,0x10050019,0x10090017,
0x00000000,0x01010001,0x10040040,0x02020002,0x0902000b,0x09030006,0x09020007,0x03030003,
0x00000000,0x0902000e,0x10040041,0x100a000c,0x0901001a,0x1009000d,0x09020008,0x10070042,
0x00000000,0x01010001,0x09030006,0x10070038,0x10030032,0x09030007,0x09020009,0x100a000d,
0x00000000,0x1004001e,0x09030007,0x1009000f,0x1006004f,0x1009000e,0x0902000a,0x09060008,
0x00000000,0x01010001,0x09020008,0x02020002,0x0902000c,0x100a000c,0x1009000d,0x10080020,
0x00000000,0x1007002b,0x09020009,0x100a000d,0x0901001b,0x1009000f,0x1009000e,0x0905000a,
0x00000000,0x01010001,0x100a000c,0x10060077,0x10030033,0x100a000d,0x1009000f,0x09050009,
0x00000000,0x1006006a,0x100a000d,0x10090010,0x10060050,0x10090010,0x10090010,0x100d000e,
0x00000000,0x01010001,0x04010002,0x02020002,0x0902000d,0x10050023,0x10040024,0x03030003,
0x100300f5,0x0902000f,0x04010003,0x1009000d,0x0901001c,0x1008000e,0x10040025,0x04040004,
0x00000000,0x01010001,0x10050023,0x10050015,0x10030034,0x10050024,0x10040026,0x09060007,
0x100201f4,0x1004001f,0x10050024,0x09050008,0x10060051,0x1008000f,0x10040027,0x10080025,
0x00000000,0x01010001,0x09030006,0x02020002,0x0902000e,0x1009000d,0x1008000e,0x1007003f,
0x100103f3,0x1007002c,0x09030007,0x1009000e,0x0901001d,0x10080010,0x1008000f,0x09060008,
0x00000000,0x01010001,0x1009000d,0x1008001a,0x10030035,0x1009000e,0x10080010,0x0905000a,
0x00000000,0x1006006b,0x1009000e,0x09050009,0x10060052,0x10080011,0x10080011,0x100c000f,
0x00000000,0x01010001,0x1003007f,0x02020002,0x0902000f,0x10050010,0x10040011,0x03030003,
0x00000000,0x09020010,0x10030080,0x09060007,0x0901001e,0x09050008,0x10040012,0x10060081,
0x00000000,0x01010001,0x10050010,0x10070039,0x10030036,0x10050011,0x10040013,0x09060008,
0x00000000,0x10040020,0x10050011,0x0905000a,0x10060053,0x09050009,0x10040014,0x10080012,
0x00000000,0x01010001,0x09020009,0x02020002,0x09020010,0x09060007,0x09050008,0x1006007e,
0x00000000,0x1007002d,0x0902000a,0x09060008,0x0901001f,0x0905000a,0x09050009,0x0905000b,
0x00000000,0x01010001,0x09060007,0x10060078,0x10030037,0x09060008,0x0905000a,0x0905000b,
0x00000000,0x1006006c,0x09060008,0x0905000b,0x10060054,0x0905000b,0x0905000b,0x09090009,
0x00000000,0x01010001,0x04010002,0x02020002,0x1006000f,0x05020003,0x05010004,0x03030003,
0x100300f6,0x10050012,0x04010003,0x06030004,0x10040019,0x06020005,0x05010005,0x04040004,
0x00000000,0x100300fc,0x05020003,0x10050016,0x10030038,0x05020004,0x05010006,0x07040005,
0x100201f5,0x10040021,0x05020004,0x07030006,0x10060055,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x1007000b,0x100300ff,0x10060010,0x06030004,0x06020005,0x10090012,
0x100103f4,0x1007002e,0x1007000c,0x06030005,0x1004001a,0x06020007,0x06020006,0x100a000d,
0x00000000,0x100201fb,0x06030004,0x1008001b,0x10030039,0x06030005,0x06020007,0x1009000e,
0x00000000,0x1006006d,0x06030005,0x07030007,0x10060056,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x10060013,0x02020002,0x10060011,0x1006000e,0x1005000f,0x03030003,
0x00000000,0x10050013,0x10060014,0x07040005,0x1004001b,0x07030006,0x10050010,0x10090015,
0x00000000,0x01010001,0x1006000e,0x1007003a,0x1003003a,0x1006000f,0x10050011,0x07040006,
0x00000000,0x10040022,0x1006000f,0x07030008,0x10060057,0x07030007,0x10050012,0x10090010,
0x00000000,0x01010001,0x1005000f,0x02020002,0x10060012,0x07040005,0x07030006,0x10080021,
0x00000000,0x1007002f,0x10050010,0x07040006,0x1004001c,0x07030008,0x07030007,0x10080011,
0x00000000,0x01010001,0x07040005,0x10060079,0x1003003b,0x07040006,0x07030008,0x1009000f,
0x00000000,0x1006006e,0x07040006,0x07030009,0x10060058,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x10060013,0x10040042,0x10030043,0x03030003,
0x100300f7,0x10050014,0x04010003,0x1006000f,0x1004001d,0x10050010,0x10030044,0x04040004,
0x00000000,0x01010001,0x10040042,0x10050017,0x1003003c,0x10040043,0x10030045,0x100a000d,
0x100201f6,0x10040023,0x10040043,0x1009000e,0x10060059,0x10050011,0x10030046,0x10070044,
0x00000000,0x01010001,0x1007000c,0x02020002,0x10060014,0x1006000f,0x10050010,0x10070040,
0x100103f5,0x10070030,0x1007000d,0x10060010,0x1004001e,0x10050012,0x10050011,0x100a000e,
0x00000000,0x01010001,0x1006000f,0x1008001c,0x1003003d,0x10060010,0x10050012,0x10090010,
0x00000000,0x1006006f,0x10060010,0x1009000f,0x1006005a,0x10050013,0x10050013,0x10090011,
0x00000000,0x01010001,0x10050022,0x02020002,0x10060015,0x10050011,0x10040012,0x03030003,
0x00000000,0x10050015,0x10050023,0x100a000d,0x1004001f,0x1009000e,0x10040013,0x10080024,
0x00000000,0x01010001,0x10050011,0x1007003b,0x1003003e,0x10050012,0x10040014,0x100a000e,
0x00000000,0x10040024,0x10050012,0x10090010,0x1006005b,0x1009000f,0x10040015,0x10080013,
0x00000000,0x01010001,0x10050010,0x02020002,0x10060016,0x100a000d,0x1009000e,0x1006007f,
0x00000000,0x10070031,0x10050011,0x100a000e,0x10040020,0x10090010,0x1009000f,0x10080012,
0x00000000,0x01010001,0x100a000d,0x1006007a,0x1003003f,0x100a000e,0x10090010,0x10090011,
0x00000000,0x10060070,0x100a000e,0x10090011,0x1006005c,0x10090011,0x10090011,0x100d000f,
0x00000000,0x01010001,0x04010002,0x02020002,0x10050012,0x05020003,0x05010004,0x03030003,
0x100300f8,0x10050016,0x04010003,0x10050024,0x10040021,0x10040025,0x05010005,0x04040004,
0x00000000,0x100300fd,0x05020003,0x10050018,0x10030040,0x05020004,0x05010006,0x1009000f,
0x100201f7,0x10040025,0x05020004,0x10080010,0x1006005d,0x10040026,0x05010007,0x05050005,
0x00000000,0x01010001,0x1006000e,0x02020002,0x10050013,0x10050024,0x10040025,0x10090013,
0x100103f6,0x10070032,0x1006000f,0x10050025,0x10040022,0x10040027,0x10040026,0x10090010,
0x00000000,0x100201fc,0x10050024,0x1008001d,0x10030041,0x10050025,0x10040027,0x10080011,
0x00000000,0x10060071,0x10050025,0x10080011,0x1006005e,0x10040028,0x10040028,0x10080026,
0x00000000,0x01010001,0x10040041,0x02020002,0x10050014,0x1006000f,0x10050010,0x03030003,
0x00000000,0x10050017,0x10040042,0x1009000f,0x10040023,0x10080010,0x10050011,0x10070043,
0x00000000,0x01010001,0x1006000f,0x1007003c,0x10030042,0x10060010,0x10050012,0x10090010,
0x00000000,0x10040026,0x10060010,0x10080012,0x1006005f,0x10080011,0x10050013,0x10090011,
0x00000000,0x01010001,0x10050011,0x02020002,0x10050015,0x1009000f,0x10080010,0x10080022,
0x00000000,0x10070033,0x10050012,0x10090010,0x10040024,0x10080012,0x10080011,0x10080013,
0x00000000,0x01010001,0x1009000f,0x1006007b,0x10030043,0x10090010,0x10080012,0x10080012,
0x00000000,0x10060072,0x10090010,0x10080013,0x10060060,0x10080013,0x10080013,0x100c0011,
0x00000000,0x01010001,0x04010002,0x02020002,0x10050016,0x10030081,0x10020082,0x03030003,
0x100300f9,0x10050018,0x04010003,0x1009000e,0x10040025,0x1008000f,0x10020083,0x04040004,
0x00000000,0x01010001,0x10030081,0x10050019,0x10030044,0x10030082,0x10020084,0x10090010,
0x100201f8,0x10040027,0x10030082,0x10080011,0x10060061,0x10080010,0x10020085,0x10060083,
0x00000000,0x01010001,0x1006000f,0x02020002,0x10050017,0x1009000e,0x1008000f,0x10070041,
0x100103f7,0x10070034,0x10060010,0x1009000f,0x10040026,0x10080011,0x10080010,0x10090011,
0x00000000,0x01010001,0x1009000e,0x1008001e,0x10030045,0x1009000f,0x10080011,0x10080013,
0x00000000,0x10060073,0x1009000f,0x10080012,0x10060062,0x10080012,0x10080012,0x100c0010,
0x00000000,0x01010001,0x10030080,0x02020002,0x10050018,0x10050012,0x10040013,0x03030003,
0x00000000,0x10050019,0x10030081,0x10090010,0x10040027,0x10080011,0x10040014,0x10060082,
0x00000000,0x01010001,0x10050012,0x1007003d,0x10030046,0x10050013,0x10040015,0x10090011,
0x00000000,0x10040028,0x10050013,0x10080013,0x10060063,0x10080012,0x10040016,0x10080014,
0x00000000,0x01010001,0x10050012,0x02020002,0x10050019,0x10090010,0x10080011,0x10060080,
0x00000000,0x10070035,0x10050013,0x10090011,0x10040028,0x10080013,0x10080012,0x10080014,
0x00000000,0x01010001,0x10090010,0x1006007c,0x10030047,0x10090011,0x10080013,0x10080014,
0x00000000,0x10060074,0x10090011,0x10080014,0x10060064,0x10080014,0x10080014,0x100c0012,
0x00000000,0x01010001,0x04010002,0x02020002,0x08010008,0x05020003,0x05010004,0x03030003,
0x100300fa,0x10050012,0x04010003,0x06030004,0x10040019,0x06020005,0x05010005,0x04040004,
0x00000000,0x100300fe,0x05020003,0x10090010,0x10070026,0x05020004,0x05010006,0x07040005,
0x100201f9,0x10080017,0x05020004,0x07030006,0x10060065,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x08020004,0x10030100,0x08010009,0x06030004,0x06020005,0x10090014,
0x100103f8,0x10070036,0x08020005,0x06030005,0x1004001a,0x06020007,0x06020006,0x08050006,
0x00000000,0x100201fd,0x06030004,0x1008001f,0x10070027,0x06030005,0x06020007,0x08040007,
0x00000000,0x10060075,0x06030005,0x07030007,0x10060066,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x10060014,0x02020002,0x0801000a,0x1006000e,0x1005000f,0x03030003,
0x00000000,0x10050013,0x10060015,0x07040005,0x1004001b,0x07030006,0x10050010,0x10090016,
0x00000000,0x01010001,0x1006000e,0x1007003e,0x10070028,0x1006000f,0x10050011,0x07040006,
0x00000000,0x10080018,0x1006000f,0x07030008,0x10060067,0x07030007,0x10050012,0x10090010,
0x00000000,0x01010001,0x1005000f,0x02020002,0x0801000b,0x07040005,0x07030006,0x10080023,
0x00000000,0x10070037,0x10050010,0x07040006,0x1004001c,0x07030008,0x07030007,0x10080011,
0x00000000,0x01010001,0x07040005,0x1006007d,0x10070029,0x07040006,0x07030008,0x08040008,
0x00000000,0x10060076,0x07040006,0x07030009,0x10060068,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x0801000c,0x10060015,0x10050016,0x03030003,
0x100300fb,0x10050014,0x04010003,0x100a000c,0x1004001d,0x1009000d,0x10050017,0x04040004,
0x00000000,0x01010001,0x10060015,0x10090011,0x1007002a,0x10060016,0x10050018,0x08050006,
0x100201fa,0x10080019,0x10060016,0x08040007,0x10060069,0x1009000e,0x10050019,0x10090017,
0x00000000,0x01010001,0x08020005,0x02020002,0x0801000d,0x100a000c,0x1009000d,0x10070042,
0x100103f9,0x10070038,0x08020006,0x100a000d,0x1004001e,0x1009000f,0x1009000e,0x08050007,
0x00000000,0x01010001,0x100a000c,0x10080020,0x1007002b,0x100a000d,0x1009000f,0x08040009,
0x00000000,0x10060077,0x100a000d,0x08040008,0x1006006a,0x10090010,0x10090010,0x100d000e,
0x00000000,0x01010001,0x10050023,0x02020002,0x0801000e,0x1009000d,0x1008000e,0x03030003,
0x00000000,0x10050015,0x10050024,0x08050006,0x1004001f,0x08040007,0x1008000f,0x10080025,
0x00000000,0x01010001,0x1009000d,0x1007003f,0x1007002c,0x1009000e,0x10080010,0x08050007,
0x00000000,0x1008001a,0x1009000e,0x08040009,0x1006006b,0x08040008,0x10080011,0x100c000f,
0x00000000,0x01010001,0x10050010,0x02020002,0x0801000f,0x08050006,0x08040007,0x10060081,
0x00000000,0x10070039,0x10050011,0x08050007,0x10040020,0x08040009,0x08040008,0x10080012,
0x00000000,0x01010001,0x08050006,0x1006007e,0x1007002d,0x08050007,0x08040009,0x0804000a,
0x00000000,0x10060078,0x08050007,0x0804000a,0x1006006c,0x0804000a,0x0804000a,0x08080008,
0x00000000,0x01010001,0x04010002,0x02020002,0x10050012,0x05020003,0x05010004,0x03030003,
0x100300fc,0x10050016,0x04010003,0x10040043,0x10040021,0x10030044,0x05010005,0x04040004,
0x00000000,0x100300ff,0x05020003,0x10090012,0x1007002e,0x05020004,0x05010006,0x100a000d,
0x100201fb,0x1008001b,0x05020004,0x1009000e,0x1006006d,0x10030045,0x05010007,0x05050005,
0x00000000,0x01010001,0x1006000e,0x02020002,0x10050013,0x10040043,0x10030044,0x10090015,
0x100103fa,0x1007003a,0x1006000f,0x10040044,0x10040022,0x10030046,0x10030045,0x10090010,
0x00000000,0x100201fe,0x10040043,0x10080021,0x1007002f,0x10040044,0x10030046,0x10080011,
0x00000000,0x10060079,0x10040044,0x1009000f,0x1006006e,0x10030047,0x10030047,0x10070045,
0x00000000,0x01010001,0x10040042,0x02020002,0x10050014,0x1006000f,0x10050010,0x03030003,
0x00000000,0x10050017,0x10040043,0x100a000d,0x10040023,0x1009000e,0x10050011,0x10070044,
0x00000000,0x01010001,0x1006000f,0x10070040,0x10070030,0x10060010,0x10050012,0x100a000e,
0x00000000,0x1008001c,0x10060010,0x10090010,0x1006006f,0x1009000f,0x10050013,0x10090011,
0x00000000,0x01010001,0x10050011,0x02020002,0x10050015,0x100a000d,0x1009000e,0x10080024,
0x00000000,0x1007003b,0x10050012,0x100a000e,0x10040024,0x10090010,0x1009000f,0x10080013,
0x00000000,0x01010001,0x100a000d,0x1006007f,0x10070031,0x100a000e,0x10090010,0x10080012,
0x00000000,0x1006007a,0x100a000e,0x10090011,0x10060070,0x10090011,0x10090011,0x100d000f,
0x00000000,0x01010001,0x04010002,0x02020002,0x10050016,0x10050024,0x10040025,0x03030003,
0x100300fd,0x10050018,0x04010003,0x1009000f,0x10040025,0x10080010,0x10040026,0x04040004,
0x00000000,0x01010001,0x10050024,0x10090013,0x10070032,0x10050025,0x10040027,0x10090010,
0x100201fc,0x1008001d,0x10050025,0x10080011,0x10060071,0x10080011,0x10040028,0x10080026,
0x00000000,0x01010001,0x1006000f,0x02020002,0x10050017,0x1009000f,0x10080010,0x10070043,
0x100103fb,0x1007003c,0x10060010,0x10090010,0x10040026,0x10080012,0x10080011,0x10090011,
0x00000000,0x01010001,0x1009000f,0x10080022,0x10070033,0x10090010,0x10080012,0x10080013,
0x00000000,0x1006007b,0x10090010,0x10080012,0x10060072,0x10080013,0x10080013,0x100c0011,
0x00000000,0x01010001,0x10030081,0x02020002,0x10050018,0x1009000e,0x1008000f,0x03030003,
0x00000000,0x10050019,0x10030082,0x10090010,0x10040027,0x10080011,0x10080010,0x10060083,
0x00000000,0x01010001,0x1009000e,0x10070041,0x10070034,0x1009000f,0x10080011,0x10090011,
0x00000000,0x1008001e,0x1009000f,0x10080013,0x10060073,0x10080012,0x10080012,0x100c0010,
0x00000000,0x01010001,0x10050012,0x02020002,0x10050019,0x10090010,0x10080011,0x10060082,
0x00000000,0x1007003d,0x10050013,0x10090011,0x10040028,0x10080013,0x10080012,0x10080014,
0x00000000,0x01010001,0x10090010,0x10060080,0x10070035,0x10090011,0x10080013,0x10080014,
0x00000000,0x1006007c,0x10090011,0x10080014,0x10060074,0x10080014,0x10080014,0x100c0012,
0x00000000,0x01010001,0x04010002,0x02020002,0x10050012,0x05020003,0x05010004,0x03030003,
0x100300fe,0x10090010,0x04010003,0x06030004,0x10080017,0x06020005,0x05010005,0x04040004,
0x00000000,0x10030100,0x05020003,0x10090014,0x10070036,0x05020004,0x05010006,0x07040005,
0x100201fd,0x1008001f,0x05020004,0x07030006,0x10060075,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x1006000e,0x10030101,0x10050013,0x06030004,0x06020005,0x10090016,
0x100103fc,0x1007003e,0x1006000f,0x06030005,0x10080018,0x06020007,0x06020006,0x10090010,
0x00000000,0x100201ff,0x06030004,0x10080023,0x10070037,0x06030005,0x06020007,0x10080011,
0x00000000,0x1006007d,0x06030005,0x07030007,0x10060076,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x10060015,0x02020002,0x10050014,0x100a000c,0x1009000d,0x03030003,
0x00000000,0x10090011,0x10060016,0x07040005,0x10080019,0x07030006,0x1009000e,0x10090017,
0x00000000,0x01010001,0x100a000c,0x10070042,0x10070038,0x100a000d,0x1009000f,0x07040006,
0x00000000,0x10080020,0x100a000d,0x07030008,0x10060077,0x07030007,0x10090010,0x100d000e,
0x00000000,0x01010001,0x1009000d,0x02020002,0x10050015,0x07040005,0x07030006,0x10080025,
0x00000000,0x1007003f,0x1009000e,0x07040006,0x1008001a,0x07030008,0x07030007,0x100c000f,
0x00000000,0x01010001,0x07040005,0x10060081,0x10070039,0x07040006,0x07030008,0x10080012,
0x00000000,0x1006007e,0x07040006,0x07030009,0x10060078,0x07030009,0x07030009,0x07070007,
0x00000000,0x01010001,0x04010002,0x02020002,0x10050016,0x10040043,0x10030044,0x03030003,
0x100300ff,0x10090012,0x04010003,0x100a000d,0x1008001b,0x1009000e,0x10030045,0x04040004,
0x00000000,0x01010001,0x10040043,0x10090015,0x1007003a,0x10040044,0x10030046,0x10090010,
0x100201fe,0x10080021,0x10040044,0x10080011,0x10060079,0x1009000f,0x10030047,0x10070045,
0x00000000,0x01010001,0x1006000f,0x02020002,0x10050017,0x100a000d,0x1009000e,0x10070044,
0x100103fd,0x10070040,0x10060010,0x100a000e,0x1008001c,0x10090010,0x1009000f,0x10090011,
0x00000000,0x01010001,0x100a000d,0x10080024,0x1007003b,0x100a000e,0x10090010,0x10080013,
0x00000000,0x1006007f,0x100a000e,0x10080012,0x1006007a,0x10090011,0x10090011,0x100d000f,
0x00000000,0x01010001,0x10050024,0x02020002,0x10050018,0x1009000f,0x10080010,0x03030003,
0x00000000,0x10090013,0x10050025,0x10090010,0x1008001d,0x10080011,0x10080011,0x10080026,
0x00000000,0x01010001,0x1009000f,0x10070043,0x1007003c,0x10090010,0x10080012,0x10090011,
0x00000000,0x10080022,0x10090010,0x10080013,0x1006007b,0x10080012,0x10080013,0x100c0011,
0x00000000,0x01010001,0x1009000e,0x02020002,0x10050019,0x10090010,0x10080011,0x10060083,
0x00000000,0x10070041,0x1009000f,0x10090011,0x1008001e,0x10080013,0x10080012,0x100c0010,
0x00000000,0x01010001,0x10090010,0x10060082,0x1007003d,0x10090011,0x10080013,0x10080014,
0x00000000,0x10060080,0x10090011,0x10080014,0x1006007c,0x10080014,0x10080014,0x100c0012,
0x00000000,0x01010001,0x04010002,0x02020002,0x10090010,0x05020003,0x05010004,0x03030003,
0x10030100,0x10090014,0x04010003,0x06030004,0x1008001f,0x06020005,0x05010005,0x04040004,
0x00000000,0x10030101,0x05020003,0x10090016,0x1007003e,0x05020004,0x05010006,0x10090010,
0x100201ff,0x10080023,0x05020004,0x10080011,0x1006007d,0x06020006,0x05010007,0x05050005,
0x00000000,0x01010001,0x100a000c,0x02020002,0x10090011,0x06030004,0x06020005,0x10090017,
0x100103fe,0x10070042,0x100a000d,0x06030005,0x10080020,0x06020007,0x06020006,0x100d000e,
0x00000000,0x10020200,0x06030004,0x10080025,0x1007003f,0x06030005,0x06020007,0x100c000f,
0x00000000,0x10060081,0x06030005,0x10080012,0x1006007e,0x06020008,0x06020008,0x06060006,
0x00000000,0x01010001,0x10040043,0x02020002,0x10090012,0x100a000d,0x1009000e,0x03030003,
0x00000000,0x10090015,0x10040044,0x10090010,0x10080021,0x10080011,0x1009000f,0x10070045,
0x00000000,0x01010001,0x100a000d,0x10070044,0x10070040,0x100a000e,0x10090010,0x10090011,
0x00000000,0x10080024,0x100a000e,0x10080013,0x1006007f,0x10080012,0x10090011,0x100d000f,
0x00000000,0x01010001,0x1009000f,0x02020002,0x10090013,0x10090010,0x10080011,0x10080026,
0x00000000,0x10070043,0x10090010,0x10090011,0x10080022,0x10080013,0x10080012,0x100c0011,
0x00000000,0x01010001,0x10090010,0x10060083,0x10070041,0x10090011,0x10080013,0x100c0010,
0x00000000,0x10060082,0x10090011,0x10080014,0x10060080,0x10080014,0x10080014,0x100c0012,
0x00000000,0x01010001,0x04010002,0x02020002,0x10090014,0x10030082,0x10020083,0x03030003,
0x10030101,0x10090016,0x04010003,0x10090010,0x10080023,0x10080011,0x10020084,0x04040004,
0x00000000,0x01010001,0x10030082,0x10090017,0x10070042,0x10030083,0x10020085,0x100d000e,
0x10020200,0x10080025,0x10030083,0x100c000f,0x10060081,0x10080012,0x10020086,0x10060084,
0x00000000,0x01010001,0x100a000d,0x02020002,0x10090015,0x10090010,0x10080011,0x10070045,
0x100103ff,0x10070044,0x100a000e,0x10090011,0x10080024,0x10080013,0x10080012,0x100d000f,
0x00000000,0x01010001,0x10090010,0x10080026,0x10070043,0x10090011,0x10080013,0x100c0011,
0x00000000,0x10060083,0x10090011,0x100c0010,0x10060082,0x10080014,0x10080014,0x100c0012,
0x00000000,0x01010001,0x10030082,0x02020002,0x10090016,0x10090010,0x10080011,0x03030003,
0x00000000,0x10090017,0x10030083,0x100d000e,0x10080025,0x100c000f,0x10080012,0x10060084,
0x00000000,0x01010001,0x10090010,0x10070045,0x10070044,0x10090011,0x10080013,0x100d000f,
0x00000000,0x10080026,0x10090011,0x100c0011,0x10060083,0x100c0010,0x10080014,0x100c0012,
0x00000000,0x01010001,0x10090010,0x02020002,0x10090017,0x100d000e,0x100c000f,0x10060084,
0x00000000,0x10070045,0x10090011,0x100d000f,0x10080026,0x100c0011,0x100c0010,0x100c0012,
0x00000000,0x01010001,0x100d000e,0x10060084,0x10070045,0x100d000f,0x100c0011,0x100c0012,
0x00000000,0x10060084,0x100d000f,0x100c0012,0x10060084,0x100c0012,0x100c0012,0x10100010
};

const uint16_t elias_delta::EliasDeltaPrefixSum8bit[]={
0x0000,0x0111,0x0412,0x0111,0x0818,0x0111,0x0514,0x0111,
0x0000,0x0111,0x0413,0x0111,0x0000,0x0111,0x0515,0x0111,
0x0000,0x0111,0x0412,0x0111,0x0000,0x0111,0x0516,0x0111,
0x0000,0x0111,0x0413,0x0111,0x0000,0x0111,0x0517,0x0111,
0x0000,0x0111,0x0412,0x0111,0x0819,0x0111,0x0514,0x0111,
0x0000,0x0111,0x0413,0x0111,0x0000,0x0111,0x0515,0x0111,
0x0000,0x0111,0x0412,0x0111,0x0000,0x0111,0x0516,0x0111,
0x0000,0x0111,0x0413,0x0111,0x0000,0x0111,0x0517,0x0111,
0x0000,0x0111,0x0412,0x0111,0x081a,0x0111,0x0514,0x0111,
0x0000,0x0111,0x0413,0x0111,0x0000,0x0111,0x0515,0x0111,
0x0000,0x0111,0x0412,0x0111,0x0000,0x0111,0x0516,0x0111,
0x0000,0x0111,0x0413,0x0111,0x0000,0x0111,0x0517,0x0111,
0x0000,0x0111,0x0412,0x0111,0x081b,0x0111,0x0514,0x0111,
0x0000,0x0111,0x0413,0x0111,0x0000,0x0111,0x0515,0x0111,
0x0000,0x0111,0x0412,0x0111,0x0000,0x0111,0x0516,0x0111,
0x0000,0x0111,0x0413,0x0111,0x0000,0x0111,0x0517,0x0111,
0x0000,0x0111,0x0412,0x0111,0x081c,0x0111,0x0514,0x0111,
0x0000,0x0111,0x0413,0x0111,0x0000,0x0111,0x0515,0x0111,
0x0000,0x0111,0x0412,0x0111,0x0000,0x0111,0x0516,0x0111,
0x0000,0x0111,0x0413,0x0111,0x0000,0x0111,0x0517,0x0111,
0x0000,0x0111,0x0412,0x0111,0x081d,0x0111,0x0514,0x0111,
0x0000,0x0111,0x0413,0x0111,0x0000,0x0111,0x0515,0x0111,
0x0000,0x0111,0x0412,0x0111,0x0000,0x0111,0x0516,0x0111,
0x0000,0x0111,0x0413,0x0111,0x0000,0x0111,0x0517,0x0111,
0x0000,0x0111,0x0412,0x0111,0x081e,0x0111,0x0514,0x0111,
0x0000,0x0111,0x0413,0x0111,0x0000,0x0111,0x0515,0x0111,
0x0000,0x0111,0x0412,0x0111,0x0000,0x0111,0x0516,0x0111,
0x0000,0x0111,0x0413,0x0111,0x0000,0x0111,0x0517,0x0111,
0x0000,0x0111,0x0412,0x0111,0x081f,0x0111,0x0514,0x0111,
0x0000,0x0111,0x0413,0x0111,0x0000,0x0111,0x0515,0x0111,
0x0000,0x0111,0x0412,0x0111,0x0000,0x0111,0x0516,0x0111,
0x0000,0x0111,0x0413,0x0111,0x0000,0x0111,0x0517,0x0111,

0x0000,0x0111,0x0412,0x0222,0x0818,0x0523,0x0514,0x0222,
0x0000,0x0111,0x0413,0x0222,0x0000,0x0625,0x0515,0x0222,
0x0000,0x0111,0x0523,0x0222,0x0000,0x0524,0x0516,0x0222,
0x0000,0x0111,0x0524,0x0222,0x0000,0x0626,0x0517,0x0222,
0x0000,0x0111,0x0824,0x0222,0x0819,0x0523,0x0625,0x0222,
0x0000,0x0111,0x0825,0x0222,0x0000,0x0627,0x0626,0x0222,
0x0000,0x0111,0x0523,0x0222,0x0000,0x0524,0x0627,0x0222,
0x0000,0x0111,0x0524,0x0222,0x0000,0x0628,0x0628,0x0222,
0x0000,0x0111,0x0412,0x0222,0x081a,0x0523,0x0514,0x0222,
0x0000,0x0111,0x0413,0x0222,0x0000,0x0625,0x0515,0x0222,
0x0000,0x0111,0x0523,0x0222,0x0000,0x0524,0x0516,0x0222,
0x0000,0x0111,0x0524,0x0222,0x0000,0x0626,0x0517,0x0222,
0x0000,0x0111,0x0412,0x0222,0x081b,0x0523,0x0625,0x0222,
0x0000,0x0111,0x0413,0x0222,0x0000,0x0627,0x0626,0x0222,
0x0000,0x0111,0x0523,0x0222,0x0000,0x0524,0x0627,0x0222,
0x0000,0x0111,0x0524,0x0222,0x0000,0x0628,0x0628,0x0222,
0x0000,0x0111,0x0412,0x0222,0x081c,0x0523,0x0514,0x0222,
0x0000,0x0111,0x0413,0x0222,0x0000,0x0625,0x0515,0x0222,
0x0000,0x0111,0x0523,0x0222,0x0000,0x0524,0x0516,0x0222,
0x0000,0x0111,0x0524,0x0222,0x0000,0x0626,0x0517,0x0222,
0x0000,0x0111,0x0825,0x0222,0x081d,0x0523,0x0625,0x0222,
0x0000,0x0111,0x0826,0x0222,0x0000,0x0627,0x0626,0x0222,
0x0000,0x0111,0x0523,0x0222,0x0000,0x0524,0x0627,0x0222,
0x0000,0x0111,0x0524,0x0222,0x0000,0x0628,0x0628,0x0222,
0x0000,0x0111,0x0412,0x0222,0x081e,0x0523,0x0514,0x0222,
0x0000,0x0111,0x0413,0x0222,0x0000,0x0625,0x0515,0x0222,
0x0000,0x0111,0x0523,0x0222,0x0000,0x0524,0x0516,0x0222,
0x0000,0x0111,0x0524,0x0222,0x0000,0x0626,0x0517,0x0222,
0x0000,0x0111,0x0412,0x0222,0x081f,0x0523,0x0625,0x0222,
0x0000,0x0111,0x0413,0x0222,0x0000,0x0627,0x0626,0x0222,
0x0000,0x0111,0x0523,0x0222,0x0000,0x0524,0x0627,0x0222,
0x0000,0x0111,0x0524,0x0222,0x0000,0x0628,0x0628,0x0222,

0x0000,0x0111,0x0412,0x0222,0x0818,0x0523,0x0514,0x0333,
0x0000,0x0111,0x0413,0x0634,0x0000,0x0625,0x0515,0x0333,
0x0000,0x0111,0x0523,0x0222,0x0000,0x0524,0x0516,0x0333,
0x0000,0x0111,0x0524,0x0736,0x0000,0x0626,0x0517,0x0333,
0x0000,0x0111,0x0824,0x0222,0x0819,0x0634,0x0625,0x0333,
0x0000,0x0111,0x0825,0x0635,0x0000,0x0627,0x0626,0x0333,
0x0000,0x0111,0x0634,0x0222,0x0000,0x0635,0x0627,0x0333,
0x0000,0x0111,0x0635,0x0737,0x0000,0x0628,0x0628,0x0333,
0x0000,0x0111,0x0412,0x0222,0x081a,0x0523,0x0514,0x0333,
0x0000,0x0111,0x0413,0x0634,0x0000,0x0736,0x0515,0x0333,
0x0000,0x0111,0x0523,0x0222,0x0000,0x0524,0x0516,0x0333,
0x0000,0x0111,0x0524,0x0738,0x0000,0x0737,0x0517,0x0333,
0x0000,0x0111,0x0412,0x0222,0x081b,0x0634,0x0736,0x0333,
0x0000,0x0111,0x0413,0x0635,0x0000,0x0738,0x0737,0x0333,
0x0000,0x0111,0x0634,0x0222,0x0000,0x0635,0x0738,0x0333,
0x0000,0x0111,0x0635,0x0739,0x0000,0x0739,0x0739,0x0333,
0x0000,0x0111,0x0412,0x0222,0x081c,0x0523,0x0514,0x0333,
0x0000,0x0111,0x0413,0x0634,0x0000,0x0625,0x0515,0x0333,
0x0000,0x0111,0x0523,0x0222,0x0000,0x0524,0x0516,0x0333,
0x0000,0x0111,0x0524,0x0736,0x0000,0x0626,0x0517,0x0333,
0x0000,0x0111,0x0825,0x0222,0x081d,0x0634,0x0625,0x0333,
0x0000,0x0111,0x0826,0x0635,0x0000,0x0627,0x0626,0x0333,
0x0000,0x0111,0x0634,0x0222,0x0000,0x0635,0x0627,0x0333,
0x0000,0x0111,0x0635,0x0737,0x0000,0x0628,0x0628,0x0333,
0x0000,0x0111,0x0412,0x0222,0x081e,0x0523,0x0514,0x0333,
0x0000,0x0111,0x0413,0x0634,0x0000,0x0736,0x0515,0x0333,
0x0000,0x0111,0x0523,0x0222,0x0000,0x0524,0x0516,0x0333,
0x0000,0x0111,0x0524,0x0738,0x0000,0x0737,0x0517,0x0333,
0x0000,0x0111,0x0412,0x0222,0x081f,0x0634,0x0736,0x0333,
0x0000,0x0111,0x0413,0x0635,0x0000,0x0738,0x0737,0x0333,
0x0000,0x0111,0x0634,0x0222,0x0000,0x0635,0x0738,0x0333,
0x0000,0x0111,0x0635,0x0739,0x0000,0x0739,0x0739,0x0333,

0x0000,0x0111,0x0412,0x0222,0x0818,0x0523,0x0514,0x0333,
0x0000,0x0111,0x0413,0x0634,0x0000,0x0625,0x0515,0x0444,
0x0000,0x0111,0x0523,0x0222,0x0000,0x0524,0x0516,0x0745,
0x0000,0x0111,0x0524,0x0736,0x0000,0x0626,0x0517,0x0444,
0x0000,0x0111,0x0824,0x0222,0x0819,0x0634,0x0625,0x0333,
0x0000,0x0111,0x0825,0x0635,0x0000,0x0627,0x0626,0x0444,
0x0000,0x0111,0x0634,0x0222,0x0000,0x0635,0x0627,0x0847,
0x0000,0x0111,0x0635,0x0737,0x0000,0x0628,0x0628,0x0444,
0x0000,0x0111,0x0412,0x0222,0x081a,0x0523,0x0514,0x0333,
0x0000,0x0111,0x0413,0x0745,0x0000,0x0736,0x0515,0x0444,
0x0000,0x0111,0x0523,0x0222,0x0000,0x0524,0x0516,0x0746,
0x0000,0x0111,0x0524,0x0738,0x0000,0x0737,0x0517,0x0444,
0x0000,0x0111,0x0412,0x0222,0x081b,0x0745,0x0736,0x0333,
0x0000,0x0111,0x0413,0x0746,0x0000,0x0738,0x0737,0x0444,
0x0000,0x0111,0x0745,0x0222,0x0000,0x0746,0x0738,0x0848,
0x0000,0x0111,0x0746,0x0739,0x0000,0x0739,0x0739,0x0444,
0x0000,0x0111,0x0412,0x0222,0x081c,0x0523,0x0514,0x0333,
0x0000,0x0111,0x0413,0x0634,0x0000,0x0625,0x0515,0x0444,
0x0000,0x0111,0x0523,0x0222,0x0000,0x0524,0x0516,0x0745,
0x0000,0x0111,0x0524,0x0847,0x0000,0x0626,0x0517,0x0444,
0x0000,0x0111,0x0825,0x0222,0x081d,0x0634,0x0625,0x0333,
0x0000,0x0111,0x0826,0x0635,0x0000,0x0627,0x0626,0x0444,
0x0000,0x0111,0x0634,0x0222,0x0000,0x0635,0x0627,0x0849,
0x0000,0x0111,0x0635,0x0848,0x0000,0x0628,0x0628,0x0444,
0x0000,0x0111,0x0412,0x0222,0x081e,0x0523,0x0514,0x0333,
0x0000,0x0111,0x0413,0x0745,0x0000,0x0847,0x0515,0x0444,
0x0000,0x0111,0x0523,0x0222,0x0000,0x0524,0x0516,0x0746,
0x0000,0x0111,0x0524,0x0849,0x0000,0x0848,0x0517,0x0444,
0x0000,0x0111,0x0412,0x0222,0x081f,0x0745,0x0847,0x0333,
0x0000,0x0111,0x0413,0x0746,0x0000,0x0849,0x0848,0x0444,
0x0000,0x0111,0x0745,0x0222,0x0000,0x0746,0x0849,0x084a,
0x0000,0x0111,0x0746,0x084a,0x0000,0x084a,0x084a,0x0444,

0x0000,0x0111,0x0412,0x0222,0x0818,0x0523,0x0514,0x0333,
0x0000,0x0111,0x0413,0x0634,0x0000,0x0625,0x0515,0x0444,
0x0000,0x0111,0x0523,0x0222,0x0000,0x0524,0x0516,0x0745,
0x0000,0x0111,0x0524,0x0736,0x0000,0x0626,0x0517,0x0555,
0x0000,0x0111,0x0824,0x0222,0x0819,0x0634,0x0625,0x0333,
0x0000,0x0111,0x0825,0x0635,0x0000,0x0627,0x0626,0x0856,
0x0000,0x0111,0x0634,0x0222,0x0000,0x0635,0x0627,0x0847,
0x0000,0x0111,0x0635,0x0737,0x0000,0x0628,0x0628,0x0555,
0x0000,0x0111,0x0412,0x0222,0x081a,0x0523,0x0514,0x0333,
0x0000,0x0111,0x0413,0x0745,0x0000,0x0736,0x0515,0x0444,
0x0000,0x0111,0x0523,0x0222,0x0000,0x0524,0x0516,0x0746,
0x0000,0x0111,0x0524,0x0738,0x0000,0x0737,0x0517,0x0555,
0x0000,0x0111,0x0412,0x0222,0x081b,0x0745,0x0736,0x0333,
0x0000,0x0111,0x0413,0x0746,0x0000,0x0738,0x0737,0x0444,
0x0000,0x0111,0x0745,0x0222,0x0000,0x0746,0x0738,0x0848,
0x0000,0x0111,0x0746,0x0739,0x0000,0x0739,0x0739,0x0555,
0x0000,0x0111,0x0412,0x0222,0x081c,0x0523,0x0514,0x0333,
0x0000,0x0111,0x0413,0x0634,0x0000,0x0625,0x0515,0x0444,
0x0000,0x0111,0x0523,0x0222,0x0000,0x0524,0x0516,0x0856,
0x0000,0x0111,0x0524,0x0847,0x0000,0x0626,0x0517,0x0555,
0x0000,0x0111,0x0825,0x0222,0x081d,0x0634,0x0625,0x0333,
0x0000,0x0111,0x0826,0x0635,0x0000,0x0627,0x0626,0x0857,
0x0000,0x0111,0x0634,0x0222,0x0000,0x0635,0x0627,0x0849,
0x0000,0x0111,0x0635,0x0848,0x0000,0x0628,0x0628,0x0555,
0x0000,0x0111,0x0412,0x0222,0x081e,0x0523,0x0514,0x0333,
0x0000,0x0111,0x0413,0x0856,0x0000,0x0847,0x0515,0x0444,
0x0000,0x0111,0x0523,0x0222,0x0000,0x0524,0x0516,0x0857,
0x0000,0x0111,0x0524,0x0849,0x0000,0x0848,0x0517,0x0555,
0x0000,0x0111,0x0412,0x0222,0x081f,0x0856,0x0847,0x0333,
0x0000,0x0111,0x0413,0x0857,0x0000,0x0849,0x0848,0x0444,
0x0000,0x0111,0x0856,0x0222,0x0000,0x0857,0x0849,0x084a,
0x0000,0x0111,0x0857,0x084a,0x0000,0x084a,0x084a,0x0555,

0x0000,0x0111,0x0412,0x0222,0x0818,0x0523,0x0514,0x0333,
0x0000,0x0111,0x0413,0x0634,0x0000,0x0625,0x0515,0x0444,
0x0000,0x0111,0x0523,0x0222,0x0000,0x0524,0x0516,0x0745,
0x0000,0x0111,0x0524,0x0736,0x0000,0x0626,0x0517,0x0555,
0x0000,0x0111,0x0824,0x0222,0x0819,0x0634,0x0625,0x0333,
0x0000,0x0111,0x0825,0x0635,0x0000,0x0627,0x0626,0x0856,
0x0000,0x0111,0x0634,0x0222,0x0000,0x0635,0x0627,0x0847,
0x0000,0x0111,0x0635,0x0737,0x0000,0x0628,0x0628,0x0666,
0x0000,0x0111,0x0412,0x0222,0x081a,0x0523,0x0514,0x0333,
0x0000,0x0111,0x0413,0x0745,0x0000,0x0736,0x0515,0x0444,
0x0000,0x0111,0x0523,0x0222,0x0000,0x0524,0x0516,0x0746,
0x0000,0x0111,0x0524,0x0738,0x0000,0x0737,0x0517,0x0555,
0x0000,0x0111,0x0412,0x0222,0x081b,0x0745,0x0736,0x0333,
0x0000,0x0111,0x0413,0x0746,0x0000,0x0738,0x0737,0x0444,
0x0000,0x0111,0x0745,0x0222,0x0000,0x0746,0x0738,0x0848,
0x0000,0x0111,0x0746,0x0739,0x0000,0x0739,0x0739,0x0666,
0x0000,0x0111,0x0412,0x0222,0x081c,0x0523,0x0514,0x0333,
0x0000,0x0111,0x0413,0x0634,0x0000,0x0625,0x0515,0x0444,
0x0000,0x0111,0x0523,0x0222,0x0000,0x0524,0x0516,0x0856,
0x0000,0x0111,0x0524,0x0847,0x0000,0x0626,0x0517,0x0555,
0x0000,0x0111,0x0825,0x0222,0x081d,0x0634,0x0625,0x0333,
0x0000,0x0111,0x0826,0x0635,0x0000,0x0627,0x0626,0x0857,
0x0000,0x0111,0x0634,0x0222,0x0000,0x0635,0x0627,0x0849,
0x0000,0x0111,0x0635,0x0848,0x0000,0x0628,0x0628,0x0666,
0x0000,0x0111,0x0412,0x0222,0x081e,0x0523,0x0514,0x0333,
0x0000,0x0111,0x0413,0x0856,0x0000,0x0847,0x0515,0x0444,
0x0000,0x0111,0x0523,0x0222,0x0000,0x0524,0x0516,0x0857,
0x0000,0x0111,0x0524,0x0849,0x0000,0x0848,0x0517,0x0555,
0x0000,0x0111,0x0412,0x0222,0x081f,0x0856,0x0847,0x0333,
0x0000,0x0111,0x0413,0x0857,0x0000,0x0849,0x0848,0x0444,
0x0000,0x0111,0x0856,0x0222,0x0000,0x0857,0x0849,0x084a,
0x0000,0x0111,0x0857,0x084a,0x0000,0x084a,0x084a,0x0666,

0x0000,0x0111,0x0412,0x0222,0x0818,0x0523,0x0514,0x0333,
0x0000,0x0111,0x0413,0x0634,0x0000,0x0625,0x0515,0x0444,
0x0000,0x0111,0x0523,0x0222,0x0000,0x0524,0x0516,0x0745,
0x0000,0x0111,0x0524,0x0736,0x0000,0x0626,0x0517,0x0555,
0x0000,0x0111,0x0824,0x0222,0x0819,0x0634,0x0625,0x0333,
0x0000,0x0111,0x0825,0x0635,0x0000,0x0627,0x0626,0x0856,
0x0000,0x0111,0x0634,0x0222,0x0000,0x0635,0x0627,0x0847,
0x0000,0x0111,0x0635,0x0737,0x0000,0x0628,0x0628,0x0666,
0x0000,0x0111,0x0412,0x0222,0x081a,0x0523,0x0514,0x0333,
0x0000,0x0111,0x0413,0x0745,0x0000,0x0736,0x0515,0x0444,
0x0000,0x0111,0x0523,0x0222,0x0000,0x0524,0x0516,0x0746,
0x0000,0x0111,0x0524,0x0738,0x0000,0x0737,0x0517,0x0555,
0x0000,0x0111,0x0412,0x0222,0x081b,0x0745,0x0736,0x0333,
0x0000,0x0111,0x0413,0x0746,0x0000,0x0738,0x0737,0x0444,
0x0000,0x0111,0x0745,0x0222,0x0000,0x0746,0x0738,0x0848,
0x0000,0x0111,0x0746,0x0739,0x0000,0x0739,0x0739,0x0777,
0x0000,0x0111,0x0412,0x0222,0x081c,0x0523,0x0514,0x0333,
0x0000,0x0111,0x0413,0x0634,0x0000,0x0625,0x0515,0x0444,
0x0000,0x0111,0x0523,0x0222,0x0000,0x0524,0x0516,0x0856,
0x0000,0x0111,0x0524,0x0847,0x0000,0x0626,0x0517,0x0555,
0x0000,0x0111,0x0825,0x0222,0x081d,0x0634,0x0625,0x0333,
0x0000,0x0111,0x0826,0x0635,0x0000,0x0627,0x0626,0x0857,
0x0000,0x0111,0x0634,0x0222,0x0000,0x0635,0x0627,0x0849,
0x0000,0x0111,0x0635,0x0848,0x0000,0x0628,0x0628,0x0666,
0x0000,0x0111,0x0412,0x0222,0x081e,0x0523,0x0514,0x0333,
0x0000,0x0111,0x0413,0x0856,0x0000,0x0847,0x0515,0x0444,
0x0000,0x0111,0x0523,0x0222,0x0000,0x0524,0x0516,0x0857,
0x0000,0x0111,0x0524,0x0849,0x0000,0x0848,0x0517,0x0555,
0x0000,0x0111,0x0412,0x0222,0x081f,0x0856,0x0847,0x0333,
0x0000,0x0111,0x0413,0x0857,0x0000,0x0849,0x0848,0x0444,
0x0000,0x0111,0x0856,0x0222,0x0000,0x0857,0x0849,0x084a,
0x0000,0x0111,0x0857,0x084a,0x0000,0x084a,0x084a,0x0777,

0x0000,0x0111,0x0412,0x0222,0x0818,0x0523,0x0514,0x0333,
0x0000,0x0111,0x0413,0x0634,0x0000,0x0625,0x0515,0x0444,
0x0000,0x0111,0x0523,0x0222,0x0000,0x0524,0x0516,0x0745,
0x0000,0x0111,0x0524,0x0736,0x0000,0x0626,0x0517,0x0555,
0x0000,0x0111,0x0824,0x0222,0x0819,0x0634,0x0625,0x0333,
0x0000,0x0111,0x0825,0x0635,0x0000,0x0627,0x0626,0x0856,
0x0000,0x0111,0x0634,0x0222,0x0000,0x0635,0x0627,0x0847,
0x0000,0x0111,0x0635,0x0737,0x0000,0x0628,0x0628,0x0666,
0x0000,0x0111,0x0412,0x0222,0x081a,0x0523,0x0514,0x0333,
0x0000,0x0111,0x0413,0x0745,0x0000,0x0736,0x0515,0x0444,
0x0000,0x0111,0x0523,0x0222,0x0000,0x0524,0x0516,0x0746,
0x0000,0x0111,0x0524,0x0738,0x0000,0x0737,0x0517,0x0555,
0x0000,0x0111,0x0412,0x0222,0x081b,0x0745,0x0736,0x0333,
0x0000,0x0111,0x0413,0x0746,0x0000,0x0738,0x0737,0x0444,
0x0000,0x0111,0x0745,0x0222,0x0000,0x0746,0x0738,0x0848,
0x0000,0x0111,0x0746,0x0739,0x0000,0x0739,0x0739,0x0777,
0x0000,0x0111,0x0412,0x0222,0x081c,0x0523,0x0514,0x0333,
0x0000,0x0111,0x0413,0x0634,0x0000,0x0625,0x0515,0x0444,
0x0000,0x0111,0x0523,0x0222,0x0000,0x0524,0x0516,0x0856,
0x0000,0x0111,0x0524,0x0847,0x0000,0x0626,0x0517,0x0555,
0x0000,0x0111,0x0825,0x0222,0x081d,0x0634,0x0625,0x0333,
0x0000,0x0111,0x0826,0x0635,0x0000,0x0627,0x0626,0x0857,
0x0000,0x0111,0x0634,0x0222,0x0000,0x0635,0x0627,0x0849,
0x0000,0x0111,0x0635,0x0848,0x0000,0x0628,0x0628,0x0666,
0x0000,0x0111,0x0412,0x0222,0x081e,0x0523,0x0514,0x0333,
0x0000,0x0111,0x0413,0x0856,0x0000,0x0847,0x0515,0x0444,
0x0000,0x0111,0x0523,0x0222,0x0000,0x0524,0x0516,0x0857,
0x0000,0x0111,0x0524,0x0849,0x0000,0x0848,0x0517,0x0555,
0x0000,0x0111,0x0412,0x0222,0x081f,0x0856,0x0847,0x0333,
0x0000,0x0111,0x0413,0x0857,0x0000,0x0849,0x0848,0x0444,
0x0000,0x0111,0x0856,0x0222,0x0000,0x0857,0x0849,0x084a,
0x0000,0x0111,0x0857,0x084a,0x0000,0x084a,0x084a,0x0888
};




} // end namespace sdsl
} // end namespace coder
